/*  -------------------------------------------------------------------------
 *
 *                Project: JRecord
 *    
 *    Sub-Project purpose: Provide support for reading Cobol-Data files 
 *                        using a Cobol Copybook in Java.
 *                         Support for reading Fixed Width / Binary / Csv files
 *                        using a Xml schema.
 *                         General Fixed Width / Csv file processing in Java.
 *    
 *                 Author: Bruce Martin
 *    
 *                License: LGPL 2.1 or latter
 *                
 *    Copyright (c) 2016, Bruce Martin, All Rights Reserved.
 *   
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *   
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 * ------------------------------------------------------------------------ */

package net.sf.JRecord.External;

import net.sf.JRecord.Details.LayoutDetail;

public interface ICsvSchemaBuilder {

	/**
	 * Add a Csv field to the schema
	 * @param name field name
	 * @param type Field type
	 * @param decimal how many decimals (for fixed length numeric types)
	 * @return this Schema-builder so other fields can be added
	 */
	public ICsvSchemaBuilder addCsvField(String name, int type, int decimal);
	
	/**
	 * Convert to schema (LayoutDetail)
	 * @return schema in the form of LayoutDetail
	 */
	public LayoutDetail asLayoutDetail();

	/**
	 * Used in interface to convert back to ExternalRecord
	 * @return this ExternalRecord
	 */
	public ExternalRecord asExternalRecord();


}
