package net.sf.JRecord.cg.update.duplicateFieldNames;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.JRecord.cg.schema.FieldDef;
import net.sf.JRecord.cg.schema.RecordDef;
import net.sf.JRecord.cgen.def.rename.ISetSelectionGroupNames;
import net.sf.JRecord.cgen.def.rename.IUpdateField;
import net.sf.JRecord.cgen.def.rename.IUpdateRecord;
import net.sf.JRecord.cgen.def.rename.IUpdateSchema;

public class RenameDuplicates {
	private static final String[] NO_GROUP_NAMES = {};
	//private final IUpdateSchema schema;
	private final List<? extends IUpdateRecord> records;
	
	//private final List<ExternalRecordHelper> children;
	//private final FieldMap fieldMap;
	

	public RenameDuplicates(IUpdateSchema schema) {
		//this.schema = schema;	
		records = schema.getUpdateRecords();
	}
	
	
	
	/**
	 * @return
	 * @see net.sf.JRecord.cg.update.duplicateFieldNames.RenameDuplicates.FieldMap#getDuplicateFieldNames()
	 */
	public List<List<FieldHelper>> getDuplicateFieldNames() {
		return fieldMapForSchema().getDuplicateFieldNames();
	}

	public void updateDuplicateNamesInEachRecord2() {
		
		//List<List<FieldHelper>> ret = new ArrayList<>();
		FieldMap fieldMap = fieldMapForSchema();
		List<List<FieldHelper>> duplicateFieldNames = fieldMap.getDuplicateFieldNames();
		for (List<FieldHelper> fieldList : duplicateFieldNames) { 
			updateUniqueGroups(fieldList);
			FieldMap dfm = updateNames(fieldList, 1);
			List<List<FieldHelper>> dupFields = dfm.getDuplicateFieldNames();
			for (List<FieldHelper> l : dupFields) { 
				checkUniqueInGroup(l);
				FieldMap dfm2 = updateNames(l, 2);
				for (List<FieldHelper> l2 : dfm2.getDuplicateFieldNames()) { 
					checkUniqueInGroup(l2);
					for (FieldHelper fh : l2) {
						fh.setUniqueGroupNames(fh.groupNames.length - fh.firstGroup);
					}
				}
			}
		}
		//return null;
	}



	/**
	 * @param fieldList
	 */
	private void updateUniqueGroups(List<FieldHelper> fieldList) {
		if (fieldList.size() > 2) {
			fieldList.sort(new Comparator<FieldHelper>() {
				@Override
				public int compare(FieldHelper h1, FieldHelper h2) {
					GroupCompare g = new GroupCompare(h1, h2);
					
					if (g.m1 >= 0 && g.m2 >= 0) {
						 return h1.groupNames[g.m1].compareToIgnoreCase(h2.groupNames[g.m2]);
					}
	
					return g.m1 - g.m2;
				}
			});
		}
		
		for (int i = 1; i < fieldList.size(); i++) {
			GroupCompare g = new GroupCompare(fieldList.get(i-1), fieldList.get(i-2));
			fieldList.get(i-1).updateFirstGroup(g.m1);
			fieldList.get(i).updateFirstGroup(g.m2);
		}
		
		checkUniqueInGroup(fieldList);
	}



	private void checkUniqueInGroup(List<FieldHelper> fieldList) {
		for (int idx = fieldList.size()-1; idx >= 0; idx--) {
			boolean searching = true;
			FieldHelper currField = fieldList.get(idx);
			for (int i = 0; i < fieldList.size() && searching; i++) {
				searching = (idx == i) || fieldList.get(i).record != currField.record;
			}
			if (searching) {
				currField.insertUniqueGroupName(currField.groupNames[0]);

				fieldList.remove(idx);
			}
		}
	}

	public List<List<FieldHelper>> updateDuplicateNamesInEachRecord() {
		List<List<FieldHelper>> dups = new ArrayList<>();
		for (IUpdateRecord rec : records) {
			List<List<FieldHelper>> duplicateFieldNames = fieldMapForRecord(rec).getDuplicateFieldNames();
			for (List<FieldHelper> fieldList : duplicateFieldNames) { 
				FieldMap dfm = updateNames(fieldList, 1);
				List<List<FieldHelper>> dupFields = dfm.getDuplicateFieldNames();
				for (List<FieldHelper> l : dupFields) { 
					updateNames(l, 2);
				}
			}

			dups.addAll(fieldMapForRecord(rec).getDuplicateFieldNames());
		}
		return dups;
	}


	public boolean renameField(String newName, String... names) {
		List<IUpdateField> matchingFields = new ArrayList<>();
		String fieldName = names[names.length - 1];
		ArrayList<String> ln = new ArrayList<String>(names.length-1);
		IUpdateRecord r = null;
		int updatedCount = 0;

		for (int i = 0; i < names.length-1; i++) {
			if (names[i] != null && ! "".equals(names[i])) {
				ln.add("." + names[i].toUpperCase() + ".");
			}
		}

		for (IUpdateRecord rec : records) {
			for (IUpdateField f : rec.getUpdateFields()) {
				if (f.getName().equalsIgnoreCase(fieldName)) {
					String groupName = f.getGroup().toUpperCase();
					boolean ok = true;
					int st = 0;
					for (String n : ln) {
						if ((st = groupName.indexOf(n, st)) < 0) {
							ok = false;
							break;
						}
					}

					if (ok) {
						matchingFields.add(f);
						r = rec;
					}
				}
			}
			switch (matchingFields.size()) {
			case 0: break;
			case 1:
				matchingFields.get(0).setName(newName);
				r.fieldUpdated();
				updatedCount += 1;
				break;
			default: throw new RuntimeException("multiple matches (" + matchingFields.size() + ") found in record "
												+ rec.getRecordName());
			}
		}

		return updatedCount > 0;
	}



	/**
	 * @param dfm
	 * @param dupNamesList
	 */
	private FieldMap updateNames(List<FieldHelper> dupNamesList, int diff) {
		int size = dupNamesList.size();
		FieldMap dfm = new FieldMap(size * 2);
//		int chg = 0;
		for (FieldHelper fieldDtl : dupNamesList) {
			List<String> gnl = new ArrayList<String>(diff);
			int groupNamesLength = fieldDtl.groupNames == null ? 0 : fieldDtl.groupNames.length;
			if (groupNamesLength >= diff) {
				int idx = fieldDtl.groupNames.length - diff;
				String name = fieldDtl.groupNames[idx] + "-" + fieldDtl.field.getName();
				fieldDtl.field.setName(name);
				dfm.addField(name.toUpperCase(), fieldDtl);
				fieldDtl.record.fieldUpdated();
				
				
				fieldDtl.setUniqueGroupNames(diff);

//				chg += 1;
//				if (chg >= size - 1) { break; }
			}
		}
		return dfm;
	}
	
	private FieldMap fieldMapForSchema() {
		int count = 0;
		for (IUpdateRecord rec : records) {
			count += rec.getUpdateFields().size();
		}
		FieldMap fieldMap = new FieldMap(Math.max(20, count * 3 / 2));

		for (IUpdateRecord rec : records) {
			fieldMapForRecord(fieldMap, rec);
		}
		
		return fieldMap;
	}
	
	
	/**
	 * @param fieldMap
	 * @param rec
	 */
	private FieldMap fieldMapForRecord(IUpdateRecord rec) {
		FieldMap fieldMap = new FieldMap(Math.max(20, rec.getUpdateFields().size() * 3 / 2));

		fieldMapForRecord(fieldMap, rec);
	
		return fieldMap;
	}
	
	
	/**
	 * @param fieldMap
	 * @param rec
	 */
	private void fieldMapForRecord(FieldMap fieldMap, IUpdateRecord rec) {
		if (rec instanceof RecordDef) {
			RecordDef record = (RecordDef) rec;
			for (FieldDef fld : record.getFields()) {
				String name = fld.getName();
				if (name != null && name.length() > 0 && ! "filler".equalsIgnoreCase(name)) {
					if (fld.isArrayItem()) {
						if (fld.getArrayIndexDetails().isFirstIndex()) {
							fieldMap.addField(name, newFieldHelper(record, fld));
						}
					} else {
						fieldMap.addField(name, newFieldHelper(record, fld));
					}
				}
			}
		} else {
			for (IUpdateField fld : rec.getUpdateFields()) {
				String name = fld.getName();
				if (name != null && name.length() > 0 && ! "filler".equalsIgnoreCase(name)) {
					fieldMap.addField(name, rec, fld);
				}
			}
		}
	}
	
	
//	private void updateForDuplicates(List<ExternalField> dups) {
//		for (ExternalRecordHelper c : children) {
//			c.updateForDuplicates(dups);
//		}
//		
//		List<List<ExternalField>> duplicateFieldNames = fieldMap.getDuplicateFieldNames();
//	}
	
	private static class FieldMap {

		Map<String, List<FieldHelper>> map;
		IUpdateRecord lastRecord = null;
		List<FieldHelper> currentRecordFields;
		ArrayList<List<FieldHelper>> recordFields = new ArrayList<>();

		FieldMap(int size) {
			map = new HashMap<>(size);
		}

		public void addField(String name, RecordDef record, FieldDef f) {
			addAfield(name, record, new FieldHelper(record, f));
		}

		public void addField(String name, IUpdateRecord record, IUpdateField f) {
			addAfield(name, record, new FieldHelper(record, f));
		}

		private void addAfield(String name, IUpdateRecord record, FieldHelper fieldDetails) {
			if (lastRecord != record || currentRecordFields == null) {
				lastRecord = record;
				currentRecordFields = new ArrayList<>();
				recordFields.add(currentRecordFields);
			}
			currentRecordFields.add(fieldDetails);
			addField(name, fieldDetails);
		}

		public void addField(String name, FieldHelper f) {
			name = name.toLowerCase();
			List<FieldHelper> fieldList = map.get(name);
			if (fieldList == null) {
				fieldList = new ArrayList<>(1);
				map.put(name, fieldList);
			}
			fieldList.add(f);
		}

		public List<List<FieldHelper>> getDuplicateFieldNames() {
			Collection<List<FieldHelper>> values = map.values();
			ArrayList<List<FieldHelper>> ret = new ArrayList<>();

			for (List<FieldHelper> v : values) {
				if (v.size() > 1) {
					ret.add(v);
				}
			}

			return ret;
		}
	}
	
	public static FieldHelper newFieldHelper(RecordDef rec, FieldDef field) {
		FieldHelper ret = new FieldHelper(rec, field);
		ret.arrayField = field.isArrayItem();
		ret.firstArrayElement = field.getArrayIndexDetails().isFirstIndex();
		
		return ret;
		
	}
	public static class FieldHelper  {
		private final IUpdateField field;
		private final IUpdateRecord record;
		private final String[] groupNames;
		private final String originalName;
		private boolean arrayField = false, firstArrayElement = false;
		int firstGroup = 0;
		List<String> uniqueGroupNames = null;
		
		public FieldHelper(IUpdateRecord record, IUpdateField field) {
			super();
			this.record = record;
			this.field = field;
			
			String g = field.getGroup();
			groupNames = g == null || g.length() == 0 ? NO_GROUP_NAMES : g.split("\\.");
			originalName = field.getName();
			firstGroup = g.length() - 1;
		}

		void updateFirstGroup(int start) {
			firstGroup = Math.min(firstGroup, start);
		}
		
		public void insertUniqueGroupName(String gName) {
			
			if (uniqueGroupNames == null) {
				uniqueGroupNames = new ArrayList<>(1);
				uniqueGroupNames.add(gName);
			} else {
				uniqueGroupNames.add(0, gName);
			}
			updateFieldGroupNames();
		}
		
		public void setUniqueGroupNames(int number) {
			
			int groupNamesLength = groupNames.length;
			if (number > groupNamesLength) { return; }

			if (uniqueGroupNames == null) {
				uniqueGroupNames = new ArrayList<>(number);
			} else {
				uniqueGroupNames.clear();
			}
			
			for (int i = number; i > 0; i--) {
				uniqueGroupNames.add(groupNames[groupNamesLength - i]);
			}

			updateFieldGroupNames();
		}
		
		private void updateFieldGroupNames() {
			if (uniqueGroupNames != null && field instanceof ISetSelectionGroupNames) {
				((ISetSelectionGroupNames) field).setSelectionGroupNames(uniqueGroupNames);
			}
		}
	}

	private static class GroupCompare  {
		int m1, m2;
		private GroupCompare(FieldHelper h1, FieldHelper h2) {
			int m1 = h1.groupNames.length - 1;
			int m2 = h2.groupNames.length - 1;
			
			while (m1 >= 0 && m2 >= 0 && h1.groupNames[m1].equalsIgnoreCase(h2.groupNames[m2]) ) {
				m1 -= 1;
				m2 -= 1;
			}
		}
	}
}
