/*  -------------------------------------------------------------------------
 *
 *            Sub-Project: RecordEditor's version of JRecord 
 *    
 *    Sub-Project purpose: Low-level IO and record translation  
 *                        code + Cobol Copybook Translation
 *    
 *                 Author: Bruce Martin
 *    
 *                License: GPL 2.1 or later
 *                
 *    Copyright (c) 2016, Bruce Martin, All Rights Reserved.
 *   
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License
 *    as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *   
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 * ------------------------------------------------------------------------ */
      
package net.sf.JRecord.External;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.ILineFieldNames;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.External.base.BaseRecordEditorXmlLoader;
import net.sf.JRecord.IO.XmlLineReader;
import net.sf.JRecord.Log.AbsSSLogger;

/**
 * Class to Load a RecordLayout (Record or Line Description)
 * from an XML file (RecordEditor-XML)
 *
 * <pre>
 *   <b>Usage:</b>
 *        CopybookLoader loader = new RecordEditorXmlLoader();
 *        LayoutDetail layout = loader.loadCopyBook(copybookName, 0, 0, "", 0, 0, null).asLayoutDetail();
 * </pre>
 *
 * @author Bruce Martin
 *
 */
public class RecordEditorXmlLoader extends BaseCopybookLoader {

	/**
	 * Load the Copybook
	 * @see CopybookLoader#loadCopyBook(String, int, int, String, int, int, AbsSSLogger)
	 */
	@Override
	public ExternalRecord loadCopyBook(String copyBookFile,
		int splitCopybookOption, int dbIdx, String font, int binFormat,
		int systemId, AbsSSLogger log) throws Exception {

		XmlLineReader reader = new XmlLineReader(true);
		reader.open(copyBookFile);

		return loadCopybook(reader, Conversion.getCopyBookId(copyBookFile), font, log);
	}


	public ExternalRecord loadCopyBook(InputStream is, String layoutName) throws IOException {

        XmlLineReader r = new XmlLineReader(true);
		r.open(is, (LayoutDetail) null);
	    return loadCopybook(r, layoutName, "", null);

	}
	

	/**
	 * @param reader
	 * @param copyBookName
	 * @param font
	 * @param log
	 * @return
	 * @throws IOException
	 */
	private ExternalRecord loadCopybook(XmlLineReader reader, String copyBookName, String font, AbsSSLogger log)
			throws IOException {
		return (new BaseRecordEditorXmlLoader<ExternalRecord>(
								new XmlReader(reader),
								new ExternalRecordBuilder()
					)).loadCopyBook(copyBookName, font, log);
	}

	private static class XmlReader implements BaseRecordEditorXmlLoader.ILineReader {

		private final XmlLineReader reader;
		
		public XmlReader(XmlLineReader reader) {
			super();
			this.reader = reader;
		}

		@Override
		public ILineFieldNames read() throws IOException {
			return reader.read();
		}

		@Override
		public void close()  throws IOException {
			reader.close();
		}
		
	}



	public static ExternalRecord getExternalRecord(String xml, String name) throws Exception {
		ByteArrayInputStream bs = new ByteArrayInputStream(xml.getBytes());

		RecordEditorXmlLoader loader = new RecordEditorXmlLoader();

		return loader.loadCopyBook(bs, name);
	}
}
