package net.sf.RecordEditor.copy;

import javax.swing.JComponent;

import net.sf.JRecord.Common.RecordRunTimeException;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.script.CreateRecordTreePnl;
import net.sf.RecordEditor.re.util.wizard.AbstractFilePnl;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReActionHandler;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.RecordEditor.utils.wizards.AbstractWizard;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;
import net.sf.RecordEditor.xml.classes.CopyDefinition;
import net.sf.RecordEditor.xml.classes.WriteXml;

public class Copy2Xml extends AbstractWizard<CopyDefinition> {

	private CopyWizardFinalPnl finalScreen;

	public Copy2Xml(IUserLayoutSelection recordSelection1) {
		this(recordSelection1, new net.sf.RecordEditor.xml.classes.CopyDefinition(), "");
	}
	/**
	 *
	 * @param recordSelection1
	 * @param definition
	 */
	@SuppressWarnings("unchecked")
	public Copy2Xml(IUserLayoutSelection recordSelection1, CopyDefinition definition, String recentFiles) {
		super("Copy to Xml file", definition);

		AbstractWizardPanel<CopyDefinition>[] pnls = new AbstractWizardPanel[4];

		recordSelection1.setMessage(super.getMessage());

		definition.type = CopyDefinition.XML_COPY;

		finalScreen = new CopyWizardFinalPnl(recordSelection1, null);
		pnls[0] = new GetFiles(recordSelection1, recentFiles);
		pnls[1] = new FieldSelection(recordSelection1, null, "");
		pnls[2] = new CreateTree(recordSelection1);
		pnls[3] = finalScreen;

		super.setPanels(pnls);
	}


	@Override
	public void finished(CopyDefinition details) {
		if (finalScreen.isToRun()) {
			finalScreen.run();
		}
	}



	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#executeAction(int)
	 */
	@Override
	public void executeAction(int action) {
       if (action == ReActionHandler.SAVE) {
    	   try {
    		   CopyDefinition diff = super.getActivePanel().getValues();

    		   if (! "".equals(diff.saveFile)) {
       			   new WriteXml(diff.saveFile).writeCopyDefinition(diff);
 			   diff.fileSaved = true;
    		   }
    	   } catch (Exception e) {
    		   e.printStackTrace();
    		   Common.logMsgRaw(FILE_SAVE_FAILED, e);
    	   }
        } else {
            super.executeAction(action);
        }
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#isActionAvailable(int)
	 */
	@Override
	public boolean isActionAvailable(int action) {
		if (action == ReActionHandler.SAVE) {
			return true;
		}
		return super.isActionAvailable(action);
	}

	private static class CreateTree  implements AbstractWizardPanel<CopyDefinition> {


		private CreateRecordTreePnl recordTree;
		private CopyDefinition copydef;
		private IUserLayoutSelection recordSelection;

		/**
		 * @param recordSelection1
		 */
		public CreateTree(IUserLayoutSelection recordSelection1) {
			this.recordSelection = recordSelection1;
			recordTree = new CreateRecordTreePnl();
		}

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getComponent()
		 */
		@Override
		public JComponent getComponent() {
			return recordTree.getPanel();
		}

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getValues()
		 */
		@Override
		public CopyDefinition getValues() throws Exception {
			copydef.treeDefinition = recordTree.getSaveDetails().getRecordTree();
			return copydef;
		}

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#setValues(java.lang.Object)
		 */
		@Override
		public void setValues(CopyDefinition detail) throws Exception {
			copydef = detail;
			recordTree.setLayout(
					recordSelection.getRecordLayout(detail.getOldfile().name)
			);
			recordTree.setFromSavedDetails(copydef.getTreeDefinition());
			recordTree.getPanel().doLayout();
		}

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#showHelp()
		 */
		@Override
		public void showHelpRE() {
			recordTree.getPanel().showHelpRE();
		}

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#skip()
		 */
		@Override
		public boolean skip() {
			return recordTree.getLayout().getRecordCount() == 1;
		}
	}

	@SuppressWarnings("serial")
	public static class GetFiles extends  AbstractFilePnl<CopyDefinition> {

		private CopyDefinition values = new net.sf.RecordEditor.xml.classes.CopyDefinition();

//		private FileChooser xx ;
		private IFileSelectCombo xmlFileName;
		private IUserLayoutSelection layoutSelection;


		public GetFiles(IUserLayoutSelection selection, String recentFiles) {
			super(selection, recentFiles);

			//xmlFileName = new TreeComboFileSelect(true, false, true, getRecentList(), getRecentDirectoryList());
			xmlFileName = RecentFiles.newFileCombo(true, false, true, recent);
			xmlFileName.setFileNameSilently(Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getWithStar());
			layoutSelection = selection;

			//setHelpURL(Common.formatHelpURL(Common.HELP_DIFF_SL));
		}


		/**
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getValues()
		 */
		@Override
		public CopyDefinition getValues() throws Exception {

			values.getOldfile().name = getCurrentFileName();
			values.getOldfile().getLayoutDetails().name = layoutSelection.getLayoutName();

			values.getNewfile().name = xmlFileName.getText();

			if (layoutSelection.getRecordLayout(getCurrentFileName()) == null) {
				throw new RecordRunTimeException("Layout Does not exist");
			}

			return values;
		}

		/**
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#setValues(java.lang.Object)
		 */
		@Override
		public void setValues(CopyDefinition detail) throws Exception {
			System.out.println("Setting Values ... ");
			values = detail;

			if (! "".equals(values.getOldfile().name)) {
				fileNameTxt.setFileNameSilently(values.getOldfile().name);
			}

			if (! "".equals(values.getNewfile().name)) {
				xmlFileName.setFileNameSilently(values.getNewfile().name);
			}

			if (! "".equals(values.getOldfile().getLayoutDetails().name)) {
				layoutSelection.setLayoutName(values.getOldfile().getLayoutDetails().name);
			}
		}

		@Override
		protected void addFileName(BaseHelpPanel pnl) {

			pnl.addLineRE("Input File", fileNameTxt);
			pnl.addLineRE("Xml output File", xmlFileName);
		}
	}
}
