/**
 *
 */
package net.sf.RecordEditor.copy;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileOutputStream;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JTextArea;

import net.sf.RecordEditor.re.layoutSelection.def.ILayoutSelectCreator;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.screenManager.ReMainFrame;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.treeCombo.FileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.RecordEditor.xml.classes.CopyDefinition;
import net.sf.RecordEditor.xml.classes.ReadXml;

/**
 * @author Bruce Martin
 *
 */
@SuppressWarnings("serial")
public class RunSavedCopy extends ReFrame {

    private static final int WIDTH_INCREASE  = 175;
    private static final int HEIGHT_INCREASE = 7;

	private IFileSelectCombo xmlFileName = FileSelectCombo.newCombo(Parameters.SAVED_COPY_LIST, 25, false, false);
	private BaseHelpPanel pnl = new BaseHelpPanel();

	private JButton runCompareDialogBtn = SwingUtilsBldr.newButton("Run Copy Dialog");
	private JButton runCompareBtn       = SwingUtilsBldr.newButton("Run Copy");
//	private JButton runHtmlCompareBtn   = SwingUtils.newButton("Run Html Copy");

	//private JTextArea message        = new JTextArea();
	private int dbIdx;;


	private String rFiles;
//	private AbstractLayoutSelection layoutReader, layoutReader2;

	private ILayoutSelectCreator<IUserLayoutSelection> layoutCreator;

	private ActionListener listner = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {

			if (e.getSource() == runCompareDialogBtn) {
				runCopyDialog();
			} else {
				runCopy();
			}
		}

	};
	/**
	 *
	 */
	@SuppressWarnings("unchecked")
	public RunSavedCopy(@SuppressWarnings("rawtypes") ILayoutSelectCreator creator, int databaseIdx, String recentFiles) {
		super("", "Run Saved Copy File:", "", null);

		rFiles = recentFiles;

        ReMainFrame frame = ReMainFrame.getMasterFrame();
		int height = frame.getDesktop().getHeight() - 1;
		int width  = frame.getDesktop().getWidth() - 1;

		String fname =  Parameters.getFileName(Parameters.COPY_SAVE_FILE);

		layoutCreator = creator;
		dbIdx = databaseIdx;

		xmlFileName.setFileNameSilently(Parameters.getFileName(Parameters.COPY_SAVE_DIRECTORY));

		if (fname == null || "".equals(fname)) {
			fname = Parameters.getFileName(Parameters.COPY_SAVE_DIRECTORY);
		}
		xmlFileName.setFileNameSilently(fname);

		pnl.setGapRE(BasePanel.GAP3);
		pnl.addLineRE("New File", xmlFileName);
		pnl.setGapRE(BasePanel.GAP5);

		pnl.addLineRE("", BasePanel.NULL_ITEM, runCompareDialogBtn);
		pnl.setGapRE(BasePanel.GAP1);
		pnl.addLineRE("", BasePanel.NULL_ITEM, runCompareBtn);
		pnl.setGapRE(BasePanel.GAP5);

		pnl.addMessage(new JTextArea());

		super.getContentPane().add(pnl);

		super.pack();
        this.setBounds(getY(), getX(),
        		Math.min(width, getWidth() + WIDTH_INCREASE),
        		Math.min(height, getHeight()+HEIGHT_INCREASE));

		super.setVisible(true);

		runCompareDialogBtn.addActionListener(listner);
		runCompareBtn.addActionListener(listner);

	}



	/**
	 * @see net.sf.RecordEditor.utils.screenManager.ReFrame#windowClosing()
	 */
	@Override
	public final void windowClosing() {
		String s = xmlFileName.getText();
		if (! "".equals(s)) {
			try {
				Properties p = Parameters.getProperties();

				if (! s.equals(p.getProperty(Parameters.COPY_SAVE_FILE))) {
					p.setProperty(Parameters.COPY_SAVE_FILE, s);

		            p.store(
		                    new FileOutputStream(Parameters.getPropertyFileName()),
		                    "RecordEditor");
				}

			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		super.windowClosing();
	}



	/**
	 * Run the Copy Dialogue from saved XML Copy Definition
	 */
	private void runCopyDialog() {
		CopyDefinition def = readXml();

		runCopyDialog(def);
	}

	/**
	 * Run the Copy Dialogue with supplied Definition
	 * @param def Copy definition
	 */
	private void runCopyDialog(CopyDefinition def) {
		if (def != null) {
			if (CopyDefinition.STANDARD_COPY.equals(def.type)) {
				new CopyTwoLayouts(createLayout(), createLayout(), def, rFiles);
			} else if (CopyDefinition.COBOL_COPY.equals(def.type)) {
				new CobolCopy(def);
			} else if (CopyDefinition.DELIM_COPY.equals(def.type)) {
				new Copy2Delim(createLayout(), def);
			} else if (CopyDefinition.VELOCITY_COPY.equals(def.type)) {
				new Copy2Delim(createLayout(), def);
			} else if (CopyDefinition.XML_COPY.equals(def.type)) {
				new Copy2Xml(createLayout(), def, rFiles);
			}
		}
	}


	/**
	 * Run the Copy from saved XML Copy Definition
	 */
	private void runCopy() {
		CopyDefinition def = readXml();

		if (def != null) {
			if (! "Yes".equalsIgnoreCase(def.complete)) {
				runCopyDialog(def);
			} else {
				try {
						DoCopy.copy(createLayout(), createLayout(), def);

				} catch (Exception e) {
					String s = "Error Running Copy";
					e.printStackTrace();
					pnl.setMessageTxtRE(s);
					Common.logMsg(s, e);
					return;
				}
			}
		}
	}

	private IUserLayoutSelection createLayout() {
		IUserLayoutSelection ret = layoutCreator.create();
		ret.setDatabaseIdx(dbIdx);
		return ret;
	}



	private CopyDefinition readXml() {
		CopyDefinition def = null;
		String filename = xmlFileName.getText();

		if (filename == null || "".equals(filename)) {
			pnl.setMessageTxtRE("You must Enter a filename");
		} else {
			try {
				def = new ReadXml(filename).readCopyDefinition();
				def.saveFile = filename;
			} catch (Exception e) {
				String s = "Error Loading Record Layout";
				e.printStackTrace();
				pnl.setMessageTxtRE(s);
				Common.logMsg(s, e);
			}
		}

		return def;
	}
}
