/**
 *
 */
package net.sf.RecordEditor.diff;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.HtmlWindow;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.treeCombo.FileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;
import net.sf.RecordEditor.xml.classes.DiffDefinition;
import net.sf.RecordEditor.xml.classes.HtmlOptions;
import net.sf.RecordEditor.xml.classes.WriteXml;

/**
 * @author Bruce Martin
 *
 */
@SuppressWarnings("serial")
public class CmpWizardFinal extends BaseHelpPanel implements AbstractWizardPanel<DiffDefinition> {

	private IFileSelectCombo saveFileName = FileSelectCombo.newCombo(Parameters.SAVED_DIFF_LIST, 15, false, false, true);
	private IFileSelectCombo htmlFileName = FileSelectCombo.newCombo(Parameters.HTML_OUTPUT_LIST, 15, false, false, true);

	private IUserLayoutSelection selection;
	private IUserLayoutSelection selection2;

	private JButton saveBtn = SwingUtilsBldr.newButton("Save", Common.getRecordIcon(Common.ID_SAVE_ICON));
	private JButton runBtn  = SwingUtilsBldr.newButton("Compare");
	private JButton htmlBtn = SwingUtilsBldr.newButton("Compare (HTML)");

	private JRadioButton stripTrailingSpaces = SwingUtilsBldr.newRadioButton("Strip Trailing Spaces");
	private JRadioButton singleTblBtn    = SwingUtilsBldr.newRadioButton("Single Table");
	private JRadioButton multipleTblsBtn = SwingUtilsBldr.newRadioButton("Table per row");

	private JRadioButton allRows = SwingUtilsBldr.newRadioButton("All Rows");
	private JRadioButton chgRows = SwingUtilsBldr.newRadioButton("Only Changed Rows");

	private JRadioButton allFieldsBtn = SwingUtilsBldr.newRadioButton("All Fields");
	private JRadioButton chgFieldsBtn = SwingUtilsBldr.newRadioButton("Only Changed Fields");

	private JCheckBox showHtmlCheckBox = SwingUtilsBldr.newCheckBox("Show Html");

	private JTextField message        = new JTextField();


	private DiffDefinition diffDetail;

	private boolean toRun = true;


	private AbstractAction btnAction = new AbstractAction() {
		@Override
		public void actionPerformed(ActionEvent e) {

			if (e.getSource() == runBtn) {
				run();
			} else if (e.getSource() == htmlBtn) {
				runHtml();
			} else {
				save();
			}
		}

	};


	/**
	 * Final wizard Screen
	 */
	public CmpWizardFinal(IUserLayoutSelection layoutSelection,
			IUserLayoutSelection layoutSelection2, boolean offerSaveOption) {
		super();
		JPanel pnl = new JPanel(new GridLayout(3,2));
		//JPanel pnlStrip = new JPanel();

		String fileName = Parameters.getFileName(Parameters.COMPARE_SAVE_DIRECTORY);
		saveFileName.setFileNameSilently(fileName);
//		saveFileName.setDefaultDirectory(Parameters.getFileName(Parameters.COMPARE_SAVE_DIRECTORY));
		htmlFileName.setFileNameSilently(Parameters.getHtmlDirectory());


		selection = layoutSelection ;
		selection2 = layoutSelection2;


		showHtmlCheckBox.setSelected(true);
		
		saveBtn.addActionListener(btnAction);
		runBtn.addActionListener(btnAction);
		htmlBtn.addActionListener(btnAction);

		super.setHelpURLre(Common.formatHelpURL(Common.HELP_DIFF_SL));

		if (offerSaveOption) {
			super.addLineRE("Save File", saveFileName);
			super.setGapRE(BasePanel.GAP1);
		}
		super.addLineRE("Html File", htmlFileName);
		super.setGapRE(BasePanel.GAP1);
		super.addLineRE("", stripTrailingSpaces);
		super.setGapRE(BasePanel.GAP1);
		super.addLineRE("HTML options:", BasePanel.NULL_ITEM);

		addRadioGrp(pnl, singleTblBtn, multipleTblsBtn);
		addRadioGrp(pnl, allRows, chgRows);
		addRadioGrp(pnl, allFieldsBtn, chgFieldsBtn);

		super.addLineRE("", pnl);
		super.setHeightRE(BasePanel.GAP5);

		if (offerSaveOption) {
			super.addLineRE("", BasePanel.NULL_ITEM, saveBtn);
		}
		super.setGapRE(BasePanel.GAP0);
		super.addLineRE("", BasePanel.NULL_ITEM, runBtn);
		super.setGapRE(BasePanel.GAP0);
		super.addLineRE("", showHtmlCheckBox, htmlBtn);
		super.setGapRE(BasePanel.GAP2);
		super.addMessage(message);
		super.setHeightRE(BasePanel.HEIGHT_1P6);
	}

	private void addRadioGrp(JPanel pnl, JRadioButton btn1, JRadioButton btn2) {
		ButtonGroup grp = new ButtonGroup();
		grp.add(btn1);
		grp.add(btn2);

		pnl.add(btn1);
		pnl.add(btn2);
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getComponent()
	 */
	@Override
	public final JComponent getComponent() {
		return this;
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getValues()
	 */
	@Override
	public final DiffDefinition getValues() throws Exception {

		if (! "".equals(saveFileName.getText())) {
			diffDetail.saveFile = saveFileName.getText();
		}
		HtmlOptions htmlOptions = diffDetail.getHtmlOptions();
		if (! "".equals(htmlFileName.getText())) {
			htmlOptions.htmlFile = htmlFileName.getText();
		}

		diffDetail.stripTrailingSpaces = stripTrailingSpaces.isSelected();
		htmlOptions.allFields = allFieldsBtn.isSelected();
		htmlOptions.allRows   = allRows.isSelected();
		htmlOptions.singleTable = singleTblBtn.isSelected();
		//System.out.println("Final Get -Layout Name: " + diffDetail.getLayoutDetails().name);

		return diffDetail;
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#isMore()
	 */
	@Override
	public final boolean skip() {
		return false;
	}



	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#setValues(java.lang.Object)
	 */
	@Override
	public final void setValues(DiffDefinition detail) throws Exception {

		String htmlFile = detail.getHtmlOptions().htmlFile;
		
		if (detail.saveFile != null && detail.saveFile.length() > 0) {
			saveFileName.setFileNameSilently(detail.saveFile);
		}
		if (htmlFile != null && htmlFile.length() > 0) {
			htmlFileName.setFileNameSilently(htmlFile);
		}
		diffDetail = detail;
		diffDetail.fileSaved = false;
		diffDetail.complete = "YES";

		stripTrailingSpaces.setSelected(diffDetail.stripTrailingSpaces);
		allFieldsBtn.setSelected(diffDetail.getHtmlOptions().allFields);
		chgFieldsBtn.setSelected(! diffDetail.getHtmlOptions().allFields);
		allRows.setSelected(diffDetail.getHtmlOptions().allRows);
		chgRows.setSelected(! diffDetail.getHtmlOptions().allRows);
		singleTblBtn.setSelected(diffDetail.getHtmlOptions().singleTable);
		multipleTblsBtn.setSelected(! diffDetail.getHtmlOptions().singleTable);
	}

	public final void run() {

		try {
			message.setText("");
			diffDetail = getValues();

			DoCompare.getInstance().compare(selection, selection2, diffDetail);

			//toRun = false;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}



	public final void runHtml() {

		String htmlFileNameStr = htmlFileName.getText();
		if ("".equals(htmlFileNameStr)) {
			setMessageTxtRE("You must enter a HTML file");
			htmlFileName.requestFocus();
		} else {
			try {
				message.setText("");
				diffDetail = getValues();
				
				File htmlFile = new File(htmlFileNameStr);
				
				if (htmlFile.isDirectory()) {
					message.setText("You can not write Html to a directory");
					htmlFileName.requestFocus();
				} else {
					DoCompare.getInstance().writeHtml(selection, selection2, diffDetail);
					
					Parameters.setHtmlDirectory(htmlFile.getParent());
	
					if (showHtmlCheckBox.isSelected()) {
						new HtmlWindow(diffDetail.getHtmlOptions().htmlFile, "Generated Html");
					}
				}
				//toRun = false;
			} catch (Exception e) {
				e.printStackTrace();
				Common.logMsg("Error generating HTML", e);
			}
		}
	}


	/**
	 * Save the file
	 */
	public final void save() {
		try {
			diffDetail = getValues();

			//jibx.unmarshal(diffDetail.saveFile, diffDetail);
			new WriteXml(diffDetail.saveFile).writeDiffDefinition(diffDetail);
			diffDetail.fileSaved = true;
		} catch (Exception e) {
 		   e.printStackTrace();
		   Common.logMsgRaw(Common.FILE_SAVE_FAILED, e);
		}
	}

	/**
	 * @return the toRun
	 */
	public final boolean isToRun() {
		return toRun;
	}
}
