/**
 *
 */
package net.sf.RecordEditor.diff;

import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.util.wizard.AbstractFilePnl;
import net.sf.RecordEditor.re.util.wizard.FieldChoice;
import net.sf.RecordEditor.utils.IGetFileName;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReActionHandler;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.RecordEditor.utils.wizards.AbstractWizard;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;
import net.sf.RecordEditor.xml.classes.DiffDefinition;
import net.sf.RecordEditor.xml.classes.WriteXml;

/**
 * @author Bruce Martin
 *
 */
public class CompareInternalLayout extends AbstractWizard<DiffDefinition> {


	private CmpWizardFinal finalScreen;

	/**
	 * Create Single layout
	 * @param selection1 record layout selection class 1
	 * @param selection2 record layout selection class 2
	 * @param recentFiles Recent files holder file
	 */
	public CompareInternalLayout(
			IUserLayoutSelection selection1, IUserLayoutSelection selection2,
			String recentFiles) {
		this(selection1,
			 selection2,
			 new net.sf.RecordEditor.xml.classes.DiffDefinition(),
			 recentFiles);
	}


	/**
	 * Create Single layout
	 * @param selection1 record layout selection class 1
	 * @param selection2 record layout selection class 2
	 * @param definition record filter definition
	 * @param recentFiles Recent files holder file
	 */
	@SuppressWarnings("unchecked")
	public CompareInternalLayout(
			IUserLayoutSelection selection1, IUserLayoutSelection selection2,
			DiffDefinition definition, String recentFiles) {
		super("File Compare", definition);

		AbstractWizardPanel<DiffDefinition>[] pnls = new AbstractWizardPanel[3];

		selection1.setMessage(super.getMessage());

		definition.type = DiffDefinition.TYPE_TWO_LAYOUTS;

		finalScreen = new CmpWizardFinal(selection1, selection2, true);
		pnls[0] = new GetFiles(selection1, selection2, recentFiles);
		pnls[1] = new FieldChoice<DiffDefinition>(selection1, selection2, "");
		pnls[2] = finalScreen;

		super.setPanels(pnls);
	}



	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#finished(java.lang.Object)
	 */
	@Override
	public void finished(DiffDefinition details) {

		if (finalScreen.isToRun()) {
			finalScreen.run();
		}
	}




	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#executeAction(int)
	 */
	@Override
	public void executeAction(int action) {
       if (action == ReActionHandler.SAVE) {
    	   try {
    		   DiffDefinition diff = super.getActivePanel().getValues();

    		   if (! "".equals(diff.saveFile)) {
    			   new WriteXml(diff.saveFile).writeDiffDefinition(diff);
    			   diff.fileSaved = true;
    		   }
    	   } catch (Exception e) {
    		   e.printStackTrace();
    		   Common.logMsgRaw(Common.FILE_SAVE_FAILED, e);
    	   }
        } else {
            super.executeAction(action);
        }
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#isActionAvailable(int)
	 */
	@Override
	public boolean isActionAvailable(int action) {
		if (action == ReActionHandler.SAVE) {
			return true;
		}
		return super.isActionAvailable(action);
	}

//	/**
//	 * get Jibx interface
//	 * @return Jibx interface
//	 */
//	private JibxCall<DiffDefinition> getJibx() {
//	   if (jibx == null) {
//		   jibx = new JibxCall<DiffDefinition>(DiffDefinition.class);
//	   }
//	   return jibx;
//	}


	@SuppressWarnings("serial")
	public static class GetFiles extends  AbstractFilePnl<DiffDefinition> {



		private DiffDefinition values
			= new net.sf.RecordEditor.xml.classes.DiffDefinition();
//		private JPanel goPanel = new JPanel();
		private IFileSelectCombo newFileName = RecentFiles.newFileCombo(true, false, true, recent);
		private IUserLayoutSelection layoutSelection1;
		private IUserLayoutSelection layoutSelection2;


		public GetFiles(IUserLayoutSelection selection1, IUserLayoutSelection selection2, String recentFiles) {
			super(selection1, recentFiles);

			newFileName.setFileNameSilently(Common.OPTIONS.DEFAULT_FILE_DIRECTORY.get());
			layoutSelection1 = selection1;
			layoutSelection2 = selection2;

			layoutSelection1.setFileNameField(new IGetFileName() {
				@Override public String getCurrentFileName() {
					return fileNameTxt.getText();
				}
			});
					
			layoutSelection2.setFileNameField(new IGetFileName() {
				@Override public String getCurrentFileName() {
					return newFileName.getText();
				}
			});

			setHelpURLre(Common.formatHelpURL(Common.HELP_DIFF_SL));
		}


		/**
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getValues()
		 */
		@Override
		public DiffDefinition getValues() throws Exception {

			values.getOldfile().name = getCurrentFileName();
			values.getNewfile().name = newFileName.getText();

			values.getOldfile().getLayoutDetails().name = layoutSelection1.getLayoutName();
			values.getNewfile().getLayoutDetails().name = layoutSelection2.getLayoutName();

			return values;
		}

		/**
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#setValues(java.lang.Object)
		 */
		@Override
		public void setValues(DiffDefinition detail) throws Exception {
			System.out.println("Setting Values ... ");
			values = detail;

			layoutSelection1.setFileNameField(new IGetFileName() {
				@Override public String getCurrentFileName() {
					return fileNameTxt.getText();
				}
			});
					//fileName);
			layoutSelection2.setFileNameField(new IGetFileName() {
				@Override public String getCurrentFileName() {
					return newFileName.getText();
				}
			});
					//newFileName);
			if (! "".equals(values.getOldfile().name)) {
				fileNameTxt.setFileNameSilently(values.getOldfile().name);
			}

			if (! "".equals(values.getNewfile().name)) {
				newFileName.setFileNameSilently(values.getNewfile().name);
			}

//			if (! "".equals(values.oldFile.getLayoutDetails().name)) {
//				layoutSelection1.setLayoutName(values.oldFile.getLayoutDetails().name);
//			}
		}

		@Override
		protected void addFileName(BaseHelpPanel pnl) {

//			pnl.addLine("Old File", fileName, fileName.getChooseFileButton());
			pnl.addLineRE("Old File", fileNameTxt);
			pnl.addLineRE("New File", newFileName);
		}
	}

//	/**
//	 * @param args
//	 */
//	public static void main(String[] args) {
//
//		 new ReMainFrame("File Compare", "");
//		 new CmpSingleLayout(new LayoutSelectionDB(new CopyBookDbReader()));
//	}

}
