/**
 *
 */
package net.sf.RecordEditor.diff;

import java.io.File;

import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.util.wizard.AbstractFilePnl;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReActionHandler;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.RecordEditor.utils.wizards.AbstractWizard;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;
import net.sf.RecordEditor.xml.classes.DiffDefinition;
import net.sf.RecordEditor.xml.classes.WriteXml;

/**
 * @author Bruce Martin
 *
 */
public class CompareSingleLayout extends AbstractWizard<DiffDefinition> {


	private CmpWizardFinal finalScreen;
//	private JibxCall<DiffDefinition> jibx = null;

	/**
	 * Single layout Compare Wizard
	 * @param selection record layout selection class
	 * @param recentFiles recent files file
	 */
	public CompareSingleLayout(IUserLayoutSelection selection, String recentFiles) {
		this(
			 selection,
			 new net.sf.RecordEditor.xml.classes.DiffDefinition(),
			 recentFiles);
	}

	/**
	 * Single layout Compare Wizard
	 * @param name compare wizard name
	 * @param selection record layout selection class
	 * @param recentFiles recent files file
	 */
	public CompareSingleLayout(String name, IUserLayoutSelection selection, String recentFiles) {
		this(name,
			 selection,
			 new net.sf.RecordEditor.xml.classes.DiffDefinition(),
			 recentFiles);
	}


	/**
	 * Single layout Compare Wizard
	 * @param selection record layout selection class
	 * @param definition record filter definition
	 * @param recentFiles recent files file
	 */
	public CompareSingleLayout(IUserLayoutSelection selection, DiffDefinition definition, String recentFiles) {
		this("Single Layout Compare", selection, definition, recentFiles);
	}
	
	
	/**
	 * Single layout Compare Wizard
	 * @param name compare wizard name
	 * @param selection record layout selection class
	 * @param definition record filter definition
	 * @param recentFiles recent files file
	 */
	@SuppressWarnings("unchecked")
	private CompareSingleLayout(String name, IUserLayoutSelection selection, DiffDefinition definition, String recentFiles) {
		super(name, definition);

		AbstractWizardPanel<DiffDefinition>[] pnls = new AbstractWizardPanel[3];

		selection.setMessage(super.getMessage());

		definition.type = DiffDefinition.TYPE_SINGLE_LAYOUT;

		finalScreen = new CmpWizardFinal(selection, null, true);
		pnls[0] = new GetFiles(selection, recentFiles);
		pnls[1] = new CmpFieldSelection(selection);
		pnls[2] = finalScreen;

		super.setPanels(pnls);
	}



	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#finished(java.lang.Object)
	 */
	@Override
	public void finished(DiffDefinition details) {

		if (finalScreen.isToRun()) {
			finalScreen.run();
		}
	}


	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#executeAction(int)
	 */
	@Override
	public void executeAction(int action) {
       if (action == ReActionHandler.SAVE) {
    	   try {
    		   DiffDefinition diff = super.getActivePanel().getValues();

    		   if (! "".equals(diff.saveFile)) {
       			   new WriteXml(diff.saveFile).writeDiffDefinition(diff);
       			   diff.fileSaved = true;
    		   }
    	   } catch (Exception e) {
    		   e.printStackTrace();
    		   Common.logMsgRaw(Common.FILE_SAVE_FAILED, e);
    	   }
        } else {
            super.executeAction(action);
        }
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#isActionAvailable(int)
	 */
	@Override
	public boolean isActionAvailable(int action) {
		if (action == ReActionHandler.SAVE) {
			return true;
		}
		return super.isActionAvailable(action);
	}




	@SuppressWarnings("serial")
	public static class GetFiles extends  AbstractFilePnl<DiffDefinition> {



		private DiffDefinition values = new net.sf.RecordEditor.xml.classes.DiffDefinition();
//		private JPanel goPanel = new JPanel();
//		private FileChooser newFileName = new FileChooser();
		private IFileSelectCombo newFileName = RecentFiles.newFileComboFav(true, false, true, recent);
		private IUserLayoutSelection layoutSelection;


		public GetFiles(IUserLayoutSelection selection, String recentFiles) {
			super(selection, recentFiles);

			newFileName.setFileNameSilently(Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getWithStar());
			layoutSelection = selection;

			setHelpURLre(Common.formatHelpURL(Common.HELP_DIFF_SL));
		}


		/**
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getValues()
		 */
		@Override
		public DiffDefinition getValues() throws Exception {

			values.getOldfile().name = getCurrentFileName();
			values.getNewfile().name = newFileName.getText();

			values.getLayoutDetails().name = layoutSelection.getLayoutName();
			if (layoutSelection.getRecordLayout(getCurrentFileName()) == null) {
				throw new RuntimeException("Layout Does not exist");
			} else if (! new File(values.getOldfile().name).exists()) {
				throw new RuntimeException("Layout Does not exist");
			}
			checkFile(values.getOldfile().name, super.fileNameTxt);
			checkFile(values.getNewfile().name, newFileName);
			
			return values;
		}
		

		/**
		 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#setValues(java.lang.Object)
		 */
		@Override
		public void setValues(DiffDefinition detail) throws Exception {
			System.out.println("Setting Values ... ");
			values = detail;

			if (! "".equals(values.getOldfile().name)) {
				fileNameTxt.setFileNameSilently(values.getOldfile().name);
			}

			if (! "".equals(values.getNewfile().name)) {
				newFileName.setFileNameSilently(values.getNewfile().name);
			}

			if (! "".equals(values.getLayoutDetails().name)) {
				layoutSelection.setLayoutName(values.getLayoutDetails().name);
			}
		}

		@Override
		protected void addFileName(BaseHelpPanel pnl) {

			pnl.addLineRE("Old File", fileNameTxt);
//			pnl.addLine("New File", newFileName, newFileName.getChooseFileButton());
			pnl.addLineRE("New File", newFileName);
		}
	}

//	/**
//	 * @param args
//	 */
//	public static void main(String[] args) {
//
//		 new ReMainFrame("File Compare", "");
//		 new CmpSingleLayout(new LayoutSelectionDB(new CopyBookDbReader()));
//	}

}
