/**
 * 
 */
package net.sf.RecordEditor.diff;


import java.io.FileOutputStream;

import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.util.wizard.StandardGetFiles;
import net.sf.RecordEditor.re.util.wizard.TwoLayoutsWizard;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.xml.classes.DiffDefinition;
import net.sf.RecordEditor.xml.classes.WriteXml;

/**
 * @author Bruce Martin
 *
 */
public class CompareTwoLayouts extends TwoLayoutsWizard<DiffDefinition> {

	public static final String TWO_LAYOUT_COMPARE = "Two Layout Compare";
	private CmpWizardFinal finalScreen;
	
	public static CompareTwoLayouts layoutToCsvCompare(
			IUserLayoutSelection selection1, IUserLayoutSelection selection2,
			String recentFiles, boolean lookupRecentLayouts) {
		return new CompareTwoLayouts(
				"FixedWidth to Csv Compares", 
				selection1, selection2, recentFiles, lookupRecentLayouts,  
				StandardGetFiles.FP_LAYOUT_TO_CSV);
	}
	public static CompareTwoLayouts cmp2Files(
			IUserLayoutSelection selection1, IUserLayoutSelection selection2,
			String recentFiles, boolean lookupRecentLayouts) {
		return new CompareTwoLayouts(
				TWO_LAYOUT_COMPARE, 
				selection1, selection2, recentFiles, lookupRecentLayouts,  
				null);
	}
	public static CompareTwoLayouts cmp2CsvFiles(
			IUserLayoutSelection selection1, IUserLayoutSelection selection2,
			String recentFiles, boolean lookupRecentLayouts) {
		return new CompareTwoLayouts(
				"Compare 2 Csv Files", 
				selection1, selection2, recentFiles, lookupRecentLayouts,  
				null);
	}
	
	/**
	 * Create Single layout
	 * @param selection record layout selection class
	 */
	public CompareTwoLayouts(
			String pnlName,
			IUserLayoutSelection selection1, IUserLayoutSelection selection2,
			String recentFiles, boolean lookupRecentLayouts,
			StandardGetFiles.FileId[] promptDtls) {
		this(
			 pnlName,
			 selection1, 
			 selection2, 
			 new net.sf.RecordEditor.xml.classes.DiffDefinition(), 
			 recentFiles,
			 lookupRecentLayouts,
			 promptDtls);
	}
	
	
	/**
	 * Create Single layout
	 * @param selection1 record layout selection class
	 * @param definition record filter definition
	 */

	public CompareTwoLayouts(String pnlName, IUserLayoutSelection selection1, IUserLayoutSelection selection2, 
			DiffDefinition definition, String recentFiles, boolean lookupRecentLayouts,
			StandardGetFiles.FileId[] promptDtls) {
		super(pnlName, definition); 
		
		definition.type = DiffDefinition.TYPE_TWO_LAYOUTS;
		
		finalScreen = new CmpWizardFinal(selection1, selection2, lookupRecentLayouts);
		
		super.setUpPanels(selection1, selection2, recentFiles, finalScreen, 
				Common.HELP_DIFF_TL, true, lookupRecentLayouts, promptDtls);
	}



	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#finished(java.lang.Object)
	 */
	@Override
	public void finished(DiffDefinition details) {
		
		if (finalScreen.isToRun()) {
			finalScreen.run();
		}
	}


	@Override
	protected void saveAsXml(DiffDefinition save) throws Exception {
		
		new WriteXml(new FileOutputStream(save.saveFile)).writeDiffDefinition(save);
				
		
	}
	
	
}