/*
 * Created on 5/09/2004
 *
 * Changes
 * # Version 0.56 Bruce Martin 2007/01/16
 *   - changed to use ReFrame (from JInternalFrame) which keeps track
 *     of the active form
 *   - Added new Create Layout and Create Layout Wizard
 *
 * # Version 0.61 Bruce Martin 2007/04/14
 *   - JRecord Creation
 *   - Code cleanup
 */
package net.sf.RecordEditor.diff;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;

import net.sf.RecordEditor.diff.xml.XmlCompareOptions;
import net.sf.RecordEditor.re.layoutSelection.LayoutSelectionCsvCreator;
import net.sf.RecordEditor.re.layoutSelection.LayoutSelectionPoTipCreator;
import net.sf.RecordEditor.re.layoutSelection.def.ILayoutSelectCreator;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReActionHandler;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.MenuLblMouseAdapter;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;



/**
 * This class display the Compare Menu
 * <p><p>
 *
 * @author Bruce Martin
 * @version 0.65
 */
@SuppressWarnings("serial")
public class Menu extends ReFrame
               implements ActionListener {

    private static final int MENU_WIDTH = SwingUtils.values().STANDARD_FONT_WIDTH * 57;
    private static final int HELP_GAP   = 15;

	private JComboBox  dbCombo    = new JComboBox();

	private JButton fromFileBtn   = new JButton("*");
	private JButton singleLayout  = new JButton("*");
	private JButton twoLayouts    = new JButton("*");
	private JButton csvCmp        = new JButton("*");
	private JButton csvDbCmp      = new JButton("*");
	private JButton poCmpBtn      = new JButton("*");
	private JButton tipCmpBtn     = new JButton("*");
	private JButton xmlFileCmpBtn = new JButton("*");
	private JButton xmlDirCmpBtn  = new JButton("*");

	private JButton btnHelp       = SwingUtilsBldr.getHelpButton();

	private BaseHelpPanel pnl     = new BaseHelpPanel();

	private String rFiles;

	//private AbstractLayoutSelection selection;
	@SuppressWarnings("rawtypes")
	private ILayoutSelectCreator layoutCreator;




	/**
	 * display a menu of actions to the user
	 *
	 * @param parentFrame parent frame
	 */
	@SuppressWarnings("unchecked")
	public Menu(@SuppressWarnings("rawtypes") ILayoutSelectCreator creator, String recentFiles)  {
		super("", "Menu", "Compare Menu", null);
		String[] dbs;

		layoutCreator = creator;
		rFiles = recentFiles;
		IUserLayoutSelection selection = layoutCreator.create();


		pnl.setVerticalGap(BasePanel.VG_GAP2, HELP_GAP);

		pnl.setHelpURLre(Common.formatHelpURL(Common.HELP_DIFF));

		dbs = selection.getDataBaseNames();

		pnl.setGapRE(BasePanel.GAP1);
		if (dbs != null) {
			for (int i = 0; (i < dbs.length) && (dbs[i] != null) && (!dbs[i].equals("")); i++) {
				dbCombo.addItem(dbs[i]);
			}
			dbCombo.setSelectedIndex(Common.getConnectionIndex());
			pnl.addLineRE("Data Base", dbCombo, btnHelp);
		}

		pnl.setGapRE(BasePanel.GAP3);

		addMenuName("Run Stored Compare", fromFileBtn);
		pnl.setGapRE(BasePanel.GAP2);
		addMenuName("Compare Single Layout", singleLayout);
		pnl.setGapRE(BasePanel.GAP1);
		addMenuName("Compare Different Layouts", twoLayouts);
		pnl.setGapRE(BasePanel.GAP2);
		addMenuName("Csv Compare", csvCmp);
		pnl.setGapRE(BasePanel.GAP1);
		addMenuName("Layout to Csv Compare", csvDbCmp);
		pnl.setGapRE(BasePanel.GAP2);
		addMenuName("GetText-Po Compare", poCmpBtn);
		pnl.setGapRE(BasePanel.GAP1);
		addMenuName("SwingX-Tip Compare", tipCmpBtn);
		pnl.setGapRE(BasePanel.GAP2);
		addMenuName("Xml File Compare", xmlFileCmpBtn);
		pnl.setGapRE(BasePanel.GAP1);
		addMenuName("Xml Directory Compare", xmlDirCmpBtn);
		
		pnl.setGapRE(BasePanel.GAP3);


		btnHelp.addActionListener(this);
		fromFileBtn.addActionListener(this);
		singleLayout.addActionListener(this);
		twoLayouts.addActionListener(this);
		csvCmp.addActionListener(this);
		csvDbCmp.addActionListener(this);
		tipCmpBtn.addActionListener(this);
		poCmpBtn.addActionListener(this);
		xmlFileCmpBtn.addActionListener(this);
		xmlDirCmpBtn.addActionListener(this);

		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		dbCombo.requestFocusInWindow();

		//pnl.done();

		this.getContentPane().add(pnl);

		pack();

		setBounds(getY(), getX(), MENU_WIDTH, getHeight());

		show();

		dbCombo.addActionListener(this);
	}

	
	private void addMenuName(String msg, JButton btn) {
		MenuLblMouseAdapter.addMenuItem(pnl, this, msg, btn);
	}

	/**
	 * @see java.awt.event.ActionListner#actionPerformed
	 */
	public void actionPerformed(ActionEvent e) {

//		String lDBid = dbCombo.getSelectedItem().toString();
		Object source = e.getSource();
		if (source == fromFileBtn) {
			new RunSavedCompare(layoutCreator, dbCombo.getSelectedIndex(), rFiles);
		} else if (source == btnHelp) {
		    pnl.showHelpRE();
		} else if (source == dbCombo) {
		    Common.setConnectionId(dbCombo.getSelectedIndex());
		} else if (source == csvCmp) {
			LayoutSelectionCsvCreator csvl = new LayoutSelectionCsvCreator();
			CompareTwoLayouts.cmp2CsvFiles(csvl.create(), csvl.create(), rFiles, false);
		} else if (source == csvDbCmp) {
			LayoutSelectionCsvCreator csvl = new LayoutSelectionCsvCreator();
			IUserLayoutSelection selection1 = layoutCreator.create();
			selection1.setDatabaseIdx(dbCombo.getSelectedIndex());

			CompareTwoLayouts.layoutToCsvCompare(selection1, csvl.create(), rFiles, false);
		} else if (source == poCmpBtn) {
			LayoutSelectionPoTipCreator newPoCreator = LayoutSelectionPoTipCreator.newPoCreator();
			new CompareSingleLayout("GetText-PO Compare Wizard -", newPoCreator.create(), rFiles);
		} else if (source == tipCmpBtn) {
			LayoutSelectionPoTipCreator newTipCreator = LayoutSelectionPoTipCreator.newTipCreator();
			new CompareSingleLayout("SwingX-Tip Compare Wizard -", newTipCreator.create(), rFiles);
		} else if (source == xmlFileCmpBtn) {
			new XmlCompareOptions(false);
		} else if (source == xmlDirCmpBtn) {
			new XmlCompareOptions(true);
		} else {
			IUserLayoutSelection selection = layoutCreator.create();
			selection.setDatabaseIdx(dbCombo.getSelectedIndex());

			if (source == singleLayout) {
				new CompareSingleLayout(selection, rFiles);
			} else if (source == twoLayouts) {
				IUserLayoutSelection selection1 = layoutCreator.create();
				selection1.setDatabaseIdx(dbCombo.getSelectedIndex());

				CompareTwoLayouts.cmp2Files(selection, selection1, rFiles, true);
			}
 		}
		//this.moveToBack();
	}


    /**
     * @see net.sf.RecordEditor.utils.common.ReActionHandler#executeAction(int)
     */
    public void executeAction(int action) {

        if (action == ReActionHandler.HELP) {
            pnl.showHelpRE();
        }
        //super.executeAction(action);
    }


    /**
     * @see net.sf.RecordEditor.utils.common.ReActionHandler#isActionAvailable(int)
     */
    public boolean isActionAvailable(int action) {

        return action == ReActionHandler.HELP;
    }


}
