package net.sf.RecordEditor.diff;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Common.IFieldDetail;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractRecordDetail;
import net.sf.RecordEditor.diff.result.ICompareResult;
import net.sf.RecordEditor.diff.result.ILineCompare;
import net.sf.RecordEditor.diff.result.LineCompareResult;
//import net.sf.RecordEditor.xml.classes.DiffDefinition;
import net.sf.RecordEditor.utils.common.Common;

public class WriteHtml {

	private static final boolean AFTER = false;
	private static final boolean BEFORE = true;
	private final static String HTML_START="<Body><h1>Formated Compare</h1>";
	private final static String HTML_SINGLE_TBL_START="<p>&nbsp;<p><table border=\"1\" cellpadding=\"2\">"
				+ "<tr><td><b>What</b></td><td><b>Line</b></td>";
	private final static String HTML_MANY_TBL_START="<p>&nbsp;<p>&nbsp;<table border=\"1\" cellpadding=\"2\">"
				+ "<tr><td><b>Field Name</b></td><td><b>Pos</b></td><td><b>Length</b></td>"
				+ "<td><b>Old Value</b></td><td><b>New Value</b></td></tr>";
	private final static String HTML_LINE_NUMBER = "<TR><td>Line Number</td><td>&nbsp;</td><td>&nbsp;</td>";
//	private final static String NEW_COLUMN = "</td>";
	private final static String HTML_END="</td></tr></table></body>";

//	private static WriteHtml instance = new WriteHtml();

	private final String newFileName, oldFileName, htmlFileName;
	
	

	public WriteHtml(String newFileName, String oldFileName, String htmlFileName) {
		super();
		this.newFileName = newFileName;
		this.oldFileName = oldFileName;
		this.htmlFileName = htmlFileName;
	}

	public final void writeSingleTbl(
			AbstractLayoutDetails recordLayout,
			ICompareResult cmpResult)
			//ArrayList<LineCompare> before, ArrayList<LineCompare> after)
	throws IOException {
		int i, j, pref;
		String f1, f2;
		BufferedWriter writer = new BufferedWriter(new FileWriter(htmlFileName));
		AbstractRecordDetail rec;
		ILineCompare old, newL;

		htmlStart(writer);
		writer.write(HTML_SINGLE_TBL_START); 		
		writer.newLine();

		rec = recordLayout.getRecord(0);
		for (i = 0; i < rec.getFieldCount(); i++) {
			writer.write("<td><b>" + rec.getField(i).getName() + "</b></td>");
		}
		writer.write("</TR>");		
		writer.newLine();

		for (i = 0; i < cmpResult.getRowCount(); i++) {
			old = cmpResult.get(BEFORE, i);
			newL = cmpResult.get(AFTER, i);
			writeLineStart(writer, "old", old);
			if (old == null) {
				pref = newL.getLine().getPreferredLayoutIdx();
				writeBlankFields(writer, pref, recordLayout);
			} else {
				pref = old.getLine().getPreferredLayoutIdx();
				if (old.getCompareResult() == LineCompareResult.DELETED || newL == null || newL.getLine() == null) {
					writeStandardFields(writer, "cyan", pref, recordLayout, old);
				} else {
					for (j = 0; j < recordLayout.getRecord(pref).getFieldCount(); j++) {
						f1 = old.getLine().getFieldValue(pref, j).asString();
						f2 = newL.getLine().getFieldValue(pref, j).asString();

						if (f1.equals(f2)) {
							writer.write("<TD>" + f1 + "</td>");
						} else {
							writer.write("<TD bgcolor=lime><b>" + f1 + "</b></td>");
						}
					}
				}
			}

			writeLineStart(writer, "new", newL);
			if (newL == null) {
				writeBlankFields(writer, pref, recordLayout);
			} else {
				if (newL.getCompareResult() == LineCompareResult.INSERT || old == null || old.getLine() == null) {
					writeStandardFields(writer, "yellow", pref, recordLayout, newL);
				} else {
					if (recordLayout.isMapPresent()) {
						f1 = old.getLine().getFieldValue(pref, Constants.KEY_INDEX).asString();
						f2 = newL.getLine().getFieldValue(pref, Constants.KEY_INDEX).asString();
						writeField(writer, f1, f2);
					}
					for (j = 0; j < recordLayout.getRecord(pref).getFieldCount(); j++) {
						f1 = old.getLine().getFieldValue(pref, j).asString();
						f2 = newL.getLine().getFieldValue(pref, j).asString();
						writeField(writer, f1, f2);
					}
				}

			}
		}
		writer.write(HTML_END);
		writer.close();
	}

	private void writeField(BufferedWriter writer, String f1, String f2) throws IOException {

		if (f1.equals(f2)) {
			writer.write("<TD>&nbsp;</td>");
		} else {
			writer.write("<TD bgcolor=lime><b>" + f2 + "</b></td>");
		}

	}

	private void writeLineStart(BufferedWriter writer, String what, ILineCompare cmp)
	throws IOException {

		writer.write("<tr><td>" + what + "</td><td>");
		if (cmp != null) {
			writer.write(cmp.getLineNumber() + "</td>" );
		} else {
			writer.write("&nbsp;</td>");
		}
		writer.newLine();
	}

	private void writeStandardFields(BufferedWriter writer, String color, int pref,
			AbstractLayoutDetails recordLayout, ILineCompare cmp)
	throws IOException {

		String start    = "<TD>";
		String endField = "</td>";

		if (! "".equals(color)) {
			start    = "<TD bgcolor=\"" + color + "\"><b>";
			endField = "</b></td>";

		}

		for (int j = 0; j < recordLayout.getRecord(pref).getFieldCount(); j++) {
			writer.write(start + cmp.getLine().getField(pref, j) + endField);
		}

		writer.write("</tr>"); writer.newLine();
	}


	private void writeBlankFields(BufferedWriter writer, int pref,
			AbstractLayoutDetails recordLayout)
	throws IOException {
		if (recordLayout.isMapPresent()) {
			writer.write("<td>&nbsp;</td>");
		}

		for (int j = 0; j < recordLayout.getRecord(pref).getFieldCount(); j++) {
			writer.write("<td>&nbsp;</td>");
		}
	}

	/**
	 * Write Compare result as HTML with each row written as a individual table.
	 * All fields are to be displayed
	 *
	 * @param filename output file name
	 * @param recordLayout record layout
	 * @param before before file details
	 * @param after  after  file details
	 * @throws IOException any IO Errors
	 */
	public final void writeTblChgFields(
			AbstractLayoutDetails recordLayout,
			ICompareResult cmpResult)
//			ArrayList<LineCompare> before, ArrayList<LineCompare> after)
	throws IOException {
		int i, pref;
		System.out.println("--> " + htmlFileName + " <");
		BufferedWriter writer = new BufferedWriter(new FileWriter(htmlFileName));
		ILineCompare l1, l2;
		IFieldDetail fld;

		htmlStart(writer);
		for (int j = 0; j < cmpResult.getRowCount(); j++) {
			l1 = cmpResult.get(BEFORE, j);
			l2 = cmpResult.get(AFTER, j);

			if (l1 == null || l2 == null) {
				writeOneRow(writer, recordLayout, l1, l2);
			} else {
				pref = l1.getLine().getPreferredLayoutIdx();
				writer.write(HTML_MANY_TBL_START);      writer.newLine();
				writer.write(HTML_LINE_NUMBER
						+ "<td>" + l1.getLineNumber() + "</td><td>" + l2.getLineNumber() + "</td></tr>");
				writer.newLine();
				for (i = 0; i < recordLayout.getRecord(pref).getFieldCount(); i++ ) {
					if (! l1.getLine().getField(pref, i).equals( l2.getLine().getField(pref, i))) {
						fld = recordLayout.getRecord(pref).getField(i);
						writeOneFieldHeader(writer, fld, i);
						writer.write("<td bgcolor=lime>" + fix(l1.getLine().getField(pref, i))
								+ "</td><td bgcolor=lime>"
								+ fix(l2.getLine().getField(pref, i)) + "</td></tr>");
						writer.newLine();
					}
				}
				writer.write("</table>");               writer.newLine();
			}
		}

		writer.write(HTML_END);							writer.newLine();
		writer.close();
	}


	/**
	 * Write Compare result as HTML with each row written as a individual table.
	 * All fields are to be displayed
	 *
	 * @param filename output file name
	 * @param recordLayout record layout
	 * @param before before file details
	 * @param after  after  file details
	 * @throws IOException any IO Errors
	 */
	public final void writeTblAllFields(
			AbstractLayoutDetails recordLayout,
			ICompareResult cmpResult)
			//ArrayList<LineCompare> before, ArrayList<LineCompare> after)
	throws IOException {
		int i;
		BufferedWriter writer = new BufferedWriter(new FileWriter(htmlFileName));

		htmlStart(writer);
		for (i = 0; i < cmpResult.getRowCount(); i++) {
			writeOneRow(writer, recordLayout, cmpResult.get(BEFORE, i), cmpResult.get(AFTER, i));
		}

		writer.write(HTML_END);
		writer.close();
	}


	private void writeOneRow(BufferedWriter writer, AbstractLayoutDetails recordLayout,
			ILineCompare l1, ILineCompare l2)
	throws IOException {
		int pref, i;
		IFieldDetail fld;

		writer.write(HTML_MANY_TBL_START);      writer.newLine();
		writer.write(HTML_LINE_NUMBER);
		if (l1 == null) {
			pref = l2.getLine().getPreferredLayoutIdx();
			writer.write("<td>&nbsp;</td><td>" + l2.getLineNumber() + "</td></tr>");
			writer.newLine();
			if (recordLayout.isMapPresent()) {
				writeKeyFieldHeader(writer);
				writeNewFieldNew(writer,
						l2.getLine().getField(pref, Common.KEY_INDEX));
			}
			for (i = 0; i < recordLayout.getRecord(pref).getFieldCount(); i++ ) {
				fld = recordLayout.getRecord(pref).getField(i);
				writeOneFieldHeader(writer, fld, i);
				writeNewFieldNew(writer, l2.getLine().getField(pref, i));
			}
		} else if (l2 == null) {
			pref = l1.getLine().getPreferredLayoutIdx();
			writer.write("<td>" + l1.getLineNumber() + "</td><td>&nbsp;</td></tr>");
			writer.newLine();
			if (recordLayout.isMapPresent()) {
				writeKeyFieldHeader(writer);
				writeNewFieldOld(writer,
						l1.getLine().getField(pref, Common.KEY_INDEX));
			}
			for (i = 0; i < recordLayout.getRecord(pref).getFieldCount(); i++ ) {
				fld = recordLayout.getRecord(pref).getField(i);
				writeOneFieldHeader(writer, fld, i);
				writeNewFieldOld(writer, l1.getLine().getField(pref, i));
			}
		} else {
			pref = l1.getLine().getPreferredLayoutIdx();
			if (recordLayout.isMapPresent()) {
				writeKeyFieldHeader(writer);
				writeNewFieldBoth(writer,
						l1.getLine().getField(pref, Common.KEY_INDEX),
						l2.getLine().getField(pref, Common.KEY_INDEX));
			}
			writer.write("<td>" + l1.getLineNumber() + "</td><td>" + l2.getLineNumber() + "</td></tr>");
			writer.newLine();
			for (i = 0; i < recordLayout.getRecord(pref).getFieldCount(); i++ ) {
				fld = recordLayout.getRecord(pref).getField(i);
				writeOneFieldHeader(writer, fld, i);

				writeNewFieldBoth(writer,
						l1.getLine().getField(pref, i), l2.getLine().getField(pref, i));
			}
		}
		writer.write("</table>");               writer.newLine();
	}

	private void writeNewFieldNew(BufferedWriter writer, Object value)
	throws IOException {

		writer.write("<td>&nbsp;</td><td bgcolor=yellow>"
				+ fix(value) + "</td></tr>");
		writer.newLine();

	}

	private void writeNewFieldOld(BufferedWriter writer,  Object value)
	throws IOException {

		writer.write("<td bgcolor=aqua>" + fix(value) + "</td><td>&nbsp;</td></tr>");
		writer.newLine();

	}

	private void writeNewFieldBoth(BufferedWriter writer,
			Object value1, Object value2)
	throws IOException {

		if ( value1.equals( value2)) {
			writer.write("<td>" + fix(value1) + "</td><td>&nbsp;</td></tr>");
		} else {
			writer.write("<td bgcolor=lime>" + fix(value1)
					+ "</td><td bgcolor=lime>" + fix(value2) + "</td></tr>");
		}
		writer.newLine();

	}

	private void htmlStart(BufferedWriter writer)
	throws IOException {

		writer.write(HTML_START);            		writer.newLine();
		writer.write("<p> <b>Old File: </b>" + oldFileName); 			writer.newLine();
		writer.write("<p> <b>New File: </b>" + newFileName + "<p>"); 	writer.newLine();
	}

	/**
	 * Write one field header
	 *
	 * @param writer writer (where HTML is to be written)
	 * @param fld field definition
	 * @param fieldId field id (number)
	 *
	 * @throws IOException any error that occurs
	 */
	private void writeOneFieldHeader(BufferedWriter writer, IFieldDetail fld, int fieldId)
	throws IOException {
		String len = "&nbsp;";

		if (fld.getLen() >= 0) {
			len = Integer.toString(fld.getLen());
		}

		writer.write("<TR><td>" + fld.getName()
				   + "</td><td>" + fld.getPos() + "</td><td>" + len + "</td>");
	}

	private void writeKeyFieldHeader(BufferedWriter writer)
	throws IOException {

		writer.write("<TR><td>Key Field</td><td>&nbsp;</td><td>&nbsp;</td>");
	}

	private String fix(Object o) {
		if (o == null || "".equals(o.toString().trim())) {
			return "&nbsp;";
		}
		return o.toString();
	}
}
