package net.sf.RecordEditor.diff;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.Common.RecordException;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.Details.Options;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.RecordEditor.utils.ExpandLineTree;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.fileStorage.DataStoreLarge;
import net.sf.RecordEditor.utils.fileStorage.FileDetails;
import net.sf.RecordEditor.utils.fileStorage.IDataStore;
import net.sf.RecordEditor.xml.classes.Layout;

public class ZOldLineBufferedReader extends BufferedReader {

	private int currLine = 0;
	private List<AbstractLine> lines;
	private AbstractLayoutDetails detail;

	private AbstractLayoutDetails filteredLayout;
	private final boolean stripSpaces;
	private ExpandLineTree expand;

	public ZOldLineBufferedReader(
			String fileName,
			AbstractLayoutDetails dtl,
			List<AbstractLine> lineArray,
			boolean stripTrailingSpaces)
	throws IOException, RecordException {
		super(new FileReader(fileName), 4);
		super.close();

		filteredLayout = dtl;
		detail = dtl;

		stripSpaces = stripTrailingSpaces;

		if (dtl.hasChildren()) {
			lines  = new ArrayList<AbstractLine>(lineArray.size() * 2);
			expand = ExpandLineTree.newExpandLineTree(lines);

			for (AbstractLine line : lineArray) {
				expand.expand(line);
			}
		} else {
			lines  = lineArray;
		}
	}


	public ZOldLineBufferedReader(String fileName, AbstractLayoutDetails dtl, AbstractLayoutDetails newDtl, Layout layoutDef,
			boolean stripTrailingSpaces)
	throws IOException, RecordException {
		super(new StringReader(""), 4); 
		super.close();

		AbstractLineReader reader = LineIOProvider.getInstance().getLineReader(dtl);
		reader.open(fileName, dtl);
		
		detail = dtl;
		stripSpaces = stripTrailingSpaces;


		processInput( newDtl, layoutDef, reader);
	}
	

	public ZOldLineBufferedReader(InputStream stream, AbstractLayoutDetails dtl, AbstractLayoutDetails newDtl, Layout layoutDef,
			boolean stripTrailingSpaces)
	throws IOException, RecordException {
		super(new StringReader(""), 4);
		super.close();

		AbstractLineReader reader = LineIOProvider.getInstance().getLineReader(dtl);
		reader.open(stream, dtl);
		
		detail = dtl;
		stripSpaces = stripTrailingSpaces;


		processInput( newDtl, layoutDef, reader);
	}



	/**
	 * @param dtl
	 * @param newDtl
	 * @param layoutDef
	 * @param stripTrailingSpaces
	 * @param reader
	 * @throws IOException
	 */
	private void processInput(AbstractLayoutDetails newDtl, Layout layoutDef,
			AbstractLineReader reader) throws IOException {
		boolean noFilter = (layoutDef == null) || layoutDef.getRecords() == null || layoutDef.getRecords().size() == 0;


		AbstractLine line;

		line = reader.read();
		detail = reader.getLayout();
		if (detail instanceof LayoutDetail) {
			lines  = createList((LayoutDetail) detail); 
		} else {
			lines = new ArrayList<AbstractLine>(256);
		}
		if (detail.hasChildren()) {
			expand = ExpandLineTree.newExpandLineTree(lines);
		}


		if (noFilter) {
			while (line != null) {
				addLine(line);
				line = reader.read();
			}
			
			filteredLayout = detail;
		} else {
			boolean[] include = new boolean[detail.getRecordCount()];
			List<AbstractLine> list;
			boolean isArrayList = ! (lines instanceof IDataStore);

			filteredLayout = newDtl;
			if (newDtl == null) {
				filteredLayout = detail.getFilteredLayout(layoutDef.getFilteredRecords());
			}

			if (detail.getRecordCount() == 1) {
				while (line != null) {
					list = ExpandLineTree.expandTree(line);
					for (AbstractLine xLine: list) {
						if (isArrayList) {
							xLine.setLayout(filteredLayout);
						}
						lines.add(xLine);
					}
					line = reader.read();
				}
			} else {
				int idx, j;
				for (j = 0; j < include.length; j++) {
					include[j] = include.length == layoutDef.getRecords().size();
				}

				if (include.length != layoutDef.getRecords().size()) {
					for (j = 0; j < layoutDef.getRecords().size(); j++) {
						include[detail.getRecordIndex(layoutDef.getRecords().get(j).name)] = true;
					}
				}


				//AbstractLine xLine;
				while (line != null) {
					list = ExpandLineTree.expandTree(line);
					for (AbstractLine xLine: list) {
						idx = xLine.getPreferredLayoutIdx();
						if (idx >= 0 && idx < include.length && include[idx]) {
							if (isArrayList) {
								xLine.setLayout(filteredLayout);
							}
							lines.add(xLine);
						}
					}
					line = reader.read();
				}
				detail = reader.getLayout();
			}
			if (! isArrayList) {
				((IDataStore<AbstractLine>) lines).setLayoutRE(filteredLayout);
			}
		}
		reader.close();
	}

	private List<AbstractLine> createList(LayoutDetail layout) {
		int fileStructure = layout.getFileStructure();
		int len = layout.getMaximumRecordLength();
		
		if (layout.getOption(Options.OPT_SUPPORTS_BLOCK_STORAGE) != Options.YES) {
			return new ArrayList<AbstractLine>();
		} else if (layout.getOption(Options.OPT_STORAGE_TYPE) == Options.TEXT_STORAGE
				|| fileStructure == IFileStructureConstants.IO_UNKOWN_FORMAT
				||  fileStructure == IFileStructureConstants.IO_WIZARD
				||  layout.getLayoutType()    == Constants.rtDelimited
				||  fileStructure == IFileStructureConstants.IO_NAME_1ST_LINE
				||  fileStructure == IFileStructureConstants.IO_GENERIC_CSV) {
			len = 100000;
		} else if (layout.getOption(Options.OPT_IS_FIXED_LENGTH) == Options.YES) {
			System.out.println("Editing using Big Fixed Length memory Model - this will reduce functionality !!!");
//			Common.logMsg(AbsSSLogger.WARNING,
//					"\nEditing using Big Fixed Length memory Model - this will reduce functionality !!!", null);
			return new DataStoreLarge(
					layout,
					FileDetails.FIXED_LENGTH,
					layout.getMaximumRecordLength());
		}
		return new DataStoreLarge(
				(LayoutDetail) layout,
				FileDetails.VARIABLE_LENGTH,
				len);

	}
	/**
	 * Add a line to the list
	 * @param l
	 */
	private void addLine(AbstractLine l) {
		if (detail.hasChildren()) {
			expand.expand(l);
		} else {
			lines.add(l);
		}
	}


	@Override
	public void close() throws IOException {

	}

	@Override
	public boolean markSupported() {
		return false;
	}

	@Override
	public int read() throws IOException {
		throw new RuntimeException("read not supported");
	}

	@Override
	public int read(char[] cbuf, int off, int len) throws IOException {
		throw new RuntimeException("read not supported");
	}

	@Override
	public String readLine() throws IOException {

		return readLine(currLine++);
	}

	public String readLine(int lineNo) throws IOException {
		if (lineNo < lines.size()) {
			AbstractLine line = lines.get(lineNo);
			StringBuffer buf = new StringBuffer();
			int pref = line.getPreferredLayoutIdx();

			if (detail.isMapPresent()) {
				Object o = line.getField(pref, Constants.KEY_INDEX);

				if (o != null) {
					buf.append(Common.trimRight(o)).append("\t");
				}
			}

			if (filteredLayout.getRecord(pref) == null) {
				Common.logMsg("Layout Record Does not exist !!!", null);
			} else if (stripSpaces) {
				for (int i = 0; i < filteredLayout.getRecord(pref).getFieldCount(); i++) {
					buf.append(Common.trimRight(getField(line, pref, i))).append("\t");
				}
			} else {
				for (int i = 0; i < filteredLayout.getRecord(pref).getFieldCount(); i++) {
					buf.append(getField(line, pref, i)).append("\t");
				}
			}

//			if (lineNo < 25) {
//			System.out.println("!! " + lineNo + " "+ buf.toString() + ": " + pref + " " + filteredLayout.getLayoutName()
//					+ " " +  filteredLayout.getRecord(pref).getRecordName()
//					+ " " + filteredLayout.getRecord(pref).getFieldCount());
//			}
			return buf.toString();
		}
		return null;
	}
	
	private String getField(AbstractLine line, int recordNo, int fldNo) {
		Object obj = line.getField(recordNo, fldNo);
		if (obj == null) {
			obj = "";
		}
		String s = obj.toString();
		if (s.indexOf('\t') >= 0 || s.indexOf('\n') >= 0) {
			StringBuilder b = Conversion.replace(new StringBuilder(s), "\\", "\\\\");
			Conversion.replace(b, "\n", "\\n");
			Conversion.replace(b, "\t", "\\t");
			s = b.toString();
		}
		
		return s;
	}

	@Override
	public boolean ready() throws IOException {
		return true;
	}

	@Override
	public void reset() throws IOException {
		currLine = 0;
	}

	@Override
	public long skip(long n) throws IOException {
		throw new RuntimeException("skip not supported");
	}

	public int getCount() {
		return lines.size();
	}

	public AbstractLine getLine(int index) {
		return lines.get(index);
	}


	/**
	 * @return the filteredLayout
	 */
	public final AbstractLayoutDetails getFilteredLayout() {
		return filteredLayout;
	}


	public AbstractLayoutDetails getLayout() {
		return detail;
	}
}
