package net.sf.RecordEditor.diff;

import java.io.File;
import java.io.IOException;

import jlibdiff.Diff;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.Common.RecordException;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.AbstractRecordDetail;
import net.sf.JRecord.IO.AbstractLineIOProvider;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.RecordEditor.diff.result.ICompareResult;
import net.sf.RecordEditor.diff.result.Visitor;
import net.sf.RecordEditor.diff.result.VisitorSummary;
import net.sf.RecordEditor.diff.tbl.TableDisplay;
import net.sf.RecordEditor.re.layoutSelection.def.ISchemaProvider;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.xml.classes.DiffDefinition;


public class Zold_DoCompare  {

//	private static final String DIR = "/home/bm/RecordEdit/HSQLDB/SampleFiles/";

	private static Zold_DoCompare instance = new Zold_DoCompare();

	/**
	 * Do compare
	 * @param layoutReader layout reader
	 * @param layoutReader2 layout reader
	 * @param def Diff (compare) definition
	 *
	 * @throws Exception any error
	 */
	public final void compare(ISchemaProvider layoutReader,
			ISchemaProvider layoutReader2, DiffDefinition def)
	throws Exception {

		if (DiffDefinition.TYPE_SINGLE_LAYOUT.equals(def.type)) {
			AbstractLayoutDetails dtl = getLayout(layoutReader, def.getLayoutDetails().name, def.getOldfile().getName());
			compare1Layout(dtl, def);
		} else {
			AbstractLayoutDetails dtl1 = getLayout(layoutReader,  def.getOldfile().getLayoutDetails().name,
					def.getOldfile().getName());
			AbstractLayoutDetails dtl2 = getLayout(layoutReader2, def.getNewfile().getLayoutDetails().name,
					def.getNewfile().getName());

			compare2Layouts(dtl1, dtl2, def);
		}
	}

	/**
	 * write HTML file
	 *
	 * @param layoutReader  layout reader
	 * @param layoutReader2 layout reader 2
	 * @param def diff definition
	 *
	 * @throws Exception any error
	 */
	public final void writeHtml(ISchemaProvider layoutReader,
			ISchemaProvider layoutReader2, DiffDefinition def)
	throws Exception {

		Diff diff = new Diff();
		LineBufferedReader oldReader ;
		LineBufferedReader newReader;
		AbstractLayoutDetails dtl1, dtl2, dtl3;
		Visitor vis;

		if (DiffDefinition.TYPE_SINGLE_LAYOUT.equals(def.type)) {
			dtl1 = getLayout(layoutReader, def.getLayoutDetails().name, def.getOldfile().getName());
			oldReader = new LineBufferedReader(def.getOldfile().getName(),
					dtl1, null, def.getLayoutDetails(), def.stripTrailingSpaces);
			newReader = new LineBufferedReader(def.getNewfile().getName(),
					dtl1, oldReader.getFilteredLayout(), def.getLayoutDetails(), def.stripTrailingSpaces);


		} else {
			dtl1 = getLayout(layoutReader,  def.getOldfile().getLayoutDetails().name, def.getOldfile().getName());
			dtl2 = getLayout(layoutReader2, def.getNewfile().getLayoutDetails().name, def.getNewfile().getName());

			oldReader = new LineBufferedReader(def.getOldfile().getName(),
					dtl1, null, def.getOldfile().getLayoutDetails(), def.stripTrailingSpaces);
			newReader = new LineBufferedReader(def.getNewfile().getName(),
					dtl2, null, def.getNewfile().getLayoutDetails(), def.stripTrailingSpaces);
		}

		vis = new Visitor(oldReader, newReader);

		diff.diffBuffer(oldReader, newReader);
		diff.accept(vis);
		
		VisitorSummary summary = new VisitorSummary(vis);
		ICompareResult result = summary.getResult(def.isAllRows() && def.isAllFields());

//		if (def.isAllRows() && def.isAllFields()) {
//			before = vis.getOldList();
//			after  = vis.getNewList();
//		} else {
//			before = vis.getOldChanged();
//			after  = vis.getNewChanged();
//		}

		dtl3 = oldReader.getFilteredLayout();
		if (dtl3 == null) {
			dtl3 = dtl1;
		}
		WriteHtml writeHtml = new WriteHtml(def.getOldfile().getName(), def.getNewfile().getName(), def.getHtmlFile());
		if (def.isSingleTable()) {
			writeHtml.writeSingleTbl(dtl3, result);
		} else if (def.isAllFields()) {
			writeHtml.writeTblAllFields(dtl3, result);
		} else {
			writeHtml.writeTblChgFields(dtl3, result);
		}
	}


	/**
	 * Get a record layout
	 *
	 * @param layoutReader Layout Reader
	 * @param name Layout name
	 * @param fileName Sample file name
	 *
	 * @return requested layout
	 * @throws Exception any error
	 */
	private AbstractLayoutDetails getLayout(ISchemaProvider layoutReader,
			String name, String fileName) throws Exception {

		try {
			return layoutReader.getRecordLayout(name, fileName);
		} catch (Exception e) {
			String s = "Error Loading Layout:";
			e.printStackTrace();
			Common.logMsg(s, e);
			throw e;
		}
	}

	/**
	 * Compare files (using 1 layout)
	 *
	 * @param dtl Record Layout
	 * @param diffDefinition diff Definition
	 * @throws IOException any IO Error
	 * @throws RecordException any record-editor exception
	 */
	private final void compare1Layout(
			AbstractLayoutDetails dtl, DiffDefinition diffDefinition)
	throws IOException, RecordException {

		LineBufferedReader oldReader = new LineBufferedReader(diffDefinition.getOldfile().getName(),
				dtl, null, diffDefinition.getLayoutDetails(), diffDefinition.stripTrailingSpaces);
		LineBufferedReader newReader = new LineBufferedReader(diffDefinition.getNewfile().getName(),
				dtl, oldReader.getFilteredLayout(), diffDefinition.getLayoutDetails(),
				diffDefinition.stripTrailingSpaces);

		compare1Layout(oldReader, newReader);
	}


	public final void compare1Layout(AbstractLayoutDetails dtl, String oldFile, String newFile)
	throws IOException {
		compare1LayoutBackEnd(dtl, oldFile, newFile, true);
	}
	

	public final Visitor compare1LayoutVistor(AbstractLayoutDetails dtl, String oldFile, String newFile)
	throws IOException {
		return compare1LayoutBackEnd(dtl, oldFile, newFile, false);
	}

	public final Visitor compare1LayoutBackEnd(AbstractLayoutDetails dtl, String oldFile, String newFile, boolean displayTbl)
	throws IOException {
		LineBufferedReader oldReader = new LineBufferedReader(
				oldFile, dtl, null, null, false);
		if (dtl.getFileStructure() == IFileStructureConstants.IO_XML_BUILD_LAYOUT) {
			dtl = oldReader.getLayout();
		}
		LineBufferedReader newReader = new LineBufferedReader(
				newFile, dtl, null, null, false);

		String oldFn = new File(oldFile).getName();
		String newFn = new File(newFile).getName();
		
		if (! oldFn.equalsIgnoreCase(newFn)) {
			oldFn = oldFn + " ~ " + newFn;
		}
		
		Visitor vis = getVisitor(oldReader, newReader);

		if (displayTbl) {
			new TableDisplay("Compare: " + oldFn, oldReader.getFilteredLayout(),
					new VisitorSummary(vis),
					//vis.getOldList(),    vis.getNewList(),
					//vis.getOldChanged(), vis.getNewChanged(), 
					true);
		}
		return vis;
	}



	

	public final void compare1Layout(LineBufferedReader oldReader, LineBufferedReader newReader)
	throws IOException {

		new TableDisplay("Single Layout Compare", oldReader.getFilteredLayout(),
				new VisitorSummary(getVisitor(oldReader, newReader)),
				//vis.getOldList(),    vis.getNewList(),
				//vis.getOldChanged(), vis.getNewChanged(), 
				true);
	}

	private Visitor getVisitor(LineBufferedReader oldReader, LineBufferedReader newReader) throws IOException {
		Diff diff = new Diff();
		Visitor vis = new Visitor(oldReader, newReader);

		//System.out.println("Starting ... ");
		diff.diffBuffer(oldReader, newReader);
		diff.accept(vis);
		return vis;
	}
	
	public final boolean isDifferent1Layout(AbstractLayoutDetails dtl, String filename1, String filename2,
			int trimFieldIdx)
	throws IOException {

		AbstractLineIOProvider ioProvider = LineIOProvider.getInstance();

		AbstractLineReader reader1 = ioProvider.getLineReader(dtl);

		AbstractLine line1, line2;
		Object o1, o2;
		
		reader1.open(filename1);
		
		line1 = reader1.read();
		boolean ok = true;

		AbstractLineReader reader2 = ioProvider.getLineReader(dtl);
		if (dtl.getFileStructure() == IFileStructureConstants.IO_XML_BUILD_LAYOUT) {
			dtl = reader1.getLayout();
		}
		reader2.open(filename2, dtl);

		while ((line2 = reader2.read()) != null && line1 != null && ok) {
			int recordIdx = line1.getPreferredLayoutIdx();
			if (recordIdx != line2.getPreferredLayoutIdx()) {
				ok = false;
			} else {
				recordIdx = Math.max(0, recordIdx);
				AbstractRecordDetail r = dtl.getRecord(recordIdx);
				if (trimFieldIdx < 0) {
					for (int i = 0; ok && i < r.getFieldCount(); i++ ) {
						o1 = line1.getField(recordIdx, i);
						o2 = line2.getField(recordIdx, i);
						
						ok = o1 == o2 || (o1 != null && o1.equals(o2));
	//					if (! ok) { 
	//						System.out.println("====>> " + i);
	//					}
					}
				} else {
					AbstractRecordDetail record = line1.getLayout().getRecord(recordIdx);
					for (int i = 0; ok && i < r.getFieldCount(); i++ ) {
						o1 = line1.getField(recordIdx, i);
						o2 = line2.getField(recordIdx, i);							
						if (trimFieldIdx == i) {
							o1 = o1 == null ? "" : o1.toString().trim();
							o2 = o2 == null ? "" : o2.toString().trim();							
						}
						ok = o1 == o2 || (o1 != null && o1.equals(o2));
					}					
				}
//				if (! ok) {
//					System.out.println(line1.getFullLine());
//					System.out.println(line2.getFullLine());
//				}
			}
			line1 = reader1.read();
		}
		ok = ok && (line1 == line2); // sneaky test of line1 == null && line2 == null
		
		reader1.close();
		reader2.close();
		
		return ! ok;
	}


	/**
	 * Compare files using 2 layouts
	 * @param dtl1 record layout 1
	 * @param dtl2 record layout 2
	 * @param diffDefinition diff Definition
	 * @throws IOException any IO Error
	 * @throws RecordException any record-editor exception
	 */
	private final void compare2Layouts(AbstractLayoutDetails dtl1,
			                                                  AbstractLayoutDetails dtl2,
								DiffDefinition diffDefinition)
	throws IOException, RecordException {

		Diff diff = new Diff();
		LineBufferedReader oldReader = new LineBufferedReader(diffDefinition.getOldfile().getName(),
				dtl1, null, diffDefinition.getOldfile().getLayoutDetails(), diffDefinition.stripTrailingSpaces);
		LineBufferedReader newReader = new LineBufferedReader(diffDefinition.getNewfile().getName(),
				dtl2, null, diffDefinition.getNewfile().getLayoutDetails(), diffDefinition.stripTrailingSpaces);


		Visitor vis = new Visitor(oldReader, newReader);

		//diff.diffString(arg0, arg1)
		diff.diffBuffer(oldReader, newReader);
		diff.accept(vis);

		new TableDisplay("Single Layout Compare", oldReader.getFilteredLayout(),
				new VisitorSummary(vis),
				//vis.getOldList(),    vis.getNewList(),
				//vis.getOldChanged(), vis.getNewChanged(), 
				true);
	}


   /**
    * @return the instance of compare files
    */
   public static final Zold_DoCompare getInstance() {
	   return instance;
   }
}
