package net.sf.RecordEditor.diff.result;

import java.util.List;

import net.sf.JRecord.Common.RecordException;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;

public class CompareSummary implements ICompareSummary {

	private final CompareResult allLinesResult, changedLinesResult;
	
	public CompareSummary(VisitorBlockResult v) {
		AbstractLayoutDetails layout = v.getLayout();
		List<AbstractLine> oldLines = v.getOldLines();
		List<AbstractLine> newLines = v.getNewLines();
	
		allLinesResult = new CompareResult(v.getFullList(), layout, oldLines, newLines);
		changedLinesResult = new CompareResult(v.getChangeList(), layout, oldLines, newLines);
	}
	
	@Override
	public ICompareResult getResult(boolean allLines) {
		return allLines ? allLinesResult :  changedLinesResult;
	}

	
	private static class CompareResult implements ICompareResult {
		final CompareBlockResult[] results;
		final List<AbstractLine> beforeLines, afterLines;
		final int lineCount, beforeRowCount,  afterRowCount;
		int  lastIndex = 0;
		private final AbstractLayoutDetails layout;
		
		public CompareResult(List<CompareBlockResult> list, AbstractLayoutDetails layout,
				List<AbstractLine> beforeLines, List<AbstractLine> afterLines) {
			super();
			this.results = list.toArray(new CompareBlockResult[list.size()]);
			this.layout = layout;
			this.beforeLines = beforeLines;
			this.afterLines = afterLines;
			int bRowCount = 0, aRowCount = 0;
			
			if (list  == null || list.size() == 0 ) {
				lineCount = 0;
			} else {
				lineCount = list.get(list.size()-1).nextCompareLineNumber();
				
				for (int i = list.size() - 1; i >= 0 && (bRowCount == 0 || aRowCount == 0); i--) {
					CompareBlockResult r = list.get(i);
					if (bRowCount == 0 &&r.compareResult.oldPresent) {
						bRowCount = r.compareLineNumber + r.lines;
					}
					if (aRowCount == 0 &&r.compareResult.newPresent) {
						aRowCount = r.compareLineNumber + r.lines;
					}
				}
			}
			beforeRowCount = bRowCount;
			afterRowCount = aRowCount;
		}


		@Override
		public int getRowCount(boolean beforeImage) {
//			System.out.println("rowCount: " + beforeImage + "\t" + beforeRowCount + "\t" + afterRowCount);
			return beforeImage ? beforeRowCount : afterRowCount;
		}

		@Override
		public AbstractLayoutDetails getLayout() {
			return layout;
		}

		@Override
		public int getRowCount() {
			return lineCount;
		}

		@Override
		public ILineCompare get(boolean beforeImage, int row) {
			if (lineCount == 0 || row >= lineCount || row < 0) { return null; }
			
			if (results[lastIndex].contains(row)) {
				
			} else if (lastIndex < results.length-1 && results[lastIndex+1].contains(row)) {
				lastIndex += 1;
				System.out.println(">> " + results[lastIndex].compareResult + "\t"
						+ results[lastIndex].oldLineNumber+ "\t" + results[lastIndex].newLineNumber);
			} else {
				if (row < results[lastIndex].compareLineNumber) {
					lastIndex = binSearch(row, Math.max(0, lastIndex - (results[lastIndex].compareLineNumber-row)), lastIndex);
				} else {
					lastIndex = binSearch(row, lastIndex, Math.min(results.length-1, lastIndex + (row-results[lastIndex].compareLineNumber)));
				}
			}
			CompareBlockResult cmpResultBlock = results[lastIndex];
			
			int diff = row - cmpResultBlock.compareLineNumber;
			int lineNumber;
			
			if (beforeImage) {
				lineNumber = cmpResultBlock.oldLineNumber + diff;
				return cmpResultBlock.compareResult.oldPresent
						? new LineCmp(lineNumber, cmpResultBlock.compareResult, 
								lineNumber <= beforeLines.size() ? beforeLines.get(lineNumber-1) : null)
						: null;
			}
			
			lineNumber = cmpResultBlock.newLineNumber + diff;
			return cmpResultBlock.compareResult.newPresent
					? new LineCmp(
							lineNumber, 
							cmpResultBlock.compareResult,
							lineNumber <= afterLines.size() ? afterLines.get(lineNumber-1) : null)
					: null;
		}
		
		private int binSearch(int searchFor, int f, int l) {
			int pivot;
			
			while (l-f > 1) {
				pivot = (l+f) / 2;
				if (searchFor < results[pivot].compareLineNumber) {
					l = pivot;
				} else if (results[pivot].contains(searchFor)) {
					return pivot;
				} else {
					f = pivot;
				}
			}
			
			if (results[f].contains(searchFor)) {
				return f;
			}
			if (f!=l && results[l].contains(searchFor)) {
				return l;
			}
			System.err.println("Low,High=" + l + ", " + f 
					+ results[f].compareLineNumber + ", " + results[f].lines
					+ results[l].compareLineNumber + ", " + results[l].lines
					+"\tSearch For=" + searchFor );
			throw new RecordException("Internal Error: searching for linenumber" + searchFor
					+ ", class=" + this.getClass().getName());
		}
	}
	
	
	static class LineCmp implements ILineCompare {
		final int lineNumber;
		final LineCompareResult result;
		final AbstractLine line;
		


		public LineCmp(int lineNumber, LineCompareResult result, AbstractLine line) {
			super();
			this.lineNumber = lineNumber;
			this.result = result;
			this.line = line;
		}

		@Override
		public int getLineNumber() {
			return lineNumber;
		}

		@Override
		public LineCompareResult getCompareResult() {
			return result;
		}

		@Override
		public AbstractLine getLine() {
			return line;
		}
		
	}
}
