package net.sf.RecordEditor.diff.result;

import java.util.ArrayList;
import java.util.List;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.RecordEditor.utils.fileStorage.IDataStore;

public class VisitorSummary implements ICompareSummary {

	private final VistorResult allLinesResult, changedLinesResult;
	
	public VisitorSummary(Visitor v) {
		ArrayList<LineCompare> oldList = v.getOldList();
		AbstractLayoutDetails layout = null;
		if (oldList instanceof IDataStore) {
			layout = ((IDataStore) oldList).getLayoutRE();
		} else if (oldList != null){
			for (ILineCompare lc : oldList) {
				if (lc != null && lc.getLine() != null) {
					layout = lc.getLine().getLayout();
					break;
				}
			}
		}
		allLinesResult = new VistorResult(oldList, v.getNewList(), layout);
		changedLinesResult = new VistorResult(v.getOldChanged(), v.getNewChanged(), layout);
	}
	
	@Override
	public ICompareResult getResult(boolean allLines) {
		return allLines ? allLinesResult :  changedLinesResult;
	}

	
	private static class VistorResult implements ICompareResult {
		final List<LineCompare> before, after;
		final int rowCount, beforeRowCount,  afterRowCount;
		private final AbstractLayoutDetails layout;
		
		public VistorResult(List<LineCompare> before, List<LineCompare> after, AbstractLayoutDetails layout) {
			super();
			this.before = before;
			this.after = after;
			this.layout = layout;
			
			beforeRowCount = before == null ? 0 : before.size();
			afterRowCount = after == null ? 0 : after.size();
			rowCount = Math.max( beforeRowCount, afterRowCount);
		}

		@Override
		public int getRowCount(boolean beforeImage) {
			return beforeImage ? beforeRowCount : afterRowCount;
		}

		@Override
		public AbstractLayoutDetails getLayout() {
			return layout;
		}

		@Override
		public int getRowCount() {
			return rowCount;
		}

		@Override
		public ILineCompare get(boolean beforeImage, int row) {
			List<LineCompare> r = beforeImage ? before : after;
			return row < 0 || r == null || r.size() <= row ? null : r.get(row);
		}
		
	}
}
