package net.sf.RecordEditor.diff.tbl;


import javax.swing.table.AbstractTableModel;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Common.IFieldDetail;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.RecordEditor.diff.result.ICompareResult;
import net.sf.RecordEditor.diff.result.ILineCompare;
import net.sf.RecordEditor.diff.result.LineCompareResult;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.LangConversion;

@SuppressWarnings("serial")
public class CmpLineModel extends AbstractTableModel {

	private static final boolean AFTER_IMAGE = false;
	private static final boolean BEFORE_IMAGE = true;
	public static final int BEFORE = 0;
	public static final int AFTER  = 1;


	private static final String[] columnNames = LangConversion.convertColHeading(
			"Compare_Line",
			new String[] {"Field", "Position", "Length", "Old", "New"});

//	private ArrayList<LineCompare>[] displayRows;
	private ICompareResult  cmpResult;
	private AbstractLayoutDetails description;
	private int[] changedFields = null;

	private static final int COLUMN_COUNT = 5;

	private int recordIdx = 0;
	private int currentRow = 0;
	private boolean displayChangedFields = false;



	public CmpLineModel(AbstractLayoutDetails layout, ICompareResult  cmpResult) {
						  //ArrayList<LineCompare> displayBefore, ArrayList<LineCompare> displayAfter) {

//		displayRows = new ArrayList[2];
		this.cmpResult = cmpResult;

		description = layout;

//		displayRows[BEFORE] = displayBefore;
//		displayRows[AFTER]  = displayAfter;

		//System.out.println("Changed: " + displayBefore.size() + " " + displayAfter.size());
	}

	@Override
	public int getColumnCount() {
		return COLUMN_COUNT;
	}

	@Override
	public int getRowCount() {
		int inc = 1;
		if (description.isMapPresent()) {
			inc = 2;
		}
		if (isAllFields()) {
			return description.getRecord(recordIdx).getFieldCount() + inc;
		}
		return getChangedFields().length + inc;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		Object ret = null;
		int idx = columnIndex - 3;
		int id = getAdjustedRow(rowIndex);

		if (columnIndex < 3) {
			if (rowIndex == 0) {
				if (columnIndex == 0) {
					ret = "Line Number";
				}
			} else if (rowIndex == 1 && description.isMapPresent()) {
				if (columnIndex == 0) {
					ret = "Key Value";
				}
			} else {
				IFieldDetail fld = description.getRecord(recordIdx).getField(id - 1);
				if (columnIndex == 0) {
					ret = fld.getName();
				} else if (columnIndex == 1) {
					ret = Integer.valueOf(fld.getPos());
				} else if (fld.getLen() < 0){
					ret = "";
				} else {
					ret = Integer.valueOf(fld.getLen());
				}

			}
		} else if (currentRow < cmpResult.getRowCount(idx == 0)) {
			ret = standardFields(id, idx);
		}

		return ret;
	}

	private Object standardFields(int rowIndex, int idx) {
		Object ret = null;
		ILineCompare cmp = cmpResult.get(idx ==0, currentRow);

		if (cmp == null) {
			ret = "";
		} else  if (rowIndex == 0) {
			ret = Integer.valueOf(cmp.getLineNumber());
		} else {
			ILineCompare before = cmpResult.get(BEFORE_IMAGE, currentRow);
			int lineIdx = rowIndex - 1 ;

			if (idx == 1 && before != null) {
				if (Common.trimRight(before.getLine().getField(recordIdx, lineIdx))
						.equals(Common.trimRight(cmp.getLine().getField(recordIdx, lineIdx)))) {
					return  "";
				}
			}

			if (lineIdx >= description.getRecord(recordIdx).getFieldCount()) {
				return "";
			}

			ret = cmp.getLine().getField(recordIdx, lineIdx);
		}
		return ret;
	}


	/**
	 *
	 * @param rowIndex
	 * @return
	 */
	private int getAdjustedRow(int rowIndex) {

		int id = rowIndex;

		if (description.isMapPresent() && rowIndex > 0) {
			if (rowIndex == 1) {
				return Constants.KEY_INDEX + 1;
			}

			id -= 1;
		}

		if ( displayChangedFields && id > 0 && getChangedFields().length >= id
		&& ! isAllFields()) {
			id = getChangedFields()[id - 1];
		}

		return id;
	}


	/**
	 * check if a particular row has changed
	 * @param rowIdx row to check
	 * @return wether it has changed
	 */
	public final boolean isFieldChanged(int rowIdx) {
		boolean ret = false;
		int row = getAdjustedRow(rowIdx);

		if (row > 0 && currentRow <= cmpResult.getRowCount()) {
			ILineCompare before = cmpResult.get(BEFORE_IMAGE, currentRow);
			ILineCompare after  = cmpResult.get(AFTER_IMAGE, currentRow);
			int lineIdx = row - 1 ;

			if (before != null && before.getLine() != null
			&& before.getCompareResult()== LineCompareResult.CHANGED
			&& after != null && after.getLine() != null) {
				if (! Common.trimRight(before.getLine().getField(recordIdx, lineIdx))
						.equals(Common.trimRight(after.getLine().getField(recordIdx, lineIdx)))) {
					ret = true;
				}
			}
		}
		return ret;
	}

	@Override
	public String getColumnName(int column) {

		return columnNames[column];
	}

	public final void setDisplayRows(ICompareResult cmpResult) {

		this.cmpResult = cmpResult;

		this.fireTableDataChanged();
	}
	/**
	 * @return the recordIdx
	 */
	protected final int getRecordIdx() {
		return recordIdx;
	}

	/**
	 * @param recordIdx the recordIdx to set
	 */
	protected final boolean setRecordIdx(int recordIndex) {
		int idx = recordIndex;

		if (idx >= description.getRecordCount()) {
			AbstractLine cmp = getLine(this.cmpResult.get(true, currentRow));
			if (cmp == null) {
				idx = this.cmpResult.get(false, currentRow).getLine().getPreferredLayoutIdx();
			} else {
				idx = cmp.getPreferredLayoutIdx();
			}
		}

		if (idx >= 0 && recordIdx != idx) {
			changedFields = null;
			this.recordIdx = idx;
			this.fireTableDataChanged();
			return true;
		}
		return false;
	}

	AbstractLine getLine(ILineCompare lc) {
		return lc == null ? null : lc.getLine();
	}
	
	/**
	 * @return the currentRow
	 */
	protected final int getCurrentRow() {
		return currentRow;
	}

	/**
	 * Get the BEFORE compare line detail
	 * @return compare line details
	 */
	public final ILineCompare getCurrentCompareLine() {
//		if (displayRows[BEFORE].get(currentRow) != null
//		&&  (displayRows[AFTER].get(currentRow) == null
//		  || displayRows[AFTER].get(currentRow).line == null)) {
//			displayRows[BEFORE].get(currentRow).code = LineCompare.DELETED;
//		}
		return cmpResult.get(BEFORE_IMAGE, currentRow);
	}

	/**
	 * @param currentRow the currentRow to set
	 */
	protected final void setCurrentRow(int newRow) {
		if (newRow >= 0 && currentRow != newRow) {
			this.currentRow = newRow;

			changedFields = null;

			if (! setRecordIdx(description.getRecordCount())) {
				this.fireTableDataChanged();
			}
		}
	}

	private int[] getChangedFields() {

		if (changedFields == null) {
			int size = 0;
			int i, j;
			ILineCompare before = cmpResult.get(BEFORE_IMAGE, currentRow);
			ILineCompare after  = cmpResult.get(AFTER_IMAGE, currentRow);

			if (before == null || after == null) {
				changedFields = new int[0];
				return changedFields;
			}

			for (i = 0; i < description.getRecord(recordIdx).getFieldCount(); i++) {
				if (! Common.trimRight(before.getLine().getField(recordIdx, i))
						.equals(Common.trimRight(after .getLine().getField(recordIdx, i)))
				) {
					size += 1;
				}
			}

			changedFields = new int[size];
			j = 0;
			for (i = 0; i < description.getRecord(recordIdx).getFieldCount(); i++) {
				if (! Common.trimRight(before.getLine().getField(recordIdx, i))
						.equals(Common.trimRight(after .getLine().getField(recordIdx, i)))
				) {
					changedFields[j++] = i + 1;
				}
			}
		}
		return changedFields;
	}

	/**
	 * show all fields
	 * @return wether to show all fields
	 */
	private boolean isAllFields() {

		if (! displayChangedFields) {
			return true;
		}

		ILineCompare before = cmpResult.get(BEFORE_IMAGE, currentRow);

		return (before == null || before.getCompareResult() == LineCompareResult.DELETED
			||  cmpResult.get(AFTER_IMAGE, currentRow) == null);
	}
	/**
	 * @param onlyChangedFields the displayChangedFields to set
	 */
	public final void setDisplayChangedFields(boolean onlyChangedFields) {

		this.displayChangedFields = onlyChangedFields;
		fireTableDataChanged();
	}

}
