package net.sf.RecordEditor.diff.tbl;

import javax.swing.table.AbstractTableModel;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.RecordEditor.diff.result.ICompareResult;
import net.sf.RecordEditor.diff.result.ILineCompare;
import net.sf.RecordEditor.diff.result.LineCompareResult;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.LangConversion;

@SuppressWarnings("serial")
public class CmpTableModel extends AbstractTableModel {

	public static final int BEFORE = 0;
	public static final int AFTER  = 1;

	//private ArrayList<LineCompare>[] displayRows;
	private ICompareResult compareResult;
	private AbstractLayoutDetails description;

	private int colCount = 0;

	private int recordIndex = 0;

	private int additionalFields = 3;
	private int keyIdx = -1;




	public CmpTableModel(AbstractLayoutDetails layout,
						 ICompareResult compareResult) {
//						  ArrayList<LineCompare> before,    ArrayList<LineCompare> after,
//						  ArrayList<LineCompare> displayBefore, ArrayList<LineCompare> displayAfter) {

		this.compareResult = compareResult;

		description = layout;


//		System.out.println("Changed: " + displayBefore.size() + " " + displayAfter.size());



		for (int i = 0; i < description.getRecordCount(); i++) {
			colCount = Math.max(colCount, description.getRecord(i).getFieldCount());
		}

		if (layout.isMapPresent()) {
			keyIdx = additionalFields;
			additionalFields += 1;
		}
		colCount += additionalFields;
	}


	@Override
	public int getColumnCount() {
		if (recordIndex >= description.getRecordCount()) {
			return colCount;
		}

		return description.getRecord(recordIndex).getFieldCount() + additionalFields;
	}

	@Override
	public int getRowCount() {
		// TODO Auto-generated method stub
		return compareResult.getRowCount() * 2;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {

		int row = rowIndex / 2;
		int idx = rowIndex - row * 2;
		boolean beforeRow = idx == 0;
		ILineCompare cmp;

		if (columnIndex == 0 || row >=  compareResult.getRowCount()) {
			return "";
		} else if ((cmp = compareResult.get(beforeRow, row)) == null) {
			return "";
		} else if (columnIndex == 1 && idx == 0) {
			if (cmp.getCompareResult() == LineCompareResult.DELETED) {
				return "Deleted";
			}
			return "Old";

		} else if (columnIndex == 1 && idx == 1) {
			if (cmp.getCompareResult() == LineCompareResult.INSERT) {
				return "Inserted";
			}
			return "New";

		} else  if (columnIndex == 2) {
			return Integer.valueOf(cmp.getLineNumber());
		} else {
			ILineCompare before = beforeRow ? cmp : compareResult.get(true, row);
			//int pref = cmp.line.getPreferredLayoutIdx();
			int lineIdx = columnIndex - additionalFields;
			int r = recordIndex;

			if (columnIndex == keyIdx) {
				lineIdx = Constants.KEY_INDEX;
			}
			
			if (beforeRow && cmp.getLineNumber() == 379) {
				System.out.print('.');
				compareResult.get(beforeRow, row);
			}
			if (r >= description.getRecordCount()) {
				if (idx == 1 && before != null && before.getLine() != null) {
					r = before.getLine().getPreferredLayoutIdx();
				} else if (cmp.getLine() != null){ 
					r = cmp.getLine().getPreferredLayoutIdx();
				} else {
					r = 0;
				}
			}


			if (idx == 1 && before != null && before.getLine() != null ) {
				if (lineIdx >= before.getLine().getLayout().getRecord(r).getFieldCount()
				||  cmp == null || cmp.getLine() == null
				||	Common.trimRight(before.getLine().getField(r, lineIdx))
						.equals(Common.trimRight(cmp.getLine().getField(r, lineIdx)))) {
					return "";
				}
			}

			if (lineIdx >= description.getRecord(r).getFieldCount() || cmp.getLine() == null) {
				return "";
			}

			return cmp.getLine().getField(r, lineIdx);
		}
	}

	@Override
	public String getColumnName(int column) {

		String ret = "";
		switch (column) {
		case 0:
		case 1:
			break;
		case 2:
			ret = LangConversion.convert(LangConversion.ST_COLUMN_HEADING, "Line No");
			break;

		default:
			if (column == keyIdx) {
				ret = LangConversion.convert(LangConversion.ST_COLUMN_HEADING, "Key Field");
			} else {
				int r = recordIndex;
				if (r >= description.getRecordCount()) {
					r = 0;
				}
				if (column - additionalFields < description.getRecord(r).getFieldCount()) {
					ret = description.getRecord(r).getField(column - additionalFields).getName();
				}
			}
			break;
		}
		return ret;
	}

	public void setDisplayRows(ICompareResult compareResult) {

		this.compareResult = compareResult;

		this.fireTableDataChanged();
	}

	/**
	 * @return the recordIdx
	 */
	protected int getRecordIndex() {
		return recordIndex;
	}

	/**
	 * @param recordIdx the recordIdx to set
	 */
	protected boolean setRecordIndex(int recordIdx) {
		int idx = Math.min(recordIdx, description.getRecordCount());

		if (idx >= 0 && recordIndex != idx) {
			this.recordIndex = idx;
			this.fireTableStructureChanged();
			return true;
		}
		return false;
	}

}
