/**
 *
 */
package net.sf.RecordEditor.diff.tbl;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.Options;
import net.sf.RecordEditor.diff.result.ICompareSummary;
import net.sf.RecordEditor.diff.result.ILineCompare;
import net.sf.RecordEditor.diff.result.LineCompareResult;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.LangConversion;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

/**
 * @author Bruce Martin
 *
 */
@SuppressWarnings("serial")
public class LineDisplay extends AbstractCompareDisplay {

    private static Color ADDED_COLOR    = Color.yellow;
    private static Color DELETED_COLOR  = Color.cyan;
    private static Color MODIFIED_COLOR = Color.green;

	private CmpLineModel model;

	private JButton[] btn       = new JButton[4];
	private String[] moveHints  = LangConversion.convertArray(LangConversion.ST_FIELD_HINT, "RecordBtns", new String[] {
	        	"Start of File", "Previous Record",
	        	"Next Record",   "Last Record"});

	private ActionListener lineListner = new ActionListener() {
		public void actionPerformed(ActionEvent event) {

				//stopCellEditing();

				if (event.getSource() == btn[0]) {
					setCurrRow(0);
					rowChanged();
				} else if (event.getSource() == btn[1]) {
					if (getCurrRow() > 0) {
						setCurrRow(getCurrRow() - 1);
						rowChanged();
					}
				} else if (event.getSource() == btn[2]) {
					if (getCurrRow() < cmpResult.getRowCount() - 1) {
						//int hold = getCurrRow();
						//System.out.println("Starting " + hold);
						setCurrRow(getCurrRow() + 1);
						rowChanged();
						//System.out.println("Ending ... " + hold + " " + getCurrRow());
					}
				} else if (event.getSource() == btn[3]) {
					setCurrRow(cmpResult.getRowCount() - 1);
					rowChanged();
				}

				//System.out.println("Lister End " + (event.getSource() == btn[2]));

		//	}
		}
	};

	/**
	 * @param name
	 * @param recordLayout
	 * @param before
	 * @param after
	 * @param chgBefore
	 * @param chgAfter
	 * @param primary
	 */
	public LineDisplay(String name,
			AbstractLayoutDetails recordLayout,
			ICompareSummary cmpSummary,
			//ArrayList<LineCompare> before,
			//ArrayList<LineCompare> after, ArrayList<LineCompare> chgBefore,
			//ArrayList<LineCompare> chgAfter, 
			boolean primary, boolean allRows) {
		super("Record Display ", name, recordLayout, cmpSummary,
				//before, after, chgBefore, chgAfter,
				primary, allRows);

		Icon[] icon = Common.getArrowIcons();
		JPanel btnPanel = new JPanel();
		for (int i = 0; i < btn.length; i++) {
			btn[i] = new JButton("", icon[i]);
			btnPanel.add(btn[i]);
			btn[i].addActionListener(lineListner);
			btn[i].setToolTipText(moveHints[i]);
		}
        super.setDisplayFields(ALL_FIELDS);

        init_100_SetupJtables();

        pnl.addComponentRE(1, 5, BasePanel.FILL, BasePanel.GAP,
                         BasePanel.FULL, BasePanel.FULL,
                         tblDetails);

        pnl.addComponentRE(1, 5, BasePanel.PREFERRED, BasePanel.GAP,
		        BasePanel.FULL, BasePanel.FULL,
                btnPanel);

        addMainComponent(pnl);


        pack();
        setBounds(1, 1,
                Math.min(this.getWidth() + 200, screenSize.width  - 1),
                this.getHeight());

        /* set tot prefered Inde */
        model.setRecordIdx(Integer.MAX_VALUE);
        super.setLayoutIndex(model.getRecordIdx());

        setVisible(true);
	}


	/**
	 * Setup Table details
	 */
	private void init_100_SetupJtables() {
		Render cellRenderer = new Render();
		model = new CmpLineModel(layout, cmpResult);

		setDisplay(displayType);
		tblDetails = new JTable(model);
		tblDetails.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		Common.calcColumnWidths(tblDetails, 0, 220);

		tblDetails.getColumnModel().getColumn(3).setCellRenderer(cellRenderer);
		tblDetails.getColumnModel().getColumn(4).setCellRenderer(cellRenderer);
		
		int tblHeight = super.layout.getOption(Options.OPT_TABLE_ROW_HEIGHT);
		
		if (tblHeight <= 1 && cmpResult.getRowCount(false) > 0) {
			AbstractLine line;
			int i = 0;
			while ((line = cmpResult.get(false, i).getLine()) == null) {
				i += 1;
			}
			if (line != null) {
				tblHeight = line.getLayout().getOption(Options.OPT_TABLE_ROW_HEIGHT);
			}
		}
		if (tblHeight > 1) {
			tblDetails.setRowHeight(tblHeight * SwingUtils.values().TABLE_ROW_HEIGHT);
		} 

	}


	/**
	 * @see net.sf.RecordEditor.diff.tbl.AbstractCompareDisplay#setDisplay(int)
	 */
	@Override
	public void setDisplay(int type) {
		super.setDisplay(type);

		model.setCurrentRow(Math.min(model.getCurrentRow(), cmpResult.getRowCount()));
		model.setDisplayRows(cmpResult);
	}

	/**
	 * @see net.sf.RecordEditor.diff.tbl.AbstractCompareDisplay#changeLayout()
	 */
	@Override
	public void changeLayout() {
		if (model != null) {
			model.setRecordIdx(getLayoutIndex());
		}
	}


	public void setDisplayFields(int fieldDisplay) {

		model.setDisplayChangedFields(fieldDisplay == CHANGED_FIELDS);
		super.setDisplayFields(fieldDisplay);
	}


	/**
	 * @see net.sf.RecordEditor.diff.tbl.AbstractCompareDisplay#getCurrRow()
	 */
	@Override
	public int getCurrRow() {

		return model.getCurrentRow();
	}

	public void setCurrRow(int newRow) {
		model.setCurrentRow(newRow);
		//model.fireTableDataChanged();
	}

	public void rowChanged() {
		super.setLayoutIndex(model.getRecordIdx());
	}


	/**
	 * @see net.sf.RecordEditor.diff.tbl.AbstractCompareDisplay#setCurrRow(int, int, int)
	 */
	@Override
	public void setCurrRow(int newRow, int layoutId, int fieldNum) {
		// TODO Auto-generated method stub

	}


	private class Render implements TableCellRenderer {


	    private DefaultTableCellRenderer text    = new DefaultTableCellRenderer();

	    public Render() {
	    	text.setBorder(null);
	    }
		/**
		 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent
		 * (javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
		 */
		public Component getTableCellRendererComponent(
			JTable tbl,
			Object value,
			boolean isCellSelected,
			boolean hasFocus,
			int row,
			int column) {

			if (value == null) {
				text.setText("");
			} else {
				text.setText(value.toString());
			}

    		ILineCompare cmp  = model.getCurrentCompareLine();
    		Color color = Color.WHITE;

    		if (row == 0) {
    		} else if (cmp == null || cmp.getLine() == null) {
    			if (column == 4) {
    				color = ADDED_COLOR;
    			}
    		} else if (cmp.getCompareResult() == LineCompareResult.DELETED  && column == 3) {
    			color = DELETED_COLOR;
    		} else if (cmp.getCompareResult() == LineCompareResult.CHANGED
    			   &&  model.isFieldChanged(row)) {
     			color = MODIFIED_COLOR;
    		}
    		text.setBackground(color);

			return text;
		}
	}
}
