package net.sf.RecordEditor.diff.xml;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.RecordEditor.diff.DoCompare;
import net.sf.RecordEditor.edit.util.StandardLayouts;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.msg.UtMessages;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.screenManager.ReMsg;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.treeCombo.FileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.recordEditor.ui.listManagers.IListItem;
import net.sf.recordEditor.ui.listManagers.IRecentList;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

/**
 * Class does either Single File compare or  
 * @author bruce
 *
 */
public class XmlCompareOptions implements ActionListener {

	protected static final String TWO_FILES_MSG = "You must enter two Xml files";
	protected static final String TWO_DIRECTORIES_MSG = "You must enter two directories";
	private final ReFrame frame;
	public final BaseHelpPanel pnl = new BaseHelpPanel("DiffXml");
	
	public final IFileSelectCombo dir1, dir2;
	public final JTextField filterTxt = new JTextField(11);
	public final JButton compareBtn = SwingUtilsBldr.newButton("compare");
	
	public final JTextField msgFld = new JTextField();
	public final JCheckBox ignoreBlankSpaceChk = new JCheckBox("Ignore blank space between tags");
	
	private final boolean directoryCompare;
	
	private final String noDirectoryMsg;
	private final ReMsg fileDoesNotExist;
	
	public XmlCompareOptions(boolean directoryCompare) {
		
		this.directoryCompare = directoryCompare;
		String xmlList  = Parameters.XML_FILE_LIST;
		String xmlList1 = Parameters.XML_FILE_LIST1;
		String frameName = "Xml Directory Compare";
		if (directoryCompare) {
			xmlList  = Parameters.XML_DIR_LIST;
			xmlList1 = Parameters.XML_DIR_LIST1;
			
			noDirectoryMsg = TWO_DIRECTORIES_MSG;
			fileDoesNotExist = UtMessages.DIRECTORY_DOES_NOT_EXIST;
		} else {
			frameName = "Xml File Compare";
			noDirectoryMsg = TWO_FILES_MSG;
			fileDoesNotExist = UtMessages.FILE_DOES_NOT_EXIST;
		}
		frame = new ReFrame("", frameName, null);
		
		this.dir1 = FileSelectCombo.newCombo(xmlList,  25, true, directoryCompare);
		this.dir2 = FileSelectCombo.newCombo(xmlList1, 25, true, directoryCompare);
		
		init_200_LayoutScreen();
		init_300_finalise();
	}

	private void init_200_LayoutScreen() {
		String prompt = "File ";
		if (directoryCompare) {
			prompt = "Directory ";
		}
		
		pnl.addLineRE(prompt + "1" , dir1)
		   .addLineRE(prompt + "2" , dir2)
		   	 .setGapRE(BaseHelpPanel.GAP1);
		
		if (directoryCompare) {
			pnl .setFieldsToActualSize();
			pnl	.addLineRE("", ignoreBlankSpaceChk)
				.addLineRE("Filter", filterTxt)
		   			.setGapRE(BaseHelpPanel.GAP1)
		   			.setFieldsToFullSize();
		}
		
		JPanel p = new JPanel(new BorderLayout());
		p.setBorder(BorderFactory.createEmptyBorder());
		p.add(BorderLayout.EAST, compareBtn);

		pnl.addLineRE("", p)
		   		.setGapRE(BaseHelpPanel.GAP0)
		   .addMessage(msgFld);
	}
	
	private void init_300_finalise() {
		int width = SwingUtils.values().CHAR_FIELD_WIDTH * 90;
		int screenWidth = ReFrame.getDesktopWidth();
		if (screenWidth > 0 && screenWidth < width) {
			width = screenWidth;
		}

		filterTxt.setText(Parameters.getString(Parameters.XML_DIR_FILTER));
		
		pnl.setPreferredSize(new Dimension(width, pnl.getPreferredSize().height));
		frame.setDefaultCloseOperation(ReFrame.DISPOSE_ON_CLOSE);
		frame.addMainComponent(pnl);

		frame.setVisible(true);
		

		compareBtn.addActionListener(this);
		
		setFileName(dir1);
		setFileName(dir2);
	}

	private void setFileName(IFileSelectCombo directoryCombo) {
		IRecentList recentList = directoryCombo.getRecentList();
		List<IListItem> itemList;
		
		if ( recentList != null 
		&&  (itemList = recentList.getItemList()) != null
		&&   itemList.size() > 0) {
			directoryCombo.setFileNameSilently(itemList.get(0).getItemText());
		}
//		List<FileTreeComboItem> list = directoryCombo.getFileComboList();
//		if (list != null && list.size() > 0) {
//			directoryCombo.setText(list.get(0).getFullname());
//		}
	}

	@Override 
	public void actionPerformed(ActionEvent e) {
		
		String dir1Str = dir1.getText();
		String dir2Str = dir2.getText();
		File f1 = new File(dir1Str);
		File f2 = new File(dir2Str);

		msgFld.setText("");
		if (dir1Str == null || dir1Str.length() == 0 || dir2Str == null || dir2Str.length() == 0) {
			Common.logMsg(noDirectoryMsg, null);
			msgFld.setText(noDirectoryMsg);
		} else if (! f1.exists()) {
			msgFld.setText(fileDoesNotExist.get(dir1Str));
			dir1.requestFocus();
		} else if (! f2.exists()) {
			msgFld.setText(fileDoesNotExist.get(dir2Str));
			dir2.requestFocus();
		} else if (f1.isDirectory() != f2.isDirectory()) {
			msgFld.setText(UtMessages.ENTER_2_DIRECTORIES.get());
			dir1.requestFocus();
		} else if (f1.isDirectory()) {
			Parameters.setProperty(Parameters.XML_DIR_FILTER, filterTxt.getText());
			compareDirectory(dir1Str, dir2Str);	
		} else {
			compareFile(dir1Str, dir2Str);
		}
	}


	/**
	 * @param dir1Str
	 * @param dir2Str
	 */
	protected void compareDirectory(String dir1Str, String dir2Str) {
		new XmlCompareDir(dir1Str, dir2Str, filterTxt.getText(), ignoreBlankSpaceChk.isSelected());
	}

	/**
	 * @param dir1Str
	 * @param dir2Str
	 */
	protected void compareFile(String dir1Str, String dir2Str) {
		try {
			DoCompare.getInstance().compare1Layout(
					StandardLayouts.getInstance().getXmlLayout(), 
					dir1Str, 
					dir2Str);
		} catch (IOException e1) {
			msgFld.setText(e1.toString());
			Common.logMsg(e1.toString(), e1);
			e1.printStackTrace();
		}
	}
}
