package net.sf.RecordEditor.edit.display;

import java.util.List;

import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.RecordDetail;
import net.sf.JRecord.detailsBasic.IItemDetails;
import net.sf.RecordEditor.re.tree.cobolItem.CobolItemNode;
import net.sf.RecordEditor.re.tree.cobolItem.ParseItems;
import net.sf.RecordEditor.utils.swing.treeTable.AbstractTreeTableModel;
import net.sf.RecordEditor.utils.swing.treeTable.TreeTableModel;

public class CobolRecordTreeModel extends AbstractTreeTableModel
implements TreeTableModel {
	
	private static final int TREE_COLUMN = 0; 
	
	private static String[] COLUMN_NAMES = {
		"Field Name",
		"Position",
		"Length",
		"Type",
		"Data"
	};
	private static ParseItems lastParser;
	public static CobolRecordTreeModel newModel(RecordDetail record, AbstractLine line) {
		List<? extends IItemDetails> cobolItems = record.getCobolItems();
		ParseItems p, lp = lastParser;
		CobolItemNode root;
		
	
		if (lp != null && cobolItems == lp.items) {
			p = lp;
			root = lp.root;
		} else {
			p = new ParseItems();
			root = p.parse(cobolItems);
			lastParser = p;
		}
		
		return new CobolRecordTreeModel(record, root, p.arrayIndexs, line);
	}
	
	public final int firstDataColumn;
	private final RecordDetail record;
	private final CobolItemNode root;
	private final int arrayCount;
	private AbstractLine line;
	
	private CobolRecordTreeModel(RecordDetail record, CobolItemNode root, int arrayCount, AbstractLine line) {
		super(root);
		
		this.record = record;
		this.root = root;
		this.arrayCount = arrayCount;
		this.line = line;
		firstDataColumn = 4 + arrayCount;
	}
	
	public AbstractLine getLine() {
		return line;
	}

	public void setLine(AbstractLine line) {
		this.line = line;
	}

	public RecordDetail getRecord() {
		return record;
	}

	@Override
	public int getColumnCount() {
		return arrayCount < 1 ? 5 : 14 + arrayCount;
	}

	@Override
	public String getColumnName(int column) {
		if (column < 4 || arrayCount < 1) {
			return COLUMN_NAMES[column];
		}
		
		if (column < firstDataColumn) {
			return "Index " + (column - 3); 
		}
		
		return "Data";
	}
	

    public Class<?> getColumnClass(int column) {


        if (column == TREE_COLUMN) {
            return TreeTableModel.class;
        }
        return super.getColumnClass(column);
    }



	@Override
	public Object getValueAt(Object node, int column) {
		CobolItemNode cblNode = (CobolItemNode) node;
	
		IItemDetails cobolItem = cblNode.getCobolItem();
		if (cobolItem == null) {
			return "";
		}
		switch (column) {
		case TREE_COLUMN: return null;
		case 1: return cobolItem.getPosition();
		case 2: return cobolItem.getStorageLength();
		case 3: 
			if (cblNode.isLeaf()) {
				return RecordDetail.getTypeName(cobolItem.getType());
			}
			return "";
		case 4: 
			if (arrayCount < 1) {
				if (cblNode.isLeaf()) {
					return line.getField(cobolItem.getFieldDefinition());
				}
				return "";
			}
		}
		
		if (column < firstDataColumn) {
			return  (column - 4) == cblNode.getIndexLvl() 
						? cblNode.getIndex(): ""; 
		}
		
		if (! cblNode.isLeaf()) {
			return "";
		}
		
		
		return line.getField(
				cobolItem
					.getArrayDefinition()
						.getField(cblNode.buildIndexArray(column - firstDataColumn)));
	}

	@Override
	public CobolItemNode getRoot() {
		return root;
	}

	public int getArrayCount() {
		return arrayCount;
	}
}
