package net.sf.RecordEditor.edit.display;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Icon;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableColumn;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import net.sf.JRecord.Common.AbstractFieldValue;
import net.sf.JRecord.Common.FieldDetail;
import net.sf.JRecord.Common.IFieldDetail;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.RecordDetail;
import net.sf.JRecord.Types.Type;
import net.sf.JRecord.cgen.def.IArrayAnyDimension;
import net.sf.JRecord.detailsBasic.IItemDetails;
import net.sf.RecordEditor.edit.display.util.MovementBtnPnl;
import net.sf.RecordEditor.re.display.AbstractFileDisplayWithFieldHide;
import net.sf.RecordEditor.re.file.FileView;
import net.sf.RecordEditor.re.popups.MenuPopupListener;
import net.sf.RecordEditor.re.tree.cobolItem.CobolItemNode;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReActionHandler;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.HexThreeLineField;
import net.sf.RecordEditor.utils.swing.treeTable.JTreeTable;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public class LineFrameCobolTree extends BaseDisplay 
implements TableModelListener, AbstractFileDisplayWithFieldHide {


	private int currRow;
	protected int stdCellHeight;
	protected int threeLineHeight;
//	protected JCheckBox oneLineHex;

	protected MovementBtnPnl btnPanel;
	
	protected JTextField lineNum = new JTextField();
	protected JTextField valueTxt = new JTextField();
	protected JTextField txtValueTxt = new JTextField();
	protected JTextField hexValueTxt = new JTextField();

	protected JTextArea fullLine;
	private HexThreeLineField hexLine;
	private JTreeTable treeTable;

	private double fullLineHeight;
//	private TableCellRenderer defaultHexRendor;
//	private TableCellRenderer hexRendor = null;
	
	private CobolRecordTreeModel record;
	
	private final boolean changeRow;
	
	private int tblCol = -1;
	private int tblRow = -1;
	
	//private IFieldDetail arrayField, arrayFieldChar;
	
	private HeaderFieldUpdater arrayFieldUpd, arrayFieldTextUpd, arrayFieldHexUpd;


	private final ActionListener listner = new ActionListener() {
			public void actionPerformed(ActionEvent event) {

//				System.out.println("Lister start " + (event.getSource() == btnPanel.btn[2]));
					stopCellEditing();

					if (event.getSource() == btnPanel.buttons[0]) {
						setCurrRow(0);
					} else if (event.getSource() == btnPanel.buttons[1]) {
						changeRow(-1);
					} else if (event.getSource() == btnPanel.buttons[2]) {
						changeRow(1);
					} else if (event.getSource() == btnPanel.buttons[3]) {
						setCurrRow(fileView.getRowCount() - 1);
//					} else if (event.getSource() == oneLineHex) {
//					    ap_100_setHexFormat();
					}
			}
	};
	
	MenuPopupListener popupListner = new MenuPopupListener();
	
	
	protected LineFrameCobolTree(
			final String screenName,
			final FileView viewOfFile,
   		 	final int currRow,
   		 	final boolean changeRow) {
		super(null, screenName, viewOfFile, changeRow, true, true, false, false, NO_OPTION_PANEL);

		this.currRow = currRow;
		this.changeRow = changeRow;
		
		init_200_setupFields(Common.getArrowIcons());
		init_300_setupScreen();
		init_400_finalise();
	}


	protected void init_200_setupFields(Icon[] icon) {
	    //int currLayout;
	    BaseHelpPanel actualPnl = getActualPnl();
	    
	    btnPanel = new MovementBtnPnl(icon, changeRow, listner);
	    AbstractLine line = fileView.getLine(currRow);
		int preferredLayoutIdx = line.getPreferredLayoutIdx();
		setLayoutIndex(preferredLayoutIdx);
		newRecord(preferredLayoutIdx, line);

		init_210_setupTable();

		setFullLine();

		fileView.addTableModelListener(this);
//		fileView.addTreeModelListener(this);
		setColWidths();

		actualPnl.setHelpURLre(Common.formatHelpURL(Common.HELP_SINGLE_RECORD));
	}
	
	/**
	 * Setup line table
	 */
	protected void init_210_setupTable() {

		treeTable = new JTreeTable(record);
		treeTable.getTree().setRootVisible(false);
		treeTable.getTree().setShowsRootHandles(true);
		expandTree(record.getRoot(), 0, 0);


		setJTable(treeTable);

		treeTable.addMouseListener(popupListner);
		treeTable.addMouseListener(new MouseAdapter() {
			@Override public void mouseReleased(MouseEvent e) {
				checkTableRowCol();
			}
		});
		treeTable.addKeyListener(new KeyAdapter() {
			@Override public void keyReleased(KeyEvent e) {
				checkTableRowCol();
			}
			
		});
		
		popupListner.setTable(treeTable);
		setRowHeight();

		if (super.showHexOptions(layout, fileMaster)) {
	//	    TableColumn tc = tblDetails.getColumnModel().getColumn(record.hexColumn);

		    stdCellHeight   = tblDetails.getRowHeight();
		    threeLineHeight = (stdCellHeight + 3) * 3;

//		    oneLineHex = new JCheckBox();
//		    oneLineHex.setSelected(true);
//		    oneLineHex.addActionListener(listner);

//			defaultHexRendor = tc.getCellRenderer();
			//defaultHexEditor = tc.getCellEditor();
			hexLine = new HexThreeLineField(this.layout.getFontName());
			fullLineHeight = BasePanel.GAP5;
			fullLine = hexLine;
		} else if (layout.isBinCSV()) {
			hexLine = new HexThreeLineField(this.layout.getFontName());
			fullLineHeight = BasePanel.GAP5;
			fullLine = hexLine;
		} else {
		    fullLine = new JTextArea();
		    fullLineHeight = BasePanel.GAP3;
		}
		fullLine.setEditable(false);
	}


	/**
	 * Layout the screen
	 * @param btnPanel panel of buttons
	 */
	public void init_300_setupScreen() {

		BaseHelpPanel actualPnl = getActualPnl();
		actualPnl.setFieldsToActualSize();
		actualPnl.addLineRE("Record", lineNum);
		actualPnl.setFieldsToFullSize();
		
		actualPnl.addLineRE("Value", valueTxt);
		if (fileMaster.isBinaryFile()) {
//		    actualPnl.addLineRE("1 line Hex", oneLineHex);
			JSplitPane sp = new JSplitPane(
					JSplitPane.HORIZONTAL_SPLIT, 
					txtValueTxt, hexValueTxt);
			sp.setResizeWeight(0.5);
			actualPnl.addLineRE("Text/Hex Value", sp);
		} else {
			actualPnl.addLineRE("Text Value", txtValueTxt);
		}
		//pnl.setGap(BasePanel.GAP1);

		actualPnl.addComponentRE(1, 5, BasePanel.FILL,
		        BasePanel.GAP,
		        BasePanel.FULL, BasePanel.FULL,
		        treeTable);


		if (changeRow) {
			if (! this.layout.isXml()) {
			    actualPnl.addComponentRE(1, 5, fullLineHeight, BasePanel.GAP,
			        BasePanel.FULL, BasePanel.FULL, new JScrollPane(fullLine));
			}

			setDirectionButtonStatus();
			actualPnl.addComponentRE(1, 5, BasePanel.PREFERRED, BasePanel.GAP,
					BasePanel.FULL, BasePanel.FULL, btnPanel);
		} else {
			Common.calcColumnWidths(tblDetails, 0);

			int w1 =  84 * SwingUtils.values().CHAR_FIELD_WIDTH;
			int w2 = tblDetails.getPreferredSize().width;
			if (w2 > 0 && w1 > w2) {
				w1 = w2;
			}
			Dimension d = new Dimension(w1 +  6 * SwingUtils.values().CHAR_FIELD_WIDTH, actualPnl.getPreferredSize().height);
			actualPnl.setPreferredSize(d);
//			System.out.println("@@@  >>> " + w1 + " " + w2 + " " + d.width + this.actualPnl.getPreferredSize().width);
		}
	}

	private void init_400_finalise() {
				
		setLineNumberDisplay();
		updateCellValueFields(0, 0);
		
		arrayFieldUpd = new HeaderFieldUpdater(valueTxt);
		arrayFieldTextUpd = new HeaderFieldUpdater(txtValueTxt);
		
		if (fileMaster.isBinaryFile()) {
			arrayFieldHexUpd = new HeaderFieldUpdater(hexValueTxt);
		}

	}
	
	/**
	 * Set the Hex display format
	 *
	 */
	protected void ap_100_setHexFormat() {
//		JTable table = getJTable();
//	    TableColumn tc = table.getColumnModel().getColumn(record.hexColumn);
//
//	    record.setOneLineHex(oneLineHex.isSelected());
//	    if (oneLineHex.isSelected()) {
//			tc.setCellRenderer(defaultHexRendor);
//
//		    table.setRowHeight(stdCellHeight);
//	    } else {
//	        if (hexRendor == null) {
//	            hexRendor = new HexThreeLineRender(getFileView().getLayout().getFontName());
//	        }
//	        table.setRowHeight(threeLineHeight);
//			tc.setCellRenderer(hexRendor);
//			//tc.setCellEditor(hexEditor);
//	    }
	}
	
	/**
	 * 
	 */
	private void checkTableRowCol() {
		
		int currTblCol = treeTable.getSelectedColumn();
		int currTblRow = treeTable.getSelectedRow();
		
		if (currTblCol >= 0 && currTblRow >= 0 
		&& (currTblCol != tblCol || currTblRow != tblRow)) {
			if (currTblRow != tblRow || this.record.getArrayCount() > 0) {
				updateCellValueFields(currTblRow, Math.max(0, tblCol - record.firstDataColumn));
			}
			
			tblRow = currTblRow;
			tblCol = currTblCol;
		}
	}


	/**
	 * @param currTblRow
	 */
	private void updateCellValueFields(int currTblRow, int col) {
		TreePath pathForRow = treeTable.getPathForRow(currTblRow);
		if (pathForRow != null) {
			CobolItemNode node = (CobolItemNode) pathForRow.getLastPathComponent();
			IItemDetails cblItem = node.getCobolItem();
			IFieldDetail arrayField;
			
			IArrayAnyDimension arrayDefinition = cblItem.getArrayDefinition();
			if (arrayDefinition != null) {
				arrayField = arrayDefinition.getField(node.buildIndexArray(Math.max(0, col - record.firstDataColumn)));
			} else {
				arrayField = cblItem.getFieldDefinition();
			}
			if (arrayField != null) {
				arrayFieldUpd.field = arrayField;
				AbstractLine line = this.fileView.getLine(currRow);
				AbstractFieldValue fieldValue = line.getFieldValue(arrayField);
				arrayFieldTextUpd.setFieldDef(arrayField, Type.ftCharNoTrim);
							
				this.valueTxt.setText(fieldValue.asString());
				Object o = line.getField(arrayFieldTextUpd.field);
				this.txtValueTxt.setText(o == null ? "" : o.toString());
				if (fileMaster.isBinaryFile()) {
					arrayFieldHexUpd.setFieldDef(arrayField, Type.ftHex);
					this.hexValueTxt.setText(fieldValue.asHex());
				}				
			}
		}
	}

	@Override
	protected int getInsertAfterPosition() {
		return currRow;
	}

	@Override
	public void fireLayoutIndexChanged() {
		int layoutIndex = super.getLayoutIndex();
		RecordDetail rec = (RecordDetail) layout.getRecord(layoutIndex);
		
		if (record.getRecord() != rec) {
			changeRecordDtl(
					fileView.getLine(currRow),
					layoutIndex);
		}
	}

	@Override
	public int getCurrRow() {
		// TODO Auto-generated method stub
		return currRow;
	}

	@Override
	public void setCurrRow(int newRow, int layoutId, int fieldNum) {
		// TODO Auto-generated method stub

	}

	@Override
	protected BaseDisplay getNewDisplay(FileView view) {
		return new LineFrameCobolTree("Record (tree):", view, currRow, changeRow);
	}

	private void changeRow(int changeAmount) {
		setCurrRow(currRow + changeAmount);
	}
	/**
	 * Set the current display row
	 *
	 * @param newRow new row to be displayed
	 */
	public void setCurrRow(int newRow) {
		if ((newRow >= 0) && (currRow != newRow)) {
			currRow = newRow;
			rowChanged();
		}
	}


    /**
     * @see net.sf.RecordEditor.utils.common.ReActionHandler#executeAction(int)
     */
    public void executeAction(int action) {

        if (action == ReActionHandler.REPEAT_RECORD_POPUP) {
        	getFileView().repeatLine(currRow);
        	changeRow(1);
        } else {
            super.executeAction(action);
        }
    }


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.edit.display.BaseLineDisplay#isActionAvailable(int)
	 */
	@Override
	public boolean isActionAvailable(int action) {
		if (action == ReActionHandler.REPEAT_RECORD_POPUP) {
			return ! fileView.isBrowse();
	    }
		return super.isActionAvailable(action);
	}

	/**
	 * This method updates the Screen Display after a change of record.
	 * It updates the layout if required
	 */
	private void rowChanged() {
		int[] colWidths = getColumnWidths();
		if (fileView != null) {
			try {
				fileView.removeTableModelListener(this);
	
			    stopCellEditing();
		
				if (currRow >= getFileView().getRowCount()) {
					if (currRow == 0) {
						this.closeWindow();
					} else {
						currRow -= 1;
					}
				}
		
				AbstractLine line = fileView.getLine(currRow);
				
				if (line == record.getLine()) {
					return;
				}
				setLineNumberDisplay();
				
				int newIdx = line.getPreferredLayoutIdx();
				RecordDetail rec = (RecordDetail) layout.getRecord(newIdx);
		
				if (record.getRecord() == rec) {
					record.setLine(line);
					treeTable.repaint();
				} else {
					changeRecordDtl(line, newIdx);
				}
//				record.setCurrentLayout(getLayoutIndex());
//		
//				record.setCurrentLine(currRow, getLayoutIndex());
//				if (record.getCurrentLine() == null) {
//					getParentFrame().close(this);
//				}
//				int newIdx = record.getCurrentLayout();
		
				if ((newIdx != Common.NULL_INTEGER)
				&&  (newIdx != getLayoutIndex())) {
			    	   setLayoutIndex(newIdx);
		
			    	   fireLayoutIndexChanged();
				}
				setColWidths();
				setDirectionButtonStatus();
				setFullLine();
		
				setColumnWidths(colWidths);
				super.notifyChangeListners();
			} finally {
				fileView.addTableModelListener(this);
			}
		}
	}

	/**
	 * This method sets the column sizes in the record layout
	 */
	private final void setColWidths() {

		JTable table = getJTable();
	    if (table != null) {
	    	int col = record.firstDataColumn;
	    	setStandardColumnWidths(col);
//	    	TableColumn tc =  table.getColumnModel().getColumn(col++);

//	        if (cellRenders != null) {
//	        	//System.out.println("BaseLineFrame Rendor 3 ~~> 1 ");
//	            tc.setCellRenderer(render);
//	        }
//	        if (cellEditors != null) {
//	            tc.setCellEditor(new ChooseCellEditor(table, cellEditors));
//	        }

//	        tc = table.getColumnModel().getColumn(col++);
//	        tc.setPreferredWidth(180);
//
//	        if (fileView != null && fileView.isBinaryFile()) {
//	        	if (col + 1 < table.getColumnModel().getColumnCount()) {
//		            tc = table.getColumnModel().getColumn(col++);
//		            tc.setPreferredWidth(180);
//	        	}
//	        }
	    }
	}


	protected final void setStandardColumnWidths(int dataCol) {

	//    if (tblDetails != null) {
		JTable tbl = getJTable();
        tbl.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

        TableColumn tc = tbl.getColumnModel().getColumn(0);
        tc.setPreferredWidth(SwingUtils.values().STANDARD_FONT_WIDTH * 21 / 2);
        tc = tbl.getColumnModel().getColumn(1);
        int stdWidth = SwingUtils.values().STANDARD_FONT_WIDTH * 10 / 3;
		tc.setPreferredWidth(stdWidth);
        if (dataCol > 2) {
	        tc = tbl.getColumnModel().getColumn(2);
	        tc.setPreferredWidth(stdWidth);
	        if (dataCol > 3) {
		        tc = tbl.getColumnModel().getColumn(3);
		        tc.setPreferredWidth(
		        		  Common.getColumnWidth(tbl, 3, 0, tbl.getRowCount(), stdWidth)
		        		+ SwingUtils.values().STANDARD_FONT_WIDTH);
	        }
        }
        tc = tbl.getColumnModel().getColumn(dataCol);
        tc.setPreferredWidth(SwingUtils.values().STANDARD_FONT_WIDTH * 11);
	//    }
	}

	/**
	 * @param line
	 * @param newIdx
	 */
	private void changeRecordDtl(AbstractLine line, int newIdx) {
		newRecord(newIdx, line);
		treeTable.setTreeTableModel(record);
		expandTree(record.getRoot(), 0, 0);
	}


	/**
	 * @param newIdx
	 * @param line
	 */
	private void newRecord(int newIdx, AbstractLine line) {
		record = CobolRecordTreeModel.newModel((RecordDetail)layout.getRecord(newIdx), line);
	}
	
	private int expandTree(CobolItemNode node, int count, int hCount) {
		int childCount = node.getChildCount();
		hCount += childCount;
		for (int i = 0; i < childCount; i++) {
			 TreeNode child = node.getChildAt(i);
			 if (child.isLeaf()) {
				count += hCount;
				TreePath path = new TreePath(node.getPath());
				treeTable.getTree().expandPath(path);
				break;
			 } else {
				 count = expandTree((CobolItemNode)child, count, hCount);
				 if (count > 500) {
					 break;
				 }
			 }
			 hCount = 0;
		}
		return count;
	}

	/**
	 * Set enabled / disabled status of the direction buttons
	 */
	protected void setDirectionButtonStatus() {

		boolean allowBack = currRow > 0;
		boolean allowForward = fileView != null && currRow < fileView.getRowCount() - 1;

	    btnPanel.buttons[0].setEnabled(allowBack);
	    btnPanel.buttons[1].setEnabled(allowBack);
	    btnPanel.buttons[2].setEnabled(allowForward);
	    btnPanel.buttons[3].setEnabled(allowForward);
	}

	/**
	 * Setup the full line data
	 */
	protected void setFullLine() {

	    AbstractLine l = record.getLine();
	    if (l == null) {
	    } else if (hexLine != null && (fileMaster.isBinaryFile() || layout.isBinCSV())) {
	    	hexLine.setHex(l.getData());
	    } else {
	        fullLine.setText(l.getFullLine());
	    }
	}


	@Override
	public void tableChanged(TableModelEvent event) {
		int firstRow = event.getFirstRow();
		switch (event.getType()) {
			case (TableModelEvent.INSERT):
				if (firstRow <= currRow) {
					changeRow(event.getLastRow() 	-  firstRow + 1);
				}
			break;
			case (TableModelEvent.DELETE):
				if (currRow > event.getLastRow()) {
					currRow -= event.getLastRow() - firstRow + 1;
					rowChanged();
				} else if (currRow > firstRow) {
					currRow -= Math.min(fileView.getRowCount(), firstRow);
					rowChanged();
				}
			break;
			default:
			    currRow = fileView.indexOf(record.getLine());
			    setLineNumberDisplay();
			    /*System.out.println("~~>> " + currRow + " | "
			            + event.getType()
			            + " ! " + event.getFirstRow()
			            + " # " + event.getLastRow());*/
				//System.out.println("LineFrame Table changed: " + event.getType() + " " + TableModelEvent.UPDATE);
				if (super.hasTheFormatChanged(event) || firstRow < 0) {
					changeRecordDtl(record.getLine(), record.getLine().getPreferredLayoutIdx());
//					record.fireTableDataChanged();
//					record.layoutChanged(layout);
				} else if (firstRow <= currRow
				&&  event.getLastRow() >= currRow
				&&  currRow >= 0 ) {		// Table Structure changed
					treeTable.fireTableDataChanged();
				}
		}
		setFullLine();
	}


	/**
	 * 
	 */
	private void setLineNumberDisplay() {
		lineNum.setText(Integer.toString(currRow + 1));
	}


	@Override
	public boolean[] getFieldVisibility(int recordIndex) {
		// TODO Auto-generated method stub
		return null;
	}


	@Override
	public void setFieldVisibility(int recordIndex, boolean[] fieldVisibility) {
		// TODO Auto-generated method stub
		
	}

	private class HeaderFieldUpdater implements FocusListener {
		final JTextField fieldTxt;
		IFieldDetail field;		
		
		public HeaderFieldUpdater(JTextField fieldTxt) {
			super();
			this.fieldTxt = fieldTxt;
			fieldTxt.addFocusListener(this);
		}
		
		void setFieldDef(IFieldDetail arrayField, int typeId) {
			field = FieldDetail
					.newFixedWidthField(arrayField.getName(), typeId, arrayField.getPos(), arrayField.getLen(), 0, arrayField.getFontName());

		}

		@Override
		public void focusGained(FocusEvent e) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void focusLost(FocusEvent e) {
			if (field != null) {
				fileView.getLine(currRow)
					.getFieldValue(field)
					.set(fieldTxt.getText());
				fileView.fireTableDataChanged();
			}
		}		
		
	}
}
