package net.sf.RecordEditor.edit.display.SaveAs;

import java.io.IOException;
import java.io.OutputStream;

import javax.swing.JCheckBox;
import javax.swing.JEditorPane;
import javax.swing.JSplitPane;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.Common.RecordException;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.RecordEditor.edit.util.ReMessages;
import net.sf.RecordEditor.edit.util.StandardLayouts;
import net.sf.RecordEditor.re.file.DisplayType;
import net.sf.RecordEditor.re.fileWriter.CsvRecordWriter;
import net.sf.RecordEditor.re.fileWriter.FieldWriter;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.util.csv.CsvCode;
import net.sf.RecordEditor.utils.swing.BasePanel;

public class SaveAsRecordCsv extends SaveAsPnlBase {

    private final JCheckBox includeLineNumberChk = new JCheckBox();
    private final JCheckBox blankLineBetweenRecordsChk = new JCheckBox();

	public SaveAsRecordCsv(CommonSaveAsFields commonSaveAsFields) {
		super(commonSaveAsFields, ".csv", CommonSaveAsFields.FMT_RECORD_CSV, RecentFiles.RF_NONE, null, false);
		
		BasePanel pnl1 = new BasePanel();
//		BasePanel pnl2 = new BasePanel();
		
		includeLineNumberChk.setSelected(false);
		blankLineBetweenRecordsChk.setSelected(true);

		pnl1.setFieldNamePrefix("Csv");
		pnl1.addLineRE(                   "Delimiter", delimiterCombo);
		pnl1.addLineRE(                       "Quote", quoteCombo);
		pnl1.addLineRE(                      "Charset", charsetCombo);
		pnl1.addLineRE(         "Include Line Number", includeLineNumberChk);
		pnl1.addLineRE( "Blank Lines between Records", blankLineBetweenRecordsChk);
//		pnl1.addLineRE("names on first line", namesFirstLine);
//		pnl1.addLineRE("Add Quote to all Text Fields", quoteAllTextFields);
		
//		pnl2.setGapRE(3);
//		pnl2.addComponentRE(1, 5, BasePanel.PREFERRED, 1, BasePanel.FULL, BasePanel.FULL,  getSelectBtnPnl());
//		pnl2.addComponentRE(1, 5, 130, BasePanel.GAP,
//		        BasePanel.FULL, BasePanel.FULL, fieldTbl);
//		pnl2.setComponentName(fieldTbl, "CsvColNames");


		JEditorPane area = new JEditorPane(
				"text/html", ReMessages.EXPORT_RECORD_CSV.get());
		panel.addComponentRE(1, 5, BasePanel.FILL, BasePanel.GAP,
		        BasePanel.FULL, BasePanel.FULL,
		        new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pnl1, area));

		setupPrintDetails(false);

	}
	

	public void save(String selection, OutputStream outStream) throws IOException, RecordException {
		String fontname = getCharset();
		String fieldSeperator = CsvCode.checkDelimiter(delimiterCombo.getDelimiter(), fontname);

		fieldSeperator = Conversion.decodeCsvField(fieldSeperator, fontname, '\t');

		
		if (fieldSeperator != null && fieldSeperator.toLowerCase().startsWith("x'") && Conversion.isMultiByte(fontname)) {
			throw new RecordException("Hex seperators can not be used with a multibyte charset");
		}
		AbstractLayoutDetails layout = commonSaveAsFields.file.getLayout();
		boolean showHex = (layout.getFileStructure() < IFileStructureConstants.IO_CSV) && (! layout.isCsvLayout()) && (! layout.isXml());


		FieldWriter writer = new CsvRecordWriter(
				outStream, fieldSeperator, fontname, 
				Conversion.decodeCharStr(getQuote(), fontname, '"'), 
				blankLineBetweenRecordsChk.isSelected(), 
				blankLineBetweenRecordsChk.isSelected(),
				showHex
		);
				
				
				
//				WriterBuilder.newCsvWriter(outStream, fieldSeperator, fontname,
//									Conversion.decodeCharStr(getQuote(), fontname, '"'),
//									quoteAllTextFields.isSelected(),
//									getIncludeFields());


		SaveAsWrite.getWriter(
				DisplayType.PREFFERED, 
				commonSaveAsFields.file,
				commonSaveAsFields.getRecordFrame())
			.writeFile(writer, true, commonSaveAsFields.getWhatToSave(selection));
	
		//save_writeFile(writer, selection);
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.edit.display.SaveAs.SaveAsPnlBase#getEditLayout()
	 */
	@Override
	public AbstractLayoutDetails getEditLayout(String ext, AbstractLayoutDetails l) {

        return StandardLayouts.getInstance().getCsvLayoutNamesFirstLine(
     			   			delimiterCombo.getDelimiter(),
     			   			charsetCombo.getText(),
     			   			getQuote(),
     			   			true);
	}



}
