package net.sf.RecordEditor.edit.display.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JEditorPane;
import javax.swing.JInternalFrame;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import net.sf.RecordEditor.edit.util.ReMessages;
import net.sf.RecordEditor.re.display.AbstractFileDisplay;
import net.sf.RecordEditor.re.display.DisplayBuilderFactory;
import net.sf.RecordEditor.re.display.IDisplayBuilder;
import net.sf.RecordEditor.re.file.FileView;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public class CreateWindows implements ActionListener{
	private static final String FIRST_LINE = "First Line";


	private static final int TIP_HEIGHT  = SwingUtils.values().STANDARD_FONT_HEIGHT * 15 + 5;


	private final BaseHelpPanel pnl = new BaseHelpPanel("CreateWindoes");
	private final ReFrame frame;
	private JEditorPane tips = new JEditorPane("text/html", ReMessages.CREATE_WINDOW_TIP.get());
	private JTextField startTxt = new JTextField(10);
	private JTextField sizeTxt = new JTextField(10);
	private JTextField numberTxt = new JTextField(10);
	private JCheckBox restOfFileChk = new JCheckBox("Split the Whole File");
	private JCheckBox passSortThruChk = new JCheckBox("Sort the Parent File rather than the View ???");
	private JButton goBtn = new JButton("Create Windows");
	private JTextArea msgTxt = new JTextArea();
	
	private final FileView fileView;
	private final AbstractFileDisplay parentFrame;
	
	
	public CreateWindows(AbstractFileDisplay parentFrame) {
		this.parentFrame = parentFrame;
		this.fileView = parentFrame.getFileView();
		
		frame = new ReFrame(
				fileView.getFileNameNoDirectory(), 
				"Create File Windows",
				fileView.getBaseFile());
		
		init_100_setValues();
		init_200_layoutScreen();
		init_300_Finishup();
	}
	
	private void init_100_setValues() {
		startTxt.setText("1");
		
		int size = 10000000;
		int numLines = fileView.getRowCount();
		if (numLines < 30000000) {
			int[] sizes = {10, 50, 100, 500, 1000, 5000, 10000, 50000, 100000, 500000, 1000000};
			int i = 0;
			do {
				size = sizes[i];
			} while (++i < sizes.length && numLines > sizes[i] * 5 );
		}
		sizeTxt.setText(Integer.toString(size));
				
		restOfFileChk.setSelected(true);
		passSortThruChk.setSelected(true);
	}
	
	private void init_200_layoutScreen() {
		pnl
			.addComponentRE(1, 5, TIP_HEIGHT, BasePanel.GAP3,
		        BasePanel.FULL, BasePanel.FULL,
				tips)
			.setFieldsToActualSize();
		pnl
			.addLineRE(FIRST_LINE, startTxt)
			.addLineRE("Window Size", sizeTxt)
			.addLineRE("", restOfFileChk)
			.addLineRE("Number of Windows", numberTxt)
				.setGapRE(BasePanel.GAP0)
			.addLineRE("", passSortThruChk)
			.addLineRE("", BasePanel.NULL_ITEM, goBtn)
				.setGapRE(BasePanel.GAP2)
				.setFieldsToFullSize();
		pnl	.addMessage(msgTxt)
				.setHeightRE(SwingUtils.values().TABLE_ROW_HEIGHT * 2);
	}
	
	private void init_300_Finishup() {
		numberTxt.setEditable(false);
		restOfFileChk.setSelected(true);		frame.addMainComponent(pnl);
		
		frame.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
		frame.setVisible(true);

		restOfFileChk.addActionListener(this);
		goBtn.addActionListener(this);
		
		msgTxt.setText("Number of Lines in View: " + fileView.getRowCount());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == restOfFileChk) {
			numberTxt.setEditable(! restOfFileChk.isSelected());
		} else if (e.getSource() == goBtn) {
			 genWindows();
		}
	}
	
	private void genWindows() {
		int start, size, numberOfWindows;
		
		start = toInt(startTxt, FIRST_LINE);
		size = toInt(sizeTxt, "Window Size");
		
		if (start <= 0 || size <= 0) {
			return;
		}
		int rowCount = fileView.getRowCount();
		if (start > rowCount) {
			msgTxt.setText("You must enter  start position before the end of the file: " + rowCount );
			startTxt.requestFocus();
			return;
		}
		
		int maxSize = (rowCount - start - 1) / size + 1;
		if (restOfFileChk.isSelected()) {
			numberOfWindows = maxSize;
		} else {
			numberOfWindows = Math.min(maxSize, toInt(sizeTxt, "Window Size"));
			if (numberOfWindows <= 0) {
				return;
			}
		}
		
		boolean passSortThrough = passSortThruChk.isSelected();
		int i = 0;
		while (start + size -1 < rowCount && i++ < numberOfWindows) {
			createView(start, start + size - 1, passSortThrough);
			start += size;
		}
		
		createView(start, -1, passSortThrough);
		frame.setVisible(false);
	}

	/**
	 * @param start
	 * @param size
	 * @param passSortThrough
	 */
	private void createView(int start, int end, boolean passSortThrough) {

		FileView view = fileView.getViewRange(start, end, passSortThrough);
		DisplayBuilderFactory.getInstance().newDisplay(IDisplayBuilder.ST_LIST_SCREEN, "From: " + start,
				parentFrame.getParentFrame(), 
				fileView.getLayout(), 
				view,
				0);
		view.setFirstRow(start-1);
	}
	
	private int toInt(JTextField fld, String name) {
		String s = fld.getText();
		int ret = Integer.MIN_VALUE;
		
		if (s.length() == 0) {
			msgTxt.setText("You must enter " + name);
			fld.requestFocus();
		} else {
			try {
				ret = Integer.parseInt(s);
				
				if (ret <= 0) {
					msgTxt.setText(name + " must be");
				}
			} catch (NumberFormatException e) {
				msgTxt.setText("Invalid " + name + " it should be an valid integer");
				fld.requestFocus();
			}
		}
		
		return ret;
	}
}
