package net.sf.RecordEditor.edit.open;

import java.io.File;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;

import net.sf.JRecord.Common.RecordException;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.Options;
import net.sf.JRecord.IO.AbstractLineIOProvider;
import net.sf.JRecord.Log.AbsSSLogger;
import net.sf.RecordEditor.re.file.FileView;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.util.re.ReTabPane;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.LangConversion;

public class LoadFile {


	private static String LOAD_ERROR = LangConversion.convert("Error Loading File:") + " ";
	
	private final RecentFiles recent;
	private final ReTabPane userLayoutSelect;
	private final JTextArea   message;
	private final int initialRow;


	public LoadFile(RecentFiles recent, ReTabPane userLayoutSelect, JTextArea message, int initialRow) {
		super();
		this.recent = recent;
		this.userLayoutSelect = userLayoutSelect;
		this.message = message;
		this.initialRow = initialRow;
	}


	/**
	 * loads a file from disk to memory
	 *
	 * @param pBrowse wether to browse (or edit the file)
	 */
	public void loadFile(JComponent parentPnl, String sFileName, AbstractLineIOProvider ioProvider, boolean pBrowse) {

		//String sFileName = fileName.getText();

		if (sFileName.equals("")) {
		    String s = LangConversion.convert("Please Enter a file name");
			Common.logMsgRaw(s, null);
			message.setText(s);
		} else if ((!Common.OPTIONS.asterixInFileName.isSelected()) && (sFileName.indexOf('*') >= 0)) {
		    String s = LangConversion.convert("invalid filename (* present) ~") + " " + sFileName;
			Common.logMsgRaw(s, null);
			message.setText(s);
		} else {
			File file = new File(sFileName);
			File f = file.getParentFile();
			
			if (f == null || ! f.exists()) {
			    String s = LangConversion.convert("Directory does not exist:") + " " + f.getPath();
				Common.logMsgRaw(s, null);
				message.setText(s);
			} else if (file.isDirectory()) {
			    String s = LangConversion.convert("You have selected a directory and not a file:") + " " + file.getPath();
				Common.logMsgRaw(s, null);
				message.setText(s);
			} else{
				AbstractLayoutDetails fileDescription = userLayoutSelect.getRecordLayout();
				if (fileDescription != null) {
				    try {
				    	fileDescription.getOption(Options.OPT_CHECK_LAYOUT_OK);
				    	processFile(sFileName,
				    			fileDescription,
				    		    ioProvider,
				    		    pBrowse);
				    	userLayoutSelect.clearData();
				    } catch (IOException ioe) {
				    	stdError(ioe);
				    } catch (RecordException e) {
				    	stdError(e);
				    } catch (RuntimeException e) {
				    	String errorMsg = e.getMessage();
				    	if (errorMsg == null) {
				    		Common.logMsgRaw("", e);
				    	} else {
				    		JOptionPane.showInternalMessageDialog(parentPnl, errorMsg);
				    		Common.logMsgRaw(errorMsg, null);
				    	}
	
						e.printStackTrace();
				    } catch (Exception e) {
					    String s = LOAD_ERROR + e.getMessage();
					    Common.logMsgRaw(s, e);
	   					message.setText(s);
	   					e.printStackTrace();
					}
				}
			}
		}
	}


	public void processFile(String sFileName,
			AbstractLayoutDetails layoutDetails,
			AbstractLineIOProvider ioProvider,
			boolean pBrowse) throws Exception {

		FileView file = new FileView(layoutDetails,
    			ioProvider,
    			pBrowse);
		StartEditorExtended startEditor = new StartEditorExtended(file, sFileName, pBrowse || layoutDetails.isReadOnly());

		startEditor.doEdit();
	}



	private void stdError(Exception e) {

	    String s = LOAD_ERROR + e.getMessage();
	    Common.logMsgRaw(s, null);
		message.setText(s);
		e.printStackTrace();
	}
	

	private class StartEditorExtended extends StartEditor {

		public StartEditorExtended(FileView file, String name, boolean browse) {
			super(file, name, browse, message, initialRow);
		}

		@Override
		public void done() {

			if (ok) {
				super.done();

				try {
					if ("".equals(file.getMsg())) {
						//list.moveToFront();

						recent.putFileLayout(fName, userLayoutSelect.getLayoutId());
						//recentList.update();
					}
				} catch (Exception e) {
					Common.logMsg(AbsSSLogger.ERROR, "Error Updating Recent-History file", e.getMessage(), null);
					e.printStackTrace();
				}
			}
		}

	}



}
