/*
 * RecEdit.java
 *
 * Created on 20 May 2004, 08:05
 *
 * # Version 0.60 Bruce Martin 2007/02/16
 *   - lookup Layout last used to edit a file (so the user does not have to
 *     retype it)
 *   - move layout selection to its own class so the code can be shared
 *     with the run velocity program
 *   - Added support of overiding methods (used by the cobol editor)
 *
 * # Version 0.61 Bruce Martin 2007/04/14
 *   - Remove call to BasePanel.done()
 *   - JRecord Spun off, code reorg
 *
 *
 * # Version 0.62 Bruce Martin 2007/04/30
 *   - adding support for enter key
 */

package net.sf.RecordEditor.edit.open;

import java.awt.Rectangle;

import javax.swing.JButton;
import javax.swing.JInternalFrame;

import net.sf.JRecord.IO.AbstractLineIOProvider;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.openFile.IOpenFileInterface;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.screenManager.ReMainFrame;
import net.sf.recordEditor.ui.xEnvironment.Env;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;


/**
 * This class displays the open file dialog for the RecordEditor
 *
 * @author  bymartin
 * @version 0.56
 */
@SuppressWarnings("serial")
public class OpenFile extends ReFrame {

	private IOpenFileInterface openFilePanel;
    private static final int FRAME_WIDTH  
    		= SwingUtils.values().STANDARD_FONT_WIDTH 
    		* (Env.IS_NIX ? 78 : 74);

//	private BaseDisplay display = null;

	private Rectangle frameSize;




	/**
	 * Creating the File & record selection screenm
	 *
	 * @param pInFile File to be read (optional)
	 * @param pInitialRow initinial Row (optional)
	 * @param pIoProvider ioProvider to use when creating
	 *        lines
	 * @param pInterfaceToCopyBooks interface to copybooks
	 * @param layoutCreate1 button 1 to create a layout
	 * @param layoutCreate2 button 2 to create a layout
	 */
	public OpenFile(final String pInFile,
	        	   final int pInitialRow,
	        	   final AbstractLineIOProvider pIoProvider,
//	        	   final CopyBookInterface pInterfaceToCopyBooks,
	        	   final JButton layoutCreate1,
	        	   final JButton layoutCreate2,
	        	   final IUserLayoutSelection selection) {
	    this(OpenFileEditPnl.OLD_FORMAT,
	    	 pInFile, pInitialRow, pIoProvider,
    		//pInterfaceToCopyBooks,
	         layoutCreate1, layoutCreate2,
	         Parameters.getApplicationDirectory() + "Files.txt",
	         Common.HELP_RECORD_MAIN,
	         selection);
	}

	/**
	 * Creating the File & record selection screenm
	 *
	 * @param pInFile File to be read (optional)
	 * @param pInitialRow initinial Row (optional)
	 * @param pIoProvider ioProvider to use when creating
	 *        lines
	 * @param pInterfaceToCopyBooks interface to copybooks
	 * @param layoutCreate1 button 1 to create a layout
	 * @param layoutCreate2 button 2 to create a layout
	 * @param propertiesFiles properties file holding the recent files
	 * @param helpScreen help screen to display
	 */
	public OpenFile(
			final int pFormat,
			final String pInFile,
			final int pInitialRow,
			final AbstractLineIOProvider pIoProvider,
//     	   final CopyBookInterface pInterfaceToCopyBooks,
			final JButton layoutCreate1,
			final JButton layoutCreate2,
			final String propertiesFiles,
			final String helpScreen,
			final IUserLayoutSelection selection) {
		this(	new OpenFileEditPnl(
						pFormat,
						pInFile,
						pInitialRow,
						pIoProvider,
						layoutCreate1,
						layoutCreate2,
						propertiesFiles,
						helpScreen,
						selection)
		);
	}

	public OpenFile(final IOpenFileInterface openFile) {
		this(openFile, FRAME_WIDTH, -1);
	}

	public OpenFile(final IOpenFileInterface openFile, int width, int height) {
		super("", "Open File", "", null);

		openFilePanel = openFile;

		this.getContentPane().add(openFilePanel.getPanel());
		this.pack();

		ReMainFrame masterFrame = ReMainFrame.getMasterFrame();
		if (height < 0) {
			height = getHeight();
		}

		height = Math.min(height, masterFrame.getDesktopHeight() - 2);
		width = Math.min(width, masterFrame.getDesktopWidth() - 2);

		if (width > 0) {
			setBounds(1 /*getY()*/,  getX(), width, height);
		} else {
			Rectangle screenSize = masterFrame.getBounds();

			setBounds(getY(), getX(), screenSize.width  - 7, height);
		}
		frameSize = this.getBounds();

		this.setDefaultCloseOperation(JInternalFrame.HIDE_ON_CLOSE);


		setVisible(true);

		setToMaximum(false);

//		this.addFocusListener(
//				new FocusAdapter() {
//
//					/* (non-Javadoc)
//					 * @see java.awt.event.FocusAdapter#focusGained(java.awt.event.FocusEvent)
//					 */
//					@Override
//					public void focusGained(FocusEvent e) {
//						// TODO Auto-generated method stub
//						super.focusGained(e);
//						System.out.println("Frame Focus Gainded");
//					}
//
//					/* (non-Javadoc)
//					 * @see java.awt.event.FocusAdapter#focusLost(java.awt.event.FocusEvent)
//					 */
//					@Override
//					public void focusLost(FocusEvent e) {
//						// TODO Auto-generated method stub
//						super.focusLost(e);
//						System.out.println("Frame Focus lost");
//					}
//				}
//			);
		
/*		this.addFocusListener(new FocusAdapter() {

			@Override
			public void focusGained(FocusEvent e) {
				System.out.println("Gained Focus: " + OpenFile.this.isMaximum() + " " + OpenFile.this.isMaximumSizeSet());
			}
			
		});
*/
		
	}



	/**
	 * set the bounds to default
	 *
	 */
	public final void setTheBounds() {
	    this.setBounds(frameSize);
	    this.setToMaximum(false);
	}


	/**
	 * @return the openFilePanel
	 */
	public final IOpenFileInterface getOpenFilePanel() {
		return openFilePanel;
	}
}