/*
 * RecEdit.java
 *
 * Created on 20 May 2004, 08:05
 *
 * # Version 0.60 Bruce Martin 2007/02/16
 *   - lookup Layout last used to edit a file (so the user does not have to
 *     retype it)
 *   - move layout selection to its own class so the code can be shared
 *     with the run velocity program
 *   - Added support of overiding methods (used by the cobol editor)
 *
 * # Version 0.61 Bruce Martin 2007/04/14
 *   - Remove call to BasePanel.done()
 *   - JRecord Spun off, code reorg
 *
 *
 * # Version 0.62 Bruce Martin 2007/04/30
 *   - adding support for enter key
 */

package net.sf.RecordEditor.edit.open;


import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextArea;

import net.sf.JRecord.IO.AbstractLineIOProvider;
import net.sf.RecordEditor.re.layoutSelection.def.IStartActionInterface;
import net.sf.RecordEditor.re.layoutSelection.def.ITableChangedListner;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.openFile.IOpenFileExtended;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.openFile.RecentFilesList;
import net.sf.RecordEditor.re.util.re.ReTabPane;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChanged;
import net.sf.RecordEditor.utils.swing.treeCombo.FileTreeComboItem;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.recordEditor.ui.form.help.HelpWindow;
import net.sf.recordEditor.ui.icons.IconManager;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;


/**
 * This class displays the open file dialog for the RecordEditor
 *
 * @author  bymartin
 * @version 0.56
 */
@SuppressWarnings({ "serial" })
public class OpenFilePnl2
extends BaseHelpPanel
implements 
IFileChanged, IStartActionInterface, IOpenFileExtended, ITableChangedListner {

	public final static int OLD_FORMAT = 1;
	public final static int NEW_FILE_FORMAT = 2;

	//protected FileChooser fileName = new FileChooser();
	protected final IFileSelectCombo fileNameTxt;
	private final RecentFiles recent;

    private static final int FRAME_WIDTH  = SwingUtils.values().STANDARD_FONT_WIDTH * 73;

	private final JTextArea   message     = new JTextArea();

//	protected AbstractLayoutDetails fileDescription;
//	protected BaseDisplay display = null;

	private final AbstractLineIOProvider ioProvider;
//	private CopyBookInterface copyBookInterface;

	private Rectangle frameSize;
	private boolean doListener = true;
	private IUserLayoutSelection layoutSelection;

	private JSplitPane splitPanel;

	
//	private boolean lookupRecentLayouts = true;
	
	private final RecentFilesList recentList;
	
	private final LoadFile loadTheFile;


	private final ReTabPane userLayoutSelect;
	
	private JTable lastTbl;

	private KeyAdapter listner = new KeyAdapter() {
		/**
		 * @see java.awt.event.KeyAdapter#keyReleased
		 */
		public final void keyReleased(KeyEvent event) {

			if (event.getKeyCode() == KeyEvent.VK_ENTER) {
				loadFile(false);
			}
		}
	};



	/**
	 * Creating the File & record selection screenm
	 *
	 * @param pInFile File to be read (optional)
	 * @param pInitialRow initinial Row (optional)
	 * @param pIoProvider ioProvider to use when creating
	 *        lines
	 * @param pInterfaceToCopyBooks interface to copybooks
	 * @param layoutCreate1 button 1 to create a layout
	 * @param layoutCreate2 button 2 to create a layout
	 * @param propertiesFiles properties file holding the recent files
	 * @param helpScreen help screen to display
	 */
	public OpenFilePnl2(
			//int pFormat,
			String pInFile,
			int initialRow,
			AbstractLineIOProvider pIoProvider,
			JButton layoutCreate1,
			JButton layoutCreate2,
			String propertiesFiles,
			String helpScreen,
			IUserLayoutSelection newLayoutSelection) {
		super();

		this.ioProvider = pIoProvider;
		this.recent = RecentFiles.getMainRecentFiles(
				propertiesFiles, newLayoutSelection, newLayoutSelection.isFileBasedLayout(), false, true,
				Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getWithStar());
		this.layoutSelection = newLayoutSelection;

		this.fileNameTxt = RecentFiles.newRecordEditorFileCombo(true, false, true, recent);
//		 new TreeComboFileSelect(true, false, true, getRecentList(), recent.getDirectoryList());
		this.recentList = recent.recentFilesList(this);
	
		this.userLayoutSelect = new ReTabPane(
				this, newLayoutSelection, message, layoutCreate1, layoutCreate2, 
				this, this, null);	
		this.loadTheFile = new LoadFile(recent, userLayoutSelect, message, initialRow - 1);
		
		layoutSelection.setMessage(message);

		init_100_RecentFiles(pInFile);
		init_200_ScreenFields();
		init_300_BldScreen(helpScreen, pInFile);
	}




	/**
	 * Load recent files details from the properties file
	 * @param pInFile file to edit
	 */
	protected void init_100_RecentFiles(String pInFile) {
	    Common.setCurrClass(this);

		String ld;

		if (pInFile != null && pInFile.length() > 0) {
		    fileNameTxt.setFileName(pInFile);
		} else if (Common.OPTIONS.useLastDir.isSelected() 
				   && (ld = recent.getLastDirectory()) != null && ld.length() > 0 && (new File(ld)).exists()) {
			fileNameTxt.setFileName(ld);
		} else {
		    fileNameTxt.setFileName(Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getWithStar());
		}
	}

	/**
	 * Initialize Screen Fields
	 */
	private void init_200_ScreenFields() {

		Common.setBounds1(HelpWindow.HELP_FRAME, "Help");

		fileNameTxt.addFileChangedListner(this);
		//addFcFocusListener(this);
	}

	/**
	 * Build the screen
	 * @param helpScreen help screen name
	 * @param layoutCreate1 layout create button 1
	 * @param layoutCreate2 layout create button 2
	 */
	private void init_300_BldScreen(String helpScreen, String pInFile) {

		//BaseHelpPanel pnl = this;
		addReKeyListener(listner);

//		if (format == OLD_FORMAT) {
//			addHelpBtnRE(SwingUtils.getHelpButton());
//		}
		setHelpURLre(Common.formatHelpURL(helpScreen));
		//G:\RecordEdit_Prj\Docs\hlpRE_RecordMain.htm
		
		if (pInFile != null && pInFile.length() > 0) {
		    updateLayoutForFile(pInFile);
		}

	}


	protected final List<FileTreeComboItem> getRecentList() {
		return recent.getFileComboList();
	}
	

	private boolean okToDo = true; 
	/**
	 * Executed prior to display
	 */
	public void done() {
		if (okToDo) {
			try {
				okToDo = false; 
				layoutScreen();
		
				super.done();
		
				setBounds(getY(), getX(), FRAME_WIDTH, getHeight());
				frameSize = this.getBounds();
			} finally {
				okToDo = true; 
			}
		} 
	}

	/**
	 * 
	 */
	private void layoutScreen() {
		
		BaseHelpPanel pnl = new BaseHelpPanel("FileSchemaPnl");
		
		lastTbl = userLayoutSelect.getCurrentJTable();
		
		//pnl.setGapRE(BasePanel.GAP1);
		addFileName(pnl);
		pnl.setGapRE(BasePanel.GAP2);

		addLayoutSelection(pnl);
		
	//	tblPnl.add(lastTbl);
		splitPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pnl, new JScrollPane(lastTbl));
		splitPanel.setDividerLocation(0.4);
	//	splitPanel.setDividerSize(10);
		
		addComponentRE(
				1, 5, BasePanel.FILL, BasePanel.GAP,
                BasePanel.FULL, BasePanel.FULL,
                splitPanel);

		addMessage(new JScrollPane(message));
		setHeightRE(BasePanel.NORMAL_HEIGHT * 4);
	}
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.layoutSelection.def.ITableChangedListner#tableHasChanged(javax.swing.JTable)
	 */
	@Override
	public void tableHasChanged(JTable newTable) {
		
		if (lastTbl != newTable && splitPanel != null) {
			//JPanel pnl = new JPanel();
			if (newTable != null) {
				splitPanel.setBottomComponent(new JScrollPane(newTable));
			} else {
				splitPanel.setBottomComponent(new JPanel());
			}
		}
		lastTbl = newTable;
	}
	
	protected void addFileName(BaseHelpPanel pnl) {

//		if (format == NEW_FILE_FORMAT) {
//			pnl.addLine3to5("File", fileName);
//		} else {
		JButton doEdit  =new JButton(IconManager.ICONS.documentEdit.icon());
		pnl.addLineRE("File", fileNameTxt, doEdit);

//	    pnl.addLineRE("", (JComponent) null, doEdit);
	    
	    doEdit.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent arg0) {
				loadFile(false);
			}
		});
//		}
	}

	protected void addLayoutSelection(BaseHelpPanel pnl) {

		pnl.addComponentRE(
				1, 5, BasePanel.FILL, BasePanel.GAP,
                BasePanel.FULL, BasePanel.FULL, userLayoutSelect.getTabPane());
	    //getLayoutSelection().addLayoutSelection(pnl, fileName, this, lCreate1, lCreate2);
		//pnl.setGapRE(BasePanel.GAP2);
	}

	/**
	 * set the bounds to default
	 *
	 */
	public void setTheBounds() {
	    this.setBounds(frameSize);
	}




	/**
	 * loads a file from disk to memory
	 *
	 * @param pBrowse wether to browse (or edit the file)
	 */
	public void loadFile(final boolean pBrowse) {

		loadTheFile.loadFile(this, fileNameTxt.getText(), ioProvider, pBrowse);
//		String sFileName = fileName.getText();
//
//		if (sFileName.equals("")) {
//		    String s = LangConversion.convert("Please Enter a file name");
//			Common.logMsgRaw(s, null);
//			message.setText(s);
//		} else if ((!Common.OPTIONS.asterixInFileName.isSelected()) && (sFileName.indexOf('*') >= 0)) {
//		    String s = LangConversion.convert("invalid filename (* present) ~") + " " + sFileName;
//			Common.logMsgRaw(s, null);
//			message.setText(s);
//		} else {
//			File f = new File(sFileName).getParentFile();
//			
//			if (f == null || ! f.exists()) {
//			    String s = LangConversion.convert("Directory Does not exist:") + " " + f.getPath();
//				Common.logMsgRaw(s, null);
//				message.setText(s);
//			} else {
//				AbstractLayoutDetails fileDescription = userLayoutSelect.getRecordLayout();
//				if (fileDescription != null) {
//				    try {
//				    	fileDescription.getOption(Options.OPT_CHECK_LAYOUT_OK);
//				    	processFile(sFileName,
//				    			fileDescription,
//				    		    ioProvider,
//				    		    pBrowse);
//				    	userLayoutSelect.clearData();
//				    } catch (IOException ioe) {
//				    	stdError(ioe);
//				    } catch (RecordException e) {
//				    	stdError(e);
//				    } catch (RuntimeException e) {
//				    	String errorMsg = e.getMessage();
//				    	if (errorMsg == null) {
//				    		Common.logMsgRaw("", e);
//				    	} else {
//				    		JOptionPane.showInternalMessageDialog(this, errorMsg);
//				    		Common.logMsgRaw(errorMsg, null);
//				    	}
//	
//						e.printStackTrace();
//				    } catch (Exception e) {
//					    String s = LOAD_ERROR + e.getMessage();
//					    Common.logMsgRaw(s, e);
//	   					message.setText(s);
//	   					e.printStackTrace();
//					}
//				}
//			}
//		}
	}


//	private void processFile(String sFileName,
//			AbstractLayoutDetails layoutDetails,
//			AbstractLineIOProvider ioProvider,
//			boolean pBrowse) throws Exception {
//
//		FileView file = new FileView(layoutDetails,
//    			ioProvider,
//    			pBrowse);
//		StartEditorExtended startEditor = new StartEditorExtended(file, sFileName, pBrowse);
//
//		startEditor.doEdit();
//	}
//
//
//	private void stdError(Exception e) {
//
//	    String s = LOAD_ERROR + e.getMessage();
//	    Common.logMsgRaw(s, null);
//		message.setText(s);
//		e.printStackTrace();
//	}


    /**
     * @return Returns the message.
     */
    public JTextArea getMessage() {
        return message;
    }


 
    @Override
	public void fileChanged(File selectedFile) {
       	if (doListener) {
    		updateLayoutForFile(fileNameTxt.getText());
    	}
	}



//	/* (non-Javadoc)
//	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
//	 */
//	@Override
//	public final void stateChanged(ChangeEvent e) {
//    	if (doListener && (e == null)) {
//    		updateLayoutForFile(fileName.getText());
//    	}
//	}

	/**
     * Get the layoutname for the file
     * @param pFile file to find the layout for
     */
    private void updateLayoutForFile(String pFile) {
	    File f = new File(pFile);
		String recentLayout = recent.getLayoutName(pFile, f);

		userLayoutSelect.readCheckPreview(f, recentLayout);
    }

	/**
	 * get The current file name
	 */
    @Override
	public String getCurrentFileName() {
	    return fileNameTxt.getText();
	}


    /* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#setCurrentFileName(java.lang.String)
	 */
	@Override
	public void setCurrentFileName(String fileName) {
		this.fileNameTxt.setFileName(fileName);
		fileChanged(null);
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#getCurrentDirectoryName()
	 */
	@Override
	public String getCurrentDirectoryName() {
		String name = fileNameTxt.getText();
		File f = new File(name);
		if (f==null || f.isDirectory()) {
			return name;
		}
		return f.getParent();
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#setCurrentDirectoryName(java.lang.String)
	 */
	@Override
	public void setCurrentDirectoryName(String fileName) {
		this.fileNameTxt.setFileName(fileName);
	}



	/**
     * @return Returns the layoutSelection.
     */
    public IUserLayoutSelection getLayoutSelection() {
    	return layoutSelection;
    }


    /**
     * @see net.sf.RecordEditor.utils.LayoutConnection#getCurrentDbName()
     */
    public String getCurrentDbName() {
        return getLayoutSelection().getDatabaseName();
    }

	/**
	 * @return
	 * @see net.sf.RecordEditor.re.openFile.RecentFilesList#getMenu()
	 */
    @Override
	public final JMenu getRecentFileMenu() {
		return recentList.getMenu();
	}


	@Override
	public JMenu getRecentDirectoryMenu() {
		return recentList.getDirectoryMenu();
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#getPanel()
	 */
	@Override
	public BasePanel getPanel() {
		return this;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.LayoutConnection#getCurrentDbIdentifier()
	 */
	@Override
	public int getCurrentDbIdentifier() {
        return getLayoutSelection().getDatabaseIdx();
	}
	
	/**
     * @see net.sf.RecordEditor.utils.LayoutConnection#setRecordLayout(int)
     */
    public void setRecordLayout(int recordId, String layoutName, String newFileName) {

    	if (newFileName == null || newFileName.length() == 0) {
    		layoutSelection.reload();
    		layoutSelection.setLayoutName(layoutName);
    	} else {
	    	try {
				setDoListener(false);
				
				layoutSelection.reload();
				fileNameTxt.setFileNameSilently(newFileName);
				userLayoutSelect.readCheckPreview(new File(newFileName), layoutName);

				if (newFileName != null && newFileName.length() > 0) {
					loadFile(false);
				}
			} finally {
				setDoListener(true);
			}
    	}
    }

    
	/**
	 * @param doListener the doListener to set
	 */
	public final void setDoListener(boolean doListener) {
		this.doListener = doListener;
	}

//
//	private class StartEditorExtended extends StartEditor {
//
//		public StartEditorExtended(FileView file, String name, boolean browse) {
//			super(file, name, browse, message, initialRow);
//		}
//
//		@Override
//		public void done() {
//
//			if (ok) {
//				super.done();
//
//				try {
//					if ("".equals(file.getMsg())) {
//						String layoutName = userLayoutSelect.getLayoutId();
//						//list.moveToFront();
//
//						recent.putFileLayout(fName, layoutName);
//						//recentList.update();
//					}
//				} catch (Exception e) {
//					Common.logMsg(AbsSSLogger.ERROR, "Error Updating Recent-History file", e.getMessage(), null);
//				}
//			}
//		}
//
//	}



}