/*
 * RecEdit.java
 *
 * Created on 20 May 2004, 08:05
 *
 * # Version 0.60 Bruce Martin 2007/02/16
 *   - lookup Layout last used to edit a file (so the user does not have to
 *     retype it)
 *   - move layout selection to its own class so the code can be shared
 *     with the run velocity program
 *   - Added support of overiding methods (used by the cobol editor)
 *
 * # Version 0.61 Bruce Martin 2007/04/14
 *   - Remove call to BasePanel.done()
 *   - JRecord Spun off, code reorg
 *
 *
 * # Version 0.62 Bruce Martin 2007/04/30
 *   - adding support for enter key 
 */

package net.sf.RecordEditor.edit.open;


import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;

import net.sf.JRecord.IO.AbstractLineIOProvider;
import net.sf.RecordEditor.re.layoutSelection.def.IStartActionInterface;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.openFile.IOpenFileExtended;
import net.sf.RecordEditor.re.openFile.RecentDirectoryList;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.openFile.RecentFilesList;
import net.sf.RecordEditor.re.openFile.RecentFilesListMgr;
import net.sf.RecordEditor.re.util.re.ReTabPane;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.favourite.UpdatableParameterList;
import net.sf.RecordEditor.utils.swing.treeCombo.FileTreeComboItem;
import net.sf.recordEditor.ui.def.FileChooserDefs;
import net.sf.recordEditor.ui.def.FileChooserDefs.FileChooserType;
import net.sf.recordEditor.ui.def.file.IFileDirectoryChangedListner;
import net.sf.recordEditor.ui.fileChooser.UiFileChooser;
import net.sf.recordEditor.ui.form.help.HelpWindow;
import net.sf.recordEditor.ui.icons.IconManager;
import net.sf.recordEditor.ui.xfiles.AllFilesFilter;


/**
 * This class displays the open file dialog for the RecordEditor
 *
 * @author  B Martin
 * @version 0.98
 */
@SuppressWarnings({ "serial" })
public class OpenFilePnl3
extends BaseHelpPanel
implements IStartActionInterface, IOpenFileExtended {

	public final static int OLD_FORMAT = 1;
	public final static int NEW_FILE_FORMAT = 2;

	private final RecentFiles recent;

	private final JTextArea   message     = new JTextArea();

//	protected AbstractLayoutDetails fileDescription;
//	protected BaseDisplay display = null;

	private final AbstractLineIOProvider ioProvider;
//	private CopyBookInterface copyBookInterface;

	private Rectangle frameSize;
	private boolean doListener = true;
	private IUserLayoutSelection layoutSelection;

	private JSplitPane splitPanel;

	private boolean okToDo = true; 

	
	private final RecentFilesList recentList;
	
	private final LoadFile loadTheFile;


	private final ReTabPane userLayoutSelect;
	
	
	private UiFileChooser fileListPnl;

	private KeyAdapter listner = new KeyAdapter() {
		/**
		 * @see java.awt.event.KeyAdapter#keyReleased
		 */
		public final void keyReleased(KeyEvent event) {

			if (event.getKeyCode() == KeyEvent.VK_ENTER) {
				loadFile(false);
			}
		}
	};



	/**
	 * Creating the File & record selection screenm
	 *
	 * @param pInFile File to be read (optional)
	 * @param pInitialRow initinial Row (optional)
	 * @param pIoProvider ioProvider to use when creating
	 *        lines
	 * @param pInterfaceToCopyBooks interface to copybooks
	 * @param layoutCreate1 button 1 to create a layout
	 * @param layoutCreate2 button 2 to create a layout
	 * @param propertiesFiles properties file holding the recent files
	 * @param helpScreen help screen to display
	 */
	public OpenFilePnl3(
			//int pFormat,
			String pInFile,
			int initialRow,
			AbstractLineIOProvider pIoProvider,
			JButton layoutCreate1,
			JButton layoutCreate2,
			String propertiesFiles,
			String helpScreen,
			IUserLayoutSelection newLayoutSelection) {
		super();

		this.ioProvider = pIoProvider;
		this.recent = RecentFiles.getMainRecentFiles(
				propertiesFiles, newLayoutSelection, newLayoutSelection.isFileBasedLayout(), false, true,
				Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getWithStar()
		);
		this.layoutSelection = newLayoutSelection;

		//this.fileName   = new TreeComboFileSelect(true, false, true, getRecentList(), recent.getDirectoryList());
		this.recentList = recent.recentFilesList(this);
	
		this.userLayoutSelect = new ReTabPane(
				this, newLayoutSelection, message, layoutCreate1, layoutCreate2, 
				null, this, null);
		this.loadTheFile = new LoadFile(recent, userLayoutSelect, message, initialRow - 1);
		this.fileListPnl = new UiFileChooser(UiFileChooser.newParamBldr()
								.chooserType(FileChooserType.open)
								.showControlButtons(FileChooserDefs.ControlButtonDisplay.FILTERS_APPROVE)
								.initialPath(new File(Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getNoStar()))
								.fileListType(FileChooserDefs.FileChooserDetailDisplay.details)
								.recentFiles(new RecentFilesListMgr(recent))
								.recentDirectory(new RecentDirectoryList(recent))
								.favouriteFiles(UpdatableParameterList.RECORDEDITOR_FAVOURITES)
								.favouriteDirectory(UpdatableParameterList.RECORDEDITOR_FAVOURITE_DIRECTORIES)
								.addFileFilterOption(AllFilesFilter.ALL_FILES)
								.addFileExtensionOption("text", "txt","csv","tsv","log")
								.addFileExtensionOption("csv", "csv","tsv")
								.addFileExtensionOption("log", "log")
								.addFileExtensionOption("xml", "xml","xls")
								.addFileExtensionOption("bin/dat", "bin","dat")
								.addFileExtensionOption("bin", "bin")
								.addFileExtensionOption("dat", "dat")
							.asBuilderAndParameter());
		fileListPnl.setAcceptButtonTxt(IconManager.ICONS.documentEdit, "Edit");

//		this.fileListPnl = new UiFileChooser(FileChooserType.open, ControlButtonDisplay.FILTERS_APPROVE, 
//				new RecentFilesListMgr(recent), new RecentDirectoryList(recent),
//				UpdatableParameterList.RECORDEDITOR_FAVOURITES, 
//				UpdatableParameterList.RECORDEDITOR_FAVOURITE_DIRECTORIES );
		
		layoutSelection.setMessage(message);

		init_100_RecentFiles(pInFile);
		init_200_ScreenFields();
		init_300_BldScreen(helpScreen);
	}



	protected final List<FileTreeComboItem> getRecentList() {
		return recent.getFileComboList();
	}
	


	/**
	 * Load recent files details from the properties file
	 * @param pInFile file to edit
	 */
	protected void init_100_RecentFiles(String pInFile) {
	    Common.setCurrClass(this);

		//fileListPnl.setViewType(FileChooserDetailDisplay.details);
		fileListPnl.addActionListner(new ActionListener() {		
			@Override public void actionPerformed(ActionEvent e) {
				if (JFileChooser.APPROVE_SELECTION.equals(e.getActionCommand())) {
					loadFile(false);
				}
			}
		});
		String ld;
		if (pInFile != null && pInFile.length() > 0) {
			File dir = new File(pInFile);
			fileListPnl.setDirectory(dir);
		    updateLayoutForFile(dir);
		} else if (Common.OPTIONS.useLastDir.isSelected() 
			   && (ld = recent.getLastDirectory()) != null && ld.length() > 0 && (new File(ld)).exists()){
			fileListPnl.setDirectory(new File(ld));
		} else {
			fileListPnl.setDirectory(new File(Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getWithStar()));
		}
	}
	/**
	 * Initialise Screen Fields
	 */
	private void init_200_ScreenFields() {

		Common.setBounds1(HelpWindow.HELP_FRAME, "Help");

		fileListPnl.addFileChooserListner(new IFileDirectoryChangedListner() {
			
			@Override
			public void directoryChanged(File file) {	}
			
			@Override
			public void fileChanged(File file) {
				if (doListener && file != null) {
					updateLayoutForFile(file);
				}				
			}
		});
	}

	/**
	 * Build the screen
	 * @param helpScreen help screen name
	 * @param layoutCreate1 layout create button 1
	 * @param layoutCreate2 layout create button 2
	 */
	private void init_300_BldScreen(String helpScreen) {

		addReKeyListener(listner);

		setHelpURLre(Common.formatHelpURL(helpScreen));
	}

	/**
	 * Executed prior to display
	 */
	public void done() {
		if (okToDo) {
			try {
				okToDo = false; 
				layoutScreen();
		
				super.done();
		
				setBounds(getY(), getX(), ReFrame.getDesktopWidth()-2, getHeight());
				frameSize = this.getBounds();
			} finally {
				okToDo = true; 
			}
		} 
	}

	/**
	 * 
	 */
	private void layoutScreen() {

		splitPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, fileListPnl.getUiContainer(), userLayoutSelect.getTabPane());
		splitPanel.setResizeWeight(0.5);
		splitPanel.setDividerLocation(0.5);

		
		addComponentRE(
				1, 5, BasePanel.FILL, BasePanel.GAP,
                BasePanel.FULL, BasePanel.FULL,
                splitPanel);

		addMessage(new JScrollPane(message));
		setHeightRE(BasePanel.NORMAL_HEIGHT * 4);
		splitPanel.setDividerLocation(0.4);
	}
	
//	
//	protected void addFileName(BaseHelpPanel pnl) {
//
//		pnl.addLineRE("File", fileName);
//	}
//
//	protected void addLayoutSelection(BaseHelpPanel pnl) {
//
//		pnl.addComponentRE(
//				1, 5, BasePanel.FILL, BasePanel.GAP,
//                BasePanel.FULL, BasePanel.FULL, userLayoutSelect.tab);
//	}

	/**
	 * set the bounds to default
	 *
	 */
	public void setTheBounds() {
	    this.setBounds(frameSize);
	}




	/**
	 * loads a file from disk to memory
	 *
	 * @param pBrowse wether to browse (or edit the file)
	 */
	public void loadFile(final boolean pBrowse) {

		loadTheFile.loadFile(this, getCurrentFileName(), ioProvider, pBrowse);
	}



    /**
     * @return Returns the message.
     */
    public JTextArea getMessage() {
        return message;
    }


 
//    /* (non-Javadoc)
//	 * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
//	 */
//	@Override
//	public final void stateChanged(ChangeEvent e) {
//    	if (doListener && (e == null)) {
//    		updateLayoutForFile(fileName.getText());
//    	}
//	}

	/**
     * Get the layoutname for the file
     * @param pFile file to find the layout for
     */
    private void updateLayoutForFile(File f) {
		String recentLayout = recent.getLayoutName(f.getAbsolutePath(), f);

		userLayoutSelect.readCheckPreview(f, recentLayout);
    }

	/**
	 * get The current file name
	 */
    @Override
	public String getCurrentFileName() {
	    File[] selectedFiles = fileListPnl.getSelectedFiles();
	    if (selectedFiles != null && selectedFiles.length > 0) {
	    	return selectedFiles[0].getPath();
	    }
		return "";
	}


    /* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#setCurrentFileName(java.lang.String)
	 */
	@Override
	public void setCurrentFileName(String fileName) {
		fileListPnl.setSelectedFile(new File(fileName));
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#getCurrentDirectoryName()
	 */
	@Override
	public String getCurrentDirectoryName() {
		File folder = fileListPnl.getCurrentDirectory();
		return folder == null ? "" : folder.getName();
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#setCurrentDirectoryName(java.lang.String)
	 */
	@Override
	public void setCurrentDirectoryName(String fileName) {
		File directory = new File(fileName);
		
		if (directory.exists() && directory.isDirectory()) {
			fileListPnl.setDirectory(directory);
		}

	}



	/**
     * @return Returns the layoutSelection.
     */
    public IUserLayoutSelection getLayoutSelection() {
    	return layoutSelection;
    }


    /**
     * @see net.sf.RecordEditor.utils.LayoutConnection#getCurrentDbName()
     */
    public String getCurrentDbName() {
        return getLayoutSelection().getDatabaseName();
    }

	/**
	 * @return
	 * @see net.sf.RecordEditor.re.openFile.RecentFilesList#getMenu()
	 */
    @Override
	public final JMenu getRecentFileMenu() {
		return recentList.getMenu();
	}


	@Override
	public JMenu getRecentDirectoryMenu() {
		return recentList.getDirectoryMenu();
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.openFile.IOpenFileInterface#getPanel()
	 */
	@Override
	public BasePanel getPanel() {
		return this;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.LayoutConnection#getCurrentDbIdentifier()
	 */
	@Override
	public int getCurrentDbIdentifier() {
        return getLayoutSelection().getDatabaseIdx();
	}
	
	/**
     * @see net.sf.RecordEditor.utils.LayoutConnection#setRecordLayout(int)
     */
    public void setRecordLayout(int recordId, String layoutName, String newFileName) {

    	if (newFileName == null || newFileName.length() == 0) {
    		layoutSelection.reload();
    		layoutSelection.setLayoutName(layoutName);
    	} else {
	    	try {
				setDoListener(false);
				
				layoutSelection.reload();
				fileListPnl.setSelectedFile(new File(newFileName));
				
				userLayoutSelect.readCheckPreview(new File(newFileName), layoutName);

				if (newFileName != null && newFileName.length() > 0) {
					loadFile(false);
				}
			} finally {
				setDoListener(true);
			}
    	}
    }


	/**
	 * @param doListener the doListener to set
	 */
	public final void setDoListener(boolean doListener) {
		this.doListener = doListener;
	}



	public void setChooserType(FileChooserDefs.FileChooserType chooserType) {
		fileListPnl.setChooserType(chooserType);
	}
}