package net.sf.RecordEditor.edit.util;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.AbstractTableModel;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.CsvParser.CsvParserManagerChar;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.Details.Line;
import net.sf.JRecord.Details.LineProvider;
import net.sf.JRecord.Details.RecordDetail;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.JRecord.Types.Type;
import net.sf.RecordEditor.re.display.DisplayBuilderFactory;
import net.sf.RecordEditor.re.file.FileView;
import net.sf.RecordEditor.utils.basicStuff.BasicTable2;
import net.sf.RecordEditor.utils.charsets.FontCombo;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.edit.ManagerRowList;
import net.sf.RecordEditor.utils.fileAnalysis.CsvAnalyser;
import net.sf.RecordEditor.utils.fileStorage.DataStoreStd;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.BmKeyedComboBox;
import net.sf.RecordEditor.utils.swing.BmKeyedComboModel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.ComboBoxs.DelimiterCombo;
import net.sf.RecordEditor.utils.swing.ComboBoxs.QuoteCombo;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;


/**
 * Create a Csv File
 * @author Bruce Martin
 *
 */
public class NewCsvFilePasteTable {

	private static final String PANEL_TST_ID = "NewCsvPasteTbl";

	private static int[][] STRUCTURES = {
		{IFileStructureConstants.IO_BIN_TEXT, IFileStructureConstants.IO_BIN_NAME_1ST_LINE},
		{IFileStructureConstants.IO_UNICODE_TEXT, IFileStructureConstants.IO_UNICODE_NAME_1ST_LINE},
	};

	public final ReFrame frame;
	public final BaseHelpPanel panel = new BaseHelpPanel(PANEL_TST_ID);

	private BmKeyedComboModel styleModel = new BmKeyedComboModel(new ManagerRowList(
			CsvParserManagerChar.getInstance(), false));

	private ColumnNameMdl colNameMdl  = new ColumnNameMdl();
	private PasteTblMdl   pasteTblMdl = new PasteTblMdl();

//	private NumField   rowFld		 = new NumField("row", 5, null);
//	private NumField   colFld		 = new NumField("row", 3, colNameMdl);
	
	private JCheckBox  namesInTblChk = new JCheckBox();
	private DelimiterCombo pasteFieldSepCombo = DelimiterCombo.NewTextDelimCombo();
	private QuoteCombo pasteQuoteCombo	 = QuoteCombo.newCombo();

	
	private JCheckBox  namesChk		 = new JCheckBox();
	private JCheckBox  unicodeChk	 = new JCheckBox();
	private FontCombo  fontCombo	 = new FontCombo();
	private DelimiterCombo fieldSepCombo = DelimiterCombo.NewDelimCombo();
	private QuoteCombo     quoteCombo	 = QuoteCombo.newCombo();
    private BmKeyedComboBox parser       = new BmKeyedComboBox(styleModel, false);

	//private JLabel     colNamesLbl   = new JLabel("Column Names:");
	private JTable     colNamesTbl	 = new JTable(colNameMdl);
	private JTable     pasteTbl	     = new JTable(pasteTblMdl);

	private JButton    goBtn	     = SwingUtilsBldr.newButton("Create");

	private JTextField msgTxt	     = new JTextField();

	private ArrayList<String> columnNames = new ArrayList<String>();
	
	private List<List<String>> pasteData = new ArrayList<List<String>>();
	private int colCount = 0;
	
	private final String pasteStr;
	private String lastPasteSep = "";
	private String lastPasteQuote = "";

	public NewCsvFilePasteTable(String tblString) {
		this(new ReFrame("","New Csv File", null), tblString);
	}

	public NewCsvFilePasteTable(ReFrame displayFrame, String tblString) {
		frame = displayFrame;
		pasteStr = tblString;
		
		init_100_Setup();
		init_200_LayoutScreen();
		init_300_Listners();

		frame.setVisible(true);
	}

	private void init_100_Setup() {

		namesChk.setSelected(true);
		fieldSepCombo.setText("<Tab>");
		quoteCombo.setText("\"");
		
		if (pasteStr.indexOf('\t') > 0) {
			pasteFieldSepCombo.setText("<Tab>");
			pasteQuoteCombo.setText("");
			updatePasteTbl("\t", "");
		} else {
			String[] lines = pasteStr.split("\n");
			CsvAnalyser analyse = new CsvAnalyser(lines, lines.length, "utf-8", false);
			String quote = analyse.getQuote();

			pasteFieldSepCombo.setText(analyse.getSeperatorId());
			pasteQuoteCombo.setTextValue(quote == null ? "" : quote);
			updatePasteTbl(pasteFieldSepCombo.getSelectedItemUseText().getKey(), quote);
		}
	}

	private void updatePasteTbl(String sep, String quote) {
		//String sep = pasteFieldSepCombo.getSelectedItemUseText().getKey();
		BasicTable2 bt = new BasicTable2(pasteStr, sep, quote);
		pasteData.clear();
		colCount = 0;
		
		while (bt.hasMoreRows()) {
			List<String> nextRow = bt.nextRowAsList();
			pasteData.add(nextRow);
			colCount = Math.max(colCount, nextRow.size());
		}
		pasteTblMdl.fireTableStructureChanged();
		lastPasteSep = sep;
		lastPasteQuote = quote;
	}

	private void init_200_LayoutScreen() {

		BasePanel pastePnl = new BasePanel(PANEL_TST_ID + "_pasteOpts");
		BasePanel csvPnl = new BasePanel(PANEL_TST_ID + "_csvOpts");
		pastePnl.setFieldsToActualSize();
		csvPnl.setFieldsToActualSize();
		pastePnl.setBorder(BorderFactory.createTitledBorder("Paste Table Details"));
		csvPnl.setBorder(BorderFactory.createTitledBorder("Csv Details"));

		BasePanel.setToCommonWidth(0, 15, fontCombo, fieldSepCombo, quoteCombo);

		pastePnl
			.addLineRE( "Names in paste Table", namesInTblChk)	
			.addLineRE("Paste Field Seperator", pasteFieldSepCombo)
			.addLineRE(          "Paste Quote", pasteQuoteCombo)
		;
		
		csvPnl
			.addLineRE(  "Names on First Line", namesChk)
			.addLineRE(      "Field Seperator", fieldSepCombo)
			.addLineRE(                "Quote", quoteCombo)
			.addLineRE(              "Unicode", unicodeChk)
			.addLineRE(                 "Font", fontCombo)
			.addLineRE(               "Parser", parser)
		;
	

		panel	
			.addComponentRE(1, 5, BasePanel.PREFERRED,
		        BasePanel.GAP1,
		        BasePanel.FULL, BasePanel.FULL,
		        new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pastePnl, csvPnl))
		
			.addComponentRE(1, 5, SwingUtils.values().TABLE_ROW_HEIGHT * 10,
			        BasePanel.GAP1,
			        BasePanel.FULL, BasePanel.FULL,
			        colNamesTbl)
			.addComponentRE(1, 5, SwingUtils.values().TABLE_ROW_HEIGHT * 10,
			        BasePanel.GAP1,
			        BasePanel.FULL, BasePanel.FULL,
			        pasteTbl)
			.addLineRE(null, BasePanel.NULL_ITEM, goBtn)
				.setGapRE(BasePanel.GAP1)
			.addMessage(msgTxt)
				.setHeightRE(BasePanel.HEIGHT_1P6);

		frame.addMainComponent(panel);
		frame.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
	}



	private void init_300_Listners() {

		namesChk.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				boolean visible = namesChk.isSelected();

				colNamesTbl.setVisible(visible);
			}
		});
		
		namesInTblChk.addActionListener(new ActionListener() {
			private boolean toUpdate = true;
			@Override public void actionPerformed(ActionEvent e) {
				if (toUpdate && namesInTblChk.isSelected()) {
					updateColNames();
					toUpdate = false;
				}
				
				pasteTblMdl.fireTableStructureChanged();
			}
		});

		goBtn.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				editFile();
			}
		});
		
		ChangeListener cl = new ChangeListener() {		
			@Override public void stateChanged(ChangeEvent e) {
				String sep = pasteFieldSepCombo.getSelectedItemUseText().getKey();
				String quote = pasteQuoteCombo.getSelectedItemUseText().getKey();
				if (sep.length() > 0 
				&& ((! sep.equals(lastPasteSep)) || (! quote.equals(lastPasteQuote)))) {
					updatePasteTbl(sep, quote);
				}
			}
		};
		pasteFieldSepCombo.addTextChangeListner(cl);	
		pasteQuoteCombo.addTextChangeListner(cl);
	}
	
	private void updateColNames() {
		if (pasteData.size() > 0) {
			columnNames.clear();
			
			List<String> row0 = pasteData.get(0);
			for (int i = 0; i < row0.size(); i++) {
				columnNames.add(row0.get(i));
			}
			
			colNameMdl.fireTableDataChanged();
			if (! namesChk.isSelected()) {
				namesChk.setSelected(true);
			}
		}
	}

	@SuppressWarnings("unchecked")
	private void editFile() {
		Common.stopCellEditing(colNamesTbl);
		
		String sep = fieldSepCombo.getDelimiter();

		if (sep.length() == 0) {
			JOptionPane.showMessageDialog(frame, "You must enter a field Delimiter");
			
			fieldSepCombo.requestFocus();
			return;
		}
		
		LayoutDetail layout = getLayout(sep);
		AbstractLine l;
		DataStoreStd<AbstractLine> store = DataStoreStd.newStore(layout);
		LineProvider<LayoutDetail, Line> p = LineIOProvider.getInstance()
										.getLineProvider(layout);
		for (int rowIdx = namesChk.isSelected() ? 1 : 0; rowIdx < pasteData.size(); rowIdx++) {
			l = p.getLine(layout);
			List<String> rowData = pasteData.get(rowIdx);
			for (int colIdx = 0; colIdx < rowData.size(); colIdx++) {
				l.setField(0, colIdx, rowData.get(colIdx));
			}

			store.add(l);
		}

		FileView file = new FileView(
				store,
				null,
				null,
				false);
		DisplayBuilderFactory.getInstance().newDisplay(DisplayBuilderFactory.ST_INITIAL_EDIT, "", null, file.getLayout(), file, 0);

		frame.setVisible(false);
	}

	private LayoutDetail getLayout(String sep) {
		LayoutDetail layout;
		int i;
		String s;
		byte[] eol = Common.SYSTEM_EOL_BYTES;
		String font = fontCombo.getText();
		String q   = quoteCombo.getQuote();
		int structure = STRUCTURES[toInt(unicodeChk.isSelected())]
		                          [toInt(namesChk.isSelected())];
		RecordDetail.FieldDetails[] flds = new RecordDetail.FieldDetails[colCount];
	    RecordDetail[] recs = new RecordDetail[1];
	
	    for (i = 0; i < flds.length; i++) {
		    s = namesChk.isSelected() && i < columnNames.size() && columnNames.get(i).length() > 0 
		    			? columnNames.get(i) 
		    			: "Column " + i;
            flds[i] = new RecordDetail.FieldDetails(s, s, Type.ftChar, 0, font, 0, "");
            flds[i].setPosOnly(i + 1);
	    }

        recs[0] = new RecordDetail(Common.GENERATED_CSV_SCHEMA_NAME, "", "", Constants.rtDelimited,
        		sep, q, font, flds,
        		((Integer)parser.getSelectedItem()).intValue(), 0);

        if (font != null && font.length() > 0) {
        	try {
				eol = "\n".getBytes(font);
			} catch (UnsupportedEncodingException e) {
			}
        }

        layout  =
            new LayoutDetail("GeneratedCsv", recs, "",
                Constants.rtDelimited,
                eol, "", font, null,
                structure
            );

		return layout;
	}

	private int toInt(boolean b) {
		int ret = 0;
		if (b) {
			ret = 1;
		}
		return ret;
	}
//  ===========================================================




	@SuppressWarnings("serial")
	private class ColumnNameMdl extends AbstractTableModel {

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		@Override
		public int getColumnCount() {
			return 1;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		@Override
		public int getRowCount() {
			return colCount;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			String s = "";

			if (rowIndex < columnNames.size()) {
				s = columnNames.get(rowIndex);
			}

			return s;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
		 */
		@Override
		public String getColumnName(int column) {
			return "Column Name";
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.AbstractTableModel#isCellEditable(int, int)
		 */
		@Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
			return true;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.AbstractTableModel#setValueAt(java.lang.Object, int, int)
		 */
		@Override
		public void setValueAt(Object value, int rowIndex, int columnIndex) {
			String s  = "";
			if (value != null) {
				s = value.toString();
			}
			while (rowIndex >= columnNames.size()) {
				columnNames.add("");
			}
			if (! s.equals(columnNames.get(rowIndex))) {
				columnNames.set(rowIndex, s);
				pasteTblMdl.fireTableStructureChanged();
			}
		}
	}
	
	@SuppressWarnings("serial")
	private class PasteTblMdl extends AbstractTableModel {

		private int rowAdj() {
			return namesInTblChk.isSelected() ? 1 : 0;
		}
				
		/* (non-Javadoc)
		 * @see javax.swing.table.AbstractTableModel#getColumnName(int)
		 */
		@Override
		public String getColumnName(int column) {
			return namesChk.isSelected() && column < columnNames.size() && columnNames.get(column).length() > 0
					? columnNames.get(column) 
					: "Column " + column;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		@Override
		public int getRowCount() {
			return pasteData.size() - rowAdj();
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		@Override
		public int getColumnCount() {
			return colCount;
		}

		/* (non-Javadoc)
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			rowIndex += rowAdj();
			if (rowIndex >= pasteData.size() || columnIndex >= pasteData.get(rowIndex).size()) {
				return "";
			}
			return pasteData.get(rowIndex).get(columnIndex);
		}
		
	}
}
