package net.sf.RecordEditor.layoutWizard;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.text.JTextComponent;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.fileAnalysis.ColumnDetails;
import net.sf.RecordEditor.utils.fileAnalysis.Details;
import net.sf.RecordEditor.utils.fileAnalysis.FieldSearch;
import net.sf.RecordEditor.utils.fileAnalysis.IBasicColumn;
import net.sf.RecordEditor.utils.fileAnalysis.RecordDefinition;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.HexTwoLineRender;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.fieldSelection.FieldSelectionTxtFld;
import net.sf.RecordEditor.utils.tblModels.LineArrayHexModel;
import net.sf.RecordEditor.utils.tblModels.LineArrayModel;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public class ColumnSelector implements FieldSelectionTxtFld.IColumInterface {

    private static final int STANDARD_TABLE_CELL_WIDTH = SwingUtils.values().CHECK_BOX_HEIGHT;
    private static final int COLUMNS_TO_NUMER = 10;

    public int stdLineHeight;

    private AbstractTableModel stdModel, hexModel, fileModel;
    private TableCellRenderer stdRender, hexRender, tblRender;

    private RecordDefinition recordDef;
    //private ArrayList<ColumnDetails> columnList;
	private Details currentDetails;

    public final JCheckBox hexChk = new JCheckBox();
    public final JCheckBox textFileChk = new JCheckBox("Text File");

    public final JCheckBox lookMainframeZoned = new JCheckBox("Mainframe Zoned Numeric");
    public final JCheckBox lookPcZoned = new JCheckBox("PC/Unix Zoned Numeric");
    public final JCheckBox lookComp3 = new JCheckBox("Comp 3");
    public final JCheckBox lookCompBigEndian = new JCheckBox("Binary Integer (Big Endian)");
    public final JCheckBox lookCompLittleEndian = new JCheckBox("Binary Integer (Little Endian)");

    private final JButton clearFieldsBtn = SwingUtilsBldr.newButton("Clear Fields");
    private final JButton addFieldsBtn = SwingUtilsBldr.newButton("Add Fields");

    private  JTextComponent msg;

 	private  int[] colorInd;
    private boolean firstTime = true;
    
    private BasePanel pnl;
    private JPanel optionPnl = new JPanel();

    private JScrollPane fileViewPane = new JScrollPane();
    public final JTable fileTbl = new JTable();
    public final FieldSelectionTxtFld fieldSelectionFld;
    
    private int tblHeight;
	private boolean lastIsTextFile; 
	



    private ActionListener listner = new ActionListener() {

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		@Override
		public void actionPerformed(ActionEvent event) {
			if (event.getSource() == clearFieldsBtn) {
				recordDef.columnDtls.clear();

				if (currentDetails.recordType == Details.RT_MULTIPLE_RECORDS) {
					recordDef.addKeyField(currentDetails, true);
				}
			} else {
				findFields();
			}
			setColorIndicator();
		}
    };


    /**
	 * @param columnsList
	 * @param details
	 * @param colorIndicator
	 */
	public ColumnSelector(JTextComponent message) {
		
		this.fieldSelectionFld = new FieldSelectionTxtFld(this);
		msg = message;

        fileTbl.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        hexChk.setSelected(false);

        lookPcZoned.setSelected(Common.OPTIONS.searchForPcZoned.isSelected());
        clearFieldsBtn.addActionListener(listner);
        addFieldsBtn.addActionListener(listner);
	}



	public void addMouseListner() {
		addMouseListner(new MouseAdapter() {
			public void mousePressed(MouseEvent m) {
			    int col = fileTbl.columnAtPoint(m.getPoint());
			    columnSelected(col);
			}
		});
	}

	public void addMouseListner(MouseListener mouseListner) {
		fileTbl.addMouseListener(mouseListner);
		fileTbl.getTableHeader().addMouseListener(mouseListner);

        hexChk.addActionListener(new ActionListener() {

			/* (non-Javadoc)
			 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
			 */
			@Override
			public void actionPerformed(ActionEvent arg0) {
				flipHex();
			}
		});
	}

	public void addFields(BasePanel pnl, int tblHeight) {
		JPanel btnPnl = new JPanel(new GridLayout(1,1));
		JPanel fileTypePnl = new JPanel(new GridLayout(1,1));

		hexChk.setText("Show Hex");
		
		this.pnl = pnl;
		this.tblHeight = tblHeight;
		
		textFileChk.setSelected(true);
		setupFileOptionPanels();

		btnPnl.add(clearFieldsBtn);
		btnPnl.add(addFieldsBtn);
		
		fileTypePnl.add(textFileChk);
		fileTypePnl.add(hexChk);

		pnl.addLineRE("", fileTypePnl)				.setGapRE(BasePanel.GAP)
		   .addLineRE("Search For", optionPnl) 		.setHeightRE(BasePanel.PREFERRED)
		   											.setGapRE(BasePanel.GAP1)
		   .addLineRE("", btnPnl)	   				.setHeightRE(BasePanel.PREFERRED);
		pnl.addComponentRE(1, 5, BasePanel.PREFERRED, 2,
		        BasePanel.FULL, BasePanel.FULL,
		        fileViewPane);
		
		textFileChk.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				rebuildScreen();
			}
		});
	}
	
	private void setupFileOptionPanels() {
		
		boolean isTextFile  = textFileChk.isSelected();
		
		optionPnl.removeAll();
		optionPnl.setLayout(new GridLayout(isTextFile ? 1 : 3, 2));
		optionPnl.add(lookMainframeZoned);
		optionPnl.add(lookPcZoned);

		hexChk.setVisible(! isTextFile);
		if (isTextFile) {
			//fieldSelectionFld.initiliseText(fileTxt, maxLineLength);
			fileViewPane.setViewportView(fieldSelectionFld.fieldSelectionTxt);
			fileViewPane.setPreferredSize(
					new Dimension(
							fileViewPane.getPreferredSize().width, 
							tblHeight + 3 * SwingUtils.values().TABLE_ROW_HEIGHT));
		} else {
			optionPnl.add(lookComp3);
			optionPnl.add(new JPanel());
			optionPnl.add(lookCompBigEndian);
			optionPnl.add(lookCompLittleEndian);
			
			fileViewPane.setViewportView(fileTbl);
			fileViewPane.setPreferredSize(new Dimension(fileViewPane.getPreferredSize().width, tblHeight));
		}
		lastIsTextFile = isTextFile; 
	}
	/**
     * User has selected a column, need to highlight it
     *
     * @param col column selected by the user
     */
    public final void columnSelected(final int col) {
    	
    	if (col == 0) { return; }
        int searchCol = col + 1;
        int pos;
        int column;

        pos = -1;

        if (recordDef.columnDtls.size() == 0) {
        	recordDef.columnDtls.add(0, new ColumnDetails(searchCol, currentDetails.defaultType.intValue()));
        } else {
            pos = columnSelected_100_getColumnPosition(searchCol);

            column = recordDef.columnDtls.get(pos).start;

            if (searchCol < column) {
                recordDef.columnDtls.add(pos,
                        new ColumnDetails(searchCol, currentDetails.defaultType.intValue()));
            } else if (searchCol == column) {
                recordDef.columnDtls.remove(pos);
            } else {
                recordDef.columnDtls.add(pos + 1,
                        new ColumnDetails(searchCol, currentDetails.defaultType.intValue()));
           }
        }

        setColorIndicator();
   }

    /**
     * Get the position of the field starting at <b>col</b>
     * in the column table <b>columnsList</b>
     *
     * @param col column that we are searching for.
     *
     * @return column table index
     */
    private int columnSelected_100_getColumnPosition(int col) {
        int ret = 0;
        int size = recordDef.columnDtls.size() - 1;

        int column = -1;

        ret = 0;

        if (size > 0) {
            column = recordDef.columnDtls.get(ret).start;
            while (ret < size && (col > column)) {
                ret += 1;
                column = recordDef.columnDtls.get(ret).start;
            }
        }

        return ret;
    }


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.fieldSelection.FieldSelectionTxtFld.IColumInterface#getColumns()
	 */
	@Override
	public List<? extends IBasicColumn> getColumns() {
		return recordDef.columnDtls;
	}

    /**
     * Set the background color for every column in the file
     * based on the fields defined in the Column table
     * <b>columnsList</b>
     */
    public final void setColorIndicator() {
        int i, j, col;
        int ind;
        int size = recordDef.columnDtls.size();

        j = 0;
        ind = 0;
        //if (size > 1) System.out.println("-- ");
        for (i = 0; i < size; i++) {
            col = Math.min(recordDef.columnDtls.get(i).start - 1, colorInd.length);
            //System.out.println("-- " + col + " " + ind);
            while (j < col) {
                colorInd[j++] = ind;
            }
            //if (size > 1) System.out.print(i + " - " + j + "~" + ind + "  ");
            ind = 1 - ind;
        }
        while (j < colorInd.length) {
            colorInd[j++] = ind;
        }
        //if (size > 1) System.out.print(colorInd.length + " " + j + "~" + ind);
        fileModel.fireTableDataChanged();
    }


    public final void setValues(Details detail, RecordDefinition recordDefinition, boolean findFields)
    throws Exception {
    	boolean cp037 = Conversion.isSingleByteEbcidic(detail.fontName);
    	boolean newRecordDef = recordDef != recordDefinition;
        currentDetails = detail;
        recordDef = recordDefinition;

        hexChk.setSelected(hexChk.isSelected() || currentDetails.textPct < 40);
        lookComp3.setSelected(cp037 || Common.OPTIONS.searchForComp3.isSelected());
        lookCompBigEndian.setSelected(cp037 || currentDetails.textPct < 70 || Common.OPTIONS.searchForCompBigEndian.isSelected());
        lookMainframeZoned.setSelected(cp037 || Common.OPTIONS.searchForMainframeZoned.isSelected());
        lookCompLittleEndian.setSelected((! lookCompBigEndian.isSelected())
        			&& Common.OPTIONS.searchForCompLittleEndian.isSelected());

        setValues_100_SetupTable(findFields);
        
        boolean isTextFile = Details.isTextFile(detail.normalCharPercent, detail.fileStructure);
        
        textFileChk.setSelected(isTextFile);
        if (newRecordDef || lastIsTextFile != isTextFile) {
        	rebuildScreen();
        }
    }



	private void rebuildScreen() {
		setupFileOptionPanels();
		
		optionPnl.revalidate();
		fileViewPane.revalidate();
		if (pnl != null) {
			fileViewPane.repaint();
		
			pnl.repaint();
		}
	}


    /**
     * Setup the file table for display
     *
     */
    private void setValues_100_SetupTable(boolean findFields) {

        stdRender = new TblRender();
        stdModel = new LineArrayModel(recordDef.records,
        		currentDetails.fontName, recordDef.numRecords);
        fileModel = stdModel;
        tblRender = stdRender;

        hexModel = new LineArrayHexModel(recordDef.records,
        		recordDef.numRecords);
        hexRender = new HexRender(currentDetails.fontName);

        colorInd = new int[fileModel.getColumnCount()];

        if (firstTime) {
        	firstTime = false;
	        fileTbl.setModel(fileModel);
	        stdLineHeight = fileTbl.getRowHeight();

	        setupTableColumns();
        }
       	flipHex();


        if (findFields
        && Common.OPTIONS.runFieldSearchAutomatically.isSelected()
        && recordDef.searchForFields) {
        	findFields();
        }

         setColorIndicator();
        //fileMdl.fireTableDataChanged();
    }


    private void findFields() {

    	if (lookCompBigEndian.isSelected() && lookCompLittleEndian.isSelected()) {
    		msg.setText("Can only look for one type of binary field !!!");
    		lookCompLittleEndian.setSelected(false);
    	}

    	(new FieldSearch(currentDetails, recordDef)).findFields(
    			lookMainframeZoned.isSelected(), lookPcZoned.isSelected(),
    			lookComp3.isSelected(),
    			lookCompBigEndian.isSelected(), lookCompLittleEndian.isSelected()
    	);

    	recordDef.searchForFields = false;
    }


    /**
     * Read the record from the file <b>detail.filename</b>
     * into the array <b>detail.records</b>
     *
     * @param detail Wizard detail record
     *
     * @throws Exception any errors generated
     */
	@SuppressWarnings("rawtypes")
	public final void readFile(Details detail, RecordDefinition recordDefinition) throws Exception {
        AbstractLineReader reader = detail.getReader();
        int i = 0;
        AbstractLine l;
        byte[] bytes;
        StringBuilder b = new StringBuilder(9000);
        String s;

        currentDetails = detail;
        recordDef = recordDefinition;

        int maxLineLength = 0;
        l = reader.read();
        while (l != null && i < recordDef.records.length) {
            bytes = l.getData();
            s = Conversion.toString(bytes, detail.fontName);
			b.append(s)
             .append('\n');
			maxLineLength = Math.max(maxLineLength, s.length());

            recordDef.records[i++] = bytes;
            l = reader.read();
        }
        recordDef.numRecords = i;
        //fileTxt = b.toString();
		fieldSelectionFld.initiliseText(b.toString(), maxLineLength);
    }
	
	public final void updateFileText(Details detail, RecordDefinition recordDefinition) throws Exception {
        StringBuilder b = new StringBuilder(9000);
        String s;
        int maxLineLength = 0;
		for (int i = 0; i < recordDefinition.numRecords; i++) {
			s = Conversion.toString(recordDefinition.records[i], detail.fontName);
			b.append(s)
             .append('\n');
			maxLineLength = Math.max(maxLineLength, s.length());
		}
		fieldSelectionFld.initiliseText(b.toString(), maxLineLength);
	}

    private void flipHex() {

    	int height = stdLineHeight;
    	fileModel = stdModel;
    	tblRender = stdRender;
    	if (hexChk.isSelected()) {
    	   	fileModel = hexModel;
        	tblRender = hexRender;
        	height = (stdLineHeight + 1) * 2;
    	}

    	fileTbl.setModel(fileModel);
    	fileTbl.setRowHeight(height);

    	setupTableColumns();
        fileModel.fireTableDataChanged();
    }


    private void setupTableColumns() {

       	TableColumn tc;
    	String s;
    	TableColumnModel tcm  = fileTbl.getColumnModel();
    	int cellWidth = STANDARD_TABLE_CELL_WIDTH;

        for (int i = 0; i < fileModel.getColumnCount(); i++) {
            tc = tcm.getColumn(i);
            cellWidth = STANDARD_TABLE_CELL_WIDTH;
           	if (Common.NIMBUS_LAF && i >= 99 && i % 10 == 9) {
         		cellWidth += 8;
          		//System.out.print(" " + i + " " + cellWidth);
        	}
            tc.setPreferredWidth(cellWidth);
            tc.setCellRenderer(tblRender);

            //if (! Common.TEST_MODE) {
            switch ((i + 1) % COLUMNS_TO_NUMER) {
            	case (0): s = "" + ((i + 1) / COLUMNS_TO_NUMER); break;
            	case (5): s = "+";                         break;
            	default : s = " ";
            }
            tc.setHeaderValue(s);
        }
    }

    /**
	 * @return the currentDetails
	 */
	public final Details getCurrentDetails() {
		return currentDetails;
	}


	/**
     * Table cell render to give each field a different
     * alternating background colors
     *
     * @author Bruce Martin
     *
     */
    public class TblRender  implements TableCellRenderer {


    	private DefaultTableCellRenderer txtFld = null;

        /**
         * Table cell render to give each field a different
         * alternating background colors
         * */
        public TblRender() {
            super();

        		txtFld = new DefaultTableCellRenderer();
        		txtFld.setBorder(BorderFactory.createEmptyBorder());

           // this.setBorder(BorderFactory.createEmptyBorder());
//            this.setAlignmentX(JTextField.CENTER_ALIGNMENT);
//            this.setAlignmentY(JTextField.CENTER_ALIGNMENT);
        }


        /**
         * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
         */
    	public Component getTableCellRendererComponent(
    			JTable tbl,
    			Object value,
    			boolean isSelected,
    			boolean hasFocus,
    			int row,
    			int column) {

    	    //this.setText(value.toString());


//    	    System.out.print(" ~> " + row + " " + column + " " + colorInd[column] + " > " + (disp.getBackground() == Color.LIGHT_GRAY)
//    	    		 + " > " + (disp.getBackground() == Color.WHITE));

    	    if (colorInd[column] == 0) {
    	    	txtFld.setBackground(Color.LIGHT_GRAY);
    	    } else {
    	    	txtFld.setBackground(Color.WHITE);
    	    }
//    	    System.out.println(" : " + (disp.getBackground() == Color.cyan)
//   	    		 + " > " + (disp.getBackground() == Color.WHITE));


    	    txtFld.setText(value.toString());



    	    return txtFld.getTableCellRendererComponent(tbl, value, isSelected, hasFocus, row, column);
    	}
    }


    public class HexRender extends HexTwoLineRender {

		/**
		 * @param font
		 */
		public HexRender(String font) {
			super(font);
		}

    	public Component getTableCellRendererComponent(
    			JTable tbl,
    			Object value,
    			boolean isSelected,
    			boolean hasFocus,
    			int row,
    			int column) {

    		Component ret = super.getTableCellRendererComponent(tbl, value, isSelected, hasFocus, row, column);
    	    if (colorInd[column] == 0) {
    	    	ret.setBackground(Color.LIGHT_GRAY);
    	    } else {
    	    	ret.setBackground(Color.WHITE);
    	    }

    		return ret;
    	}

    }
}
