/*
 * @Author Bruce Martin
 * Created on 8/01/2007
 *
 * Purpose:
 * 1st panel of the wizard where we get File and
 * Record Layout details
 *
 * Changes
 * # Version 0.61 Bruce Martin 2007/04/14
 *   - Name changes, remove done method call
 *
 * # Version 0.61b Bruce Martin 2007/05/05
 *  - Changes o support user selecting the default type
 *
 */
package net.sf.RecordEditor.layoutWizard;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.text.BadLocationException;

import net.sf.JRecord.ByteIO.AbstractByteReader;
import net.sf.JRecord.ByteIO.ByteIOProvider;
import net.sf.JRecord.ByteIO.FixedLengthByteReader;
import net.sf.JRecord.ByteIO.TextReader;
import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.util.BuildTypeComboList;
import net.sf.RecordEditor.utils.charsets.FontCombo;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.StreamUtil;
import net.sf.RecordEditor.utils.fileAnalysis.CsvAnalyser;
import net.sf.RecordEditor.utils.fileAnalysis.Details;
import net.sf.RecordEditor.utils.fileAnalysis.FileAnalyser;
import net.sf.RecordEditor.utils.lang.LangConversion;
import net.sf.RecordEditor.utils.swing.AbsRowList;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.BmKeyedComboBox;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChanged;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.TreeCombo;
import net.sf.RecordEditor.utils.tblModels.LineArrayModel;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

/**
 * 1st panel of the wizard where we get File and
 * Record Layout details
 *
 * @author Bruce Martin
 *
 */
@SuppressWarnings("serial")
public class Pnl1File extends WizardPanel  implements IFileChanged {
	
	private static final int FILE_DISPLAY_HEIGHT =  SwingUtils.values().TABLE_ROW_HEIGHT * 12;

	private static final int COLUMNS_TO_NUMBER = 10;

    private JEditorPane tips;
    IFileSelectCombo filenameFld = RecentFiles.newFileCombo(true, false, true);


	JRadioButton fixedLengthBtn	= new JRadioButton("Fixed Length Fields");
	JRadioButton delimitedBtn 	= new JRadioButton("Delimited Fields");
	JRadioButton fixedMultiBtn 	= new JRadioButton("Multiple Records (fixed length)");

	BmKeyedComboBox fileStructureCombo;

	FontCombo fontNameCombo = new FontCombo();
	TreeCombo defaultType;
	final JLabel lengthLbl = new JLabel("Record Length");
	final JTextField lengthTxt = new JTextField(10);

	final JTable fileTbl = new JTable();
	final JTextArea fileTextArea = new JTextArea();
	final JScrollPane fileDisplayPane = new JScrollPane();
	
	JComponent fileDisplay = null;
	


//	private byte[] fileData;

	private LineArrayModel fileMdl;

    private Details wizardDetail;
    
    private boolean doListner = true;
    private int initFormat, initRecLength, textPercent, normalCharPercent;
    

   
    private MouseAdapter setRecordLengthTblListner = new MouseAdapter() {

    	private long last;
		/**
		 * @see java.awt.event.MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
		 */
		@Override
		public void mouseReleased(MouseEvent e) {
			long when = e.getWhen();
			if (doListner && when != last) {
				try {
		           int  row = fileTbl.rowAtPoint(e.getPoint());
		           int  length = fileTbl.columnAtPoint(e.getPoint());
		
		           for (int i = 0; i < row; i++) {
		        	   //System.out.println(" --> " + i + " " + length);
		        	   length += fileMdl.getLine(i).length;
		           }
		    	   //System.out.println(" --> " + (row - 1)  + " "+ length);
		
		           doListner = false;
		           lengthTxt.setText(Integer.toString(length));
		
//		           System.out.println(" --> setRecordLengthTblListner" );
		           readFile(false);
		           last = when;
				} finally {
					doListner = true;
				}
			}
		}
	};
	
    private MouseAdapter setRecordLengthTxtListner = new MouseAdapter() {

       	private long last;
		/**
		 * @see java.awt.event.MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
		 */
		@Override
		public void mouseReleased(MouseEvent e) {
			
			long when = e.getWhen();
			if (doListner && when != last) {
				try {
					int cpos = fileTextArea.getCaretPosition();
					int lineNum = fileTextArea.getLineOfOffset(cpos);
					
					doListner = false;
					lengthTxt.setText(Integer.toString(cpos - lineNum));
	
//			        System.out.println(" --> setRecordLengthTxtListner" );
					readFile(false);
					last = when;
				} catch (BadLocationException e1) {
					e1.printStackTrace();
				} finally {
					doListner = true;
				}
			}
		}
    };

    
//    private static RecentFiles mainRecentFile() {
//    	RecentFiles rf = RecentFiles.getMainRecentFile();
//    	return rf == null
//    			? new RecentFiles(
//    					Parameters.getApplicationDirectory() + "Files.txt",
//    					null, true, 
//    					Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getNoStar())
//    			: rf;
//    }

    /**
     * Panel1 File Details
     *
     * @param connectionId connection Identifier
     */
    public Pnl1File(AbsRowList  structureList, AbsRowList typeList) {
        super();

        init_100_setupFields(structureList, typeList);
		init_200_layoutScreen();
		init_300_listners();
    }
    
    private void init_100_setupFields(AbsRowList  structureList, AbsRowList typeList) {
		defaultType = new TreeCombo(BuildTypeComboList.getList(typeList));

		fileStructureCombo  = new BmKeyedComboBox(structureList, false);

		String formDescription = LangConversion.convertId(LangConversion.ST_MESSAGE, "FileWizard_1",
				"This wizard will build a <b>Record-Layout</b> from "
		    	+ "a sample file. <p> you need to enter a<ol>"
		    	+ "<li>sample file in the new layout</li>"
		    	+ "<li>the file structure (use <b>Default Reader</b>"
		    	+      " for standard windows / Unix files) "
		    	+      " for unknown file format use <b>Unknown Format</b></li>"
		    	+ "<li>the new layout name</li>");
		tips = new JEditorPane("text/html", formDescription);
		
		fileDisplayPane.setBorder(BorderFactory.createEmptyBorder());
		fileTextArea.setFont(SwingUtils.getMonoSpacedFont());
		fileTbl.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    }
    
    private void init_200_layoutScreen() {
		JPanel    recordTypePnl = init_210_createRecordTypePnl();
		JPanel fileStructurePnl = init_220_createFileStructurePnl();

		this.setHelpURLre(Common.formatHelpURL(Common.HELP_WIZARD));
		this.addComponentRE(
				1, 5, TIP_HEIGHT, BasePanel.GAP3, BasePanel.FULL, BasePanel.FULL,
				tips);
		this.setGapRE(BasePanel.GAP1);

		this.addLineRE(     "File Name", filenameFld);
		this.addLineRE("File Structure", fileStructurePnl)
											.setGapRE(BasePanel.GAP0)
											.setFieldsToActualSize();
		this.addLineRE(   "Record Type", recordTypePnl)
											.setHeightRE(PREFERRED)
											.setGapRE(BasePanel.GAP0);

		this.addLineRE(     "Font Name", fontNameCombo).setFieldsToFullSize();
		this.addLineRE(  "Default Type", defaultType)
											.setGapRE(BasePanel.GAP1);
		this.addComponentRE(
				1, 5, 
				FILE_DISPLAY_HEIGHT, BasePanel.GAP1,
		        BasePanel.FULL, BasePanel.FULL,
		        fileDisplayPane);
   }


	private JPanel init_210_createRecordTypePnl() {
		ButtonGroup grp = new ButtonGroup();
		JPanel recordTypePnl = new JPanel(new GridLayout(2,2));
		
		init_211_addRadioGrp(recordTypePnl, grp, fixedLengthBtn);
		init_211_addRadioGrp(recordTypePnl, grp, delimitedBtn);
		//addRadioGrp(recordTypePnl, grp, delimitedUnicodeBtn);
		init_211_addRadioGrp(recordTypePnl, grp, fixedMultiBtn);
		
		return recordTypePnl;
	}

	private void init_211_addRadioGrp(JPanel pnl, ButtonGroup grp, JRadioButton btn1) {
		grp.add(btn1);

		pnl.add(btn1);
	}



	private JPanel init_220_createFileStructurePnl() {
		JPanel fileStructurePnl = new JPanel(new BorderLayout());
		JPanel lengthPnl = new JPanel(new BorderLayout());
		
		fileStructurePnl.add(fileStructureCombo, BorderLayout.WEST);
		fileStructurePnl.add(new JPanel(), BorderLayout.CENTER);
		
		lengthPnl.add(lengthLbl, BorderLayout.WEST);
		lengthPnl.add(lengthTxt, BorderLayout.EAST);
		
		fileStructurePnl.add(lengthPnl, BorderLayout.EAST);
		
		if (this.isNameComponents()) {
			lengthTxt.setName("Record Length_Txt");
		}
		
		
		return fileStructurePnl;
	}

    private void init_300_listners() {

    	 FocusAdapter focusHandler = new FocusAdapter() {
    	    	public void focusLost(FocusEvent e) {
    	    		if (doListner) {
//    			       System.out.println(" --> focusHandler" );
    			       readFile(false);
    	    		}
    	    	}
    	    };
		fixedLengthBtn.setSelected(true);
		lengthTxt.addFocusListener(focusHandler);
		fontNameCombo.addTextChangeListner(new ChangeListener() {
			@Override public void stateChanged(ChangeEvent e) {
   	    		if (doListner) {
// 			       System.out.println(" --> Change" );
 			       readFile(false);
 	    		}
			}
		});
		//fontNameCombo.addFocusListener(focusHandler);
		
		filenameFld.addFileChangedListner(/*(IFileChanged) */this); 
		
		fileStructureCombo.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				if (doListner) {
					checkLengthFieldVisibilty();
//			           System.out.println(" --> ???" );
					readFile(false);
				}
			}
		});
    }


    private File lastFile;
    
	/**
	 *  {@link IFileChanged}}
	 *  
	 * @see net.sf.RecordEditor.utils.swing.filechooser.IFileChanged#fileChanged(java.io.File)
	 */
	@Override
	public void fileChanged(File selectedFile) {
		if (selectedFile != null 
		&& ! selectedFile.equals(lastFile)) {
			try {
				doListner = false;
				wizardDetail.fileData = StreamUtil.read(new FileInputStream(selectedFile), 512000);
				FileAnalyser fa = FileAnalyser.getAnaylser(wizardDetail.fileData, "");
				
				int fileStructure = fa.getFileStructure();
				String fontName = fa.getFontName();
				if (fileStructure >= 0) {
					fileStructureCombo.setSelectedItem(Integer.valueOf(fileStructure));
					
					if (isFixedLength(fileStructure)) {
						lengthTxt.setText(Integer.toString(fa.getRecordLength()));
						fixedLengthBtn.setSelected(true);
					} else if (Details.isTextIo(fileStructure)) {
						if (CsvAnalyser.isLikelyToBeCsv(wizardDetail.fileData, fontName)) {
							delimitedBtn.setSelected(true);
						}
					} else {
						fixedLengthBtn.setSelected(true);
					}
				}
				fontNameCombo.setText(fontName);
				fontNameCombo.setFontList(fa.getCharsetDetails().likelyCharsets);
				textPercent = fa.getTextPct();
				normalCharPercent = fa.getNormalCharPercent();
				
				checkLengthFieldVisibilty();
				
		        readFile(true);
				
				lastFile = selectedFile;
			} catch (FileNotFoundException e) {
				Common.logMsg("File: " + selectedFile.getPath() + " not found", null);
			} catch (IOException e) {
				e.printStackTrace();
			} finally {
				doListner = true;
			}
		}
	}

    private void checkLengthFieldVisibilty() {
   	
    	boolean isFixedLength = isFixedLength(getFileStructure());
		lengthLbl.setVisible(isFixedLength);
    	lengthTxt.setVisible(isFixedLength);
    	
    	if (isFixedLength) {
    		fileTbl.addMouseListener(setRecordLengthTblListner);
    		fileTextArea.addMouseListener(setRecordLengthTxtListner);
    	} else {
       		fileTbl.removeMouseListener(setRecordLengthTblListner);
    		fileTextArea.removeMouseListener(setRecordLengthTxtListner);
    	}
    }

	private boolean isFixedLength(int fileStructure) {
		boolean fixedLengthStructure = false;
    	
//		System.out.println("--> " + fileStructure + " " + IFileStructureConstants.IO_FIXED_LENGTH);
		switch (fileStructure) {
    	case IFileStructureConstants.IO_FIXED_LENGTH:
    	case IFileStructureConstants.IO_FIXED_LENGTH_CHAR:
    	case IFileStructureConstants.IO_FIXED_BYTE_ENTER_FONT:
    	case IFileStructureConstants.IO_FIXED_CHAR_ENTER_FONT:
    		fixedLengthStructure = true;
    	}
		return fixedLengthStructure;
	}

    /**
     *
     */
    public final Details getValues() throws Exception {

        wizardDetail.filename   = filenameFld.getText();
        wizardDetail.fileStructure = getFileStructure();
        wizardDetail.normalCharPercent = normalCharPercent;
        wizardDetail.textPercent = textPercent;
 //       wizardDetail.layoutName = layoutName.getText();
 //       try {
//            wizardDetail.defaultType = (Integer) defaultType.getSelectedItem();
        if (defaultType.getSelectedItem() != null) {
            wizardDetail.defaultType =  defaultType.getSelectedItem().key;
        }
//        } catch (Exception e) {
//        }
        wizardDetail.fontName   = fontNameCombo.getText();


        if ("".equals(wizardDetail.filename)) {
            filenameFld.requestFocus();
            throw new Exception("You must enter a file name");
        }

        if (wizardDetail.fileStructure == Common.IO_NAME_1ST_LINE) {
        	delimitedBtn.setSelected(true);
        }

        wizardDetail.recordType = Details.RT_FIXED_LENGTH_FIELDS;
        if (delimitedBtn.isSelected()) {
        	wizardDetail.recordType = Details.RT_DELIMITERED_FIELDS;
        } else if (fixedMultiBtn.isSelected()) {
        	wizardDetail.recordType = Details.RT_MULTIPLE_RECORDS;
        }
        
        int len;
        if (isFixedLength(wizardDetail.fileStructure)) {
	        try {
	            len = Integer.parseInt(lengthTxt.getText());
	        } catch (Exception e) {
	        	lengthTxt.requestFocus();
	            throw new Exception("Invalid Record Length: " + e.getMessage());
	        }
	
	        if (len <= 0) {
	        	lengthTxt.requestFocus();
	            throw new Exception("Invalid Record Length: " + len);
	        }
	        wizardDetail.recordLength = len;
        } else {
        	len = getLength();
            if (len >= 0) {
            	wizardDetail.recordLength = len;
            }
        }
        
        if (initFormat != wizardDetail.fileStructure 
        || (		isFixedLength(initFormat)
        		&&	initRecLength != wizardDetail.recordLength)) {
        	wizardDetail.setFieldSearch();
        }

        return wizardDetail;
    }

    /**
     * @see net.sf.RecordEditor.layoutWizard.WizardPanel#setValues(net.sf.RecordEditor.utils.fileAnalysis.LayoutWizard.Details)
     */
    public final void setValues(Details detail) {

    	wizardDetail = detail;
		initFormat = detail.fileStructure;
		initRecLength = wizardDetail.recordLength;
		normalCharPercent = wizardDetail.normalCharPercent;
		textPercent = wizardDetail.textPercent;



		doListner = false;
		filenameFld.removeFileChangedListner(this);
		
        try {
			filenameFld.setFileName(detail.filename);
			fileStructureCombo.setSelectedItem(Integer.valueOf(detail.fileStructure));

			fontNameCombo.setText(detail.fontName);
			defaultType.setSelectedKey(detail.defaultType);

			if (wizardDetail.recordLength > 0) {
				lengthTxt.setText(Integer.toString(wizardDetail.recordLength));
			}
		} finally {
			doListner = true;
			filenameFld.addFileChangedListner(this);
		}

        File f;
        if (detail.filename != null && detail.filename.length() > 0
        && ((f = new File(detail.filename)).exists() && f.isFile())) {
        	fileChanged(f);
        }
        
        checkLengthFieldVisibilty();
    }
    

    @SuppressWarnings("deprecation")
	private void readFile(boolean fixedLengthMsg) {
    	int structure = getFileStructure();
    	byte[][] lines = new byte[30][];
    	AbstractByteReader reader;
    	int count = 0;
        TableColumnModel tcm;
        TableColumn tc;
        String s;
  

    	try {
    	    switch (structure) {
    		case (IFileStructureConstants.IO_FIXED_LENGTH):
    			if ("".equals(lengthTxt.getText())) {
    				if (fixedLengthMsg) {
    					setMessageTxtRE("You Must enter a Length for fixed Length files");
    				}
    				lengthTxt.requestFocus();
    				return;
    			}
    			reader = new FixedLengthByteReader(Integer.parseInt(lengthTxt.getText()));
    			break;
    		case (IFileStructureConstants.IO_TEXT_LINE):
       		case (IFileStructureConstants.IO_UNICODE_TEXT):
       		case (IFileStructureConstants.IO_NAME_1ST_LINE):
       		case (IFileStructureConstants.IO_XML_USE_LAYOUT):
       		case (IFileStructureConstants.IO_XML_BUILD_LAYOUT):
       			reader = new TextReader(fontNameCombo.getText());
       			break;
    		default:
    			reader = ByteIOProvider.getInstance().getByteReader(structure);
    		}

    		if (reader == null) {
    			setMessageTxtRE("Selected File Structue is not supported here");
    		} else {
	    		reader.open(new ByteArrayInputStream(wizardDetail.fileData));
		    	while (count < lines.length && (lines[count++] = reader.read()) != null) {
		    		// Do nothing
		    	}
		    	reader.close();


		    	if (Details.isTextFile(normalCharPercent, structure)) {
		    		String fontName = fontNameCombo.getText();
		    		StringBuilder b = new StringBuilder(Math.min(count * 150, wizardDetail.fileData.length ));
		    		for (int i = 0; i < count; i++) {
		    			b.append(Conversion.toString(lines[i], fontName)).append("\n");
		    		}
		    		fileTextArea.setText(b.toString());
		    		fileTextArea.setCaretPosition(0);
		    		if (fileDisplay != fileTextArea) {
		    			fileDisplayPane.setViewportView(fileTextArea);
		    			//fileDisplayPane.revalidate();
		    			fileDisplay = fileTextArea;
		    		}
		    	} else {
			    	fileMdl = new LineArrayModel(lines, fontNameCombo.getText(), count - 1);
	
			    	fileTbl.setModel(fileMdl);
	
				    tcm  = fileTbl.getColumnModel();
	
			        for (int i = 0; i < fileMdl.getColumnCount(); i++) {
			            //colorInd[i] = 0;
			            tc = tcm.getColumn(i);
			        	//System.out.println(" ~~~> " + i + " " + tc.getCellRenderer());
			            tc.setPreferredWidth(SwingUtils.values().ONE_CHAR_TABLE_CELL_WIDTH);
	
			            switch ((i + 1) % COLUMNS_TO_NUMBER) {
		            	case (0): s = "" + ((i + 1) / COLUMNS_TO_NUMBER); break;
		            	case (5): s = "+";                         break;
		            	default : s = " ";
			            }
			            tc.setHeaderValue(s);
			        }
			    	fileMdl.fireTableDataChanged();
		    		if (fileDisplay != fileTbl) {
		    			fileDisplayPane.setViewportView(fileTbl);
		    			//fileDisplayPane.revalidate();
		    			fileDisplay = fileTbl;
		    		}
		    	}

	    		fileDisplayPane.repaint();
   		}
	   	} catch (Exception e) {
	   		setMessageTxtRE("Error Reading File:", e.getMessage());
			e.printStackTrace();
		}

	   	//lengthTxt.setEnabled(structure == IFileStructureConstants.IO_FIXED_LENGTH);
    }


    public int getFileStructure() {
    	return ((Integer) fileStructureCombo.getSelectedItem()).intValue();
    }

    public int getLength() {
    	int ret = 10;

    	try {
    		ret = Integer.parseInt(lengthTxt.getText());
    	} catch (Exception e) {
		}
    	return ret;
    }
}
