package net.sf.RecordEditor.layoutWizard;

import javax.swing.JComponent;

import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.fileAnalysis.Details;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;

@SuppressWarnings("serial")
public class Pnl2FileFormat extends PnlUnknownFileFormat 
implements AbstractWizardPanel<Details> {
	private Details wizardDetails;

	private int initRecLength, initFormat;
	

	/**
	 * 
	 */
	public Pnl2FileFormat() {
		super();
		
		this.setHelpURLre(Common.formatHelpURL(Common.HELP_WIZARD_FILE_STRUCTURE));
	}

	@Override
	public JComponent getComponent() {
		return this;
	}

	@Override
	public Details getValues() throws Exception {

		wizardDetails.fileStructure = super.getFileStructure();
		wizardDetails.fontName = super.fontNameCombo.getText();
		wizardDetails.textPct = textPct;

        int len;
        if (wizardDetails.fileStructure == Common.IO_FIXED_LENGTH) {
	        try {
	            len = Integer.parseInt(super.lengthTxt.getText());
	        } catch (Exception e) {
	        	super.lengthTxt.requestFocus();
	            throw new Exception("Invalid Record Length: " + e.getMessage());
	        }
	
	        if (len <= 0) {
	        	super.lengthTxt.requestFocus();
	            throw new Exception("Invalid Record Length: " + len);
	        }
	        wizardDetails.recordLength = len;
        } else {
            try {
                len = Integer.parseInt(super.lengthTxt.getText());
                if (len >= 0) {
                	wizardDetails.recordLength = len;
                }
            } catch (Exception e) {
            }
        }
        
        if (initFormat != wizardDetails.fileStructure 
        || (		initFormat == IFileStructureConstants.IO_FIXED_LENGTH
        		&&	initRecLength != wizardDetails.recordLength)) {
        	wizardDetails.setFieldSearch();
        }

		return wizardDetails;
	}

	@Override
	public void setValues(Details detail) throws Exception {
		wizardDetails = detail;
		
		super.fontNameCombo.setText(detail.fontName);

		initFormat = detail.fileStructure;
		initRecLength = wizardDetails.recordLength;
		
		if (wizardDetails.recordLength > 0) {
			super.lengthTxt.setText(Integer.toString(wizardDetails.recordLength));
		}
		
		super.open(detail.filename);
		
		if ( detail.fileStructure == IFileStructureConstants.IO_FIXED_LENGTH) {
			int structure = super.getFileStructure();
			switch (structure) {
			case (IFileStructureConstants.IO_BIN_TEXT):
			case (IFileStructureConstants.IO_TEXT_LINE):
			case (IFileStructureConstants.IO_UNICODE_TEXT):
				initFormat = IFileStructureConstants.IO_FIXED_LENGTH;
				super.structureCombo.setSelectedItem(Integer.valueOf(IFileStructureConstants.IO_FIXED_LENGTH));
			}
		}
//		
//		if (detail.fontName != null && ! "".equals(detail.fontName)) {
//			super.fontNameTxt.setText(detail.fontName);
//		}
	}

	@SuppressWarnings("deprecation")
	@Override
	public boolean skip() {

		boolean ret = true;
		
		
		switch (wizardDetails.fileStructure) {
		case IFileStructureConstants.IO_FIXED_LENGTH:
		case IFileStructureConstants.IO_UNKOWN_FORMAT:
			ret = false;
			break;
		case IFileStructureConstants.IO_DEFAULT:
			int fs = super.getFileStructure();
			switch (fs) {
			case IFileStructureConstants.IO_VB:
			case IFileStructureConstants.IO_VB_DUMP:
			case IFileStructureConstants.IO_VB_FUJITSU:
			case IFileStructureConstants.IO_VB_GNU_COBOL:
			case IFileStructureConstants.IO_MICROFOCUS:
			case IFileStructureConstants.IO_FIXED_LENGTH:
			case IFileStructureConstants.IO_VBS:
				wizardDetails.fileStructure = fs;
				ret = false;
			}
		}
		
		return ret;
	}

}
