package net.sf.RecordEditor.re.cobol;

import java.io.File;
import java.io.FileInputStream;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.event.ChangeListener;
import javax.swing.table.AbstractTableModel;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.FieldDetail;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.External.CobolCopybookLoader;
import net.sf.JRecord.External.CopybookLoader;
import net.sf.JRecord.External.CopybookLoaderFactory;
import net.sf.JRecord.External.ExternalRecord;
import net.sf.JRecord.External.Def.ExternalField;
import net.sf.JRecord.External.Item.IItemJRecUpd;
import net.sf.JRecord.External.base.FieldCreatorHelper;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.JRecord.Log.AbsSSLogger;
import net.sf.JRecord.Log.ScreenLog;
import net.sf.JRecord.Option.ICobolSplitOptions;
import net.sf.RecordEditor.re.db.Table.TableDB;
import net.sf.RecordEditor.re.db.Table.TableRec;
import net.sf.RecordEditor.re.layoutSelection.ComputerOptionCombo;
import net.sf.RecordEditor.re.layoutSelection.SplitCombo;
import net.sf.RecordEditor.re.util.CopybookLoaderFactoryDB;
import net.sf.RecordEditor.re.util.csv.ColumnDateFormatChk;
import net.sf.RecordEditor.re.util.csv.SchemaAnalyser;
import net.sf.RecordEditor.re.util.csv.StdDateFormatStr;
import net.sf.RecordEditor.utils.charsets.FontCombo;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReConnection;
import net.sf.RecordEditor.utils.common.StreamUtil;
import net.sf.RecordEditor.utils.edit.ManagerRowList;
import net.sf.RecordEditor.utils.fileAnalysis.FileAnalyser;
import net.sf.RecordEditor.utils.jdbc.DBComboModel;
import net.sf.RecordEditor.utils.lang.LangConversion;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.BmKeyedComboBox;
import net.sf.RecordEditor.utils.swing.BmKeyedComboModel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.Combo.ComboStdOption;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChanged;
import net.sf.RecordEditor.utils.swing.treeCombo.FileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.NormalCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.TreeCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.TreeComboItem;
import net.sf.RecordEditor.utils.swing2.helpers.EventDetails;
import net.sf.RecordEditor.utils.swing2.helpers.FocusActionListnerAdapter;
import net.sf.cb2xml.def.Cb2xmlConstants;
import net.sf.cb2xml.def.IItem;



public class CblLoadData implements IGetXRecord {
	
//	private final static String[] EMPTY_FIELDS = {"", "", "", "", ""};
	
	private final static int CS_NORMAL = 1;
	private final static int CS_NEW_COPYBOOK = 2;
	private final static int CS_NEW_COPYBOOK_FILE = 3;
	
	@SuppressWarnings("unchecked")
	private static ComboStdOption<Integer>[] copybookFormatOptions = new ComboStdOption[] {
		    getItem(Cb2xmlConstants.USE_STANDARD_COLUMNS, "Use Standard Cobol Columns (6-72)"),
		    getItem(Cb2xmlConstants.USE_COLS_6_TO_80, "Cobol Columns (6-80)"),
		    getItem(Cb2xmlConstants.USE_LONG_LINE, "Use Very long line"),
		    getItem(Cb2xmlConstants.FREE_FORMAT, "Free format (start column 2)"),
	};

	private TableDB         systemTable = new TableDB();
	private BmKeyedComboModel structureModel = new BmKeyedComboModel(new ManagerRowList(
			LineIOProvider.getInstance(), false));

	private DBComboModel<TableRec>    systemModel
			= new DBComboModel<TableRec>(systemTable, 0, 1, true, false);

	public final IFileSelectCombo  copybookFileCombo, sampleFileCombo; 
	public final FontCombo    fontNameCombo     = new FontCombo();
	private final ComputerOptionCombo
	                          dialectCombo      = new ComputerOptionCombo();
	public final JButton      genJRecordBtn     = SwingUtilsBldr.newButton("Generate JRecord Code");
	public final JButton      goBtn             = SwingUtilsBldr.newButton("Load Cobol");
	public final JButton      regenSelectionBtn = SwingUtilsBldr.newButton("Regenerate Selection Values");
	public final JButton      addSelectionBtn   = SwingUtilsBldr.newButton("Add Selection Values");
	public final JButton      helpBtn           = SwingUtilsBldr.getHelpButton();
	private final JComboBox   copybookFormatCombo 
	                                            = new JComboBox(copybookFormatOptions);
	private final SplitCombo  splitOptionsCombo = new SplitCombo();
	private final BmKeyedComboBox
						      fileStructureCombo;
	public final JCheckBox    lookForDatesChk   = new JCheckBox();

    public final BmKeyedComboBox      system;
    public final TreeCombo fields = new TreeCombo();
    
    public final ScreenLog msgField;// = new ScreenLog(pnl);
    public final JCheckBox dropCopybookNameChk = new JCheckBox("Drop copybookname from fieldnames");
    
    private AbstractTableModel recordMdl = null;

	//private Document document = null;
//    private List<? extends IItem> cobolItems;
//    private ICb2XmlBuilder bldr;
    private final CobolCopybookReader copybookRead = new CobolCopybookReader();

	private final CobolCopybookLoader loader; // = new Cb2xmlLoaderDB();
	private CobolCopybookLoader jRecLoader = null; //new XmlCopybookLoaderDB();
	private ExternalRecord xRecord = null;
	private final int connectionId;


	private byte[] fileBytes = null;
	private String lastFile = "";
	
	private String cobolCopybook = "";
	
	private final ChangeListener copybookParseChanged, copybookAttrChanged;
	private boolean doSplitChanged = true, doAttrChanged = true;
	private int lastDialect, lastCopybookFormat, lastSplit, lastFileStructure; 
	private String lastFont;
	
	private int defaultRow = -1;
	private int copybookChangedStatus = CS_NORMAL;
	private boolean copybookChanged = true;
	//private ArrayList<String[]> recordSelect = new ArrayList<String[]>();
	private RecordSelectionDetails recordSelection = new RecordSelectionDetails();
	
//	private ArrayList<AbstractLine>[] dataLines = null;
//	private ArrayList<AbstractLine> allLines = null;
	private LinesBuffer linesBuf;
	
	private ArrayList<NormalCombo> fieldCombos = new ArrayList<NormalCombo>();
	
	private IFileChanged copybookChangedListner;
	private boolean[] allowUpdate;
	private final ReAnalyseData reAnalyseData;

	FocusActionListnerAdapter attrChange = new FocusActionListnerAdapter() {	
		@Override public void doAction(EventDetails e) {
			checkAttrChange();			
		}
	};

	public CblLoadData(
			int connectionId, JComponent pnl, 
			ChangeListener copybookParseChanged, ChangeListener copybookAttrChanged,
	        boolean useRecordEditorDb, boolean lookForDates,
	        ReAnalyseData reAnalyseData) {
		//String dir = Common.OPTIONS.DEFAULT_COBOL_DIRECTORY.getWithStar();

		this.connectionId = connectionId;
		this.copybookParseChanged = copybookParseChanged;
		this.copybookAttrChanged = copybookAttrChanged;
		this. msgField = new ScreenLog(pnl);
		this.reAnalyseData = reAnalyseData;
		
		if (useRecordEditorDb) {
			loader = new CopybookLoaderFactoryDB.CobolCopybookLoaderDB();
		} else {
			loader = new CobolCopybookLoader();			
		}
		
		copybookFileCombo = FileSelectCombo.newCombo(Parameters.COBOL_COPYBOOK_LIST, 25, true, false, true);
		sampleFileCombo   = FileSelectCombo.newCombo(Parameters.SAMPLE_FILE_LIST, 25, true, false, true);
	    set(copybookFileCombo, Common.OPTIONS.DEFAULT_COBOL_DIRECTORY.getSlashNoStar());
	    set(sampleFileCombo,   Common.OPTIONS.DEFAULT_FILE_DIRECTORY .getSlashNoStar());

	    splitOptionsCombo.setSelectedIndex(0);

		systemTable.setConnection(new ReConnection(connectionId));

//		structureTable.setParams(Common.TI_FILE_STRUCTURE);
		systemTable.setParams(Common.TI_SYSTEMS);

		fileStructureCombo  = new BmKeyedComboBox(structureModel, false);
		system         = new BmKeyedComboBox(systemModel, false);

		String s = Common.OPTIONS.DEFAULT_IO_NAME.get();
	    if (! "".equals(s)) {
	    	fileStructureCombo.setSelectedDisplay(s);
	    }

	    s = Common.OPTIONS.DEFAULT_BIN_NAME.get();
	    if (! "".equals(s)) {
	    	dialectCombo.setEnglishText(s);
	    }

	    lookForDatesChk.setSelected(lookForDates);

//	    copybookFormatCombo.addItem(getItem(Cb2xmlConstants.USE_STANDARD_COLUMNS, "Use Standard Cobol Columns (6-72)"));
//	    copybookFormatCombo.addItem(getItem(Cb2xmlConstants.USE_COLS_6_TO_80, "Cobol Columns (6-80)"));
//	    copybookFormatCombo.addItem(getItem(Cb2xmlConstants.USE_LONG_LINE, "Use Very long line"));
//	    copybookFormatCombo.addItem(getItem(Cb2xmlConstants.FREE_FORMAT, "Free format (start column 2)"));
	    copybookFormatCombo.setSelectedIndex(0);
	    
	    dialectCombo.setSelectedIndex(1);
	    
	    lastSplit = getSplitOption();
		lastCopybookFormat = getCopybookFormat();
		
		lastFont = fontNameCombo.getText();
		lastFileStructure = getFileStructure();
		lastDialect = getDialect();
	    
		FocusActionListnerAdapter fa1 = new FocusActionListnerAdapter() {	
			@Override public void doAction(EventDetails e) {
				checkParseChange();
			}
		};
		FocusActionListnerAdapter fa2 = new FocusActionListnerAdapter() {	
			@Override public void doAction(EventDetails e) {
				checkAttrChange();		
			}
		};
		
		splitOptionsCombo.addFocusListener(fa1);
		splitOptionsCombo.addActionListener(fa1);
		copybookFormatCombo.addFocusListener(fa1);
		copybookFormatCombo.addActionListener(fa1);
		
		dialectCombo.addFocusListener(fa2);
		dialectCombo.addActionListener(fa2);
		fileStructureCombo.addFocusListener(fa2);
		fileStructureCombo.addActionListener(fa2);
		
		fontNameCombo.addTextChangeListner(fa2);;
		
		copybookFileCombo.addFileChangedListner(new IFileChanged() {
			@Override public void fileChanged(File selectedFile) {
				checkCobolCopybook(selectedFile);
				if (copybookChangedListner != null) {
					copybookChangedListner.fileChanged(selectedFile);
				}
			}
		});
	}
	
	
	private void checkCobolCopybook(File selectedFile) {
		String path = selectedFile.getPath();
		SchemaAnalyser analyser = new SchemaAnalyser(path, null);
		if (analyser.schemaType() == CopybookLoaderFactory.COBOL_LOADER) {
			int copybookFormat = analyser.cobolCopybookFormat();
			
			for (int i = 0; i < copybookFormatOptions.length; i++) {
				if (copybookFormatOptions[i].key.intValue() == copybookFormat) {
					copybookFormatCombo.setSelectedIndex(i);
					break;
				}
			}
		}
	}

	private void checkParseChange() {
		int cSplit = getSplitOption();
		int cCopybookFormat = getCopybookFormat();
		if (doSplitChanged
		&& (  lastSplit != cSplit
		  ||  lastCopybookFormat != cCopybookFormat)
		) {
			xRecord = null;
			copybookRead.clear();

			copybookChanged = true;
			copybookParseChanged.stateChanged(null);
			
			lastSplit = cSplit;
			lastCopybookFormat = cCopybookFormat;
		}
	}

	
	private void checkAttrChange() {
		int cFileStructure = getFileStructure();
		int cDialect = getDialect();
		String cFont = fontNameCombo.getText();
		if (doAttrChanged
		&& (  lastFileStructure != cFileStructure
		  ||  lastDialect != cDialect
		  ||  ! lastFont.equals(cFont))
		) {
			xRecord = null;
			copybookRead.clear();

			copybookChanged = true;
			copybookAttrChanged.stateChanged(null);
			
			lastFileStructure = cFileStructure;
			lastDialect = cDialect;
			lastFont = cFont;
		}
	}

	/**
	 * @param dir
	 */
	private void set(IFileSelectCombo fileSelect, String dir) {
		if (dir != null) {
	    	fileSelect.setFileName(dir);
	    }
	}
	
	

	private static ComboStdOption<Integer> getItem(int key, String s) {
		return new ComboStdOption<Integer>(key, s, LangConversion.convertComboItms("CobolLineFmt", s));
	}


	public final boolean setCobolCopybook(String s) {
		copybookRead.clear();
		if (s.length() == 0) {
			xRecord = null;
			copybookChanged = true;
			return true;
		} else if (! cobolCopybook.equals(s)) {
			return readCobol(s);
		}
		return false;
	}
	
	private void readCobol() {
		if ((! copybookRead.hasCobolData()) && cobolCopybook != null && cobolCopybook.length() > 0) {
			readCobol(cobolCopybook);
		}
	}

	/**
	 * @param s
	 * @return
	 */
	private boolean readCobol(String s) {
		StringReader r = new StringReader(s);
		boolean checkMultiRecords = cobolCopybook != null && (! cobolCopybook.equals(s));
		copybookChanged = true;
		xRecord = null;
		cobolCopybook = s;
		try {
			copybookRead.readCobol(r, "", getDialect(), getCopybookFormat());

			copybookChangedStatus = CS_NEW_COPYBOOK;
			
			if (checkMultiRecords) {
				checkForMultipleRecords();
			}

			return true;
		} catch (Exception e) {
			msgField.logMsg(AbsSSLogger.ERROR, "Error parsing Cobol: " + e);
			e.printStackTrace();
		}
		return false;
	}
	

	/**
	 * Check if there are multiple Records (01's or Redefines) in the copybook
	 */
	private void checkForMultipleRecords() {
		
		int split = copybookRead.checkForMultipleRecords();
		if (split >= 0) {
			setSplitInternal(CopybookLoader.SPLIT_01_LEVEL);
		}
	}
	
//	private boolean itemHasChildren(IItem item) {
//		return item.getChildItems() != null && item.getChildItems().size() > 0;
//	}
//
//
//	/**
//	 * @param node
//	 * @return
//	 */
//	private boolean hasPicture(IItem item) {
//		String picture = item.getPicture();
//		return picture == null || picture.length() == 0;
//	}
//


	/**
	 * 
	 */
	private void setSplitInternal(int split) {
		try {
		   doSplitChanged = false;
		   splitOptionsCombo.setSplitId(split);
		} finally {
		   doSplitChanged = true;
		}
	}


//	/**
//	 * @return the document
//	 */
//	public final Document getDocument() {
//		return document;
//	}

	/**
	 * @return the xRec
	 */
	@Override
	public final ExternalRecord getXRecord() {
		xRecord = getXRecord(loader, xRecord);
		return xRecord;
	}
	
	
	/**
	 * Get External record and update for Dates;
	 * @return External-Record
	 */
	public final ExternalRecord getXRecordDateCheck() {
		xRecord = getXRecord(loader, null);
		updateXRecordForDates(xRecord, true);
		return xRecord;
	}


	/**
	 * 
	 */
	protected void updateXRecordForDates(ExternalRecord xRec, boolean updateBuffer) {
		if (lookForDatesChk.isSelected() 
		&& linesBuf != null && linesBuf.getLineCount() > 5) {
//			List<? extends IItemJr> items = xRecord.getItems();
			LayoutDetail l = xRec.asLayoutDetail();
			int recordCount = l.getRecordCount();
			boolean update = false;
			FieldCreatorHelper fieldHelper =  new FieldCreatorHelper(
					ICobolSplitOptions.SPLIT_NONE, dialectCombo.getSelectedValue(), true, "", xRec.getFontName(), true);
			if (xRec.getNumberOfRecords() == 0) {
				update = processCobolItems(fieldHelper, linesBuf.getAllLines(), xRec);
			} else {
				linesBuf.setLayout(xRec.asLayoutDetail());
				int num = Math.min(recordCount, linesBuf.getRecordCount());
				for (int i = 0; i < num; i++) {
					if (linesBuf.getRecordLineCount(i) > 5) {
						update = update 
							  || processCobolItems(fieldHelper, linesBuf.getRecordLines(i), xRec.getRecord(i));
					}
				}
			}

			if (updateBuffer) {
				if (update) {
					linesBuf.setLayout(xRec.asLayoutDetail());
				}	
			} else {
				linesBuf.setLayout(xRec.asLayoutDetail());
			}
		}
	}
	
	private boolean processCobolItems(FieldCreatorHelper fieldHelper, List<AbstractLine> lines, ExternalRecord rec) {
		List<? extends IItemJRecUpd> items = rec.getUpdatebleItems();
		boolean u = processCobolItems(fieldHelper, lines, items);
		
		if (u) {
			rec.setItems(rec.getCopybookPref(), rec.getParentGroupNames(), rec.getDialectCode(), items);
		}
		
		return u;
	}
	
	private boolean processCobolItems(FieldCreatorHelper fieldHelper, List<AbstractLine> lines, List<? extends IItemJRecUpd> cobolItms) {
		boolean changed = false;
		
		for (int i = 0; i < cobolItms.size(); i++) {
			IItemJRecUpd itm = cobolItms.get(i);
			List<? extends IItemJRecUpd> childItems = itm.getChildItems();
			if (childItems != null && childItems.size() > 0) {
				processCobolItems(fieldHelper, lines, childItems);
			} else  {
				//IItemJrUpd itm = (IItemJrUpd) itemJr;
				FieldDetail fd = new FieldDetail(itm.getFieldName(), "", itm.getType(),
						fieldHelper.calculateDecimalSize(itm.getType(), itm.getPicture(), itm.getScale()),
						fieldHelper.getFont(), itm.getFormatId(), itm.getParameter());
				fd.setPosLen(fix(itm.getPosition()), fix(itm.getStorageLength()));
				
				ColumnDateFormatChk dateChk = new ColumnDateFormatChk();
				String s;
				Object o;
				for (int ii = 0; ii < lines.size(); ii++) {
					o = lines.get(ii).getField(fd);
					if (o != null && (s = o.toString()).length() > 0) {
						dateChk.add(s);
					}
				}
				
				List<StdDateFormatStr> possibleDateFormats = dateChk.getPossibleDateFormats();
				if (possibleDateFormats != null && possibleDateFormats.size() > 0) {
					StdDateFormatStr dateFormatDtls = possibleDateFormats.get(0);
					itm.setFormat(dateFormatDtls.fieldFormat, dateFormatDtls.dateFormatStr);
					changed = true;
				}
			}
		}
		return changed;
	}

	
	private int fix(int val) {
		return val == IItem.NULL_INT_VALUE ? 0 : val;
	}

	/**
	 * @return the xRec
	 */
	public final ExternalRecord getXRecordJR(boolean dropCopybookFromName) {
		if (jRecLoader == null) {
			jRecLoader = new CobolCopybookLoader(true);
			jRecLoader.setDropCopybookFromFieldNames(dropCopybookFromName);
		}
		copybookRead.clear();
		ExternalRecord ret = getXRecord(jRecLoader, null);
		copybookRead.clear();
		
		Parameters.setProperty(Parameters.USES_COBOL, "Y");

		updateXRecordForDates(ret, false);
		
		return ret;
	}
	
	public String getCopybookName() {
		return copybookFileCombo.getText();
	}
	
	/**
	 * @return the xRec
	 */
	private final ExternalRecord getXRecord(CobolCopybookLoader cobolLoader, ExternalRecord xRec ) {
		readCobol();

		int systemId = 0;
		Object sys;
		if (system != null && (sys = system.getSelectedItem()) != null && sys instanceof Integer) {
			systemId = ((Integer) sys).intValue();
		}
		String s = "";
		try {
			String copybookFile =  copybookFileCombo.getText();
			File f = new File(copybookFile);
			if (f.exists()) {
				s = Conversion.getCopyBookId(copybookFile);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		xRec = copybookRead.getXRecord(
				cobolLoader, xRec, connectionId, 
				splitOptionsCombo.getSelectedValue(), systemId, 
				getFileStructure(), dialectCombo.getSelectedValue(), defaultRow,  
				fontNameCombo.getText(), s, recordSelection);
		
		if (xRec != null
		&& (allowUpdate == null || allowUpdate.length != xRec.getNumberOfRecords())
		&& xRec.getNumberOfRecords() > 1) {
			allowUpdate = new boolean[xRec.getNumberOfRecords()];
			Arrays.fill(allowUpdate, true);
		}

		Parameters.setProperty(Parameters.USES_COBOL, "Y");

		return xRec;
	}
//	
//	private int numberOfCriteria(int idx) {
//		int num = 0;
//		for (int i = 1; i < EMPTY_FIELDS.length; i++) {
//			if (getRecordSelection(idx, 1).length() > 0) {
//				num += 1;
//			}
//		}
//		
//		return num;
//	}

	public void checkDataFile() {
		byte[] b = getFileBytes();
		if (b == null || b.length < 10) {return;}
		
		FileAnalyser analyser =  FileAnalyser.getAnaylserNoLengthCheck(b, "40");
		String font = analyser.getFontName();
		
		try {
			doAttrChanged = false;
		
			if (Conversion.isMultiByte(font)) {
				font = Conversion.getDefaultSingleByteCharacterset();
			}
			fontNameCombo.setTextSilently(font);
			
			fontNameCombo.setFontList(analyser.getCharsetDetails().getLikelyCharsets(Conversion.getDefaultSingleByteCharacterset(), 10));
			
			
			int fileStructure = analyser.getFileStructure();
			
			fileStructureCombo.removeActionListener(attrChange);
			try {
				fileStructureCombo.setSelectedItem(fileStructure);
			} finally {
				fileStructureCombo.addActionListener(attrChange);
			}
			xRecord = null;
		} finally {
			doAttrChanged = true;
		}
	}
	
	
	/**
	 * Get the all the possible record-type values
	 * @return possible record-type values
	 */
	public final String[] getRecordTypeValues() {
		CblMultiRecAnalysis mrAnalysis = getMultiRecordAnalysis();
		return mrAnalysis == null ? null : mrAnalysis.getRecordTypeValues();
//		ExternalRecord xRecord = this.getXRecord();
//		byte[] bytes = this.getFileBytes();
//		
//		if (xRecord == null || xRecord.getNumberOfRecords() < 2 || bytes == null) {return null;}
//		
//		LayoutDetail schema = xRecord.asLayoutDetail();
//		//IFieldDetail fld = schema.getFieldFromName(fldName);
//		ArrayList<IFieldDetail> flds = getRecordSelectionFields(xRecord, schema);
////				recordSelection
//		
//		if (flds.size() == 0) {return null;}
//		
//		TreeSet<String> vals = new TreeSet<String>();
//		AbstractLineReader r = LineIOProvider.getInstance().getLineReader(schema);
//		AbstractLine l;
//		String s;
//		
//		try {
//			r.open(new ByteArrayInputStream(bytes), schema);
//			while ((l = r.read()) != null) {
//				for (int fldIdx = 0; fldIdx < flds.size(); fldIdx++) {
//					s = l.getFieldValue(flds.get(fldIdx)).asString();
//					if (s != null && s.length() > 0 && ! vals.contains(s)) {
//						vals.add(s);
//					}
//				}
//			}
//			r.close();
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//		
//		
////		TreeComboItem[] tcValues;// = TreeComboItem.toTreeItemArray(vals.toArray(new String[vals.size()]));
//		return vals.toArray(new String[vals.size()]);
	}


	@Override
	public CblMultiRecAnalysis getMultiRecordAnalysis() {
		ExternalRecord xRecord = this.getXRecord();
		byte[] bytes = this.getFileBytes();
		
		if (xRecord == null || xRecord.getNumberOfRecords() < 2 || bytes == null) {return null;}
		
		return new CblMultiRecAnalysis(this.fields.getText(), recordSelection, xRecord, bytes);
	}
	
	/**
	 * @return the fileBytes
	 */
	public final byte[] getFileBytes() {
		String samplefile = sampleFileCombo.getText();
		if (fileBytes == null || ! lastFile.equals(samplefile)) {
			fileBytes = null;
			try {
				File file = new File(samplefile);
				if (file.exists() && ! file.isDirectory()) {
					fileBytes = StreamUtil.read(new FileInputStream(file), 0x100000);
					if (copybookChangedStatus == CS_NEW_COPYBOOK) {
						copybookChangedStatus = CS_NEW_COPYBOOK_FILE;
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
					
			lastFile = samplefile;
			copybookChanged = true;
		}
		return fileBytes;
	}


	/**
	 * @param dataLines the dataLines to set
	 */
	public final void setDataLines(LinesBuffer linesBuf,
			boolean update, boolean updateDone) {
		this.linesBuf = linesBuf;
		if (update) {
			updateTestValues(updateDone);
			if (copybookChangedStatus == CS_NEW_COPYBOOK_FILE) {
				copybookChangedStatus = CS_NORMAL;
			}
		}
	}
	
	public final void updateTestValues(boolean updateDone) {
		if (updateDone) {
			if (recordMdl != null) {
				recordMdl.fireTableDataChanged();
			}
		} else if (linesBuf!= null && linesBuf.getLineCount() > 1
		&& recordSelection != null && recordSelection.size() == linesBuf.getRecordCount()) {
			boolean found = false;
			
			for (int i = linesBuf.getRecordCount() - 1; i >= 0; i--) {
				String fieldName = recordSelection.getFieldName(i);
				if (linesBuf.getRecordLineCount(i) > 0 
				&& fieldName.length() > 0 
				&& recordSelection.getFieldValue(i, 0).length() == 0
				&& (found || i > 0)) {
					setFieldValue(i, 0, linesBuf.getRecordLine(i, 0).getFieldValue(fieldName).asString(), false);
					found = true;
				}
			}
			
			if (found && recordMdl != null) {
				recordMdl.fireTableDataChanged();
			}
		}
	}

	/**
	 * @return the defaultRow
	 */
	public final int getDefaultRow() {
		return defaultRow;
	}

	/**
	 * @param defaultRow the defaultRow to set
	 */
	public final void setDefaultRow(int defaultRow) {
		
		if ((this.defaultRow != defaultRow)) {
			copybookChanged = true;
			this.defaultRow = defaultRow;
			checkIfAllSelectionsDefined();
		}
	}

	
	public void initRecordSelection(ExternalRecord xRec) {
		
		ArrayList<TreeComboItem> items = getFieldCounts();
		
		fields.setText("");
		
		recordSelection.initRecordSelection(xRec);
		
		initialiseFieldCombos(xRec.getNumberOfRecords());
		
		if (items.size() > 0) {
			String selectionFieldName = items.get(0).getEnglish();

			fields.setText(selectionFieldName);
			fields.setTree(items.toArray(new TreeComboItem[items.size()]));
			
			ExternalField selField = CobolCopybookAnalyser.getSelectionField(xRec, recordSelection, selectionFieldName);		
			
			if (selField != null) {
				updateTestValues(false);
			}
		}

	}

	private ArrayList<TreeComboItem> getFieldCounts() {
		HashMap<String, Integer> fldCount = new HashMap<String, Integer>(400);
		ArrayList<TreeComboItem> items = new ArrayList<TreeComboItem>(20);
		
		int firstField = 0;
		if (xRecord != null && xRecord.getNumberOfRecords() > 0 && xRecord.getRecord(0).getNumberOfRecordFields() > 0
		&& xRecord.getRecord(0).getRecordField(0).getLen() < 4) {
			firstField = 1;
			
			String name = xRecord.getRecord(0).getRecordField(0).getName();
			items.add(new TreeComboItem(items.size(), name, name));
		}
		
		for (int i = 0; i < xRecord.getNumberOfRecords(); i++) {
			HashSet<String> used = new HashSet<String>(50);
			ExternalRecord xr = xRecord.getRecord(i);
			for (int j = firstField; j < xr.getNumberOfRecordFields(); j++) {
				ExternalField fld = xr.getRecordField(j);
				String key = fld.getName().trim().toLowerCase();
				if (key.length() > 0 &&(!"filler".equals(key)) && (! used.contains(key))) {
					Integer num = fldCount.get(key);
					
					fldCount.put(key, num==null?1:num+1);
					used.add(key);
				}
			}
		}
		
		Set<Entry<String, Integer>> entrySet = fldCount.entrySet();
		for (Entry<String, Integer> e : entrySet) {
			if (e.getValue().intValue() > 1) {
				items.add(new TreeComboItem(items.size(), e.getKey(), e.getKey()));
			}
		}
		return items;
	}

	private void initialiseFieldCombos(int size) {
		copybookChanged = true;
		fieldCombos.clear();
		fieldCombos.ensureCapacity(size);
		for (int i = 0; i < size; i++) {
			fieldCombos.add(null);
		}
	}
	
//	public String getRecordSelection(int recordIdx, int fieldIdx) {
//		if (recordIdx >= recordSelect.size()) {return "";}
//		return recordSelect.get(recordIdx)[fieldIdx];
//	}
//	
//	
//	private void setRecordSelection(int recordIdx, int fieldIdx, String value, boolean check) {
//		
//		if (recordSelection.setRecordSelection(recordIdx, fieldIdx, value)) {
//			notifyOfSelectionUpdate(check);
//			
//		}
//	}
	
//	public void addRecordSelection(int recordIdx, String value, boolean check) {
//		initRecordSelect(recordIdx);
//
//		String[] currValues = recordSelect.get(recordIdx);
//		for (int i = 1; i < currValues.length; i++) {
//			if ( currValues[i].equalsIgnoreCase(value)) {
//				return;
//			}
//		}
//		for (int i = 1; i < currValues.length; i++) {
//			if ( currValues[i].length() == 0) {
//				currValues[i] = value;
//				break;
//			}
//		}
//	}

//	/**
//	 * @param recordIdx
//	 */
//	private void initRecordSelect(int recordIdx) {
//		while (recordIdx >= recordSelect.size()) {
//			recordSelect.add(EMPTY_FIELDS.clone());
//		}
//	}

	
	public void setFieldName(int idx, String name, boolean check) {
		
		
		if (recordSelection.setFieldName(idx, name)) {
			int numberOfRecords = xRecord.getNumberOfRecords();
			if (name != null && name.length() > 0 && numberOfRecords > 1) {
				ExternalField fld = getField(xRecord.getRecord(idx), name);
//				System.out.println(" --->\t" + fld.getName() + " " + fld.getPos() + " " + fld.getLen());

				if (fld != null) {
					for (int recIdx = 0; recIdx < numberOfRecords; recIdx++) {
						String fieldName = recordSelection.getFieldName(idx);
						if (recIdx != idx 
						&& (fieldName == null || fieldName.length() == 0 || (allowUpdate != null && allowUpdate.length > idx && allowUpdate[recIdx])) ) {
							ExternalRecord rec = xRecord.getRecord(recIdx);
							for (int i = 0; i < rec.getNumberOfRecordFields(); i++) {
								ExternalField field = rec.getRecordField(i);
								if (fld.getPos() == field.getPos() && fld.getLen() == field.getLen()) {
									String fldName = field.getName();
									if (fldName.length() > 0 && ! "filler".equalsIgnoreCase(fldName)) {
										recordSelection.setFieldName(recIdx, fldName);
//										System.out.println(" ~~~ " + fldName);
									}
									break;
//								} else {
//									System.out.print("\t" + field.getName() + " " + field.getPos() + " " + field.getLen());
								}
							}
//						} else {
//							System.out.println(" &&& " + recIdx + " "  + allowUpdate[recIdx]);
						}
					}
					if (reAnalyseData != null) {
						reAnalyseData.reanalyse();
					}
					recordMdl.fireTableDataChanged();
				}
			}
			if(allowUpdate != null && allowUpdate.length > idx) {
				allowUpdate[idx] = false;
			}
			notifyOfSelectionUpdate(check);
		}
	}
	
	private ExternalField getField(ExternalRecord rec, String name) {
		for (int i = 0; i < rec.getNumberOfRecordFields(); i++) {
			if (name.equalsIgnoreCase(rec.getRecordField(i).getName())) {
				return rec.getRecordField(i);
			}
		}
		return null;
	}

	public void setFieldValue(int idx, int fieldNumber, String value, boolean check) {
		if (recordSelection.setFieldValue(idx, fieldNumber, value)) {
			notifyOfSelectionUpdate(check);
		}
	}


	private void notifyOfSelectionUpdate(boolean check) {
		copybookChanged = true;
		xRecord = null;
		if (check) {
			checkIfAllSelectionsDefined();
		}
	}

	public String getFieldName(int idx) {
		return recordSelection.getFieldName(idx);
	}

	public String getFieldValue(int idx, int number) {
		return recordSelection.getFieldValue(idx, number);
	}

	public void addRecordSelection(int recordIdx, String value, boolean check) {
		recordSelection.addRecordSelection(recordIdx, value, check);
	}

	public RecordSelectionDetails getRecordSelection() {
		return recordSelection;
	}

	public void checkIfAllSelectionsDefined() {
		
		if (doAttrChanged && recordSelection.areAllSelectionsDefined(xRecord, defaultRow)) {
			copybookAttrChanged.stateChanged(null);
		}
//		if (xRecord != null && xRecord.getNumberOfRecords() > 1 
//		&& (recordSelect.size() >= xRecord.getNumberOfRecords()
//		   || (recordSelect.size() == xRecord.getNumberOfRecords()-1)
//		     && defaultRow == recordSelect.size())) {
//			
//			for (int i = 0; i < recordSelect.size(); i++) {
//				if (defaultRow == i || recordSelect.get(i)[0].length() > 0) {
//					
//				} else {
//					return;
//				}
//			}
//			copybookAttrChanged.stateChanged(null);
//		}
	}
	/**
	 * @return the fieldCombos
	 */
	public final NormalCombo getFieldCombo(int recordIdx) {
		if (xRecord == null || recordIdx > xRecord.getNumberOfRecords()) { return null;}
		
		for (int i = fieldCombos.size(); i <= recordIdx; i++) {
			fieldCombos.add(null);
		}
		if (fieldCombos.get(recordIdx) == null) {
			ExternalRecord r = xRecord.getRecord(recordIdx);
			String dfltField = fields.getText();
			ArrayList<String> fl = new ArrayList<String>(r.getNumberOfRecordFields()+1);
			
			if (!"".equals(dfltField)) {
				fl.add(dfltField);
			}
			for (int i = 0; i < r.getNumberOfRecordFields(); i++) {
				ExternalField fld = r.getRecordField(i);
				if (! dfltField.equalsIgnoreCase(fld.getName())) {
					fl.add(fld.getName());
				}
			}
			fieldCombos.set(recordIdx, new NormalCombo(fl.toArray(new String[fl.size()])));
		}
		return fieldCombos.get(recordIdx);
	}

	/**
	 * @return the copybookChanged
	 */
	public final boolean isCopybookChanged() {
		return copybookChanged;
	}

	public final boolean isAnalysisRequired() {
		return copybookChangedStatus == CS_NEW_COPYBOOK_FILE;
	}
	
	/**
	 * @param copybookChanged the copybookChanged to set
	 */
	public final void setCopybookChanged(boolean copybookChanged) {
		this.copybookChanged = copybookChanged;
	}

	public final int getDialect() {
		return  dialectCombo.getSelectedValue();
	}

	public final int getSplitOption() {
		return  splitOptionsCombo.getSelectedValue();
	}

	public final int getFileStructure() {
		return ((Integer) this.fileStructureCombo.getSelectedItem()).intValue();
	}
	
	public final int getCopybookFormat() {
		return  ((ComboStdOption<Integer>) this.copybookFormatCombo.getSelectedItem()).key;
	}
	
	/**
	 * @return the binaryOptions
	 */
	public final JComboBox getDialectCombo() {
		return dialectCombo;
	}


	/**
	 * @return the copybookFormatCombo
	 */
	public final JComboBox getCopybookFormatCombo() {
		return copybookFormatCombo;
	}


	/**
	 * @return the splitOptions
	 */
	public final JComboBox getSplitOptionsCombo() {
		return splitOptionsCombo;
	}


	/**
	 * @return the fileStructure
	 */
	public final BmKeyedComboBox getFileStructureCombo() {
		return fileStructureCombo;
	}

	/**
	 * @param recordMdl the recordMdl to set
	 */
	public final void setRecordMdl(AbstractTableModel recordMdl) {
		this.recordMdl = recordMdl;
	}

	public void setCopybookChangedListner(IFileChanged copybookChangedListner) {
		this.copybookChangedListner = copybookChangedListner;
	}

	public static interface ReAnalyseData {
		public void reanalyse();
	}
}
