package net.sf.RecordEditor.re.cobol;

import java.io.ByteArrayInputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;

import net.sf.JRecord.Common.IFieldDetail;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.External.ExternalRecord;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.JRecord.IO.LineIOProvider;

public class CblMultiRecAnalysis {

//	private final String fldName;
	private final RecordSelectionDetails recordSelection;

	private final ExternalRecord xRecord;
	private final LayoutDetail schema;
	private final byte[] bytes;
	
	
	private final ArrayList<IFieldDetail> selectionFields;
	
	public CblMultiRecAnalysis(String fldName, RecordSelectionDetails recordSelection, ExternalRecord xRecord,
			byte[] bytes) {
		super();
		this.recordSelection = recordSelection;
		this.xRecord = xRecord;
		this.bytes = bytes;
		
		schema = xRecord.asLayoutDetail();
		//IFieldDetail fld = schema.getFieldFromName(fldName);
		selectionFields = getRecordSelectionFields(xRecord, schema, fldName);
	}
	/**
	 * @param xRecord
	 * @param schema
	 * @return
	 */
	private ArrayList<IFieldDetail> getRecordSelectionFields(ExternalRecord xRecord, LayoutDetail schema, String fldName) {
//		String fldName = this.fields.getText();
		ArrayList<IFieldDetail> flds = new ArrayList<IFieldDetail>();
		boolean includeDflt = false;
		
		for (int i = 0;i < xRecord.getNumberOfRecords(); i++) {
			String fieldName = recordSelection.getFieldName(i);
			if (fieldName == null ||fieldName.length() == 0) {
				includeDflt = true;
			}
			addField(flds, schema, fieldName);
		}
		
		if (includeDflt) {
			addField(flds, schema, fldName);
		}
		return flds;
	}
	
	/**
	 * @param flds
	 * @param schema
	 * @param fieldName
	 */
	private void addField(ArrayList<IFieldDetail> flds, LayoutDetail schema, String fieldName) {
		if (fieldName != null && fieldName.length() > 0) {
			IFieldDetail fld = schema.getFieldFromName(fieldName);
			if (fld != null) {
				boolean searching = true;
				for (int fldIdx = 0; fldIdx < flds.size(); fldIdx++) {
					IFieldDetail fldDetail = flds.get(fldIdx);
					if (fld.getPos() == fldDetail.getPos()
					&& fld.getLen() == fldDetail.getLen()
					&& fld.getType() == fldDetail.getType()) {
						searching = false;
						break;
					}
				}
				if (searching) {
					flds.add(fld);
				}
			}
		}
	}
	
	public final String[] getRecordTypeValues() {
		if (selectionFields.size() == 0) {return null;}
		TreeSet<String> vals = new TreeSet<String>();
		
		List<AbstractLine> lines = getLines();
		String s;
		
		for (int i = 0; i < lines.size(); i++) {
			for (int fldIdx = 0; fldIdx < selectionFields.size(); fldIdx++) {
				s = lines.get(i).getFieldValue(selectionFields.get(fldIdx)).asString();
				if (s != null && s.length() > 0 && ! vals.contains(s)) {
					vals.add(s);
				}
			}

		}
		return vals.toArray(new String[vals.size()]);
	}
	
	public List<AbstractLine> getLines() {
		AbstractLineReader r = LineIOProvider.getInstance().getLineReader(schema);
		AbstractLine l;
		ArrayList<AbstractLine> lines = new ArrayList<AbstractLine>();
		try {
			r.open(new ByteArrayInputStream(bytes), schema);
			while ((l = r.read()) != null) {
				lines.add(l);
			}
			r.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return lines;
	}

	public final ArrayList<IFieldDetail> getSelectionFields() {
		return selectionFields;
	}
	public final LayoutDetail getSchema() {
		return schema;
	}

}
