package net.sf.RecordEditor.re.cobol;

import javax.swing.JComboBox;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.External.Def.BasicConversion;
import net.sf.RecordEditor.utils.swing.Combo.ComboStdOption;

public class CobolFileStructureComboContainer {
	@SuppressWarnings("deprecation")
	private static final int[] COBOL_FILE_STRUCTURES = {
		IFileStructureConstants.IO_BIN_TEXT,  
		IFileStructureConstants.IO_UNICODE_TEXT,         	
		IFileStructureConstants.IO_FIXED_LENGTH,         	
		IFileStructureConstants.IO_FIXED_LENGTH_CHAR,    	
		IFileStructureConstants.IO_BINARY_IBM_4680,      	
		IFileStructureConstants.IO_VB,                   	
		IFileStructureConstants.IO_VB_DUMP,              	
		IFileStructureConstants.IO_VB_FUJITSU,           	
		IFileStructureConstants.IO_VB_GNU_COBOL,         	
/*		IFileStructureConstants.IO_FIXED_BYTE_ENTER_FONT,	
		IFileStructureConstants.IO_FIXED_CHAR_ENTER_FONT,	
		IFileStructureConstants.IO_TEXT_BYTE_ENTER_FONT, 	
		IFileStructureConstants.IO_TEXT_CHAR_ENTER_FONT, 	
		IFileStructureConstants.IO_CONTINOUS_NO_LINE_MARKER,*/
		IFileStructureConstants.IO_VBS, 
		IFileStructureConstants.IO_VB_DUMP2,              	
	};
	
	@SuppressWarnings("unchecked")
	private static final ComboStdOption<Integer>[] CBL_FILE_STRUCTURES = new ComboStdOption[COBOL_FILE_STRUCTURES.length];
	
	static {
		String lineSequentialByte = "Line Sequential (byte)";
		String lineSequentialChar = "Line Sequential (Unicode)";
		CBL_FILE_STRUCTURES[0] = new ComboStdOption<Integer>(IFileStructureConstants.IO_BIN_TEXT, lineSequentialByte, lineSequentialByte);
		CBL_FILE_STRUCTURES[1] = new ComboStdOption<Integer>(IFileStructureConstants.IO_BIN_TEXT, lineSequentialChar, lineSequentialChar);
		
		for (int i = 2; i < COBOL_FILE_STRUCTURES.length; i++) {
			String s = BasicConversion.getStructureName(COBOL_FILE_STRUCTURES[i]);
			
			StringBuilder b = s.endsWith("_VB")
					? Conversion.replace(s, "_VB", " Variable Length Record")
					: new StringBuilder(s);
			Conversion.replace(b, "_", " ");
			CBL_FILE_STRUCTURES[i] = new ComboStdOption<Integer>(COBOL_FILE_STRUCTURES[i],  b.toString(), s);
		}
	}
	
	public JComboBox cobolFileStrucures = new JComboBox(CBL_FILE_STRUCTURES);
	
	public void setFileStructure(int fileStructure) {
		for (int i = 0; i < COBOL_FILE_STRUCTURES.length; i++) {
			if (COBOL_FILE_STRUCTURES[i] == fileStructure) {
				cobolFileStrucures.setSelectedIndex(i);
				return;
			}
		}
	}
	
	public int getFileStructure() {
		int idx = cobolFileStrucures.getSelectedIndex();
		
		return 	idx >= 0 
			? COBOL_FILE_STRUCTURES[idx]  : IFileStructureConstants.IO_BIN_TEXT;
	}
}
