package net.sf.RecordEditor.re.cobol;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import net.sf.JRecord.Common.IFieldDetail;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.Details.RecordDetail;
import net.sf.JRecord.Details.RecordDetail.FieldDetails;
import net.sf.JRecord.Types.TypeManager;
import net.sf.RecordEditor.utils.fileAnalysis.Details;
import net.sf.RecordEditor.utils.fileAnalysis.FieldSearch;
import net.sf.RecordEditor.utils.fileAnalysis.RecordDefinition;

public class LinesBuffer {
	
	List<AbstractLine> EMPTY_LIST = Collections.emptyList();

	private List<AbstractLine> fileLines = EMPTY_LIST;
	private List<AbstractLine>[] recordLines;
	boolean toLoad = true;

	/**
	 * @param fileLines the fileLines to set
	 */
	public synchronized final void setFileLines(ArrayList<AbstractLine> fileLines) {
		this.fileLines = fileLines == null ? EMPTY_LIST : fileLines;
		this.toLoad = true;
	}
	
	public synchronized final void setLayout(LayoutDetail layout) {
		for (int i = fileLines.size() - 1; i >= 0; i--) {
			fileLines.get(i).setLayout(layout);
		}
		this.toLoad = true;
	}
	
	public int getLineCount() {
		return fileLines.size();
	}
	
	public AbstractLine getLine(int lineNum) {
		return fileLines.get(lineNum);
	}

	public int getRecordCount() {
		return fileLines == null || fileLines.size() == 0 ? 0 : fileLines.get(0).getLayout().getRecordCount();
	}
	
	
	public int getRecordLineCount(int recordIdx) {
		return getRecordLines(recordIdx).size();
	}
	
	public AbstractLine getRecordLine(int recordIdx, int lineIdx) {
		return getRecordLines(recordIdx).get(lineIdx);
	}
	
	public synchronized List<AbstractLine> getAllLines() {
		return fileLines;
	}

	
	@SuppressWarnings("unchecked")
	public synchronized List<AbstractLine> getRecordLines(int recordIdx) {
		if (toLoad && fileLines != null && fileLines.size() > 0) {
			AbstractLayoutDetails layout = fileLines.get(0).getLayout();
			if (layout.getRecordCount() == 1) {
				recordLines = new ArrayList[1];
				recordLines[0] = fileLines;
			} else {
				if (recordLines == null || recordLines.length != layout.getRecordCount()) {
					recordLines = new ArrayList[layout.getRecordCount()];
					for (int i = 0; i < recordLines.length; i++) {
						recordLines[i] = new ArrayList<AbstractLine>();
					}
				} else {
					for (int i = 0; i < recordLines.length; i++) {
						recordLines[i].clear();
					}
				}
				for (int i = 0; i < fileLines.size(); i++) {
					AbstractLine line = fileLines.get(i);
					
					recordLines[Math.max(0, line.getPreferredLayoutIdx())].add(line);
				}
			}
			toLoad = false;
		}
		
		return recordLines == null || recordIdx < 0 || recordIdx >= recordLines.length
				? EMPTY_LIST 
				: recordLines[recordIdx];
	}
	
	public void doLineAnalysis(
			IContinueCheck continueCheck, 
			IGetXRecord dtls) {
		CblMultiRecAnalysis mrAnalysis = dtls.getMultiRecordAnalysis();
		LayoutDetail schema;
		if (mrAnalysis == null || (schema = mrAnalysis.getSchema()) == null) { return; }
		RecordSelectionDetails recSel = dtls.getRecordSelection();
		ArrayList<IFieldDetail> selectionFields = mrAnalysis.getSelectionFields();
		IFieldDetail keyFld = selectionFields == null || selectionFields.size() == 0 ? null : selectionFields.get(0);
		int recordCount = schema.getRecordCount();
		
//		// Find the Record-Type fields
//		for (int i = 0; i < recordCount; i++) {
//			keyFieldName = recSel.getFieldName(i);
//			if (keyFieldName.length() > 0 
//			&& (keyFld = schema.getFieldFromName(keyFieldName)) != null) {
//				break;
//			}
//		}
		if (keyFld != null) {
			HashMap<String, ArrayList<AbstractLine>> linesByKey = new HashMap<String, ArrayList<AbstractLine>>(recordCount * 3 + 10);
			int intialCount = Math.max(10, fileLines.size() / recordCount);
			String key;
			
			// Group the lines by record type
			for (AbstractLine l : fileLines) {
				if (continueCheck != null && ! continueCheck.isOkToContinue()) { return;}
				key = l.getFieldValue(keyFld).asString();
				if (key != null) {
					key = key.toLowerCase();
					ArrayList<AbstractLine> list = linesByKey.get(key);
					if (list == null) {
						list = new ArrayList<AbstractLine>(intialCount);
						linesByKey.put(key, list);
					}
					list.add(l);
				}
			}
			Set<Entry<String, ArrayList<AbstractLine>>> keyListSet = linesByKey.entrySet();
			
			// For each record-type group, try and work out the fields 
			// and compare with the fields in the copybook.
			for (Entry<String, ArrayList<AbstractLine>> e : keyListSet) {
				ArrayList<AbstractLine> matchingLines = e.getValue();
				int numRecords = matchingLines.size();
				if (numRecords > 4) {
					Details details = new Details();
					RecordDefinition recordDefinition = details.standardRecord;
					details.fileStructure = schema.getFileStructure();
					details.fontName = schema.getFontName();
					
					recordDefinition.columnDtls.clear();
					
					if (recordDefinition.records.length < numRecords) {
						recordDefinition.records = new byte[numRecords][];
					}
					recordDefinition.numRecords = numRecords;
					
					for (int i = 0; i < numRecords; i++) { 
						recordDefinition.records[i] = matchingLines.get(i).getData();
					}
					
					FieldSearch fieldSearch = new FieldSearch(details, recordDefinition);
					
					fieldSearch.findFields(true, false, true, true, false);
					
					int currRec = -1, currMatch = -1;
					for (int j = 0; j < recordCount; j++) {
						if (continueCheck != null && ! continueCheck.isOkToContinue()) {return ;}
						RecordDetail record = schema.getRecord(j);
						int i1 =0, i2 = 0, match = 0, matchType = 0;
						while (i1 < record.getFieldCount() && i2 < recordDefinition.columnDtls.size()) {
							FieldDetails field = record.getField(i1);
							if (field.getPos() < recordDefinition.columnDtls.get(i2).getStart()) {
								i1 += 1;
							} else if (field.getPos() > recordDefinition.columnDtls.get(i2).getStart()) {
								i2 += 1;
							} else {
								match += 1;
								if (field.getLen() == recordDefinition.columnDtls.get(i2).getLength()) {
									match += Math.min(4, (field.getLen() + 2 ) / 3);											
								}
				
								int type = field.getType();
								if (type == recordDefinition.columnDtls.get(i2).getType()) {
									matchType +=3;
									if (TypeManager.isNumeric(type)) {
										matchType +=8;
										if (TypeManager.isBinary(type )) {
											matchType +=10;
										}
									}
								}
								i1 += 1;
								i2 += 1;
							}
						}
						
//						System.out.println(" +++ " + e.getKey() + " ~ " + j + " " + schema.getRecord(j).getRecordName()
//								+ " " + (match * 2 + matchType) + " " + currMatch);
						if (match * 2 + matchType > currMatch) {
							currRec = j;
							currMatch = match * 2 + matchType;
						}
					}
					//System.out.println(" >>> " + recordCount);
					recSel.addRecordSelection(currRec,  matchingLines.get(0).getFieldValue(keyFld).asString(), false);
				}
			}
			
			LayoutDetail newSchema = dtls.getXRecord().asLayoutDetail();
			for (AbstractLine l : fileLines) {
				if (continueCheck != null && ! continueCheck.isOkToContinue()) {return ;}
				l.setLayout(newSchema);
			}
		}
	}

}
