package net.sf.RecordEditor.re.cobol.tree;

import java.util.ArrayList;
import java.util.List;

import javax.swing.JTree;
import javax.swing.table.TableCellEditor;
import javax.swing.tree.TreePath;

import net.sf.JRecord.External.Item.IItemJRecUpd;
import net.sf.JRecord.Types.TypeManager;
import net.sf.JRecord.cg.schema.type.JavaTypeDetails;
import net.sf.RecordEditor.re.codeGen.type.AbstractTypeCellAdapter;
import net.sf.RecordEditor.re.codeGen.type.CheckForTypeUpdate;
import net.sf.RecordEditor.re.codeGen.type.DateComboFactory;
import net.sf.RecordEditor.re.codeGen.type.IDateComboBuilder;
import net.sf.RecordEditor.re.util.csv.StdDateFormatStr;
import net.sf.RecordEditor.utils.swing.interfaces.ITableCellAdapter;
import net.sf.RecordEditor.utils.swing.treeTable.AbstractTreeTableModel;
import net.sf.RecordEditor.utils.swing.treeTable.JTreeTable;
import net.sf.RecordEditor.utils.swing.treeTable.TreeTableModel;

public class CobolItemTreeModel extends AbstractTreeTableModel
implements TreeTableModel {
	
	private static final String DATE_TYPE = "LocalDate";
	private static final int TREE_COLUMN = 0; 
	public static final int JAVA_TYPE_COLUMN = 4; 
	public static final int FORMAT_COLUMN = 5; 
	
	public static final IProcessCobolItem
					TO_SHORT = new ProcesItem(CheckForTypeUpdate.TO_SHORT),
					TO_INT   = new ProcesItem(CheckForTypeUpdate.TO_INT),
					TO_BIGDECIMAL = new ProcesItem(CheckForTypeUpdate.TO_BIGDECIMAL),
					TO_DOUBLE = new ProcesItem(CheckForTypeUpdate.TO_DOUBLE);
	
	private static String[] COLUMN_NAMES = {
		"Field Name",
		"Position",
		"Length",
		"Picture",
		"Java Type",
		"Format"
	};
	
	
	private static DateComboFactory formatFactory = new DateComboFactory();
	


//	private static ParseItems lastParser;
//	public static CobolItemTreeModel newModel(RecordDetail record, AbstractLine line) {
//		List<? extends IItemDetails> cobolItems = record.getCobolItems();
//		ParseItems p, lp = lastParser;
//		CobolItemNode root;
//		
//	
//		if (lp != null && cobolItems == lp.items) {
//			p = lp;
//			root = lp.root;
//		} else {
//			p = new ParseItems();
//			root = p.parse(cobolItems);
//			lastParser = p;
//		}
//		
//		return new CobolItemTreeModel(record, root, p.arrayIndexs, line);
//	}
	

	//private final RecordDetail record;
	private final CobolItem root;
	
	public static CobolItemTreeModel newTreeModel(List<? extends IItemJRecUpd> items) {
		CobolItem root = new CobolItem(null, null);
		for (int i = 0; i < items.size(); i++) {
			new CobolItem(root, items.get(i));
		}
		
		return new CobolItemTreeModel(root);
	}
	
	
	private CobolItemTreeModel(CobolItem root) {
		super(root);
		
		this.root = root;
	}
	
	
	public List<TreePath> getFirstPaths(int numItems) {
		ArrayList<TreePath> paths = new ArrayList<TreePath>();
		traverseTree(numItems, paths, root, new ArrayList<CobolItemTreeModel.CobolItem>());
		
		return paths;
	}
	
	private int traverseTree(int numItems, List<TreePath> paths, CobolItem itm, List<CobolItem> currentPaths) {
		int count = 1;
		if (itm.childCount() == 0) {
			paths.add(new TreePath(currentPaths.toArray(new Object[currentPaths.size()])));
		} else {
			currentPaths.add(itm);
			count += traverseTree(numItems, paths, itm.children.get(0), currentPaths);
			for (int i = 1; count < numItems && i < itm.childCount(); i++ ) {
				CobolItem childItm = itm.children.get(i);
				if (childItm.hasChildren()) {
					count += traverseTree(numItems, paths, childItm, currentPaths);
				} else {
					count += 1;
				}
			}
			currentPaths.remove(currentPaths.size() - 1);
		}
		return count;
	}
	
	public void updateCobolItemType() {
		//TODO
		//TODO
		applyToTree(new IProcessCobolItem() {
			@Override public void process(CobolItem item) {
				String jType = item.javaType;
				if (item.javaType.equals(item.getTypeDtls().javaTypeName)) {
					jType = null;
				}
				item.cobolItem.setJavaType(jType);;
			}
		});
	}
	
	public void applyToTree(IProcessCobolItem process) {
		applyToTree(root, process);
	}
	
	private void applyToTree(CobolItem item, IProcessCobolItem process) {
		if (item.hasChildren()) {
			for (int i = 0; i < item.childCount(); i++ ) {
				applyToTree(item.children.get(i), process);
			}
		} else {
			process.process(item);
		}
	}

	public ITableCellAdapter newAdapter(JTreeTable treeTbl) {
		return new TypeCellAdapter(treeTbl, this);
	}

	@Override
	public int getColumnCount() {
		return COLUMN_NAMES.length;
	}

	@Override
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}
	

//    public Class<?> getColumnClass(int column) {
//
//
//        if (column == TREE_COLUMN) {
//            return TreeTableModel.class;
//        }
//        return super.getColumnClass(column);
//    }



	@Override
	public Object getValueAt(Object node, int column) {
		if (column == TREE_COLUMN) { return null;}
		
		CobolItem cobNode = (CobolItem) node;
		IItemJRecUpd cblDtl = cobNode.cobolItem;
		
		if ((cobNode.hasChildren() && column > 2)
		|| (cblDtl == null)) { 
			return ""; 
		}
	
		switch (column) {
		case 1: return cblDtl.getPosition();
		case 2: return cblDtl.getStorageLength();
		case 3: return cblDtl.getPicture();
		case JAVA_TYPE_COLUMN: return cobNode.javaType;
		case FORMAT_COLUMN   : 
			if (DATE_TYPE.equals(cobNode.javaType)) {
				return cobNode.dateFormat;
			}
		}

		return "";
	}

	@Override
	public CobolItem getRoot() {
		return root;
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeTable.AbstractTreeTableModel#getChildCount(java.lang.Object)
	 */
	@Override
	public int getChildCount(Object node) {
		return ((CobolItem) node).childCount();
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeTable.AbstractTreeTableModel#getChild(java.lang.Object, int)
	 */
	@Override
	public Object getChild(Object node, int i) {
		return ((CobolItem) node).children.get(i);
	}

    public Class<?> getColumnClass(int column) {

        if (column == TREE_COLUMN) {
            return TreeTableModel.class;
        }
        return super.getColumnClass(column);
    }

//
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeTable.AbstractTreeTableModel#isCellEditable(java.lang.Object, int)
	 */
	@Override
	public boolean isCellEditable(Object node, int column) {
		CobolItem cobNode = (CobolItem) node;
		
		if (! cobNode.hasChildren()) {
			switch (column) {
			case JAVA_TYPE_COLUMN: return true;
			case FORMAT_COLUMN   : 
				//System.out.println((cobNode.item == null ? "~~" : cobNode.item.getFieldName()) + "\t" + cobNode.item.getFormatId());
				return DATE_TYPE.equals(cobNode.javaType);
			}
		}
		return super.isCellEditable(node, column);
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeTable.AbstractTreeTableModel#setValueAt(java.lang.Object, java.lang.Object, int)
	 */
	@Override
	public void setValueAt(Object aValue, Object node, int column) {
		CobolItem cobNode = (CobolItem) node;

		switch (column) {
		case JAVA_TYPE_COLUMN: 
			if (aValue != null) {
				String oldType = cobNode.javaType;
				cobNode.javaType = aValue.toString();
				if (! oldType.equals(cobNode.javaType)) {
					int oldFormat = cobNode.cobolItem.getFormatId();
					if (DATE_TYPE.equals(cobNode.javaType)) {
						cobNode.cobolItem.setFormat(cobNode.dateFormat.fieldFormat, cobNode.dateFormat.dateFormatStr);
					} else {
						cobNode.cobolItem.setFormat(0, "");
					}
					if (oldFormat != cobNode.cobolItem.getFormatId()) {
						super.fireTreeNodesChanged(cobNode, cobNode.getPath().toArray(), null, null);
					}
				}
			}
			return;
		case FORMAT_COLUMN: 
			if (aValue instanceof StdDateFormatStr) {
				cobNode.dateFormat = (StdDateFormatStr) aValue;
				cobNode.cobolItem.setFormat(cobNode.dateFormat.fieldFormat, cobNode.dateFormat.dateFormatStr);
			}
			return;
		}

		super.setValueAt(aValue, node, column);
	}

	public static class CobolItem {
		final IItemJRecUpd cobolItem;
		String javaType="";
		StdDateFormatStr dateFormat;
		List<CobolItem> children;
		IDateComboBuilder dateComboBldr = formatFactory.allFormats;
		final CobolItem parent;
		
		public CobolItem(CobolItem parent, IItemJRecUpd item) {
			this.cobolItem = item;
			this.parent = parent;
			
			if (parent == null || item == null) {
				children = new ArrayList<CobolItemTreeModel.CobolItem>();
				return;
			}
			parent.children.add(this);
			
			List<? extends IItemJRecUpd> childItems = item.getChildItems();
			int childCount = childItems.size();
			if (childCount == 0) {
				javaType = getTypeDtls().javaTypeName;
				
				if (TypeManager.isNumeric(item.getType())) {
					dateComboBldr = formatFactory.numericFormats;
					if (item.getDisplayLength() < 8 && item.getScale() == 0) {
						dateComboBldr = formatFactory.shortNumericFormats;
					}
				} 
				
				dateFormat = StdDateFormatStr.getStandardFormat(item.getFormatId(), item.getParameter());

//				System.out.println(item.getFieldName() + "\t" + javaType + "\t" + format + "\t" + item.getParameter() );
				return;
			}
			
			children = new ArrayList<CobolItemTreeModel.CobolItem>(childCount);
			for (int i = 0; i < childCount; i++) {
				new CobolItem(this, childItems.get(i));
			}
		}
		
		public List<CobolItem> getPath() {
			ArrayList<CobolItem> list = new ArrayList<CobolItem>();
			CobolItem itm = this;
			do {
				list.add(itm);
				itm = itm.parent;
			} while (itm != null);
			
			int size = list.size();
			ArrayList<CobolItem> ret = new ArrayList<CobolItem>(size);
			for (int i = size - 1; i >= 0; i--) {
				ret.add(list.get(i));
			}
			return ret;
		}
		

		/**
		 * @param cobolItem
		 * @return
		 */
		private JavaTypeDetails getTypeDtls() {
			JavaTypeDetails td = new JavaTypeDetails(false, cobolItem.getType(), null, cobolItem.getStorageLength(), cobolItem.getScale(), 
					cobolItem.getFormatId(), cobolItem.getParameter());
			return td;
		}
		
		
		int childCount() {
			return children == null ? 0 : children.size() ;
		}
		
		boolean hasChildren() {
			return children != null && children.size() > 0;
		}


		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() {
			return cobolItem == null ? "" : cobolItem.getFieldName();
		}
	}
	
	public static class TypeCellAdapter extends AbstractTypeCellAdapter {
		final CobolItemTreeModel mdl;
		final JTree tree;
		//final ComboBoxRender typeRender = new ComboBoxRender(TYPES_STR);	
		
		
		TypeCellAdapter(JTreeTable treeTbl, CobolItemTreeModel model) {
			super(CobolItemTreeModel.JAVA_TYPE_COLUMN, CobolItemTreeModel.FORMAT_COLUMN);
			tree = treeTbl.getTree();
			this.mdl = model;
		}
		
		
		protected boolean isEditable(int row, int column) {
			TreePath path = tree.getPathForRow(row);
			return mdl.isCellEditable(path.getLastPathComponent(), column);
		}

		@Override
		protected TableCellEditor getFormatEditor(int row) {
			TableCellEditor ret = null;
			TreePath path = tree.getPathForRow(row);
			//if (mdl.isCellEditable(path.getLastPathComponent(), CobolItemTreeModel.FORMAT_COLUMN)) {
				ret =  ((CobolItem) path.getLastPathComponent()).dateComboBldr.asTableCellEditor();
			//}
			
			return ret;
		}
	}

	public static interface IProcessCobolItem {
		void process(CobolItem item);
	}
	
	public static class ProcesItem implements IProcessCobolItem {
		final CheckForTypeUpdate typeUpdate;
		
		public ProcesItem(CheckForTypeUpdate typeUpdate) {
			super();
			this.typeUpdate = typeUpdate;
		}

		public void process(CobolItem item) {
			IItemJRecUpd cblItem = item.cobolItem;
			if (cblItem != null && typeUpdate.updateTheJavaType(cblItem.getType(), cblItem.getScale(), cblItem.getStorageLength())) {
				//System.out.println(cblItem.getFieldName() + " >> " + type1 + " " + type2);
				item.javaType = typeUpdate.javaType;
			}
		}
	}
}
