package net.sf.RecordEditor.re.cobol.tree;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JComponent;
import javax.swing.JTabbedPane;

import net.sf.JRecord.External.ExternalRecord;
import net.sf.RecordEditor.re.codeGen.type.JavaTypeOptions;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;

/**
 * Display a Cobol Copybook (schema) so the user can select the java type
 * 
 * @author Bruce Martin
 *
 */
public class CobolSchemaDisplay implements AbstractWizardPanel<GenWizData>, ActionListener {
	
	
//	public final JRadioButton useBigDecimalBtn  = new JRadioButton("Use BigDecimal"),
//							  useDoubleBtn  = new JRadioButton("Use double");
//	public final JCheckBox    useShortChk = new JCheckBox("Use short instead of int");
	
	public final BaseHelpPanel pnl = new BaseHelpPanel("CobolSchemaDisplay");
	public final JavaTypeOptions typeOptions = new JavaTypeOptions(pnl, this);
	
	private GenWizData data;
	
	private final ArrayList<CobolRecordDisplay> recDisplay;

	public CobolSchemaDisplay(ExternalRecord rec) {
				
		int numberOfRecords = rec.getNumberOfRecords();
		if (numberOfRecords == 0) {
			recDisplay = new ArrayList<CobolRecordDisplay>(1);
			recDisplay.add(new CobolRecordDisplay(rec, pnl));
		} else {
			JTabbedPane tab = new JTabbedPane();
			recDisplay = new ArrayList<CobolRecordDisplay>(numberOfRecords);
			for (int i = 0 ; i < numberOfRecords; i++) {
				CobolRecordDisplay cobolRecordDisplay = new CobolRecordDisplay(rec.getRecord(i), null);
				tab.addTab(rec.getRecord(i).getRecordName(), cobolRecordDisplay.pnl);
				recDisplay.add(cobolRecordDisplay);
			}
			this.pnl.addComponentRE(1, 3, BasePanel.FILL, BaseHelpPanel.GAP, BasePanel.FULL, BasePanel.FULL, tab);
		}
	}
	


	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		
		CobolItemTreeModel.IProcessCobolItem typeUpdate;
		if (e.getSource() == typeOptions.useBigDecimalBtn) {
			typeUpdate = CobolItemTreeModel.TO_BIGDECIMAL;
		} else if (e.getSource() == typeOptions.useDoubleBtn) {
			typeUpdate = CobolItemTreeModel.TO_DOUBLE;
		} else if (e.getSource() == typeOptions.useShortChk) {
			typeUpdate = typeOptions.useShortChk.isSelected()
					? CobolItemTreeModel.TO_SHORT
					: CobolItemTreeModel.TO_INT;
		} else {
			return;
		}
		
		for (CobolRecordDisplay rec : recDisplay) {
			rec.cobolUpdate(typeUpdate);
		}
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#setValues(java.lang.Object)
	 */
	@Override
	public void setValues(GenWizData detail) throws Exception {
		this.data = detail;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getValues()
	 */
	@Override
	public GenWizData getValues() throws Exception {
		for (CobolRecordDisplay rec : recDisplay) {
			rec.updateCobolItemType();
		}

		return data;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#skip()
	 */
	@Override
	public boolean skip() {
		return ! data.updateTypes;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getComponent()
	 */
	@Override
	public JComponent getComponent() {
		return pnl;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#showHelpRE()
	 */
	@Override
	public void showHelpRE() {
		// TODO Auto-generated method stub
		
	}
	
	
}
