package net.sf.RecordEditor.re.codeGen;

import java.awt.event.ActionEvent;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.schema.IMultiFormatSchema;
import net.sf.RecordEditor.re.schema.ICreateFieldIterator;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.ReSpecificScreenAction;
import net.sf.RecordEditor.utils.screenManager.IActiveCheckActive;

@SuppressWarnings("serial")
public abstract class AbstractCodeGenAction extends ReSpecificScreenAction implements IActiveCheckActive {
	
	private final int schemaRetrieveOption;
	private JDialog dialog;
	private Object lastSchema;
	
	public AbstractCodeGenAction(String name, int schemaRetrieveOption) {
		super(name);
		
		this.schemaRetrieveOption = schemaRetrieveOption;
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public final void actionPerformed(ActionEvent e) {
		net.sf.RecordEditor.utils.screenManager.ReFrame actionHandler 
				= net.sf.RecordEditor.utils.screenManager.ReFrame.getActiveFrame();

		IMultiFormatSchema schemaMf = null;
		AbstractLayoutDetails schema = null;
		net.sf.RecordEditor.re.schema.IGetSchemaAndFields schemaSource = getDisplay(net.sf.RecordEditor.re.schema.IGetSchemaAndFields.class, actionHandler);
		if (schemaSource != null) {
			schemaMf = schemaSource.getSchema(schemaRetrieveOption);
			schema = schemaMf.asAbstractLayoutDetails();
		}
		if (schema == null ||  ! (schema instanceof net.sf.JRecord.Details.LayoutDetail)) {
			String message = net.sf.RecordEditor.utils.msg.UtMessages.CAN_NOT_RETRIEVE_SCHEMA_MSG.get();
			Common.logMsg(message, null);
			JOptionPane.showConfirmDialog(null, message);
			return;
		}

		if (! checkSchemaOk(schema)) { return ; }
		
		if (dialog == null || schema != lastSchema) {
			String ds = "";
			net.sf.RecordEditor.utils.BasicLayoutCallback callback = getDisplay(
					net.sf.RecordEditor.utils.BasicLayoutCallback.class, actionHandler);
			net.sf.RecordEditor.utils.interfaces.IGetFileName gfn = getDisplay(
					net.sf.RecordEditor.utils.interfaces.IGetFileName.class, actionHandler);
			if (gfn == null) {
				net.sf.RecordEditor.re.display.AbstractFileDisplay ad = getDisplay(
						net.sf.RecordEditor.re.display.AbstractFileDisplay.class);
				if (ad != null) {
					ds = ad.getFileView().getFileName();
				}
			} else  {
				ds = gfn.getFileName();
			}
			ds = ds == null ? "" : ds;
	
			dialog = createDialog(
					schemaSource, schemaMf, 
					schemaSource.getSchema(net.sf.RecordEditor.re.schema.IGetSchemaAndFields.SCHEMA_DERIVE_TYPE),
					ds, callback);
			lastSchema = schema;
		}
		dialog.setVisible(true);
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.screenManager.AbstractActiveScreenAction#checkActionEnabled()
	 */
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.screenManager.AbstractActiveScreenAction#checkActionEnabled()
	 */
	@Override
	public final void checkActionEnabled() {
		net.sf.RecordEditor.re.schema.IGetSchemaAndFields schemaSource = getDisplay(net.sf.RecordEditor.re.schema.IGetSchemaAndFields.class);

		setEnabled(schemaSource != null && checkSchemaMatches(schemaSource, schemaSource.schemaChecking()));
	}

	protected abstract boolean checkSchemaMatches(net.sf.RecordEditor.re.schema.IGetSchemaAndFields schemaSource, int schemaCheckingType);
	protected abstract boolean checkSchemaOk(AbstractLayoutDetails schema);
	
	protected abstract JDialog createDialog(ICreateFieldIterator fieldIt, IMultiFormatSchema schema, IMultiFormatSchema schemaDeriveTypes, String ds,
			net.sf.RecordEditor.utils.BasicLayoutCallback callback);

	
}
