package net.sf.RecordEditor.re.codeGen;

import javax.swing.JDialog;
import javax.swing.JOptionPane;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.schema.IMultiFormatSchema;
import net.sf.RecordEditor.re.schema.ICreateFieldIterator;
import net.sf.RecordEditor.utils.screenManager.IActiveCheckActive;

@SuppressWarnings("serial")
public class CodeGenAction extends AbstractCodeGenAction implements IActiveCheckActive {

	private static final String TEMPLATE_DIRECTORY = GenerateTreeItem.TEMPLATE_DIRECTORY;
	
	private final TemplateOption schemaType;
	private final String template;
	private final boolean multiRecord;

	private final boolean onlyText, onlyCsv, csv;
	
	public CodeGenAction(String name, String template, boolean multiRecord,
			TemplateOption schemaType, int schemaRetrieveOption) {
		super(name, schemaRetrieveOption);
		
		this.template = template;
		this.multiRecord = multiRecord;
		this.schemaType = schemaType;
		
		onlyText = ! schemaType.bin;
		onlyCsv  = schemaType.csv && ! schemaType.fixed;
		csv = schemaType.csv;
//		onlyText = schemaType == ST_TEXT_CSV || schemaType == ST_TEXT_FIXED 
//				|| schemaType == ST_TEXT_CSV_FIXED;
//		onlyCsv  = schemaType == ST_TEXT_CSV || schemaType == ST_CSV;
//		csv = onlyCsv || schemaType == ST_TEXT_CSV_FIXED || schemaType == ST_ANY;
	}

	

	protected JDialog createDialog(
			ICreateFieldIterator fieldIt,
			IMultiFormatSchema schema, IMultiFormatSchema schema1, String ds,
			net.sf.RecordEditor.utils.BasicLayoutCallback callback) {
//		return (new  net.sf.RecordEditor.re.codeGen.CodeGenOptions(
//								(net.sf.JRecord.Details.LayoutDetail)schema, 
//								template, 
//								TEMPLATE_DIRECTORY,
//								ds,
//								callback)).dialog;
		return net.sf.RecordEditor.re.codeGen.CodeGenOptionWiz.newStdWizard(
				fieldIt,
				schema, template, TEMPLATE_DIRECTORY, ds, callback).dialog;
	}

	protected boolean checkSchemaOk(AbstractLayoutDetails schema) {
		if ( ! schemaOk4Action(schema)) {
			String fileTypeStr = schemaType.schemaTypeName;
			if (! multiRecord) {
				fileTypeStr += " (Single Record)";
			}
			//Common.logMsg(message, null);
			JOptionPane.showMessageDialog(null, net.sf.RecordEditor.utils.msg.UtMessages.INVALID_FILE_TYPE_MSG.get(fileTypeStr));
			return false;
		}
		return true;
	}

	protected boolean checkSchemaMatches(net.sf.RecordEditor.re.schema.IGetSchemaAndFields schemaSource, int schemaCheckingType) {
		boolean ok;
		switch (schemaCheckingType) {
		case net.sf.JRecord.Details.IGetSchema.ST_CHECK_SCHEMA:
			ok =  schemaOk4Action(schemaSource.getSchema(net.sf.JRecord.Details.IGetSchema.SCHEMA_4_GEN).asAbstractLayoutDetails());
			break;
		case  net.sf.JRecord.Details.IGetSchema.ST_OTHER_SCHEMA:
			ok = false;
			break;
		case net.sf.JRecord.Details.IGetSchema.ST_NO_CHECK_ON_SCHEMA:
			ok = true;
			break;
		case net.sf.JRecord.Details.IGetSchema.ST_CSV:
			ok = csv;
			break;
		case net.sf.JRecord.Details.IGetSchema.ST_TEXT_CSV:
			ok = csv && onlyText;
			break;
		case net.sf.JRecord.Details.IGetSchema.ST_FIXED:
			ok = ! onlyCsv;
			break;
		default:
			ok = true;
		}
		return ok;
	}
	
	private boolean schemaOk4Action(AbstractLayoutDetails schema) {
//		net.sf.JRecord.Details.IGetSchema schemaSource = getDisplay(net.sf.JRecord.Details.IGetSchema.class);
//		AbstractLayoutDetails schema = null;
//		if (schemaType == ST_TEXT_FIXED)
//			System.out.println("**Fixed->> " + onlyCsv + " " + (schema.getOption(Options.OPT_IS_CSV)) 
//					+ " " + (schema.getOption(Options.OPT_IS_CSV) == Options.YES));
		boolean ok = true;
		if (schema == null || (schema.isBinary() && onlyText)){
			ok = false;
		} else if (schema.getOption(net.sf.JRecord.Details.Options.OPT_IS_CSV) == net.sf.JRecord.Details.Options.YES) {
			ok = csv;
		} else {
			ok = ! onlyCsv;
		}
		
		return (ok && (multiRecord || schema == null || schema.getRecordCount() == 1));
	}
	
	
}
