package net.sf.RecordEditor.re.codeGen;

import javax.swing.JComponent;
import javax.swing.JDialog;

import net.sf.JRecord.Details.IGetSchema;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.Details.schema.IMultiFormatSchema;
import net.sf.JRecord.External.ExternalRecord;
import net.sf.JRecord.cg.details.TemplateDtls;
import net.sf.RecordEditor.re.cobol.ISetVisible;
import net.sf.RecordEditor.re.cobol.tree.GenWizData;
import net.sf.RecordEditor.re.codeGen.type.TypePnl;
import net.sf.RecordEditor.re.schema.ICreateFieldIterator;
import net.sf.RecordEditor.utils.BasicLayoutCallback;
import net.sf.RecordEditor.utils.wizards.AbstractWizard;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;

public class CodeGenOptionWiz extends AbstractWizard<GenWizData> implements ISetVisible {

	private static final int INITIAL_JRECORD_VERSION_IDX = 1;
	public  final JDialog dialog;
	private final ExternalRecord xRecord;
	private final IOptionPnl codeGenOptions;
	private final IMultiFormatSchema mSchema, mSchemaType;
	
//	public CodeGenOptionWiz(CblLoadData data) {
//		this(new JDialog(ReMainFrame.getMasterFrame()), data, new GenWizData());
//	}
	
	public static CodeGenOptionWiz newStdWizard(
			ICreateFieldIterator createFldIterator,
			IMultiFormatSchema mSchema, String template, String templateResourceDir, String dataFileName,
			BasicLayoutCallback callback) {
		JDialog jDialog = new JDialog();
		return new CodeGenOptionWiz(
						createFldIterator,
						jDialog, 
						new OptionPnl(
								jDialog, (LayoutDetail) mSchema.asAbstractLayoutDetails(), 
								template, templateResourceDir, dataFileName, callback),
						mSchema, null,
						new GenWizData()
				);
	}

	public static CodeGenOptionWiz newTreeWizard(
			ICreateFieldIterator createFldIterator,
			IMultiFormatSchema mSchema, IMultiFormatSchema mSchemaType,
			String dataFileName,
			BasicLayoutCallback callback) {
		JDialog jDialog = new JDialog();
		return new CodeGenOptionWiz(
						createFldIterator,
						jDialog, 
						new GenOptionTreePnl(
								jDialog, 
								mSchemaType.isTypesDerived(),
								(LayoutDetail) mSchema.asAbstractLayoutDetails(), 
								(LayoutDetail) mSchemaType.asAbstractLayoutDetails(),
								dataFileName, callback),
						mSchema, mSchemaType,
						new GenWizData()
				);
	}

	private CodeGenOptionWiz(
			ICreateFieldIterator createFldIterator,
			JDialog dialog, IOptionPnl optionPnl, 
			IMultiFormatSchema mSchema, IMultiFormatSchema mSchemaType,
			GenWizData wizData) {
		super(dialog, wizData);
		
		this.dialog = dialog;
		this.xRecord = mSchema.asExternalRecord();
		this.codeGenOptions = optionPnl;
		this.mSchema = mSchema;
		this.mSchemaType = mSchemaType;
		
		wizData.xRecord = xRecord;
		wizData.xRecordDerivedTypes = mSchemaType == null ? null : mSchemaType.asExternalRecord();
		
		@SuppressWarnings("unchecked")
		AbstractWizardPanel<GenWizData>[] pnls = new AbstractWizardPanel[2];


		pnls[0] = codeGenOptions;
		pnls[1] = new TypePnl(xRecord, wizData.xRecordDerivedTypes, createFldIterator);

		super.setPanels(pnls);
	}
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#finished(java.lang.Object)
	 */
	@Override
	public void finished(GenWizData details) {
		LayoutDetail l = details.typesUpdated
				? xRecord.asLayoutDetail() 
				: mSchemaType == null || codeGenOptions.useStdLayout()
					? (LayoutDetail) mSchema.asAbstractLayoutDetails()
					: (LayoutDetail) mSchemaType.asAbstractLayoutDetails();
		codeGenOptions.generateTemplate(l);
	}
	
	/**
	 * @param b
	 * @see java.awt.Dialog#setVisible(boolean)
	 */
	public void setVisible(boolean b) {
		dialog.setVisible(b);
	}	


	public static interface IOptionPnl extends AbstractWizardPanel<GenWizData> {
		public void generateTemplate(LayoutDetail l);
		public boolean useStdLayout();
	}	


	public static class OptionPnl implements IOptionPnl {
		private final CodeGenOptionsPnl optionPnl;
		private final JDialog dialog;
		
		public OptionPnl(
				JDialog dialog,
				LayoutDetail schema, String template, String templateResourceDir, String dataFileName,
				BasicLayoutCallback callback) {
			super();
			
			this.dialog = dialog;
			this.optionPnl = new CodeGenOptionsPnl(schema, dataFileName, callback, null, template, templateResourceDir);
		}

		GenWizData data;
		@Override
		public void setValues(GenWizData detail) throws Exception {
			data = detail;
		}

		@Override
		public GenWizData getValues() throws Exception {
			data.updateTypes = optionPnl.hasOption(TemplateDtls.T_UPDATE_TYPES, false);
			return data;
		}

		@Override
		public boolean skip() {
			return false; //! data.updateTypes;
		}

		@Override
		public JComponent getComponent() {
			return optionPnl.codeGenOptionPnl;
		}

		@Override
		public void showHelpRE() {
			// TODO Auto-generated method stub
			
		}

		public void generateTemplate(LayoutDetail l) {
			optionPnl.generateTemplate(dialog, l);
		}

		@Override
		public boolean useStdLayout() {
			return true;
		}	
	}

	public static class GenOptionTreePnl implements IOptionPnl {
		private final GenerateSelection treeOptionPnl;
		private final JDialog dialog;
		private final boolean derivedType;
//		private boolean useStdLayout = true;
		
		public GenOptionTreePnl(
				JDialog dialog,
				boolean derivedType,
				LayoutDetail schema, LayoutDetail schemaDeriveTypes,
				String dataFileName,
				BasicLayoutCallback callback) {
			super();
			
			this.dialog = dialog;
			this.treeOptionPnl = new GenerateSelection(schema, schemaDeriveTypes, dataFileName, callback);
			this.derivedType = derivedType;
		}

		private GenWizData data;
		@Override
		public void setValues(GenWizData detail) throws Exception {
			data = detail;
		}

		private String lastTemplate = "";
		@Override
		public GenWizData getValues() throws Exception {
			
			GenerateTreeItem genItem = treeOptionPnl.getGenItem();
			if (genItem != null) {
				data.usingStdTypes = genItem.schemaRetrieveOption == IGetSchema.SCHEMA_DEFAULT;
				
				String template = genItem.template;
				if (! lastTemplate.equals(template)) {
					TemplateDtls templateDtls = new TemplateDtls(
							null, template, genItem.templateDirectory, data.xRecord.getNumberOfRecords() > 1,
							INITIAL_JRECORD_VERSION_IDX, null);
//							genItem.templateDirectory, template, data.xRecord.getNumberOfRecords() > 1,
//							INITIAL_JRECORD_VERSION_IDX);
					data.updateTypes = templateDtls.hasOption(TemplateDtls.T_UPDATE_TYPES, false);
					
					lastTemplate = template;
				}
			}

			return data;
		}

		@Override
		public boolean skip() {
			return false; //! data.updateTypes;
		}

		@Override
		public JComponent getComponent() {
			return treeOptionPnl.mainPane;
		}

		@Override
		public void showHelpRE() {
			// TODO Auto-generated method stub
			
		}

		public void generateTemplate(LayoutDetail l) {
			treeOptionPnl.generateTemplate(dialog, l);
		}

		@Override
		public boolean useStdLayout() {
			GenerateTreeItem genItem = treeOptionPnl.getGenItem();
			return genItem == null || genItem.schemaRetrieveOption != IGetSchema.SCHEMA_DERIVE_TYPE;
		}	
	}
}
