package net.sf.RecordEditor.re.codeGen.type;

import java.util.Iterator;
import java.util.List;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;

import net.sf.RecordEditor.re.schema.ICreateFieldIterator;
import net.sf.RecordEditor.re.util.csv.ColumnDateFormatChk;
import net.sf.RecordEditor.re.util.csv.StdDateFormatStr;

@SuppressWarnings("serial")
public class FieldTblMdl extends AbstractTableModel {

	private static final String DATE_TYPE = "LocalDate";
	public static final int JAVA_LENGTH_COLUMN = 2; 
	public static final int JAVA_TYPE_COLUMN = 3; 
	public static final int FORMAT_COLUMN = 4; 

	//private List<ExternalFieldJType> fields;
	
	private boolean changed = false;
	
	private static String[] COLUMN_NAMES = {
			"Field Name",
			"Position",
			"Length",
			"Java Type",
			"Format"
		};
	
	private TypeCellAdapter tcAdapter;
	private final int recordIdx;
	private final ICreateFieldIterator createFieldIterator;

	
	public FieldTblMdl(List<ExternalFieldJType> fields, int recordIdx, ICreateFieldIterator createFieldIterator) {
		super();
		this.tcAdapter = new TypeCellAdapter(fields);
		this.recordIdx = recordIdx;
		this.createFieldIterator = createFieldIterator;
	}
	
	public AbstractTypeCellAdapter newTypeCellAdapter() {
		return tcAdapter;
	}
	
	public void apply(CheckForTypeUpdate check) {
		List<ExternalFieldJType> fields = tcAdapter.fields;
		for (int i = 0; i < fields.size(); i++) {
			ExternalFieldJType fld= fields.get(i);
			if (check.updateTheJavaType(fld.getType(), fld.getDecimal(), fld.getLen())) {
				changed = changed || fld.javaType == null || ! fld.javaType.equals(check.javaType);
				fld.javaType = check.javaType;
			}
		}
		if (changed) {
			super.fireTableDataChanged();
		}
	}

	@Override
	public int getRowCount() {
		return tcAdapter.fields.size();
	}

	@Override
	public int getColumnCount() {
		return COLUMN_NAMES.length;
	}

	@Override
	public Object getValueAt(int rowIndex, int columnIndex) {
		ExternalFieldJType field = tcAdapter.fields.get(rowIndex);

		switch (columnIndex) {
		case 0: return field.getName();
		case 1: return field.getPos();
		case 2: return field.getLen() < 0 ? "" : field.getLen();
		case JAVA_TYPE_COLUMN: return field.javaType;
		case FORMAT_COLUMN   : 
			if (DATE_TYPE.equals(field.javaType)) {
				return field.dateFormat;
			}
		}
		return "";
	}

	@Override
	public String getColumnName(int column) {
		return COLUMN_NAMES[column];
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return isCellEditable(tcAdapter.fields.get(rowIndex), columnIndex);
	}

	/**
	 * @param fld
	 * @param columnIndex
	 * @return
	 */
	private static boolean isCellEditable(ExternalFieldJType fld, int columnIndex) {
		switch (columnIndex) {
		case JAVA_TYPE_COLUMN: return true;
		case FORMAT_COLUMN   : 
//			System.out.println(
//					fld.getName() + " " + fld.javaType 
//					+ "\t" + fld.dateFormat + " " + (fld.dateFormat instanceof StdDateFormatStr));
			return DATE_TYPE.equals(fld.javaType) && fld.dateFormat instanceof StdDateFormatStr;
		}
		return false;
	}

	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		ExternalFieldJType fld = tcAdapter.fields.get(rowIndex);
		switch (columnIndex) {
		case JAVA_TYPE_COLUMN: 
			if (aValue != null) {
				String oldType = fld.javaType;
				fld.javaType = aValue.toString();
				if (! oldType.equals(fld.javaType)) {
					int newFormat = 0;
					String param = "";
					if (DATE_TYPE.equals(fld.javaType)) {
//						if (fld.dateFormat == null) {
//							fld.dateFormat=StdDateFormatStr.NULL_DATE_FORMAT;
//						}
						if (fld.dateFormat==StdDateFormatStr.NULL_DATE_FORMAT && createFieldIterator != null) {
							Iterator<String> fieldIterator = createFieldIterator.getFieldIterator(recordIdx, rowIndex);
							if (fieldIterator != null) {
								ColumnDateFormatChk cd = new ColumnDateFormatChk();
								cd.addAll(fieldIterator);
								List<StdDateFormatStr> possibleDateFormats = cd.getPossibleDateFormats();
								if (possibleDateFormats != null && possibleDateFormats.size() > 0) {
									fld.dateFormat = possibleDateFormats.get(0);
								}
							}
						}
						newFormat = fld.dateFormat.fieldFormat;
						param = fld.dateFormat.dateFormatStr;
					}
					changed = changed || newFormat != fld.getCellFormat() || ! param.equals(fld.getParameter());
					fld.setCellFormat(newFormat);
					fld.setParameter(param);
					super.fireTableRowsUpdated(rowIndex, rowIndex);
				}
			}
			return;
		case FORMAT_COLUMN: 
			if (aValue instanceof StdDateFormatStr) {
				changed = changed || fld.dateFormat != aValue;
				fld.dateFormat = (StdDateFormatStr) aValue;

				fld.setCellFormat(fld.dateFormat.fieldFormat);
				fld.setParameter(fld.dateFormat.dateFormatStr);
			}
		}
	}

	public final boolean isChanged() {
		return changed;
	}

	public final void setFields(List<ExternalFieldJType> flds) {
		this.tcAdapter.fields = flds;

		super.fireTableDataChanged();
	}

	private static class TypeCellAdapter extends AbstractTypeCellAdapter {
		private List<ExternalFieldJType> fields;
		
		public TypeCellAdapter(List<ExternalFieldJType> fields) {
			super(JAVA_TYPE_COLUMN, FORMAT_COLUMN);

			this.fields = fields;
		}

		@Override
		protected TableCellEditor getFormatEditor(int row) {
			IDateComboBuilder dateComboBldr = fields.get(row).dateComboBldr;
			return dateComboBldr == null ? null : dateComboBldr.asTableCellEditor();
		}

		@Override
		protected boolean isEditable(int row, int column) {
			return isCellEditable(fields.get(row), column);
		}
	}
}
