package net.sf.RecordEditor.re.codeGen.type;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JComponent;
import javax.swing.JTabbedPane;

import net.sf.JRecord.External.ExternalRecord;
import net.sf.RecordEditor.re.cobol.tree.GenWizData;
import net.sf.RecordEditor.re.schema.ICreateFieldIterator;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;

public class TypePnl implements AbstractWizardPanel<GenWizData>  {

	private final ArrayList<RecordPnl> recDisplay;
	private BasePanel pnl = new BasePanel("FldType");
	public final JavaTypeOptions typeOptions = new JavaTypeOptions(pnl, new ActionListener() {
		@Override public void actionPerformed(ActionEvent e) {
			CheckForTypeUpdate typeUpdate;
			if (e.getSource() == typeOptions.useBigDecimalBtn) {
				typeUpdate = CheckForTypeUpdate.TO_BIGDECIMAL;
			} else if (e.getSource() == typeOptions.useDoubleBtn) {
				typeUpdate = CheckForTypeUpdate.TO_DOUBLE;
			} else if (e.getSource() == typeOptions.useShortChk) {
				typeUpdate = typeOptions.useShortChk.isSelected()
						? CheckForTypeUpdate.TO_SHORT
						: CheckForTypeUpdate.TO_INT;
			} else {
				return;
			}
			
			for (RecordPnl rec : recDisplay) {
				rec.apply(typeUpdate);
			}
		}
	});

	
	
	public TypePnl(ExternalRecord rec, ExternalRecord recWithTypes, ICreateFieldIterator createFldIterator) {
		int numberOfRecords = rec.getNumberOfRecords();
		if (numberOfRecords == 0) {
			recDisplay = new ArrayList<RecordPnl>(1);
			recDisplay.add(new RecordPnl(pnl, rec, recWithTypes, 0, createFldIterator));
		} else {
			JTabbedPane tab = new JTabbedPane();
			recDisplay = new ArrayList<RecordPnl>(numberOfRecords);
			if (recWithTypes == null) {
				for (int i = 0 ; i < numberOfRecords; i++) {
					RecordPnl recordDisplay = new RecordPnl(null, rec.getRecord(i), null, i, createFldIterator);
					tab.addTab(rec.getRecord(i).getRecordName(), recordDisplay.pnl);
					recDisplay.add(recordDisplay);
				}
			} else {
				for (int i = 0 ; i < numberOfRecords; i++) {
					RecordPnl recordDisplay = new RecordPnl(null, rec.getRecord(i), recWithTypes.getRecord(i), i, createFldIterator);
					tab.addTab(rec.getRecord(i).getRecordName(), recordDisplay.pnl);
					recDisplay.add(recordDisplay);
				}
			}
			this.pnl.addComponentRE(1, 3, BasePanel.FILL, BaseHelpPanel.GAP, BasePanel.FULL, BasePanel.FULL, tab);
		}
		
	}
	
	private GenWizData data;
	private boolean usingStdTypes = true;
	@Override
	public void setValues(GenWizData detail) throws Exception {
		data = detail;
		if (data.xRecordDerivedTypes != null && data.usingStdTypes != usingStdTypes) {
			for (int i = 0; i < recDisplay.size(); i++) {
				recDisplay.get(i).setFields(data.usingStdTypes);
			}
			usingStdTypes = data.usingStdTypes;
		}
	}

	@Override
	public GenWizData getValues() throws Exception {
		for (int i = 0; i < recDisplay.size() && ! data.typesUpdated; i++) {
			data.typesUpdated = recDisplay.get(i).isChanged();
		}
		return data;
	}

	@Override
	public boolean skip() {
		return ! data.updateTypes;
	}

	@Override
	public JComponent getComponent() {
		return pnl;
	}

	@Override
	public void showHelpRE() {
		// TODO Auto-generated method stub
		
	}
}
