/*
 * Changes
 *
 * # Version 0.61 Bruce Martin 2007/04/14
 *   - Changes due to changes in the RecordFieldsRec,
 *     creation of JRecord etc
 */
package net.sf.RecordEditor.re.db.Record;

import java.sql.PreparedStatement;
import java.sql.SQLException;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.jdbc.AbsDB;
import net.sf.RecordEditor.utils.lang.LangConversion;
import net.sf.RecordEditor.utils.params.Parameters;

/**
 * This class provides DB Access using
 *
 *   <pre>
 *       Select
 *              Field_Pos as  Pos,
 *              Field_Length as Len,
 *              Field_Name as Name,
 *              Field_Description,
 *              Field_Type as Type,
 *              Decimal_Pos as Decimal,
 *              Cell_Format cellFormat,
 *              Field_Parameter parameter,
 *              Default_Value as Default,
 *              Cobol_Name,
 *              SubKey
 *       From Tbl_RF1_RecordFields
 *       where RecordId = ?
 *       order by Field_Pos, Field_Length
 *
 *   </pre>
 * it also provides Insert / Update / Delete routines (depending on the options selected)
 *
 * @Author Generated by BuildJava.Rexx by Bruce Martin
 */

public class RecordFieldsDB  extends AbsDB<RecordFieldsRec> {

	private static int fetchSize = -121;

  private static final String[] COLUMN_NAMES = LangConversion.convertColHeading(
			"DB-RecordFields Columns",
			new String[] {
                   "Position"
                 , "Length"
                 , "FieldName"
                 , "Description"
                 , "FieldType"
                 , "DecimalPos"
                 , "Cell_Format"
                 , "Parameter"
                 , "DefaultValue"
                 , "Cobol Name"
                 , "Field Id"
  });


  private PreparedStatement delAllRecordFields = null;
  private int paramRecordId;

  public RecordFieldsDB(boolean forceSortOnPosition) {

      resetSearch();

      sSQL = " Select  Field_Pos, Field_Length, Field_Name, FIELD_DESCRIPTION, Field_Type, Decimal_Pos, Cell_Format, Field_Parameter, Default_Value, Cobol_Name, Sub_Key";
      sFrom = "  From Tbl_RF1_RecordFields";
      sWhereSQL = "  where RecordId = ?";
      if (forceSortOnPosition || Common.OPTIONS.sortFieldsOnPos.isSelected()) {
    	  sOrderBy = "  order by Field_Pos, Field_Length Desc";
      } else {
    	  sOrderBy = "  order by Sub_Key";
      }
      updateSQL = "Update Tbl_RF1_RecordFields  "
                   +  " Set Field_Pos= ? "
                   +  "   , Field_Length= ? "
                   +  "   , Field_Name= ? "
                   +  "   , FIELD_DESCRIPTION= ? "
                   +  "   , Field_Type= ? "
                   +  "   , Decimal_Pos= ? "
                   +  "   , Cell_Format= ? "
                   +  "   , Field_Parameter= ? "
                   +  "   , Default_Value= ? "
                   +  "   , Cobol_Name= ? "
                   +  "   , Sub_Key= ? "
                   +  " Where RecordId= ? "
                   +  "   and Sub_Key= ? "
                        ;

      deleteSQL = "Delete From  Tbl_RF1_RecordFields  "
                   +  " Where RecordId= ? "
                   +  "   and Sub_Key= ? "
                        ;

      insertSQL = "Insert Into  Tbl_RF1_RecordFields  ("
                      + "    Field_Pos"
                      + "  , Field_Length"
                      + "  , Field_Name"
                      + "  , Field_Description"
                      + "  , Field_Type"
                      + "  , Decimal_Pos"
                      + "  , Cell_Format"
                      + "  , Field_Parameter"
                      + "  , Default_Value"
                      + "  , Cobol_Name"
                      + "  , Sub_Key"
                      + "  , RecordId"
                      + ") Values ("
                      +    "     ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?"
                      + ")";

      super.columnNames = RecordFieldsDB.COLUMN_NAMES;
  }


  /**
   * sets up the DB parameters
   *
   * @param RecordId
   *
   */
  public void setParams( int RecordId) {

      paramRecordId = RecordId;
  }

  /**
   *  This method opens a SQL query
   */
  public void open() {

     prepareCursor();

      try {
          sqlCursor.setInt(1, paramRecordId);

          setStringArgs(1);


          rsCursor  = sqlCursor.executeQuery();
		  int fs = getFetchSize();
		  if (fs > 0) {
			rsCursor.setFetchSize(fs);
		  }

          message = "";
      } catch (Exception ex) {
           setMessage(ex.getMessage(), ex);
      }
  }
  
  private static int getFetchSize() {
  	if (fetchSize < 0) {
  		int fs = 0;
  		String s = Parameters.getString(Parameters.FETCH_SIZE);
  		if (s != null && s.length() > 0) {
  			try {
					fs = Integer.parseInt(s);
				} catch (Exception e) {	}
  		}
  		fetchSize = fs;
  	}
  	
  	return fetchSize;
  }



//  /**
//   *  This method returns the next record (AbsRecord) from the cursor
//   */
//  public AbsRecord absFetch() {
//      return fetch();
//  }


  /**
   *  This method returns the next record from the cursor
   */
  public RecordFieldsRec fetch() {
  RecordFieldsRec ret = null;

      try {
          if (rsCursor.next()) {
             ret = new RecordFieldsRec(
                        rsCursor.getInt(1)
                      , rsCursor.getInt(2)
                      , rsCursor.getString(3)
                      , rsCursor.getString(4)
                      , rsCursor.getInt(5)
                      , rsCursor.getInt(6)
                      , rsCursor.getInt(7)
                      , rsCursor.getString(8)
                      , rsCursor.getString(9)
                      , rsCursor.getString(10)
                      , rsCursor.getInt(11)
                   );
          }
          message = "";
      } catch (Exception ex) {
           setMessage(ex.getMessage(), ex);
      }

      return ret;
  }


  /**
   *  Get the number of columns returned by the SQL
   */
  public int getColumnCount() {
      return 10;
  }





  /**
   *
   * @param statement statement that needs parameteres set
   * @param value Value to assign to the Statement parameters
   * @param idx parameter index
   *
   * @return updated index
   * @throws SQLException SQL error
   */
  protected int setSQLParams(PreparedStatement statement, RecordFieldsRec value, boolean insert, int idx)
                             throws SQLException {
      RecordFieldsRec val = value;

      statement.setInt(idx++, val.getValue().getPos());
      statement.setInt(idx++, val.getValue().getLen());
      statement.setString(idx++, correctStr(val.getValue().getName()));
      statement.setString(idx++, correctStr(val.getValue().getDescription()));
      statement.setInt(idx++, val.getValue().getType());
      statement.setInt(idx++, val.getValue().getDecimal());
      statement.setInt(idx++, val.getValue().getCellFormat());
      statement.setString(idx++, correctStr(val.getValue().getParameter()));
      statement.setString(idx++, correctStr(val.getValue().getDefault()));
      statement.setString(idx++, correctStr(val.getValue().getCobolName()));
      statement.setInt(idx++, val.getValue().getSubKey());

      if (insert) {
    	  statement.setInt(idx++, paramRecordId);
      }

      return idx;
  }


  /**
   * Setup the where parameters
   *
   * @param statement SQL statement
   * @param value Value to assign to the Statement parameters
   * @param idx current index
   * @throws SQLException SQL error
   */
  protected void setWhere(PreparedStatement statement, RecordFieldsRec value, int idx)
                          throws SQLException {

      statement.setInt(idx++, paramRecordId);
      statement.setInt(idx, value.initSubKey);
  }


  /**
   *  This method deletes all records matching the parameters
   */
  public void deleteAll() {

	  
      try {
    	  open();
    	  boolean doDelete = fetch() != null;
    	  close();
    	  if (doDelete) {
	          if (isPrepareNeeded(delAllRecordFields)) {
	              delAllRecordFields = connect.getUpdateConnection().prepareStatement(
	                   "Delete From  Tbl_RF1_RecordFields  "
	                   +  " Where RecordId= ? "
	                        );
	          }
	
	          delAllRecordFields.setInt(1, paramRecordId);
	
	          delAllRecordFields.executeUpdate();
	          message = "";
    	  }
      } catch (Exception ex) {
           setMessage(ex.getMessage(), ex);
      } finally {
     	 freeConnection();
      }
  }



  /**
   * This method gets the next key
   */
  protected int getNextKey() {
      final String sql = "Select max(Sub_Key) From  Tbl_RF1_RecordFields "
                   +  " Where RecordId= ? "
                 ;

      return getNextIntSubKey(sql, paramRecordId);
  }





  /**
   *  This method inserts one record
   *
   *  @param val value to be inserted
   */
  public void insert(RecordFieldsRec value) {
      //RecordFieldsRec val = (RecordFieldsRec) value;

      int i = 0;
      boolean free = super.isSetDoFree(false);

      int key = getNextKey();

      value.getValue().setSubKey(key++);
      while ((i++ < 10) && (! tryToInsert(value))) {
          value.getValue().setSubKey(key++);
      }

      super.setDoFree(free);
  }



  /**
   *  This method inserts one record
   *
   *  @param val value to be inserted
   */
  public void insert(RecordFieldsRec value, int key) {
      //RecordFieldsRec val = (RecordFieldsRec) value;

      int i = 0;
      boolean free = super.isSetDoFree(false);

      value.getValue().setSubKey(key++);
      while ((i++ < 10) && (! tryToInsert(value))) {
          value.getValue().setSubKey(key++);
      }

      super.setDoFree(free);
  }



/**
 *   Close the prepared statments
 */
 public void fullClose() {

     super.fullClose();

     closeStatement(delAllRecordFields);

 }

}
