package net.sf.RecordEditor.re.display;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.GZIPOutputStream;

import javax.swing.AbstractAction;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.msg.UtMessages;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChanged;
import net.sf.RecordEditor.utils.swing.treeCombo.FileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.recordEditor.ui.common.StdComponents.UiButton;
import net.sf.recordEditor.ui.form.FormBuilder;
import net.sf.recordEditor.ui.form.FormHelper;
import net.sf.recordEditor.ui.form.ThreeColumnFormBuilder;
import net.sf.recordEditor.ui.styles.StyleManager;
import net.sf.recordEditor.ui.text.UiTextField;

/**
 * Purpose: Write a small extract file from an existing file
 * 
 * @author Bruce Martin
 *
 */
public class FileExtract {

	private final IFileSelectCombo fileNameTxt  = RecentFiles.newFileCombo(true, false, false);
	private final IFileSelectCombo extractFileTxt = FileSelectCombo.newCombo(Parameters.SAVED_COPY_LIST, 25, false, false);
	private final UiTextField sizeTxt = new UiTextField(StyleManager.EMPTY_STYLE);
	private final JTextArea messageTxt = new JTextArea();
	@SuppressWarnings("serial")
	private final UiButton extractBtn = new UiButton(StyleManager.EMPTY_STYLE, new AbstractAction("Create Extract") {
		@Override public void actionPerformed(ActionEvent arg0) {
			doCopy();
		}
	} );

	private ReFrame frame;
	public FileExtract() {
		
		ThreeColumnFormBuilder form = FormBuilder.threeColumnForm("Extract Small S", null)	.form();
		
		String source = Common.OPTIONS.DEFAULT_FILE_DIRECTORY.getNoStar();
		fileNameTxt.setFileNameSilently(source);
		extractFileTxt.setFileNameSilently(source);
		
		sizeTxt.setText("100");
		JPanel pnl = form
					.l(        "File Name", fileNameTxt)
					.l("Extract File Name", extractFileTxt)	.setGap(FormHelper.GAP1)
					.l("Extract size (kb)", sizeTxt,    extractBtn)	.setGap(FormHelper.GAP2)
					.ls(messageTxt, FormBuilder.FULL_WIDTH, 1, 1, form.lastColumn)
				.build();
		pnl	.setVisible(true);
		
		fileNameTxt.addFileChangedListner(new IFileChanged() {		
			@Override public void fileChanged(File selectedFile) {
				if (selectedFile.exists() && selectedFile.isFile()) {
					extractFileTxt.setFileNameSilently(selectedFile.getAbsolutePath() + "_extract.gz");
				}
			}
		}); 
		frame = new ReFrame("", "File Extract", null);
		frame.addMainComponent(pnl);
		frame.setVisible(true);
	}
	
	
	private void doCopy()  {
		String filename = fileNameTxt.getText();
		String extractFile = extractFileTxt.getText();
		
		if (filename == null || filename.length() == 0) {
			messageTxt.setText(UtMessages.SUPPLY_SOURCE_FILE.get());
		} else if (extractFile == null || extractFile.length() == 0) {
			messageTxt.setText(UtMessages.SUPPLY_EXTRACT_FILE.get());
		} else {
			try {
				int size = Integer.parseInt(sizeTxt.getText());
				copy(filename, extractFile, size);
				frame.setVisible(false);
				Common.logMsg("Extract file : " + extractFile + ", created", null);
			} catch (IOException e) {
				e.printStackTrace();
			} catch (Exception e) {
			}
		}
	}
	private void copy(String filename, String outFilename, int size) throws FileNotFoundException, IOException {
		FileInputStream in = new FileInputStream(filename);
		GZIPOutputStream out = new GZIPOutputStream(new FileOutputStream(outFilename));
		
		int total = 0;
		byte[] buf = new byte[1024 * size];
		int num = in.read(buf, total, buf.length - total);

		while (num >= 0 && total + num < buf.length) {
			total += num;
			num = in.read(buf, total, buf.length - total);
		}
		
		if (num > 0) {
			total += num;
		}

		out.write(buf, 0, total);
		
		in.close();
		out.close();
	}
}
