/*
 * @Author Bruce Martin
 * Created on 24/01/2007 for version 0.60
 *
 * Purpose:
 * Edit RecordEditor Startup options
 *
 * # Version 0.61 Bruce Martin 2007/04/14
 *   - Major rewrite with tags being organised into
 *     logical groups + code removed to EditParams
 *   - JRecord Spun off, code reorg
 */
package net.sf.RecordEditor.re.editProperties;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.External.CopybookWriterManager;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.JRecord.Numeric.ConversionManager;
import net.sf.JRecord.Types.Type;
import net.sf.RecordEditor.re.util.CopybookLoaderFactoryDB;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.LangConversion;
import net.sf.RecordEditor.utils.lang.ReAbstractAction;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.params.ProgramOptions;
import net.sf.RecordEditor.utils.screenManager.ReMainFrame;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.Combo.ComboStrOption;
import net.sf.RecordEditor.utils.swing.ComboBoxs.EnglishStrModel;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

/**
 * Edit RecordEditor Startup options
 *
 * @author Bruce Martin
 *
 */

public class EditOptions {
	
	public final static int SHOW_COBOL = 1;
	public static void newEditProperties(int id) {
		EditOptions p = new EditOptions(false, false, false, true);
		switch (id) {
		case SHOW_COBOL: p.mainTabbed.setSelectedComponent(p.cobolTabbed);		break;
		}
	}

    private static final int PROGRAM_DESCRIPTION_HEIGHT
            = Math.min(
                    SwingUtils.values().NORMAL_FIELD_HEIGHT * 21,
                    Toolkit.getDefaultToolkit().getScreenSize().height * 3 / 5);
    
    private List<SaveScreens> screensToCheck = new ArrayList<SaveScreens>(2);
    private EditParams params = new EditParams();

    private JFrame frame = new JFrame("Record Editor Options Editor");
    private JEditorPane programDescription;

    private JTabbedPane mainTabbed  = new JTabbedPane();
    private JTabbedPane propertiesTabbed = new JTabbedPane();
    private JTabbedPane cobolTabbed = new JTabbedPane();
    private JTabbedPane csvTabbed   = new JTabbedPane();
    private JTabbedPane textTabbed  = new JTabbedPane();
    private JTabbedPane xmlTabbed   = new JTabbedPane();
    private JTabbedPane jdbcTabbed  = new JTabbedPane();
    private JTabbedPane jarsTabbed  = new JTabbedPane();
    private JTabbedPane userTabbed  = new JTabbedPane();
    private JTabbedPane looksTabbed = new JTabbedPane();
    //private JTextArea msgFld = new JTextArea("");

    @SuppressWarnings("serial")
    private AbstractAction saveBtn = new ReAbstractAction("Save", Common.ID_SAVE_ICON) {
        public void actionPerformed(ActionEvent e) {
            for (SaveScreens a : screensToCheck) {
            	a.options.save();
            }
            save();
//            params.writeProperties();
//            params.writeJarFiles();
            
         }
    };
    
    private final EditCsvLists editDelims = EditCsvLists.newDelimiterEditor(params);
    private final EditCsvLists editQuotes = EditCsvLists.newQuoteEditor(params); 

    private String description
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_PnlDescription",

          "<h2>RecordEditor (" + Common.currentVersion() + ") Properties Editor</h2>"
        + "This program lets you edit the <b>RecordEditor (" + Common.currentVersion() + ") </b> properties files "
        + "and jar files."
        + "<br>There is little validation, so be very careful what changes you make."
        + "<br>Many changes will not take affect until the next time"
        + "the <b>RecordEditor</b> / <b>Layout Editor</b> is run again."
        + "<br><br>There are 5 basic tab types in the program: "
        + "<table border=\"1\" cellpadding=\"3\">"
        + "<tr><TD><b>Properties</b></td>"
           +  "<td>Lets you update system properties like "
           +  "<b>Screen Position, Directories, Other Options, Wizard Option and Big File Options</b></td></tr>"
        + "<tr><td><b>Xml</b></td><td>Xml Xslt properties / jars "
        + "<tr><td><b>Jars</b></td><td>These options let you update "
            + "the various JAR (java libraries) used by the <b>RecordEditor</b>."
            + "<br>The only time you should need to do this "
            + "is if you are adding your own code to the <b>RecordEditor</b> or installing Velocity.</td></tr>"
        + "<tr><td><b>Extensions</b></td><td>These option let you define your own Types, formats to "
           + "the <b>RecordEditor</b>.</td></tr>"
        + "<tr><td><b>Looks</b></td><td>This option lets you define the look and feel of the RecordEditor</td</tr>"
        + "</table>"
        + "<br/><b>Files being updated are:</b><br/><pre>"
        + "System       Jars File={0}"
        + "<br>System JDBC  Jars File={1}"
        + "<br>  User       Jars File={2}"
       //+ "<br>Editor Jars File=" + CommonCode.EDITOR_JAR_FILE
        + "<br>       Properties file={3}</pre>",
        new Object[] {
            CommonCode.SYSTEM_JAR_FILE,
            CommonCode.SYSTEM_JDBC_JAR_FILE,
            CommonCode.USER_JAR_FILE,
            Parameters.getPropertyFileName()
        });

//    tabbed.addTab("Copybook Loaders", loadersPnl);
//    tabbed.addTab("User Types", typePnl);
//    tabbed.addTab("User Formats", formatPnl);

    private String directoryDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_Directories",

        "<h2>Directories</h2>"
        + "The properties on this panel are for the various directories "
        + "used by the <b>RecordEditor</b>");

    private Object[] codeGenDirecory = {
    		Parameters.CODEGEN_DIRECTORY,   "Java~JRecord code generation directory",
    		Parameters.CODEGEN_DIRECTORY_DFLT, EditPropertiesPnl.FLD_DIR, null};
    private Object[] defaultCobolDirectory = {
    		Parameters.DEFAULT_COBOL_DIRECTORY, "The Directory where Cobol Copybooks are stored.",
    		null, EditPropertiesPnl.FLD_DIR, null};

    private Object[][] directoryParams1 = {
            {"HelpDir",	"Directory holding the help files", null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.DEFAULT_DIRECTORY,	"Directory where the Editor Starts in (if no file specified)", null, EditPropertiesPnl.FLD_DIR, null},
            defaultCobolDirectory,
            {Parameters.VELOCITY_TEMPLATE_DIRECTORY, "Velocity Template directory (Editor)", Parameters.DEFAULT_VELOCITY_TEMPLATE_DIRECTORY, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.VELOCITY_COPYBOOK_DIRECTORY, "Velocity Template directory (Copybooks)", Parameters.DEFAULT_VELOCITY_COPYBOOK_DIRECTORY, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.COPYBOOK_DIRECTORY, "Directory to read / write file copybooks to", null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.XSLT_TEMPLATE_DIRECTORY, "Xslt Template directory (Editor)", Parameters.encodeVars(Common.OPTIONS.DEFAULT_XSLT_DIRECTORY.get()), EditPropertiesPnl.FLD_DIR, null},
            {Parameters.EXPORT_SCRIPT_DIRECTORY, "Export Script directory", Parameters.encodeVars(Common.OPTIONS.DEFAULT_SCRIPT_EXPORT_DIRECTORY.defaultValue), EditPropertiesPnl.FLD_DIR, null},
            {Parameters.SCRIPT_DIRECTORY, "Extension Script directory", Parameters.encodeVars(Common.OPTIONS.DEFAULT_SCRIPT_DIRECTORY.get()), EditPropertiesPnl.FLD_DIR, null},
    };


    private Object[][] directoryParams2 = {
            {Parameters.COMPARE_SAVE_DIRECTORY,  "Compare Save Directory",         null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.FILTER_SAVE_DIRECTORY,   "Filter Save Directory",          null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.FIELD_SAVE_DIRECTORY,     "Fields used Save Directory",    null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.SORT_TREE_SAVE_DIRECTORY, "Sort Tree Save Directory",      null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.RECORD_TREE_SAVE_DIRECTORY, "Record Tree Save Directory",  null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.COPY_SAVE_DIRECTORY,      "Copy Save Directory",        Parameters.COPY_SAVE_DIRECTORY_DFLT, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.LAYOUT_EXPORT_DIRECTORY,  "Layout Velocity Export Directory", Parameters.LAYOUT_EXPORT_DIRECTORY_DFLT, EditPropertiesPnl.FLD_DIR, null},
            codeGenDirecory,
    };

    private String testDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_Test",

          "<h2>Test Properties</h2>"
        + "The options on this screen are used in Testing the Editor.");

    private Object[][] testParams = {
            {Parameters.PROPERTY_TEST_MODE, "Weather we are running automated Tests (Marathon ?) or not", null, EditPropertiesPnl.FLD_BOOLEAN, "Test Mode"},
 //           {Parameters.BRING_LOG_TO_FRONT, "Bring Log to the Front if Data is written to it", null, EditPropertiesPnl.FLD_BOOLEAN, "Bring log to Front"}, // Checked
 //           {Parameters.ASTERIX_IN_FILE_NAME, "Allow the asterix ('*') character in file Names", null, EditPropertiesPnl.FLD_BOOLEAN, null},
 //           {Parameters.PREFERED_AS_DEFAULT, "Default to prefered layout", null, EditPropertiesPnl.FLD_BOOLEAN, null},
            {Parameters.WARN_BINARY_FIELDS_DEFAULT, "Warn the user if Binary-Fields and Structure=Default", null, EditPropertiesPnl.FLD_BOOLEAN, "Warn on Structure change"}, // CHECK
            {Parameters.PROPERTY_LOAD_FILE_BACKGROUND, "Load File in Background thread", null, EditPropertiesPnl.FLD_BOOLEAN, "Load In background"}, // Checked
            {Parameters.USE_NEW_TREE_EXPANSION, "Use New Tree Expansion", null, EditPropertiesPnl.FLD_BOOLEAN,  "Use New Tree Expansion"},
            {Parameters.SEARCH_ALL_FIELDS, "Search: All Fields", null, EditPropertiesPnl.FLD_BOOLEAN,  "On Search Screen default to \"All Fields\""},
            {Parameters.NAME_FIELDS, "Add names to screen Components", null, EditPropertiesPnl.FLD_BOOLEAN,  "Add names to JComponents for use by testing tools"},
            {Parameters.LOG_TEXT_FIELDS,  "Keep a record of all Text Fields going through the Language section. This can be used to generate the language conversion file (GetText PO)", null, EditPropertiesPnl.FLD_BOOLEAN,  "Record Text Fields",},
            {Parameters.SPECIAL_FIND_BTN_NAME,  "This option adds >> to the search button (on the find search screen) to better identifier it in testing", null, EditPropertiesPnl.FLD_BOOLEAN,  "Rename Search btn",},
            {Parameters.INCLUDE_TYPE_NAME,  "Include the type name on the record screen", null, EditPropertiesPnl.FLD_BOOLEAN,  "Include Type Name on Record Screen",},
            {Parameters.ADD_FILE_SEARCH_BTN,  "Add file search button to Open Screen", null, EditPropertiesPnl.FLD_BOOLEAN,  "Add file search button to Open Screen",},
            {Parameters.FILE_FIELDS_START_IN_EDIT_MODE,  "Start File Fields in Edit Mode", null, EditPropertiesPnl.FLD_BOOLEAN,  "Start File Fields in Edit Mode",},
            {Parameters.FILE_FIELDS_AUTO_COMPLETE,  "Wether to display the Auto complete screen for File Fields", null, EditPropertiesPnl.FLD_BOOLEAN,  "Auto complete File Fields",},

//            {Parameters.SHOW_ALL_EXPORT_OPTIONS, "Show all export panels", null, EditPropertiesPnl.FLD_BOOLEAN,  "Show all export panels on the export Screen"},
//            {Parameters.DEL_SELECTED_WITH_DEL_KEY, "Delete Selected Rows using the delete key", null, EditPropertiesPnl.FLD_BOOLEAN,  "Delete Selected with delete key"},
//            {Parameters.WARN_WHEN_USING_DEL_KEY,  "Warn the user before deleteing Selected Rows using the delete key", null, EditPropertiesPnl.FLD_BOOLEAN,  "Warn user with delete key"},
    };

    private String behaviourDescription
    = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_Behaviour",

      "<h2>Behavior Properties</h2>"
    + "The options on this screen affect the behavior of the editor.<br/>"
    + "<b>Bring Log to the Front</b>: Wether the log file will be brought to the fron when "
    + "data is written to it.<br/>"
    + "<b>Default to prefered</b>: Wether to use the prefered layout by default when editting<br/>"
    + "<b>Show all export panels</b>: Wether to show all export options or just the option selected<br/>");

    
//    private String[][] EXTENDED_FILE_CHOOSER = {
//    		{Parameters.FILE_CHOOSER_NORMAL,   "Standard FileChooser"}, 
// //   		{Parameters.FILE_CHOOSER_OPTIONAL,   "FileChoose Based on LAF"},
//    		{Parameters.FILE_CHOOSER_OPT_HIDE, "Recent Pane visible Based on LAF"},
//    		{Parameters.FILE_CHOOSER_EXTENDED, "Extended FileChooser"}};
    private String[][] EXTENDED_FILE_CHOOSER1 = {
    		
      		{Parameters.FILE_CHOOSER_NEW,   "New Standard"}, 
      		{Parameters.FILE_CHOOSER_NORMAL,   "Old Standard FileChooser"}, 
      		 //   		{Parameters.FILE_CHOOSER_OPTIONAL,   "FileChoose Based on LAF"},
    		{Parameters.FILE_CHOOSER_OPT_HIDE, "Recent Pane visible Based on LAF"},
    		{Parameters.FILE_CHOOSER_EXTENDED, "Extended FileChooser"}};
    
    private String[][] RE_OPEN_OPTION = {
    		{Parameters.FILE_CHOOSER_RE_OLD, "Use Original Open Panel"}, 
 //   		{Parameters.FILE_CHOOSER_OPTIONAL,   "FileChoose Based on LAF"},
    		{Parameters.FILE_CHOOSER_RE_CLASSIC, "Use Classic Open"},
    		{Parameters.FILE_CHOOSER_RE_SELECT,  "Use Choose file dialog"},
  	};

    private Object[][] behaviourParamsCsv = {
            {Parameters.FILE_OPEN_CSV_EDIT,    "Type of FileChooser to use: Standard, old Standard, Extended, or choose based on Look and feel / screen size", null, EditPropertiesPnl.FLD_LIST, "File Chooser Type (Csv Editor)", EXTENDED_FILE_CHOOSER1},
    };
    private Object[][] behaviourParams = {
        {Parameters.BRING_LOG_TO_FRONT,       "Bring Log to the Front if Data is written to it", null, EditPropertiesPnl.FLD_BOOLEAN, "Bring log to Front"}, // Checked
        {Parameters.ASTERIX_IN_FILE_NAME,     "Allow the asterix ('*') character in file Names", null, EditPropertiesPnl.FLD_BOOLEAN, null},
        {Parameters.PREFERED_AS_DEFAULT,      "Default to prefered layout", null, EditPropertiesPnl.FLD_BOOLEAN, null},
        {Parameters.SHOW_ALL_EXPORT_OPTIONS,  "Show all export panels", null, EditPropertiesPnl.FLD_BOOLEAN,  "Show all export panels on the export Screen"},
        {Parameters.DEL_SELECTED_WITH_DEL_KEY,"Delete Selected Rows using the delete key", null, EditPropertiesPnl.FLD_BOOLEAN,  "Delete Selected rows with the delete key"},
        {Parameters.WARN_WHEN_USING_DEL_KEY,  "Warn the user before deleteing Selected Rows using the delete key", null, EditPropertiesPnl.FLD_BOOLEAN,  "Warn when deleteing rows via delete key"},
        {Parameters.USE_FILE_WIZARD,          "Use File Wizard when no Layout is known for the file", null, EditPropertiesPnl.FLD_BOOLEAN, "Use file Wizard"},
        {Parameters.HIGHLIGHT_MISSING_TRANSLATIONS,  "Highlight text for which there is no translation by adding a # to the start of the text", null, EditPropertiesPnl.FLD_BOOLEAN,  "Highlight missing translations",},
        {Parameters.SEPERATE_WINDOWS,  "Create views in seperate windows instead of tabs on the file panel", null, EditPropertiesPnl.FLD_BOOLEAN,  "Create Screens in seperate Windows",},
        {Parameters.SHOW_RECORDEDITOR_TIPS,   "Show RecordEditor Tips on program startup", null, EditPropertiesPnl.FLD_BOOLEAN,  "Show RecordEditor Tips",},
        {Parameters.EDIT_RAW_TEXT,  "Allow the user to edit file as Raw Text ??", null, EditPropertiesPnl.FLD_BOOLEAN,  "Allow Editting Raw Text",},
        {Parameters.CSV_SHOW_FILECHOOSER_OPTIONS,  "Show File Options on Csv Open Screen", null, EditPropertiesPnl.FLD_BOOLEAN,  "Show File Options on Csv Open",},
        {Parameters.OPEN_IN_LAST_DIRECTORY,   "Open program using Last used Directory (instead of the default Directory)", null, EditPropertiesPnl.FLD_BOOLEAN,  "Open in Last Directory",},
        {Parameters.SORT_FIELDS_POSITION,     "<html>Sort File Fields on Field Position or use DB Field-Id Sequence<br/>Using Field-Id sequence may have unforseen consquences</html>", null, EditPropertiesPnl.FLD_BOOLEAN,  "Sort Fields on Position",},

        {"", "", null, EditPropertiesPnl.FLD_EMPTY, null},
//        {Parameters.FILE_CHOOSER_OPTION_CSV,      "Type of FileChooser to use: Normal, Extended, or choose based on Look and feel", null, EditPropertiesPnl.FLD_LIST, "File Chooser Type", EXTENDED_FILE_CHOOSER},
//        {Parameters.FILE_OPEN_CSV_EDIT,    "Type of FileChooser to use: Standard, old Standard, Extended, or choose based on Look and feel / screen size", null, EditPropertiesPnl.FLD_LIST, "File Chooser Type (Csv Editor)", EXTENDED_FILE_CHOOSER1},
//  		
//        {"", "", null, EditPropertiesPnl.FLD_EMPTY, null},
        {Parameters.FILE_CHOOSER_UI,    "Wether to use the Extended File Chooser or Look and Feel File Chooser", null, EditPropertiesPnl.FLD_BOOLEAN, "Extended/Laf File Chooser"},
        {Parameters.FILE_OPEN_RECORD_EDIT,    "File Open panel", null, EditPropertiesPnl.FLD_LIST, "File Open panel", RE_OPEN_OPTION},
 //       {"", "", null, EditPropertiesPnl.FLD_EMPTY, null},
    };
    

    private String readWriteDescription
    = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_ReadWrite",

      "<h2>Read/Write Properties</h2>"
    + "This panels lists various options for reading / writing files.");
    private Object[][] readWriteParams = {
            {Parameters.BYPASS_RECSVEDIT_PREVIEW, "Bypass ReCsvEdit preview screen when starting with a file", null, EditPropertiesPnl.FLD_BOOLEAN, "Bypass ReCsvEdit preview screen", },
            {Parameters.RENAME_OLD_FILE, "Rename the old file when doing a save", null, EditPropertiesPnl.FLD_BOOLEAN, "Rename old File before save", },
    };

    private String fileDescription
    = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_FileParams",

      "<h2>File Properties</h2>"
    + "This panels lists various File related parameters.");

    private Object[][] fileParams = {
            {"UserInitilizeClass", "This User written class will be invoked when the <b>RecordEditor</b starts.", EditPropertiesPnl.FLD_TEXT, "User Init Class"},
            {Parameters.INVALID_FILE_CHARS, "Characters that are invalid in a file Name", null, EditPropertiesPnl.FLD_TEXT, null},
            {Parameters.FILE_REPLACEMENT_CHAR, "Char to Replace invalid Filename Chars", null, EditPropertiesPnl.FLD_TEXT, null},
            {"DateFormat", "Date Format String eg dd/MM/yy or dd.MMM.yy. the field is case sensitive, it uses Java date Formatting", null, EditPropertiesPnl.FLD_DATE, "Date Format"},
            {"SignificantCharInFiles.1", "Number of characters to use when looking up record layouts (small)", null, EditPropertiesPnl.FLD_INT, "Significant chars 1"},
            {"SignificantCharInFiles.2", "Number of characters to use when looking up record layouts (medium)", null, EditPropertiesPnl.FLD_INT, "Significant chars 2"},
            {"SignificantCharInFiles.3", "Number of characters to use when looking up record layouts (large)", null, EditPropertiesPnl.FLD_INT, "Significant chars 3"},
    };

    private String layoutWizardParamsDescription
    = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_WizardParams",
      "<h2>Layout Wizard Properties</h2>"
    + "This panels holds various Field Search options used by the Layout Wizard");

    private Object[][] layoutWizardParams = {
            {Parameters.FS_RUN_AUTOMATIC, "Weather to Run the field search Automatically or not", null, EditPropertiesPnl.FLD_BOOLEAN, "Run the field search Automatically"}, // Check
            {Parameters.FS_MAINFRAME_ZONED, "Look for Mainframe Zoned numeric fields", null, EditPropertiesPnl.FLD_BOOLEAN, null},
            {Parameters.FS_PC_ZONED, "Look for PC Zoned numeric fields (Cobol PIC 9 fields)", null, EditPropertiesPnl.FLD_BOOLEAN, null},
            {Parameters.FS_COMP3, "Look for comp-3 fields", null, EditPropertiesPnl.FLD_BOOLEAN, null},
            {Parameters.FS_COMP_BIG_ENDIAN, "Look for Big Endian Binary", null, EditPropertiesPnl.FLD_BOOLEAN, null},
            {Parameters.FS_COMP_LITTLE_ENDIAN, "Look for Little Endian Binary", null, EditPropertiesPnl.FLD_BOOLEAN, null},
   };


    private String bigModelDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_BigFiles",

          "<h2>Big Model Properties</h2>"
        + "This panels lists parameters for the \"Big File\" Data Models.<br\\>"
        + "You can use these options to optimise the Read Time for very big files");

//    private String[][] bigModelParams = {
//            {Parameters.PROPERTY_BIG_FILE_PERCENT, "File Size to Memory Percent to Start using the Big-File-Model", null},
//            {Parameters.PROPERTY_BIG_FILE_CHUNK_SIZE, "Big-File-Model Memory Chunks (KB)", null},
//            {Parameters.PROPERTY_BIG_FILE_COMPRESS_OPT, "Big-File-Model compress option (N - No, R - Read, F - Read (fast cpu), S - Space, Y - Yes)", null},
//            {Parameters.PROPERTY_BIG_FILE_FILTER_LIMIT, "Big-File Filter/Tree limit (thousands)", null},
//            {Parameters.PROPERTY_BIG_FILE_DISK_FLAG, "Force Storing chunks on Disk (Used in Testing)", null},
//            {Parameters.PROPERTY_BIG_FILE_USE_SPECIAL_FIXED_MODEL, "Use Fixed Length Model (Used in Testing)", null},
//            {Parameters.PROPERTY_BIG_FILE_LARGE_VB, "Use Large VB Model (Y/N)", null},
//            {Parameters.PROPERTY_LOAD_FILE_BACKGROUND, "Load File in Background thread (default Y)", null},
//    };

    private String[][] COMPRESS_OPTION = {{"N", "No"}, {"R", "Read"}, {"F", "Read (fast cpu)"}, {"S", "Space"}, {"Y", "Yes"}};
    private String[][] YNT_OPTION = {{"Y", "Yes"}, {"N", "No"}, {"T", "Test"}};
    private String[][] YNT_OPTION2 = {{"Y", "Mock"}, {"N", "No"}, {"T", "Yes"}};
    private Object[][] bigModelParams = {
            {Parameters.PROPERTY_BIG_FILE_PERCENT, "File Size to Memory Percent to Start using the Big-File-Model", null, EditPropertiesPnl.FLD_INT, "Big File Percentage"},
            {Parameters.PROPERTY_BIG_FILE_CHUNK_SIZE, "Big-File-Model Memory Chunks (KB) default=1000", null, EditPropertiesPnl.FLD_INT, "Chunk Size (KB)"},
            {Parameters.PROPERTY_BIG_FILE_COMPRESS_OPT, "Big-File-Model compress option (Default Read)", null, EditPropertiesPnl.FLD_LIST, "Compress Option", COMPRESS_OPTION},
            {Parameters.PROPERTY_BIG_FILE_FILTER_LIMIT, "Big-File Filter/Tree limit (thousands)", null, EditPropertiesPnl.FLD_INT, null},
            {Parameters.PROPERTY_BIG_FILE_DISK_FLAG, "Force Storing chunks on Disk (Used in Testing)", null, EditPropertiesPnl.FLD_LIST, "Storing chunks on Disk", YNT_OPTION2},
            {Parameters.PROPERTY_USE_OVERFLOW_FILE, "When space is tight, Try to store data in an overflow file ", null, EditPropertiesPnl.FLD_BOOLEAN, "Use Overflow file"},
            {Parameters.PROPERTY_BIG_FILE_USE_SPECIAL_FIXED_MODEL, "Use Fixed Length Model (Used in Testing)", null, EditPropertiesPnl.FLD_BOOLEAN, "Use Fixed Length Model"}, // Checked
            {Parameters.PROPERTY_BIG_FILE_LARGE_VB, "Use Large VB Model (Default Yes)", null, EditPropertiesPnl.FLD_LIST, "Use Large VB Model", YNT_OPTION}, // Checked
            {Parameters.PROPERTY_LOAD_FILE_BACKGROUND, "Load File in Background thread", null, EditPropertiesPnl.FLD_BOOLEAN, "Load In background"}, // Checked
    };
    
    private String cobolOptDescription
    = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_BigFiles",

      "<h2>Cobol / JRecord Option</h2>"
    + "This panel holds Cobol options + Java~JRecord Code Generation options"
    + "<p>In the package id you can use ${schema} and ${template}.");

    private final String[][] TEMPLATE_OPTIONS = {{"standard", "standard",}, {"lineWrapper", "lineWrapper"}, {"stdPojo", "stdPojo"}, {"basic", "basic"}};
//TODO
//TODO
    private Object[][] cobolOptions = {
    		{Parameters.DEFAULT_COBOL_DIALECT, "Default Cobol Dialect", null, EditPropertiesPnl.FLD_LIST, "Cobol Dialect", toList( cobolDialectOptMdl), 
    				Parameters.DEFAULT_COBOL_DIALECT_NAME},
    		defaultCobolDirectory,
    		{"", "", null, EditPropertiesPnl.FLD_EMPTY, null},
            {Parameters.CODEGEN_TEMPLATE, "The Default Code Gen ", null, EditPropertiesPnl.FLD_LIST, "JRecord CodeGen Template", TEMPLATE_OPTIONS},
    		codeGenDirecory,
    		{Parameters.TEMPLATE_DIRECTORY,   "User CodeGen Template directory", null, EditPropertiesPnl.FLD_DIR, null},
            {Parameters.CODEGEN_PACKAGEID,
    				"<html><body>Default CodeGen package-id (when nothing is entered."
            		+ " You can use <b>${schema}</b> and <b>${template}</b></body></html>", 
            		null, EditPropertiesPnl.FLD_TEXT, "Default CodeGen package-id"},
            {Parameters.CODEGEN_PACKAGE_DEF, 
            			"<html><body>Several JRecord 0.90 packages use <b>def</b> in there name.<br/>"
            			+ "This conflicts with Jython <b>def</b> statment.<br/>"
            			+ "Future versions of JRecord may change <b>def</b> to something else.</body></html>", 
            			null, EditPropertiesPnl.FLD_TEXT, "JRecord definition id"},
    };
    private EditPropertiesPnl directoryPnl1
        = new EditPropertiesPnl(params, directoryDescription, directoryParams1);
    private EditPropertiesPnl directoryPnl2
        = new EditPropertiesPnl(params, directoryDescription, directoryParams2);
    private EditPropertiesPnl testPnl
        = new EditPropertiesPnl(params, testDescription, testParams);
    private EditPropertiesPnl behaviourPnl;
//         = new EditPropertiesPnl(params, behaviourDescription, behaviourParams);
    private EditPropertiesPnl readWritePnl
         = new EditPropertiesPnl(params, readWriteDescription, readWriteParams);
    private EditPropertiesPnl file2Pnl
    	= new EditPropertiesPnl(params, fileDescription, fileParams);
   private EditPropertiesPnl layoutWizardPnl
        = new EditPropertiesPnl(params, layoutWizardParamsDescription, layoutWizardParams);
    private EditPropertiesPnl bigModelPnl
        = new EditPropertiesPnl(params, bigModelDescription, bigModelParams);
    private EditPropertiesPnl cobolOptPnl
    	= new EditPropertiesPnl(params, cobolOptDescription, cobolOptions);
    
    private String specialCharsetDescription
    	= LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_BigFiles",

		      "<h2>Special Character Sets</h2>"
		    + "There are several character sets that the <b>RecordEditor</b> needs to know:<ul>"
		    + "<li>Binary fields only work correctly in single byte character-sets,<br/>"
		    +  "So if the default character-set is multi-byte (say utf-8);<br/>"
		    +  "The <b>RecordEditor</b> needs to know what character-set to use<br/>"
		    +  "CP1252 is often a good choice."
		    +  "<li>If you transfer files from the mainframe to a PC/*nix machine<br>"
		    +  "the <b>RecordEditor</b> needs to know the original mainframe character-set<br>"
		    +  "to correctly translate Zoned decimal (Cobol s999 fields)."
		    + "</ul"
    
    );

    private Object[][] specialCharsetParams = {
            {Parameters.PROPERTY_DEFAULT_SINGLE_BYTE_CHARSET, "Single byte charater-set to be used for binary files when no character set is suppplied", null, EditPropertiesPnl.FLD_SINGLE_BYTE_CHARSET, "Default Single byte character set"},
            {Parameters.PROPERTY_USE_SINGLE_BYTE_CHARSET,     "Controls wether default Characterset (above) is used when font=\"\" for all Layout's or only binary layouts", null, EditPropertiesPnl.FLD_BOOLEAN, "Alway use above charset when font=\"\""},
            {Parameters.PROPERTY_DEFAULT_EBCDIC_CHARSET, "Default EBCDIC characterset",null, EditPropertiesPnl.FLD_CHARSET, "Default Ebcdic Character set"}
    };
    private Object[][] specialCharsetParams1 = {specialCharsetParams[2]};
   
//    private EditPropertiesPnl specialCharsetPnl
//    	= new EditPropertiesPnl(params, specialCharsetDescription, specialCharsetParams);
//

    private String xsltDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_Xslt",

          "<h2>Xslt Properties</h2>"
        + "This panels let you specify XSLT related jars and the XSLT transform class<br>"
        + "For Saxon or Xalan, you can just enter Saxon or Xalan, Or you can enter<pre>"
        + "        net.sf.saxon.TransformerFactoryImpl\n"
        + "    or  org.apache.xalan.processor.TransformerFactoryImpl</pre>");

    private Object[][] xsltParams = {

            {Parameters.XSLT_ENGINE, "Xslt Transform factory class", null, EditPropertiesPnl.FLD_TEXT, null},
    };
    private EditPropertiesPnl xsltPnl
        = new EditPropertiesPnl(params, xsltDescription, xsltParams);

    private EditJarsPanel xsltJarsPnl = new EditJarsPanel(
            params,   "EditProps_XsltJars",
            "<h2>Xslt Jars</h2>This panel lets you specify your XSLT jars",
            params.xsltJars,
            "xslt",
            true);


    private String csvDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_CsvParams",
          "<h2>Csv Editor Properties</h2>"
        + "This panels holds Csv-Editor specific options<br>");

    private Object[][] csvParams = {

            {Parameters.CSV_LOOK_4_FIXED_WIDTH, "Check for binary Fixed Width", null, EditPropertiesPnl.FLD_BOOLEAN, "Should the program check for binary Fixed Width Files ???"},
    };
    private EditPropertiesPnl csvPnl
        = new EditPropertiesPnl(params, csvDescription, csvParams);

    private String poDescription
    	= LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_CsvParams",
    			"<h2>GetText-Po Properties</h2>"
    		  + "This panels holds GetText-PO editor specific options<br>");

    String[][] poChildPosOptions = {{Parameters.CHILD_SCREEN_RIGHT, "Right"}, {Parameters.CHILD_SCREEN_BOTTOM, "Bottom"},};
    private Object[][] poParams = {
            {Parameters.PO_EDIT_CHILD_SCREEN_POS, "Position of Child Screen", null, EditPropertiesPnl.FLD_LIST, null, poChildPosOptions},
            {Parameters.PO_EDIT_OPEN_FUZZY_VIEW, "Open Fuzzy / Blank View", null, EditPropertiesPnl.FLD_BOOLEAN, "Open a view containing Just the Fuzzy / Blank translations"},
    };


    private EditPropertiesPnl poPnl
    	= new EditPropertiesPnl(params, poDescription, poParams);

    private EditJdbcParamsPanel jdbcParamsPnl = new EditJdbcParamsPanel(params, params.jdbcJars);

    private EditJarsPanel jdbcPnl = new EditJarsPanel(
            params,
            "EditProps_JdbcJars",
            "<h2>JDBC Jars</h2>This panel lets you change the "
          + "JDBC (Java Database Conectivity) Jars that are needed by "
          + "the <b>RecordEditor</b>."
          + "<br><br>If you click on a row in the table, the fields at the bottom "
          + "will be updated with the values from the selected row.<br>"
          + "You can update values either using the fields at the bottom "
          + "of the screen or directly into the table itself.",
          params.jdbcJars,
            "jdbc.",
            true);
    private EditJarsPanel systemPnl = new EditJarsPanel(
            params, "EditProps_Jars",
            "<h2>System Jars</h2>This panel lets you change the "
          + "Jars supplied with the <b>RecordEditor</b> like cb2xml."
          + "<br>These jars are used by the <b>RecordEditor</b> but are written "
          + "and maintained by other people",
            params.systemJars,
            "",
            false);
    private EditJarsPanel optionalPnl = new EditJarsPanel(
            params, "EditProps_OptionalJars",
            "<h2>Optional Jars</h2>This panel lets you specify your own jars. <br/>"
          + "This could include User code or scripting languages (JRuby.jar, Jython.jar etc).",
            params.optionalJars,
            "optional.",
            true
            );
    private EditJarsPanel userPnl = new EditJarsPanel(
            params, "EditProps_UserJars",
            "<h2>User Jars</h2>This panel lets you specify your own jars "
          + "<br>These jars will be used when the <b>RecordEditor</b starts. "
          + "<br>When adding your own jars, <br>you should also invoke your own "
          + "initialisation class via the <i>UserInitilizeClass</i> property "
          + "on the other screen.",
            params.userJars,
            "User.",
            true
            );

    private static String jarsNote = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_JarsNote",

       "<b>Note:</b>You will also need to add the jar with your class "
    + "to the <b>User Jars</b> tab.");

    private static String loaderDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_CopyBook_Loaders",
          "<h2>Copybook Loaders</h2>"
        + "This tab is for defining user written <b>Copybook Loader's</b> to the <b>RecordEditor</b>."
        + "<br>A <b>Copybook Loader's</b> is a Java Class that can load "
        + "a copybook from a file into the <b>RecordEditor</b>."
        + "<br><br><b>Copybook Loader's</b> are used in <ul>"
        + "<li>Load Copybook function in the <b>LayoutEditor</b>"
        + "<li>Cobol Editor</ul><br>")
        + jarsNote;

    private static String pluginDescription
        =  LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_Plugin",
          "<h2>Plugins</h2>"
        + "This tab is for defining user written <b>Functions</b> that will listed on the <br/><b>Plugin</b> "
        + "Drop down menu in the   <b>RecordEditor</b>.<br> "
        + "Local Functions must implement the Plugin interface");

    private static String optionDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_DefaultOpts",
          "<h2>Default Options</h2>"
        + "This Tab is for defining the default value for several Combo Box's used by the Package"
        + "<br/><br/>Click on a row to change the default Value");

    private static final String[] LOADER_COLUMN_HEADINGS = {"Loader Name", "Loader Class"};
    private static final String[] LOADER_COLUMN_NAMES = {
            Parameters.PROPERTY_COPYBOOK_NAME_PREFIX, Parameters.PROPERTY_COPYBOOK_CLASS_PREFIX
    };
    private static final String[] PLUGIN_COLUMN_HEADINGS = {"Name", "Class Name", "Parameter"};
    private static final String[] PLUGIN_COLUMN_NAMES = {
        Parameters.PROPERTY_PLUGIN_FUNC_NAME,
        Parameters.PROPERTY_PLUGIN_FUNC_CLASS,
        Parameters.PROPERTY_PLUGIN_FUNC_PARAM
    };

    private static String typeDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_TypeDef",

          "<h2>Type Definition</h2>"
        + "This tab for defining user written <b>Type's</b> to the <b>RecordEditor</b>."
        + "<br>A <b>Type's</b> is a Java Class that converts a fields "
        + "between the external representation and the internal Java Representation."
        + "<br>Columns in the Table are <br><table>"
        + "<tr><td>Type Number</td><td>Unique number used to identify the type. "
        + "It should be between {0} and {1}</td></tr>"
        + "<tr><td>Type Name</td><td>Name of the Type</td></tr>"
        + "<tr><td>Type Class</td><td>Java Type class.</td></tr>"
        + "<tr><td>Format Class</td><td>Class that implements table cell Formating. Leave blank if there is not one</td></tr>"
        + "</table><br>"
        + jarsNote,

        new Object[] {
             Type.USER_RANGE_START,
             (Type.USER_RANGE_START + Type.DEFAULT_USER_RANGE_SIZE - 1)
        });
    private static String formatDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_Format",
              "<h2>Format Definition</h2>"
            + "This tab for defining user written <b>Format's</b> to the <b>RecordEditor</b>."
            + "<br><b>Format's</b> define TableCellRenders and TableCellEditors for a field."
            + "<br><br>You may want to use Format to edit Dates using a Date Popup");


    private static final String[] TYPE_COLUMN_HEADINGS = {"Type Number", "Type Name", "Type Class", "Format Class"};
    private static final String[] TYPE_COLUMN_NAMES = {
            Parameters.TYPE_NUMBER_PREFIX, Parameters.TYPE_NAME_PREFIX,
            Parameters.TYPE_CLASS_PREFIX, Parameters.TYPE_FORMAT_PREFIX
    };

    private static final String[] FORMAT_COLUMN_HEADINGS = {"Format Number", "Format Name", "Format Class"};
    private static final String[] FORMAT_COLUMN_NAMES = {
            Parameters.FORMAT_NUMBER_PREFIX, Parameters.FORMAT_NAME_PREFIX,
            Parameters.FORMAT_CLASS_PREFIX
    };

    private EditPropertiesTblPanel loadersPnl
        = new  EditPropertiesTblPanel(params, loaderDescription,
               LOADER_COLUMN_NAMES, LOADER_COLUMN_HEADINGS,
               Parameters.NUMBER_OF_LOADERS);
    private EditPropertiesTblPanel pluginPnl
        = new EditPropertiesTblPanel(params, pluginDescription,
              PLUGIN_COLUMN_NAMES, PLUGIN_COLUMN_HEADINGS,
              Parameters.NUMBER_OF_USER_FUNCTIONS);
   private EditPropertiesTblPanel typePnl
        = new EditPropertiesTblPanel(params, typeDescription,
              TYPE_COLUMN_NAMES, TYPE_COLUMN_HEADINGS,
              Parameters.NUMBER_OF_TYPES);
    private EditPropertiesTblPanel formatPnl
        = new EditPropertiesTblPanel(params, formatDescription,
                FORMAT_COLUMN_NAMES, FORMAT_COLUMN_HEADINGS,
                Parameters.NUMBER_OF_FORMATS);

    private static String[][] defaultDetails = {
            {Parameters.DEFAULT_COPYBOOK_READER,	"The default copybook reader"},
            {Parameters.DEFAULT_COPYBOOK_WRITER,	"The default copybook writer"},
            {Parameters.DEFAULT_DATABASE,			"The default Database to use"},
            {Parameters.DEFAULT_IO,					"The default IO Routine to use"},
            {Parameters.DEFAULT_COBOL_DIALECT_NAME,	"The default Cobol Dialect",    Parameters.DEFAULT_COBOL_DIALECT},
    };

    private String screenLocationDescription
        = LangConversion.convertId(LangConversion.ST_MESSAGE, "EditProps_ScreenLocation",

          "<h2>Program initial Size</h2>"
        + "This screen controls the program size when it inially opens. "
        + "The options are <b>Maximised</b>, <b>Last Size</b>,<br/>"
        + "<b>Space around Edges</b> and <b>Initial Size</b>.<br/>"
        + "The <b>retrieve screen size</b> button at the bottom of the screen."
        + "retrieves the current program Size.");
//    private String[][] screenLocationParams = {
//            {"spaceAtBottomOfScreen", "Space to be left at the bottom of the screen.", null},
//            {"spaceAtTopOfScreen", "Space to be left at the top of the screen.", null},
//            {"spaceAtLeftOfScreen", "Space to be left at the left of the screen.", null},
//            {"spaceAtRightOfScreen", "Space to be left at the Right of the screen.", null},
//    };

    private String applId = getApplId();


    private static final String[][] SIZE_OPTION = {
            {String.valueOf(ProgramOptions.SIZE_MAXIMISED), "Maximized"},
            {String.valueOf(ProgramOptions.SIZE_LAST), "Last Screen Size"},
            {String.valueOf(ProgramOptions.SIZE_LAST_FORCED), "Last Screen Size (force)"},
            {String.valueOf(ProgramOptions.SIZE_SPACE_AROUND), "Use Space around parameters"},
            {String.valueOf(ProgramOptions.SIZE_SPECIFIED), "Height, Width Below"},
            {String.valueOf(ProgramOptions.SIZE_SPECIFIED_FORCED), "Height, Width Below (force)"},
            {String.valueOf(ProgramOptions.SIZE_NO_RESIZE), "No Resize, whatever java chooses"},
            {String.valueOf(ProgramOptions.SIZE_SCREEN_1), "Screen 1"},
            {String.valueOf(ProgramOptions.SIZE_SCREEN_2), "Screen 2"},
            {String.valueOf(ProgramOptions.SIZE_SCREEN_3), "Screen 3"},
 //           {String.valueOf(ProgramOptions.SIZE_SCREEN_1_MAX), "Screen 1 (maximized)"},
   };
    
    static {
    	try {
			GraphicsDevice[] gd = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
			int en = Math.min(3, gd.length);
			for (int i = 0; i <= en; i++) {
				Rectangle bounds = gd[i].getDefaultConfiguration().getBounds();
				SIZE_OPTION[SIZE_OPTION.length - 3 + i][1] = "Screen " + (i+1) 
						+ " " + bounds.width + "*" + bounds.height + " " + gd[i].getIDstring();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
    }

    private Object[][] screenLocationParams = {
            {Parameters.SCREEN_SIZE_OPTION, "Program Initial Size", null, EditPropertiesPnl.FLD_LIST,  "Size of the program when it opens", SIZE_OPTION},
            {"spaceAtBottomOfScreen", "Space to be left at the bottom of the screen.", null, EditPropertiesPnl.FLD_INT, null},
            {"spaceAtTopOfScreen",    "Space to be left at the top of the screen.",    null, EditPropertiesPnl.FLD_INT, null},
            {"spaceAtLeftOfScreen",   "Space to be left at the left of the screen.",   null, EditPropertiesPnl.FLD_INT, null},
            {"spaceAtRightOfScreen",  "Space to be left at the Right of the screen.",  null, EditPropertiesPnl.FLD_INT, null},
            {"", "", null, EditPropertiesPnl.FLD_EMPTY, null},
            {applId + Parameters.SCREEN_START_HEIGHT, "Screen Height", null, EditPropertiesPnl.FLD_INT, null},
            {applId + Parameters.SCREEN_START_WIDTH,  "Screen Width",  null, EditPropertiesPnl.FLD_INT, null},
            {"", "", null, EditPropertiesPnl.FLD_RETRIEVE_SIZE, null}
    };

    private EditPropertiesPnl screenPosPnl
    = new EditPropertiesPnl(params, screenLocationDescription, screenLocationParams);

    private final static EnglishStrModel cobolDialectOptMdl = DefaultOptModel.newModel(ConversionManager.getInstance());
    private static  EnglishStrModel[] defaultModels = {// new EnglishStrModel[5];
         DefaultOptModel.newModel(CopybookLoaderFactoryDB.getInstance()),
         DefaultOptModel.newModel(CopybookWriterManager.getInstance()),
         DefaultOptModel.newModel( Common.getSourceId()),
         DefaultOptModel.newModel(LineIOProvider.getInstance()),
         cobolDialectOptMdl
    };
//    static {
////        CopybookLoaderFactory loaders = CopybookLoaderFactoryDB.getInstance();
////        DefaultComboBoxModel mdl = new DefaultComboBoxModel();
////        int i;
////        for (i = 0; i < loaders.getNumberOfEntries(); i++) {
////            mdl.addElement(loaders.getName(i));
////        }
//
//        defaultModels[0] =  DefaultOptModel.newModel(CopybookLoaderFactoryDB.getInstance());
//
////        CopybookWriterManager manager = CopybookWriterManager.getInstance();
////        String s;
////        mdl = new DefaultComboBoxModel();
////        for (i = 0; i < manager.getNumberOfEntries(); i++) {
////            s = manager.getName(i);
////            if (s != null && ! "".equals(s)) {
////                mdl.addElement(s);
////            }
////        }
//        defaultModels[1] = DefaultOptModel.newModel(CopybookWriterManager.getInstance());
//
////        String[] ids = Common.getSourceId();
////        mdl = new DefaultComboBoxModel();
////
////        for (i = 0; i < ids.length; i++) {
////            if (ids[i] != null && ! "".equals(ids[i])) {
////                mdl.addElement(ids[i]);
////            }
////        }
//        defaultModels[2] = DefaultOptModel.newModel( Common.getSourceId());
//
//        defaultModels[3] = DefaultOptModel.newModel(LineIOProvider.getInstance());
//        defaultModels[4] = DefaultOptModel.newModel(ConversionManager.getInstance());
//    }


//    private static ComboBoxModel getManagerModel(AbstractManager manager) {
//        String s;
//        DefaultComboBoxModel mdl = new DefaultComboBoxModel();
//        for (int i = 0; i < manager.getNumberOfEntries(); i++) {
//            s = manager.getName(i);
//            if (s != null && ! "".equals(s)) {
//                mdl.addElement(s);
//            }
//        }
//        return mdl;
//    }

    /**
     * @param theDefaultDetails the defaultDetails to set
     */
    public static final void setDefaultDetails(
            String[][] theDefaultDetails, EnglishStrModel[] theDefaultModels) {

        defaultDetails = theDefaultDetails;
        defaultModels = theDefaultModels;
    }

//	/**
//     * Edit the record Editor Parameter and Jar files
//     */
//    public EditOptions() {
//        this(true, true, true, true);
//    }

    /**
     * @param terminateOnExit terminate java on exit
     *
     * Edit the record Editor Parameter and Jar files
     *
     */
    public EditOptions(final boolean terminateOnExit, boolean includeJDBC, boolean includeWizardOptions) {
        this(terminateOnExit, includeJDBC, includeWizardOptions, true);
    }


    private EditPropertiesPnl[] propertiesPnl; /* = {
    		 directoryPnl1,
    		 directoryPnl2,
    		 testPnl,
    		 behaviourPnl,
    		 readWritePnl,
    		 file2Pnl,
    		 layoutWizardPnl,
    		 bigModelPnl,
    		 xsltPnl,
    		 csvPnl,
    		 poPnl,
    		 screenPosPnl,
    };*/

    /**
     * @param terminateOnExit terminate java on exit
     *
     * Edit the record Editor Parameter and Jar files
     *
     */
    public EditOptions(final boolean terminateOnExit, boolean includeJDBC, boolean includeWizardOptions, boolean display) {
        super();

        init_100_ScreenFields(terminateOnExit);
        init_200_Screen(includeJDBC, includeWizardOptions);
        if (display) {
            displayScreen();
        }
    }



    /**
     * Initialise screen fields
     *
     * @param terminateOnExit terminate jave on exit from class
     */
    private void init_100_ScreenFields(boolean terminateOnExit) {
    	
    	if (Common.OPTIONS.programType == ProgramOptions.ProgramType.CSV_EDITOR) {
    		System.arraycopy(behaviourParamsCsv, 0, behaviourParams, 
    				behaviourParams.length - behaviourParamsCsv.length, behaviourParamsCsv.length);
    	}
    	
    	behaviourPnl = new EditPropertiesPnl(params, behaviourDescription, behaviourParams);
    	propertiesPnl = new EditPropertiesPnl[] {
       		 directoryPnl1,
       		 directoryPnl2,
       		 testPnl,
       		 behaviourPnl,
       		 readWritePnl,
       		 file2Pnl,
       		 layoutWizardPnl,
       		 bigModelPnl,
       		 xsltPnl,
       		 csvPnl,
       		 poPnl,
       		 screenPosPnl,
       };

       saveBtn.putValue(AbstractAction.SHORT_DESCRIPTION, LangConversion.convert(LangConversion.ST_MESSAGE, "Save ..."));

       programDescription = new JEditorPane("text/html", description);
       xsltJarsPnl.setInitialValues();
       jdbcPnl.setInitialValues();
       systemPnl.setInitialValues();
       optionalPnl.setInitialValues();
       userPnl.setInitialValues();

       jdbcParamsPnl.buildJarCombo();

       mainTabbed.setTabPlacement(JTabbedPane.LEFT);

       if (terminateOnExit) {
           frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
       } else {
           frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
       }

       frame.addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent e) {
        	   save();
 //              params.writeProperties();
 //              params.writeJarFiles();
           	   for (EditPropertiesPnl p : propertiesPnl) {
            	   p.clear();  
               }
               
               for (SaveScreens a : screensToCheck) {
            	   if (a.options.isCloseRequired()) {
            		   mainTabbed.setSelectedComponent(a.tabbedPane);
            		   a.tabbedPane.setSelectedComponent(a.tabPnl);
            		   a.options.close();
            	   }
               }

           }
       });
    }

    /**
     * Setup the screen
     *
     */
    private void init_200_Screen(boolean includeJDBC, boolean includeWizardOptions) {
        JPanel pnl = new JPanel();
        JPanel topPnl = new JPanel();
        JToolBar toolBar = new JToolBar();

        String[][] defaultDescription = defaultDetails.clone();

        for (int i = 0; i < defaultDescription.length; i++) {
        	defaultDescription[i][1] = LangConversion.convert(LangConversion.ST_MESSAGE, defaultDescription[i][1]);
        }

        toolBar.add(saveBtn);
        

        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Directories", directoryPnl1);
        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Save Directories", directoryPnl2);
        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Test", testPnl);
        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Behaviour", behaviourPnl);
        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Read/Write", readWritePnl);
        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","File Options", file2Pnl);

        if (includeWizardOptions) {
            SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Layout Wizard", layoutWizardPnl);
        }
        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Big Model", bigModelPnl);

        SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_Properties","Defaults",
                new EditDefaults(params, optionDescription,
                		defaultDescription, defaultModels
        ));
        
        Object[][] scParams = specialCharsetParams1;
        if (Conversion.DEFAULT_CHARSET_DETAILS.isMultiByte) {
        	scParams = specialCharsetParams;
        }
        //TODO
        SwingUtilsBldr.addTab(cobolTabbed, "Cobol_Opts", "Cobol/JRecord", cobolOptPnl);

        addTab(csvTabbed, "EditOpts_EditDelims", "Edit Csv Field Delimiters", editDelims.panel, editDelims);
        addTab(csvTabbed, "EditOpts_EditQuotes", "Edit Csv Quote", editQuotes.panel, editQuotes);
        if (Common.CSV_PROGRAM_ID.equals(getApplId())) {
        	SwingUtilsBldr.addTab(csvTabbed, "EditOpts_CsvOptions", "Csv Options", csvPnl);
        }

    
  
        SwingUtilsBldr.addTab(textTabbed, "EditOpts_EditColors", "Field Colors", EditColors.getFieldColorEditor(frame, params));
        SwingUtilsBldr.addTab(textTabbed, "EditOpts_EditColors", "Special Colors", EditColors.getSpecialColorEditor(frame, params));
        SwingUtilsBldr.addTab(textTabbed, "Special_Chars", "Special Fonts", new EditPropertiesPnl(params, specialCharsetDescription, scParams));
      
        SwingUtilsBldr.addTab(xmlTabbed, "EditOpts_Xml","Xslt Options", xsltPnl);
        SwingUtilsBldr.addTab(xmlTabbed, "EditOpts_Xml","Xslt Jars", xsltJarsPnl);

        SwingUtilsBldr.addTab(jarsTabbed, "EditOpts_Jars","System Jars", systemPnl);
        SwingUtilsBldr.addTab(jarsTabbed, "EditOpts_Jars","Optional Jars", optionalPnl);

        SwingUtilsBldr.addTab(userTabbed, "EditOpts_User","User Jars", userPnl);
        SwingUtilsBldr.addTab(userTabbed, "EditOpts_User","Copybook Loaders", loadersPnl);
        SwingUtilsBldr.addTab(userTabbed, "EditOpts_User","Plugins", pluginPnl);
        SwingUtilsBldr.addTab(userTabbed, "EditOpts_User","User Types", typePnl);
        SwingUtilsBldr.addTab(userTabbed, "EditOpts_User","User Formats", formatPnl);
        if (includeJDBC) {
            SwingUtilsBldr.addTab(propertiesTabbed, "EditOpts_User","Date Types", new EditDateTypes(params));
        }

        SwingUtilsBldr.addTab(looksTabbed, "EditOpts_User","Look and Feel", new LooksPanel(params));
        SwingUtilsBldr.addTab(looksTabbed, "EditOpts_User","Icons", new EditIcons(params));
        SwingUtilsBldr.addTab(looksTabbed, "EditOpts_User","Initial Program Size", screenPosPnl);

        addMainTab("Description", init_310_Screen());
        addMainTab("Properties", propertiesTabbed);

        System.out.println("Application Id: " + getApplId());
        addMainTab("Csv", csvTabbed);
//        if (Common.CSV_PROGRAM_ID.equals(getApplId())) {
//            addMainTab("Csv Options", csvPnl);
//        }
        if (Common.OPTIONS.getTextPoPresent.isSelected()) {
            addMainTab("Special Formats", poPnl);
        }
//poPnl
        addMainTab("Cobol/JRecord", cobolTabbed);
        addMainTab("Text", textTabbed);
       
        addMainTab("Xml", xmlTabbed);
        if (includeJDBC) {
            SwingUtilsBldr.addTab(jdbcTabbed, "EditOpts_JDBC","JDBC Jars", jdbcPnl);
            SwingUtilsBldr.addTab(jdbcTabbed, "EditOpts_JDBC","JDBC Properties", jdbcParamsPnl);
            addMainTab("JDBC Parameters", jdbcTabbed);
        }
        addMainTab("Language", new EditLanguage(params));
        addMainTab("Jars", jarsTabbed);
        addMainTab("Extensions", userTabbed);
        addMainTab("Looks", looksTabbed);

        topPnl.setLayout(new BorderLayout());
        topPnl.add("North", toolBar);
        //topPnl.add("Center", tips);
        topPnl.add("South", new JPanel());
        pnl.setLayout(new BorderLayout());
        pnl.add("North", topPnl);
        pnl.add("Center", mainTabbed);
        pnl.add("South", params.msgFld);
        frame.getContentPane().add(pnl);

//        printTabDetails("mainTabbed", mainTabbed);
//        printTabDetails("propertiesTabbed", propertiesTabbed);
//        printTabDetails("jarsTabbed", jarsTabbed);
//        printTabDetails("userTabbed", userTabbed);
//        printTabDetails("looksTabbed", looksTabbed);
    }


    private void addMainTab(String name, JComponent item) {
        item.setName(name + "Tab");
        SwingUtilsBldr.addTab(mainTabbed, "EdProps_MainTab_" + name, name, item);
    }
//    private void printTabDetails(String name, JTabbedPane tab) {
//
//    	System.out.println(name + " " + tab.getTabCount());
//    	for (int i = 0; i < tab.getTabCount(); i++) {
//    		Component c = tab.getComponentAt(i);
//    		if (c != null) {
//	    		System.out.println(name + " " + i
//	    				+ "\t" + c.getPreferredSize().height
//	    				+ "\t" + c.getClass().getName());
//    		} else {
//    			System.out.print('.');
//    		}
//    	}
//
//    }

    private void save() {
        params.writeProperties();
        params.writeJarFiles();

//    	for (EditPropertiesPnl p : propertiesPnl) {
//    		p.save();
//    	}
    }

    public EditParams getParams() {
        return params;
    }

    public final void displayScreen() {

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        frame.pack();
        frame.setBounds(
                frame.getY(), frame.getX(),
                Math.min(frame.getWidth(),
                         screenSize.width  - Common.getSpaceAtRightOfScreen()
                                            - frame.getY()),
                Math.min(frame.getHeight(),
                         screenSize.height - Common.getSpaceAtBottomOfScreen()
                           - SwingUtils.values().NORMAL_FIELD_HEIGHT * 2
                            - frame.getX()));
        frame.setVisible(true);
    }

    /**
     * Add a component to the startup properties editor
     * @param name Tab name
     * @param component component to add
     */
    public void add(String name, JComponent component) {
        SwingUtilsBldr.addTab(mainTabbed, "EditOptions_MainTab", name, component);
    }

    /**
     * Build Program description panel
     * @return Program description panel
     */
    private BasePanel init_310_Screen() {
        BasePanel pnl = new BasePanel();

        pnl.addComponentRE(1, 5, PROGRAM_DESCRIPTION_HEIGHT, BasePanel.GAP1,
                BasePanel.FULL, BasePanel.FULL,
                new JScrollPane(programDescription));

        return pnl;
    }

    public void addTab(JTabbedPane tab, String tabId, String name, JComponent tabComponenet, Object item) {
    	SwingUtilsBldr.addTab(tab, tabId, name, tabComponenet);
    	if (item instanceof ISaveOptions) {
    		screensToCheck.add(new SaveScreens(tab, tabComponenet, (ISaveOptions) item));
    	}
    }

    private static String getApplId() {
        String ret = "";
        ReMainFrame f = ReMainFrame.getMasterFrame();
        if (f != null) {
            ret = f.getApplicationId();
        }

        return ret;
    }
    
    private static String[][] toList(EnglishStrModel cobolDialectOptMdl) {
    	String[][] r = new String[cobolDialectOptMdl.getSize()][]; 
    	for (int i=0; i < r.length; i++) {
    		ComboStrOption e = (ComboStrOption) cobolDialectOptMdl.getElementAt(i); 
    		r[i] = new String[2];
    		r[i][0] = e.key;
    		r[i][1] = e.toString();
    	}
    	return r;
    }

//    /**
//     *  Run the options editor
//     * @param args program arguments
//     */
//    public static void main(String[] args) {
//
//     	boolean jdbc = ! (args != null && args.length > 0 && "nojdbc".equalsIgnoreCase(args[args.length-1]));
//
//        new EditOptions(true, jdbc, true);
//    }
    
    private static class SaveScreens {
    	final JTabbedPane tabbedPane;
    	final JComponent tabPnl;
    	final ISaveOptions options;
    	
    	
		protected SaveScreens(JTabbedPane tabbedPane, JComponent tabPnl,
				ISaveOptions options) {
			super();
			this.tabbedPane = tabbedPane;
			this.tabPnl = tabPnl;
			this.options = options;
		}
    	
    	
    }
}
