package net.sf.RecordEditor.re.fileWriter;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.ArrayList;

import net.sf.JRecord.Common.IFieldDetail;
import net.sf.JRecord.CsvParser.BasicCsvLineParserExtended;
import net.sf.JRecord.CsvParser.CsvDefinition;
import net.sf.JRecord.External.base.ExternalConversion;

public class CsvRecordWriter extends BaseWriter {

	private BufferedWriter fileWriter;
	private String fieldSep;
	public final String quote;
	private final int columnCount;
	private int lineNumber = 0;
	private final BasicCsvLineParserExtended csvParser = BasicCsvLineParserExtended.getInstance();
	private final CsvDefinition csvDef;
	
	private final boolean includeLineNumber, blankLineBetweenRecords, showHex;

	public CsvRecordWriter(
					OutputStream outStream, String delimiter,
					String fontName, String quoteStr, 
					boolean includeLineNumber, boolean blankLineBetweenRecords,
					boolean showHex
					) throws IOException {

		this.quote = quoteStr;
		this.includeLineNumber = includeLineNumber;
		this.blankLineBetweenRecords = blankLineBetweenRecords;
		this.showHex = showHex;
		this.csvDef = new CsvDefinition(delimiter, quoteStr);
		
		int sz = 4096 * 4;
		if (fontName == null || "".equals(fontName)) {
			fileWriter = new BufferedWriter(new OutputStreamWriter(outStream), sz);
		} else {
			fileWriter = new BufferedWriter(new OutputStreamWriter(outStream, fontName), sz);
		}
		fieldSep = delimiter;

		if ("<tab>".equalsIgnoreCase(delimiter)) {
			fieldSep = "\t";
		} else if ("<space>".equalsIgnoreCase(delimiter)) {
			fieldSep = " ";
		}
		
		String lineNumberHeader= "", hexHeading="";
		
		int colCount = 6; 
		if (includeLineNumber) {
			lineNumberHeader = "LineNumber" + fieldSep;
			colCount += 1;
		}
		if (showHex) {
			hexHeading =  fieldSep + "Hex";
			colCount += 1;
		}
		columnCount = colCount;

		fileWriter.write(
				  lineNumberHeader	
				+ "FieldName" + fieldSep
				+ "Position"  + fieldSep
				+ "Length"    + fieldSep
				+ "Type"      + fieldSep
				+ "Value"     + fieldSep
				+ "Text"     
				+ hexHeading
		);
		fileWriter.newLine();
	}

	@Override
	public void newLine() throws IOException {
		if (blankLineBetweenRecords && lineNumber > 0) {
			fileWriter.newLine();
		}
		lineNumber+=1;
	}

	public void writeFieldDetails(IFieldDetail fldDetail, String fieldValue,
			String textValue, String hexValue) throws IOException {
		ArrayList<String> cols = new ArrayList<String>(columnCount);
		int len = fldDetail.getLen();
		
		if (includeLineNumber) {
			cols.add(Integer.toString(lineNumber));
		}
		
		cols.add(fldDetail.getName());
		cols.add(Integer.toString(fldDetail.getPos()));
		cols.add(len < 0 ? "" : Integer.toString(len));
		cols.add(ExternalConversion.getTypeAsString(0, fldDetail.getType()));
		cols.add(fieldValue);
		cols.add(textValue);
		if (showHex) {
			cols.add(hexValue);
		}
		
		fileWriter.write(csvParser.formatFieldList(cols, csvDef, null)); 
		fileWriter.newLine();
	};

	@Override
	public void close() throws IOException {
		fileWriter.close();
	}
	
	@Override public void writeFieldHeading(String field) throws IOException { }
	@Override public void writeField(String field) throws IOException {	}

}
