package net.sf.RecordEditor.re.firstTime.wizard;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public abstract class BaseWizPnl implements AbstractWizardPanel<InitialSetupData> {

	BaseHelpPanel pnl;
	JSplitPane splitPane;
	private int rightWidth = 0;
	
//	private final String helpName;
	
	BaseWizPnl(String id, String helpName) {
		pnl = new BaseHelpPanel(id);
//		this.helpName = helpName;
	}

	@Override
	public boolean skip() {
		return false;
	}

	@Override
	public final JComponent getComponent() {
		return splitPane == null ? pnl : splitPane;
	}

	@Override
	public final void showHelpRE() {
		// TODO Auto-generated method stub

	}

	public void setRightComponent(Component comp) {
		if (splitPane == null) {
			splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pnl	, comp);
			
			//splitPane.setResizeWeight(0.5);
		} else {
			splitPane.setRightComponent(comp);
			
			setSplitPaneSize();
		}
		
		if (comp != null) {
			rightWidth = Math.max(comp.getPreferredSize().width, rightWidth);
		}
	}
	protected  void layoutScreen(List<? extends RadioBtnImageOption> options, JEditorPane tip) {
		pnl.addComponentRE(1, 3, BasePanel.FILL, BasePanel.GAP2,
                BasePanel.FULL, BasePanel.FULL,
                tip);
		
		for (RadioBtnImageOption lafOpt : options) {
			pnl.addLineRE("", lafOpt.radioBtn);
		}
		
		updatePaneSizes(options);
	}

	public void updatePaneSizes(List<? extends RadioBtnImageOption> options) {
		updatePaneSizesGeneric(options);
		
		options.get(0).radioBtn.setSelected(true);
		for (RadioBtnImageOption lafOpt : options) {
			lafOpt.radioBtn.addActionListener(lafOpt);
		}
	}

	public void updatePaneSizesGeneric(List<? extends IRightDisplayOption> options) {
		
		int width = 0;
		for (IRightDisplayOption lafOpt : options) {
			width = Math.max(width, lafOpt.getRightDisplayOption().getPreferredSize().width);
		}

		for (IRightDisplayOption lafOpt : options) {
			JComponent rightDisplayOption = lafOpt.getRightDisplayOption();
			Dimension preferredSize = rightDisplayOption.getPreferredSize();

			if (preferredSize.width < width) {
				rightDisplayOption.setPreferredSize(new Dimension(width, preferredSize.height));
			}
		}

		setRightComponent(options.get(0).getRightDisplayOption());
	}
	

	public void setPreferredSize(Dimension preferredSize) {
		pnl.setPreferredSize(preferredSize);
	}
	
	void setSplitPaneSize() {
		int width;
		if (splitPane != null && (width = splitPane.getPreferredSize().width) > rightWidth) {
			splitPane.setDividerLocation(width - rightWidth - SwingUtils.values().CHAR_FIELD_WIDTH * 3);
		}
	}
	
	
	static interface IRightDisplayOption {
		public JComponent getRightDisplayOption();
	}
	
	abstract class RadioBtnImageOption implements IRightDisplayOption, ActionListener {
		final JLabel imageLabel;
		final JCheckBox radioBtn;
		final JPanel pnl = new JPanel();
		
		RadioBtnImageOption(String text, String toolTip, String iconName) {
			this.radioBtn = createChk(text, toolTip);
			
			imageLabel = iconName == null || iconName.length() == 0
					? new JLabel("")
					: new JLabel(new ImageIcon ( this.getClass().getResource ( "pics/" + iconName )) );
			imageLabel.setBorder(BorderFactory.createEmptyBorder());
			pnl.add(imageLabel);
		}
		
		public JComponent getRightDisplayOption() {
			return imageLabel;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (radioBtn.isSelected()) {
				setRightComponent(imageLabel);
			}
		}
	}
	
	
	class OptionDtl extends RadioBtnImageOption {
		final String value, value2;
	
		
		public OptionDtl(String description, String toolTip, String value, String imageFilename) {
			super(description, toolTip, imageFilename);
			this.value = value;
			this.value2 = null;
		}
		public OptionDtl(String description, String toolTip, String value, String value2, String imageFilename) {
			super(description, toolTip, imageFilename);
			this.value = value;
			this.value2 = value2;
		}
	}

	static JCheckBox  createCobolChk() {
		JCheckBox useCobolChk = new JCheckBox("Display Cobol Options");
		
		useCobolChk.setToolTipText( 
				"<html>Select if you want to update the<br/>"
				+ "<B>COBOL</b> defaults.<br/>"
				+ "Only need if you will be editing<br/>"
				+ "COBOL data files with a COBOL copybook</html>");
		return useCobolChk;
	}
	

	static JCheckBox createChk(String text, String toolTip) {
		JCheckBox checkbox = new JCheckBox(text);
		
		if (toolTip != null) {
			checkbox.setToolTipText( "<html>" + toolTip + "</html>");
		}
		return checkbox;
	}

//	static WebCheckBox createCobolChk() {
//		WebCheckBox useCobolChk = new WebCheckBox("Display Cobol Options");
//		
//		useCobolChk.setToolTip(new JEditorPane(
//				"text/html", 
//				"Select if you want to update the<br/>"
//				+ "<B>COBOL</b> defaults.<br/>"
//				+ "Only need if you will be editing<br/>"
//				+ "COBOL data files with a COBOL copybook"));
//		return useCobolChk;
//	}

}
