package net.sf.RecordEditor.re.firstTime.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JEditorPane;
import javax.swing.JLabel;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.treeCombo.FileFieldCombo;
import net.sf.recordEditor.ui.def.FileChooserDefs.FileChooserType;
import net.sf.recordEditor.ui.fileFields.params.FileFieldParamBldr;

public class FileFieldPnl extends BaseWizPnl {

	//private final JEditorPane tipTxt;
	
	private static final String PROGRAM_NAME = "<b>" + Common.OPTIONS.programType.programName + "</b>";
	
	
	private final ArrayList<OptionDtl> ffOptions = new ArrayList<OptionDtl>();
	private final ButtonGroup btnGrp = new ButtonGroup();
	
	private final ButtonGroup openDirBtnGrp = new ButtonGroup();
	private final JCheckBox openLastDirChk = createOpenningChk(
			"Start in the last directory", 
			"<html>When starting the " + PROGRAM_NAME + ", start<br>" +
			"in the directory you used last time<br>" +
			"you ran the " + PROGRAM_NAME + ".</html>");
	private final JCheckBox openSetDirChk = createOpenningChk(
			"Start in specific directory",
			"<html>When starting the " + PROGRAM_NAME + ", start<br>" 
				+ "in a specific directory.</html>");
			
	private final JLabel directoryLbl = new JLabel("Startup Directory");
	private final FileFieldCombo directoryFld = new FileFieldCombo(FileFieldParamBldr.newFileFieldParamBuilder()
					.directory(true)
					.completeFile(true)
					.fileChooserOptions(FileFieldParamBldr.newFileChooserOptionBuilder()
									.chooserType(FileChooserType.open)
							.asBuilderAndParameter())
				.asBuilderAndParameter());
	private InitialSetupData data;
//	public LafWizPnl() {
//		this(Env.OS_TYPE);
//	}
	
	
	public FileFieldPnl() {
		super("FileFldWiz", "help-file-name");
		
		init_100_setup();
		//tipTxt = new JEditorPane("text/html", tipHtml);
		init_200_layoutScreen(ffOptions);
	}

	/**
	 * @param osType
	 * @return
	 */
	public void init_100_setup() {
		String opt = "There is also <b>recent, favourite</b>,<br/>and <b>File Search</b> options";
		addFileOption(new OptionDtl(
				"Breadcrumb style File Field", 
				"Breadcrumb style file field where you can<br>select directories/files.<br/>" + opt, 
				"N", "Y", "ffBreadCrumb.png"));
		addFileOption(new OptionDtl(
				"Text File Field with selection assistance", 
				"Text field with an auto complete list.<br/>" + opt, 
				"Y", "Y", "ffTextAutoComplete.png"));
		addFileOption(new OptionDtl(
				"Text File Field", 
				"Plain Text field.<br/>" + opt, 
				"Y", "N", "ffText.png"));
		
		directoryFld.setExpandVars(true);
	}
	
	private  void init_200_layoutScreen(List<? extends RadioBtnImageOption> options) {
		pnl.addComponentRE(1, 3, BasePanel.FILL, BasePanel.GAP2,
                BasePanel.FULL, BasePanel.FULL,
                new JEditorPane("text/html", FtMessages.FILE_FIELD_DESC.get()));
		
		String prompt = "File Field Style:";
		for (RadioBtnImageOption lafOpt : options) {
			pnl.addLineRE(prompt, lafOpt.radioBtn);
			prompt = "";
		}
		pnl														.setGapRE(BasePanel.GAP1)
			.addLineRE("Starting Directory", openLastDirChk)
			.addLineRE(                  "", openSetDirChk)		.setGapRE(BasePanel.GAP0)
			.addLineRE(        directoryLbl, directoryFld)
			;
		
		updatePaneSizes(options);
		
		ActionListener setDirListner = new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				checkDirectoryDisplay();
			}
		};
		openLastDirChk.addActionListener(setDirListner);
		openSetDirChk.addActionListener(setDirListner);
	}

	private void checkDirectoryDisplay() {
		boolean visible = openSetDirChk.isSelected();
		
		directoryLbl.setVisible(visible);
		directoryFld.setVisible(visible);
	}
	
	
	void addFileOption(OptionDtl optionDef) {

		ffOptions.add(optionDef);
		btnGrp.add(optionDef.radioBtn);
	}
	
	private JCheckBox createOpenningChk(String text, String tip) {
		JCheckBox opt = new JCheckBox(text);
		opt.setToolTipText(tip);
		openDirBtnGrp.add(opt);
		
		return opt;
	}
	
	@Override
	public void setValues(InitialSetupData detail) throws Exception {
		data = detail;
		
		setSplitPaneSize();
		
		if (data.startInLastDir) {
			openLastDirChk.setSelected(true);
		} else {
			openSetDirChk.setSelected(true);
		}
		directoryFld.setFileNameSilently(data.openDirectory);
		checkDirectoryDisplay();
		
		if (data.init) { return; }
		
		for (OptionDtl lafOpt : ffOptions) {
			if (data.editMode.equals((lafOpt.value))
			&&  data.autoComplete.equals((lafOpt.value2))) {
				lafOpt.radioBtn.setSelected(true);
				setRightComponent(lafOpt.imageLabel);
				return;
			}
		}
	}

	@Override
	public InitialSetupData getValues() throws Exception {
		String oldEditMode = data.editMode;
		data.setSavePropertyChanges(false);			
		if (data.startInLastDir != openLastDirChk.isSelected()) {
			data.startInLastDir = openLastDirChk.isSelected(); 
		}
		
		data.openDirectory = directoryFld.getText();
		if (openSetDirChk.isSelected()) {
			Parameters.setProperty(Parameters.DEFAULT_DIRECTORY, data.openDirectory);
		}
		for (OptionDtl lafOpt : ffOptions) {
			if (lafOpt.radioBtn.isSelected()) {
				data.editMode = lafOpt.value;
				data.autoComplete = lafOpt.value2;
				Common.OPTIONS.OPEN_IN_EDIT_MODE.set("Y".equals(data.editMode));
				Common.OPTIONS.AUTO_COMPLETE.set("Y".equals(data.autoComplete));
				data.setSavePropertyChanges(true);
				
				break;
			}
		}
		Common.OPTIONS.AUTO_COMPLETE.set("Y".equals(data.autoComplete));
		Parameters.setProperty(Parameters.OPEN_IN_LAST_DIRECTORY, data.startInLastDir ? "Y" : "N");
		
		if ((!data.init) && data.updateOpenPnl != null
		&& (Parameters.FILE_CHOOSER_RE_CLASSIC.equals(data.openFileOption) || Parameters.FILE_CHOOSER_RE_OLD.equals(data.openFileOption))
		&& (oldEditMode == null || ! oldEditMode.equalsIgnoreCase(data.editMode))) {
			data.updateOpenPnl.updateOpenPanel();
		}

		return data;
	}
}
