package net.sf.RecordEditor.re.firstTime.wizard;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.RootPaneContainer;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.wizards.AbstractWizard;
import net.sf.recordEditor.ui.xEnvironment.Env;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public class FirstTimeWizard extends AbstractWizard<InitialSetupData> {

	public static void runCsvEdFirstTimeOpt(JFrame owner, IUpdateOpenPnl updateOpenPnl, boolean init, Env.OsType osType) {
		if (init) {
			String s = Parameters.getString(Parameters.CSV_ED_RUN_INIT);
			if (s == null || s.length() == 0) {
				runCsvFT(owner, updateOpenPnl, init, osType);
				Parameters.setProperty(Parameters.CSV_ED_RUN_INIT, Common.CURRENT_VERSION);
				Parameters.writePropertiesNow();
			}
		} else {
			runCsvFT(owner, updateOpenPnl, init, osType);
		}
	}

	public static void runRecEdFirstTimeOpt(JFrame owner, IUpdateOpenPnl updateOpenPnl, boolean init, Env.OsType osType) {
		if (init) {
			String s = Parameters.getString(Parameters.RE_RUN_INIT);
			if (s == null || s.length() == 0) {
				runReFT(owner, updateOpenPnl, init, osType);
				Parameters.setProperty(Parameters.RE_RUN_INIT, Common.CURRENT_VERSION);
				Parameters.writePropertiesNow();
			}
		} else {
			runReFT(owner, updateOpenPnl, init, osType);
		}
	}

	/**
	 * @param owner
	 * @param init
	 * @param osType
	 */
	private static void runCsvFT(JFrame owner, IUpdateOpenPnl updateOpenPnl, boolean init, Env.OsType osType) {
		InitialSetupData details = new InitialSetupData(updateOpenPnl, init, false);
		
		runFt(owner, init, osType, details, "ReCsvEditor", new OpenTypePnl.Csv(osType, details));
	}

	private static void runReFT(JFrame owner, IUpdateOpenPnl updateOpenPnl, boolean init, Env.OsType osType) {
		InitialSetupData details = new InitialSetupData(updateOpenPnl, init, true);
		runFt(owner, init, osType, details, "RecordEditor", new OpenTypePnl.RecordEditor(osType, details));
	}

	private static void runFt(
			JFrame owner, boolean init, 
			Env.OsType osType, InitialSetupData details,
			String appl, OpenTypePnl openTypePnl) {
		BaseWizPnl[] pnls;
		
		int idx = 0;
		
		if (init) {
			pnls = new BaseWizPnl[6];
			pnls[idx++] = new InfoPnl(details.recordEditor ? "RecordEditor" : "ReCsvEditor");
		} else {
			pnls = new BaseWizPnl[5];
		}
		
		pnls[idx++] = new LafWizPnl(details, osType);
		pnls[idx++] = openTypePnl; //new OpenTypePnl.RecordEditor(osType, details);
		pnls[idx++] = new FileFieldPnl();
		pnls[idx++] = new TabOptionPnl(details.recordEditor);
		pnls[idx++] = new CobolPnl();

		JDialog dialog = new JDialog(owner, init);
		
		new FirstTimeWizard(dialog, details, osType, pnls);
	}

	
	private final Component frame;
	
	public <T extends Component & RootPaneContainer> FirstTimeWizard(T frame,
			InitialSetupData details, 
			Env.OsType osType,
			BaseWizPnl[] pnls) {
		super(frame, details);
		this.frame = frame;
		
		super.setPanels(pnls, false);
		
		frame.doLayout();
		Dimension preferredSize = frame.getPreferredSize();
		int heightAdj = SwingUtils.values().CHAR_FIELD_HEIGHT*3;
		int width = preferredSize.width  + SwingUtils.values().CHAR_FIELD_WIDTH*4;
		int height = preferredSize.height + heightAdj;
		frame.setPreferredSize(
				new Dimension(
						width, 
						height));
	
		preferredSize = frame.getPreferredSize();
		frame.setBounds(frame.getY(), frame.getX(), width, height);

		Dimension prefSize = pnls[0].getComponent().getPreferredSize();
		Dimension d = new Dimension(prefSize.width, prefSize.height + SwingUtils.values().CHAR_FIELD_HEIGHT);
		for (BaseWizPnl pnl : pnls) {
			pnl.setPreferredSize(d);
		}
		
		frame.setVisible(true);
	}

	@Override
	public void finished(InitialSetupData details) {
		Parameters.writePropertiesNow();
		frame.setVisible(false);
	}

}
