package net.sf.RecordEditor.re.firstTime.wizard;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.IUpdatableBooleanOption;
import net.sf.RecordEditor.utils.params.Parameters;

/**
 * Data Class for initial Setup wizard
 * 
 * @author Bruce Martin
 *
 */
public class InitialSetupData {
	
	public final IUpdateOpenPnl updateOpenPnl;
	public boolean doInit, useReFileChooser, usesCobol = false, showLaf;
	public final boolean init, recordEditor;
	public boolean startInLastDir = Common.OPTIONS.useLastDir.isSelected();
	public int lafIndex;
	public String lafClassName="--", openFileOption=Parameters.FILE_CHOOSER_NEW;
	public String editMode, autoComplete, cobolDialect, cobolDirectory,
		openDirectory = Parameters.getString(Parameters.DEFAULT_DIRECTORY);
	public final String applicationPref;
	public final IUpdatableBooleanOption showSchemaScreen;
	
	
	public final String fileOpenParam, fileChooserOption;
	
	public InitialSetupData(IUpdateOpenPnl updateOpenPnl, boolean init, boolean recordEditor) {
		this.updateOpenPnl = updateOpenPnl;
		this.init = init;
		this.doInit = init;
		this.recordEditor = recordEditor;
		this.showLaf = true;
		
		if (recordEditor) {
			fileOpenParam = Parameters.FILE_OPEN_RECORD_EDIT;
			fileChooserOption = Parameters.FILE_CHOOSER_OPTION_RE;
			applicationPref = Parameters.AP_RECORD_EDITOR;
		} else {
			fileOpenParam = Parameters.FILE_OPEN_CSV_EDIT;
			fileChooserOption = Parameters.FILE_CHOOSER_OPTION_CSV;
			applicationPref = Parameters.AP_RECORD_EDITOR;
		}
		showSchemaScreen = Common.OPTIONS.getBooleanOption(applicationPref + Parameters.OP_TAB_SCHEMA);
		
		if (init) {
			cobolDialect= "Mainframe";
		} else {
			usesCobol =  "Y".equalsIgnoreCase(getProperty(Parameters.USES_COBOL));
			cobolDialect = getProperty(Parameters.DEFAULT_COBOL_DIALECT_NAME);
			lafClassName = getProperty(Parameters.PROPERTY_LOOKS_CLASS_NAME);
			try {
		       lafIndex = Integer.parseInt(
		                    getProperty(Parameters.PROPERTY_LOOKS_CLASS_INDEX));
		    } catch (Exception e) {
		    }
			
			openFileOption = getProperty(fileOpenParam);
		}
		useReFileChooser = ! "N".equalsIgnoreCase(getProperty(Parameters.FILE_CHOOSER_UI));
		editMode = "Y".equalsIgnoreCase(getProperty(Parameters.FILE_FIELDS_START_IN_EDIT_MODE))
						? "Y" : "N";
		autoComplete  = "N".equalsIgnoreCase(getProperty(Parameters.FILE_FIELDS_AUTO_COMPLETE))
						? "N" : "Y";
		cobolDirectory = getProperty(Parameters.DEFAULT_COBOL_DIRECTORY);
	}
	
	public String getProperty(String key) {
		return Parameters.getString(key);
	}
	
	public void setProperty(String key, String value) {
		Parameters.setProperty(key, value);
	}
	
	public void setProperty(String key, int value) {
		Parameters.setProperty(key, value);
	}
	
	public void setSavePropertyChanges(boolean saveChanges) {
		Parameters.setSavePropertyChanges(saveChanges);
		if (saveChanges && Common.TEST_MODE) {
			Parameters.notifyOfPropertyChange();
		}
	}
}
