package net.sf.RecordEditor.re.firstTime.wizard;

import java.util.ArrayList;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JEditorPane;

import net.sf.RecordEditor.re.editProperties.LooksPanel;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.recordEditor.ui.xEnvironment.Env;

public class LafWizPnl extends BaseWizPnl {

	private final JEditorPane tipTxt;
	
	private final ArrayList<LafOption> lafOptions = new ArrayList<LafWizPnl.LafOption>();
	private final ButtonGroup btnGrp = new ButtonGroup();
	private final JCheckBox useCobolChk = super.createCobolChk();
	
	private InitialSetupData data;
//	public LafWizPnl() {
//		this(Env.OS_TYPE);
//	}
	
	
	public LafWizPnl(InitialSetupData details, Env.OsType osType) {
		super("LafWiz", "help-file-name");
		
		this.data = details;
		
		String tipHtml = init_100_setup(osType);
		tipTxt = new JEditorPane("text/html", tipHtml);
		
		layoutScreen(lafOptions, tipTxt);
		if (!data.init) {
			pnl	.setGapRE(BasePanel.GAP2)
				.addLineRE("", useCobolChk);
		}

	}

	/**
	 * @param osType
	 * @return
	 */
	public String init_100_setup(Env.OsType osType) {
		String tipHtml;

		LafOption nimbusLaf = new LafOption(FtMessages.NIMBUS_LAF_TEXT.get(), FtMessages.NIMBUS_LAF_DESC.get(),
				LooksPanel.NIMBUS, "nimbusLaf.png");
		LafOption metalLaf  = new LafOption(FtMessages.METAL_LAF_TEXT.get(),  FtMessages.METAL_LAF_DESC.get(), 
				LooksPanel.METAL_LOOK_AND_FEEL, "metalLaf.png");
		LafOption systemLaf = new LafOption(LooksPanel.SYSTEM_DEFAULT_INDEX, "System Look and Feel", 
				FtMessages.SYSTEM_LAF_DESC.get(), "", null);

		switch (osType) {
		case  WINDOWS:
			tipHtml = FtMessages.WIN_START_LAF_MSG.get();
			
			addLafOption(new LafOption(
					LooksPanel.SYSTEM_DEFAULT_INDEX, FtMessages.WINDOWS_LAF_TEXT.get(), 
					FtMessages.WINDOWS_LAF_DESC.get(), "", "windowsLaf.png"));
			addLafOption(nimbusLaf);
			addLafOption(metalLaf);
			
			break;
		case  MAC:
			tipHtml = FtMessages.MAC_START_LAF_MSG.get();
			
			if (Env.getEnvironmentDefaults().isQuauaAvailable()) {
				addLafOption(new LafOption("Quaqua Look and Feel (Apple Mac Laf)", FtMessages.NIMBUS_LAF_DESC.get(),
						LooksPanel.QUAQUA, "nimbusLaf.png"));
			}
			addLafOption(systemLaf);
			addLafOption(nimbusLaf);
			addLafOption(metalLaf);
			
			break;
		default:
			tipHtml = FtMessages.NIX_START_LAF_MSG.get();
			addLafOption(nimbusLaf);
			addLafOption(metalLaf);
			addLafOption(systemLaf);
		}
		
	
//		int width = 0;
//		for (LafOption lafOpt : lafOptions) {
//			width = Math.max(width, lafOpt.imageLabel.getPreferredSize().width);
//		}
//		for (LafOption lafOpt : lafOptions) {
//			Dimension preferredSize = lafOpt.imageLabel.getPreferredSize();
//			if (preferredSize.width < width) {
//				lafOpt.imageLabel.setPreferredSize(new Dimension(width, preferredSize.height));
//			}
//		}

//		lafOptions.get(0).radioBtn.setSelected(true);
		//updatePicture();
		return tipHtml;
	}
	
	private void addLafOption(LafOption lafOption) {
		if (lafOption.index >= 0) {
			lafOptions.add(lafOption);
			btnGrp.add(lafOption.radioBtn);
		}
//		lafOption.radioBtn.addActionListener(new ActionListener() {
//			@Override public void actionPerformed(ActionEvent e) {
//				updatePicture();
//			}
//		});
	}


	
	@Override
	public void setValues(InitialSetupData detail) throws Exception {
		data = detail;
		
		setSplitPaneSize();

		useCobolChk.setSelected(data.usesCobol);
		
		if (data.init) { return; }
		
		for (LafOption lafOpt : lafOptions) {
			if (lafOpt.index == data.lafIndex) {
				lafOpt.radioBtn.setSelected(true);
				setRightComponent(lafOpt.imageLabel);
				return;
			}
		}
		
		int idx = LooksPanel.findIndex(detail.lafClassName);
		String name = "Current Look and Feel";
		if (idx <= 0) {
			idx = LooksPanel.NUM_LOOKS - 1;
		} else {
			name = LooksPanel.getLooksClassName(idx);
		}
		LafOption lafOption = new LafOption(idx, name, null, detail.lafClassName, null);
		addLafOption(lafOption);
		lafOption.radioBtn.setSelected(true);
		setRightComponent(lafOption.imageLabel);
	}

	@Override
	public InitialSetupData getValues() throws Exception {
		data.usesCobol = useCobolChk.isSelected();
		
		for (LafOption lafOpt : lafOptions) {
			if (lafOpt.radioBtn.isSelected()) {
				data.lafIndex = lafOpt.index;

				data.setSavePropertyChanges(false);
				data.setProperty(Parameters.PROPERTY_LOOKS_CLASS_INDEX, Integer.toString(lafOpt.index));
				data.setSavePropertyChanges(true);
				data.setProperty(Parameters.PROPERTY_LOOKS_CLASS_NAME, lafOpt.lafClassName);
				
				break;
			}
		}
		return data;
	}
	
	@Override
	public boolean skip() {
		return data == null || (! data.showLaf);
	}
	
	private class LafOption extends RadioBtnImageOption {
		final String lafClassName;
		final int index;

		public LafOption(String text, String description, String lafCode, String iconName) {
			this(LooksPanel.findIndex(lafCode), text, description, lafCode, iconName);
		}

		
		public LafOption(int index, String text,String toolTip, String lafCode, String imageFileName) {
			super(text, toolTip, imageFileName);
			this.index = index;
			this.lafClassName = lafCode;
		}
	}
	


}
