package net.sf.RecordEditor.re.firstTime.wizard;

import java.awt.event.ActionEvent;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JEditorPane;
import javax.swing.JLabel;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.IUpdatableBooleanOption;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.BasePanel;


public class TabOptionPnl extends BaseWizPnl {

	private final JEditorPane tipTxt = new JEditorPane("text/html", "");
	
	private InitialSetupData data;
	private final JCheckBox  showLafPnl = super.createChk(
			"Show Look and Feel options",
			"Change the java <i>Look and Feel</i><br/>"
					+ "If selected you can change the Look and Feel<br/>"
					+ "Most people do not need this option");
	private final JCheckBox useCobolChk = super.createCobolChk();
	
	private final boolean isRecordeditor;


	
	public TabOptionPnl(boolean isRecordeditor) {
		super("InfoPnl", "help-file-name");
		
		this.isRecordeditor = isRecordeditor;

		init_100_setup();
		
		init_200_LayoutScreen();
	}
	


	/**
	 * @param osType
	 * @return
	 */
	public void init_100_setup() {
		
		String msg = isRecordeditor
				? FtMessages.RE_TAB_INIT_MSG.get() 
				: FtMessages.RE_TAB_CSV_INIT_MSG.get();
		tipTxt.setText(msg);
	}
	



	
	private void init_200_LayoutScreen() {
		
		pnl	.addComponentRE(1, 3, BasePanel.FILL, BasePanel.GAP2,
                BasePanel.FULL, BasePanel.FULL,
                tipTxt);
		
		CheckAction[] actions;
		String pict;
		
		if (isRecordeditor) {
			String id = Parameters.AP_RECORD_EDITOR;
			
			pict = "reOpenClassic";
			actions = new CheckAction[] {
					new CheckAction("Xml - Display Xml files in a Tree",
							"<html>display <b>Xml</b> files in a Tree Format. This panel will added</br>"
							+ "to the tab if a Xml file is detected,</html>",
							id + Parameters.OP_TAB_XML),
					new CheckAction("Schema/Cobol - Use a file description (Xml/Cobol) to interpret the file", 
							"<html>Fields are descriibed by a <b>file Schema</b> or"
							+ "<b>File Description<b>. This file description can either be"
							+ "<b>Xml</b> File or a <b>Cobol</b> Copybook</html>",
							id + Parameters.OP_TAB_SCHEMA),
					new CheckAction("Regular Expression - split line into fields using a regular expresions",
							"Split a line into fields using a regular expresion",
							id + Parameters.OP_TAB_REG_EXP),
					new CheckAction("Other - PO translation files / SwingX tip property files",
							"PO translation files / SwingX tip property files",
							id + Parameters.OP_TAB_OTHER),
			};
		} else {
			String id = Parameters.AP_RECSVEDITOR;
			pict = "csvTab";
			actions = new CheckAction[] {
					new CheckAction("Fixed Width - fixed width fields",
							"<html>File where fields are aligned in <b>fixed sized columns</b>,<br/>"
							+ "each field has the same length on every record<br/>"
							+ "different fields can have defferent lengths</htm>l",
							id + Parameters.OP_TAB_FIXED),
					new CheckAction("Schema - Use a file description (Xml/Cobol) to interpret the file", 
							"<html>Fields are descriibed by a <b>file Schema</b> or"
							+ "<b>File Description<b>. This file description can either be"
							+ "<b>Xml</b> File or a <b>Cobol</b> Copybook</html>",
							id + Parameters.OP_TAB_SCHEMA),
					new CheckAction("Xml - Display Xml files in a Tree",
							"<html>display <b>Xml</b> files in a Tree Format. This panel will added</br>"
							+ "to the tab if a Xml file is detected,</html>",
							id + Parameters.OP_TAB_XML),
					new CheckAction("Regular Expression - split line into fields using a regular expresions",
							"Split a line into fields using a regular expresion",
							id + Parameters.OP_TAB_REG_EXP),
					new CheckAction("Other - PO translation files / SwingX tip property files",
							"PO translation files / SwingX tip property files",
							id + Parameters.OP_TAB_OTHER),
			};

		}
		
		for (CheckAction a : actions) {
			pnl.addLineRE("", a.chk);
		}
	
		String picName = "pics/" + pict + ".png";
		URL pictResource = TabOptionPnl.class.getResource(picName);
		if (pictResource == null) {
			//System.out.println("~~> picture not found: " + picName);
			super.setRightComponent(new JLabel());
		} else {
			//System.out.println("~~> " + picName + " " + pictResource.getFile());
			super.setRightComponent(
					new JLabel(
						new ImageIcon (
							pictResource)));
		}
		super.splitPane.setDividerLocation(0.5);
		super.splitPane.setResizeWeight(0.5);
	}

	@Override
	public InitialSetupData getValues() throws Exception {
		data.usesCobol = useCobolChk.isSelected();
		data.showLaf = showLafPnl.isSelected();
		data.setProperty(Parameters.USES_COBOL, data.usesCobol ? "Y" : null);
		return data;
	}
	

	@Override
	public void setValues(InitialSetupData detail) throws Exception {
		data = detail;
		useCobolChk.setSelected(data.usesCobol);
		showLafPnl.setSelected(data.showLaf);
		
		super.splitPane.setDividerLocation(0.5);
	}
	
	@SuppressWarnings("serial")
	private static class CheckAction extends AbstractAction {

		//final String helpstr;
		final IUpdatableBooleanOption opt;
		final JCheckBox chk;
		
		CheckAction(String name, String help, String key) {
			super(name);
			
			opt = Common.OPTIONS.getBooleanOption(key);
			
			chk = new JCheckBox(this);
			chk.setToolTipText(help);
			chk.setSelected(opt.isSelected());
		}
		
		@Override public void actionPerformed(ActionEvent arg0) {
			opt.set(chk.isSelected(), true);
		}
	}
}
