/*
 * @Author Bruce Martin
 * Created on 9/02/2007 version 0.60
 *
 * Purpose:
 * A String based check box format with supplied yes / no strings
 */
package net.sf.RecordEditor.re.jrecord.format;

import java.awt.Font;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import net.sf.JRecord.Common.IFieldDetail;
import net.sf.recordEditor.ui.swing.tableCell.CheckboxTableRenderStringBased;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;


/**
 * A String based check box format with supplied yes / no strings
 *
 * @author Bruce Martin
 *
 */
public class CheckBoxFormat implements ICellFormat {

    private CheckboxTableRenderStringBased render;
    private String yesStr;
    private String noStr;
    private boolean defaultVal;


    /**
     * A String based check box format with supplied yes / no strings
     * @param yes yes string
     * @param no  no string
     * @param defaultValue default value
     */
    public CheckBoxFormat(final String yes,
                final String no, final boolean defaultValue) {
        yesStr = yes;
        noStr  = no;
        defaultVal = defaultValue;

        render = new CheckboxTableRenderStringBased(yesStr, noStr, defaultVal, false);
    }


    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldHeight()
     */
    public int getFieldHeight() {
        return SwingUtils.values().CHECK_BOX_HEIGHT;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldWidth()
     */
    public int getFieldWidth() {
        return SwingUtils.values().CHECK_BOX_WIDTH;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellEditor(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellEditor getTableCellEditor(IFieldDetail fld) {
        return new CheckboxTableRenderStringBased(yesStr, noStr, defaultVal, false);
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellRenderer(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellRenderer getTableCellRenderer(IFieldDetail fld) {
        return render;
    }
    
	@Override
	public void setTableFont(Font font) {
	}

}
