/*
 * @Author Bruce Martin
 * Created on 9/02/2007 version 0.60
 *
 * Purpose:
 * Provide a date formater with a popup date selector
 */
package net.sf.RecordEditor.re.jrecord.format;

import java.awt.Font;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Common.IFieldDetail;
import net.sf.JRecord.CsvParser.BasicCsvLineParser;
import net.sf.JRecord.CsvParser.CsvDefinition;
import net.sf.RecordEditor.utils.swing.CsvArray;
import net.sf.RecordEditor.utils.swing.CsvArrayTableEditor;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;


/**
 * Provide a date formater with a popup date selector
 *
 * @author Bruce Martin
 *
 */
public class CsvArrayFormat implements ICellFormat {


	private static int HEIGHT = Math.max(SwingUtils.values().TABLE_ROW_HEIGHT,
			Math.min(SwingUtils.values().CHECK_BOX_HEIGHT, SwingUtils.values().COMBO_TABLE_ROW_HEIGHT));
    private CsvArray render = null;
    private Font font;



    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldHeight()
     */
    public int getFieldHeight() {
        return HEIGHT;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldWidth()
     */
    public int getFieldWidth() {
        return Constants.NULL_INTEGER;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellEditor(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellEditor getTableCellEditor(IFieldDetail fld) {
        String[] f = getFields(fld);
        CsvArrayTableEditor csvArrayTableEditor = new CsvArrayTableEditor(f[1], f[2], f[0]);
        if (font != null) {
        	csvArrayTableEditor.setTableFont(font);
		}
		return csvArrayTableEditor;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellRenderer(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellRenderer getTableCellRenderer(IFieldDetail fld) {
        if (render == null) {
            String[] f = getFields(fld);
            render = new CsvArray(f[1], f[2], f[0]);
            if (font != null) {
    			render.setTableFont(font);
    		}
        }
        return render;
    }

//    /**
//     * @see javax.swing.CellEditor#isCellEditable(java.util.EventObject)
//     */
//    public boolean isCellEditable(EventObject anEvent) {
//        if (anEvent instanceof MouseEvent) {
//    		return ((MouseEvent)anEvent).getClickCount() >= 2;
//   	    }
//       return true;
//    }

    /**
     * Get the Swing checkbox
     * @param fld field definition
     * @return Swing checkbox
     */
    private String[] getFields(IFieldDetail fld) {
        String s = fld.getParamater();
        BasicCsvLineParser p = BasicCsvLineParser.getInstance();

        return p.split(s.substring(1), new CsvDefinition(s.substring(0, 1), ""), 3);

        /*String[] r = p.split(s.substring(1), s.substring(0, 1), "", 3);

        System.out.print("~~>" + s + " > " + r.length + "  ");

        for (int i = 0; i < r.length; i++) {
            System.out.print(": " + r[i] + " :");
        }

        return r;*/
    }

	public void setTableFont(Font font) {
		this.font = font;
		if (render != null) {
			render.setTableFont(font);
		}
	}
}
