/*
 * @Author Bruce Martin
 * Created on 9/02/2007 version 0.60
 *
 * Purpose:
 * Provide a date formater with a popup date selector
 */
package net.sf.RecordEditor.re.jrecord.format;

import java.awt.Font;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import com.zbluesoftware.java.bm.IZDateFormat;
import com.zbluesoftware.java.bm.ZDateField;
import com.zbluesoftware.java.bm.ZDateFormat;
import com.zbluesoftware.java.bm.ZDateTableRender;

import net.sf.JRecord.Common.IFieldDetail;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;


/**
 * Provide a date formater with a popup date selector
 *
 * @author Bruce Martin
 *
 */
public class DateFormat implements ICellFormat {

    private static final int CELL_WIDTH  = SwingUtils.values().STANDARD_FONT_WIDTH * 8;

    private ZDateTableRender render = null;
    private boolean useDateFormat;
    //private SimpleDateFormat df = null;
    //private String dateFormat;
    private IZDateFormat dateFormat;

    /**
     * Date format for the record editor. It provides
     * basic date selection capability
     *
     * @param isDateFormat wether it will receive/send dates in date
     * format (or string format)
     *
     * @param dateFormatString date format to use
     */
    public DateFormat(final boolean isDateFormat, final String dateFormatString) {
        useDateFormat = isDateFormat;

        if (dateFormatString != null && dateFormatString.length() > 0) {
        	dateFormat = new ZDateFormat(dateFormatString);
        }
    }
    public DateFormat(final boolean isDateFormat, IZDateFormat dateFmt) {
    	useDateFormat = isDateFormat;
    	dateFormat = dateFmt;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldHeight()
     */
    public int getFieldHeight() {
        return SwingUtils.values().COMBO_TABLE_ROW_HEIGHT;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldWidth()
     */
    public int getFieldWidth() {
        return CELL_WIDTH;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellEditor(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellEditor getTableCellEditor(IFieldDetail fld) {
        return getDateDisplay(fld);
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellRenderer(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellRenderer getTableCellRenderer(IFieldDetail fld) {
        if (render == null) {
            render = getDateDisplay(fld);
        }
        return render;
    }

    /**
     * Get the Swing checkbox
     * @param fld field definition
     * @return Swing checkbox
     */
    private ZDateTableRender getDateDisplay(IFieldDetail fld) {
        ZDateTableRender ret = null;

        try {
            ret = dateFormat == null 
            		? new ZDateTableRender(useDateFormat, fld.getParamater())
            		: new ZDateTableRender(useDateFormat, new ZDateField(dateFormat) );
      
        } catch (Exception e) {
            //System.out.println("@@ format > " + s);
            e.printStackTrace();
        }

        return ret;
    }
    
    
	@Override
	public void setTableFont(Font font) {
	}

}
