/*
 * @Author Bruce Martin
 * Created on 9/02/2007 version 0.60
 *
 * Purpose:
 * Provide a date formater with a popup date selector
 */
package net.sf.RecordEditor.re.jrecord.format;

import java.awt.Font;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import com.zbluesoftware.java.bm.AbstractGenericCombo;
import com.zbluesoftware.java.bm.GenericComboTableRender;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Common.IFieldDetail;
import net.sf.RecordEditor.utils.swing.MultiLineCombo;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;


/**
 * Provide a date formater with a popup date selector
 *
 * @author Bruce Martin
 *
 */
public class MultiLineFormat implements ICellFormat {


   private static int HEIGHT = Math.max(SwingUtils.values().TABLE_ROW_HEIGHT,
		   Math.min(SwingUtils.values().CHECK_BOX_HEIGHT, SwingUtils.values().COMBO_TABLE_ROW_HEIGHT));
   private TableCellRenderer render = null;
   
   private Font font;



    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldHeight()
     */
    public int getFieldHeight() {
        return HEIGHT;
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getFieldWidth()
     */
    public int getFieldWidth() {
        return Constants.NULL_INTEGER;
    }

    @SuppressWarnings("serial")
	private GenericComboTableRender getTableRender() {
        return new GenericComboTableRender(false, new MultiLineCombo(font)) {
			@Override
			protected AbstractGenericCombo getCombo() {
				return new MultiLineCombo(font);
			}
        };
    }


    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellEditor(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellEditor getTableCellEditor(IFieldDetail fld) {
        return getTableRender();
	//			 						   new MultiLineCombo());
    }

    /**
     * @see net.sf.RecordEditor.re.jrecord.format.ICellFormat#getTableCellRenderer(net.sf.RecordEditor.record.types.FieldDetail)
     */
    @Override
    public TableCellRenderer getTableCellRenderer(IFieldDetail fld) {
        if (render == null) {
            render = getTableRender();
        }
        return render;
    }

	public void setTableFont(Font font) {
		
		Font lastFont = font;
		this.font = font;
		if (render != null && (font != null || lastFont != null)) {
            render = getTableRender();
        }
	}
}
