/*
 * @Author Bruce Martin
 * Created on 13/01/2007
 *
 * Purpose:
 *   Create a popup menu listner (with edit options)
 *
 * # Version 0.60 Bruce Martin 2007/02/16
 *   - added Filter, Save & save_as options
 */
package net.sf.RecordEditor.re.popups;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

import net.sf.RecordEditor.re.script.VelocityPopup;
import net.sf.RecordEditor.re.script.XsltPopup;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReActionHandler;
import net.sf.RecordEditor.utils.lang.ReAbstractAction;
import net.sf.RecordEditor.utils.screenManager.ReAction;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;


/**
 * Create a popup menu listner (with edit options)
 *
 * @author Bruce Martin
 *
 */
public class MenuPopupListener2 extends MouseAdapter {
    private JPopupMenu popup = new JPopupMenu();

    private JTable tbl = null;

    private int popupCol, popupRow;

	
    /**
     * Create a popup menu listner (with edit options)
     *
     * @param userAction any user actions
     * @param isFileEdit wether to add file editing actions action
     * @param table table being acted on
     */
	public MenuPopupListener2(
			ReActionHandler handler,
			final Action[] userAction, final boolean isFileEdit, final JTable table) {
		this(handler, userAction, isFileEdit, table, null, false);
	}

		
    /**
     * Create a popup menu listner (with edit options)
     *
     * @param userAction any user actions
     * @param isFileEdit wether to add file editing actions action
     * @param table table being acted on
     * @param addCellActions add cell actions
     */
    @SuppressWarnings("serial")
	public MenuPopupListener2(
			ReActionHandler handler,
			final Action[] userAction, final boolean isFileEdit, final JTable table,
			JMenuItem extraAction, boolean addCellActions) {
        super();

        tbl = table;
	    popup.add(new ReAction(ReActionHandler.COPY_RECORD, handler));
	    popup.add(new ReAction(ReActionHandler.CUT_RECORD, handler));
	    popup.add(new ReAction(ReActionHandler.PASTE_RECORD_PRIOR_POPUP, handler));
	    popup.add(new ReAction(ReActionHandler.PASTE_RECORD_POPUP, handler));
	    popup.add(new ReAction(ReActionHandler.INSERT_RECORD_PRIOR_POPUP, handler));
	    popup.add(new ReAction(ReActionHandler.INSERT_RECORDS_POPUP, handler));
	    popup.add(new ReAction(ReActionHandler.DELETE_RECORD_POPUP, handler));

	    if (isFileEdit) {
	    	JMenu saveMenu = SwingUtilsBldr.newMenu("Save ...");
	    	JMenu selectedCellsMenu = SwingUtilsBldr.newMenu("Cell Actions");

	        popup.add(new ReAction(ReActionHandler.REPEAT_RECORD_POPUP, handler));

//	        if (Common.OPTIONS.highlightEmpty.isSelected()) {
		        popup.add(new ReAbstractAction("Clear Field") {
					@Override public void actionPerformed(ActionEvent arg0) {
						if (popupRow >= 0) {
							tbl.setValueAt(Common.MISSING_VALUE, popupRow, popupCol);
						}
					}
		        });
		        selectedCellsMenu.add(new ReAction(ReActionHandler.COPY_SELECTED_CELLS, handler));
		        selectedCellsMenu.add(new ReAction(ReActionHandler.PASTE_TABLE_OVER_SELECTION, handler));
//		        ReAbstractAction clearSelectedAction = new ReAbstractAction("Clear Selected Cells") {
//					@Override public void actionPerformed(ActionEvent arg0) {
//						int[] selectedRows = tbl.getSelectedRows();
//						int[] selectedColumns = tbl.getSelectedColumns();
//						if (selectedRows != null && selectedColumns != null) {
//							for (int row : selectedRows) {
//								for (int col : selectedColumns) {
//									tbl.setValueAt(Common.MISSING_VALUE, row, col);
//								}
//							}
//						}
//					}
//		        };
		        if (addCellActions) {
		        	selectedCellsMenu.add(new ReAction(ReActionHandler.PASTE_INSERT_CELLS, handler));
		        	//selectedCellsMenu.add(clearSelectedAction);
		        	selectedCellsMenu.add(new ReAction(ReActionHandler.CUT_SELECTED_CELLS, handler));
		        	selectedCellsMenu.add( new ReAction(ReActionHandler.DELETE_SELECTED_CELLS, handler));
		        //} else {
		        }
	        	selectedCellsMenu.add(new ReAction(ReActionHandler.CLEAR_SELECTED_CELLS, handler));

		        popup.add(selectedCellsMenu);
		        
		        if (extraAction != null) {
		        	popup.add(extraAction);
		        }
//	        }
		        

	        popup.addSeparator();
	        popup.add( new ReAction(ReActionHandler.FIND, handler));
	        popup.add(new ReAction(ReActionHandler.FILTER, handler));
	        popup.addSeparator();
	        popup.add(saveMenu);

	        saveMenu.add(new ReAction(ReActionHandler.SAVE, handler));
	        saveMenu.add(new ReAction(ReActionHandler.SAVE_AS, handler));
	        //if (Common.isVelocityAvailable()) {
	        saveMenu.addSeparator();
	        //}


	        saveMenu.add(new ReAction(ReActionHandler.EXPORT, handler));
	        saveMenu.add(new ReAction(ReActionHandler.EXPORT_AS_CSV, handler));
	        saveMenu.add(new ReAction(ReActionHandler.EXPORT_AS_FIXED, handler));
	        saveMenu.add(new ReAction(ReActionHandler.EXPORT_AS_HTML, handler));
	        saveMenu.add(new ReAction(ReActionHandler.EXPORT_AS_HTML_TBL_PER_ROW, handler));
	        saveMenu.add(new ReAction(ReActionHandler.EXPORT_HTML_TREE, handler));

	        if (Common.isVelocityAvailable()) {
	        	saveMenu.add(new ReAction(ReActionHandler.EXPORT_VELOCITY, handler));
	        	saveMenu.add(VelocityPopup.getPopup());
	        }

	        if (Common.OPTIONS.xsltAvailable.isSelected()) {
	        	saveMenu.add(new ReAction(ReActionHandler.EXPORT_XSLT, handler));
	        	saveMenu.add(new XsltPopup());
	        }
	    }

	    if (userAction != null) {
	        popup.addSeparator();

	        addUserActions(userAction);
	    }
    }

    public MenuPopupListener2() {

    }

	public MenuPopupListener2(final Action userAction) {
		popup.add(userAction);
    }

    /**
     * Menu with no default options
     * @param userAction user actions
     */
    public MenuPopupListener2(final Action[] userAction) {
    	addUserActions(userAction);
    }

    protected final void addUserActions(final Action[] userAction) {
	    if (userAction != null) {
	        for (int i = 0; i < userAction.length; i++) {
	            if (userAction[i] == null) {
	        	    popup.addSeparator();
	            } else {
	                popup.add(userAction[i]);
	            }
	        }
	    }
    }

    /**
     * @see MouseAdapter#mousePressed(java.awt.event.MouseEvent)
     */
    public void mousePressed(MouseEvent e) {
      	if (tbl != null) {
       		if (! tbl.hasFocus()) {
       	   		tbl.requestFocusInWindow();
       		}
      	}
        maybeShowPopup(e);
    }

    /**
     * @see MouseAdapter#mouseReleased(java.awt.event.MouseEvent)
     */
    public void mouseReleased(MouseEvent e) {
         maybeShowPopup(e);
    }

    /**
     * show popup if it is a popup request
     * @param e the mouse event that triggered this action
     */
    private void maybeShowPopup(MouseEvent e) {
         if (e != null && e.isPopupTrigger() && isOkToShowPopup(e)) {
        	 int x = e.getX(),
        	     y = e.getY();
        	 popupRow = -1;
        	 if (tbl != null) {
        		popupCol = tbl.columnAtPoint(e.getPoint());
             	popupRow = tbl.rowAtPoint(e.getPoint());
        	 }
        	 JPopupMenu popupMenu = getPopup();
        	 if (Common.TEST_MODE) {
        		 try {
	        		 Dimension d = popupMenu.getPreferredSize();
	        		 int vpHeight = tbl.getParent().getHeight();
	        		 if ( y + d.height > vpHeight) {
	        			 y = Math.max(1, vpHeight - d.height);
	        		 }
        		 } catch (Exception exc) {
				 }
        	 }

             popupMenu.show(e.getComponent(),
                       x, y);
        }
    }

    /**
     * Check if it is ok to show the popup menu
     * @param e the mouse event that triggered this action
     * @return whether to show the popup or not
     */
    protected boolean isOkToShowPopup(MouseEvent e) {
        return true;
    }


    /**
     * @return Returns the popup.
     */
    public JPopupMenu getPopup() {
        return popup;
    }

    /**
	 * @return the popupRow
	 */
	public int getPopupRow() {
		return popupRow;
	}

	/**
	 * @return the popupCol
	 */
	public int getPopupCol() {
		return popupCol;
	}


	/**
	 * @param table the tbl to set
	 */
	public final void setTable(JTable table) {
		this.tbl = table;
	}


//    /**
//     * Get Popup Menu listner
//     * @param userAction list of user actions
//     * @param addFind wether to add find button
//     * @return the mouse adapter
//     */
//    public static MouseAdapter getEditMenuPopupListner(final Action[] userAction,
//    		final boolean addFind, final JTable table) {
//        return new MenuPopupListener2(userAction, addFind, table);
//    }


}
