package net.sf.RecordEditor.re.script;


import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.edit.RecordList;
import net.sf.RecordEditor.utils.lang.LangConversion;
import net.sf.RecordEditor.utils.msg.UtMessages;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.BmKeyedComboBox;
import net.sf.RecordEditor.utils.swing.BmKeyedComboBoxRender;
import net.sf.RecordEditor.utils.swing.BmKeyedComboModel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.saveRestore.ISaveUpdateDetails;
import net.sf.RecordEditor.utils.swing.saveRestore.SaveLoadPnl;
import net.sf.RecordEditor.xml.LoadEditorTask;
import net.sf.RecordEditor.xml.classes.EditorTask;
import net.sf.RecordEditor.xml.classes.RecordParent;
import net.sf.RecordEditor.xml.classes.RecordTree;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public class CreateRecordTreePnl  implements ISaveUpdateDetails<EditorTask> {

	private static final int PARENT_INDEX     = 1;
	public static final Integer BLANK_PARENT = Integer.valueOf(-1);

	private static final String[] LAYOUT_COLUMN_HEADINGS = LangConversion.convertColHeading(
			"Create_Record_Tree",
			new String[] {
					"Record", "Parent Record"
	});

	public final BaseHelpPanel panel = new BaseHelpPanel();

	private LayoutTblMdl recordMdl = new LayoutTblMdl();
	public final JTable recordTbl = new JTable(recordMdl);

	public final JButton execute
	  = SwingUtilsBldr.newButton("Build", Common.getRecordIcon(Common.ID_TREE_ICON));
	//private final JTextField msgTxt = new JTextField();


	private AbstractLayoutDetails layout;
	private Integer[] parent;


	public CreateRecordTreePnl() {
		init_200_FormatScreen(false, 10);
	}

	public CreateRecordTreePnl(AbstractLayoutDetails layoutDetails, boolean addSave) {
		super();

		setLayout(layoutDetails);

		init_200_FormatScreen(addSave, recordTbl.getRowCount());
	}

	/**
	 * Set the layout
	 * @param layoutDetails new layout details
	 */
	public final void setLayout(AbstractLayoutDetails layoutDetails) {

		layout = layoutDetails;
		parent = new Integer[layout.getRecordCount()];
		for (int i = 0; i < parent.length; i++) {
			parent[i] = BLANK_PARENT;
		}

		init_100_ScreenFields();
	}

	/**
	 * @return the layout
	 */
	public final AbstractLayoutDetails getLayout() {
		return layout;
	}

	/**
	 * Setup screen fields etc
	 *
	 */
	private void init_100_ScreenFields() {
		TableColumn tc;
		RecordList records = new RecordList(layout, false, true, true);
		BmKeyedComboBox editor = new BmKeyedComboBox(new BmKeyedComboModel(records), false);
		//pnl.registerComponent(parentFrame);
		panel.registerComponentRE(editor);

		tc = recordTbl.getColumnModel().getColumn(1);
		tc.setCellRenderer(new BmKeyedComboBoxRender(new BmKeyedComboModel(records), false));
		tc.setCellEditor(new DefaultCellEditor(editor));

		recordTbl.setRowHeight(SwingUtils.values().COMBO_TABLE_ROW_HEIGHT);
	}


	/**
	 *
	 * Format the screen
	 */
	private void init_200_FormatScreen(boolean addSave, int recs) {

		int height = SwingUtils.calculateComboTableHeight(
				recs,
				ReFrame.getDesktopHeight() / 2);

		panel.setHelpURLre(Common.formatHelpURL(Common.HELP_RECORD_TREE));

		panel.addComponentRE(1, 5,
		         height,
		         BasePanel.GAP1,
		         BasePanel.FULL, BasePanel.FULL,
				 recordTbl);
		panel.setComponentName(recordTbl, "Recs");

		if (addSave) {
			SaveLoadPnl<EditorTask> saveLoad = new SaveLoadPnl<EditorTask>(
					this, Parameters.getFileName(Parameters.RECORD_TREE_SAVE_DIRECTORY), new LoadEditorTask());


			panel.addLineRE("", saveLoad.panel, SwingUtils.getPanelWith(execute));
		}
		panel.setGapRE(BasePanel.GAP2);

		panel.addMessageRE();

		panel.done();
//		if (parentFrame != null) {
//			parentFrame.addMainComponent(pnl);
//		}
	}




	/**
	 * @see net.sf.RecordEditor.utils.swing.saveRestore.ISaveDetails#getSaveDetails()
	 */
	@Override
	public final EditorTask getSaveDetails() {
		RecordTree tree = new net.sf.RecordEditor.xml.classes.RecordTree();
		int i, j, size;

		Common.stopCellEditing(recordTbl);
		size = 0;
		for (i = 0; i < parent.length; i++) {
			if (parent[i] != CreateRecordTreePnl.BLANK_PARENT) {
				size += 1;
			}
		}

		j = 0;
		tree.clearParentRelationship();
		for (i = 0; i < parent.length; i++) {
			if (parent[i] != CreateRecordTreePnl.BLANK_PARENT) {
				net.sf.RecordEditor.xml.classes.RecordParent recordParent = new net.sf.RecordEditor.xml.classes.RecordParent();
				tree.getParentRelationship().add(recordParent);
				recordParent.recordName   = layout.getRecord(i).getRecordName();

				recordParent.parentName   = layout.getRecord(parent[i].intValue()).getRecordName();
			}
		}
		return (new EditorTask())
					.setRecordTree(layout.getLayoutName(), tree);
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.saveRestore.IUpdateDetails#update(java.lang.Object)
	 */
	@Override
	public void update(EditorTask serialisedData) {

		if (serialisedData == null || serialisedData.getRecordTree() == null) {
			Common.logMsgRaw(UtMessages.NOT_A_RECORD_TREE.get(), null);
		} else {
			setFromSavedDetails(serialisedData.getRecordTree());
			recordMdl.fireTableDataChanged();
		}
	}

	/**
	 * update tree definition from save details
	 * @param saveDetails details that have been saved
	 */
	public final void setFromSavedDetails(EditorTask saveDetails) {
		setFromSavedDetails(saveDetails.getRecordTree());
	}

	public final void setFromSavedDetails(RecordTree recordTree) {
		int i, idx;
		List<RecordParent> parentRel;

		if (recordTree != null && (parentRel =  recordTree.getParentRelationship()) != null) {
			//parent = new Integer[parentRel.length];
			for (i = 0; i < parentRel.size(); i++) {
				idx = layout.getRecordIndex(parentRel.get(i).recordName);
				if (idx >= 0) {
					parent[idx] = Integer.valueOf(layout.getRecordIndex(parentRel.get(i).parentName));
				}
			}
		}

	}


	/**
     * Table model to display records
     *
     * @author Bruce Martin
     *
     */
    @SuppressWarnings("serial")
	private class LayoutTblMdl extends AbstractTableModel {


        /**
         * @see javax.swing.table.TableModel#getColumnCount
         */
        public int getColumnCount() {
            return LAYOUT_COLUMN_HEADINGS.length;
        }


        /**
         * @see javax.swing.table.TableModel#getColumnName
         */
        public String getColumnName(int columnIndex) {
            return LAYOUT_COLUMN_HEADINGS[columnIndex];
        }


        /**
         * @see javax.swing.table.TableModel#getRowCount
         */
        public int getRowCount() {
        	if (layout == null) {
        		return 0;
        	}
            return layout.getRecordCount();
        }


        /**
         * @see javax.swing.table.TableModel#getValueAt
         */
        public Object getValueAt(int rowIndex, int columnIndex) {
            if (columnIndex == PARENT_INDEX) {
                return parent[rowIndex];
            }
            return layout.getRecord(rowIndex).getRecordName();
        }


        /**
         * @see javax.swing.table.TableModel#isCellEditable
         */
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return columnIndex == PARENT_INDEX;
        }


        /**
         * @see javax.swing.table.TableModel#setValueAt
         */
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {

        	if (aValue == null) {
        		parent[rowIndex] = BLANK_PARENT;
        	} else if (aValue instanceof Integer) {
				parent[rowIndex] = (Integer) aValue;
        	} else {
        		parent[rowIndex] = Integer.valueOf(aValue.toString());
        	}
        }
    }

	/**
	 * @return the pnl
	 */
	public final BaseHelpPanel getPanel() {
		return panel;
	}

	/**
	 * @return the parent
	 */
	public final Integer[] getParent() {
		return parent;
	}

	/**
	 * @param parent the parent to set
	 */
	public final void setParent(Integer[] parent) {
		this.parent = parent;
	}

//	/**
//	 *  Execute standard RecordEditor actions
//	 *
//	 * @param action action to perform
//	 */
//	public void executeAction(int action) {
//		if (action == ReActionHandler.HELP) {
//		    pnl.showHelp();
//		} else {
//			super.executeAction(action);
//		}
//	}
//
//	/**
//	 * Check if action is available
//	 *
//	 * @param action action to be checked
//	 *
//	 * @return wether action is available
//	 */
//	public boolean isActionAvailable(final int action) {
//		return  (action == ReActionHandler.HELP)
//		    ||  super.isActionAvailable(action);
//	}



}
