package net.sf.RecordEditor.re.util;

import net.sf.JRecord.External.ExternalRecord;
import net.sf.RecordEditor.re.db.Record.ExtendedRecordDB;
import net.sf.RecordEditor.re.db.Record.RecordRec;
import net.sf.RecordEditor.utils.common.ReConnection;
import net.sf.RecordEditor.utils.jdbc.AbsDB;
import net.sf.RecordEditor.utils.jdbc.AbsRecord;

public class XRecDBInterface {


	private int lastDBidx = -1;

    private ExtendedRecordDB recordDB   = null;

	   /**
     * Allocate the Database Interfaces
     *
     * @param dbIdx Database index
     */
    public void allocDBs(final int dbIdx) {

        if (lastDBidx != dbIdx || recordDB == null) {
            this.recordDB   = new ExtendedRecordDB();
            this.recordDB.setDoFree(false);

            this.recordDB.setConnection(new ReConnection(dbIdx));
         } else if (lastDBidx != dbIdx) {
            this.recordDB.fullClose();
            this.recordDB.setDoFree(true);
            this.recordDB.setDoFree(false);

            this.recordDB.setConnection(new ReConnection(dbIdx));
        }

       lastDBidx = dbIdx;
    }

    
    public void freeDBs(int pDbIdx) {
    	if (recordDB != null) {
    		recordDB.close();
    		recordDB.setDoFree(true);
       		recordDB.setDoFree(false);
    	}	
    }


    /**
     * Update the record details. This method will read current
     * DB details (if there are any) and apply standard updates to
     * the record
     *
     * @param copyBook Copybook name
     * @param rec Record Details
     * @param updateRequired Wether a update of existing record is required
      */
    public void updateRecord(String copyBook,
            				 ExternalRecord rec,
            				 boolean updateRequired,
            				 String fontname,
            				 int system) {
        RecordRec oldRec;


        if (copyBook != null && copyBook.length() > 0) {
	        recordDB.resetSearch();
	        recordDB.setSearchArg("COPYBOOK", AbsDB.opEquals, "'" + copyBook + "'");
	        //System.out.println("Searching >" + copyBook + "<");
	        recordDB.open();
	
	        if ((oldRec =  recordDB.fetch()) != null) {
	            ExternalRecord oldValues = oldRec.getValue();
	            rec.setRecordId(oldValues.getRecordId());
	            rec.setNew(false);
	            //System.out.println("Found old record ...");
	            //fieldsDB.setParams(rec.getRecordId());
	
	            if (updateRequired) {
	                rec.setUpdateStatus(AbsRecord.UPDATED);
	            } else {
	                String oldFont = oldValues.getFontName();
	
	                if (oldFont == null) {
	                    oldFont = "";
	                }
	
	                if (oldFont == null && fontname != null
	                || fontname != null && ! fontname.toUpperCase().equals(oldFont.toUpperCase())) {
	                    oldValues.setFontName(fontname);
	                    rec = oldValues;
	                    rec.setUpdateStatus(AbsRecord.UPDATED);
	                }
	            }
	        } else {
	            updateRecord(copyBook, rec, system);
	        }
        } else {
            updateRecord(copyBook, rec, system);
        }
        recordDB.freeConnection();
    }

	@SuppressWarnings("deprecation")
	private void updateRecord(String copyBook,
			ExternalRecord rec,
			int system) {

     rec.setSystem(system);
     rec.setCopyBook(copyBook);
     rec.setNew(true);
 }


}
