package net.sf.RecordEditor.re.util.csv;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.Iterator;

import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import net.sf.JRecord.External.ExternalRecord;
import net.sf.JRecord.External.base.CopybookWriter;
import net.sf.JRecord.External.base.RecordEditorXmlWriter;
import net.sf.JRecord.Log.AbsSSLogger;
import net.sf.RecordEditor.re.schema.ICreateFieldIterator;
import net.sf.RecordEditor.re.util.openTabPane.IDisplaySchemaScreen;
import net.sf.RecordEditor.re.util.openTabPane.IFilePreview;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.INotifyOptionChanged;
import net.sf.RecordEditor.utils.params.IUpdatableBooleanOption;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;

public abstract class BaseFilePreview implements IFilePreview {

	private ICreateFieldIterator createFieldIterator;
	private IGetFileTable fileTblSource;
	protected JTable tbl;
	protected BaseHelpPanel pnl = new BaseHelpPanel();
	
	private final IUpdatableBooleanOption  dispayPnl;
	
	private IDisplaySchemaScreen displaySchemaScreen;
	//final String displayOptionName; 
	
	
	public BaseFilePreview(String optName, boolean def) {
		super();
		
		if (optName == null) {
			dispayPnl = new StaticBoolOpt(def);
		} else {
			dispayPnl = Common.OPTIONS.getBooleanOption(optName);
		}
	}

	@Override
	public String getSeperator() {
		return null;
	}

	@Override
	public String getQuote() {
		return null;
	}


	@Override
	public void setDisplaySchemaScreen(IDisplaySchemaScreen displaySchemaScreen) {
		this.displaySchemaScreen = displaySchemaScreen;
	}

	@Override
	public BaseHelpPanel getPanel() {
		return pnl;
	}

	@Override
	public final void setHelpURL(URL helpUrl) {
		getPanel().setHelpURLre(helpUrl);
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.util.csv.IFilePreview#getJTable()
	 */
	@Override
	public JTable getJTable() {
		return tbl !=  null ? tbl 
							: fileTblSource != null ? fileTblSource.getFileTable() 
													: null;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.util.csv.IFilePreview#notifyHasReceivedFocus()
	 */
	@Override
	public void notifyHasReceivedFocus() {
		
	}

	@Override
	public TableCellRenderer getTableRenderer() {
		return null;
	}

	@Override
	public Iterator<String> getFieldIterator(int recordIdx, int fieldIdx) {
		return createFieldIterator == null ? null : createFieldIterator.getFieldIterator(recordIdx, fieldIdx);
	}


	@Override
	public Icon getIcon() {
		return null;
	}

	@Override
	public IUpdatableBooleanOption displayThePnl() {
		return dispayPnl;
	}

	public <X extends ICreateFieldIterator & IGetFileTable > void setCreateFieldIterator(X createFieldIterator) {
		this.createFieldIterator = createFieldIterator;
		this.fileTblSource = createFieldIterator;
	}
//	public void setCreateFieldIterator(ICreateFieldIterator createFieldIterator) {
//		this.createFieldIterator = createFieldIterator;
//	}

	protected void writeSchema(File f, ExternalRecord rec) {
		try {
			CopybookWriter w = new RecordEditorXmlWriter();	
			w.writeCopyBook(new FileOutputStream(f), rec, Common.getLogger());
			
			Common.logMsg(AbsSSLogger.WARNING, "Xml Schema: " + f.getPath() + " has been created !!!", null);
			if (displaySchemaScreen != null) {
				displaySchemaScreen.displaySchemaSelection(f.getPath());
			}
		} catch (FileNotFoundException e) {
			Common.logMsg("IO error writing Xml schema: " + e, null);
			e.printStackTrace();
		} catch (Exception e) {
			Common.logMsg("Error writing Xml schema: " + e, null);
			e.printStackTrace();
		}
	}

	public static class StaticBoolOpt implements IUpdatableBooleanOption {
		private final boolean val;
		
		public StaticBoolOpt(boolean val) {
			super();
			this.val = val;
		}

		@Override
		public boolean isSelected() {
			return val;
		}

		@Override public void set(boolean newVal) { }
		@Override public void set(boolean newVal, boolean notify) { }

		@Override public void addListner(INotifyOptionChanged notify) { }
	}
}
