package net.sf.RecordEditor.re.util.csv;

import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.zip.GZIPInputStream;

import javax.swing.Action;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.text.JTextComponent;

import net.sf.JRecord.Common.Conversion;
import net.sf.RecordEditor.re.layoutSelection.def.IFormatFileName;
import net.sf.RecordEditor.re.util.fw.FixedWidthSelectionPane;
import net.sf.RecordEditor.re.util.openTabPane.IFilePreview;
import net.sf.RecordEditor.re.util.openTabPane.OpenTab;
import net.sf.RecordEditor.re.util.openTabPane.TabItem;
import net.sf.RecordEditor.re.util.regExp.RegExpPreview;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.StreamUtil;
import net.sf.RecordEditor.utils.fileAnalysis.CheckEncoding;
import net.sf.RecordEditor.utils.fileAnalysis.FileAnalyser;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReFrame;


/**
 *
 * @author Bruce Martin
 *
 * Class that holds a JTabbedPane of Normal / Unicode
 * CSV preview panels.
 */

public class CsvTabPane implements IFormatFileName {

	private static final int PREVIEW_SIZE = 0x80000;

	public final static int NORMAL_CSV  = 0,
							UNICODE_CSV = 1,
							FIXED_WIDTH = 2,
							SCHEMA = 3,
							XML_FILE = 4,
							REGEXP_FILE = 5,
							OTHER_FILE = 6;

	private final static byte[][] oneBlankLine = {{}};

	//private	ByteTextReader r = new ByteTextReader();

//	public final JTabbedPane tab = new JTabbedPane();
	
	private final OpenTab tabDef;

	public final CsvSelectionPanel csvDetails;
	public final CsvSelectionPanel unicodeCsvDetails;
	public final XmlSelectionPanel xmlSelectionPanel;
	public final FixedWidthSelectionPane fixedWidthPanel;
	public final SchemaSelection schemaSelectionPanel;
	public final RegExpPreview regExpPreview;
	public final OtherSelection otherSelectionPanel;

	private JTextComponent msgFld;

//	private IFilePreview[] csvPanels = {null, null, null, null, null, null};

	private final boolean fixedTab, xmlTab;

	private ReFrame parentFrame = null;


	/**
	 * Class that holds a JTabbedPane of Normal / Unicode
	 * CSV preview panels.
	 * @param msgField message field to display error messages on
	 */
	public CsvTabPane(JTextComponent msgField, boolean allowNonCsvTabs, boolean adjustableTblHeight) {
		this(msgField, allowNonCsvTabs, allowNonCsvTabs, adjustableTblHeight, false, null);
	}

	/**
	 * Class that holds a JTabbedPane of Normal / Unicode
	 * CSV preview panels.
	 * @param msgField message field to display error messages on
	 */
	public CsvTabPane(JTextComponent msgField, boolean allowFixed, boolean allowXml, boolean adjustableTblHeight,
					boolean showGenBtn, Action wizardAction) {

		msgFld = msgField;
		fixedTab = allowFixed;
		xmlTab = allowXml;

		ArrayList<TabItem> pnls = new ArrayList<TabItem>();
		String sysId = Parameters.AP_RECSVEDITOR;

		csvDetails        = new CsvSelectionPanel(oneBlankLine, "", false, "File Preview", 
									msgField, adjustableTblHeight,  showGenBtn, wizardAction);
		unicodeCsvDetails = new CsvSelectionPanel(oneBlankLine[0], "", false, "Unicode File Preview", 
									msgField, adjustableTblHeight, showGenBtn, wizardAction);

		pnls.add(new TabItem(NORMAL_CSV, csvDetails, "Normal Csv", "Normal Ascii Csv"));
		pnls.add(new TabItem(UNICODE_CSV, unicodeCsvDetails, "Unicode Csv",  "Csv file (specify the characterset)"));

/*		csvPanels[NORMAL_CSV]  = csvDetails;
		csvPanels[UNICODE_CSV] = unicodeCsvDetails;
	
		tab.addTab("Normal Csv",  null, csvDetails.getPanel(),        "Normal Ascii Csv");
		tab.addTab("Unicode Csv", null, unicodeCsvDetails.getPanel(), "Csv file (specify the characterset)");*/

		if (fixedTab) {
			fixedWidthPanel = FixedWidthSelectionPane.newPane(sysId);
			schemaSelectionPanel = new SchemaSelection(sysId, msgFld);
			//csvPanels[XML_FILE] = new XmlSelectionPanel("Xml Preview", msgField);
			//tab.add("Xml", xmlPanel);
			//csvPanels[XML_FILE].getPanel().setVisible(false);

			pnls.add(new TabItem(FIXED_WIDTH, fixedWidthPanel, "Fixed Width", "Basic Fixed Width File"));
			pnls.add(new TabItem(SCHEMA, schemaSelectionPanel, "Schema",  "View files using a Xml Schema or Cobol Copybook"));
//			csvPanels[FIXED_WIDTH] = fixedWidthPanel;
//			csvPanels[SCHEMA] = schemaSelectionPanel;
//			tab.addTab("Fixed Width", null, csvPanels[FIXED_WIDTH].getPanel(),
//					"Basic Fixed Width File");
//			tab.addTab("Schema", null, csvPanels[SCHEMA].getPanel(), "View files using a Xml Schema or Cobol Copybook");
		} else {
			schemaSelectionPanel = null;
			fixedWidthPanel = null;
		}
		
		
		RegExpPreview tmpRegExpPreview = null;
		if (xmlTab) {
			xmlSelectionPanel = new XmlSelectionPanel(sysId, "Xml Preview", msgFld, true);
			otherSelectionPanel = new OtherSelection(sysId, msgFld, true);
			//csvPanels[XML_FILE] = new XmlSelectionPanel("Xml Preview", msgField);
			//tab.add("Xml", xmlPanel);
			//csvPanels[XML_FILE].getPanel().setVisible(false);

			pnls.add(new TabItem(XML_FILE, xmlSelectionPanel, "Xml", "View Xml files (suitable for Xml data files)"));
			if (fixedTab) {
				tmpRegExpPreview = new RegExpPreview(sysId, msgFld, true, new JTable());
				pnls.add(new TabItem(REGEXP_FILE, tmpRegExpPreview, "Regular Expression", 
						"Split a line using a regular expresion"));
			}
			pnls.add(new TabItem(OTHER_FILE, otherSelectionPanel, "Other", 
					"View edit either: SwingX Tip files or PO Language translation files"));

//			csvPanels[XML_FILE] = xmlSelectionPanel;
//			tab.addTab("Xml", null, csvPanels[XML_FILE].getPanel(), "View Xml files (suitable for Xml data files)");
//			csvPanels[OTHER_FILE] = otherSelectionPanel;
//			tab.addTab("Other", null, csvPanels[OTHER_FILE].getPanel(),
//					"View edit either: SwingX Tip files or PO Language translation files");
		} else {
			xmlSelectionPanel = null;
			otherSelectionPanel = null;
		}
		regExpPreview = tmpRegExpPreview;
		
		tabDef = new OpenTab(pnls);
	}

	public void addEditAction(ActionListener editAction) {
		tabDef.addEditAction(editAction);
	}

	public void readOtherTab(String filename, byte[] data) throws IOException {
		try {
			switch (tabDef.getSelectedCode()) {
			case NORMAL_CSV:
				unicodeCsvDetails.setData(filename, data, false, null);
				break;
			case UNICODE_CSV:
				setNormalCsv(filename, data);
				break;
			}
		} catch (Exception e) {
			msgFld.setText("Only one tab is set: " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void readCheckPreview(File f, boolean allowTabSwap, String layoutId) {

		if ((f != null) && (f.isFile()) ) {
			String fileName = f.getPath();
			if (schemaSelectionPanel != null) {
				schemaSelectionPanel.setFilename(fileName);
			}

			try {
				readCheckPreview(readFile(fileName), allowTabSwap, fileName, layoutId);
			} catch (IOException ex) {
				Common.logMsg("Error Reading File:", ex);
			}
		}
	}

	public void readCheckPreview(byte[] data, boolean allowTabSwap, String filename, String layoutId)
	throws IOException {
		String charSet = CheckEncoding.determineCharSet(data).charset;
		boolean couldBeXml = this.xmlTab && maybeXml(data, charSet);

		if (allowTabSwap) {
			if (layoutId != null // && ! "".equals(layoutId)
			&& filename != null && ! filename.toLowerCase().endsWith(".csv")) {
				if (tabDef.findTab(layoutId, filename, data, IFilePreview.NO) >= 0) {
					return;
				}
//				int count = Math.min(csvPanels.length, tab.getTabCount());
//				int likely = IFilePreview.NO;
//				int tabIdx = -1;
//				int likelyVal;
//				for (int i = 0; i < count; i++) {
//					if (csvPanels[i] != null 
//					&&  (likelyVal = csvPanels[i].isMyLayout(layoutId, filename, data)) > likely) {
//						tabIdx = i;
//						likely = likelyVal;
//						//Rectangle tabSize =  tab.getBounds();
//						//System.out.println(" !! 1 " + tab.getPreferredSize().height + " " + tab.getHeight());
//						//csvPanels[i].setData(filename, data, false, layoutId);
//						//tabDef.setTabCode(i);
//						//tab.setBounds(tabSize);
//						//System.out.println(" !! 2 " + tab.getPreferredSize().height + " " + tab.getHeight());
//						//return;
//					}
//				}
//				if (tabIdx >= 00) { 
//					tabDef.setTabCode(tabIdx);
//					return;
//				}
			}
			if (xmlTab){
				if (couldBeXml) {
					//csvPanels[XML_FILE].getPanel().setVisible(true);
					tabDef.setTabCode(XML_FILE);
					//System.out.println("Setting Xml Tab ");
//				} else if (tab.getTabCount() >= XML_FILE){
					//csvPanels[XML_FILE].getPanel().setVisible(false);
				} else if (tabDef.getSelectedCode() >= XML_FILE) {
					tabDef.setTabCode(NORMAL_CSV);
				}
			}


			switch (tabDef.getSelectedCode()) {
			case NORMAL_CSV:
			case UNICODE_CSV:
			case XML_FILE:
				if (! couldBeXml
				&& xmlTab
				&& Common.OPTIONS.csvSearchFixed.isSelected()
				&& FileAnalyser.getTextPct(data, "") < 50
				&& FileAnalyser.getTextPct(data, charSet) < 50) {
					tabDef.setTabCode(SCHEMA);
				}
				break;
			case FIXED_WIDTH:
			case SCHEMA:
				if (FileAnalyser.getTextPct(data, "") >= 50
				&& FileAnalyser.getTextPct(data, charSet) >= 50) {
					int idx = UNICODE_CSV;
					if ("".equals(charSet)) {
						idx = NORMAL_CSV;
					}
					tabDef.setTabCode(idx);
				} else if (tabDef.getSelectedCode() == FIXED_WIDTH 
					   && (fixedWidthPanel.isMyLayout("", filename, data) == IFilePreview.NO) ) {
					tabDef.setTabCode(SCHEMA);
				}
			}
		}


		switch (tabDef.getSelectedCode()) {
		case NORMAL_CSV:
			if (allowTabSwap && ! "".equals(charSet)) {
				unicodeCsvDetails.setCharset(charSet);
				unicodeCsvDetails.setData(filename, data, true, layoutId);

				checkNormalCsv(filename, data);
			} else {
				setNormalCsv(filename, data);
			}
			break;
		case UNICODE_CSV:
			if (allowTabSwap && "".equals(charSet)) {
				setNormalCsv(filename, data);
				tabDef.setTabCode(NORMAL_CSV);
			} else {
				unicodeCsvDetails.setCharset(charSet);
				unicodeCsvDetails.setData(filename, data, false, layoutId);

				if (allowTabSwap) {
					checkNormalCsv(filename, data);
				}
			}
			break;
//		case FIXED_WIDTH:
//		case SCHEMA:
//			//long time1 = System.nanoTime();
//			tabDef.getSelectedFilePreview().setData(filename, data, false, layoutId);
//			//System.out.println("Time Used: " + (System.nanoTime() - time1) +  " - " + time1);
//			break;
		case XML_FILE:
			if (couldBeXml) {
				xmlSelectionPanel.setData(filename, data, false, layoutId);

				if (parentFrame != null) {
					parentFrame.pack();

					System.out.println("Done pack ....");
				}
			} else {
				setNormalCsv(filename, data);
				checkNormalCsv(filename, data);
			}
			break;
//			case OTHER_FILE:
//				otherSelectionPanel.setData(filename, data, false, layoutId);
		default:
			tabDef.getSelectedFilePreview().setData(filename, data, false, layoutId);
			break;
		}
	}

//	private void setData(byte[] data, String filename, String layoutId) {
//
//		switch (tab.getSelectedIndex()) {
//		case NORMAL_CSV:
//			setNormalCsv(data);
//			break;
//		case UNICODE_CSV:
//			unicodeCsvDetails.setData(filename, data, false, layoutId);
//
//			break;
//		case FIXED_FILE:
//			//long time1 = System.nanoTime();
//			csvPanels[FIXED_FILE].setData(filename, data, false, layoutId);
//			//System.out.println("Time Used: " + (System.nanoTime() - time1) +  " - " + time1);
//			break;
//		case XML_FILE:
//
//				csvPanels[XML_FILE].setData(filename, data, false, layoutId);
//
//				if (parentFrame != null) {
//					parentFrame.pack();
//				}
//
//			break;
//		}
//
//	}

	private boolean maybeXml(byte[] data, String charSet) {
		byte[] check = Conversion.getBytes("<", charSet);
		if (data == null || data.length < check.length) return false;

		for (int i = 0; i < check.length; i++) {
			if (check[i] != data[i]) {
				return false;
			}
		}
		return true;
	}

	private void checkNormalCsv(String filename, byte[] data) throws IOException {
		if (unicodeCsvDetails.getColumnCount() == 1) {
			setNormalCsv(filename, data);
			if (csvDetails.getColumnCount() > 1) {
				tabDef.setTabCode(NORMAL_CSV);
				return;
			}
		}
		tabDef.setTabCode(UNICODE_CSV);
	}

	private void setNormalCsv(String filename, byte[] data)  {
//		byte[] line;
//		byte[][] lines = new byte[30][];
//		int i = 0;
//
//		r.open(new ByteArrayInputStream(data));
//		while (i < lines.length && (line = r.read()) != null) {
//			lines[i++] = line;
//		}
//		r.close();
//		csvDetails.setLines(lines, "", lines.length);
		csvDetails.setData(filename, data, false, "");
	}


	private byte[] readFile(String fileName)
	throws IOException {
		byte[] data;
		FileInputStream in = null;
		
		try {
			in = new FileInputStream(fileName);
			if (fileName.toLowerCase().endsWith(".gz")) {
				GZIPInputStream inGZip = new GZIPInputStream(in);
				data = StreamUtil.read(inGZip, 16000);
				inGZip.close();
			} else {
				data = StreamUtil.read(in, PREVIEW_SIZE); //new byte[8000];
			}
		} finally {
			if (in != null) {
				in.close();
			}
		}

	    return data;
	}


//
//	private int toTabIndex(int idx) {
//		return Math.max(idx, tab.getTabCount() - 1);
//	}
//
//
//	private int tabIndexToConst(int idx) {
//		return Math.min(idx, tab.getTabCount() - 1);
//	}
//


	public void setRecordLayout(int layoutId, String layoutName) {
		try {

			if (layoutName != null && ! "".equals(layoutName)) {
				if (layoutName.startsWith(CsvSelectionPanel.NORMAL_CSV_STRING)) {
					csvDetails.setFileDescription(layoutName);
					tabDef.setTabCode(NORMAL_CSV);
				} else {
					unicodeCsvDetails.setFileDescription(layoutName);
					tabDef.setTabCode(UNICODE_CSV);
				}
			}
		} catch (Exception e) {
			// TODO: handle exception
		}
	}

	

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.openFile.FormatFileName#formatLayoutName(java.lang.String)
	 */
	@Override
	public String formatLayoutName(String layoutName) {
		return layoutName;
	}

	public IFilePreview getSelectedCsvDetails() {
		return tabDef.getSelectedFilePreview();
	}

	public void setParentFrame(ReFrame parentFrame) {
		this.parentFrame = parentFrame;
	}
	
	public final void setGoVisible(boolean visible) {
		csvDetails.goBtn.setVisible(visible);
		unicodeCsvDetails.goBtn.setVisible(visible);
	}
//
//	public LayoutDetail getLayout(String font, byte[] recordSep) {
//		return getSelectedCsvDetails().getLayout(font, recordSep);
//	}
	public JTabbedPane getTabPane() {
		return tabDef.tab;
	}

	public int getSelectedCode() {
		return tabDef.getSelectedCode();
	}

	public void setTabCode(int code) {
		tabDef.setTabCode(code);
	}

	public MouseListener getPopAction() {
		return tabDef.getPopAction();
	}
}
