package net.sf.RecordEditor.re.util.csv;

import java.awt.Component;
import java.awt.GridLayout;
import java.io.ByteArrayInputStream;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.text.JTextComponent;

import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.RecordEditor.re.file.FieldInViewIterator;
import net.sf.RecordEditor.re.file.FileView;
import net.sf.RecordEditor.re.schema.ICreateFieldIterator;
import net.sf.RecordEditor.utils.common.Common;

public class FileTableDisplay  implements ICreateFieldIterator, IGetFileTable {

	//int fileStructure;
	final IFileStructureChanged fileStructureChangedListner;
	String fontName = "";
	
	private FileView fileTblMdl;
	
	public final JTable fileTable;
	@SuppressWarnings("serial")
	private static AbstractTableModel emptyModel = new AbstractTableModel() {
		@Override public Object getValueAt(int rowIndex, int columnIndex) { return null; }
		@Override public int getRowCount() { return 0; }
		@Override public int getColumnCount() { return 0; }
	};

	public FileTableDisplay(JTable fileTable, IFileStructureChanged fileStructureChangedListner) {
		this.fileTable = fileTable;
		this.fileStructureChangedListner = fileStructureChangedListner;
		fileTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
	}




	@SuppressWarnings("unchecked")
	public boolean buildFileLayout(AbstractLayoutDetails layout, JTextComponent msg, byte[] data) {
		
		try {	
			msg.setText("");
			if (layout == null) {
				msg.setText("No Layout Generated !!!");
				fileTable.setModel(emptyModel);
				emptyModel.fireTableDataChanged();
				return false;
			} else {
				//System.out.println("-->>" +layout.getFileStructure());
				switch (layout.getFileStructure()) {
				case IFileStructureConstants.IO_GENERIC_CSV:
				case IFileStructureConstants.IO_WIZARD:
				case IFileStructureConstants.IO_UNKOWN_FORMAT:
					//msg.setText("Layout is built during reading - no preview");
					fileTable.setModel(emptyModel);
					emptyModel.fireTableDataChanged();
					return false;
				}
			}
			int fileStructure = layout.getFileStructure();
			if (fileStructureChangedListner != null) {
				fileStructureChangedListner.fileStructureChanged(fileStructure);
			}
			fontName = layout.getFontName();

			if (data == null) {return false;}
			
			ByteArrayInputStream is = new ByteArrayInputStream(data);
			LineIOProvider iop = LineIOProvider.getInstance();

			@SuppressWarnings("rawtypes")
			AbstractLineReader reader = iop.getLineReader(layout);// iop.getLineReader(fileStructure);
			fileTblMdl = new FileView(layout, iop, false);
			AbstractLine l;
			int i=0;

			try {
				reader.open(is, layout);
				while ( i++ < 60 && (l = reader.read()) != null) {
					fileTblMdl.add(l);
				}
				layout = reader.getLayout();
				fileTblMdl.setLayout(layout);
			} catch (Exception e) {

			} finally {
				reader.close();
				is.close();
			}

			assignModelToTbl();

		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}
	
	public void assignModelToTbl() {
		
		if (fileTblMdl == null) { return; }
		
		fileTable.setModel(fileTblMdl);
		fileTblMdl.fireTableStructureChanged();

		TableColumnModel tcm = fileTable.getColumnModel();
		TableColumn tc = tcm.getColumn(0);
		HeaderRender headerRender = new HeaderRender();

        for (int i = 2; i < tcm.getColumnCount(); i++) {
            tcm.getColumn(i).setHeaderRenderer(headerRender);
        }
		if (tc != null) {
			fileTable.getColumnModel().removeColumn(tc);
		}
	}


	/**
	 * @return the fileTable
	 */
	@Override
	public JTable getFileTable() {
		return fileTable;
	}
	
	
	@Override
	public Iterator<String> getFieldIterator(int recordIdx, int fieldIdx) {
		return new FieldInViewIterator(this.fileTblMdl, recordIdx, fieldIdx);
	}

	
	public static interface IFileStructureChanged {
		
		void fileStructureChanged(int newFileStructure);
	}

	@SuppressWarnings("serial")
	private static class HeaderRender extends JPanel implements TableCellRenderer {

        /**
         * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent
         * 		(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
         */
        public Component getTableCellRendererComponent(
            JTable tbl,
            Object value,
            boolean isFldSelected,
            boolean hasFocus,
            int row,
            int column) {

            removeAll();
            setLayout(new GridLayout(2, 1));

            if (column >= 0 && value != null) {

                String s = (String) value;
                String first = s;
                String second = "";
                int pos = s.indexOf(Common.COLUMN_LINE_SEP);
                if (pos > 0) {
                    first = s.substring(pos + 1);
                    second = s.substring(0, pos);
                }
                JLabel label = new JLabel(first);
                add(label);
                if ((!second.equals(""))) {
                    label = new JLabel(second);

                    add(label);
                }
            }
            this.setBorder(BorderFactory.createEtchedBorder());

            return this;
        }
    }



}
