package net.sf.RecordEditor.re.util.csv;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.JTextComponent;

import net.sf.JRecord.Common.CommonBits;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.CsvParser.ICsvCharLineParser;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.IGetSchema;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.External.CobolCopybookLoader;
import net.sf.JRecord.External.CopybookLoaderFactory;
import net.sf.JRecord.External.ExternalRecord;
import net.sf.JRecord.External.Def.ExternalField;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.JRecord.Log.AbsSSLogger;
import net.sf.JRecord.Option.ICobolSplitOptions;
import net.sf.JRecord.Types.Type;
import net.sf.JRecord.parser.CharParserManager;
import net.sf.RecordEditor.layoutWizard.WizardFileLayout;
import net.sf.RecordEditor.re.cobol.CobolFileStructureComboContainer;
import net.sf.RecordEditor.re.layoutSelection.ComputerOptionCombo;
import net.sf.RecordEditor.re.util.openTabPane.ISchemaSelection;
import net.sf.RecordEditor.utils.BasicLayoutCallback;
import net.sf.RecordEditor.utils.charsets.FontCombo;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.fileAnalysis.Details;
import net.sf.RecordEditor.utils.fileAnalysis.FileAnalyser;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReFrame;
import net.sf.RecordEditor.utils.screenManager.ReMainFrame;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChanged;
import net.sf.RecordEditor.utils.swing.treeCombo.FileSelectCombo;
import net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo;
import net.sf.recordEditor.ui.icons.IconManager;


/**
 * This class lets the user select a <i>file schema</i> that defines a file
 * 
 * @author Bruce Martin
 *
 */
public class SchemaSelection extends  BaseFilePreview
implements BasicLayoutCallback, ChangeListener, FileTableDisplay.IFileStructureChanged, ISchemaSelection {

	private static final int SRC_OTHER = 0;
	private static final int SRC_FONT = 1;
	private static final int SRC_FILE_STRUCTURE = 2;
//	private static final int SRC_DIALECT = 3;
	private static final int SRC_NO_UPDATE = 9;
	
	//private static final String SCHEMA_ID = "SCHEMA";
//	private BaseHelpPanel pnl = new BaseHelpPanel();

	private IFileSelectCombo layoutFile = FileSelectCombo.newCombo(Parameters.SCHEMA_LIST, 25, true, false);
	private final JLabel dialectLbl = new JLabel("Cobol Dialect");
	private final JLabel    fontLbl = new JLabel("Font");
	private final JLabel fileOrganisationLbl = new JLabel("File Organisation");
	private final ComputerOptionCombo dialectCombo = new ComputerOptionCombo();
	private final FontCombo fontCombo = new FontCombo();
	
//	private final BmKeyedComboModel structureModel = new BmKeyedComboModel(new ManagerRowList(
//			LineIOProvider.getInstance(), false));
//	private final BmKeyedComboBox fileStructureCombo = new BmKeyedComboBox(structureModel, false);
	
	CobolFileStructureComboContainer fileStructCombo = new CobolFileStructureComboContainer();


	private JButton editBtn = SwingUtilsBldr.newButton("Edit");
	private JButton schemaWizardBtn = SwingUtilsBldr.newButton("Wizard", Common.getRecordIcon(Common.ID_WIZARD_ICON));
	
//
//	private JTable fileTable = new JTable();
	private final FileTableDisplay fileTblDisplay;

	private  JTextComponent msg;

//	private String fontName = "";
	private String filename = "";
//	private int fileStructure;ICreateFieldIterator
//	private int recordLength = 0;

	private byte[] lastData;
//	private String lastLayoutFile = null;
	private final String defaultCopybookDir = Parameters.getFileName(Parameters.COPYBOOK_DIRECTORY);
	
	private int schemaType = -1,	copybookFormat = -1;
	private String lastLayoutFile;
	private boolean layoutBuildFailed = false;
	
//	private boolean doingChange = false;
	
//	private boolean externalTbl = true;


	/**
	 * This class lets the user select a <i>file schema</i> that defines a file
	 * 
	 * @param message message field where message are to be written
	 */
	public SchemaSelection(String pgmId, JTextComponent message) {
		this(pgmId, message, true, new JTable());
//		externalTbl = false;
	}
	
	public SchemaSelection(String pgmId, JTextComponent message, boolean addTable, JTable table) {
		super(pgmId + Parameters.OP_TAB_SCHEMA, Parameters.AP_RECORD_EDITOR.equals(pgmId));

		this.msg = message;
		this.fileTblDisplay = new FileTableDisplay(table, this);
		super.setCreateFieldIterator(fileTblDisplay);

//		fileTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		layoutFile.setFileName(defaultCopybookDir);

//		System.out.println();
//		System.out.println("~> " + layoutFile.getPreferredWidth() + " " + layoutFile.getMinimumWidth());
//		System.out.println("=> " + layoutFile.getPreferredSize().width + " " + layoutFile.getMinimumSize().width);
//		System.out.println("*> " + dialectCombo.getPreferredSize().width + " " + dialectCombo.getMinimumSize().width);
	
		pnl.setFieldsToFullSize();
		pnl.addLineRE(      "Layout File", layoutFile, schemaWizardBtn)
													.setFieldsToActualSize();

		pnl.addLineRE(         dialectLbl, dialectCombo) 
		   .addLineRE(            fontLbl, fontCombo)
		   .addLineRE(fileOrganisationLbl, fileStructCombo.cobolFileStrucures)
											//.setGapRE(BasePanel.GAP0)
													.setFieldsToFullSize();
		pnl.addLineRE(               null, BasePanel.NULL_ITEM,   editBtn);

		if (addTable) {
			pnl.addComponentRE(
					1,  BasePanel.LAST_USED_COLUMN, BasePanel.FILL, BasePanel.GAP0,
			        BasePanel.FULL, BasePanel.FULL,
			        fileTblDisplay.getFileTable());
		}
		
		layoutFile.addFileChangedListner(new IFileChanged() {
			@Override public void fileChanged(File selectedFile) {
				//if (lastLayoutFile == null || ! lastLayoutFile.equals(layoutFile.getText())) {
				//}
				setCobolVisiblity();
				buildFileLayout(lastData, SRC_OTHER);
			}
		});


		schemaWizardBtn.addActionListener(new ActionListener() {
			@Override public void actionPerformed(ActionEvent e) {
				new WizardFileLayout(new ReFrame("", "File Wizard", "", null), filename, SchemaSelection.this, false, true);		
			}
		});
		fontCombo.addTextChangeListner(this);

		setCobolVisiblity();
		
		ActionListener comboListner = new ActionListener() {		
			@Override public void actionPerformed(ActionEvent e) {
				updateFileLayout(lastData );
			}
		};
		dialectCombo.addActionListener(comboListner);
		fileStructCombo.cobolFileStrucures.addActionListener(comboListner);
	}

	@Override
	public void fileStructureChanged(int newFileStructure) {
		
		if (newFileStructure != fileStructCombo.getFileStructure()) {
			fileStructCombo.setFileStructure(newFileStructure);
			System.out.println(" ## " + newFileStructure);
		}
	}

	@Override public void stateChanged(ChangeEvent e) {
		updateFileLayout(lastData);
	}
	
//	@Override
//	public BaseHelpPanel getPanel() {
//		return pnl;
//	}

	@Override
	public JButton getGoButton() {
		return editBtn;
	}

	@Override
	public boolean setData(String filename, byte[] data, boolean checkCharset, String layoutId) {
		
		this.filename = filename;
		if (lastData == data) {
			return true;
		}

		if (layoutId != null && ! "".equals(layoutId)) {
			setFileDescription(layoutId);
		}

		lastData = data;
		return buildFileLayout(data, SRC_OTHER);
	}

	private boolean updateFileLayout(byte[] data) {
		return fileTblDisplay.buildFileLayout(getLocalLayout(fontCombo.getText(), data, SRC_NO_UPDATE), msg, data);
	}


	private boolean buildFileLayout(byte[] data, int source) {
		return fileTblDisplay.buildFileLayout(getLocalLayout(null, data, source), msg, data);
	}

//	@Override
//	public String getSeperator() {
//		return "";
//	}
//
//	@Override
//	public String getQuote() {
//		return "";
//	}

	@Override
	public boolean setLines(byte[][] newLines, String font, int numberOfLines) {

		return false;
	}

	@Override
	public void setLines(String[] newLines, String font, int numberOfLines) {

	}


	@Override
	public LayoutDetail getLayout(String font, byte[] recordSep, int requestType) {
		LayoutDetail l = getLocalLayout(font, null, SRC_OTHER);
		if (l != null || requestType == IGetSchema.SCHEMA_4_GEN) {
			return l;
		} else if (layoutBuildFailed) {
			return null;
		}

		JDialog d = new JDialog(ReMainFrame.getMasterFrame(), true);
		WizardFileLayout wiz = new WizardFileLayout(d, filename, this, false, false);

		Details details = wiz.getWizardDetails();
		details.fileStructure = fileStructCombo.getFileStructure();
		details.fontName = font;
//		details.recordLength = recordLength;
		details.generateFieldNames = true;

		wiz.changePanel(WizardFileLayout.FORWARD, false);
		if (wiz.getPanelNumber() == 1
		&& (details = wiz.getWizardDetails()) != null
		&& (details.recordLength > 0 || details.fileStructure != IFileStructureConstants.IO_FIXED_LENGTH)) {
			wiz.changePanel(WizardFileLayout.FORWARD, false);
		}

		d.setVisible(true);

		ExternalRecord rec = wiz.getExternalRecord();
		if (rec != null) {
			try {
				return rec.asLayoutDetail();
			} catch (Exception e) {
				Common.logMsg(AbsSSLogger.ERROR, "Layout Generation Failed:", e.getMessage(), null);
			}
		}

		return null;
	}
	

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.util.csv.FilePreview#getSchemaCheckType()
	 */
	@Override
	public int getSchemaCheckType() {
		
		return IGetSchema.ST_NO_CHECK_ON_SCHEMA;
	}



	@Override
	public String getFileDescription() {

		int cobolDialect = dialectCombo.getSelectedIndex();
		if (dialectCombo.isVisible()) {
			Parameters.setSavePropertyChanges(false);
			Parameters.setProperty(Parameters.USES_COBOL, "Y");
			Parameters.setSavePropertyChanges(true);
			Parameters.setProperty(Parameters.LAST_COBOL_DIALECT, cobolDialect);
		}
		return SCHEMA_ID+ SEP + getStr(layoutFile.getText())
						+ SEP + cobolDialect
						+ SEP + getFontName()
						+ SEP + NULL_STR;
	}


	public void setFileDescription(String val) {
		StringTokenizer tok = new StringTokenizer(val, SEP, false);

		try {
			if (SCHEMA_ID.equalsIgnoreCase(tok.nextToken())) {
				layoutFile.setFileNameSilently(Parameters.expandVars(getStringTok(tok)));
				dialectCombo.setSelectedIndex(getIntTok(tok));
				fileTblDisplay.fontName = getStringTok(tok);
				setCobolVisiblity();
			}
		} catch (Exception e) {

		}
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.re.util.csv.FilePreview#isMyLayout(java.lang.String)
	 */
	@Override
	public int isMyLayout(String layoutId, String filename, byte[] data) {
		int ret = (layoutId != null && layoutId.startsWith(SCHEMA_ID)) ? YES : NO;
		
		this.filename = filename;
		if (filename != null && filename.length() > 0 && ret < YES) {
			File f = new File(filename);
			String schemaName = Common.OPTIONS.DEFAULT_COPYBOOK_DIRECTORY.getSlashNoStar() + f.getName() + ".xml";
			if (new File(schemaName) . exists()) {
				ret = YES;
				layoutFile.setFileName(schemaName);
				layoutId = null;
			}
		}
		if (ret > NO) {
			setData(filename, data, false, layoutId);
		}
		
		return ret;
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.BasicLayoutCallback#setRecordLayout(int, java.lang.String, java.lang.String)
	 */
	@Override
	public void setRecordLayout(int layoutId, String layoutName, String filename) {
		layoutFile.setFileNameSilently(layoutName);
		if (lastData != null) {
			buildFileLayout(lastData, SRC_OTHER);
		}
	}


	private LayoutDetail lastLayout;
	private LayoutDetail getLocalLayout(String font, byte[] data, int source) {

		layoutBuildFailed = false;
		if (lastLayout != null && data == null) {
			return lastLayout;
		}
		lastLayout = null;
		if (layoutFileExists()) {
			String layoutFileName = layoutFile.getText();
			try {
//					if (isCobol() && CobolCopybookLoader.isAvailable()) {
//						loader = new CobolCopybookLoader();
//						if (font == null && data != null) {
//							FileAnalyser analyser = FileAnalyser.getAnaylserNoLengthCheck(data, "");
//							font = analyser.getFontName();
//							fileStructure = analyser.getFileStructure();
//						}
//					} else {
//						loader = new RecordEditorXmlLoader();
//					}
				runCopybookAnalyser();
				if (schemaType >= 0) {
					int fileStructure = fileStructCombo.getFileStructure();
					ExternalRecord rec;
					if (schemaType == CopybookLoaderFactory.COBOL_LOADER && CobolCopybookLoader.isAvailable()) {
						if (font == null && data != null) {
							FileAnalyser analyser = FileAnalyser.getAnaylserNoLengthCheck(data, "");
							if (font == null && data != null) {
								font = analyser.getFontName();
							}
							fileStructure = analyser.getFileStructure();
							if (source != SRC_NO_UPDATE) {

								if (source != SRC_FILE_STRUCTURE) {
									fileStructCombo.setFileStructure(fileStructure);
									System.out.print(" -- " + fileStructure);
								}
								if (source != SRC_FONT) {
									try {
										fontCombo.removeTextChangeListner(this);
										fontCombo.setText(font);
										fontCombo.setFontList(analyser.getCharsetDetails().likelyCharsets);
									} finally {
										fontCombo.addTextChangeListner(this);
									}
								}
							}
							System.out.println();
						}
						rec = (new CobolCopybookLoader()).loadCopyBook(
								layoutFileName, ICobolSplitOptions.SPLIT_NONE, 0, 
								font, this.copybookFormat, dialectCombo.getSelectedValue(), 
								0, Common.getLogger());
						if (fileStructure >= 0) {
							rec.setFileStructure(fileStructure);
						}
					} else {
						rec = CopybookLoaderFactory.getInstance().getLoader(schemaType)
								.loadCopyBook(layoutFileName, 0, 0, font, dialectCombo.getSelectedValue(), 0, Common.getLogger());
						
						fileStructure = rec.getFileStructure();
						ICsvCharLineParser parser;
						int numberOfFields = rec.getNumberOfRecordFields();
						if (data != null
						&& rec.getNumberOfRecords() == 0
						&& numberOfFields < 2
						&& ! CommonBits.areFieldNamesOnTheFirstLine(fileStructure)
						&& rec.getRecordStyle() > 0 
						&& (parser = CharParserManager.getParser(rec.getRecordStyle(), rec.getLineFormatParam())) != null) {
							int fieldCount = 1;
							try {
								LayoutDetail schema = rec.asLayoutDetail();
								AbstractLineReader<LayoutDetail> reader = LineIOProvider.getInstance().getLineReader(schema);
								AbstractLine line;
								int lineNum = 0;
								
								reader.open(new ByteArrayInputStream(data), schema);
								
								while (lineNum++ < 100 &&(line =reader.read())!= null) {
									List<String> fieldList = parser.getFieldList(line.getFullLine(), schema.getRecord(0));
									if (fieldList != null) {
										fieldCount = Math.max(fieldCount, fieldList.size());
									}
								}
								
								reader.close();
							} catch (Exception e) {
								e.printStackTrace();
							}
							
							if (fieldCount > numberOfFields) {
								int type = Type.ftChar, format = 0;
								String param = "";
								if (numberOfFields > 0) {
									ExternalField recordField = rec.getRecordField(0);
									type = recordField.getType();
									format = recordField.getCellFormat();
									param = recordField.getParameter();
									recordField.setName("1");
								}
								for (int i = numberOfFields; i < fieldCount; i++) {
									rec.addRecordField(new ExternalField(i+1, -1, "" + (i + 1), "", type, 0, format, param, "", "", 0));
								}
								
								lastLayout = rec.asLayoutDetail();
								return rec.asLayoutDetail();
							}
						}
					}
					
					return rec.asLayoutDetail();
				}
			} catch (Exception e) {
				msg.setText("Layout Gen failed: " + e);
				e.printStackTrace();
				layoutBuildFailed = true;
			} 
			
		}
		return null;
	}

	private boolean layoutFileExists() {
		String layoutFileName = layoutFile.getText();
		boolean exists = false;
		if (layoutFileName != null && layoutFileName.length() > 0) {
			File f = new File(layoutFileName);

			exists = (f.exists() && ! f.isDirectory());
		}
		return exists;

	}
	private void setCobolVisiblity() {
		boolean cobol = isCobol();

		dialectCombo.setVisible(cobol);
		dialectLbl.setVisible(cobol);
		fontCombo.setVisible(cobol);
		fontLbl.setVisible(cobol);
		fileOrganisationLbl.setVisible(cobol);
		fileStructCombo.cobolFileStrucures.setVisible(cobol);
	}

	private boolean isCobol() {
		return CobolCopybookLoader.isAvailable()
			&& runCopybookAnalyser() == CopybookLoaderFactory.COBOL_LOADER;
	}

	private int runCopybookAnalyser() {
		String layoutFileName = layoutFile.getText();
		
		if (! layoutFileName.equals(lastLayoutFile)) {
			SchemaAnalyser analyser = new SchemaAnalyser(layoutFileName);
			schemaType = analyser.schemaType();
			copybookFormat = analyser.cobolCopybookFormat();
			lastLayoutFile = layoutFileName;
		}
		return schemaType;
	}

	private String getStr(String s) {
		if (s == null || "".equals(s)) {
			s = NULL_STR;
		}

		return s;
	}


	private String getStringTok(StringTokenizer tok) {

		String s = tok.nextToken();
		if (s == null || NULL_STR.equals(s)) {
			s = "";
		}
		return s;
	}

	private int getIntTok(StringTokenizer tok) {
		int ret = 0;
		try {
			ret = Integer.parseInt(tok.nextToken());
		} catch (Exception e) {
		}

		return ret;
	}

	@Override
	public String getFontName() {
		return fileTblDisplay.fontName;
	}


    public void setLayoutFile(String lFile) {
		this.layoutFile.setFileName(lFile);
	}


	public void setFilename(String filename) {
		if (!this.filename.equals(filename)) {
			this.layoutFile.setFileName(defaultCopybookDir);
		}
		this.filename = filename;
	}

//	/* (non-Javadoc)
//	 * @see net.sf.RecordEditor.re.util.csv.IFilePreview#getJTable()
//	 */
//	@Override
//	public JTable getJTable() {
//		return fileTblDisplay.fileTable;
//	}
	

//	/* (non-Javadoc)
//	 * @see net.sf.RecordEditor.re.util.csv.IFilePreview#notifyHasReceivedFocus()
//	 */
//	@Override
//	public void notifyHasReceivedFocus() {
//		
//	}
	
//	/* (non-Javadoc)
//	 * @see net.sf.RecordEditor.re.util.csv.IFilePreview#setHelpURL(java.net.URL)
//	 */
//	@Override
//	public void setHelpURL(URL helpUrl) {
//		getPanel().setHelpURLre(helpUrl);
//	}

	@Override
	public Icon getIcon() {
		return IconManager.ICONS.cobolIcon.icon();
	}

}
