package net.sf.RecordEditor.re.util.csv;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.sf.RecordEditor.re.jrecord.format.ICellFormat;

public class StdDateFormatStr implements Comparable<StdDateFormatStr>{

	private static final String[] SEPS = {"", "/", ".", "-", " "};

	public static final StdDateFormatStr NULL_DATE_FORMAT = new StdDateFormatStr("", "");
	public static final List<StdDateFormatStr> DATE_FORMATS = new ArrayList<StdDateFormatStr>(30);
	
	public static ICheckDateString CYYMMDD_CHECK = new ICheckDateString() {
		public boolean check(String dateStr) {
			try {
				int d = Integer.parseInt(dateStr);
				int cyy = d / 10000;
				int mm = (d / 100) % 100;
				int dd = d % 100;
				
				return cyy > 19 && cyy < 220 && mm > 0 && mm <= 12 && dd > 0 && dd < 32;
			} catch (Exception e) {
			}
			
			return false;
		}
	};
	
	static {
		load(DATE_FORMATS, "dd", "MM", "yy");
		load(DATE_FORMATS, "dd", "MM", "yyyy");
		load(DATE_FORMATS, "MM", "dd", "yy");
		load(DATE_FORMATS, "MM", "dd", "yyyy");
		load(DATE_FORMATS, "yy", "MM", "dd");
		load(DATE_FORMATS, "yyyy", "MM", "dd");
		
		DATE_FORMATS.add(new StdDateFormatStr("dd MMM yy", " "));
		DATE_FORMATS.add(new StdDateFormatStr("dd MMM yyyy", " "));
		DATE_FORMATS.add(new StdDateFormatStr("MMM dd yy", " "));
		DATE_FORMATS.add(new StdDateFormatStr("MMM dd yyyy", " "));
		DATE_FORMATS.add(new StdDateFormatStr("yy MMM dd", " "));
		DATE_FORMATS.add(new StdDateFormatStr("yyyy MMM dd", " "));
		
		DATE_FORMATS.add(new StdDateFormatStr("", "CYYMMDD",  "", ICellFormat.FMT_DATE_CYMD));
		
		Collections.sort(DATE_FORMATS);
		DATE_FORMATS.add(0, NULL_DATE_FORMAT);
	}
	
	
	/**
	 * Find the date Format
	 * @param format format identifier
	 * @param param date format string
	 * @return
	 */
	public static StdDateFormatStr getStandardFormat(int format, String param) {
		StdDateFormatStr dateFormat = StdDateFormatStr.NULL_DATE_FORMAT;
		if (format > 0) {
			List<StdDateFormatStr> dateFormats = StdDateFormatStr.DATE_FORMATS;
			for (int i = 1; i < dateFormats.size(); i++) {
				StdDateFormatStr dateFmt = dateFormats.get(i);
//				System.out.println(item.getFieldName() + "\t" + dateFmt.fieldFormat + " " + dateFmt.dateFormatStr);
				if ( dateFmt.fieldFormat == format 
				&& ( dateFmt.dateFormatStr.length() == 0 
				  || dateFmt.dateFormatStr.equals(param))) {
					dateFormat = dateFmt;
					break;
				}
			}
		}
		return dateFormat;
//		System.out.println(item.getFieldName() + "\t" + javaType + "\t" + format + "\t" + item.getParameter() );

	}
	
	private static void load(List<StdDateFormatStr> dates, String d1, String d2, String d3) {
		for (int i = 0; i < SEPS.length; i++) {
			dates.add(new StdDateFormatStr(d1 + SEPS[i] + d2 + SEPS[i] + d3, SEPS[i]));
		}
	}

	public final String dateFormatStr, dateDisplayStr, sep, dateName;
	
	public final boolean dayFirst, monthFirst, yyFirst, yyyyFirst, dayLast, monthLast, yyLast,
			monthMiddle, yyyyPresent;
	public final int fieldFormat, positionMM, idxStringMonth, idxYYYY;
	
	public StdDateFormatStr(String dateFormat, String sep) {
		this(dateFormat, dateFormat.toUpperCase(), sep, ICellFormat.FMT_DATE);
	}
	
	public StdDateFormatStr(String dateFormat, String dateName, String sep, int format) {
		super();
		this.dateFormatStr = dateFormat;
		this.dateDisplayStr = dateFormat.toUpperCase();
		this.dateName = dateName;
		this.sep = sep;
		this.fieldFormat = format;

		//int dateLength = dateDisplayStr.length();
		
		idxStringMonth = dateDisplayStr.indexOf("MMM") ;
		dayFirst  = dateDisplayStr.startsWith("DD");
		monthFirst= idxStringMonth < 0 && dateDisplayStr.startsWith("MM");
		monthLast = idxStringMonth < 0 && dateDisplayStr.endsWith("MM");
		monthMiddle = idxStringMonth < 0 && (! monthFirst) && (! monthLast);
		yyyyFirst = dateDisplayStr.startsWith("YYYY");
		yyFirst = (! yyyyFirst) && dateDisplayStr.startsWith("YY");
		dayLast   = dateDisplayStr.endsWith("DD"); 
		//yyyyLast  = dateDisplayStr.endsWith("YYYY"); 
		yyLast  = dateDisplayStr.endsWith("YY"); 
		yyyyPresent = dateDisplayStr.indexOf("YYYY") >= 0;
		positionMM =  idxStringMonth>=0 ? -1 : dateDisplayStr.indexOf("MM");
		//middleMM    = (!stringMonth) && positionMM > 0 &&  (positionMM < dateDisplayStr.length() - 2); 
		int idx = -1;
		if (! yyyyPresent) {
			
		} else if (yyyyFirst) {
			idx = 0;
		} else if (yyLast) {
			idx = 2;
		} else {
			idx = 1;
		}
		idxYYYY = idx;
	}
	
	public String toString() {
		return dateName;
	}
//	
//	public SimpleDateFormat toDateFormat() {
//		if (dateFormatStr.length() == 0) { return null; }
//
//		return  new SimpleDateFormat(dateFormatStr);
//	}
//	
	public ICheckDateString getDateCheck() {
		return fieldFormat == ICellFormat.FMT_DATE_CYMD
				? CYYMMDD_CHECK
				: new DateChk(dateFormatStr);
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(StdDateFormatStr o) {
		return dateDisplayStr.compareTo(o.dateDisplayStr);
	}
	
	public static class DateChk implements ICheckDateString {

		final SimpleDateFormat dateFormat;
		DateChk(String dateFmt) {
			dateFormat = new SimpleDateFormat(dateFmt);
			dateFormat.setLenient(false);
		}
	 	/* (non-Javadoc)
		 * @see net.sf.RecordEditor.re.util.csv.ICheckDateString#check(java.lang.String)
		 */
		@Override
		public boolean check(String dateStr) {
			try {
				dateFormat.parse(dateStr);
				
				return true;
				
			} catch (Exception e) { }
			return false;
		}
		
	}
}
