package net.sf.RecordEditor.re.util.openTabPane;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Arrays;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTabbedPane;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.INotifyOptionChanged;

public class OpenTab implements IDisplaySchemaScreen, INotifyOptionChanged {
	private static final int START_OF_OPTIONAL_ITEMS = 2;
	public final JTabbedPane tab = new JTabbedPane();
	private final TabItem[] tabItems, codeItems;
	private final List<TabItem> items;
//	private final int[]  toUser;
	private  MouseAdapter popupMenuAction;
	//private ISchemaSelection schemaSelection;
	private TabItem schemaTabItm; 
	
	public OpenTab(List<TabItem> items) {
		this.items = items;
		int arraySize = items.size();
		for (TabItem ti : items) {
			arraySize = Math.max(arraySize, ti.code);
		}
		this.tabItems = new TabItem[arraySize];
		this.codeItems = new TabItem[arraySize];
		//this.toUser = new int[arraySize];
		//Arrays.fill(this.toUser, -1);
		
		for (TabItem ti : items) {
			if (ti.isDisplayed) {
				addItem(ti);
			}
			codeItems[ti.code] = ti;
			if (ti.pnlDtls instanceof ISchemaSelection) {
				//schemaSelection = (ISchemaSelection) ti.pnlDtls;
				schemaTabItm = ti;
			}
			ti.displayThePnl().addListner(this);
		}
		
		if (schemaTabItm != null) {
			for (TabItem ti : items) {
				ti.pnlDtls.setDisplaySchemaScreen(this);
			}
		}
		
		if (items.size() > START_OF_OPTIONAL_ITEMS) {
			getPopAction();
		}
	}
	
	private void addItem(TabItem ti) {
		int idx = tab.getTabCount();
		tabItems[idx] = ti;
		String tagName = ti.tagName;
		Icon icon = ti.pnlDtls.getIcon();
		
		if (icon != null && ! Common.TEST_MODE) {
			tagName += " ";
		}
		tab.addTab(tagName, icon, ti.pnlDtls.getPanel(), ti.tagDescription);
		//toUser[ti.code] = idx;
		ti.isDisplayed = true;
		ti.tabIdx = idx;
		//System.out.println("~~~>> " + ti.tagName + " " + ti.code + " " + idx);
	}
	
	public void addEditAction(ActionListener editAction) {
		for (TabItem ti : items) {
			JButton goButton = ti.pnlDtls.getGoButton();
			if (goButton != null) {
				goButton.addActionListener(editAction);
			}
		}
	}
	public IFilePreview getSelectedFilePreview() {
		return getSelectedTabItem().pnlDtls;
	}
	
	public int getSelectedCode() {
		return getSelectedTabItem().code;
	}

	private TabItem getSelectedTabItem() {
		return tabItems[tab.getSelectedIndex()];
	}
	
	
	public void setTabCode(int code) {
		if (code >= 0 && code < tabItems.length) {
			int idx = addTab(code, false);
			if (idx >= 0) {
				tab.setSelectedIndex(idx);
			}
		}
	}
	
	@Override
	public void displaySchemaSelection(String schemaFileName) {
		if (schemaTabItm != null) {
			addTab(schemaTabItm.code, true);
		}
	}

	@Override
	public void optionChanged(Object option) {

		for (TabItem ti : items) {
			if (ti.displayThePnl() == option) {
				//System.out.println("~~ " + ti.code + " " + ti.displayThePnl().isSelected() + " ");
				if (ti.displayThePnl().isSelected()) {
					addTab(ti.code, false);
				} else {
					removeTab(ti.code);
				} 
			}
		}
	}

	private int addTab(int code, boolean updateStartup) {
		int idx = -1;
		TabItem tabItem = codeItems[code];
//		System.out.print(code + " " + ( tabItem != null));
//		if ( tabItem != null ) {
//			System.out.print(" " +  tabItem.tabIdx + " " +  tabItem.tagName);
//		}
		
		if ( tabItem != null && (idx = tabItem.tabIdx) < 0) {
			synchronized (this) {
				if (tabItem.tabIdx < 0) {
					addItem(tabItem);
					if (updateStartup) {
						tabItem.displayThePnl().set(true, false);
					}
				}
				idx = tabItem.tabIdx;
			}
		}
		//System.out.println();
		return idx;
	}


	private void removeTab(int code) {
		TabItem tabItem = codeItems[code];
		if (tabItem != null && tabItem.tabIdx >= 0) {
			synchronized (this) {
				int idx = tabItem.tabIdx;
				if (idx >= 0) {
					Arrays.fill(tabItems, null);
					tabItem.tabIdx = -1;
					tabItem.displayThePnl().set(false, false);
					for (TabItem ti : items) {
						if (ti.tabIdx > idx) {
							ti.tabIdx -= 1;
						}
						if (ti.tabIdx>= 0) {
							tabItems[ti.tabIdx] = ti;
						}
					}
					tab.remove(idx);
					//tabItem.displayThePnl().set(true, false);
					//System.out.println("Removed " + code + " " + tabItem.tagName + " " + idx);
				}
			}
		}
	}
	
	public int findTab(String layoutId, String filename, byte[] data, int likely) {
		int tabCode = -1;
		int likelyVal;
		for (int i =0; i <  items.size(); i++) { 
			TabItem ti = items.get(i);
			if ((likelyVal = ti.pnlDtls.isMyLayout(layoutId, filename, data)) > likely) {
				tabCode = i;
				likely = likelyVal;
			}
		}
		if (tabCode >= 00) { 
			this.setTabCode(tabCode);
			return tabCode;
		}
		return -1;
	}
	
	public MouseListener getPopAction() {
		if (popupMenuAction == null && items.size() > START_OF_OPTIONAL_ITEMS) {
			synchronized (this) {
				if (popupMenuAction == null) {
					popupMenuAction = new MouseAdapter() {
						JPopupMenu popup;
						@Override
						public void mousePressed(MouseEvent e) {
//							System.out.println("~~~~>> mouse pressed " + e.isPopupTrigger() + " " + e.isConsumed()
//					   		+ " " + e.getComponent().getClass().getName());
//						}
//						@Override
//						public void mouseReleased(MouseEvent e) {
//						   System.out.println("~~~~>> mouse mousePressed " + e.isPopupTrigger() + " " + e.isConsumed()
//						   		+ " " + e.getComponent().getClass().getName());
						   showPopup(e);
						   super.mousePressed(e);
						}
						protected void showPopup(MouseEvent e) {
							if (e != null && e.isPopupTrigger() && ! e.isConsumed()) {
							       	int x = e.getX(),
							       	    y = e.getY();
							        if (popup == null) {
							        	popup = new JPopupMenu("Show/Hide Tabs");
							        	
							        	for (int i = START_OF_OPTIONAL_ITEMS; i < items.size(); i++) {
							        		TabItem item = items.get(i);
							        		
//							        		System.out.println(" -- " + item.tagName);
											AddRempoveTabAction action = new AddRempoveTabAction(item.tagName, item.code);
											JCheckBoxMenuItem menuItem = new JCheckBoxMenuItem(action);
											action.menuItem = menuItem;
											menuItem.setSelected(item.tabIdx >= 0);
											//menuItem.set
											popup.add(menuItem);
							        	}
							        }
							        
							        popup.show(e.getComponent(),
							                   x, y);
							        e.consume();
							   }
						}
						public void mouseReleased(MouseEvent e) {
//							System.out.println("~~~~>> mouse mouseReleased " + e.isPopupTrigger() + " " + e.isConsumed()
//					   		+ " " + e.getComponent().getClass().getName());
							showPopup(e);
							super.mouseReleased(e);
						}
					};
				}
			}
			tab.addMouseListener(popupMenuAction);
			for (TabItem ti : items) {
				ti.pnlDtls.getPanel().addMouseListener(popupMenuAction);
			}
		}
		return popupMenuAction;
	}
	
	@SuppressWarnings("serial")
	private  class AddRempoveTabAction extends AbstractAction {
		final int code;
		JCheckBoxMenuItem menuItem;
		public AddRempoveTabAction(String name, int code) {
			super(name);
			this.code = code;
		}


		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (menuItem.isSelected()) {
				addTab(code, true);
			} else {
				removeTab(code);
			}
			
		}
		
	}
}
