package net.sf.RecordEditor.re.util.re;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.zip.GZIPInputStream;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.JTextComponent;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.IGetSchema;
import net.sf.RecordEditor.re.layoutSelection.def.IFormatFileName;
import net.sf.RecordEditor.re.layoutSelection.def.IStartActionInterface;
import net.sf.RecordEditor.re.layoutSelection.def.ITableChangedListner;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.re.openFile.RecentFiles;
import net.sf.RecordEditor.re.util.csv.CsvSelectionPanel;
import net.sf.RecordEditor.re.util.csv.OtherSelection;
import net.sf.RecordEditor.re.util.csv.SchemaSelection;
import net.sf.RecordEditor.re.util.csv.XmlSelectionPanel;
import net.sf.RecordEditor.re.util.openTabPane.IFilePreview;
import net.sf.RecordEditor.re.util.openTabPane.OpenTab;
import net.sf.RecordEditor.re.util.openTabPane.TabItem;
import net.sf.RecordEditor.re.util.regExp.RegExpPreview;
import net.sf.RecordEditor.utils.LayoutConnection;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.StreamUtil;
import net.sf.RecordEditor.utils.fileAnalysis.CsvAnalyser;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.screenManager.ReMainFrame;


/**
 *
 * @author Bruce Martin
 *
 * Class that holds a JTabbedPane of Normal / Unicode
 * CSV preview panels.
 */

public class ReTabPane implements IFormatFileName {
	private static final int PREVIEW_SIZE = 0x80000;

	public final static int LAYOUT  = 0,
							UNICODE_CSV = 1,
//							FIXED_WIDTH = 2,
							XML_FILE = 2,
							SCHEMA = 3,
							REGULAR_EXPRESSION = 4,
							OTHER_FILE = 5;

	private final static byte[][] oneBlankLine = {{}};

	//private	ByteTextReader r = new ByteTextReader();

//	public final JTabbedPane tab = new JTabbedPane();
	private final OpenTab tabDef;


	public final RecordEditorPreview recordEditorPreview;
	public final CsvSelectionPanel unicodeCsvDetails;
	public final XmlSelectionPanel xmlSelectionPanel;
//	public final FixedWidthSelectionPane fixedWidthPanel;
	public final RegExpPreview regExpPanel;
	public final SchemaSelection schemaSelectionPanel;
	public final OtherSelection otherSelectionPanel;

	private final JTextComponent msgFld;

//	private IFilePreview[] csvPanels = {null, null, null, null, null};
	private final ITableChangedListner tblChangeListner;

//	private ReFrame parentFrame = null;
	private JTable  previewTbl;
	//private final TreeComboFileSelect fileFld;
	
	private byte[] lastData;
	private String lastFileName;
	
	private final boolean addTblToPnl;
	private TableCellRenderer stdRender = new DefaultTableCellRenderer();
	private TableCellRenderer lastRender = stdRender;



	/**
	 * Class that holds a JTabbedPane of Normal / Unicode
	 * CSV preview panels.
	 * @param msgField message field to display error messages on
	 */
	public ReTabPane(
			LayoutConnection fileSource,
			final IUserLayoutSelection userLayoutSelection, 
			JTextArea message, 
			final JButton layoutCreate1,
			final JButton layoutCreate2, 
			ITableChangedListner tblChangeListner,
			IStartActionInterface startAction,
			Action wizardAction) {

		this.tblChangeListner = tblChangeListner;
		this.msgFld = message;
		this.previewTbl = new JTable();

		addTblToPnl = tblChangeListner == null;
		JTable csvTbl = previewTbl;
		JTable schemaTbl = previewTbl;
		JTable regExpTbl = previewTbl;
		if (addTblToPnl) {
			csvTbl = new JTable();
			schemaTbl = new JTable();
			regExpTbl = new JTable();
		}

		recordEditorPreview = new RecordEditorPreview(
				fileSource, userLayoutSelection, msgFld, layoutCreate1, layoutCreate2, addTblToPnl, previewTbl);
		recordEditorPreview.browse.addActionListener(new StartEditAction(true, startAction));
	
		String sysId = Parameters.AP_RECORD_EDITOR;
		unicodeCsvDetails    = new CsvSelectionPanel(oneBlankLine[0], "", false, true, "", 
									msgFld, true,  wizardAction, 
									addTblToPnl,  csvTbl, "Edit");
		xmlSelectionPanel    = new XmlSelectionPanel(sysId, "Xml Preview", msgFld, addTblToPnl);
		schemaSelectionPanel = new SchemaSelection(sysId, msgFld, addTblToPnl, schemaTbl);
		regExpPanel			 = new RegExpPreview(sysId, msgFld, addTblToPnl, regExpTbl);
		otherSelectionPanel  = new OtherSelection(sysId, msgFld, addTblToPnl);
	
		tabDef = init_100_BuildTabPane();
		init_300_Finalise(startAction);
	}


	/**
	 * @param fileFld
	 * @param cpyInterface
	 * @param layoutCreate1
	 * @param layoutCreate2
	 * @param tblChangeListner
	 * @param adjustableTblHeight
	 * @param wizardAction
	 */
	private OpenTab init_100_BuildTabPane() {
		//OpenTab tabDef;
		
		ArrayList<TabItem> tabs = new ArrayList<TabItem>();
		
		tabs.add(new TabItem(LAYOUT, recordEditorPreview, "Record Layout", "<html>Edit File using <b>Record Layout</b> definitions </html"));
		tabs.add(new TabItem(UNICODE_CSV, unicodeCsvDetails, "Csv", "Csv file (specify the characterset)"));
		tabs.add(new TabItem(XML_FILE, xmlSelectionPanel, "Xml", "View Xml files (suitable for Xml data files)"));
		tabs.add(new TabItem(SCHEMA, schemaSelectionPanel, "Schema/Cobol", "View Files with Cobol / Xml schema"));
		tabs.add(new TabItem(REGULAR_EXPRESSION, regExpPanel, "Regular Expresion",
				"View Files where lines are split by regular expressions"));
		tabs.add(new TabItem(OTHER_FILE, otherSelectionPanel, "Other", 
				"View edit either: SwingX Tip files or PO Language translation files"));
		return new OpenTab(tabs);
//		csvPanels[LAYOUT]  = recordEditorPreview;
//		csvPanels[UNICODE_CSV] = unicodeCsvDetails;
//
//	 
//		tab.addTab("Record Layout",  null, csvPanels[LAYOUT].getPanel(),        "Normal Ascii Csv");
//		tab.addTab("Csv", null, unicodeCsvDetails.getPanel(), "Csv file (specify the characterset)");
//
//		csvPanels[XML_FILE] = xmlSelectionPanel;
//		csvPanels[SCHEMA] = schemaSelectionPanel;
//		csvPanels[OTHER_FILE] = otherSelectionPanel;
//		tab.addTab("Xml", null, csvPanels[XML_FILE].getPanel(), "View Xml files (suitable for Xml data files)");
//		tab.addTab("Schema/Cobol", null, csvPanels[SCHEMA].getPanel(), "View Files with Cobol / Xml schema");
//		tab.addTab("Other", null, csvPanels[OTHER_FILE].getPanel(),
//				"View edit either: SwingX Tip files or PO Language translation files");
	}

	private JTable lastTbl = null;

	private void init_300_Finalise(IStartActionInterface startAction) {
		
		tabDef.addEditAction(new StartEditAction(false, startAction));
//		StartEditAction editAction = new StartEditAction(false, startAction);
//		for (IFilePreview p : csvPanels) {
//			if (p != null) {
//				p.getGoButton().addActionListener(editAction);
//			}
//		}
		
		tabDef.tab.addChangeListener(new ChangeListener() {
			@Override public void stateChanged(ChangeEvent e) {
				File f;
				if (lastFileName != null && (f = new File(lastFileName)) != null && f.exists()) {
					tabDef.tab.removeChangeListener(this);
	                try {
		                if (! addTblToPnl) {
		                	TableCellRenderer  render = /*csvPanels[tab.getSelectedIndex()]*/
		                			 getSelectedCsvDetails().getTableRenderer();
		                	render = render == null ? stdRender : render;
		                	
		                	if (render != lastRender) {
		                		previewTbl.setDefaultRenderer(Object.class, render);
		                		lastRender = render;
		                	}
		                }
						IFilePreview newActivePnl =  getSelectedCsvDetails(); //csvPanels[tab.getSelectedIndex()];
						
						newActivePnl.notifyHasReceivedFocus();
						if (lastData == null) {
							try {
								lastData = readFile(lastFileName);
							} catch (IOException e1) { }
						}
						newActivePnl.setData(lastFileName, lastData, true, "");
	                	
		                ReMainFrame.getMasterFrame().checkIfActionsAvailable();
		                if (tblChangeListner != null) {
							chgTable(newActivePnl);
		                }
		                
	                } finally {
	                	tabDef.tab.addChangeListener(this);
	                }
				}
			}
		});		
	}
	private void chgTable(IFilePreview newActivePnl) {
		JTable tbl =  newActivePnl.getJTable();
		
		if (tbl != lastTbl && tbl != null) {
			tblChangeListner.tableHasChanged(tbl);
		}
		
		lastTbl = tbl;
	}
	
	public void clearData() {
		lastData = null;
	}

	//private String lastFileName = "";
	public void readCheckPreview(File f, String layoutId) {

		if ((f != null) && (f.isFile()) ) {
			String fileName = f.getPath();

			try {
				byte[] fileData = lastData;
				if (fileData == null || ! lastFileName.equals(fileName)) {
					fileData= readFile(fileName);
				}
				readCheckPreview(fileData, fileName, layoutId);
			} catch (IOException ex) {
				Common.logMsg("Error Reading File:", ex);
			}
		}
	}

	public void readCheckPreview(byte[] data, String filename, String layoutId)
	throws IOException {
		
		lastFileName = filename;
		lastData = data;
		
		if (layoutId == null) {
			layoutId = "";
		}
		if (filename == null) {
			filename = "";
		}
		
		int tabIdx = tabDef.findTab(layoutId, filename, data, IFilePreview.NO);

		if (tabIdx == 0) {
			if (layoutId.toLowerCase().indexOf("generic csv - enter details") >= 0) {
				tabIdx = 1;
			} else if (layoutId.toLowerCase().indexOf("xml - build layout") >= 0) {
				tabIdx = 2;
			}
		}

		String lcFilename = filename.toLowerCase();
		int extensionIdx = -1;
		if (tabIdx == 0) {
		} else if (lcFilename.endsWith(".po")) {
			recordEditorPreview.setData(filename, data, true, RecentFiles.GET_TEXT_PO_LAYOUT);
			extensionIdx = 3;
		} else if (lcFilename.endsWith(".properties")) {
			recordEditorPreview.setData(filename, data, true, RecentFiles.TIP_LAYOUT);
			extensionIdx = 3;
		} else if (lcFilename.endsWith(".xml") || isBufferXml(data)) {
			recordEditorPreview.setData(filename, data, true, RecentFiles.XML_BUILD_LAYOUT);
			extensionIdx = 2;
		}
		
		if (tabIdx >= 0) {
			tabDef.setTabCode(tabIdx);
		} else  if (extensionIdx >= 0) {
			tabDef.setTabCode(extensionIdx);
		} else if (lcFilename.endsWith(".csv")
			   ||  CsvAnalyser.newCsvAnalyser(data).getLikelyCsv() == CsvAnalyser.LIKELY_TO_BE_CSV) {
			recordEditorPreview.setData(filename, data, true, RecentFiles.GENERIC_CSV_LAYOUT);
			//csvPanels[1].setData(filename, data, true,"");
			tabDef.setTabCode(1);	
		}
		
		IFilePreview filePreview = tabDef.getSelectedFilePreview();
		filePreview.setData(filename, data, true, layoutId);
		if (tblChangeListner != null) {
			chgTable(filePreview);
		}
	}
	
	
//	
//	private int getTabIdx(byte[] data, String filename, String layoutId) {
//		int count = Math.min(csvPanels.length, tab.getTabCount());
//		int tabIdx = -1;
//		int likely = IFilePreview.NO;
//		int likelyVal;
//		for (int i = 0; i < count; i++) {
//			if (csvPanels[i] != null 
//			&&  (likelyVal = csvPanels[i].isMyLayout(layoutId, filename, data)) > likely) {
//				tabIdx = i;
//				likely = likelyVal;
//				break;
//				//Rectangle tabSize =  tab.getBounds();
//				//System.out.println(" !! 1 " + tab.getPreferredSize().height + " " + tab.getHeight());
//				//csvPanels[i].setData(filename, data, false, layoutId);
//				//tab.setSelectedIndex(i);
//				//tab.setBounds(tabSize);
//				//System.out.println(" !! 2 " + tab.getPreferredSize().height + " " + tab.getHeight());
//				//return;
//			}
//		}
//		return tabIdx;
//	}
	
	private boolean isBufferXml(byte[] data) {
		if ( data == null || data.length <= 5) {return false;}
		String s = new String(data, 0, Math.min(400, data.length));
		int p;

		return s.startsWith("<")
			&& (p = s.indexOf('>')) > 0 
			&& s.indexOf('<', p) > 0
			;
			
 	}

//	private void setData(byte[] data, String filename, String layoutId) {
//
//		switch (tab.getSelectedIndex()) {
//		case NORMAL_CSV:
//			setNormalCsv(data);
//			break;
//		case UNICODE_CSV:
//			unicodeCsvDetails.setData(filename, data, false, layoutId);
//
//			break;
//		case FIXED_FILE:
//			//long time1 = System.nanoTime();
//			csvPanels[FIXED_FILE].setData(filename, data, false, layoutId);
//			//System.out.println("Time Used: " + (System.nanoTime() - time1) +  " - " + time1);
//			break;
//		case XML_FILE:
//
//				csvPanels[XML_FILE].setData(filename, data, false, layoutId);
//
//				if (parentFrame != null) {
//					parentFrame.pack();
//				}
//
//			break;
//		}
//
//	}

//	private boolean maybeXml(byte[] data, String charSet) {
//		byte[] check = Conversion.getBytes("<", charSet);
//		if (data == null || data.length < check.length) return false;
//
//		for (int i = 0; i < check.length; i++) {
//			if (check[i] != data[i]) {
//				return false;
//			}
//		}
//		return true;
//	}
//


	private byte[] readFile(String fileName)
	throws IOException {
		byte[] data;
		FileInputStream in = null;
		
		try {
			in = new FileInputStream(fileName);
			if (fileName.toLowerCase().endsWith(".gz")) {
				GZIPInputStream inGZip = new GZIPInputStream(in);
				data = StreamUtil.read(inGZip, 16000);
				inGZip.close();
			} else {
				data = StreamUtil.read(in, PREVIEW_SIZE); //new byte[8000];
			}
			lastData = data;
		} finally {
			if (in != null) {
				in.close();
			}
		}

	    return data;
	}


//
//	private int toTabIndex(int idx) {
//		return Math.max(idx, tab.getTabCount() - 1);
//	}
//
//
//	private int tabIndexToConst(int idx) {
//		return Math.min(idx, tab.getTabCount() - 1);
//	}
//

//
//	public void setRecordLayout(int layoutId, String layoutName) {
//		try {
//
//			if (layoutName != null && ! "".equals(layoutName)) {
//				if (layoutName.startsWith(CsvSelectionPanel.NORMAL_CSV_STRING)) {
//					csvDetails.setFileDescription(layoutName);
//					tab.setSelectedIndex(NORMAL_CSV);
//				} else {
//					unicodeCsvDetails.setFileDescription(layoutName);
//					tab.setSelectedIndex(UNICODE_CSV);
//				}
//			}
//		} catch (Exception e) {
//			// TODO: handle exception
//		}
//	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.openFile.FormatFileName#formatLayoutName(java.lang.String)
	 */
	@Override
	public String formatLayoutName(String layoutName) {
		return layoutName;
	}

	public IFilePreview getSelectedCsvDetails() {
		return tabDef.getSelectedFilePreview();
	}
	
	public String getLayoutId() {
		return getSelectedCsvDetails().getFileDescription();
	}
	
	public AbstractLayoutDetails getRecordLayout() {
		return getSelectedCsvDetails().getLayout(getSelectedCsvDetails().getFontName(), null, IGetSchema.SCHEMA_DEFAULT);
	}
	
	public void addEditAction(ActionListener editAction) {
		tabDef.addEditAction(editAction);
	}


	public JTable getCurrentJTable() {
		return getSelectedCsvDetails().getJTable();
	}
	
	public JComponent getTabPane() {
		return tabDef.tab;
	}

	private class StartEditAction implements ActionListener {

		final boolean browse;
		final IStartActionInterface startAction;

		public StartEditAction(boolean browse, IStartActionInterface startAction) {
			super();
			this.browse = browse;
			this.startAction = startAction;
		}



		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			startAction.loadFile(browse);
			ReTabPane.this.lastData = null;
		}
		
	}
//	public void setParentFrame(ReFrame parentFrame) {
//		this.parentFrame = parentFrame;
//	}
	
//	public final void setGoVisible(boolean visible) {
//		//csvDetails.goBtn.setVisible(visible);
//		unicodeCsvDetails.goBtn.setVisible(visible);
//	}
//
//	public LayoutDetail getLayout(String font, byte[] recordSep) {
//		return getSelectedCsvDetails().getLayout(font, recordSep);
//	}
}
