package net.sf.RecordEditor.re.util.regExp;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.text.JTextComponent;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.CsvParser.CsvDefinition;
import net.sf.JRecord.CsvParser.ICsvCharLineParser;
import net.sf.JRecord.CsvParser.ICsvParserIds;
import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.IGetSchema;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.External.ExternalRecord;
import net.sf.JRecord.External.Def.ExternalField;
import net.sf.JRecord.Types.Type;
import net.sf.JRecord.charIO.ICharReader;
import net.sf.JRecord.charIO.StandardCharReader;
import net.sf.JRecord.parser.CharParserManager;
import net.sf.RecordEditor.re.util.csv.BaseFilePreview;
import net.sf.RecordEditor.re.util.csv.FileTableDisplay;
import net.sf.RecordEditor.re.util.csv.FileTableDisplay.IFileStructureChanged;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.OpenSaveAction;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.BasePanel;
import net.sf.RecordEditor.utils.swing.SwingUtilsBldr;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public class RegExpPreview extends BaseFilePreview implements IFileStructureChanged {

	private static final int SRC_OTHER = 0;
//	private static final int SRC_NO_UPDATE = 9;

	private static final String REG_EXP_ID = "REGEXP~";

//	private BaseHelpPanel pnl = new BaseHelpPanel();
	private final JTextArea regExpEP = new JTextArea();
	private final JCheckBox namesOnFirstLineChk = new JCheckBox();
	private final JTextComponent messageTxt;
	private final FileTableDisplay fileTblDisplay;
	public final JButton goBtn, 
				saveAsXmlSchemaBtn ,
				refreshBtn;

	private String filename = "";

	private byte[] lastData;

	@SuppressWarnings("serial")
	public RegExpPreview(String pgmId, JTextComponent message, boolean addTable, JTable table) {
		super(pgmId + Parameters.OP_TAB_REG_EXP, false);
		
		this.messageTxt = message;
		this.fileTblDisplay = new FileTableDisplay(table, this);
		this.goBtn = SwingUtilsBldr.newButton("Go");
		this.saveAsXmlSchemaBtn = new JButton(new OpenSaveAction(super.pnl, false, "Save as Xml Schema") {
			
			@Override public void processFile(File selectedFile) {
				String regExp = regExpEP.getText();
				if ("".equals(regExp.trim())) {
					messageTxt.setText("No Regular expression to write");
				} else {
					writeSchema(selectedFile, createExternalRec("", regExp));
				}
			}
			
			@Override public File getDefaultFile() {			
				String dirName = Common.OPTIONS.DEFAULT_COPYBOOK_DIRECTORY.getNoStar();
				String suf = ".xml";
				String fileName = dirName + "Def" + suf;
				if (filename != null && filename.length() > 0) {
					fileName = dirName + new File(filename).getName() + suf;
				}
				return new File(fileName);
			}
		});

		this.refreshBtn = new JButton("Refresh Table");
		
		init_200_LayoutScreen(addTable);
		init_300_Listners();
		
		
		//super.displayThePnl().set(false);
	}
	
	private void init_200_LayoutScreen(boolean addTable) {
		namesOnFirstLineChk.setSelected(false);
		pnl.addLineRE("Regular Expresion", regExpEP)
				.setHeightRE(SwingUtils.values().NORMAL_FIELD_HEIGHT *5)
		   .addLineRE("Column names on First Line", namesOnFirstLineChk)
		   .addLineRE("", saveAsXmlSchemaBtn, refreshBtn)	.setGapRE(BasePanel.GAP0)
		   .addLineRE("", BasePanel.NULL_ITEM, goBtn);

		if (addTable) {
			pnl.addComponentRE(
					1,  BasePanel.LAST_USED_COLUMN, BasePanel.FILL, BasePanel.GAP0,
			        BasePanel.FULL, BasePanel.FULL,
			        fileTblDisplay.getFileTable());
		}
	}

	private void init_300_Listners() {
		
		ActionListener redisplayAction = new ActionListener() {
			@Override public void actionPerformed(ActionEvent arg0) {
				redisplayScreen();
			}
		};
		regExpEP.addFocusListener(new FocusAdapter() {
			@Override public void focusLost(FocusEvent e) {
				redisplayScreen();
				super.focusLost(e);
			}
		});
		namesOnFirstLineChk.addActionListener(redisplayAction);
		refreshBtn.addActionListener(redisplayAction);
	}

	private void redisplayScreen() {
		if (lastData != null) {
			LayoutDetail layout = getLocalLayout("", SRC_OTHER);
			if (layout != null) {
				fileTblDisplay.buildFileLayout(layout, messageTxt, lastData);
			}
		}
	}	

//	@Override
//	public JPanel getPanel() {
//		return pnl;
//	}

	@Override
	public JButton getGoButton() {
		return goBtn;
	}

	@Override
	public boolean setData(String filename, byte[] data, boolean checkCharset, String layoutId) {
		
		this.filename = filename;
		if (lastData == data) {
			return true;
		}

		setFileDescription(layoutId);
	

		lastData = data;
		return buildFileLayout(data, SRC_OTHER);
	}

//	private boolean updateFileLayout(byte[] data) {
//		return fileTblDisplay.buildFileLayout(getLocalLayout("", SRC_NO_UPDATE), messageTxt, data);
//	}


	private boolean buildFileLayout(byte[] data, int source) {
		return fileTblDisplay.buildFileLayout(getLocalLayout("", source), messageTxt, data);
	}
	
	private void setFileDescription(String layoutId) {
		if (layoutId != null && layoutId.startsWith(REG_EXP_ID)) {
			decodeRegExp(layoutId.substring(REG_EXP_ID.length()));
		}
	}
	

	@Override
	public String getFileDescription() {
		return REG_EXP_ID + encodeRegExp();
	}

	private String encodeRegExp() {
		String reg = regExpEP.getText();
		StringBuilder b = new StringBuilder(reg.length() * 4 / 3 + 1);
		b.append(namesOnFirstLineChk.isSelected() ? 'Y' : 'N');
		for (int i = 0; i < reg.length(); i++) {
			char c = reg.charAt(i);
			switch (c) {
			case ',':
			case '\\':
				b.append('\\').append(c);
				break;
			case '\r': b.append('\\').append('r'); break;
			case '\n': b.append('\\').append('n'); break;
			case '\t': b.append('\\').append('t'); break;
			case ' ' : b.append('\\').append('s'); break;
			default:
				b.append(c);
			}
		}
		
		return b.toString();
	}

	
	private void decodeRegExp(String s) {
		StringBuilder b = new StringBuilder(s.length() + 1);
		boolean escFound = false;
		
		namesOnFirstLineChk.setSelected(s.startsWith("Y") || s.startsWith("y"));
		
		for (int i = 1; i < s.length(); i++) {
			char c = s.charAt(i);
			if (escFound) {
				switch (c) {
				case 'r': b.append('\r'); break;
				case 'n': b.append('\n'); break;
				case 't': b.append('\t'); break;
				case 's': b.append(' '); break;
				default:
					b.append(c);
				}
				escFound = false;
			} else if (c == '\\') {
				escFound = true;
			} else {
				b.append(c);
			}
		}
		
		regExpEP.setText(b.toString());
	}

		
	@Override
	public boolean setLines(byte[][] newLines, String font, int numberOfLines) {
		return false;
	}

	@Override
	public void setLines(String[] newLines, String font, int numberOfLines) {

	}

	@Override
	public AbstractLayoutDetails getLayout(String font, byte[] recordSep, int requestType) {
		return getLocalLayout(font, SRC_OTHER);
	}
	


//	private LayoutDetail lastLayout;
	private LayoutDetail getLocalLayout(String font, int source) {

//		layoutBuildFailed = false;
//		if (lastLayout != null && data == null) {
//			return lastLayout;
//		}
//		lastLayout = null;
		
//		ExternalRecord re = createExteranlRecord(font);
		int fldCount = 1;
		String regExp = regExpEP.getText();
		if (lastData != null && ! namesOnFirstLineChk.isSelected()) {
			ICharReader reader;// = new StandardCharReader();
			ICsvCharLineParser lineParser = CharParserManager.getParser(
					ICsvParserIds.REGULAR_EXPRESSION_PARSER, 
					regExp);
			CsvDefinition csvDef = new CsvDefinition(",", "");
			String s;
			try {
				reader =  new StandardCharReader().open(new ByteArrayInputStream(lastData), font);
				while ((s = reader.read()) != null) {
					List<String> fieldList = lineParser.getFieldList(s, csvDef);
					fldCount = Math.max(fldCount, fieldList.size());
				}
				reader.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		ExternalRecord re = createExternalRec(font, regExp);

		for (int i = 1; i <= fldCount; i++) {
			re.addRecordField(new ExternalField(i, 1, "f" + i, "", Type.ftChar, 0, 0, "", "", "", 0));
		}

		return  re.asLayoutDetail();
	}

	private ExternalRecord createExternalRec(String font, String regExp) {
		int ioCode = namesOnFirstLineChk.isSelected() 
				? IFileStructureConstants.IO_UNICODE_NAME_1ST_LINE
				: IFileStructureConstants.IO_UNICODE_TEXT ;
		ExternalRecord re = new ExternalRecord(
				-1, "", "",
				Constants.rtDelimited, 0, "Y", "",
				"", "", 0, Constants.DEFAULT_STRING,
				Constants.SYSTEM_EOL_BYTES, font,  ICsvParserIds.REGULAR_EXPRESSION_PARSER,
				ioCode, false);
		re.setLineFormatParam(regExp);
		return re;
	}


	@Override
	public String getFontName() {
		return null;
	}

	@Override
	public int isMyLayout(String layoutId, String filename, byte[] data) {
		return (layoutId != null && layoutId.startsWith(REG_EXP_ID))
				? YES 
				: NO;
	}

	@Override
	public int getSchemaCheckType() {
		return IGetSchema.ST_NO_CHECK_ON_SCHEMA;
	}
//
//	@Override
//	public void setHelpURL(URL helpUrl) {
//		pnl.setHelpURLre(helpUrl);
//	}

	@Override
	public void fileStructureChanged(int newFileStructure) {
		
	}

}
