package net.sf.RecordEditor.re.util.wizard;

import net.sf.JRecord.Common.RecordRunTimeException;
import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.msg.UtMessages;
import net.sf.RecordEditor.utils.swing.BaseHelpPanel;
import net.sf.RecordEditor.xml.classes.BaseCopyDif;
import net.sf.RecordEditor.xml.classes.File;


@SuppressWarnings("serial")
public class StandardGetFiles<Save extends BaseCopyDif> extends  AbstractFilePnl<Save> {

	public static enum FileId {
		OLD("Old File Name", false),
		NEW("New File Name", true),

		LAYOUT_FILE("Layout File Name", false),
		CSV_FILE("CSV File Name", true);

		final String prompt;
		final boolean isNewFile;
		FileId(String prompt, boolean isNewFile) {
			this.prompt = prompt;
			this.isNewFile = isNewFile;
		}
	}
	
	public static final FileId[] FP_STD_NAMES = {FileId.OLD, FileId.NEW};
	public static final FileId[] FP_LAYOUT_TO_CSV = {FileId.LAYOUT_FILE, FileId.CSV_FILE};
	
//	public static final int OLD = 0;
//	public static final int NEW = 1;
//
//	private static final String[] FILE_PROMPT = {"Old File Name", "New File Name"};;

	private Save values; // = new save();
//		private JPanel goPanel = new JPanel();
	private IUserLayoutSelection layoutSelection;
	//int fileNo;
	FileId fileId;
	private final boolean fileMustExist;

	public StandardGetFiles(IUserLayoutSelection selection, FileId fileId, String recentFiles, String help, boolean fileMustExist) {
		super(selection, recentFiles);

		this.layoutSelection = selection;
		this.fileId =  fileId;
		this.fileMustExist = fileMustExist;

		setHelpURLre(Common.formatHelpURL(help /*Common.HELP_DIFF_TL*/));
	}


	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#getValues()
	 */
	@Override
	public Save getValues() throws Exception {

		File fileDtl = values.getOldfile();
		if (fileId.isNewFile) {
			fileDtl = values.getNewfile();
		}

		layoutSelection.forceLayoutReload();
		fileDtl.name = getCurrentFileName();
		fileDtl.getLayoutDetails().name = layoutSelection.getLayoutName();
		if (layoutSelection.getRecordLayout(fileDtl.name) == null) {
			throw new RecordRunTimeException("Layout Does not exist");
		}
		if (fileMustExist) {
			checkFile(fileDtl.name, fileNameTxt);
		} else if ((new java.io.File(fileDtl.name)).isDirectory()) {
			fileNameTxt.requestFocus();
			throw new RuntimeException(UtMessages.DIRECTORY_NOT_ALLOWED.get(fileDtl.name));
		}

		return values;
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizardPanel#setValues(java.lang.Object)
	 */
	@Override
	public void setValues(Save detail) throws Exception {
		File fileDtl;
		String layoutName;

		//System.out.println("Setting Values ... ");
		values = detail;

		fileDtl = values.getOldfile();
		if (fileId.isNewFile) {
			fileDtl = values.getNewfile();
		}

		if (fileDtl.name != null && fileDtl.name.length() > 0) {
			fileNameTxt.setFileName(fileDtl.name);
		}

//		if (! "".equals(fileDtl.name)) {
//			fileName.setText(fileDtl.name);
//		}

		layoutName = fileDtl.getLayoutDetails().name;
		if (layoutName != null && layoutName.length() > 0) {
			layoutSelection.setLayoutName(layoutName);
		}
	}


	/**
	 * Add a file name
	 */
	@Override
	protected void addFileName(BaseHelpPanel pnl) {

		pnl.addLineRE(fileId.prompt, fileNameTxt);
	}
}
