/**
 *
 */
package net.sf.RecordEditor.re.util.wizard;

import net.sf.RecordEditor.re.layoutSelection.def.IUserLayoutSelection;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.common.ReActionHandler;
import net.sf.RecordEditor.utils.wizards.AbstractWizard;
import net.sf.RecordEditor.utils.wizards.AbstractWizardPanel;
import net.sf.RecordEditor.xml.classes.BaseCopyDif;


/**
 * @author Bruce Martin
 *
 */
public abstract class TwoLayoutsWizard<Save extends BaseCopyDif> extends AbstractWizard<Save> {

	//private JibxCall<Save> jibx = null;

	//private IUserLayoutSelection[] recordSelection = new IUserLayoutSelection[2];



	/**
	 * Create Single layout
	 * @param selection1 record layout selection class
	 * @param definition record filter definition
	 */
	public TwoLayoutsWizard(String id, Save definition) {
		super(id, definition);
	}



	protected void setUpPanels(IUserLayoutSelection selection1, IUserLayoutSelection selection2,
			String recentFiles, AbstractWizardPanel<Save> finalScreen,
			String help, boolean secondFileRequired, boolean lookupRecentLayout,
			StandardGetFiles.FileId[] promptDtls) {
		@SuppressWarnings("unchecked")
		AbstractWizardPanel<Save>[] pnls = new AbstractWizardPanel[4];
		
		if (promptDtls == null) {
			promptDtls = StandardGetFiles.FP_STD_NAMES;
		}

		
		StandardGetFiles<Save> firstPnl = new StandardGetFiles<Save>(selection1, promptDtls[0], recentFiles, help, true);
		StandardGetFiles<Save> secondPnl = new StandardGetFiles<Save>(selection2, promptDtls[1], recentFiles, help, secondFileRequired);

		firstPnl.setLookupRecentLayouts(lookupRecentLayout);
		secondPnl.setLookupRecentLayouts(lookupRecentLayout);
		
		pnls[0] = firstPnl;
		pnls[1] = secondPnl;
		pnls[2] = new FieldChoice<Save>(selection1, selection2, help);
		pnls[3] = finalScreen;

		super.setPanels(pnls);
	}

//	/**
//	 * @see net.sf.RecordEditor.utils.screenManager.AbstractWizard#finished(java.lang.Object)
//	 */
//	@Override
//	public void finished(save details) {
//
//		if (finalScreen.isToRun()) {
//			finalScreen.run();
//		}
//	}




	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#executeAction(int)
	 */
	@Override
	public void executeAction(int action) {
		if (action == ReActionHandler.SAVE) {
			try {
				Save diff = super.getActivePanel().getValues();

				if (! "".equals(diff.saveFile)) {
					saveAsXml(diff);
					diff.fileSaved = true;
				}
			} catch (Exception e) {
				e.printStackTrace();
				Common.logMsgRaw(Common.FILE_SAVE_FAILED, e);
			}
		} else {
			super.executeAction(action);
		}
	}

	/**
	 * @see net.sf.RecordEditor.utils.wizards.AbstractWizard#isActionAvailable(int)
	 */
	@Override
	public boolean isActionAvailable(int action) {
		if (action == ReActionHandler.SAVE) {
			return true;
		}
		return super.isActionAvailable(action);
	}

	/**
	 * get Jibx interface
	 * @return Jibx interface
	 */
	protected abstract void saveAsXml(Save save) throws Exception;
//		if (save == null) {
//			return;
//		}
//	   if (jibx == null) {
//		   jibx = new JibxCall<Save>(save.getClass());
//	   }
//
//	   jibx.unmarshal(save.saveFile, save);
//	}
}
