package net.sf.RecordEditor.utils.basicStuff;

import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;

import net.sf.JRecord.CsvParser.BasicCsvLineParserExtended;
import net.sf.JRecord.CsvParser.CsvDefinition;
import net.sf.JRecord.CsvParser.ICsvDefinition;

/**
 * Retrieves a table 1 row / column at a time
 * @author Bruce Martin
 *
 */
public class BasicTable2 implements IRetrieveTable {

	private final StringTokenizer rowTokenizer;
	private BasicCsvLineParserExtended columnTokenizer 
			= new BasicCsvLineParserExtended(false, ICsvDefinition.NORMAL_SPLIT, true, false);
	private final CsvDefinition csvDef;
	private String[] rowData = {};
	
	private int colIdx = 0;
	
	public BasicTable2(String tblString, String sep, String quote) {
		super();
		rowTokenizer = new StringTokenizer(tblString, "\n");
		csvDef = new CsvDefinition(sep, quote);
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.basicStuff.ITableReader#hasMoreRows()
	 */
	@Override
	public boolean hasMoreRows() {
		return rowTokenizer.hasMoreElements();
	}

	@Override
	public List<String> nextRowAsList() { 
		nextRow();

		return Arrays.asList(rowData);
	}
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.basicStuff.ITableReader#nextRow()
	 */
	@Override
	public String nextRow() {
		String s = rowTokenizer.nextToken();
		rowData = columnTokenizer.split(s, csvDef, 0);
		return s;
	}
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.basicStuff.ITableReader#hasMoreColumns()
	 */
	@Override
	public boolean hasMoreColumns() {
		return colIdx < rowData.length;
	}
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.basicStuff.ITableReader#nextColumn()
	 */
	@Override
	public String nextColumn() {
		return rowData[colIdx++];
	}
}
