package net.sf.RecordEditor.utils.charsets;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import net.sf.RecordEditor.utils.screenManager.ReMainFrame;
import net.sf.recordEditor.ui.common.StdComponents.UiPnlButton;
import net.sf.recordEditor.ui.standard.component.ComboLikeObject;
import net.sf.recordEditor.ui.styles.StyleManager;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

/**
 * This class implements a Font-Combo box that lists
 * common fonts in a combo-box and has a font search button
 * 
 * @author Bruce Martin
 *
 */
@SuppressWarnings("serial")
public class FontCombo extends ComboLikeObject {
	
	/**
	 *  This class implements a Font-Combo box that lists
     * common fonts in a combo-box and has a font search button
	 */
	public FontCombo() {
		this(CharsetMgr.getCommonCharsets(), new UiPnlButton(StyleManager.styles().fontChooser(),".."));
	}
	
	/**
	 *  This class implements a Font-Combo box that lists
     * supplied fonts in a combo-box and has a font search button
	 */
	public FontCombo(String[] fonts) {
		this(lookupDescription(fonts), new UiPnlButton(StyleManager.styles().fontChooser(),".."));
	}
	
	
	private FontCombo(String[][] fonts, UiPnlButton btn) {
		super("fontTxt", btn.getUiContainer());
		
		setFontList(fonts);
		
		this.setPreferredSize(
				new Dimension(
						Math.max(this.getPreferredSize().width, SwingUtils.values().CHAR_FIELD_WIDTH *25), 
						this.getPreferredSize().height));
		btn.getMainComponent().addActionListener(new ActionListener() {	
			@Override public void actionPerformed(ActionEvent e) {
				showFontScreen();
			}
		});
	}
	
	/**
	 * Set the drop down combo to a list fonts
	 * @param fonts
	 */
	public void setFontList(String[] fonts) {
		setFontList(lookupDescription(fonts));
	}
	
	private static String[][] lookupDescription(String[] fonts) {
		String[][] ret = new String[fonts.length][];
		
		for (int i =0; i < fonts.length; i++) {
			ret[i] = new String[2];
			ret[i][0] = fonts[i];
			ret[i][1] = CharsetMgr.getDescription(fonts[i]);
		}
		
		return ret;
	}
	
	/**
	 * Set the drop down combo to a list fonts
	 * @param fonts
	 */
	public void setFontList(String[][] fonts) {
		
		if (fonts != null) {
			JPopupMenu menu = new JPopupMenu();
			
			for (String[] s : fonts) {
				menu.add(new FontAction(s[0], s[1]));
			}
		
			super.setCurrentPopup(menu);
//			System.out.print("Windows LAF: " + SwingUtils.values().IS_WINDOWS_LAF);
//			if (SwingUtils.values().IS_WINDOWS_LAF) {
//				super.setPopupHeight(SwingUtils.values().COMBO_TABLE_ROW_HEIGHT * Math.min(50, fonts.length * 2 + 1));
//				System.out.print("\t" + (SwingUtils.values().COMBO_TABLE_ROW_HEIGHT * Math.min(50, fonts.length * 2 + 1)));
//			}
		}
	}
	protected int getPopupWidth(JPopupMenu currentPopup) {
		return Math.max(this.getWidth(), currentPopup.getPreferredSize().width);
	}

	private void showFontScreen() {
		super.setText(
				(new CharsetSelection(
						ReMainFrame.getMasterFrame(),
						super.getText()))
					.getCharset());
	}
	
	private class FontAction extends JMenuItem implements ActionListener   {
		private final String fontName;
		private FontAction(String fontName, String description) {
			super(fontName + ":  \t" + description);
			this.fontName = fontName;
			this.addActionListener(this);
		}

		/* (non-Javadoc)
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			FontCombo.this.setText(fontName);
		}
	}
}
