package net.sf.RecordEditor.utils.fileAnalysis;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.HashSet;

import org.mozilla.intl.chardet.nsDetector;

import net.sf.JRecord.Common.Conversion;

public class CheckEncoding {

	 
	private static ByteOrderMarker[] BOM = {
			new ByteOrderMarker("UTF-32BE", new byte[] {0, 0, (byte)0xFE,  (byte)0xFF}),
			new ByteOrderMarker("UTF-32LE", new byte[] {(byte)0xFF, (byte)0xFE,  0, 0}),
			new ByteOrderMarker("UTF-16BE", new byte[] {(byte)0xFE, (byte)0xFF}),
			new ByteOrderMarker("UTF-16LE", new byte[] { (byte)0xFF,  (byte)0xFE}),
	};
	/**
	 *
	 * @param bytes Data to check
	 * @param returnLikelyCharsets whether to return a list of likely charsets
	 *
	 * @return charset Details
	 */
	public static CharsetDetails determineCharSet(byte[] bytes) {
		if (bytes == null || bytes.length == 0) {
			return new CharsetDetails("", new String[] {""});
		}
		String s = "";
		nsDetector det = new nsDetector();
		String bom = bomCheck(bytes);
		int zeroCount = 0;
		//int evenZeroCount = 0;
		for (int i = 0; i < bytes.length; i++) {
			if (bytes[i] == 0) {
				zeroCount += 1;
//				if (i % 2 == 0) {
//					evenZeroCount += 1;
//				}
			}
		}
	
		boolean possibleEBCDIC = bytes != null && bytes.length > 50 
				&& BasicCharsetChecker.getValidCharsRatio(Conversion.toString(bytes, "cp037")) > 0.80;
		String[] charSets = null;

		//if (returnLikelyCharsets || bom != null || possibleEBCDIC || utfCheck(bytes) || ! det.isAscii(bytes, bytes.length)) {		
			det.DoIt(bytes, bytes.length, false);
			det.DataEnd();
			charSets = det.getProbableCharsets();
			if (charSets != null && charSets.length > 0) {
				ArrayList<String> extraEncodings = new ArrayList<String>();
				s = charSets[0];
				if (possibleEBCDIC) {
					s = "cp037";
					extraEncodings.add(0, "CP037");
				}
				extraEncodings.add(0, "");
				
				String defaultCharsetName = Charset.defaultCharset().displayName();
				if (bom != null) {
					extraEncodings.add(bom);
					
					if (s.length() == 0 || isUtfPresent(charSets)) {
						s = bom;
					}
				} else {
					String secondCharset = charSets.length > 1 ? charSets[1].toLowerCase() : "";
					if ("utf-8".equalsIgnoreCase(charSets[0]) && zeroCount > 0 ) {
						for (int i = 1; i < charSets.length; i++) {
							if ((! "utf-8".equalsIgnoreCase(charSets[i])) && charSets[i].toLowerCase().startsWith("utf")) {
								s = charSets[i];
								break;
							}
						}
					}
					if ("".equals(s) ||  "utf-8".equalsIgnoreCase(s)) {
						if (charSets.length > 1
						&&  (secondCharset.startsWith("cp")
						  || (secondCharset.startsWith("utf") && ! "utf-8".equalsIgnoreCase(secondCharset)) )) {
							s = charSets[1];
						}
					} 
					if ("utf-8".equalsIgnoreCase(s) && isWin1252Present(charSets)) {
						s = "";
					}
				}
				if (defaultCharsetName.equalsIgnoreCase(s)) {
					s = "";
				}
				
				if (extraEncodings.size() > 0) {
					if (charSets != null && charSets.length > 0) {
						HashSet<String> check = new HashSet<String>(extraEncodings);
						for (int i = 0; i < charSets.length; i++) {
							if (! check.contains(charSets[i].toUpperCase())) {
								extraEncodings.add(charSets[i]);
							}
						}
					}
					charSets = extraEncodings.toArray(new String[extraEncodings.size()]);
				}
			} else if (possibleEBCDIC) {
				s = "cp037";
				charSets = new String[] {s, ""};
			}
//		} else {
//			System.out.println("is ascii " + Charset.defaultCharset().displayName() + "windows-1252");
		//}

		//System.out.println("\n----> " + s);
		return new CharsetDetails(s, charSets);
	}
	
	private static boolean isUtfPresent(String[] charSets) {
		for (int i = 0; i < charSets.length; i++) {
			String ss = charSets[i].toLowerCase();
			if ("windows-1252".equals(ss) || (ss.startsWith("utf"))) {
				return true;
			}
		}
		return false;
	}
	
	private static boolean isWin1252Present(String[] charSets) {
		for (int i = 0; i < charSets.length; i++) {
			String ss = charSets[i].toLowerCase();
			if ("windows-1252".equals(ss) ) {
				return true;
			}
		}
		return false;
	}

//	private static boolean utfCheck(byte[] bytes) {
//		return bytes != null && bytes.length > 0 && (bytes[0] == -1 || bytes[0] == -2);
//	}

	/**
	 * Check if po9ssibly UTF16 (containing mostly ascii chars)
	 *
	 * @param bytes vbytes to check
	 * @return
	 */
	private static String bomCheck(byte[] buffer) {

		if (buffer != null && buffer.length > 0) {
			for (int i = 0; i < BOM.length; i++) {
				if (BOM[i].isEncoding(buffer)) {
					System.out.print("\t--> " + BOM[i].charset);
					return BOM[i].charset;
				}
			}
		}

		return null;
	}
	
	private static class ByteOrderMarker {
		final byte[] bytes;
		final String charset;
		public ByteOrderMarker(String charset, byte[] bytes) {
			super();
			this.charset = charset;
			this.bytes = bytes;
		}
		
		private boolean isEncoding(byte[] buffer) {
			if (buffer.length > bytes.length) {
				System.out.print("\n\n" + charset);
				for (int i = 0; i < bytes.length; i++) {
					System.out.print(buffer[i] + " " + bytes[i] + "\t");
					if (buffer[i] != bytes[i]) {
						return false;
					}
				}
				return true;
			}
			return false;
		}
	}
}
