package net.sf.RecordEditor.utils.fileAnalysis;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.CsvParser.BasicCsvByteLineParserExtended;
import net.sf.JRecord.CsvParser.BasicCsvLineParserExtended;
import net.sf.JRecord.CsvParser.CsvDefinition;
import net.sf.JRecord.Types.Type;
import net.sf.JRecord.Types.TypeManager;
import net.sf.JRecord.detailsBasic.CsvCharDetails;
import net.sf.RecordEditor.re.util.csv.ColumnDateFormatChk;
import net.sf.RecordEditor.re.util.csv.StdDateFormatStr;
import net.sf.RecordEditor.trove.map.hash.TIntObjectHashMap;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.swing.common.CsvTextItem;

/**
 *
 * This Class Evaluates a CSV file and tries to decide on the
 * fields type and wether there the column names are on the first
 * line or not
 * @author Bruce Martin
 *
 */
public class CsvAnalyser {

	public static final int LINES_TO_CHECK = 400;
	private static final char DEFAULT_QUOTE_CHAR = '"';
	private static final char DEFAULT_FIELD_DELIMETER_CHAR = '\t';
	public static final int COLUMN_NAMES_YES   = 1;
	public static final int COLUMN_NAMES_NO    = 2;
	public static final int COLUMN_NAMES_MAYBE = 3;
	
	public static final int NOT_ANALYSED = 0;
	public static final int UNLIKELY_TO_BE_CSV = 1;
	public static final int COULDBE_TO_BE_CSV = 5;
	public static final int LIKELY_TO_BE_CSV = 10;
	
	private static char[] SPECIAL_CHARS = {'~', '@', '#', '$', '%', '^', '*', };
	private static final char[] STANDARD_FIELD_DELIMITERS = {',', ';', '\t', '|'};
	private static final char[] STANDARD_QUOTES = {'\'', '"'};
	private String[] columnNames;

	public static CsvAnalyser newCsvAnalyser(byte[] data) {
		BufferedReader r = null ;
		List<String> lines = null;
		try {
			r = new BufferedReader(new StringReader(new String(data)));
			lines = new ArrayList<String>(Math.max(25, data.length / 20));
			String line;
		
			while ((line = r.readLine()) != null) {
				lines.add(line);
			}

		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (r != null) {
				try {
					r.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		
		String[] linesArray = lines == null ? new String[0] : lines.toArray(new String[lines.size()]);
		return new CsvAnalyser(linesArray, linesArray.length, "", false);
	}

//	public static final HashSet<String> STANDARD_CHARS = new HashSet<String>();
//
//	static {
//		String s = Common.STANDARD_CHARS;
//
//		for (int i = 0; i < s.length(); i++) {
//			STANDARD_CHARS.add(s.substring(i, i+1));
//		}
//
//		for (int i = 0 ; i < Common.FIELD_SEPARATOR_TEXT_LIST.length; i++) {
//			STANDARD_CHARS.add(Common.FIELD_SEPARATOR_LIST1_VALUES[i]);
//		}
//		STANDARD_CHARS.add("\n");
//	}

//	private int seperatorIdx = 0;
	private String  seperatorId = "<tab>";
//	private int quoteIdx = 0;
	private String quote;
	private int colNamesOnFirstLine = COLUMN_NAMES_MAYBE;
	private int fieldNameLineNo = 1;
	private int numberOfColumns = 0;
	private int lineCount=0;
	//private int[] colTypes, afterDecimal, textTypes = null;
	private FieldTypeDetails[] colTypes, textTypes;
	private Character specialQuote = null;
	private boolean csvFile;

	private String font;

	private ArrayList<ArrayList<String>> listOfLines = null;
	//public boolean validChars = true;
	private double ratio;

	private boolean embeddedCr = false, likelyEmbeddedCr = false;
	private final int lineNumber;
	
	private int likelyCsv = NOT_ANALYSED;

	public CsvAnalyser(byte[][] lines, int numberOfLines, String fontname, boolean embeddedCr) {
		this(new ByteArrayLineList(lines, 0, numberOfLines), fontname, embeddedCr);
	}
	
	public CsvAnalyser(ByteArrayLineList lineLst, String fontname, boolean embeddedCr) {
		font = fontname;
		this.embeddedCr = embeddedCr;

		int noLines = 0;
		int firstLine = lineLst.firstLineNumber + 1;
		fieldNameLineNo = firstLine;
		if (lineLst.lines != null) {
			noLines = lineLst.getLineCount();

			byte sep = getSeperatorQuote(lineLst);
			int lastLineNumber = lineLst.getLastLineNumber();
			if (sep == ' ' && lineLst.firstLineNumber == 0 && lastLineNumber > 3) {
				ByteArrayLineList tl = new ByteArrayLineList(lineLst.lines, lastLineNumber-2, lastLineNumber);
				String q = quote;
				byte sep1 = getSeperatorQuote(tl);
				if (sep1 != sep) {
					int pos = lastLineNumber-2;
					byte[] l;
					while ((l = lineLst.getLine(pos)) != null && pos(l, sep1) >= 0) {
						pos -= 1;
					}
					
					fieldNameLineNo = pos + 1;
					sep = sep1;
				}
				quote = q;
			}

			listOfLines = getList(lineLst, sep);

			checkForColNames(listOfLines, firstLine);
			//findColTypes(listOfLines);
		} 
		lineNumber = noLines;
	}
	
	private int pos(byte[] bytes, byte b) {
		for (int i = 0; i < bytes.length; i++) {
			if (bytes[i] == b) {
				return i;
			}
		}
		return -1;
	}


	public CsvAnalyser(ByteArrayLineList lineLst, String fontname, byte sep, String quote, boolean embeddedCr) {
		font = fontname;
		this.embeddedCr = embeddedCr;
		this.quote = quote;

		int noLines = 0;
		int firstLine = lineLst.firstLineNumber + 1;
		fieldNameLineNo = firstLine;
		if (lineLst.lines != null) {
			noLines = lineLst.getLineCount();

			listOfLines = getList(lineLst, sep);

			checkForColNames(listOfLines, firstLine);
		}
		lineNumber = noLines;
	}

	public CsvAnalyser(String[] lines, int numberOfLines, String fontname, boolean embeddedCr) {
		this(new StringArrayLineList(lines, 0, numberOfLines), fontname, embeddedCr);
	}

	public CsvAnalyser(StringArrayLineList lineDtls, String fontname, boolean embeddedCr) {
		font = fontname;
		this.embeddedCr = embeddedCr;

		int noLines = 0;
		int firstLine = lineDtls.firstLineNumber + 1;
		fieldNameLineNo = firstLine;
		if (lineDtls.lines != null && lineDtls.getLineCount() > 0 && lineDtls.getLine(0) != null) {
			ratio = BasicCharsetChecker.getValidCharsRatio(lineDtls.getLine(0));

			char sep = getSeperatorString(lineDtls);
			
			int lastLineNumber = lineDtls.getLastLineNumber();
			if (sep == ' ' && lineDtls.firstLineNumber == 0 && lastLineNumber > 3) {
				StringArrayLineList tl = new StringArrayLineList(lineDtls.lines, lastLineNumber-2, lastLineNumber);
				char sep1 = getSeperatorString(tl);
				if (sep1 != sep) {
					int pos = lastLineNumber-2;
					String l;
					while ((l = lineDtls.getLine(pos)) != null && l.indexOf(sep1) >= 0) {
						pos -= 1;
					}
					
					fieldNameLineNo = pos + 1;
					sep = sep1;
				}
			}
			listOfLines = getList(lineDtls, sep);

			checkForColNames(listOfLines, firstLine);
		}
		lineNumber = noLines;
	}

	public CsvAnalyser(StringArrayLineList lineDtls, String fontname, char sep, String quote, boolean embeddedCr) {
		font = fontname;
		this.embeddedCr = embeddedCr;
		this.quote = quote;

		int noLines = 0;
		int firstLine = lineDtls.firstLineNumber + 1;
		fieldNameLineNo = firstLine;
		if (lineDtls.lines != null && lineDtls.getLineCount() > 0 && lineDtls.getLine(0) != null) {
			ratio = BasicCharsetChecker.getValidCharsRatio(lineDtls.getLine(0));

			listOfLines = getList(lineDtls, sep);
			
			if (quote == null || quote.length() == 0) {
				decideOnQuote(lineDtls, sep);
			}

			checkForColNames(listOfLines, firstLine);
		}
		lineNumber = noLines;
	}
	
	public CsvAnalyser(ILineTable lineTbl, String[] colNames) {
		this.lineNumber = lineTbl.lineCount();
		this.columnNames = colNames;
		this.numberOfColumns = colNames.length;
		this.lineCount = lineTbl.lineCount();

		determineFieldTypes(lineTbl, 0, lineNumber);
	}

	private ArrayList<ArrayList<String>> getList(ByteArrayLineList lineLst, byte sep) {
		ArrayList<ArrayList<String>> listOfLines = new ArrayList<ArrayList<String>>(Math.max(0, lineLst.getLineCount()));
		ArrayList<String> line;
//		int st;
//		CsvDefinition csvDef = new CsvDefinition(
//				CsvCharDetails.newDelimDefinition(new String(sepA), font),
//				CsvCharDetails.newQuoteDefinition(quote, font), 
//				embeddedCr);

//		if (quote == null || quote.length() == 0) {
//			for (int i = 0; i < noLines; i++) {
//				st = 0;
//				line = new ArrayList<String>();
//	
//				if (lines[i] != null) {
//					for (int j = 0; j < lines[i].length; j++) {
//						if (lines[i][j] == sep) {
//							line.add(getField(lines[i], st, j));
//							st = j+1;
//						}
//					}
//					line.add(getField(lines[i], st, lines[i].length));
//				}
//	
//				listOfLines.add(line);
//			}
//		} else {
		
		BasicCsvByteLineParserExtended lp = BasicCsvByteLineParserExtended.getInstance();
		CsvCharDetails q = quote == null || quote.length() == 0
				? CsvCharDetails.EMPTY_QUOTE
				: CsvCharDetails.newQuoteDefinition(quote, font);
		CsvDefinition csvDef = new CsvDefinition(
					font,
					CsvCharDetails.newDef(sep, font),
					q, //CsvCharDetails.newQuoteDefinition(quote, font), 
					embeddedCr);
		for (int i = 0; i < lineLst.getLineCount(); i++) {
			line = new ArrayList<String>();
			String[] fields = lp.split(lineLst.getLine(i), csvDef, 0);
			for (int j = 0; j < fields.length; j++) {
				line.add(fields[j]);
			}
			
			listOfLines.add(line);
		}
			
//		}
		
		likelyEmbeddedCr = embeddedCr;
		return listOfLines;
	}


//	private String getField(byte[] line, int st, int en) {
//		String ret = "";
//		if (en > st) {
//			try {
//				ret = Conversion.getString(line, st, en, font);
//			} catch (Exception e) {
//			}
//		}
//
//		return ret;
//	}

	private ArrayList<ArrayList<String>> getList(StringArrayLineList lines, char sep) {
		int lineCount2 = lines.getLineCount();
		ArrayList<ArrayList<String>> listOfLines = new ArrayList<ArrayList<String>>(lineCount2);
		ArrayList<String> line;
		char[] sepA = {sep};
		BasicCsvLineParserExtended csvParser = BasicCsvLineParserExtended.getInstance();
		CsvDefinition csvDef = new CsvDefinition(
				font,
				CsvCharDetails.newDelimDefinition(new String(sepA), font),
				CsvCharDetails.newQuoteDefinition(quote, font), 
				embeddedCr);

		for (int i = 0; i < lineCount2; i++) {
			line = new ArrayList<String>();

			String l = lines.getLine(i);
			if (l != null) {
				String[] fields = csvParser.split(l, csvDef, 0);
				for (int j = 0; j < fields.length; j++) {
					line.add(fields[j]);
				}
//				for (int j = 0; j < lines[i].length(); j++) {
//					if (lines[i].charAt(j) == sep) {
//						line.add(getField(lines[i], st, j));
//						st = j+1;
//					}
//				}
//				line.add(getField(lines[i], st, lines[i].length()));
			}

			listOfLines.add(line);
		}
		
		int colNum;
		likelyEmbeddedCr = embeddedCr;
		if (quote != null && quote.length() > 0 && ! embeddedCr) {
			for (int i = 0; i < lineCount2; i++) {
				ArrayList<String> arrayList = listOfLines.get(i);
				if (arrayList != null && (colNum = arrayList.size()) > 0) {
					if (arrayList.get(colNum-1).startsWith(quote) && ! lines.getLine(i).endsWith(quote)) {
						likelyEmbeddedCr = true;
						break;
					}
				}
			}
		}
		return listOfLines;
	}

//	private String getField(String line, int st, int en) {
//		String ret = "";
//		if (en > st) {
//			try {
//				ret = line.substring(st, en);
//			} catch (Exception e) {
//			}
//		}
//
//		return ret;
//	}

	private char getSeperatorString(StringArrayLineList lines) {

		CharData delimiterCharDtls = toCharArray(CsvTextItem.DELIMITER.getCsvList(false, false), STANDARD_FIELD_DELIMITERS, font, '\t');

		for (int i = 0; i < lines.getLineCount(); i++) {
			String line = lines.getLine(i);
			if (line != null) {
				delimiterCharDtls.updateCounts(line);
			}
		}

		int seperatorIdx = getSeperatorIdx(delimiterCharDtls, lines.getLineCount(), delimiterCharDtls.spaceIdx);
		seperatorId = delimiterCharDtls.items[seperatorIdx].value;

		char sepChar = delimiterCharDtls.chars[seperatorIdx];

		decideOnQuote(lines, sepChar);

		return sepChar; //sepChars[seperatorIdx];
	}

	private void decideOnQuote(StringArrayLineList lines, char sepChar) {
		int i;
		int j;
		CharData specialDtls = new CharData(SPECIAL_CHARS, null, null);
		specialDtls.setLength(SPECIAL_CHARS.length);
		CharData quoteDtls = toCharArray(CsvTextItem.QUOTE.getCsvList(false, false), STANDARD_QUOTES, font, '"');
		for (i = 0; i < lines.getLineCount(); i++) {
			String line = lines.getLine(i);
			if (line != null && line.length() > 0) {
//				last = 0;
//				System.out.println();
//				firstChar = line.charAt(0);
				for (j = 0; j < line.length() - 1; j++) {
					char charAt = line.charAt(j);
					if (j == 0 || j == line.length() - 1) {
						quoteDtls.updateCounts(charAt);
						specialDtls.updateCounts(charAt);
					} else if (charAt == sepChar) {
						char ch = line.charAt(j-1);
						quoteDtls.updateCounts(ch);
						specialDtls.updateCounts(ch);
						
						ch = line.charAt(j+1);
						quoteDtls.updateCounts(ch);
						specialDtls.updateCounts(ch);
					}
//					if (charToCheck >= 0) {
//						for (k = 0; k < quoteDtls.length; k++) {
//							if (last == quoteDtls.chars[k]) {
//								quoteCount[k] += 1;
//							}
//
////							System.out.print("\t*" + line.charAt(j+1));
//							if (line.charAt(charToCheck) == quoteDtls.chars[k]) {
//								quoteCount[k] += 1;
//							}
//							
//							if (firstChar == last) {
//								if (specialCount.containsKey(firstChar)) {
//									specialCount.put(firstChar, specialCount.get(firstChar) + 1);
//								}
//							}
//							firstChar = line.charAt(j+1);
//						}
//					}
//					last = charAt;

//					System.out.print(last);
				}
				
//				if (firstChar == line.charAt(line.length() - 1)) {
//					if (specialCount.containsKey(firstChar)) {
//						specialCount.put(firstChar, specialCount.get(firstChar) + 1);
//					}
//				}

			}
		}

		int quoteIdxMax = getMax(quoteDtls.count);

		specialQuote = null;
		if (quoteDtls.count[quoteIdxMax] > 5
		|| (embeddedCr && quoteDtls.count[quoteIdxMax] > 1)) {
			quote = quoteDtls.items[quoteIdxMax].value;
		} else {
			setSpecialQuote(lines.getLineCount() * 2, specialDtls);
		}
	}
	
	protected static CharData toCharArray(List<CsvTextItem> items, char[] extra, String font, char defaultChar) {

		CharData cd = new CharData(new char[items.size() + (extra == null ? 0 : extra.length)], null, null);
		char[] chars = new char[items.size() + (extra == null ? 0 : extra.length)];
		int j=0;
		for (CsvTextItem t : items) {
			try {
				char[] decodeChars = Conversion.decodeChar(t.value, font, defaultChar);
				if (decodeChars.length == 1) {
					if (decodeChars[0] == ' ') {
						cd.spaceIdx = j;
					}
					cd.chars[j] = decodeChars[0];
					cd.items[j++] = t;
				}
			} catch (Exception e) {
			}
		}
		if (extra != null) {
			int num = j;
			for (char c : extra) {
				boolean searching = true;
				
				for (int i = 0; i < num; i++) {
					if (cd.items[i].isText && chars[i] == c) {
						searching = false;
						break;
					}
				}
				if (searching) {
					chars[j] = c;
					String t = "\\t";
					if (c != '\t') {
						t = new String(new char[]{c});
					}
					cd.items[j++] = new CsvTextItem(t, t);

				}
			}
		}
		cd.setLength(j);
		return cd;
	}
	
	
	protected static CharData toByteArray(List<CsvTextItem> items, char[] extra, String font, boolean multiByte, char defaultChar) {

		int num = items.size() + (extra == null ? 0 : extra.length);
		CharData cd = new CharData(null, new byte[num], new byte[num][]);
		int j=0;
		int extraCount = extra == null ? 0 : extra.length;
		
		extra = extra.clone();
		for (CsvTextItem t : items) {
			try {
				if (t.isText) {
					char[] decodeChar = Conversion.decodeChar(t.value, font, defaultChar);
					byte[] decodeBytes = Conversion.getBytes(new String(decodeChar), font);
					if (multiByte || decodeBytes.length == 1) {
						if (decodeChar[0] == ' ') {
							cd.spaceIdx = j;
						}

						cd.bytes[j] = decodeBytes;
						cd.singleBytes[j] = decodeBytes[0];
						cd.items[j++] = t;
						for (int i = 0; i < extraCount; i++) {
							if (extra[i] == decodeChar[0]) {
								extraCount -= 1;
								for (int k = i; k < extraCount; k++) {
									extra[k] = extra[k+1];
								}
								break;
							}
						}
					}
				} else {
					byte byteFromHexString = Conversion.getByteFromHexString(t.value);
					cd.singleBytes[j] = byteFromHexString;
					cd.bytes[j] = new byte[] { byteFromHexString };
					cd.items[j++] = t;
				}
			} catch (Exception e) {
			}
		}
		if (extra != null) {
			for (int i = 0; i < extraCount; i++) {
				char c = extra[i];
				cd.bytes[j] = Conversion.getBytes(new String(new char[]{c}), font);
				cd.singleBytes[j] = cd.bytes[j][0];
				String t = "\\t";
				if (c != '\t') {
					t = new String(new char[]{c});
				}
				cd.items[j++] = new CsvTextItem(t, t);
			}
		}
		cd.setLength(j);
		return cd;
	}


	private void setSpecialQuote(int compare, TIntObjectHashMap<Integer> specialCount) {
		char cc = 0;
		int max = -1;
		int v;
		for (char c : SPECIAL_CHARS) {
			v =  specialCount.get(c);
			if (v > compare && v > max) {
				cc = c;
				max = v;
			}
		}
		
		if (max > 0) {
			specialQuote = cc;
		}
	
	}

	private void setSpecialQuote(int compare, CharData specialCount) {
		char cc = 0;
		int max = -1;
		int v;
		for (int i = 0; i < specialCount.getLength(); i++) {
			v =  specialCount.count[i];
			if (v > compare && v > max) {
				cc = specialCount.chars[i];
				max = v;
			}
		}
		
		if (max > 0) {
			specialQuote = cc;
		}
	
	}

	private byte getSeperatorQuote(ByteArrayLineList lineLst) {
		int i,j, k;

//		String[] sep = /*(String[])*/ Common.FIELD_SEPARATOR_LIST1_VALUES.clone();
		//byte[] sepBytes = getSepBytes(font);
		byte sepByte;
		String[] sep = /*(String[])*/ Common.FIELD_SEPARATOR_LIST1_VALUES;

//		int st = 2;
		
//		byte[][] quoteBytes = new byte[charDtls.length][];


		
		CharData charDtls = toByteArray(CsvTextItem.DELIMITER.getCsvList(false, true), STANDARD_FIELD_DELIMITERS, font, false, '\t');
//		char[] delimiterCharArray = charDtls;
//		byte[] sepBytes = getBytes(csvDelimiterList, delimiterCharArray, font);

		int sepIdx = getSeperatorIndex2(lineLst, font, charDtls);
		int seperatorIdx = Math.max(0, sepIdx);
		sepByte = charDtls.singleBytes[seperatorIdx];
		csvFile = sepIdx >= 0;
		
		if (csvFile) {
			if (sepByte == 0) {
				try {
					sepByte = Conversion.getBytes(sep[seperatorIdx], "")[0];
				} catch (Exception e) {
				}
			}
			
			byte firstByte;
			char[] byteMap = new char[256];
			TIntObjectHashMap<Integer> specialCount = new TIntObjectHashMap<Integer>(SPECIAL_CHARS.length * 2);
			Arrays.fill(byteMap, (char) 0);
			byte[] b = Conversion.getBytes(new String(SPECIAL_CHARS), font);
			
			
			for (j = 0; j < SPECIAL_CHARS.length; j++) {
				char c = SPECIAL_CHARS[j];
				specialCount.put(c, 0);
				byteMap[128 + b[j]] = SPECIAL_CHARS[j];
			}
	
//	
//			for (i = 0; i < quoteDtls.length; i++) {
//				quoteCount[i] = 0;
//				quoteBytes[i - st] = Conversion.getBytes(Common.QUOTE_LIST[i], font);
//			}
	
	//		for (i = Math.max(0, numberOfLines - 15); i < numberOfLines; i++) {
			CharData quoteDtls = toByteArray(CsvTextItem.QUOTE.getCsvList(false, false), STANDARD_QUOTES, font, true, '"');
			int[] quoteCount = new int[quoteDtls.getLength()];
			for (i = 0; i < lineLst.getLineCount(); i++) {
				byte[] line = lineLst.getLine(i);
				if (line != null && line.length > 0) {
					firstByte = line[0];
					for (j = 0; j < line.length - 1; j++) {
						if (j == 0 || line[j] == sepByte ) {
							for (k = 0; k < quoteDtls.getLength(); k++) {
								if (equals(line, j - quoteDtls.bytes[k].length, quoteDtls.bytes[k])) {
									quoteCount[k] += 1;
								}
								if (equals(line, j + 1, quoteDtls.bytes[k])) {
									quoteCount[k] += 1;
								}
							}
								
							char c = byteMap[128 + firstByte];
							if (j > 0 && firstByte == line[j-1] && specialCount.contains(c)) {
								specialCount.put(c, specialCount.get(c) + 1);
							}
							firstByte =  line[j+1];
						}
					}
					char c = byteMap[128 + firstByte];
					if (j > 0 && firstByte == line[j-1] && specialCount.contains(c)) {
						specialCount.put(c, specialCount.get(c) + 1);
					}
				}
			}
	
			int quoteIdxMax = getMax(quoteCount);
	
			if (quoteCount[quoteIdxMax] > 5
			|| (embeddedCr && quoteCount[quoteIdxMax] > 1)) {
				quote = quoteDtls.items[quoteIdxMax].value;
			} else {
				setSpecialQuote(lineLst.getLineCount() * 2, specialCount);
			}
		}
		seperatorId = charDtls.items[seperatorIdx].value;
		return charDtls.singleBytes[seperatorIdx];
	}

	
	/**
	 * @return the lineCount
	 */
	public final int getLineCount() {
		return lineCount;
	}

	/**
	 * @return the columnNames
	 */
	public final String[] getColumnNames() {
		return columnNames;
	}


	/**
	 * @param columnNames the columnNames to set
	 */
	public final void setColumnNames(String[] columnNames) {
		this.columnNames = columnNames;
	}

	private boolean equals(byte[] line, int pos, byte[] cmp) {
		boolean ret = false;
		if (pos >= 0 && pos + cmp.length <= line.length) {
			ret = true;
			for (int i = 0; i < cmp.length && ret; i++) {
				ret = line[pos + i] == cmp[i];
			}
		}
		return ret;
	}

	public static String getSeperator(byte[][] lines, int numberOfLines, String font) {

		CharData charDtls = toByteArray(CsvTextItem.DELIMITER.getCsvList(false, true), STANDARD_FIELD_DELIMITERS, font, false, DEFAULT_FIELD_DELIMETER_CHAR);

		int seperatorIndex = getSeperatorIndex2(new ByteArrayLineList(lines, 0, numberOfLines), font, charDtls);
		return seperatorIndex < 0 || seperatorIndex > charDtls.getLength()
					? null 
					: charDtls.items[seperatorIndex].value;
	}
//
//	private static byte[] getSepBytes(String font) {
//		String[] sep = /*(String[])*/ Common.FIELD_SEPARATOR_LIST1_VALUES;
//		byte[] sepBytes = new byte[sep.length];
//		for (int k = 0; k < sep.length; k++) {
//			sepBytes[k] = 0;
//			try {
//				sepBytes[k] = Conversion.getCsvDelimBytes(sep[k], font)[0];
//			} catch (Exception e) {
//			}
//		}
//		return sepBytes;
//	}
//
//	private static int getSeperatorIndex(byte[][] lines, int numberOfLines, String font, byte[] sepBytes) {
//		int i,j, k;
//		String s;
//		int[] count = new int[Common.FIELD_SEPARATOR_LIST1_VALUES.length];
//		String[] sep = /*(String[])*/ Common.FIELD_SEPARATOR_LIST1_VALUES.clone();
//		int spaceIdx = -1;
////		byte[] sepBytes = new byte[sep.length];
//		if (numberOfLines < 0) {
//			numberOfLines = lines.length;
//		}
//		//sep[0] = ",";
//		//sep[1] = "\t";
//
//		for (i = 0; i < count.length; i++) {
//			count[i] = 0;
//			if (" ".equals(sep[i])) {
//				spaceIdx = i;
//			}
//		}
//
//
//		for (i = Math.max(0, numberOfLines - 45); i < numberOfLines; i++) {
//			if (lines[i] != null) {
//				s = new String(lines[i]);
//				for (j = 0; j < s.length(); j++) {
//					for (k = 0; k < sep.length; k++) {
//						if (sep[k].startsWith("x'")) {
//							if (lines[i][j] == sepBytes[k]) {
//								count[k] += 1;
//								break;
//							}
//						} else if (sep[k].equals(s.substring(j, j+1))) {
//							count[k] += 1;
//							break;
//						}
//					}
//				}
//			}
//		}
//
//		return  getSeperatorIdx(count, numberOfLines, spaceIdx);// getMax(count);
//	}

	private static int getSeperatorIndex2(ByteArrayLineList lineLst, String font, CharData charDtls) {
		int i;
//		int[] count = new int[charDtls.length];
//		int[] lineCount = new int[charDtls.length];
//		String[] sep = /*(String[])*/ Common.FIELD_SEPARATOR_LIST1_VALUES.clone();
//		int spaceIdx = -1;
//		byte[] sepBytes = new byte[sep.length];
//		if (numberOfLines < 0) {
//			numberOfLines = lines.length;
//		}
		//sep[0] = ",";
		//sep[1] = "\t";
		
//		for (i = 0; i < count.length; i++) {
//			count[i] = 0;
//			if (chars[i] == ' ') {
//				spaceIdx = i;
//			}
//		}
		
		int start = lineLst.firstLineNumber > 0 ? 0 : Math.max(0, lineLst.getLastLineNumber() - 500);
		int lineCount2 = lineLst.getLineCount();
		for (i = start; i < lineCount2; i++) {
			byte[] line = lineLst.getLine(i);
			if (line != null) {	
				charDtls.updateCounts(line);
			}
		}



//		for (i = 0; i < count.length; i++) {
//			count[i] = 0;
//			if (" ".equals(sep[i])) {
//				spaceIdx = i;
//			}
//		}


//		int start = Math.max(0, numberOfLines - 200);
//		for (i = start; i < numberOfLines; i++) {
//			if (lines[i] != null) {
//				s = new String(lines[i]);
//				for (k = 0; k < sep.length; k++) {
//					boolean inLine = false;
//					for (j = 0; j < s.length(); j++) {
//						if (sep[k].startsWith("x'")) {
//							if (lines[i][j] == sepBytes[k]) {
//								count[k] += 1;
//								if (! inLine) {
//									lineCount[k] += 1;
//									inLine = true;
//								}
//								//break;
//							}
//						} else if (sep[k].equals(s.substring(j, j+1))) {
//							count[k] += 1;
//							if (! inLine) {
//								lineCount[k] += 1;
//								inLine = true;
//							}
//							//break;
//						}
//					}
//				}
//			}
//		}

		return  getSeperatorDtls(charDtls.count, charDtls.lineCount, lineCount2 - start, charDtls.spaceIdx);// getMax(count);
	}

	/**
	 * Check if column names exist in the file
	 * @param lines to check
	 */
	private void checkForColNames(ArrayList<ArrayList<String>> lines, int adj) {
		if (lines.size() == 0) { return; }
		ListLineTable lineTbl = new ListLineTable(lines);
		int i;


		numberOfColumns = 0;
		lineCount = lineTbl.lineCount();
		if (lineTbl == null || lineCount == 0) {
			return;
		}
		
		int[] colCounts = new int[21];
		for (i = 0; i < lineCount; i++) {
			int fieldCount = lineTbl.fieldCount(i);
			numberOfColumns = Math.max(numberOfColumns, fieldCount);
			colCounts[Math.min(fieldCount, colCounts.length-1)] +=1;
		}

		int fldNumLine = 0;
		if (lineTbl.fieldCount(0) < 5) {
			int num = colCounts[2];
			int cn = 2;
			for(int j = 3; j < colCounts.length; j++) {
				if (colCounts[j] > num) {
					cn = j;
					num = colCounts[j];		
				}
			}
						
			cn = Math.max(2, cn - 2);
			
			int m = Math.min(Math.max(LINES_TO_CHECK, fieldNameLineNo+20), lineCount - 1);
			int flineNum = fieldNameLineNo;
			boolean looking = true;
			while (flineNum < m
			&& (looking = lineTbl.fieldCount(flineNum - adj) == 0
						|| getSize(lineTbl, (flineNum - adj)) < cn)) {
				flineNum += 1;
			}
			
			if (! looking) {
				fieldNameLineNo = flineNum;
			}
			
			fldNumLine = fieldNameLineNo - adj;
		}
		//ArrayList<String> line = lines.get(fldNumLine - 1);
		int columns = lineTbl.lines.get(fldNumLine).size();
		for (i = 0; i < columns; i++) {
			try {
				Integer.parseInt(lineTbl.getField(fldNumLine, i));
				colNamesOnFirstLine = COLUMN_NAMES_NO;
				break;
			} catch (Exception e) {
			}
		}

		//if (colNamesOnFirstLine != COLUMN_NAMES_NO) {
		//int noNums, crCount, htmlCount;

		int limit = (lineCount - fldNumLine +1) / 3;
		int st =  fldNumLine ;

		if (colNamesOnFirstLine != COLUMN_NAMES_NO) {
			columnNames = new String[lineTbl.fieldCount(st)];
			for (int ii = 0; ii <columnNames.length; ii++) {
				columnNames[ii] = lineTbl.getField(st, ii);
			}
			st = fldNumLine+1;
		} 
			

		determineFieldTypes(lineTbl, st, limit);
	}

	private void determineFieldTypes(ILineTable lineTbl, int start, int limit) {
		int i;
		int noNums;
		int crCount;
		int htmlCount;
		colTypes = new FieldTypeDetails[numberOfColumns];
		for (int j = 0; j < numberOfColumns; j++) {
			noNums = 0;
			crCount = 0;
			htmlCount = 0;
			boolean decimalSame = true;
			boolean first = true;
			int decimal = 0;
			ColumnDateFormatChk dateChk = new ColumnDateFormatChk();
			colTypes[j] = FieldTypeDetails.CHAR_TYPE_DETAILS;
			for (i = start; i < lineTbl.lineCount(); i++) {
				//line = lines.get(i);

				if (j < lineTbl.fieldCount(i)) {
					String fieldValue = lineTbl.getField(i, j);
					dateChk.add(fieldValue);
					try {
						new BigDecimal(fieldValue);
						noNums += 1;
						
						if (decimalSame) {
							int pos = fieldValue.indexOf('.');
							int dec = pos < 0 ? 0 : fieldValue.length() - pos;
							decimalSame = decimal == dec || first;
							decimal = dec;
						}
						first = false;
					} catch (Exception e) {
					}
					if (fieldValue.indexOf('\n') >= 0 || fieldValue.indexOf('\r') >= 0) {
						crCount += 1;
					}
					if (Conversion.isHtml(fieldValue)) {
						htmlCount += 1;
					}
				}
			}
			

			//System.out.print("\t" + noNums);
			if (noNums > lineTbl.lineCount() - 2 && decimalSame && lineTbl.lineCount() > 40) {
				colNamesOnFirstLine = colNamesOnFirstLine == COLUMN_NAMES_NO ? COLUMN_NAMES_NO : COLUMN_NAMES_YES;
				colTypes[j] = decimal == 0 
								? FieldTypeDetails.CSV_LONG_TYPE_DETAILS 
								: new FieldTypeDetails(Type.ftNumLeftJustified, decimal, 0, "");
			} else if (noNums > 3 && noNums > limit) {
				colNamesOnFirstLine = colNamesOnFirstLine == COLUMN_NAMES_NO ? COLUMN_NAMES_NO : COLUMN_NAMES_YES;
				colTypes[j] = FieldTypeDetails.NUM_TYPE_DETAILS;
			} else if (htmlCount > limit) {
				colTypes[j] = FieldTypeDetails.HTML_TYPE_DETAILS;
			} else if (crCount > 0) {
				colTypes[j] = FieldTypeDetails.MULTI_LINE_TYPE_DETAILS;
			}

			List<StdDateFormatStr> possibleDateFormats;
			if ( lineTbl.lineCount() > 6 && (possibleDateFormats = dateChk.getPossibleDateFormats()).size() > 0
			&&  possibleDateFormats.get(0).dateFormatStr.length() > 0
			&&  (lineTbl.lineCount() > 18 || possibleDateFormats.get(0).dateFormatStr.length() > 6)
			) {
				//System.out.println("** " + possibleDateFormats.get(0).dateFormatStr);
				colTypes[j] = new FieldTypeDetails(Type.ftChar, 0,
						possibleDateFormats.get(0).fieldFormat, possibleDateFormats.get(0).dateFormatStr);
			}
		}

		boolean multiLineOrHtml = false;
		if (colTypes != null) {
			for (int k = 0; k < colTypes.length && textTypes == null; k++) {
				switch (colTypes[k].type) {
				case Type.ftMultiLineEdit:
				case Type.ftHtmlField:
					textTypes = new FieldTypeDetails[colTypes.length];
					TypeManager typeManager = TypeManager.getInstance();
					
					multiLineOrHtml = true;

					for (int j = 0; j < textTypes.length; j++) {
						textTypes[j] = colTypes[j];
						Type type ;
						if (textTypes[j] == null 
						|| (type = typeManager.getType(textTypes[j].type)) == null || type.isNumeric()) {
							textTypes[j] = FieldTypeDetails.CHAR_TYPE_DETAILS;
						}
					}

				}
			}
		}

		if (lineTbl.lineCount() <= fieldNameLineNo + 4) {
			if (multiLineOrHtml) {
				for (int k = 0; k < colTypes.length; k++) {
					switch (colTypes[k].type) {
					case Type.ftChar:
					case Type.ftMultiLineEdit:
					case Type.ftHtmlField:
						break;
					default:
						colTypes[k] = FieldTypeDetails.CHAR_TYPE_DETAILS;
					}
				}
			} else { 
				colTypes = null;
			}
		}
		//}

		if (quote == null || quote.length() == 0) {
			int j, k, l;
			CharData quoteDtls = toCharArray(CsvTextItem.QUOTE.getCsvList(false, false), STANDARD_QUOTES, font, DEFAULT_QUOTE_CHAR);

			for (i = 0; i < lineTbl.lineCount(); i++) {
				//line = lines.get(i);
				int fieldCount = lineTbl.fieldCount(i);
				for (j = 0; j < fieldCount - 1; j++) {
					for (k = 0; k < quoteDtls.getLength(); k++) {
						String lineStr = lineTbl.getField(i, j);
						if ( lineStr.length() > 0
						&&	 lineStr.charAt(0) == quoteDtls.chars[k]
						&&   lineStr.charAt(lineStr.length()-1) == quoteDtls.chars[k]) {
							for (l = j+1; l < fieldCount; l++) {
								String s = lineTbl.getField(i, l);
								if (s.length() > 0 && s.charAt(s.length() - 1) == quoteDtls.chars[k]) {
										//.endsWith(Common.QUOTE_LIST[k])) {
									quote = new String(new char[]{ quoteDtls.chars[k]} );
									return;
								}
							}
						}
					}

				}
			}
		}
	}


	private int getSize(ILineTable t, int lineNum) {
		int i = t.fieldCount(lineNum) - 1;
		String f;

		while (i >= 0 && ((f =t.getField(lineNum, i)) == null || f.length() == 0)) {
			i -= 1;
		}

		return i+1;
	}



	private int getSeperatorIdx(CharData delimiterCharDtls, int lineCount, int spaceIdx) {

		int[] array = delimiterCharDtls.count;
		int idxMax = 0;
		int idxMax2 = 0;
		int max = -1;
		int max2 = -1;
		
		likelyCsv = UNLIKELY_TO_BE_CSV;
		int cmp = delimiterCharDtls.getLength() * 17 / 20;
		if (lineNumber < 10) {
			for (int i = 0; i < array.length; i++) {
				if (max < array[i]) {
					max2 = max;
					idxMax2 = idxMax;
					max = array[i];
					idxMax = i;
				} else if (max2 < array[i]) {
					max2 = array[i];
					idxMax2 = i;
				}
			}
		} else {
			int[] linesWithChar = delimiterCharDtls.linesWithChar;
			int maxCmp = delimiterCharDtls.getLength() - 3;
			for (int i = 0; i < array.length; i++) {
				if (max < array[i]
				&& cmp < Math.min(maxCmp, linesWithChar[i])) {
					max2 = max;
					idxMax2 = idxMax;
					max = array[i];
					idxMax = i;
					if (delimiterCharDtls.isStdCsvDelimiter(idxMax)) {
						likelyCsv = LIKELY_TO_BE_CSV;
					}
				} else if (max2 < array[i]) {
					max2 = array[i];
					idxMax2 = i;
				}
			}
		}
		
		if (idxMax == spaceIdx && array[idxMax2] >= lineCount * 2) {
			idxMax = idxMax2;
			max = max2;
		}
		
		if (likelyCsv == UNLIKELY_TO_BE_CSV && idxMax >= 0 
		&& delimiterCharDtls.isStdCsvDelimiter(idxMax)) {
			if (max > cmp * 2 && delimiterCharDtls.linesWithChar[idxMax] > cmp) {
				likelyCsv = LIKELY_TO_BE_CSV;
			} else if (max > cmp) {
				likelyCsv = COULDBE_TO_BE_CSV;
			}
		}

		return idxMax;
	}


	private static int getSeperatorDtls(int[] array, int[] sepLineCounts, int lineCount, int spaceIdx) {

		int idxMax = 0;
//		int idxMax2 = 0;
		int max = -1;
		int max2 = -1;
		int minLineCount = lineCount * 4 / 5;
		for (int i = 0; i < array.length; i++) {
			if (i == spaceIdx || sepLineCounts[i] < minLineCount) {
			} else if (max < array[i]) {
				max2 = max;
				//idxMax2 = idxMax;
				max = array[i];
				idxMax = i;
			} else if (max2 < array[i]) {
				max2 = array[i];
				//idxMax2 = i;
			}
		}


		return idxMax;
	}

	private static int getMax(int[] array) {

		int idxMax = 0;
		int max = -1;
		for (int i = 0; i < array.length; i++) {
			if (max < array[i]) {
				max = array[i];
				idxMax = i;
			}
		}

		return idxMax;
	}

	/**
	 * @return the seperatorIdx
	 */
	public String getSeperatorId() {
		return seperatorId;
	}

	/**
	 * @return the quoteIdx
	 */
	public String getQuote() {
		return quote;
	}

	public final boolean isLikelyEmbeddedCr() {
		return embeddedCr || likelyEmbeddedCr;
	}

	public final Character getSpecialQuote() {
		return specialQuote;
	}

	/**
	 * @return the colNamesOnFirstLine
	 */
	public int getColNamesOnFirstLine() {
		return colNamesOnFirstLine;
	}

	public int getFieldNameLineNo() {
		return fieldNameLineNo;
	}

	/**
	 * @return the numberOfColumns
	 */
	public int getNumberOfColumns() {
		return numberOfColumns;
	}


	public FieldTypeDetails[] getTypes() {
		FieldTypeDetails[] ret = null;
		if (colTypes != null) {
			ret = colTypes.clone();
		}

		return ret;
	}
	


	public FieldTypeDetails[] getTextTypes() {

		if (textTypes != null) {
			return textTypes.clone();
		}

		return null;
	}

	/**
	 * @return the validChars
	 */
	public boolean isValidChars() {
		return ratio > 0.75;
	}

	public boolean isCsvFile() {
		return csvFile;
	}

	private static final class CharData {
		private int lengthVal = 0;
		int spaceIdx = -1;
		int[] count, lineCount;
		int[] linesWithChar;
		
		final char[] chars;
		final byte[] singleBytes;
		final byte[][] bytes;
		final CsvTextItem[] items;
		
		
		protected CharData(char[] chars, byte[] singleBytes, byte[][] bytes) {
			super();
			this.chars = chars;
			this.singleBytes = singleBytes;
			this.bytes = bytes;
			
			this.items = new CsvTextItem[chars!=null? chars.length : bytes.length];
		}
		/**
		 * @return the length
		 */
		public final int getLength() {
			return lengthVal;
		}
		
		public boolean isStdCsvDelimiter(int charIdx) {
			char ch = this.chars[charIdx];
			return ch == ',' || ch == '\t' || ch == ';' || ch == '|';
		}
		/**
		 * @param length the length to set
		 */
		public final void setLength(int length) {
			this.lengthVal = length;
			count = new int[length];
			lineCount = new int[length];
			linesWithChar = new int[length];

		}
		
		public final void updateCounts(String s) {
			for (int i = 0; i < lengthVal; i++) {
				boolean found = false;
				for (int j = 0; j < s.length(); j++) {
					if (chars[i] == s.charAt(j)) {
						count[i] += 1;
						found = true;
					}	
				}
				
				if (found) {
					linesWithChar[i] += 1;
				}
			}
		}
		
		public final void updateCounts(char ch) {
			for (int i = 0; i < lengthVal; i++) {
				if (chars[i] == ch) {
					count[i] += 1;
					break;
				}
			}
		}
		
		public final void updateCounts(byte[] line) {
			for (int k = 0; k < lengthVal; k++) {
				boolean inLine = false;
				for (int j = 0; j < line.length; j++) {
					if (line[j] == singleBytes[k]) {
						count[k] += 1;
						if (! inLine) {
							lineCount[k] += 1;
							inLine = true;
						}
					}
				}
			}

		}
	}

	/**
	 * @return the likelyCsv
	 */
	public int getLikelyCsv() {
		return likelyCsv;
	}
	
	private static final byte[] CSV_CHARS = Conversion.getBytes("\n,\t;|", Conversion.DEFAULT_ASCII_CHARSET);
	private static final byte[] EBCDIC_CSV_CHARS = Conversion.getBytes("\n,\t;|", "cp037");
	
	public static boolean isLikelyToBeCsv(byte[] data, String font) {
		if (data == null || data.length == 0) { return false; }

		byte[] csvChars = Conversion.isEbcidic(font) ? EBCDIC_CSV_CHARS : CSV_CHARS;
		
		int[] byteIdx = new int[255];
		
		Arrays.fill(byteIdx, -1);
		
		for (int i = 0; i < csvChars.length; i++) {
			byteIdx[csvChars[i] + 128] = i;
		}
		
		int lineCount = 0;
//		int[] counts = new int[csvChars.length];
		int[] countForCurrentLine = new int[csvChars.length];
		int[] presentLineCount = new int[csvChars.length];
		Arrays.fill(presentLineCount, 0);
		Arrays.fill(countForCurrentLine, 0);
		
		for (int i = 0; i < data.length; i++) {
			int idx = byteIdx[data[i] + 128];
			if (idx == 0) {
				updateLineCounts(countForCurrentLine, presentLineCount);
				lineCount += 1;
			} else if (idx > 0 && idx < countForCurrentLine.length) {
				countForCurrentLine[idx]  += 1;
			}
		}
		updateLineCounts(countForCurrentLine, presentLineCount);
		
		if ( lineCount > 0) {
			for (int j = 1; j < presentLineCount.length; j++) {
				if (presentLineCount[j]  * 100 / lineCount > 80) {
					return true;
				}
			}
		}
		return false;
	}

	private static void updateLineCounts(int[] countForCurrentLine, int[] presentLineCount) {
		for (int j = 1; j < countForCurrentLine.length; j++) {
			if (countForCurrentLine[j] > 0) {
				presentLineCount[j] += 1;
			}
		}
		Arrays.fill(countForCurrentLine, 0);
	}
	
	
	public static final class ListLineTable implements ILineTable {
		final ArrayList<ArrayList<String>> lines;

		public ListLineTable(ArrayList<ArrayList<String>> lines) {
			super();
			this.lines = lines;
		}

		@Override
		public int lineCount() {
			return lines.size();
		}

		@Override
		public int fieldCount(int lineNum) {
			return lines.get(lineNum) == null 
					? 0 
					: lines.get(lineNum).size();
		}

		@Override
		public String lastField(int lineNum) {
			int fieldCount = fieldCount(lineNum);
			return fieldCount == 0 ? "" : getField(lineNum, fieldCount - 1);
		}

		@Override
		public String getField(int lineNum, int colNum) {
			return lines.get(lineNum).get(colNum);
		}
		
		
	}
	public static interface ILineTable {
		
		int lineCount();
		int fieldCount(int lineNum);
		String lastField(int lineNum);
		String getField(int lineNum, int colNum);
		
	}
}
