package net.sf.RecordEditor.utils.fileAnalysis;

import java.io.File;

import net.sf.JRecord.Common.Constants;
import net.sf.JRecord.CsvParser.CsvParserManagerChar;
import net.sf.JRecord.CsvParser.ICsvCharLineParser;
import net.sf.JRecord.CsvParser.ICsvParserIds;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.Details.RecordDetail;
import net.sf.JRecord.fieldNameConversion.FieldNameConversionManager;
import net.sf.JRecord.fieldNameConversion.IFieldNameConversion;
import net.sf.RecordEditor.utils.common.Common;

public class CsvSchemaBuilder {

	private final CsvAnalyser analyser;
	private final int fileStructure;
	private final String filename;
	private final boolean fieldNamesInFile;
	private int parserType = ICsvParserIds.EXTENDED_BASIC_CSV_PARSER;
	private String font = "", seperator=",", quote="\"";
	private boolean embeddedCr, deriveTheTypes;
	private byte[] recordSep;
//	private int firstCsvLineNumber = -1;

	public CsvSchemaBuilder(CsvAnalyser analyser, String filename, int fileStructure, boolean fieldNamesInFile) {
		super();
		this.analyser = analyser;
		this.filename = filename;
		this.fileStructure = fileStructure;
		this.fieldNamesInFile = fieldNamesInFile;
	}

	/**
	 * @param parserType the parserType to set
	 */
	public final CsvSchemaBuilder setParserType(int parserType) {
		this.parserType = parserType;
		
		return this;
	}

	/**
	 * @param font the font to set
	 */
	public final CsvSchemaBuilder setFont(String font) {
		this.font = font;
		
		return this;
	}

	/**
	 * @param sperator the sperator to set
	 */
	public final CsvSchemaBuilder setSeperator(String seperator) {
		this.seperator = seperator;
		
		return this;
	}

	/**
	 * @param quoter the quoter to set
	 */
	public final CsvSchemaBuilder setQuote(String quote) {
		this.quote = quote;
		
		return this;
	}

	/**
	 * @param embeddedCr the embeddedCr to set
	 */
	public final CsvSchemaBuilder setEmbeddedCr(boolean embeddedCr) {
		this.embeddedCr = embeddedCr;
		
		return this;
	}
	
	/**
	 * @param deriveTheTypes the deriveTheTypes to set
	 */
	public final CsvSchemaBuilder setDeriveTheTypes(boolean deriveTheTypes) {
		this.deriveTheTypes = deriveTheTypes;
		
		return this;
	}

	/**
	 * @param recordSep the recordSep to set
	 */
	public final CsvSchemaBuilder setRecordSep(byte[] recordSep) {
		this.recordSep = recordSep;
		
		return this;
	}


	public LayoutDetail buildSchema() {
		LayoutDetail layout;
		String s;
		FieldTypeDetails fieldType ;
	    FieldTypeDetails[] fieldTypes = null;
	    
	    int numCols = analyser.getNumberOfColumns();
	    String[] colNames= analyser.getColumnNames();
	    ICsvCharLineParser csvParser = CsvParserManagerChar.getInstance().get(parserType);
//	    boolean quoteInColumnNames = csvParser.isQuoteInColumnNames();

        RecordDetail.FieldDetails[] flds = new RecordDetail.FieldDetails[numCols];
        RecordDetail[] recs = new RecordDetail[1];
		if (deriveTheTypes) {
         	fieldTypes = analyser.getTypes();
        } else {
         	fieldTypes = analyser.getTextTypes();
        }
		
		if (csvParser.isQuoteInColumnNames()) {
			for (int i = 0; i < colNames.length; i++) {
				if (colNames[i] != null && (colNames[i].startsWith("\"") || colNames[i].endsWith("\""))) {
					int len = colNames[i].length();
					colNames[i] = colNames[i].substring(
							colNames[i].startsWith("\"") ? 1 : 0,
							len > 1 && colNames[i].endsWith("\"") ? len - 1 : len);
				}

			}
		}

	    for (int i = 0; i < numCols; i++) {
	    	fieldType = FieldTypeDetails.CHAR_TYPE_DETAILS;
	    	if (fieldTypes != null && i < fieldTypes.length) {
	    		fieldType = fieldTypes[i];
	    	}
	    	if (colNames == null || colNames.length <= i || colNames[i] == null || colNames[i].length() == 0) {
	    		s = "f" + i;
	    	} else {
	    		s = colNames[i];
	    	}

	    	flds[i] = new RecordDetail.FieldDetails(s, s, fieldType.type, fieldType.decimal,
                        font, fieldType.format, fieldType.paramter);
            flds[i].setPosOnly(i + 1);
	    }
	    
	    //TODO update
        boolean useThisLayout = true;
	    if (fieldTypes == null 
	    && analyser.getColNamesOnFirstLine() != CsvAnalyser.COLUMN_NAMES_NO
	    && analyser.getLineCount() == 0) {
	    	useThisLayout = false; 
	    }

        String layoutName = "GeneratedCsv";
        String recordName = Common.GENERATED_CSV_SCHEMA_NAME;
        if (filename != null && filename.length() > 0) {
        	String fname = new File(filename).getName();
        	int pos = fname.indexOf('.');
        	if (pos > 0) {
        		fname = fname.substring(0, pos);
        	}
            IFieldNameConversion conversion = FieldNameConversionManager.STD_CBL_FIELD_NAME_CONVERSION;
        	String javaId = conversion.string2JavaId(fname);
			layoutName = conversion.toClassName(javaId);
        	recordName = "Rec" + conversion.toSuffix(javaId);
        }
        recs[0] = new RecordDetail(recordName, "", "", Constants.rtDelimited,
        		seperator,  quote, font, flds,
        		parserType, 0, embeddedCr, csvParser);

        
		layout  =
            new LayoutDetail(layoutName, recs, "",
            		fileStructure,
                recordSep, "", font, null,
                fileStructure
            );

        layout.setUseThisLayout(useThisLayout);
        

        int fieldNameLineNo = analyser.getFieldNameLineNo();
        
        if (fieldNameLineNo > 1) {
        	if (! fieldNamesInFile) {
        		fieldNameLineNo -= 1;
        	}
        	layout.setLineNumberOfFieldNames(fieldNameLineNo);
        }
        
		return layout;
	}
}
