/**
 * 
 */
package net.sf.RecordEditor.utils.fileStorage;

import java.util.Collection;
import java.util.Comparator;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import net.sf.JRecord.Details.AbstractLine;
import net.sf.RecordEditor.trove.TIntArrayList;
import net.sf.RecordEditor.utils.swing.common.IProgressDisplay;

/**
 * @author Bruce Martin
 *
 */
public class DataStoreLargeView<L extends AbstractLine>
extends DataStoreBase
implements IDataStoreView<AbstractLine>, ISortNotify, TableModelListener, IParentLineNumber {

//	private IDataStore<? extends AbstractLine> parent;
	private final IDataStore<L> parent;
	private TIntArrayList list;
//	private boolean doingSort;
	int count;
	
	public DataStoreLargeView(IDataStore<L> parent, int... listValue) {
		this(parent, true, listValue);
	}
	protected DataStoreLargeView(IDataStore<L> parent, boolean notifyParent, int... listValue) {

		super(parent); 
		this.parent = parent;
		this.list = new TIntArrayList(listValue);
		
		int max = parent.size();
		for (int i = list.size() - 1; i >= 0; i--) {
			if (list.get(i) < 0 || list.get(i) >= max) {
				list.removeAt(i);
			}
		}
		
		if (notifyParent) {
			parent.addChildViewRE(this);
		}
	}
	

	public DataStoreLargeView<L> addRange(int start, int end) {
		
		list.ensureCapacity(end - start);
		for (int i = start; i <= end; i++) {
			list.add(i);
		}
		return this;
	}
	
//
//	@Override
//	public void addChildView(ISortNotify notify) {
//		throw new RuntimeException("AddChildView is not supported DataStoreRange");
//	}
	
	@Override
	public void addCopyRE(int pos, IDataStore<? extends AbstractLine> cpyLines,
			IProgressDisplay progressDisplay) {
		parent.addCopyRE(list.get(pos), cpyLines, progressDisplay);
	}

	@Override
	public void extractLinesRE(IDataStore<AbstractLine> cpyLines,
			int[] linesToExtract, IProgressDisplay progressDisplay) {
		
		int[] parentLines = new int[linesToExtract.length]; 
		for (int i = 0; i < parentLines.length; i++) {
			parentLines[i] = list.get(linesToExtract[i]);
		}
		parent.extractLinesRE(cpyLines, parentLines, progressDisplay);
	}


	public void ensureCapacity(int capacity) {
		list.ensureCapacity(capacity);
	}


	@Override
	public AbstractLine getTempLineRE(int index) {
		if (index < 0 || index > list.size()) {
			return null;
		}
		return parent.getTempLineRE(list.get(index));
	}

//	@Override
//	public String getSizeDisplay() {
//		return "";
//	}
//
//	@Override
//	public long getSpace() {
//		return 0;
//	}

	@Override
	public IDataStore<AbstractLine> newDataStoreRE() {
		return new DataStoreLargeView<L>(parent, list.toArray());
	}

	@Override
	public DataStoreLargeView<L> newDataStoreRE(int[] lines) {
		return new DataStoreLargeView<L>(parent, getParentIndexs(lines));
	}
	
	private int[] getParentIndexs(int[] lines) {
		int[] parentLines = new int[lines.length];
		for (int i = 0; i <parentLines.length; i++) {
			parentLines[i] = list.get(lines[i]);
		}
		return parentLines;
	}
	

	//@Override
	public void sort(Comparator<? super AbstractLine> c) {
		sortRE(c);
	}
	@Override
	public void sortRE(Comparator<? super AbstractLine> compare) {
//		doingSort = true;
		count = 0;
		int[] array =  list.toArray();
		quickSort(array, 0, array.length-1, compare);
		list = new TIntArrayList(array);
//		doingSort = false;
	}

	int partition(int arr[], int left, int right, Comparator<? super AbstractLine> compare) {

		int i = left, j = right;
		int tmp;
		AbstractLine pivot =  parent.getNewLineRE(arr[(left + right) / 2]);
//		if (right - left > 1000) {
//			System.out.println(" >>> " + left + " " + right);
//		}

		while (i <= j) {
			while (compare.compare(parent.getTempLineRE(arr[i]), pivot) < 0) { //arr[i] < pivot) {
				i++;
			}

			while (compare.compare(parent.getTempLineRE(arr[j]), pivot) > 0) { //arr[j] > pivot) {
				j--;
			}

			if (i <= j) {
				tmp = arr[i];
				arr[i] = arr[j];
				arr[j] = tmp;
				i++;
				j--;
			}
		}

		return i;

	}

	void quickSort(int arr[], int left, int right, Comparator<? super AbstractLine> compare) {

		int index = partition(arr, left, right, compare);

		if (left < index - 1) {
			quickSort(arr, left, index - 1, compare);
		}

		if (index < right) {
			quickSort(arr, index, right, compare);
		}

	}
	
	@Override
	public void sortRE(int[] rows, Comparator<? super AbstractLine> compare) {
		DataStoreLargeView<L> sv = this.newDataStoreRE(rows);
		
		sv.sortRE(compare);
		for (int i = 0; i < rows.length; i++) {
			list.set(rows[i], sv.list.get(i));
		}
		sv.clear();
	}



/**
	 * @return
	 * @see net.sf.RecordEditor.trove.TIntArrayList#toArray()
	 */
	public int[] getBaseListIndexs() {
		return list.toArray();
	}
//	@Override
//	public AbstractLine addCopy(int index, AbstractLine line) {
//		throw new RuntimeException("addCopy should not be called for views");
//	}
//
//	@Override
//	public AbstractLayoutDetails getLayout() {
//		return parent.getLayout();
//	}
//
//	@Override
//	public void setLayout(AbstractLayoutDetails layout) {
//		parent.setLayout(layout);
//	}
//
//	@Override
//	public boolean add(AbstractLine e) {
//		// TO DO Auto-generated method stub
//		return super.add(e);
//	}

	
	@Override
	public AbstractLine get(int index) {
		if (index < 0 || index > list.size()) {
			return null;
		}

		//if (doingSort && count++ % 2000 == 0) System.out.print("$$ " + index + " " + list.get(index) + " $$");
		//System.out.println(" ~~> " + index + " " + list.get(index));
		return parent.get(list.get(index));
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.fileStorage.IDataStore#getLineLength(int)
	 */
	@Override
	public int getLineLengthRE(int index) {
		if (index < 0 || index > list.size()) {
			return 0;
		}

		return parent.getLineLengthRE(list.get(index));
	}
	
	@Override
	public AbstractLine getNewLineRE(int index) {
		if (index < 0 || index > list.size()) {
			return null;
		}

		return parent.getNewLineRE(list.get(index));
	}
	
	
//
//	@Override
//	public byte[] getLineAsBytes(int index) {
//		return parent.getLineAsBytes(list.get(index));
//	}


	@Override
	public AbstractLine set(int index, AbstractLine element) {
		int parentIndex = getParentIndex(element);
		if (parentIndex < 0 ) {
			return null;
		}
		list.set(index, parentIndex);
		return element;
	}

	@Override
	public void add(int index, AbstractLine element) {
		
		//System.out.print(" ==> " + index);
		int size = parent.size();
		if (index < 0 || index >= size) {
			return;
		} 
		int parentIndex = getParentIndex(element);

		//System.out.println(" : " + index + " " + parentIndex);
		if (parentIndex < 0) {
			if (index < size() && list.get(index) < size - 1) {
				parentIndex = list.get(index) + 1;
				parent.add(list.get(index) + 1, (L) element);
			} else {
				parentIndex = size;
				parent.add((L) element);
			}
		}
		list.insert(index, parentIndex);
	}



	@Override
	public boolean addAll(int index, Collection<? extends AbstractLine> c) {
		if (c instanceof DataStoreLargeView) {
			@SuppressWarnings("rawtypes")
			int[] newLines = ((DataStoreLargeView) c).list.toArray();
			list.insert(index, newLines);
			return newLines.length > 0;
		} else if (c instanceof DataStoreRange) {
			DataStoreRange cr = (DataStoreRange) c;
			int[] il = new int[cr.size()];
			for (int i = 0; i < il.length; i++) {
				il[i] = cr.getStart() + i;
			} 
			list.insert(index, il);
			return il.length > 0;
		}
		return super.addAll(index, c);
	}

	@Override
	public boolean remove(Object o) {
		int idx = parent.indexOf(o);
		if (idx >= 0) {
			int idx2 = list.indexOf(idx);
			if (idx2 >= 0) {
				list.remove(idx2);
				parent.remove(idx);
				return true;
			}
		}
		return false;
	}

	@Override
	public AbstractLine remove(int index) {
		if (index < 0 || index > list.size()) {
			return null;
		}
		list.removeAt(index);
		return null;
	}

	@Override
	public int indexOf(Object o) {
		
		int ret = getParentIndex(o);
		if (ret >= 0) {
			ret = list.indexOf(ret);
		}
		
		return ret;
	}

	@Override
	public int lastIndexOf(Object o) {
		int ret = getParentIndex(o);
		if (ret >= 0) {
			ret = list.lastIndexOf(ret);
		}
		
		return ret;
	}

	@Override
	public void clear() {
		list.clear();
	}


	@Override
	public int size() {
		return list.size();
	}
	
	
	@Override
	public int[] asIntArray() { 
		return list.toArray(); 
	}
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.fileStorage.SortNotify#updateForParentSort(java.util.Map)
	 */
	@Override
	public final void updateForParentSort(int[] newOrder) {
		for (int i = 0; i < list.size(); i++) {
			if (newOrder[list.get(i)] >= 0) {
				list.set(i, newOrder[list.get(i)]);
			}
		}
	}
	

	
	@Override
	public void tableChangedUpdateRE(TableModelEvent event) {

		int firstRow = event.getFirstRow();
		int lastRow = event.getLastRow();
		int adj = lastRow - firstRow + 1;
		switch (event.getType()) {
		case (TableModelEvent.INSERT):
		    for (int i = 0; i < list.size(); i++) {
		    	if (list.get(i) >= firstRow) {
		    		list.set(i, list.get(i) + adj);
		    	}
		    }
		break;
		case (TableModelEvent.DELETE):
		    for (int i = 0; i < list.size(); i++) {
		    	if (list.get(i) > lastRow) {
		    		list.set(i, list.get(i) - adj);
		    	}
		    }
		break;
		}
	}

//	private int getParentIndex(Object o) {
//		if (o instanceof IChunkLine) {
//			@SuppressWarnings("rawtypes")
//			IChunkLine lc = (IChunkLine) o;
//			return (lc.getActualLine());
//		}
//		
//		return parent.indexOf(o);
//	}

	@Override
	public int getParentIndexRE(int index) {
		return list.get(index);
	}
	@Override
	public int getLineNumForBaseLine(int lineNum) {
		return list.indexOf(lineNum);
	}
	
	
}
