/**
 * 
 */
package net.sf.RecordEditor.utils.fileStorage;

import java.util.Comparator;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.RecordEditor.utils.swing.common.IProgressDisplay;

/**
 * @author Bruce Martin
 *
 */
public class DataStoreWindow<L extends AbstractLine>
extends DataStoreBase
implements IDataStoreView<AbstractLine>, TableModelListener, IParentLineNumber {

//	private IDataStore<? extends AbstractLine> parent;
//	private TIntArrayList list;
//	private boolean doingSort;
	//int count;
	private int first, last;
	private final boolean passSortThrough;	
	private final IDataStore<L> parent;
	

	
	public DataStoreWindow(IDataStore<L> parent, int first, int last, boolean passSortThrough) {
		super(parent); 
		this.first = first;
		this.last = last;
		this.passSortThrough = passSortThrough;
		this.parent = parent;
	}
	

//	public DataStoreWindow addRange(int start, int end) {
//		
//		list.ensureCapacity(end - start);
//		for (int i = start; i <= end; i++) {
//			list.add(i);
//		}
//		return this;
//	}
	

	
	@Override
	public void addCopyRE(int pos, IDataStore<? extends AbstractLine> cpyLines,
			IProgressDisplay progressDisplay) {
		parent.addCopyRE(pos + first, cpyLines, progressDisplay);
	}

	@Override
	public void extractLinesRE(IDataStore<AbstractLine> cpyLines,
			int[] linesToExtract, IProgressDisplay progressDisplay) {
		parent.extractLinesRE(cpyLines, parentLines(linesToExtract), progressDisplay);
	}


	/**
	 * @param linesToExtract
	 * @return
	 */
	private int[] parentLines(int[] lines) {
		int[] parentLines = new int[lines.length]; 
		for (int i = 0; i < parentLines.length; i++) {
			parentLines[i] = lines[i] + first;
		}
		return parentLines;
	}


//	public void ensureCapacity(int capacity) {
//		parent.ensureCapacity(capacity);
//	}


	@Override
	public AbstractLine getTempLineRE(int index) {
		if (outsideRange(index)) { return null; }
		return parent.getTempLineRE(index + first);
	}
	
	@Override
	public int getParentIndexRE(int lineNum) {
		return lineNum + first;
	}

	@Override
	public int getLineNumForBaseLine(int lineNum) {
		return lineNum - first;
	}


	@Override
	public IDataStore<AbstractLine> newDataStoreRE() {
		return new DataStoreWindow<L>(parent, first, lastVal(), passSortThrough);
	}

	@Override
	public IDataStore<AbstractLine> newDataStoreRE(int[] lines) {
		return new DataStoreLargeView(parent, parentLines(lines)); 
	}
	

	

	//@Override
	public void sort(Comparator<? super AbstractLine> c) {
		sortRE(c);
	}
	
	
	@Override
	public void sortRE(Comparator<? super AbstractLine> compare) {
		
		if (passSortThrough) {
			parent.sort(compare);
		} else {
			int lastVal = lastVal();
			int[] rows = new int[lastVal-first + 1];
			for (int i = first; i <= lastVal; i++) {
				rows[i-first] = i;
			}
			parent.sortRE(rows, compare);
		}
	}

	
	@Override
	public void sortRE(int[] rows, Comparator<? super AbstractLine> compare) {
		parent.sortRE(rows, compare);
	}



	
	@Override
	public AbstractLine get(int index) {
		if (outsideRange(index)) { return null; }
		return parent.get(index + first);
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.fileStorage.IDataStore#getLineLength(int)
	 */
	@Override
	public int getLineLengthRE(int index) {
		if (outsideRange(index)) {
			return 0;
		}
		return parent.getLineLengthRE(index+first);
	}
	
	@Override
	public AbstractLine getNewLineRE(int index) {
		if (outsideRange(index)) { return null; }
		return parent.getNewLineRE(index + first);
	}
	
	
//
//	@Override
//	public byte[] getLineAsBytes(int index) {
//		return parent.getLineAsBytes(list.get(index));
//	}


	@Override
	public AbstractLine set(int index, AbstractLine element) {
		parent.set(index+first, (L) element);
		return element;
	}

	@Override
	public void add(int index, AbstractLine element) {	
		int idx = index + first;
		if (idx >= parent.size()) {
			parent.add((L) element);
		} else {
			parent.add(idx, (L) element);
		}
		if (last >= 0) {
			last += 1;
		}
	}



//	@Override
//	public boolean addAll(int index, Collection<? extends AbstractLine> c) {
//		
//		return parent.addAll(index+first, c);
//	}

	@Override
	public boolean remove(Object o) {
		return parent.remove(o);
	}

	@Override
	public AbstractLine remove(int index) {
		if (outsideRange(index)) {
			throw new RuntimeException("nvalid Index: " + index);
		}
		return parent.remove(index + first);
	}

	@Override
	public int indexOf(Object o) {
		return checkIfInrange(parent.indexOf(o) - first);
	}

	@Override
	public int lastIndexOf(Object o) {
		return checkIfInrange(parent.lastIndexOf(o) - first);
	}

	@Override
	public void clear() {
		for (int i = lastVal(); i >= first; i--) {
			parent.remove(i);
		}
	}


	/**
	 * @param index
	 * @return
	 */
	private int checkIfInrange(int index) {
		if (outsideRange(index)) {
			index = -1;			
		}
		return index;
	}


	/**
	 * @param index
	 * @return
	 */
	private boolean outsideRange(int index) {
		return index < 0 || index > lastVal()-first;
	}


	@Override
	public int size() {
		return lastVal()-first+1;
	}


	private int lastVal() {
		int lval = parent.size();
		return last >= 0 ? Math.min(last, lval) : lval-1; 
	}
	

	
	@Override
	public void tableChangedUpdateRE(TableModelEvent event) {

		int lastRow = event.getLastRow();
		int firstRow = event.getFirstRow();
		int adj = lastRow - firstRow + 1;
		switch (event.getType()) {
		case (TableModelEvent.DELETE):
			if (firstRow > lastVal()) {
				
			} else if (first > lastRow ) {
		    	first -= adj;
		    	if (last >= 0) {
		    		last -= adj;
		    	}
		    } else {
		    	if (first >= firstRow ) {
		    		first = firstRow;
		    	}
		    	if (last >= 0) {
		    		last = last > lastRow ? last - adj : firstRow;
		    	}
		    }
			break;
		case (TableModelEvent.INSERT):
		    if (first > firstRow ) {
		    	first += adj;
		    	if (last >= 0) {
		    		last += adj;
		    	}
		    }
		break;
		}
	}

	@Override
	public void update(AbstractLayoutDetails newLayout, int[] trans, IProgressDisplay pd) {
		parent.update(newLayout, trans, pd);
	}

//	private int getParentIndex(Object o) {
//		if (o instanceof IChunkLine) {
//			@SuppressWarnings("rawtypes")
//			IChunkLine lc = (IChunkLine) o;
//			return (lc.getActualLine());
//		}
//		
//		return parent.indexOf(o);
//	}
//
//	@Override
//	public int getParentIndexRE(int index) {
//		return index + first;
//	}
	
	
}
