package net.sf.RecordEditor.utils.fileStorage;import java.util.Comparator;
import java.util.Iterator;

import net.sf.JRecord.Details.AbstractLine;


/**
 * This class stores a sorted section of a file.
 * 
 * @author Bruce Martin
 *
 */
public class SortSection {
	final int[] filePositions;
	final DataStoreLarge<? extends IChunkLine, ? extends IRecordStore> sourceDs, sortedDs;

	final int start, end;
	
	private Iterator<AbstractLine> lineIteratorThatClearsFile;
	
	private AbstractLine currentLine;
	private int currSortedLineNum = -1;
	
	public SortSection(
			DataStoreLarge<? extends IChunkLine, ? extends IRecordStore> sourceDs, 
			int start, int end,
			Comparator<? super AbstractLine> lineCompare) {
		super();
		
		this.sourceDs = sourceDs;
		this.start = start;
		this.end = end;
		
		DataStoreLargeView sv = new DataStoreLargeView(sourceDs).addRange(start, end);
		sv.sort(lineCompare);
		filePositions = sv.getBaseListIndexs();
		
		FileDetails fd = sourceDs.getFileDetails();
		FileDetails fileDetails = fd.newSortFileDetails();
		fileDetails.setRecentSizeTest(5);
		
		sortedDs = new DataStoreLarge<IChunkLine, IRecordStore>(fileDetails); 
		
		for (int i = 0; i < sv.size(); i++) {
			sortedDs.add(sv.getNewLineRE(i));
		}
		
		sourceDs.clear(start, end);
		
		lineIteratorThatClearsFile = sortedDs.lineIteratorThatClearsFileAsItGoes();
		
		nextLine();
	}

	public AbstractLine nextLine() {
		currSortedLineNum += 1;
		return (currentLine = lineIteratorThatClearsFile.next());
	}
	
	public boolean hasNextLine() {
		return lineIteratorThatClearsFile.hasNext();
	}
	/**
	 * @return the currentLine
	 */
	public AbstractLine getCurrentLine() {
		return currentLine;
	}
	
	public int getCurrentBaseFileLineNo() {
		return filePositions[currSortedLineNum];
	}
}
