package net.sf.RecordEditor.utils.fileStorage;

import net.sf.RecordEditor.utils.common.Common;


public class ZOld_RecordStoreVariableLength extends RecordStoreBase implements ISimpleRecStore {
	
	private static final int INDEX_RECORD = 8;

	private int maxLen;
	private int[] recIndex = null;
	private int recIndexValidTo = -1;
	private int lastLookUpPosition = -11;
	private int lastLookUpBlockIndex = -1;


	public ZOld_RecordStoreVariableLength(int length, int recLen, int recordOverhead, int recs) {
		super(recs, recordOverhead);
		
		if (length >= 0) {
			this.store = new byte[length];
		}

		this.maxLen = Math.max(1, recLen);
	}


	@Override
	public void add(int idx, byte[] rec) {
		super.add(idx, rec);
		size += rec.length + super.recordOverhead;
//		System.out.println("       Add: " + this.hashCode()
//				+ " > " + idx + " " + super.recordCount
//				+ " > " + rec.length + " " + size);
		checkRecordIndex(idx);
	}
	
	


	@Override
	public void add(int idx, IRecordStore rStore) {
		
		super.add(idx, rStore);
		checkRecordIndex(idx);
	}
	
	protected final void checkRecordIndex(int idx) {
		
		if (recIndex != null
		&& ( idx < recordCount - 1
		||  (idx % INDEX_RECORD == INDEX_RECORD - 1))) {
			resetIndex(idx);
		} else {
			lastLookUpBlockIndex = -11;
		}
	}

	@Override
	public byte[] getCompressed() {
		byte[] b = super.getCompressed();
		if (store == null) {
			recIndex = null;
		}
		return b;
	}


	@Override
	public int remove(int idx) {
		int len = super.remove(idx);
		size -= len ;
		resetIndex(idx);
		
		return len;
	}


	@Override
	public void put(int idx, byte[] rec) {
		LineDtls p = getLinePositionLength(idx, rec.length);
		
		if (idx >= recordCount) {
			size += rec.length + super.recordOverhead;
//			System.out.print(" +" + idx);
		} else if (rec.length != p.len) {
			super.moveData(p.pos + p.len, p.pos + rec.length);
			
			adjRecordIndex(idx, rec.length - p.len);
			
			size += rec.length - p.len;
		} 
//		System.out.println(" put " + this.hashCode() + " " + size + " " 
//				+ rec.length);
		
		put(p, rec);
	}
	
	
	protected void adjRecordIndex(int idx, int diff) {
		
		if (recIndex != null) {
			synchronized (recIndex) {
				int index = idx / INDEX_RECORD;
				for (int i = Math.max(0, index); i < Math.min(recIndex.length, recordCount / INDEX_RECORD); i++) {
					//System.out.print("\t" + i + ": " + recIndex[i]);
					recIndex[i] += diff;
				}
				
				lastLookUpPosition = -11;
//				System.out.print(" @" + idx);
//				if (lastLookUpBlockIndex >= idx) {
//					System.out.print('@' + " " + lastLookUpBlockIndex + " " + idx + " "+ diff + "! ");
//					lastLookUpPosition += diff;
//				}
			}
		}
		
		if (idx < lastLookUpPosition) {
			lastLookUpPosition = -11;
		} 
	}

	
//
//	@Override
//	public void add(byte[] rec) {
//		int p = size+lengthSize;
//		System.arraycopy(rec, 0, store, p, rec.length);
//
//		addLength(p, rec.length);
//		
//		size += rec.length+lengthSize;
//		recordCount += 1;
//	}
	
	@Override
	protected void put(LineDtls p, byte[] rec) { 
//		System.out.println("^^ " + p.index + " -- " + p.pos + " " + p.len + " " + b.length
//				+ " " + store.length + " " + rec.length);
		System.arraycopy(rec, 0, store, p.pos, rec.length);
		addLength(p.pos, rec.length);
	}
	
	private void addLength(int pos, int length) {
		
		
		switch (super.recordOverhead) {
		case 4: store[pos-4] = (byte) ((length >>>  24) & 0xFF);
		case 3: store[pos-3] = (byte) ((length >>>  16) & 0xFF);
		case 2: store[pos-2] = (byte) ((length >>>  8) & 0xFF);
		case 1: store[pos-1] = (byte) ((length >>>  0) & 0xFF);
		}
//		byte[] b = BigInteger.valueOf(length).toByteArray();
//	
//		//System.out.println("^^ " + p.index + " -- " + p.pos + " " + p.len + " " + b.length);
//		store[pos-1] = b[b.length - 1];
//		
//		switch (super.lengthSize) {
//		case 3:
//			store[pos-3] = 0;
//			if (b.length > 2) {
//				store[pos-3] = b[b.length - 3];
//			}	
//		case 2:
//			store[pos-2] = 0;
//			if (b.length > 1) {
//				store[pos-2] = b[b.length - 2];
//			}
//		}	
	}

	

	@Override
	public int getSize() {
		return size;
	}

	@Override
	protected LineDtls getLinePositionLength(int idx, int newLen) {
		
		int lastP;
		int l = super.recordOverhead;
		if (idx >= super.recordCount) {
			lastP = size;
			l = 0;
		} else {
			int p = 0;
			int count = idx - lastLookUpPosition;
			if (lastLookUpPosition>= 0 && count >= 0 && (count < 6) && recIndex != null) {
				p = this.lastLookUpBlockIndex;
//				System.out.print(" ." + idx);
			} else {
				int index = idx / INDEX_RECORD - 1;
				
				count = idx % INDEX_RECORD;
				
				if (index >= 0) {
					if (recIndex == null || index > recIndexValidTo) {
						buildIndex();
					}
				
					p = recIndex[index];
				}
			}
			lastP = p;
			//System.out.print(" . " + count);
			for (int i = 0; i <= count; i++) {
				lastP = p;
				l = getLength(p);
				p += l + super.recordOverhead;
			}
		}
		lastLookUpPosition = idx;
		lastLookUpBlockIndex = lastP;
		
		return new LineDtls(lastP + super.recordOverhead, l, newLen, idx, 0, super.recordOverhead, true);
	}
	
	

	protected void resetIndex(int resetAt) {
		recIndexValidTo = Math.min(recIndexValidTo, resetAt / INDEX_RECORD) - 1;
		if (resetAt < lastLookUpPosition) {
			lastLookUpPosition = -11;
		}
	}


	protected void buildIndex() {
			if (recordCount < INDEX_RECORD) {
				return;
			}
			int i, l;
			int j = 0;
			int k = 0;
			int p = 0;
			lastLookUpBlockIndex = -11;
	
			if (recIndex == null || recIndex.length < this.recordCount / INDEX_RECORD) {
				//System.out.println("Index ");
				recIndex = new int[Math.max(this.getRecordCount(), store.length / maxLen) / INDEX_RECORD + 10];
			}
			
			synchronized (recIndex) {
				for (i = 0; i < this.recordCount; i++) {
					l = getLength(p);
					p += l + this.recordOverhead;
					
					//System.out.println(" --> " + j + "\t" + p + "\t" + l);
					
					j += 1;
					if (j >= INDEX_RECORD) {
						j = 0;
						if (k < recIndex.length) {
							recIndexValidTo = k;
							recIndex[k++] = p;
						}
					}
				}
				
				//System.out.print("$");
				size = p;
	//			System.out.println(" ==> Size Check: " + this.hashCode()
	//					+ " > "+ p + " " + size + " " + super.getRecordCount());
				
	//			System.out.print("Rebuilt Index count=" + recordCount + " " + k + " : ");
	//			for (i = 0; i < k; i++) {
	//				System.out.print(i + ": " + recIndex[i] + "\t");
	//			}
	//			System.out.println();
			}
	
		}

	
	public final int getLength(int p) {
		if (p+super.recordOverhead  > store.length) {
			return 0;
		}
		int r = (store[p]) & 0xFF;
		if (recordOverhead >= 2) {
			r = (r << 8) + ((store[p+1]) & 0xFF);
			if (recordOverhead > 2) {
				r = (r << 8) + ((store[p+2]) & 0xFF);
			}
		}
		return r;
		
//		byte[] b = {0,0,0,0};
//		
//		if (p+super.lengthSize  > store.length) {
//			return 0;
//		}
//
//		b[4-super.lengthSize] = store[p];
//		if (super.lengthSize == 2) {
//			b[3] = store[p+1];
//		}
//		if (super.lengthSize > 2) {
//			b[2] = store[p+1];
//			b[3] = store[p+2];
//		}
//		return (new BigInteger(b)).intValue();
	}

	@Override
	public ZOld_RecordStoreVariableLength[] split(int blockSize) {
		buildIndex();
		ZOld_RecordStoreVariableLength[] ret = new ZOld_RecordStoreVariableLength[ ((size -1)) / (blockSize) + 1];
		int used = 0;
		int last = -1;
		int j = 0;
		for (int i = 0; i < recordCount / INDEX_RECORD; i++) {
			if (recIndex[i] - used > blockSize) {
				ret[j] = allocateStore(recIndex[i] - used, used, (i - last) * INDEX_RECORD, recIndex[i] - used);
//				ret[j] = new RecordStoreVariableLength(recIndex[i] - used, maxLen, (i - last) * INDEX_RECORD);
////				System.out.println(" --> " + i + " pos: "+ recIndex[i]
////				                  + " used: " + used
////				                  + " Records: " + ((i - last) * INDEX_RECORD)
////				                  + " From: " + (used - super.lengthSize)
////				                  + " Length: " + (recIndex[i] - used)
////				                  + " From Size: " + store.length
////				                  + " To Size: " + ret[j].store.length);
//				System.arraycopy(store, used, ret[j].store, 0, recIndex[i] - used);
//				ret[j].size = recIndex[i] - used;
				used = recIndex[i];
			
				last = i;
				j += 1;
			}
		}
		
		lastLookUpBlockIndex = -11;
		
		if (used != size) {
			int l = size - used; 
			ret[j] = allocateStore(l, used, recordCount - (last + 1) * INDEX_RECORD, l);
//			ret[j] = new RecordStoreVariableLength(l, maxLen, recordCount - (last + 1) * INDEX_RECORD);
//			System.arraycopy(store, used, ret[j].store, 0, l);
//			ret[j].size = l;
		}
		return ret;
	}
	
	public ZOld_RecordStoreVariableLength[] splitAt(int recNumber) {
		ZOld_RecordStoreVariableLength[] ret = new ZOld_RecordStoreVariableLength[2];
		LineDtls posLen = getLinePositionLength(recNumber, 0);
		int cpyLen0 = posLen.pos - super.recordOverhead;
		int cpyLen1 = size - cpyLen0;
		int defaultBlkSize = Common.OPTIONS.chunkSize.get();
		
		ret[0] = allocateStore(Math.max(defaultBlkSize, cpyLen0), 0,       recNumber,               cpyLen0);
		ret[1] = allocateStore(Math.max(defaultBlkSize, cpyLen1), cpyLen0, recordCount - recNumber, cpyLen1);
		
		return ret;
	} 

	
	private ZOld_RecordStoreVariableLength allocateStore(int bsize, int start, int recCount, int cpyLen) {

		ZOld_RecordStoreVariableLength ret = new ZOld_RecordStoreVariableLength(bsize, maxLen, recordOverhead, recCount);
		                             // new RecordStoreVariableLength(recIndex[i] - used, maxLen, (i - last) * INDEX_RECORD);

		if (cpyLen >= 0) {
			System.arraycopy(store, start, ret.store, 0, cpyLen);
		}
		ret.size = cpyLen;
		
		return ret;
	}
}
