package net.sf.RecordEditor.utils.lang;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.Icon;
import javax.swing.JFileChooser;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChooserWrapper;
import net.sf.RecordEditor.utils.swing.filechooser.JrFileChooserBldr;
import net.sf.recordEditor.ui.fileFields.params.FileFieldParamBldr;


@SuppressWarnings("serial")
public abstract class OpenSaveAction extends ReAbstractAction {
	private final boolean open;
	private IFileChooserWrapper fileChooser;
	private final Component parent;

	public OpenSaveAction(Component parent, boolean open) {
		this(parent, open, getName(open));
	}


	public OpenSaveAction(Component parent, boolean open, String name) {
		super(name, getIcon(open));

		this.parent = parent;
		this.open = open;
	}

	private static String getName(boolean open) {
		String ret = "Save As";
		if (open) {
			ret = "Open";
		}

		return ret;
	}


	private static Icon getIcon(boolean open) {
		Icon ret = Common.getRecordIcon(Common.ID_SAVE_AS_ICON);
		if (open) {
			ret = Common.getRecordIcon(Common.ID_OPEN_ICON);
		}

		return ret;
	}

	/**
	 * @return the fileChoose
	 */
	private IFileChooserWrapper getFileChooser() {
		if (fileChooser == null) {
			fileChooser = JrFileChooserBldr.newChooser(
					null, FileFieldParamBldr.newFileChooserParamAndBuilder());
		}
		return fileChooser;
	}

	/**
	 * Set the initial file / directory
	 * @param f initial file
	 */
	public void setFileName(File f) {
		if (f != null) {
			if (fileChooser == null) {
				fileChooser = JrFileChooserBldr.newChooser(
						null, 
						FileFieldParamBldr.newFileChooserParamBuilder()
									.initialPath(f)
								.asBuilderAndParameter());
			} else {
				getFileChooser().setSelectedFile(f);
			}
		}
	}

	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		askUser(getDefaultFile());
	}
	
	public void askUser(File defaultFile) {
		setFileName(defaultFile);

		IFileChooserWrapper fc = getFileChooser();
		int ret;

		if (open) {
			ret = fc.showOpenDialog(parent);
		} else {
			ret = fc.showSaveDialog(parent);
		}

		if (ret == JFileChooser.APPROVE_OPTION) {
			processFile(fc.getSelectedFile());
		}
	}
	

	/**
	 * process the file selected by the user
	 * @param selectedFile file selected by the user
	 */
	public abstract File getDefaultFile();

	/**
	 * process the file selected by the user
	 * @param selectedFile file selected by the user
	 */
	public abstract void processFile(File selectedFile);
}
