package net.sf.RecordEditor.utils.params;

import java.io.File;
import java.util.HashMap;

import net.sf.JRecord.Numeric.ICopybookDialects;

public class ProgramOptions { 
	private static final String RE_SCHEMA = Parameters.AP_RECORD_EDITOR + Parameters.OP_TAB_SCHEMA;

	public static enum ProgramType {
		RECORD_EDITOR("RecordEditor"),
		CSV_EDITOR("ReCsvEditor"),
		PROTOBUF_EDITOR("ProtoBufEditor"),
		AVRO_EDITOR("AvroEditor"),
		OTHER_PGM("Other");
		
		public final String programName;
		ProgramType(String programName) {
			this.programName = programName;
		}
	}
	public static final String FILE_SEPERATOR  =  System.getProperty("file.separator");
	
    public static final char COMPRESS_SPACE = 'S';
    public static final char COMPRESS_READ  = 'R';
    public static final char COMPRESS_READ_FAST_CPU = 'F';
    public static final char COMPRESS_NO    = 'N';
    public static final char COMPRESS_YES   = 'Y';

    public static final char LARGE_VB_YES   = 'Y';
    public static final char LARGE_VB_NO    = 'N';
    public static final char LARGE_VB_TEST  = 'T';

    public static final char SIZE_MAXIMISED = 'M';
    public static final char SIZE_LAST      = 'L';
    public static final char SIZE_LAST_FORCED = 'F';
    public static final char SIZE_SPACE_AROUND = 'S';
    public static final char SIZE_NO_RESIZE = 'N';
    public static final char SIZE_SPECIFIED = 'D';
    public static final char SIZE_SPECIFIED_FORCED = 'E';
    public static final char SIZE_SCREEN_1 = '1';
    public static final char SIZE_SCREEN_2 = '2';
    public static final char SIZE_SCREEN_3 = '3';
//    public static final char SIZE_SCREEN_1_MAX = 'Z';

    private static final char[] compressOptions = {COMPRESS_SPACE, COMPRESS_READ, COMPRESS_READ_FAST_CPU, COMPRESS_NO, COMPRESS_YES};
    private static final char[] largeOptions = {LARGE_VB_NO, LARGE_VB_YES, LARGE_VB_TEST};
    private static final char[] startSize = {SIZE_MAXIMISED, SIZE_LAST, SIZE_LAST_FORCED, SIZE_SPACE_AROUND, 
    	SIZE_SPECIFIED, SIZE_SPECIFIED_FORCED, SIZE_NO_RESIZE,
    	SIZE_SCREEN_1, SIZE_SCREEN_2, SIZE_SCREEN_3};
    
    private final HashMap<String, IUpdatableBooleanOption> stdBooleanOptions = new HashMap<String, IUpdatableBooleanOption>();

    public ApplicationDetails applicationDetails = new ApplicationDetails(
    		"Recordeditor", "https://sourceforge.net/projects/record-editor/");
    public ProgramType programType = ProgramType.OTHER_PGM;
	public final BoolOpt searchAllFields = new BoolOpt(Parameters.SEARCH_ALL_FIELDS);
	public final MultiValOpt screenStartSizeOpt  = new MultiValOpt(Parameters.SCREEN_SIZE_OPTION, startSize, SIZE_MAXIMISED);

	public final BoolOpt usePrefered = new BoolOpt(Parameters.PREFERED_AS_DEFAULT);
	public final BoolOpt warnBinaryFieldsAndStructureDefault = new BoolOpt(Parameters.WARN_BINARY_FIELDS_DEFAULT);
	public final BoolOpt runFieldSearchAutomatically = new BoolOpt(Parameters.FS_RUN_AUTOMATIC);
	public final BoolOpt searchForMainframeZoned = new BoolOpt(Parameters.FS_MAINFRAME_ZONED);
	public final BoolOpt searchForPcZoned = new BoolOpt(Parameters.FS_PC_ZONED);
	public final BoolOpt searchForComp3 = new BoolOpt(Parameters.FS_COMP3);
	public final BoolOpt searchForCompBigEndian = new BoolOpt(Parameters.FS_COMP_BIG_ENDIAN);
	public final BoolOpt useNewTreeExpansion = new BoolOpt(Parameters.USE_NEW_TREE_EXPANSION);
	public final BoolOpt searchForCompLittleEndian = new BoolOpt(Parameters.FS_COMP_LITTLE_ENDIAN) {

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.utils.common.ProgramOptions.BoolOpt#isSelected()
		 */
		@Override
		public boolean isSelected() {
			return (! searchForCompBigEndian.isSelected())
				&& super.isSelected();
		}

	};

	public final BoolOpt openPoFuzzyWindow = new BoolOpt(Parameters.PO_EDIT_OPEN_FUZZY_VIEW, true);
	public final StringOpt poChildScreenPosition = new StringOpt(Parameters.PO_EDIT_CHILD_SCREEN_POS, Parameters.CHILD_SCREEN_RIGHT);

	public final BoolOpt showRecordEditorTips = new BoolOpt(Parameters.SHOW_RECORDEDITOR_TIPS, true);
	public final BoolOpt logToFront = new BoolOpt(Parameters.BRING_LOG_TO_FRONT);

	public final BoolOpt loadInBackgroundThread = new BoolOpt(Parameters.PROPERTY_LOAD_FILE_BACKGROUND);
	public final BoolOpt asterixInFileName  = new BoolOpt(Parameters.ASTERIX_IN_FILE_NAME);
	public final BoolOpt useBigFixedModel   = new BoolOpt(Parameters.PROPERTY_BIG_FILE_USE_SPECIAL_FIXED_MODEL);
	public final BoolOpt showAllExportPnls  = new BoolOpt(Parameters.SHOW_ALL_EXPORT_OPTIONS);
	public final BoolOpt deleteSelectedWithDelKey = new BoolOpt(Parameters.DEL_SELECTED_WITH_DEL_KEY);
	public final BoolOpt warnWhenUsingDelKey= new BoolOpt(Parameters.WARN_WHEN_USING_DEL_KEY);

	public final BoolOpt csvSearchFixed     = new BoolOpt(Parameters.CSV_LOOK_4_FIXED_WIDTH);
	public final BoolOpt useFileWizard      = new BoolOpt(Parameters.USE_FILE_WIZARD);
	public final BoolOpt useSeperateScreens = new BoolOpt(Parameters.SEPERATE_WINDOWS);
	public final BoolOpt typeOnRecordScreen = new BoolOpt(Parameters.INCLUDE_TYPE_NAME, true);
	public final BoolOpt allowTextEditting  = new BoolOpt(Parameters.EDIT_RAW_TEXT, true);
	public final BoolOpt renameOldFile      = new BoolOpt(Parameters.RENAME_OLD_FILE, true);
//	public final BoolOpt showCsvFChooserOptions = new BoolOpt(
//														Parameters.CSV_SHOW_FILECHOOSER_OPTIONS,
//														Parameters.isWindowsLAF());
	public final BoolOpt addFileSearchBtn   = new BoolOpt(Parameters.ADD_FILE_SEARCH_BTN, false);
	public final IUpdatableBooleanOption sortFieldsOnPos    = new UpdateableBoolOpt(Parameters.SORT_FIELDS_POSITION, true);

	public final IUpdatableBooleanOption highlightEmpty = new UpdateableBoolOpt(Parameters.PROPERTY_HIGHLIGHT_EMPTY);
	public final BooleanOption highlightEmptyActive = new BooleanOption(false);

	public final BooleanOption loadPoScreens       = new BooleanOption(false);
	public final BooleanOption xsltAvailable       = new BooleanOption(true);
	public final BooleanOption fileWizardAvailable = new BooleanOption(false);
	public final BooleanOption getTextPoPresent    = new BooleanOption(false);
	public final BooleanOption addTextDisplay      = new BooleanOption(false);
	public final BooleanOption standardEditor      = new BooleanOption(false);
	public final BooleanOption useRowColSelection  = new BooleanOption(true);
	
	public final BooleanOption agressiveCompress   = new BooleanOption(true);
	public final BooleanOption doCompress          = new BooleanOption(true);
	public final BooleanOption overWriteOutputFile = new BooleanOption(false);

	public final IntOpt significantCharInFiles1 = new IntOpt("SignificantCharInFiles.1", 6, 1);
	public final IntOpt significantCharInFiles2 = new IntOpt("SignificantCharInFiles.2", 12, 1);
	public final IntOpt significantCharInFiles3 = new IntOpt("SignificantCharInFiles.3", 18, 1);
	public final IntOpt launchIfMatch = new IntOpt("LauchEditorIfMatch", 8, 1);

	public final IntOpt jrecordVersion = new IntOpt("jrecordVersion", 0, 1);

	public final DoubleIntOpt cobolDialect = new DoubleIntOpt(
					Parameters.DEFAULT_COBOL_DIALECT, Parameters.LAST_COBOL_DIALECT, ICopybookDialects.FMT_MAINFRAME);

	public final IntOpt chunkSize = new IntOpt(Parameters.PROPERTY_BIG_FILE_CHUNK_SIZE, 1048576, 1024);
	public final IntOpt filterLimit = new IntOpt(Parameters.PROPERTY_BIG_FILE_FILTER_LIMIT, 10000000, 1000);
	public final IntOpt bigFilePercent = new IntOpt(Parameters.PROPERTY_BIG_FILE_PERCENT, 14, 1);


	public final MultiValOpt compressOption = new MultiValOpt(Parameters.PROPERTY_BIG_FILE_COMPRESS_OPT, compressOptions, COMPRESS_READ);
	public final MultiValOpt largeVbOption = new MultiValOpt(Parameters.PROPERTY_BIG_FILE_LARGE_VB, largeOptions, LARGE_VB_YES);

	public final StringOpt cobolOptionFile = new StringOpt(
			Parameters.COBOL_OPT_FILE,
			Parameters.getPropertiesDirectoryWithFinalSlash() + "User/CobolOpts/CobolOptions.bin");

	public final StringOpt DEFAULT_IO_NAME = new StringOpt(Parameters.DEFAULT_IO, "");
	public final StringOpt DEFAULT_BIN_NAME = new StringOpt(Parameters.DEFAULT_COBOL_DIALECT_NAME, "");
	public final StringOpt COPYBOOK_READER = new StringOpt(Parameters.DEFAULT_COPYBOOK_READER, "");

	public final StringOpt XSLT_ENGINE = new StringOpt(Parameters.XSLT_ENGINE, "");

	public final FileNameOptWithDefault DEFAULT_CODEGEN_EXPORT_DIRECTORY
						= new FileNameOptWithDefault(Parameters.CODEGEN_DIRECTORY, Parameters.CODEGEN_DIRECTORY_DFLT); 
	public final StringOpt DEFAULT_CODEGEN_DIRECTORY_EXTENSION 
					= new StringOpt(Parameters.CODEGEN_DIRECTORY_EXTENSION, Parameters.CODEGEN_DIRECTORY_EXTENSION_DFLT);

	public final FileNameOpt DEFAULT_USER_TEMPLATE_DIRECTORY = new FileNameOpt(Parameters.TEMPLATE_DIRECTORY);

	public final FileNameOpt DEFAULT_FILE_DIRECTORY = new FileNameOpt("DefaultFileDirectory");
	public final FileNameOpt DEFAULT_COBOL_DIRECTORY = new FileNameOpt(Parameters.DEFAULT_COBOL_DIRECTORY);//"DefaultCobolDirectory");
	public final FileNameOpt DEFAULT_COPYBOOK_DIRECTORY = new FileNameOpt(Parameters.COPYBOOK_DIRECTORY);
	public final FileNameOptWithDefault DEFAULT_VELOCITY_DIRECTORY = new FileNameOptWithDefault(Parameters.VELOCITY_TEMPLATE_DIRECTORY, Parameters.DEFAULT_VELOCITY_TEMPLATE_DIRECTORY);
	public final FileNameOptWithDefault copybookVelocityDirectory  = new FileNameOptWithDefault(Parameters.VELOCITY_COPYBOOK_DIRECTORY, Parameters.DEFAULT_VELOCITY_COPYBOOK_DIRECTORY);
	public final FileNameOptWithDefault velocityScriptDir = new FileNameOptWithDefault(
			Parameters.VELOCITY_SCRIPT_DIR,
			Parameters.getPropertiesDirectoryWithFinalSlash() + "User/VelocityTemplates/Script/");
	public final FileNameOptWithDefault layoutExportDirectory = new FileNameOptWithDefault(Parameters.LAYOUT_EXPORT_DIRECTORY, Parameters.LAYOUT_EXPORT_DIRECTORY_DFLT);
	public final FileNameOptWithDefault DEFAULT_XSLT_DIRECTORY = new FileNameOptWithDefault(Parameters.XSLT_TEMPLATE_DIRECTORY, "<reproperties>/User/Xslt/");
	public final FileNameOptWithDefault DEFAULT_SCRIPT_EXPORT_DIRECTORY = new FileNameOptWithDefault(Parameters.EXPORT_SCRIPT_DIRECTORY, "<reproperties>/User/ExportScripts/");
	public final FileNameOptWithDefault DEFAULT_SCRIPT_DIRECTORY = new FileNameOptWithDefault(Parameters.SCRIPT_DIRECTORY, "<reproperties>/User/Scripts/");
	public final FileNameOpt XSLT_JAR1 = new FileNameOpt(Parameters.XSLT_JAR1);
	public final FileNameOpt XSLT_JAR2 = new FileNameOpt(Parameters.XSLT_JAR2);
	
	public final RelativeFileNameOpt schemaBuDir = new RelativeFileNameOpt(Parameters.COPYBOOK_DIRECTORY, "SchemaBu");
	
	public BoolOpt useLastDir = new BoolOpt(Parameters.OPEN_IN_LAST_DIRECTORY, true);
	
	
	public IUpdatableBooleanOption showBinaryTextFileWarning 
		= new UpdateableBoolOpt("binaryTextFileWarn", true);
	public IUpdatableBooleanOption showEbcdicBinaryTextFileWarning 
		= new UpdateableBoolOpt("EbcdicBinaryTextFileWarn", true);
	
	public IUpdatableBooleanOption showMainframeFileWarning 
		= new UpdateableBoolOpt("mainframeFileWarn", true);

	public IUpdatableBooleanOption showBinaryTextFileLoadWarning 
		= new UpdateableBoolOpt("binaryTextFileLoadWarn", true);
	public IUpdatableBooleanOption showEbcdicBinaryTextFileLoadWarning 
		= new UpdateableBoolOpt("EbcdicBinaryTextFileLoadWarn", true);
	
	public IUpdatableBooleanOption showMainframeFileLoadWarning 
		= new UpdateableBoolOpt("mainframeFileLoadWarn", true);

	public IUpdatableBooleanOption AUTO_COMPLETE 
				= new UpdateableBoolOpt(Parameters.FILE_FIELDS_AUTO_COMPLETE, true);

	public final IUpdatableBooleanOption OPEN_IN_EDIT_MODE 
				= new UpdateableBoolOpt(Parameters.FILE_FIELDS_START_IN_EDIT_MODE);


	public IUpdatableBooleanOption lookForDatesInLoad 
				= new UpdateableBoolOpt("lookForDatesLoad", true);

	public IUpdatableBooleanOption lookForDatesInCodeGen 
				= new UpdateableBoolOpt("lookForDatesCodeGen", true);




	public final IUpdatableBooleanOption getBooleanOption(String key) {
		IUpdatableBooleanOption ret = this.stdBooleanOptions.get(key);
		if (ret == null) {
			synchronized (this.stdBooleanOptions) {
				if ((ret = this.stdBooleanOptions.get(key)) == null) {
					boolean def = RE_SCHEMA.equals(key)
							|| key.endsWith(Parameters.OP_TAB_FIXED)
							|| key.endsWith(Parameters.OP_TAB_XML)
							;

					ret = new UpdateableBoolOpt(key, def);
					this.stdBooleanOptions.put(key, ret);
				}
			}
		}
		
		return ret;
	}


	public static final class DoubleIntOpt {
		private final String param1, param2;
		private final int defValue;
		
		public DoubleIntOpt(String value1, String value2, int defaultValue) {
			param1 = value1;
			param2 = value2;
			defValue = defaultValue;
		}
		public final int get() {
			int ret = -121;

			ret = getParam(param1, ret);
			if (ret < 0) {
				ret = defValue;
				ret = getParam(param2, ret);
			}
			
			return ret;
		}
		
		private int getParam(String param, int ret) {
			try {
				ret = Parameters.getParamInt(param, -121, 1);
			} catch (Exception e) { }
			
			return ret;
		}
		
//		public void updateParameter1(int val) {
//			Parameters.setProperty(param1, Integer.toString(val));
//		}
//		public void updateParameter2(int val) {
//			Parameters.setProperty(param2, Integer.toString(val));
//		}
	}

	public static final class IntOpt {
		private final String param;
		private final int defValue;
		private final int mult;

		public IntOpt(String value, int defaultValue, int multBy) {
			param = value;
			defValue = defaultValue;
			mult = multBy;
		}

		public final int get() {
			int ret = defValue;

			try {
				ret = Parameters.getParamInt(param, defValue, mult);
			} catch (Exception e) { }
			return ret;
		}
		
		public void updateParameter(int val) {
			Parameters.setProperty(param, val);
		}
	}

	public static class StringOpt {
		protected final String param;
		private String defValue;


		public StringOpt(String value, String defaultValue) {
			param = value;
			defValue = defaultValue;
		}

		public final String get() {
			String ret = defValue;

			String s = Parameters.getString(param);
			if (s != null) {
				ret = s;
			}

			return ret;
		}

		public void set(String newVal) {
			Parameters.setProperty(param, newVal);
		}
	}


	public static class BasicFileNameOpt {
		protected String param;

		public BasicFileNameOpt(String value) {
			param = value;
		}

		public String get() {
			return  Parameters.getFileName(param);
		}
		public String getNoStar() {
			return  Parameters.dropStar(get());
		}
		public String getSlashNoStar() {
			String s=  Parameters.dropStar(get());
			if (s.endsWith("/") || s.endsWith("\\")) {
				
			} else {
				s = s + FILE_SEPERATOR;
			}
			return s;
		}

		public String getWithStar() {
			String s = get();
			if (s == null) {
				s = "";
			} else {
				if (s.endsWith("/") || s.endsWith("\\")) {
					s = s + "*";
				} else if (! s.endsWith("*")) {
					s = s + FILE_SEPERATOR + "*";
				}
			}
			return s;
		}
	}
	

	public static class FileNameOpt extends BasicFileNameOpt {

		public FileNameOpt(String value) {
			super(value);
		}


		public void set(String newVal) {
			Parameters.setProperty(param, newVal);
		}
	}
	
	public static class RelativeFileNameOpt extends BasicFileNameOpt {
		final String fileName;
		private RelativeFileNameOpt(String varName, String fileName) {
			super(varName);
			
			this.fileName = fileName;
		}
		
		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.utils.params.ProgramOptions.FileNameOpt#get()
		 */
		@Override
		public String get() {
			String filename = Parameters.dropStar(super.get());
			String s = "G:\\Users\\BruceTst01\\RecordEditor_HSQL\\CopyBook\\";;
			if (filename != null) {
				File file = new File(filename);
				File parentFile;
				if (file != null && (parentFile = file.getParentFile()) != null) {
					s = parentFile.toString();
				}
			}
			
			return s + File.separatorChar + fileName + File.separatorChar;
		}
	}

	public static final class FileNameOptWithDefault extends BasicFileNameOpt{
	//	private String param;
		public final String defaultValue;
		
		public FileNameOptWithDefault(String value, String defaultValue) {
			super(value); // = value;

			this.defaultValue = defaultValue;
		}

		public String get() {
			String s = Parameters.getFileName(param);

			if (s == null || "".equals(s)) {
				s = Parameters.expandVars(defaultValue);
//				try {
//					Parameters.setSavePropertyChanges(false);
//					Parameters.setProperty(param, defaultValue);
//				} finally {
//					Parameters.setSavePropertyChanges(true);
//				}
//			} else {
				
			}
			return s;
		}
		
	
		
		public String getSlashNoStar() {
			String s = get();
			
			if (s != null) {
				if (s.endsWith("*")) {
					s = s.substring(0, s.length() - 1);
				}
				if ( ! (s.endsWith("/") || s.endsWith("\\"))) {
					s = s + FILE_SEPERATOR;
				}
			}
			return s;
		}

	}

	public static final class MultiValOpt {
		private String param;
		private char[] options;
		private char defaultVal;


		public MultiValOpt(String param, char[] options, char defaultVal) {
			super();
			this.param = param;
			this.options = options;
			this.defaultVal = defaultVal;
		}

		public char get() {
			char ret = defaultVal;

			String s = Parameters.getString(param);

	    	if (s == null || "".equals(s)) {

	    	} else {
	    		s = s.toUpperCase();
	    		for (int i =0; i < options.length; i++) {
	    			if (s.charAt(0) == options[i]) {
	    				ret = options[i];
	    				break;
	    			}
	    		}
	    	}

	    	return ret;
		}
	}
	
	public static class ApplicationDetails {
		final String applicationId, website;

		public ApplicationDetails(String applicationId, String website) {
			super();
			this.applicationId = applicationId;
			this.website = website;
		}

		/**
		 * @return the applicationId
		 */
		public final String getApplicationId() {
			return applicationId;
		}

		/**
		 * @return the website
		 */
		public final String getWebsite() {
			return website;
		}
		
		
	}
}
