package net.sf.RecordEditor.utils.swing;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.event.KeyEvent;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;

import net.sf.RecordEditor.utils.basicStuff.BasicTable;
import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.LangConversion;



public class SwingUtilsBldr {


   /**
     * Get Help button
     *
     * @return HelpButton;
     */
    public static final JButton getHelpButton() {
        return SwingUtilsBldr.newButton("Help", Common.getRecordIcon(Common.ID_HELP_ICON));
    }

	public static JMenu newMenu(String s) {
		return new JMenu(LangConversion.convert(LangConversion.ST_MENU, s));
	}

	public static JMenuItem newMenuItem(String s) {
		return new JMenuItem(LangConversion.convert(LangConversion.ST_MENU, s));
	}

	public static JButton newButton(String s) {
		return new JButton(LangConversion.convert(LangConversion.ST_BUTTON, s));
	}
	
	public static JRadioButton newRadioButton(String s) {
		return new JRadioButton(LangConversion.convert(LangConversion.ST_BUTTON, s));
	}

	public static JCheckBox newCheckBox(String s) {
		return new JCheckBox(LangConversion.convert(LangConversion.ST_BUTTON, s));
	}

	public static JFlipBtn newFlipButton(String text, String altText) {
		return new JFlipBtn(LangConversion.convert(LangConversion.ST_BUTTON, text),
							LangConversion.convert(LangConversion.ST_BUTTON, altText));
	}


	public static JButton newButton(String s, Icon icon) {
		return new JButton(LangConversion.convert(LangConversion.ST_BUTTON, s), icon);
	}

	public static void addTab(JTabbedPane tab, String tabId, String name, JComponent tabComponenet) {
		tab.addTab(LangConversion.convertDesc(LangConversion.ST_TAB, name, "Tab: " + tabId), tabComponenet);
	}


	public static void insertTab(JTabbedPane tab, String tabId, String name, JComponent tabComponenet, String tip, int index) {
		tab.insertTab(
				LangConversion.convertDesc(LangConversion.ST_TAB, name, "Tab: " + tabId),
				null, tabComponenet, tip, index);
	}

//	public static JPanel getPanelWith(JComponent c) {
//		JPanel p = new JPanel();
//		p.add(c);
//		return p;
//	}

	public static void clickOpenBtn(JFileChooser fileChooser, boolean doEnter) {
		String s = System.getProperty("java.version").substring(0, 4);
		try {
			if (s.startsWith("1.5.") || s.startsWith("1.6.")) {
				if (doEnter) {
					fileChooser.getActionForKeyStroke(
							KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0))
								.actionPerformed(null);
				}
			} else {
				JButton btn = fileChooser.getUI().getDefaultButton(fileChooser);
				if (btn != null) {
					btn.doClick();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void copySelectedCells(JTable dtlTbl) {
		int startRow = dtlTbl.getSelectedRow();
		int startCol = dtlTbl.getSelectedColumn();
		int endRow = startRow + dtlTbl.getSelectedRowCount();
		int endCol = startCol + dtlTbl.getSelectedColumnCount();
		StringBuilder b = new StringBuilder();
		String fldSep = ""; 
		String lineSep = "";
		Object val;
		
		if (endRow >= startRow && endCol >= startCol) {
			for (int i = startRow; i < endRow; i++) {
				b.append(lineSep);
				lineSep = "\n";
				for (int j = startCol; j < endCol; j++) {
					b.append(fldSep);
					fldSep = "\t";
					val = dtlTbl.getValueAt(i, j);
					if (val != null) {
						b.append(val.toString());
					}
				}
			}
			Clipboard system = Toolkit.getDefaultToolkit().getSystemClipboard();
			system.setContents(new StringSelection(b.toString()), null);
		}
	}

	
	public static void pasteOver(Object requestor, JTable tblDetails, int startRow, int startCol, int numRows, int numCols) {
		if (startRow >= 0) {
			Clipboard system = Toolkit.getDefaultToolkit().getSystemClipboard();
			try {
				pasteTable(tblDetails, 
						startRow, startCol,
						numRows, numCols,
						(String) (system.getContents(requestor)
								.getTransferData(DataFlavor.stringFlavor)));

			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
//	public static void showInBrowser(URI uri) {
//		if (java.awt.Desktop.isDesktopSupported()) {
//			try {
//				java.awt.Desktop desktop = java.awt.Desktop.getDesktop();
//
//                desktop.browse(uri);
//            } catch ( Exception ex ) {
//                System.err.println( "Error showing Web page" + ex.getMessage() );
//            }
//		}
//	}


	public static final void pasteTable(JTable tblDetails, int startRow, int startCol, int rowCount, int colCount, String trstring) {
		String val;
		BasicTable tbl = new BasicTable(trstring);

		rowCount = Math.min(rowCount, tblDetails.getRowCount() - startRow);
		colCount = Math.min(colCount, tblDetails.getColumnCount() - startCol);

		Common.stopCellEditing(tblDetails);

		for (int i = 0; tbl.hasMoreRows() && i < rowCount ; i++) {
			tbl.nextRow();
			for (int j = 0; tbl.hasMoreColumns() && j < colCount ; j++) {
				val = tbl.nextColumn();
				if (tblDetails.isCellEditable(startRow + i, startCol + j)) {	
					tblDetails.setValueAt(val, startRow + i, startCol + j);
				}
			}
		}
	}

}
