/**
 * 
 */
package net.sf.RecordEditor.utils.swing.favourite;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.lang.LangConversion;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.params.ProgramOptions;
import net.sf.recordEditor.ui.fileFields.params.FileFieldParamBldr;
import net.sf.recordEditor.ui.fileFields.params.IRecentAndFavouriteFiles;
import net.sf.recordEditor.ui.listManagers.BasicListItem;
import net.sf.recordEditor.ui.listManagers.IListChangeListner;
import net.sf.recordEditor.ui.listManagers.IListItem;
import net.sf.recordEditor.ui.listManagers.IRecentList;
import net.sf.recordEditor.ui.listManagers.IUpdateableList;

/**
 * @author bruce
 *
 */
public class UpdatableParameterList implements IUpdateableList, IRecentList {

	public static final String DIRECTORIES = "Directories";
	public static final String FILES = "Files";
	
	public static final UpdatableParameterList RECORDEDITOR_FAVOURITES = newFileList(Parameters.FILE_FAVOURITES, 25, false);
	public static final UpdatableParameterList RECORDEDITOR_FAVOURITE_DIRECTORIES
			= newDirectoryList(Parameters.FILE_FAVOURITES + Parameters.DIRECTORY_EXTENSION, 25, false);
	public static final UpdatableParameterList CSVEDITOR_FAVOURITES
			= newFileList(Parameters.CSV_EDIT_FAV + "." + UpdatableParameterList.FILES, 25, false);
	public static final UpdatableParameterList CSVEDITOR_FAVOURITE_DIRECTORIES
			= newDirectoryList(Parameters.CSV_EDIT_FAV + Parameters.DIRECTORY_EXTENSION, 25, false);

	public static final UpdatableParameterList FAVOURITE_FILES = Common.OPTIONS.programType == ProgramOptions.ProgramType.CSV_EDITOR
							? CSVEDITOR_FAVOURITES : RECORDEDITOR_FAVOURITES;
	public static final UpdatableParameterList FAVOURITE_DIRECTORIES = Common.OPTIONS.programType == ProgramOptions.ProgramType.CSV_EDITOR
							? CSVEDITOR_FAVOURITE_DIRECTORIES : RECORDEDITOR_FAVOURITE_DIRECTORIES;
	
	public static UpdatableParameterList newFileList(String paramKey, int numEntries, boolean canBeNew) {
		return new UpdatableParameterList(FILES, 
				paramKey, numEntries, canBeNew, false);
	}
	
	public static UpdatableParameterList newDirectoryList(String paramKey, int numEntries, boolean canBeNew) {
		return new UpdatableParameterList(DIRECTORIES, 
						paramKey, numEntries, canBeNew, true);
	}
	
	public static IRecentAndFavouriteFiles newRecentFavourites(boolean isDirectory, String paramKey, int numEntries, boolean canBeNew) {
		if (isDirectory) {
			return FileFieldParamBldr.newFileFieldParamBuilder()
					.recentFiles(UpdatableParameterList.newDirectoryList( paramKey, numEntries, canBeNew))
					.favouriteFiles(
							UpdatableParameterList.newDirectoryList(paramKey + ".Fav" , numEntries, canBeNew))
					.directory(true)
				.asBuilderAndParameter();
		}
		return FileFieldParamBldr.newFileFieldParamBuilder()
					.recentFiles(UpdatableParameterList.newFileList(paramKey, numEntries, canBeNew))
					.recentDirectory(
							UpdatableParameterList.newDirectoryList(paramKey + '.' + UpdatableParameterList.DIRECTORIES,
							numEntries, false))
					.favouriteFiles(
							UpdatableParameterList.newFileList(paramKey + ".Fav" , numEntries, canBeNew))
					.favouriteDirectory(
							UpdatableParameterList.newDirectoryList(
									paramKey + ".Fav." + UpdatableParameterList.DIRECTORIES, 
									numEntries, false))
				.asBuilderAndParameter();
	}
	


	private final List<IListChangeListner> changeListners = new ArrayList<IListChangeListner>(5);
	private       List<IListItem> itemList;
	private final String listName;
	private final String paramKey;
	private final int numEntries;
	private final boolean canBeNew, directory;
	
	public UpdatableParameterList(String listName, String paramKey, int numEntries, boolean canBeNew, boolean directory) {
		this.listName = LangConversion.convert(listName);
		this.paramKey = paramKey;
		this.numEntries = numEntries;
		this.canBeNew = canBeNew;
		this.directory = directory;
		
		this.itemList = new ArrayList<IListItem>(numEntries);
		loadList(itemList);
	}
	
	private void loadList(List<IListItem> itmList) {
		List<String> files = Parameters.getStringList(paramKey, numEntries);

		HashSet<String> used = new HashSet<String>(files.size() * 3 / 2);

		File f;
		for (String filename : files) {
			if (! used.contains(filename) && ! "".equals(filename.trim())
			&& ((f = new File(filename)).exists() || canBeNew)
			&& (f.isDirectory() == directory)) {
				itemList.add(BasicListItem.basicFileListItem(f, filename));
				used.add(filename);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see com.alee.extended.bm.listManagers.IBasicList#getItemList()
	 */
	@Override
	public List<IListItem> getItemList() {
		return itemList;
	}

	/* (non-Javadoc)
	 * @see com.alee.extended.bm.listManagers.IUpdateableFileList#updateList(java.util.List)
	 */
	@Override
	public void setItemList(List<IListItem> items) {
		itemList = items;
		notifyListners();
	}
	/* (non-Javadoc)
	 * @see com.alee.extended.bm.listManagers.IBasicList#addChangeListner(com.alee.extended.bm.listManagers.IListChangeListner)
	 */
	@Override
	public void addChangeListner(IListChangeListner listner) {
		changeListners.add(listner);
	}

	/* (non-Javadoc)
	 * @see com.alee.extended.bm.listManagers.IBasicList#getListName()
	 */
	@Override
	public String getListName() {
		return listName;
	}

	/* (non-Javadoc)
	 * @see com.alee.extended.bm.listManagers.IRecentList#moveToTop(com.alee.extended.bm.listManagers.IListItem)
	 */
	@Override
	public void moveToTop(IListItem listItem) {
		
		synchronized (itemList) {
			if ( itemList.size() == 0 || itemList.get(0) != listItem) {
				itemList.remove(listItem);
				itemList.add(0, listItem);
				notifyListners();
			}
		}
	}

	/* (non-Javadoc)
	 * @see com.alee.extended.bm.listManagers.IUpdateableFileList#addItem(java.lang.String)
	 */
	@Override
	public void addFileItem(String itemText) {
		
		synchronized (itemList) {
			for (int i = itemList.size() - 1; i >= 0; i--) {
				if (itemList.get(i).getItemText().equals(itemText)) {
					itemList.remove(i);
					break;
				}
			}
			itemList.add(0, BasicListItem.basicFileListItem(itemText));
			notifyListners();
		}
	}


	/**
	 * @param i
	 */
	public void removeItem(int i) {
		synchronized (itemList) {
			itemList.remove(i);
			notifyListners();
		}
	}

	private void notifyListners() {
		saveList();
		for (IListChangeListner listner : changeListners) {
			listner.listChanged(this);
		}
	}
	
	private void saveList() {
		int num = Math.min(numEntries, itemList.size());
		
		for (int i = 0; i < num; i++) {
			Parameters.setArrayItem(paramKey, i, itemList.get(i).getItemText());
		}
		
		Parameters.writeListProperties();
	}
	
	/* (non-Javadoc)
	 * @see com.alee.extended.bm.listManagers.IUpdateableFileList#allowUpdates()
	 */
	@Override
	public boolean allowUpdates() {
		return true;
	}
	

}
