package net.sf.RecordEditor.utils.swing.fieldSelection;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.List;

import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.PlainView;
import javax.swing.text.Segment;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.Utilities;
import javax.swing.text.View;
import javax.swing.text.ViewFactory;

import net.sf.RecordEditor.utils.fileAnalysis.IBasicColumn;
import net.sf.recordEditor.ui.xEnvironment.SwingUtils;

public class FieldSelectionTxtFld {
	private static final Font MONOSPACED_FONT = new Font("Monospaced", Font.PLAIN,  SwingUtils.values().STANDARD_FONT_HEIGHT * 5 / 4);

	public final JTextPane fieldSelectionTxt = new JTextPane();

	public final Segment viewSegment = new Segment(new char[1], 0, 0);
	
	private final IColumInterface columnInterface;

	private MouseAdapter mouseAction = new MouseAdapter() {
		
		@Override
		//public void mouseReleased(MouseEvent e) {
		public void mouseClicked(MouseEvent e) { 
			doMouseClicked(e);
		}
	};
	
	
	public FieldSelectionTxtFld(IColumInterface columnInterface) {
		
		this.columnInterface = columnInterface;
		DefaultCaret dc = new DefaultCaret();
		dc.setSize(0, 0);

		fieldSelectionTxt.setFont(MONOSPACED_FONT);
		fieldSelectionTxt.setEditorKit(new FieldDisplayEditorKit());
		
		fieldSelectionTxt.setHighlighter(null);
		fieldSelectionTxt.setEditable(false);
		fieldSelectionTxt.setCaret(dc);
		
		fieldSelectionTxt.addMouseListener(mouseAction);
	}

	public void doMouseClicked(MouseEvent e) {
		if (e.getButton() != MouseEvent.BUTTON1) { return; }
		int cpos = fieldSelectionTxt.getCaretPosition();
		Element paragraphElement = fieldSelectionTxt.getStyledDocument().getParagraphElement(cpos);
		if (paragraphElement != null ) {					
			int lineStartOffset = paragraphElement.getStartOffset();
			int col = cpos - lineStartOffset;
			
			if (col != 0) {
				columnInterface.columnSelected(col);
				fieldSelectionTxt.repaint();
				fieldSelectionTxt.setCaretPosition(cpos);
			}
		}
	}

	public void initiliseText(String v, int maxLineLength) {

		fieldSelectionTxt.removeMouseListener(mouseAction);
		
		setupViewSegment(v, maxLineLength);
		
		setDocText();
		
		fieldSelectionTxt.addMouseListener(mouseAction);
	}


	public void setupViewSegment(String v, int maxLineLength) {

		int len = v.length() + maxLineLength * 2 + 2;
		viewSegment.array = new char[len];
		viewSegment.count = len;
		
		StringBuilder b = newRulerSB(maxLineLength);
		for (int i = 0; i < maxLineLength; i++) {
			viewSegment.array[maxLineLength + i + 1] = b.charAt(i);
		}
		
		System.arraycopy(v.toCharArray(), 0, viewSegment.array, maxLineLength*2 + 2, v.length());
		Arrays.fill(viewSegment.array, 0, maxLineLength, ' ');
		viewSegment.array[maxLineLength] = '\n';
		viewSegment.array[maxLineLength*2+1] = '\n';
	}


	public void setDocText() {
		fieldSelectionTxt.setText(viewSegment.toString());
		fieldSelectionTxt.setCaretPosition(0);
		fieldSelectionTxt.repaint();
	}

	
	private static StringBuilder newRulerSB(int length) {
		String rule = "----+---- ";
		
		StringBuilder b = new StringBuilder(length);
		int i = 1;
		String iStr;
		int ruleLen = rule.length() - 1;
		while (b.length() + ruleLen < length) {
			iStr = Integer.toString(i++);
			b.append(rule.substring(0, ruleLen - iStr.length() + 1))
			 .append(iStr);
		}

		if (b.length() < length) {
			b.append(rule.substring(0, length - b.length()));
		}
		return b;
	}

	@SuppressWarnings("serial")
	private class FieldDisplayEditorKit extends StyledEditorKit {
		
		public static final String NAME = "color";

		private ViewFactory viewFactory = new ViewFactory() {
			@Override public View create(Element elem) {
				return new FieldDisplayDocView(elem); 
			}
		};
		
	    @Override
		public ViewFactory getViewFactory() {
		    return viewFactory;
		}

		@Override
		public String getContentType() {
		    return NAME;
		}

	}
	
	
	private static final Color HEADER_BACKGROUND = new Color(255, 255, 230); //Color.YELLOW;
	private static final Color HEADER_HIGHLIGHTED_BACKGROUND = new Color(235, 235, 255); //Color.CYAN;
	/**
	 * Highlight the Text for the various fields
	 * @author Bruce Martin
	 *
	 */
	private final class FieldDisplayDocView extends PlainView {


		public FieldDisplayDocView(Element elem) {
			super(elem);
		}
		
		@Override
		protected int drawSelectedText(Graphics g, int x, int y, int p0, int p1) throws BadLocationException {
			return drawUnselectedText(g, x, y, p0, p1);
		}

		protected int drawUnselectedText(Graphics g, int x, int y, int p0, int p1)
				throws BadLocationException {

			try {
				Document doc = super.getDocument();
				Element element = super.getElement();
				int elIdx = element.getElementIndex(p0);
				if (elIdx < 0 || elIdx >= element.getElementCount() || elIdx == 1) {
					return super.drawSelectedText(g, x, y, p0, p1);
				}
				int startOffset = element.getElement(elIdx).getStartOffset();
				int pos = p0 - startOffset; 
				
				Segment segment = getLineBuffer();     
				   
				int t1 = p0;
				doc.getText(p0 - pos, p1 - p0 + pos, segment);       
				  

				int count = segment.count;

				
				boolean highLight = true;
				//Iterator<Integer> iterator = columns.keySet().iterator();
				//ColumnDetails[] flds = fieldListMgr.getFieldArray();
				List<? extends IBasicColumn> flds = columnInterface.getColumns();
				int idx = 0;
				
				int next = 0;
				int last = 0;
				Color highlightColor = Color.CYAN;
				Color highlightBackgroundColor = Color.BLUE;
				Color normalColor = Color.BLACK;
				Color normalBackgroundColor = null;
				if (startOffset == 0) {
					highlightColor = Color.BLUE;
					highlightBackgroundColor = HEADER_HIGHLIGHTED_BACKGROUND;
					normalColor = Color.BLACK;
					normalBackgroundColor = HEADER_BACKGROUND;
				}
				if (flds.size() > 0) {
					next = flds.get(idx++).getStart() - 1;
					while (next < pos && idx < flds.size()) {
						next = flds.get(idx++).getStart() - 1;
					}
					while (pos < count) {
						if (highLight) {
							x = writeText(g, x, y, t1, next - last, segment, highlightColor, highlightBackgroundColor);
						} else {
							x = writeText(g, x, y, t1, next - last, segment, normalColor, normalBackgroundColor);						
						}
						
						highLight = ! highLight;
						if (idx >= flds.size()) { break; }
						last = next;
						next = flds.get(idx++).getStart() - 1;
					}
				}
				if (highLight) {
					x = writeText(g, x, y, t1, count - next, segment, highlightColor, highlightBackgroundColor);
				} else {
					x = writeText(g, x, y, t1, count - next, segment, normalColor, normalBackgroundColor);						
				}

				return x;
			} catch (Exception e) {
				return super.drawUnselectedText(g, x, y, p0, p1);
			}
		}
		
		
		private int  writeText(Graphics g, int x, int y, int t1, int count, Segment segment, Color foreground, Color backgroundColor) {

			if (count > 0) {
				segment.count = count;
				//System.out.print((backgroundColor != null) + " " + y + ", " + x + " : " + segment.offset + " " + count);
				if (backgroundColor != null/* && dh != null*/) {
					g.setColor(backgroundColor);
					FontMetrics fontMetrics = g.getFontMetrics();
					int fontHeight = fontMetrics.getHeight();
		
					int tabbedTextWidth = Utilities.getTabbedTextWidth(segment, fontMetrics, x, this, t1);
					g.fillRect(x, y - Math.max(0, y - 1) % fontHeight, tabbedTextWidth, fontHeight);
					//System.out.println(" ~ " + tabbedTextWidth + " ~ " + x + ", " + y + " ~ " + (y - (y - 1) % fontHeight) + " " + fontHeight);
				}
				
				if (foreground == null) {
					g.setColor(Color.BLACK);
				} else {
					g.setColor(foreground);
				}
				//System.out.println();
				x = Utilities.drawTabbedText(segment, x, y, g, this, t1);
				segment.offset += count;
			}
			return x;
		}
	}

	public static interface IColumInterface {
		
		public void columnSelected(int column);
		public List<? extends IBasicColumn> getColumns();
	}

	/**
	 * @return
	 * @see javax.swing.JEditorPane#getText()
	 */
	public String getText() {
		return fieldSelectionTxt.getText();
	}


	/**
	 * @param position
	 * @see javax.swing.text.JTextComponent#setCaretPosition(int)
	 */
	public void setCaretPosition(int position) {
		fieldSelectionTxt.setCaretPosition(position);
	}
}
