/**
 * 
 */
package net.sf.RecordEditor.utils.swing.treeCombo;

import java.io.File;

import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChanged;
import net.sf.recordEditor.ui.fileFields.UiFileField;
import net.sf.recordEditor.ui.fileFields.params.IFileFieldParam;
import net.sf.recordEditor.ui.listManagers.IRecentList;

/**
 * @author bruce
 *
 */
@SuppressWarnings("serial")
public class FileFieldCombo extends UiFileField implements IFileSelectCombo {

	private boolean expandVars = false;
	private final FileListnerTransformer listnerTransformer;
	private boolean toAddListner = true;

	public FileFieldCombo(IFileFieldParam param) {
		super(param);
		this.listnerTransformer = new FileListnerTransformer(param.isDirectory());
	}


	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#addFileChangedListner(net.sf.RecordEditor.utils.swing.filechooser.IFileChanged)
	 */
	@Override
	public void addFileChangedListner(IFileChanged listner) {
		
		if (toAddListner) {
			super.addFileChangedListner(listnerTransformer);
			toAddListner = false;
		}
		
		listnerTransformer.fileChangedList.add(listner);
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#removeFileChangedListner(net.sf.RecordEditor.utils.swing.filechooser.IFileChanged)
	 */
	@Override
	public void removeFileChangedListner(IFileChanged listner) {
		listnerTransformer.fileChangedList.remove(listner);
	}

	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#getText()
	 */
	@Override
	public String getText() {
		File f = this.getSelectedFile();
		String fileName =  f == null ? "" : f.getAbsolutePath();
		IRecentList recentList = super.getRecentList();
		if (recentList != null) {
			recentList.addFileItem(fileName);
		}
		
		if (expandVars) {
			fileName = Parameters.encodeVars(fileName);
		}
		return fileName;
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#setTextSilently(java.lang.String)
	 */
	@Override
	public void setFileNameSilently(String fileName) {
		if (fileName != null) {
			try {
				listnerTransformer.notifyListners = false;
				//super.setFileNameSilently(expandFileName(fileName));
				super.setSelectedFile(new File(expandFileName(fileName)), false);
			} finally {
				listnerTransformer.notifyListners = true;
			}
		}
	}
	
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#setFileName(java.lang.String)
	 */
	@Override
	public void setFileName(String fileName) {
		if (fileName != null) {
			super.setSelectedFile(new File(expandFileName(fileName)), true);
		}
	}

	private String expandFileName(String fileName) {
		if (expandVars) {
			fileName = Parameters.expandVars(fileName);
		}
		if (fileName.endsWith("*") && fileName.length() > 0) {
			fileName = fileName.substring(0, fileName.length()-1);
		}
		return fileName;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#setExpandVars(boolean)
	 */
	@Override
	public void setExpandVars(boolean expandVars) {
		this.expandVars = expandVars;
	}
	
}
