package net.sf.RecordEditor.utils.swing.treeCombo;

import java.io.File;

import net.sf.RecordEditor.utils.common.Common;
import net.sf.RecordEditor.utils.params.Parameters;
import net.sf.RecordEditor.utils.swing.favourite.UpdatableParameterList;
import net.sf.RecordEditor.utils.swing.filechooser.IFileChanged;
import net.sf.recordEditor.ui.def.FileChooserDefs.FileChooserType;
import net.sf.recordEditor.ui.fileChooser.UiFileChooser;
import net.sf.recordEditor.ui.fileFields.ChooseFileBtn;
import net.sf.recordEditor.ui.fileFields.UiBreadCrumbFileField;
import net.sf.recordEditor.ui.fileFields.params.FileFieldParamBldr;
import net.sf.recordEditor.ui.fileFields.params.IFileFieldParam;
import net.sf.recordEditor.ui.fileFields.params.IFileFieldParamAndBuilder;
import net.sf.recordEditor.ui.listManagers.BasicItemList;
import net.sf.recordEditor.ui.listManagers.IRecentList;
import net.sf.recordEditor.ui.listManagers.IUpdateableList;

/**
 * This class creates a File Selection field and uses the parameter
 * file to store / retrieve the last used files
 * 
 * @author Bruce
 *
 */
@SuppressWarnings("serial")
public class FileSelectCombo extends UiBreadCrumbFileField
implements IFileSelectCombo {

	private boolean expandVars = false;
	private final FileListnerTransformer listnerTransformer;
	private boolean toAddListner = true;
	
	static {
		try {
			updateFileChooserOption(Parameters.getString(Parameters.FILE_CHOOSER_UI));
		} catch (Exception e) {
		}
	}

	private static void updateFileChooserOption(String fcUi) {
		ChooseFileBtn.setUseUiFileChooser(! "N".equalsIgnoreCase(fcUi));
	}
	
	public static IFileSelectCombo newFileCombo(
			boolean isOpen, boolean isDirectory, 
			boolean allowNewFiles) {
		FileSelectCombo fsc =  new FileSelectCombo(FileFieldParamBldr.newFileFieldParamBuilder()
						.recentFiles(new BasicItemList("File"))
						.directory(isDirectory)
						.allowNewFiles(allowNewFiles)
						.fileChooserOptions(UiFileChooser.newOptionBldr()
										.chooserType(isOpen ? FileChooserType.open : FileChooserType.save)
								.asBuilderAndParameter())
					.asBuilderAndParameter()
				);
//				null, new BasicFilesList("File"), 
//				null, null,
//				null, true, isDirectory, allowNewFiles);
		
//		if (isOpen) {
//			fsc.setChooserType(FileChooserType.open_TextField);
//		} else {
//			fsc.setChooserType(FileChooserType.save);
//		}

		return fsc;
	}
	
	/**
	 * 
	 * This class creates a File Selection field and uses the paramter
	 * file to store / retrieve the last used files
	 * 
	 * @param paramKey file name key for extracting last used files
	 * @param numEntries number of entries to display in the list
	 * @param isOpen
	 * @param isDirectory wether the files are directories or not
	 * @param allowNewFiles allow new files
	 */
	public static IFileSelectCombo newCombo(
			String paramKey, 
			int numEntries, boolean isOpen, boolean isDirectory, boolean allowNewFiles) {
		
		return newCombo(FileFieldParamBldr.newFileFieldParamBuilder()
						.setRecentFavourite(UpdatableParameterList.newRecentFavourites(isDirectory, paramKey, numEntries, allowNewFiles))
						.directory(isDirectory)
						.fileChooserOptions(UiFileChooser.newOptionBldr()
									.chooserType(isOpen ? FileChooserType.open : FileChooserType.save)
								.asBuilderAndParameter())
//						.chooseFileBtnType(isOpen ? FileChooserType.open : FileChooserType.save)
					.asBuilderAndParameter()	);
	}

	/**
	 * 
	 * This class creates a File Selection field and uses the paramter
	 * file to store / retrieve the last used files
	 * 
	 * @param paramKey file name key for extracting last used files
	 * @param numEntries number of entries to display in the list
	 * @param isOpen
	 * @param isDirectory wether the files are directories or not
	 */
	public static IFileSelectCombo newCombo(String paramKey, int numEntries, boolean isOpen, boolean isDirectory) {
		return newCombo(paramKey, numEntries, isOpen, isDirectory, false);
	}

	public static IFileSelectCombo newCombo(final File path, 
    		IRecentList recentList, IUpdateableList favouriteList,
    		IRecentList recentDir,  IUpdateableList favouriteDir,
    		boolean chooseFileBtnType, boolean isDirectory, boolean allowNewFiles) {
		return newCombo(FileFieldParamBldr.newFileFieldParamBuilder()
						.initialPath(path)
						.recentFiles(recentList)
						.recentDirectory(recentDir)
						.favouriteFiles(favouriteList)
						.favouriteDirectory(favouriteDir)
						.fileChooserOptions(UiFileChooser.newOptionBldr()
									.chooserType(chooseFileBtnType ? FileChooserType.open : FileChooserType.save)
								.asBuilderAndParameter())
//						.chooseFileBtnType(chooseFileBtnType ? FileChooserType.open : FileChooserType.save)
						.directory(isDirectory)
					.asBuilderAndParameter()
				);
	}

	public static IFileSelectCombo newCombo(final File path, 
    		IRecentList recentList, IUpdateableList favouriteList,
    		IRecentList recentDir,  IUpdateableList favouriteDir,
    		net.sf.recordEditor.ui.def.FileChooserDefs.FileChooserType chooseFileBtnType, boolean isDirectory, boolean allowNewFiles) {
		return newCombo(FileFieldParamBldr.newFileFieldParamBuilder()
						.initialPath(path)
						.recentFiles(recentList)
						.recentDirectory(recentDir)
						.favouriteFiles(favouriteList)
						.favouriteDirectory(favouriteDir)
						.fileChooserOptions(UiFileChooser.newOptionBldr()
								.chooserType(chooseFileBtnType)
							.asBuilderAndParameter())
//						.chooseFileBtnType(chooseFileBtnType)
						.directory(isDirectory)
						.completeFile(Common.OPTIONS.AUTO_COMPLETE.isSelected())
					.asBuilderAndParameter()
				);
//		if (OPEN_IN_EDIT_MODE.isSelected()) {
//			//System.out.println("Edit mode B !!!");
//			return new FileFieldCombo(
//						path, recentList, favouriteList, recentDir, favouriteDir, addChooseFileBtn, isDirectory);
//		} 
//		//System.out.println("Normal mode A !!!");
//		return new FileSelectCombo(
//				path, recentList, favouriteList, recentDir, favouriteDir, addChooseFileBtn, isDirectory, allowNewFiles);
		
	}

	public static IFileSelectCombo newCombo(IFileFieldParamAndBuilder param) {
		param.completeFile(Common.OPTIONS.AUTO_COMPLETE.isSelected());
		if (Common.OPTIONS.OPEN_IN_EDIT_MODE.isSelected()) {
//			System.out.println("Edit mode B !!!");
			return new FileFieldCombo(param);
		} 
//		System.out.println("Normal mode A !!!");
		return new FileSelectCombo(param);
		
	}

//	private String paramKey;
//	private final boolean isDirectory;//, allowNewFiles;
//	private boolean notifyListners = true;
	//private File lastFile = null;
	
	
	//private ArrayList<IFileChanged> fileChangedList = new ArrayList<IFileChanged>(2);


//	/**
//	 * 
//	 * This class creates a File Selection field and uses the paramter
//	 * file to store / retrieve the last used files
//	 * 
//	 * @param paramKey file name key for extracting last used files
//	 * @param numEntries number of entries to display in the list
//	 * @param isOpen
//	 * @param isDirectory wether the files are directories or not
//	 */
//	public FileSelectCombo(String paramKey, int numEntries, boolean isOpen, boolean isDirectory) {
//		this(paramKey, numEntries, isOpen, isDirectory, false);
//	}
//	
//	public FileSelectCombo(WebPathFileField3 baseFld, boolean isDirectory, boolean allowNewFiles) {
//		super(null, baseFld, true);
//		this.paramKey = baseFld instanceof FileSelectCombo 
//				? ((FileSelectCombo) baseFld).paramKey
//				: "";
//		this.isDirectory = isDirectory;
//		this.allowNewFiles = allowNewFiles;
//		init();
//	}

//	/**
//	 * 
//	 * This class creates a File Selection field and uses the paramter
//	 * file to store / retrieve the last used files
//	 * 
//	 * @param paramKey file name key for extracting last used files
//	 * @param numEntries number of entries to display in the list
//	 * @param isOpen
//	 * @param isDirectory wether the files are directories or not
//	 * @param allowNewFiles allow new files
//	 */
//	public FileSelectCombo(String paramKey, int numEntries, boolean isOpen, boolean isDirectory, boolean allowNewFiles) {
//		this(	null,
//				UpdatableParameterList.newFileList(paramKey, numEntries, allowNewFiles), 
//				UpdatableParameterList.newDirectoryList(paramKey + '.' + UpdatableParameterList.DIRECTORIES, numEntries, false), 
//				UpdatableParameterList.newFileList(
//						paramKey + ".Fav" , numEntries, allowNewFiles), 
//				UpdatableParameterList.newDirectoryList(
//						paramKey + ".Fav." + UpdatableParameterList.DIRECTORIES, 
//						numEntries, false), 
//				true,  isDirectory,  allowNewFiles);
////		this.paramKey = paramKey;
//	}

	protected FileSelectCombo(IFileFieldParam param) {
		super(param);
		this.listnerTransformer = new FileListnerTransformer(param.isDirectory());
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#addFileChangedListner(net.sf.RecordEditor.utils.swing.filechooser.IFileChanged)
	 */
	@Override
	public void addFileChangedListner(IFileChanged listner) {
		
		if (toAddListner) {
			super.addFileChangedListner(listnerTransformer);
			toAddListner = false;
		}
		
		listnerTransformer.fileChangedList.add(listner);
	}



	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#removeFileChangedListner(net.sf.RecordEditor.utils.swing.filechooser.IFileChanged)
	 */
	@Override
	public void removeFileChangedListner(IFileChanged listner) {
		listnerTransformer.fileChangedList.remove(listner);
	}

	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#getText()
	 */
	@Override
	public String getText() {
		File f = this.getSelectedFile();
		String fileName =  f == null ? "" : f.getAbsolutePath();
		IRecentList recentList = super.getRecentList();
		if (recentList != null) {
			recentList.addFileItem(fileName);
		}
		
		if (expandVars) {
			fileName = Parameters.encodeVars(fileName);
		}
		return fileName;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#setTextSilently(java.lang.String)
	 */
	@Override
	public void setFileNameSilently(String fileName) {
		if (fileName != null) {
			try {
				listnerTransformer.notifyListners = false;
				//super.setFileNameSilently(expandFileName(fileName));
				super.setSelectedFile(new File(expandFileName(fileName)), false);
			} finally {
				listnerTransformer.notifyListners = true;
			}
		}
	}
	
	
	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#setFileName(java.lang.String)
	 */
	@Override
	public void setFileName(String fileName) {
		if (fileName != null) {
			super.setSelectedFile(new File(expandFileName(fileName)), true);
		}
	}

	private String expandFileName(String fileName) {
		if (expandVars) {
			fileName = Parameters.expandVars(fileName);
		}
		if (fileName.endsWith("*") && fileName.length() > 0) {
			fileName = fileName.substring(0, fileName.length()-1);
		}
		return fileName;
	}

	/* (non-Javadoc)
	 * @see net.sf.RecordEditor.utils.swing.treeCombo.IFileSelectCombo#setExpandVars(boolean)
	 */
	@Override
	public void setExpandVars(boolean expandVars) {
		this.expandVars = expandVars;
	}
	
}
