package net.sf.RecordEditor.xml.classes;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.Reader;
import java.util.ArrayList;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import net.sf.RecordEditor.xml.support.BasicProcessXmlTag;
import net.sf.RecordEditor.xml.support.ICreateProcessXmlTag;
import net.sf.RecordEditor.xml.support.IProcessXmlTag;
import net.sf.RecordEditor.xml.support.ReaderXml;

public class  ReadXml {

    private final XMLStreamReader reader;
    
    public ReadXml(String filename) throws XMLStreamException, FactoryConfigurationError, FileNotFoundException {
        this(new FileReader(filename));                                                                                   
    }
 
    public ReadXml(Reader reader) throws XMLStreamException, FactoryConfigurationError {
        this.reader = XMLInputFactory.newInstance().createXMLStreamReader(reader);                                                                                   
    }

    public ReadXml(XMLStreamReader reader) {
        this.reader = reader;                                                                                   
    }
  

    public EditorTask readEditorTask() throws XMLStreamException {

        ReaderXml rx = new ReaderXml(reader);
        CreateEditorTask pt = new CreateEditorTask();
   
       return rx.readFile("EditorTask", pt) ? pt.pd.itm : null;
    }
	
	private static class CreateEditorTask implements ICreateProcessXmlTag {
		ProcessEditorTask pd; 
		@Override
		public IProcessXmlTag create(ReaderXml reader) {
			pd = new ProcessEditorTask(null, reader);
			return pd;
		}
	}
	
    public DiffDefinition readDiffDefinition() throws XMLStreamException {

        ReaderXml rx = new ReaderXml(reader);
        CreateDiffDefinition pt = new CreateDiffDefinition();
   
       return rx.readFile("diff", pt) ? pt.pd.itm : null;
    }
	
	private static class CreateDiffDefinition implements ICreateProcessXmlTag {
		ProcessDiffDefinition pd; 
		@Override
		public IProcessXmlTag create(ReaderXml reader) {
			pd = new ProcessDiffDefinition(null, reader);
			return pd;
		}
	}
	
    public CopyDefinition readCopyDefinition() throws XMLStreamException {

        ReaderXml rx = new ReaderXml(reader);
        CreateCopyDefinition pt = new CreateCopyDefinition();
   
       return rx.readFile("copy", pt) ? pt.pd.itm : null;
    }
	
	private static class CreateCopyDefinition implements ICreateProcessXmlTag {
		ProcessCopyDefinition pd; 
		@Override
		public IProcessXmlTag create(ReaderXml reader) {
			pd = new ProcessCopyDefinition(null, reader);
			return pd;
		}
	}
	
	
		


    private static class ProcessEditorTask extends BasicProcessXmlTag {
        private final EditorTask itm = new EditorTask();
        
        
        private final ReaderXml readerXml;
        
        public ProcessEditorTask(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.type = readerXml.getString("type", itm.type);
	        itm.layoutName = readerXml.getString("layoutName", itm.layoutName);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "layout": 
               ProcessLayout rfilter = new ProcessLayout (this, readerXml);
               ret  = rfilter;
               itm.filter = rfilter.itm;
               break;
           case "sorttree": 
               ProcessSortTree rsortTree = new ProcessSortTree (this, readerXml);
               ret  = rsortTree;
               itm.sortTree = rsortTree.itm;
               break;
           case "recordtree": 
               ProcessRecordTree rrecordTree = new ProcessRecordTree (this, readerXml);
               ret  = rrecordTree;
               itm.recordTree = rrecordTree.itm;
               break;
           case "fieldsequence": 
               ProcessFieldSequence rfieldSequence = new ProcessFieldSequence (this, readerXml);
               ret  = rfieldSequence;
               itm.fieldSequence = rfieldSequence.itm;
               break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }

    private static class ProcessLayout extends BasicProcessXmlTag {
        private final Layout itm = new Layout();
        
        
        private final ReaderXml readerXml;
        
        public ProcessLayout(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.name = readerXml.getString("name", itm.name);
	        itm.saveFile = readerXml.getString("saveFile", itm.saveFile);
	        itm.groupHeader = readerXml.getString("groupHeader", itm.groupHeader);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "record": 
               ProcessRecord rrecords = new ProcessRecord (this, readerXml);
               ret  = rrecords;
	           if (itm.records == null) {
	              itm.records = new ArrayList<Record>();
	           }
               itm.records.add(rrecords.itm);
               break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }

    private static class ProcessRecord extends BasicProcessXmlTag {
        private final Record itm = new Record();
        
	    private ArrayList<String> fields = new ArrayList<String>();
        
        private final ReaderXml readerXml;
        
        public ProcessRecord(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.name = readerXml.getString("name", itm.name);
	        itm.include = readerXml.getBoolean("include", itm.include);
	        itm.inGroup = readerXml.getBoolean("inGroup", itm.inGroup);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "fieldtest": 
               ProcessFieldTest rfieldTest = new ProcessFieldTest (this, readerXml);
               ret  = rfieldTest;
	           if (itm.fieldTest == null) {
	              itm.fieldTest = new ArrayList<FieldTest>();
	           }
               itm.fieldTest.add(rfieldTest.itm);
               break;
           case "field": 
                fields.add(readerXml.getTagString("field", null));
                break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           
       @Override
       public IProcessXmlTag getParent() {
       
	       itm.fields = fields.toArray(new String[fields.size()]);
           return super.getParent();
       }

    }

    private static class ProcessFieldTest extends BasicProcessXmlTag {
        private final FieldTest itm = new FieldTest();
        
        
//        private final ReaderXml readerXml;
        
        public ProcessFieldTest(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
 //           this.readerXml = readerXml;
	        itm.booleanOperator = readerXml.getString("booleanOperator", itm.booleanOperator);
	        itm.recordName = readerXml.getString("recordName", itm.recordName);
	        itm.value = readerXml.getString("value", itm.value);
	        itm.operator = readerXml.getString("operator", itm.operator);
	        itm.fieldName = readerXml.getString("fieldName", itm.fieldName);
	        itm.groupOperator = readerXml.getint("groupOperator", itm.groupOperator);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           
           return ret;
       }
           

    }

    private static class ProcessSortTree extends BasicProcessXmlTag {
        private final SortTree itm = new SortTree();
        
        
        private final ReaderXml readerXml;
        
        public ProcessSortTree(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.recordName = readerXml.getString("recordName", itm.recordName);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "sortfields": 
               ProcessSortFields rsortFields = new ProcessSortFields (this, readerXml);
               ret  = rsortFields;
	           if (itm.sortFields == null) {
	              itm.sortFields = new ArrayList<SortFields>();
	           }
               itm.sortFields.add(rsortFields.itm);
               break;
           case "sortsummary": 
               ProcessSortSummary rsortSummary = new ProcessSortSummary (this, readerXml);
               ret  = rsortSummary;
	           if (itm.sortSummary == null) {
	              itm.sortSummary = new ArrayList<SortSummary>();
	           }
               itm.sortSummary.add(rsortSummary.itm);
               break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }

    private static class ProcessSortFields extends BasicProcessXmlTag {
        private final SortFields itm = new SortFields();
        
        
 //       private final ReaderXml readerXml;
        
        public ProcessSortFields(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
//            this.readerXml = readerXml;
	        itm.fieldName = readerXml.getString("fieldName", itm.fieldName);
	        itm.ascending = readerXml.getboolean("ascending", itm.ascending);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           
           return ret;
       }
           

    }

    private static class ProcessSortSummary extends BasicProcessXmlTag {
        private final SortSummary itm = new SortSummary();
        
        
//        private final ReaderXml readerXml;
        
        public ProcessSortSummary(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
 //           this.readerXml = readerXml;
	        itm.fieldName = readerXml.getString("fieldName", itm.fieldName);
	        itm.operator = readerXml.getString("operator", itm.operator);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           
           return ret;
       }
           

    }

    private static class ProcessRecordTree extends BasicProcessXmlTag {
        private final RecordTree itm = new RecordTree();
        
        
        private final ReaderXml readerXml;
        
        public ProcessRecordTree(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "recordparent": 
               ProcessRecordParent rrecordParent = new ProcessRecordParent (this, readerXml);
               ret  = rrecordParent;

               itm.getParentRelationship().add(rrecordParent.itm);
               break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }

    private static class ProcessRecordParent extends BasicProcessXmlTag {
        private final RecordParent itm = new RecordParent();
        
        
//        private final ReaderXml readerXml;
        
        public ProcessRecordParent(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
//            this.readerXml = readerXml;
	        itm.recordName = readerXml.getString("recordName", itm.recordName);
	        itm.parentName = readerXml.getString("parentName", itm.parentName);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           
           return ret;
       }
           

    }

    private static class ProcessFieldSequence extends BasicProcessXmlTag {
        private final FieldSequence itm = new FieldSequence();
        
	    private ArrayList<String> fields = new ArrayList<String>();
	    private ArrayList<String> fixedFields = new ArrayList<String>();
        
        private final ReaderXml readerXml;
        
        public ProcessFieldSequence(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.name = readerXml.getString("name", itm.name);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "field": 
                fields.add(readerXml.getTagString("Field", null));
                break;
           case "fixedfield": 
                fixedFields.add(readerXml.getTagString("FixedField", null));
                break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           
       @Override
       public IProcessXmlTag getParent() {
       
	       itm.fields = fields.toArray(new String[fields.size()]);
	       itm.fixedFields = fixedFields.toArray(new String[fixedFields.size()]);
           return super.getParent();
       }

    }

    private static class ProcessDiffDefinition extends BasicProcessXmlTag {
        private final DiffDefinition itm = new DiffDefinition();
        
        
        private final ReaderXml readerXml;
        
        public ProcessDiffDefinition(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.type = readerXml.getString("type", itm.type);
	        itm.complete = readerXml.getString("complete", itm.complete);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "layout": 
               ProcessLayout rlayoutDetails = new ProcessLayout (this, readerXml);
               ret  = rlayoutDetails;
               itm.layoutDetails = rlayoutDetails.itm;
               break;
           case "oldfile": 
               ProcessFileContainer roldfile = new ProcessFileContainer (this, readerXml);
               ret  = roldfile;
               itm.oldfile = roldfile.itm;
               break;
           case "newfile": 
               ProcessFileContainer rnewfile = new ProcessFileContainer (this, readerXml);
               ret  = rnewfile;
               itm.newfile = rnewfile.itm;
               break;
           case "htmloptions": 
               ProcessHtmlOptions rhtmlOptions = new ProcessHtmlOptions (this, readerXml);
               ret  = rhtmlOptions;
               itm.htmlOptions = rhtmlOptions.itm;
               break;
           case "striptrailingspaces": 
                itm.stripTrailingSpaces = readerXml.getTagboolean("stripTrailingSpaces", itm.stripTrailingSpaces);
                break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }

    private static class ProcessFileContainer extends BasicProcessXmlTag {
        private final FileContainer itm = new FileContainer();
        
        
        private final ReaderXml readerXml;
        
        public ProcessFileContainer(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "file": 
               ProcessFile rfile = new ProcessFile (this, readerXml);
               ret  = rfile;
               itm.file = rfile.itm;
               break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }

    private static class ProcessFile extends BasicProcessXmlTag {
        private final File itm = new File();
        
        
        private final ReaderXml readerXml;
        
        public ProcessFile(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.name = readerXml.getString("name", itm.name);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "layout": 
               ProcessLayout rlayoutDetails = new ProcessLayout (this, readerXml);
               ret  = rlayoutDetails;
               itm.layoutDetails = rlayoutDetails.itm;
               break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }

    private static class ProcessHtmlOptions extends BasicProcessXmlTag {
        private final HtmlOptions itm = new HtmlOptions();
        
        
 //       private final ReaderXml readerXml;
        
        public ProcessHtmlOptions(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
 //           this.readerXml = readerXml;
	        itm.htmlFile = readerXml.getString("HtmlFile", itm.htmlFile);
	        itm.allRows = readerXml.getboolean("AllRows", itm.allRows);
	        itm.allFields = readerXml.getboolean("AllFields", itm.allFields);
	        itm.singleTable = readerXml.getboolean("SingleTable", itm.singleTable);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           
           return ret;
       }
           

    }

    private static class ProcessCopyDefinition extends BasicProcessXmlTag {
        private final CopyDefinition itm = new CopyDefinition();
        
        
        private final ReaderXml readerXml;
        
        public ProcessCopyDefinition(IProcessXmlTag parent, ReaderXml readerXml) {
            super(parent);
            
            this.readerXml = readerXml;
	        itm.type = readerXml.getString("type", itm.type);
	        itm.delimiter = readerXml.getString("delimiter", itm.delimiter);
	        itm.fieldErrorFile = readerXml.getString("fieldErrorFile", itm.fieldErrorFile);
	        itm.complete = readerXml.getString("complete", itm.complete);
	        itm.maxErrors = readerXml.getint("maxErrors", itm.maxErrors);
	        itm.namesOnFirstLine = readerXml.getboolean("namesOnFirstLine", itm.namesOnFirstLine);
        }


       @Override
       public IProcessXmlTag processTag(String name) {
           IProcessXmlTag ret = this;

           switch (name) {
           case "treedefinition": 
               ProcessRecordTree rtreeDefinition = new ProcessRecordTree (this, readerXml);
               ret  = rtreeDefinition;
               itm.treeDefinition = rtreeDefinition.itm;
               break;
           case "layoutdetails": 
               ProcessLayout rlayoutDetails = new ProcessLayout (this, readerXml);
               ret  = rlayoutDetails;
               itm.layoutDetails = rlayoutDetails.itm;
               break;
           case "oldfile": 
               ProcessFileContainer roldfile = new ProcessFileContainer (this, readerXml);
               ret  = roldfile;
               itm.oldfile = roldfile.itm;
               break;
           case "newfile": 
               ProcessFileContainer rnewfile = new ProcessFileContainer (this, readerXml);
               ret  = rnewfile;
               itm.newfile = rnewfile.itm;
               break;
           case "htmloptions": 
               ProcessHtmlOptions rhtmlOptions = new ProcessHtmlOptions (this, readerXml);
               ret  = rhtmlOptions;
               itm.htmlOptions = rhtmlOptions.itm;
               break;
           case "striptrailingspaces": 
                itm.stripTrailingSpaces = readerXml.getTagboolean("stripTrailingSpaces", itm.stripTrailingSpaces);
                break;
           case "quote": 
                itm.quote = readerXml.getTagString("quote", itm.quote);
                break;
           case "font": 
                itm.font = readerXml.getTagString("font", itm.font);
                break;
           case "velocitytemplate": 
                itm.velocityTemplate = readerXml.getTagString("velocityTemplate", itm.velocityTemplate);
                break;
           default:
        	   ret = null;
	
           }
           
           return ret;
       }
           

    }
	
	
}
