package net.sf.RecordEditor.xml.classes;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import net.sf.RecordEditor.xml.support.WriterXml;

public class  WriteXml {

    private WriterXml xmlWriter;

    public WriteXml(String file)  throws XMLStreamException, FileNotFoundException  {
    	this(new FileOutputStream(file));
    }
    
    public WriteXml(OutputStream os)  throws XMLStreamException  {
 		XMLStreamWriter xmlStream = XMLOutputFactory.newInstance().createXMLStreamWriter(os);
         xmlWriter = new WriterXml(xmlStream, true);
     }

    public WriteXml(XMLStreamWriter writer)  throws XMLStreamException  {
        xmlWriter = new WriterXml(writer, true);
    }

	 public void writeEditorTask(EditorTask itm) throws XMLStreamException {
	 
	     writeOneEditorTask("EditorTask", itm);
	     xmlWriter.close();
	 }
	 public void writeDiffDefinition(DiffDefinition itm) throws XMLStreamException {
	 
	     writeOneDiffDefinition("diff", itm);
	     xmlWriter.close();
	 }
	 public void writeCopyDefinition(CopyDefinition itm) throws XMLStreamException {
	 
	     writeOneCopyDefinition("copy", itm);
	     xmlWriter.close();
	 }
    

	 public void writeOneEditorTask(String xmlTag, EditorTask itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("type", itm.type);
	     xmlWriter.writeAttribute("layoutName", itm.layoutName);

	
         writeOneLayout("layout", itm.filter);
         writeOneSortTree("SortTree", itm.sortTree);
         writeOneRecordTree("recordTree", itm.recordTree);
         writeOneFieldSequence("fieldSequence", itm.fieldSequence);
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneLayout(String xmlTag, Layout itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("name", itm.name);
	     xmlWriter.writeAttribute("saveFile", itm.saveFile);
	     xmlWriter.writeAttribute("groupHeader", itm.groupHeader);

	
	     if (itm.records != null) {
	         for (Record val : itm.records) {
	             writeOneRecord("record", val);
	         }
	     }
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneRecord(String xmlTag, Record itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("name", itm.name);
	     xmlWriter.writeAttribute("include", itm.include);
	     xmlWriter.writeAttribute("inGroup", itm.inGroup);

	     xmlWriter.writeRepeatingTag("field", itm.fields);
	
	     if (itm.fieldTest != null) {
	         for (FieldTest val : itm.fieldTest) {
	             writeOneFieldTest("FieldTest", val);
	         }
	     }
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneFieldTest(String xmlTag, FieldTest itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("booleanOperator", itm.booleanOperator);
	     xmlWriter.writeAttribute("recordName", itm.recordName);
	     xmlWriter.writeAttribute("value", itm.value);
	     xmlWriter.writeAttribute("operator", itm.operator);
	     xmlWriter.writeAttribute("fieldName", itm.fieldName);
	     xmlWriter.writeAttribute("groupOperator", itm.groupOperator);

	
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneSortTree(String xmlTag, SortTree itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("recordName", itm.recordName);

	
	     if (itm.sortFields != null) {
	         for (SortFields val : itm.sortFields) {
	             writeOneSortFields("SortFields", val);
	         }
	     }
	     if (itm.sortSummary != null) {
	         for (SortSummary val : itm.sortSummary) {
	             writeOneSortSummary("SortSummary", val);
	         }
	     }
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneSortFields(String xmlTag, SortFields itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("fieldName", itm.fieldName);
	     xmlWriter.writeAttribute("ascending", itm.ascending);

	
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneSortSummary(String xmlTag, SortSummary itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("fieldName", itm.fieldName);
	     xmlWriter.writeAttribute("operator", itm.operator);

	
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneRecordTree(String xmlTag, RecordTree itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);


	
	     if (itm.parentRelationship != null) {
	         for (RecordParent val : itm.parentRelationship) {
	             writeOneRecordParent("RecordParent", val);
	         }
	     }
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneRecordParent(String xmlTag, RecordParent itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("recordName", itm.recordName);
	     xmlWriter.writeAttribute("parentName", itm.parentName);

	
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneFieldSequence(String xmlTag, FieldSequence itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("name", itm.name);

	     xmlWriter.writeRepeatingTag("Field", itm.fields);
	     xmlWriter.writeRepeatingTag("FixedField", itm.fixedFields);
	
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneDiffDefinition(String xmlTag, DiffDefinition itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("type", itm.type);
	     xmlWriter.writeAttribute("complete", itm.complete);

	     xmlWriter.writeTag("stripTrailingSpaces", itm.stripTrailingSpaces);
	
         writeOneLayout("layout", itm.layoutDetails);
         writeOneFileContainer("oldfile", itm.oldfile);
         writeOneFileContainer("newfile", itm.newfile);
         writeOneHtmlOptions("HtmlOptions", itm.htmlOptions);
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneFileContainer(String xmlTag, FileContainer itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);


	
         writeOneFile("file", itm.file);
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneFile(String xmlTag, File itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("name", itm.name);

	
         writeOneLayout("layout", itm.layoutDetails);
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneHtmlOptions(String xmlTag, HtmlOptions itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("HtmlFile", itm.htmlFile);
	     xmlWriter.writeAttribute("AllRows", itm.allRows);
	     xmlWriter.writeAttribute("AllFields", itm.allFields);
	     xmlWriter.writeAttribute("SingleTable", itm.singleTable);

	
	     xmlWriter.writeEndElement();
	 }

	 public void writeOneCopyDefinition(String xmlTag, CopyDefinition itm) throws XMLStreamException {
	 
	     if (itm == null) { return; }
	     xmlWriter.writeStartElement(xmlTag);

	     xmlWriter.writeAttribute("type", itm.type);
	     xmlWriter.writeAttribute("delimiter", itm.delimiter);
	     xmlWriter.writeAttribute("fieldErrorFile", itm.fieldErrorFile);
	     xmlWriter.writeAttribute("complete", itm.complete);
	     xmlWriter.writeAttribute("maxErrors", itm.maxErrors);
	     xmlWriter.writeAttribute("namesOnFirstLine", itm.namesOnFirstLine);

	     xmlWriter.writeTag("stripTrailingSpaces", itm.stripTrailingSpaces);
	     xmlWriter.writeTag("quote", itm.quote);
	     xmlWriter.writeTag("font", itm.font);
	     xmlWriter.writeTag("velocityTemplate", itm.velocityTemplate);
	
         writeOneRecordTree("treeDefinition", itm.treeDefinition);
         writeOneLayout("layoutDetails", itm.layoutDetails);
         writeOneFileContainer("oldfile", itm.oldfile);
         writeOneFileContainer("newfile", itm.newfile);
         writeOneHtmlOptions("HtmlOptions", itm.htmlOptions);
	     xmlWriter.writeEndElement();
	 }
	
	
	public void close() throws XMLStreamException {
		xmlWriter.close();
	}

}