package net.sf.RecordEditor.xml.support;

import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

/**
 * Get attributes from the XmlStream 
 * @author Bruce Martin
 *
 */
public class ReaderXml {
	private final XMLStreamReader xmlStreamReader; 
	
	public ReaderXml(XMLStreamReader reader) {
		super();
		this.xmlStreamReader = reader;

	}
	
	public boolean readFile(String tag, ICreateProcessXmlTag pt) throws XMLStreamException {
        int type = xmlStreamReader.getEventType();
        String name;
        IProcessXmlTag processTheTag = null;
        boolean ok = false;
        
        while (type != XMLStreamConstants.END_DOCUMENT) {
            switch (type) {
            case (XMLStreamConstants.START_ELEMENT) :
                name = xmlStreamReader.getName().toString().toLowerCase();
            
                if (processTheTag == null) {
                   if (name.equalsIgnoreCase(tag)) {
                       processTheTag = pt.create(this);
                       ok = true;
                   }
                } else {
                      IProcessXmlTag newProcessTag = processTheTag.processTag(name);
                      processTheTag = newProcessTag == null ? new BasicProcessXmlTag(processTheTag) : newProcessTag;
                }
                break;
            case (XMLStreamConstants.END_ELEMENT):
                if (processTheTag != null) {
                    processTheTag = processTheTag.getParent();
                    if (processTheTag  == null) {
                        return ok;
                    }
                }
                break;
            }
            type = xmlStreamReader.next();
        }	
        return ok;
    }
	
	public String getString(String localName, String defaultValue) {
		String attributeValue = xmlStreamReader.getAttributeValue(null, localName);
		return attributeValue == null ? defaultValue : attributeValue;
	}
	
	
	public Integer getInteger(String localName, Integer defaultValue) {
		String s = getString(localName, null);
		if (s != null && s.length() > 0) {
			return Integer.valueOf(s);
		}
		return defaultValue;
	}

			
	public Double getDouble(String localName, Double defaultValue) {
		String s = getString(localName, null);
		if (s != null && s.length() > 0) {
			return Double.valueOf(s);
		}
		return defaultValue;
	}
			
	public Float getFloat(String localName, Float defaultValue) {
		String s = getString(localName, null);
		if (s != null && s.length() > 0) {
			return Float.valueOf(s);
		}
		return defaultValue;
	}

	public Boolean getBoolean(String localName, Boolean defaultValue) {
		String s = getString(localName, null);
		Boolean b = defaultValue;
		if (s != null) {
			b = "true".equalsIgnoreCase(s);
		}
		return b;
	}

	
	public String getTagString(String localName, String defaultValue) {
		 String s = defaultValue; 
		 try {			
			 s = xmlStreamReader.getElementText();
			 if (! xmlStreamReader.isEndElement()) {
				 int type;
				 while ((type = xmlStreamReader.next()) != XMLStreamConstants.END_ELEMENT) {
					 System.out.print(" " + type);
				 }
			 }
		} catch (XMLStreamException e) {
			throw new RuntimeException("Error in reading Tag", e);
		}
		 
		 return s;
	}
	

	public boolean getTagboolean(String localName, boolean defaultValue) {
		String s = getTagString(localName, null);
		return s== null ? defaultValue : "true".equalsIgnoreCase(s);
	}

	public int getint(String localName, int defaultValue) {
		String s = getString(localName, null);
		return s == null ? defaultValue : Integer.parseInt(s);
	}



	public boolean getboolean(String localName, boolean defaultValue) {
		String s = getString(localName, null);
		return s== null ? defaultValue : "true".equalsIgnoreCase(s);
	}

	public XMLStreamReader getXmlStreamReader() {
		return xmlStreamReader;
	}

}
