package net.sf.RecordEditor.xml.support;

import java.util.List;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import net.sf.cb2xml.util.IndentXmlWriter;

public class WriterXml {

	private final XMLStreamWriter writer;

	public WriterXml(XMLStreamWriter writer, boolean indent) throws XMLStreamException {
		super();
		
		if (indent) {
			writer = new IndentXmlWriter(writer);
		}
		this.writer = writer;
		
		writer.writeStartDocument();
	}
	
	public void writeAttribute(String attrName, Object o) throws XMLStreamException {
		if (o != null) {
			writer.writeAttribute(attrName, o.toString());
		}
	}
	
	public void writeAttribute(String attrName, long l) throws XMLStreamException {
		writer.writeAttribute(attrName, Long.toString(l));
	}
	
	public void writeAttribute(String attrName, boolean b) throws XMLStreamException {
		writer.writeAttribute(attrName, Boolean.toString(b));
	}
	
	
	public void writeRepeatingTag(String tagName, List<?> ol) throws XMLStreamException {
		if (ol != null) {
			for (Object o : ol) {
				writer.writeStartElement(tagName);
				writer.writeCharacters(o.toString());
				writer.writeEndElement();
			}
		}
	}
	
	public <X> void writeRepeatingTag(String tagName, X[] ol) throws XMLStreamException {
		if (ol != null) {
			for (Object o : ol) {
				writer.writeStartElement(tagName);
				writer.writeCharacters(o.toString());
				writer.writeEndElement();
			}
		}
	}

	public void writeTag(String tagName, Object o) throws XMLStreamException {
		if (o != null) {
			writer.writeStartElement(tagName);
			writer.writeCharacters(o.toString());
			writer.writeEndElement();
		}
	}

	public void close() throws XMLStreamException {
		writer.close();
	}

	public void flush() throws XMLStreamException {
		writer.flush();
	}

	public void writeEndElement() throws XMLStreamException {
		writer.writeEndElement();
	}

	public void writeStartElement(String arg0) throws XMLStreamException {
		writer.writeStartElement(arg0);
	}

}
