package diff;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import org.junit.Test;

import net.sf.RecordEditor.diff.xml.XmlCompareOptions;
import net.sf.RecordEditor.utils.msg.UtMessages;

/**
 * Basic JUnit tests for XmlCompareOptions class
 * 
 * @author Bruce Martin
 *
 */
public class TstXmlCompare {

	@Test
	public void testFileBasic() {

		boolean directoryCompare = false;
		XmlCompareOptions fileOptions = new XmlCompareOptions(directoryCompare);
		
		fileOptions.dir1.setFileName("");
		fileOptions.dir2.setFileName("");
		
		fileOptions.compareBtn.doClick();
		assertEquals(XmlCmp.getTwoFileMsg(), fileOptions.msgFld.getText());
		
		fileOptions.dir1.setFileName("aaa");
		fileOptions.dir2.setFileName("");
		
		fileOptions.compareBtn.doClick();
		assertEquals(XmlCmp.getTwoFileMsg(), fileOptions.msgFld.getText());
		
		fileOptions.dir1.setFileName("aaa");
		fileOptions.dir2.setFileName("bbb");
		
		fileOptions.compareBtn.doClick();
		assertEquals("File: aaa does not exist", fileOptions.msgFld.getText());
		//assertTrue(fileOptions.dir1.hasFocus());
		
		String fileNameX1 = getFullFileName("x1.xml");
		String fileNameX2 = getFullFileName("x2.xml");
		
		fileOptions.dir1.setFileName(fileNameX1);
		fileOptions.compareBtn.doClick();
		assertEquals("File: bbb does not exist", fileOptions.msgFld.getText());
		//assertTrue(fileOptions.dir2.hasFocus());

		XmlCmp xmlCmp = new XmlCmp(directoryCompare);
		xmlCmp.dir1.setFileName(fileNameX1);
		xmlCmp.dir2.setFileName(fileNameX2);
		xmlCmp.compareBtn.doClick();
		assertTrue(xmlCmp.fileCmp);
	}


	@Test
	public void testDirBasic() {
		boolean directoryCompare = true;
		XmlCompareOptions fileOptions = new XmlCompareOptions(directoryCompare);
		
		fileOptions.dir1.setFileName("");
		fileOptions.dir2.setFileName("");
		
		fileOptions.compareBtn.doClick();
		assertEquals(XmlCmp.getTwoDirMsg(), fileOptions.msgFld.getText());
		
		fileOptions.dir1.setFileName("aaa");
		fileOptions.dir2.setFileName("");
		
		fileOptions.compareBtn.doClick();
		assertEquals(XmlCmp.getTwoDirMsg(), fileOptions.msgFld.getText());
		
		fileOptions.dir1.setFileName("aaa");
		fileOptions.dir2.setFileName("bbb");
		
		fileOptions.compareBtn.doClick();
//		assertEquals("Directory: aaa does not exist", fileOptions.msgFld.getText());
		assertEquals(XmlCmp.getTwoDirMsg(), fileOptions.msgFld.getText());
		//assertTrue(fileOptions.dir1.hasFocus());
		
		String dirName1 = getFullFileName("dir1");
		String dirName2 = getFullFileName("dir2");
		
		fileOptions.dir1.setFileName(dirName1);
		fileOptions.compareBtn.doClick();
		assertEquals("Directory: bbb does not exist", fileOptions.msgFld.getText());
		//assertTrue(fileOptions.dir2.hasFocus());

		XmlCmp xmlCmp = new XmlCmp(directoryCompare);
		xmlCmp.dir1.setFileName(dirName1);
		xmlCmp.dir2.setFileName(dirName2);
		xmlCmp.compareBtn.doClick();
		assertTrue(xmlCmp.dirCmp);
		
		fileOptions.dir2.setFileName(getFullFileName("x2.xml"));
		fileOptions.compareBtn.doClick();
		assertEquals(UtMessages.ENTER_2_DIRECTORIES.get(), fileOptions.msgFld.getText());

		fileOptions.dir1.setFileName(getFullFileName("x1.xml"));
		fileOptions.dir2.setFileName(dirName2);
		fileOptions.compareBtn.doClick();
		assertEquals(UtMessages.ENTER_2_DIRECTORIES.get(), fileOptions.msgFld.getText());

	}


	/**
	 * @return
	 */
	protected String getFullFileName(String name) {
		return TstXmlCompare.class.getResource(name).getFile();
	}


	
	private static class XmlCmp extends XmlCompareOptions {
		
		boolean dirCmp = false, fileCmp = false;
		
		
		public XmlCmp(boolean directoryCompare) {
			super(directoryCompare);
		}
		
		static String getTwoFileMsg() {
			return TWO_FILES_MSG;
		}

		static String getTwoDirMsg() {
			return TWO_DIRECTORIES_MSG;
		}

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.diff.xml.XmlCompareOptions#compareDirectory(java.lang.String, java.lang.String)
		 */
		@Override
		protected void compareDirectory(String dir1Str, String dir2Str) {
			dirCmp = true;
		}

		/* (non-Javadoc)
		 * @see net.sf.RecordEditor.diff.xml.XmlCompareOptions#compareFile(java.lang.String, java.lang.String)
		 */
		@Override
		protected void compareFile(String dir1Str, String dir2Str) {
			fileCmp = true;
		}
		
		
	}
}
