package layoutWizard;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.Test;

import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.RecordEditor.layoutWizard.Pnl1File;
import net.sf.RecordEditor.layoutWizard.ZTst_Helper;
import net.sf.RecordEditor.re.db.Table.TypeList;
import net.sf.RecordEditor.utils.edit.ManagerRowList;
import net.sf.RecordEditor.utils.fileAnalysis.Details;
import net.sf.RecordEditor.utils.swing.AbsRowList;
import net.sf.RecordEditor.utils.swing.treeCombo.FileSelectCombo;
import xCommon.XCommonCode;

public class TstPnl1File {

	@Test
	public void testDTAR020() throws InterruptedException {
		TstPnl1 pnl = new TstPnl1("DTAR020.bin");
		
		assertTrue(pnl.screenFields.fixedLengthBtn.isSelected());
		assertEquals(IFileStructureConstants.IO_FIXED_LENGTH, pnl.getFileStructure());
		assertEquals("27", pnl.screenFields.lengthTxt.getText());
		assertEquals("cp037", pnl.screenFields.fontNameCombo.getText());
		assertEquals(27, pnl.screenFields.fileTbl.getColumnCount());

		assertTrue(pnl.screenFields.lengthTxt.isVisible());
		assertTrue(pnl.screenFields.fileTbl.isVisible());
		assertTrue(pnl.screenFields.fileTbl.getParent() != null);
		assertTrue(pnl.screenFields.fileTextArea.getParent() == null);
//		System.out.println(pnl.screenFields.fileTextArea.getParent().getClass());
	}
	
	@Test
	public void testDTAR1000() throws InterruptedException {
		checkDTAR1000("DTAR1000_Store_file_std.bin", IFileStructureConstants.IO_VB);
	}

	
	@Test
	public void testDTAR1000U() throws InterruptedException {
		checkDTAR1000("DTAR1000_Store_file_std_recfm_U.bin", IFileStructureConstants.IO_VB_DUMP);
	}

	private void checkDTAR1000(String fileName, int expectedIo) throws InterruptedException {
		TstPnl1 pnl = new TstPnl1(fileName);
		
		assertTrue(pnl.screenFields.fixedLengthBtn.isSelected());
		assertEquals(expectedIo, pnl.getFileStructure());
//		assertEquals("27", pnl.screenFields.lengthTxt.getText());
		assertEquals("cp037", pnl.screenFields.fontNameCombo.getText());
		assertEquals(60, pnl.screenFields.fileTbl.getColumnCount());

		assertFalse(pnl.screenFields.lengthTxt.isVisible());
		assertTrue(pnl.screenFields.fileTbl.getParent() != null);
		assertTrue(pnl.screenFields.fileTextArea.getParent() == null);
	}
	
	@Test
	public void testAmsLocation() throws InterruptedException {
		TstPnl1 pnl = new TstPnl1("Ams_LocDownload_20041228.txt");
		
		assertTrue(pnl.screenFields.fixedLengthBtn.isSelected());
		assertEquals(IFileStructureConstants.IO_BIN_TEXT, pnl.getFileStructure());
		assertEquals("", pnl.screenFields.fontNameCombo.getText());

		assertFalse(pnl.screenFields.lengthTxt.isVisible());
		assertTrue(pnl.screenFields.fileTbl.getParent() == null);
		assertTrue(pnl.screenFields.fileTextArea.getParent() != null);
	}
	
	
	private static class TstPnl1 {
	    AbsRowList       typeList = new TypeList(0, true, false);
	    AbsRowList  structureList = new ManagerRowList(LineIOProvider.getInstance(), false);
		Pnl1File pnl1File = new Pnl1File(structureList, typeList);
		ZTst_Helper.Pnl1FileScreenFields screenFields = new ZTst_Helper.Pnl1FileScreenFields(pnl1File);
		Details details = new Details();
		
		TstPnl1(String fileName) throws InterruptedException {
			pnl1File.setValues(details);
			
			String file = XCommonCode.getFileName(fileName);
			if (screenFields.filenameFld instanceof FileSelectCombo) {
				((FileSelectCombo) screenFields.filenameFld).setSelectedFile(new File(file), false);
			} else {
				screenFields.filenameFld.setFileName(file);
			}
			
			Thread.sleep(250);	
		}
		
		public int getFileStructure() {
			return (Integer) screenFields.fileStructureCombo.getSelectedItem();
		}
	}
}
