package layoutWizard;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JTextField;

import org.junit.Test;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.IFileStructureConstants;
import net.sf.JRecord.Types.Type;
import net.sf.RecordEditor.layoutWizard.ColumnSelector;
import net.sf.RecordEditor.layoutWizard.Pnl3Table;
import net.sf.RecordEditor.utils.fileAnalysis.ColumnDetails;
import net.sf.RecordEditor.utils.fileAnalysis.Details;
import xCommon.XCommonCode;

public class TstPnl3Table {

	private static final int DTAR020_COLUMN_COUNT = 27;
	private static final int AMS_LOCATION_COLUMN_COUNT = 173;

	private static final byte[] DTAR020_DATA = CommonWizardTstBits.DTAR020_DATA;
	private static final int[][] DTAR020_FIELDS = {
		{1, 8, Type.ftNumRightJustified},
		{9, 2, Type.ftPackedDecimal},
		{11, 4, Type.ftPackedDecimal},
		{15, 2, Type.ftPackedDecimal},
		{17, 5, Type.ftPackedDecimal},
		{22, 6, Type.ftPackedDecimal},
	};

	private static final int[][] AMS_LOCATION_FIELDS = {
			{1, 3, Type.ftChar},
			{4, 4, Type.ftNumRightJustified},
			{8, 37, Type.ftChar},
			{45, 40, Type.ftChar},
			{85, 40, Type.ftChar},
			{125, 35, Type.ftChar},
			{160, 4, Type.ftNumRightJustified},
			{164, 6, Type.ftChar},
			{170, 4, Type.ftChar},
	};

	@Test
	public void tstDTAR020a() throws Exception {
		Details details = getDTAR020details();
		JTextField msgFld = new JTextField();
		
		ColumnSelector cs = new ColumnSelector(msgFld);
		Pnl3Table pnl3Table = new Pnl3Table(cs);
		
		pnl3Table.setValues(details);
		
        assertFalse(cs.textFileChk.isSelected());

        assertTrue(cs.lookComp3.isSelected());
        assertTrue(cs.lookCompBigEndian.isSelected());
        assertFalse(cs.lookCompLittleEndian.isSelected());
        assertTrue(cs.lookMainframeZoned.isSelected());
        assertFalse(cs.lookPcZoned.isSelected());
        
        assertTrue(cs.hexChk.getParent() != null);
        assertTrue(cs.lookComp3.getParent() != null);
        assertTrue(cs.lookCompBigEndian.getParent() != null);
        assertTrue(cs.lookCompLittleEndian.getParent() != null);
        assertTrue(cs.lookMainframeZoned.getParent() != null);
        assertTrue(cs.lookPcZoned.getParent() != null);

        assertTrue(cs.fileTbl.getParent() != null);
        assertTrue(cs.fieldSelectionFld.fieldSelectionTxt.getParent() == null);
        
        assertEquals(379, cs.fileTbl.getRowCount());
        assertEquals(DTAR020_COLUMN_COUNT,  cs.fileTbl.getColumnCount());
        
        for (int i = 0; i < 50; i++) {
        	chkSku(cs, i);
        }
        
        List<int[]> expectedColDetails = Arrays.asList(DTAR020_FIELDS);
        checkColuimns(details, expectedColDetails);
		checkColor(cs, DTAR020_COLUMN_COUNT, expectedColDetails);
		
		System.out.println();
		System.out.println(details.standardRecord.columnDtls.size());
//		MouseListener[] mouseListeners = cs.fileTbl.getMouseListeners();
//		Component component = cs.fileTbl.getColumnModel().getColumn(3).getCellRenderer()
//		  .getTableCellRendererComponent(cs.fileTbl, " ", false, false, 0, 3);
//		//Point mousePosition = component		.getMousePosition();
//		int width = cs.fileTbl.getColumnModel().getColumn(0).getPreferredWidth();
//		MouseEvent mouseEvent = new MouseEvent(cs.fileTbl, 1, System.currentTimeMillis(), 0, 
//				5, width*7/2, 1, false);
//		
//		for (MouseListener listner : mouseListeners) {
//			listner.mousePressed(mouseEvent);
//		}
		
		cs.columnSelected(3);
		
		ArrayList<ColumnDetails> colDtls = details.standardRecord.columnDtls;
		int idx = 0;
		for (int i = 0; i < colDtls.size(); i++) {
			ColumnDetails col = colDtls.get(i);
        	//System.out.println("\t\t{" + col.start + ", " + col.length + ", " + col.type + "},");
			if (i == 1) {
				assertEquals("start\t" + i, 4, col.start);
			} else {
				assertEquals("start\t" + i, expectedColDetails.get(idx++)[0], col.start);
			}
		}
		//System.out.println(details.standardRecord.columnDtls.size());
     }

	private void checkColuimns(Details details, List<int[]> expectedColDetails) {
		ArrayList<ColumnDetails> colDtls = details.standardRecord.columnDtls;
		for (int i = 0; i < colDtls.size(); i++) {
			ColumnDetails col = colDtls.get(i);
        	//System.out.println("\t\t{" + col.start + ", " + col.length + ", " + col.type + "},");
			assertEquals("start\t" + i, expectedColDetails.get(i)[0], col.start);
			assertEquals("length\t" + i, expectedColDetails.get(i)[1], col.length);
			assertEquals("type\t" + i, expectedColDetails.get(i)[2], col.type);
        }
		
		
	}

	private void checkColor(ColumnSelector cs, int colCount, List<int[]> expectedColDetails) {
		int idx = 0;
		boolean expectGrey = true;
		
		//System.out.println();
		for (int i = 0; i < colCount; i++) {
			if (idx < expectedColDetails.size() && expectedColDetails.get(idx)[0] == i+1) {
				expectGrey = ! expectGrey;
				idx += 1;
			}
			boolean grey = cs.fileTbl.getColumnModel().getColumn(i).getCellRenderer()
					  .getTableCellRendererComponent(cs.fileTbl, " ", false, false, 0, i)
					  .getBackground()
					  .equals(Color.LIGHT_GRAY);
			//System.out.print(grey ? "G" : "W");
			assertEquals(i + ", " + idx, expectGrey, grey);
			//System.out.print("\t" + cs.fileTbl.getColumnModel().getColumn(i).getPreferredWidth());
		}
	}

	private void chkSku(ColumnSelector cs, int row) {
		byte[] b = new byte[8];
        
        System.arraycopy(DTAR020_DATA, row * 27, b, 0, 8);
        
        String s = Conversion.toString(b, "cp037");
        for (int i = 0; i < s.length(); i++) {
        	assertEquals(""+ i, s.substring(i,i+1), cs.fileTbl.getValueAt(row, i));
        }
	}

	
	private Details getDTAR020details() {
		Details details = new Details();
		details.filename = XCommonCode.getFileName(CommonWizardTstBits.DTAR020_FILE);
	;	details.fileStructure = IFileStructureConstants.IO_FIXED_LENGTH;
		details.fontName = "CP037";
		details.recordLength = 27;
		details.normalCharPercent = 40;
		details.textPercent = 70;
		details.fileData = CommonWizardTstBits.DTAR020_DATA;
		
		return details;
	}

	@Test
	public void tstAmsLocation() throws Exception {
		Details details = getAmsLocationdetails();
		
		JTextField msgFld = new JTextField();
		
		ColumnSelector cs = new ColumnSelector(msgFld);
		Pnl3Table pnl3Table = new Pnl3Table(cs);
		
		pnl3Table.setValues(details);
		
        assertTrue(cs.textFileChk.isSelected());
        
//        assertTrue(cs.lookMainframeZoned.isSelected());
//        assertFalse(cs.lookPcZoned.isSelected());
 
        assertTrue(cs.lookComp3.getParent() == null);
        assertTrue(cs.lookCompBigEndian.getParent() == null);
        assertTrue(cs.lookCompLittleEndian.getParent() == null);
        assertTrue(cs.lookMainframeZoned.getParent() != null);
        assertTrue(cs.lookPcZoned.getParent() != null);
        

        assertTrue(cs.fileTbl.getParent() == null);
        assertTrue(cs.fieldSelectionFld.fieldSelectionTxt.getParent() != null);
        
//        System.out.println(cs.fileTbl.getRowCount() + " " + cs.fileTbl.getColumnCount() );
        assertEquals(174, cs.fileTbl.getRowCount());
        assertEquals(AMS_LOCATION_COLUMN_COUNT,  cs.fileTbl.getColumnCount());
        
//		ArrayList<ColumnDetails> colDtls = details.standardRecord.columnDtls;
//		for (int i = 0; i < colDtls.size(); i++) {
//			ColumnDetails col = colDtls.get(i);
//        	System.out.println("\t\t{" + col.start + ", " + col.length + ", " + col.type + "},");
//		}
        List<int[]> expectedColDetails = Arrays.asList(AMS_LOCATION_FIELDS);
        checkColuimns(details, expectedColDetails);

	}
		

	private Details getAmsLocationdetails() {
		Details details = new Details();
		details.filename = XCommonCode.getFileName(CommonWizardTstBits.AMS_LOCATION_FILE);
		details.fileStructure = IFileStructureConstants.IO_BIN_TEXT;
		details.fontName = Conversion.DEFAULT_ASCII_CHARSET;
		details.recordLength = 27;
		details.normalCharPercent = 100;
		details.textPercent = 100;
		details.fileData = CommonWizardTstBits.AMS_LOCATION_DATA;
		
		return details;
	}

}
