package net.sf.RecordEditor.test.diff;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.util.ArrayList;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.RecordEditor.diff.DoCompare;
import net.sf.RecordEditor.diff.Zold_DoCompare;
import net.sf.RecordEditor.diff.result.CompareSummary;
import net.sf.RecordEditor.diff.result.ICompareResult;
import net.sf.RecordEditor.diff.result.ICompareSummary;
import net.sf.RecordEditor.diff.result.ILineCompare;
import net.sf.RecordEditor.diff.result.VisitorSummary;
import net.sf.RecordEditor.diff.tbl.CmpTableModel;
import xCommon.XCommonCode;
import xCommon.XSchemas;

public class CommonTstVisitorSummary { 

	public static enum TestType {
		DTAR020("DTAR020.bin", "DTAR020_CmpZ", ".bin" ),
		DTAR020_CmpZ3("DTAR020_CmpZ3.bin", "DTAR020_CmpZ", ".bin" ),
		AMS_PO("Ams_PODownload_20041231.txt", "Ams_PODownload_20041231_cmpZ", ".txt" ),
		
		;
		
		private final String file1, file2a, file2b;
		private TestType(String file1, String file2a, String file2b) {
			this.file1 = file1;
			this.file2a = file2a;
			this.file2b = file2b;
		}
		
		
	}

	private final boolean normalFileFirst;
	private final String file1, file2;
	private final TestType testType;
	
	
	public CommonTstVisitorSummary(String tstId) {
		this(true, tstId);
	}

	public CommonTstVisitorSummary(boolean normalFileFirst, String tstId) {
		this(TestType.DTAR020, normalFileFirst, tstId);
	}

	
	public CommonTstVisitorSummary(TestType testType, boolean normalFileFirst, String tstId) {
		super();
		this.normalFileFirst = normalFileFirst;
		this.testType = testType;
		
		String f1 = testType.file1;
		String f2 = testType.file2a + tstId + testType.file2b;
		if (! normalFileFirst) {
			f1 = f2;
			f2 = testType.file1;
		}
		file1 = f1;
		file2 = f2;
	}

	
	
	public void tstResult(ICompareResult cmp, String[] expected, int maxSize) {
		expected = updateExpected(expected);
		
		int rowCount = Math.min(maxSize, cmp.getRowCount());
		if (expected == null) {
			System.out.println("\n");
			for (int i = 0; i < rowCount; i++) {
				System.out.println("\t\"" + combine(cmp.get(true, i), "\\t") + "\",");
				System.out.println("\t\"" + combine(cmp.get(false, i), "\\t") + "\",");
			}
		} else {
			for (int i = 0; i < rowCount; i++) {
				cmpOneRow(i, cmp, expected);
			}
			for (int i = rowCount-1; i >=0; i--) {
				cmpOneRow(i, cmp, expected);
			}
			for (int i = 0; i <= rowCount/2; i++) {
				cmpOneRow(i, cmp, expected);
				cmpOneRow(rowCount-i-1, cmp, expected);
			}
			for (int i = rowCount/2; i >=0; i--) {
				cmpOneRow(i, cmp, expected);
				cmpOneRow(rowCount-i-1, cmp, expected);
			}
		}
	}
	
	private String[] updateExpected(String[] expected) {
		if (normalFileFirst || expected == null) { return expected;}
		
		String[] updated = new String[expected.length];
		for (int i = 0; i < expected.length; i += 2) {
			updated[i] = Conversion.replace(expected[i+1], "INSERT", "DELETED").toString();
			updated[i+1] = Conversion.replace(expected[i], "DELETED", "INSERT").toString();
		}
		
		return updated;
	}

	/**
	 * @param i
	 * @param cmp
	 * @param expected
	 */
	private void cmpOneRow(int i, ICompareResult cmp, String[] expected) {
		String msg = i + "";
		
		if (! expected[i*2].equals(combine(cmp.get(true,  i), "\t"))) {
			assertEquals(msg, expected[i*2],   combine(cmp.get(true,  i), "\t"));
		}
		if (! expected[i*2+1].equals(combine(cmp.get(false,  i), "\t"))) {
			assertEquals(msg, expected[i*2+1], combine(cmp.get(false, i), "\t"));
		}
	}

/*
 * 
 * 			"11\tCHANGED\t66624458\t20\t40118\t957\t1\t0.89",
			"11\tCHANGED\t66624458\t20\t40118\t957\t1\t1.89",
			"17\tDELETED\t61664713\t59\t40118\t335\t-1\t-17.99",
			"",

		Old	2	69684558	20	40118	280	-1	-19.00
		New	2		201				
		Old	5	69694158	20	40118	280	-1	-19.00
		New	5			40120			
		Old	8	62684671	20	40118	685	1	69.99
		New	8					11	
		Old	11	66624458	20	40118	957	1	0.89
		New	11						1.89
		Deleted	17	61664713	59	40118	335	-1	-17.99
									
		Deleted	18	68634752	59	40118	410	1	8.99
									
		Deleted	19	60614487	59	40118	878	1	5.95
									
									
		Inserted	28	61664713	59	40118	335	-1	-17.99
									
		Inserted	29	68634752	59	40118	410	1	8.99
									

 */
	/**
	 * @param tbl
	 */
	public void tstTable(CmpTableModel tbl, String[] expected) {
		//int rNum = tbl.getRowCount();
		int cNum = tbl.getColumnCount();

		expected = updateExpected(expected);

		//assertEquals(expected.length, tbl.getRowCount());
		for (int row = 0; row < expected.length; row+=2) {
			String[] beforeFlds = expected[row].split("\t");

			String[] beforeCmpFlds = formatTblCmp(null, beforeFlds, "".equals(expected[row+1]));
			String[] afterCmpFlds = formatTblCmp(beforeFlds, expected[row+1].split("\t"), "".equals(expected[row]));
			for (int col = 0; col < cNum; col++) {
				String message = row + ", " + col + " : " + expected[row] ;
				if (row == 762) {
					System.out.print('.');
				}
				assertEquals(message, beforeCmpFlds.length > col ? beforeCmpFlds[col] : "", tbl.getValueAt(row, col+1).toString());
				assertEquals(message,  afterCmpFlds.length > col ? afterCmpFlds[col] : "", tbl.getValueAt(row+1, col+1).toString());
			}

//			System.out.println();
//			System.out.println();
//			System.out.println(" >>> " + rNum + " " + cNum);
//			for (int col = 0; col < cNum; col++) {
//				System.out.print("\t" + tbl.getValueAt(row, col));
//			}
//			System.out.println();
		}
	}
	
	private final String[] emptyArray = {};
	private String[] formatTblCmp(String[] prev, String[] curr, boolean only) {
		System.out.print("\t" + curr.length);
		if (curr.length == 0 || (curr.length == 1) && curr[0].length() == 0) {
			return emptyArray;
		}
		ArrayList<String> flds = new ArrayList<String>();
		
		if (prev == null) {
			flds.add(only ? "Deleted" : "Old");
		} else {
			flds.add(only ? "Inserted" : "New");
		}
		
		int[] idxs = {0, 2, 3, 4, 5, 6, 7};
		
		for (int i : idxs) {
			flds.add( 
					i < curr.length
				&&	(  prev == null || i== 0
					|| i >= prev.length
					|| ! curr[i].equals(prev[i]))
						? curr[i] : "");
		}
		return flds.toArray(new String[flds.size()]);
	}
	
	private String combine(ILineCompare lineCmp, String sep) {
		if (lineCmp == null) { return ""; }
		
		StringBuilder b = new StringBuilder();
		AbstractLine line = lineCmp.getLine();
		
		b.append(lineCmp.getLineNumber()).append(sep).append(lineCmp.getCompareResult());
		if (line != null) {
			int count = line.getLayout().getRecord(0).getFieldCount();
			
			for (int i = 0; i < count; i++) {
				b.append(sep).append(line.getFieldValue(0, i).asString());
			}
		}
		
		return b.toString();
	}
	
	public ICompareSummary getSummary() throws IOException {
		return new VisitorSummary(
						Zold_DoCompare.getInstance().compare1LayoutVistor(
								getSchema(), 
								XCommonCode.getFileName(file1),
								XCommonCode.getFileName(file2)));
	}



	
	
	public ICompareSummary getCompareSummary() throws IOException {
		return	DoCompare.getInstance().compare1LayoutVistor(
								getSchema(), 
								XCommonCode.getFileName(file1),
								XCommonCode.getFileName(file2));
	}



	

	/**
	 * @return
	 * @throws IOException
	 */
	private LayoutDetail getSchema() throws IOException {
		return testType == TestType.DTAR020 || testType == TestType.DTAR020_CmpZ3
			? XSchemas.getDtar020()
			: XCommonCode.getAmsPoSchema();
	}

}
