package net.sf.RecordEditor.test.diff;

import static org.junit.Assert.*;

import java.io.ByteArrayInputStream;
import java.io.IOException;

import org.junit.Test;

import net.sf.JRecord.Common.Conversion;
import net.sf.JRecord.Common.RecordException;
import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.Details.RecordDetail;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.RecordEditor.diff.LineBufferedReader;
import net.sf.RecordEditor.edit.util.StandardLayouts;
import xCommon.XCommonCode;
import xCommon.XSchemas;
import xCommon.XStdSchemas;

public class TstLineBufferedReader {

	
	@Test
	public void testFixedWidth() throws IOException {
		tst(XSchemas.getDtar020(), XCommonCode.readTestDataFile("DTAR020.bin"));
	}
	
	
	@Test
	public void testVb() throws IOException {
		tst(XStdSchemas.DTAR1000.toLayout(), XCommonCode.readTestDataFile("DTAR1000_Store_file_std.bin"));
	}

	@Test
	public void testPo() throws IOException {
		tst(XStdSchemas.GETTEXT_PO.toLayout(), XCommonCode.readTestDataFile("TstDupFilter01.po"));
	}
	
	@Test
	public void testTip() throws IOException {
		tst(XStdSchemas.TIP_DETAILS.toLayout(), XCommonCode.readTestDataFile("TipOfTheDay.properties"));
	}

	@Test
	public void testCsv() throws IOException {
		
		byte[] dtar020Data = XCommonCode.readTestDataFile("DTAR020.bin");
		tstCsvFile(",",  "", dtar020Data);
		tstCsvFile(";",  "utf-8", dtar020Data);
		tstCsvFile("\t", "utf-16", dtar020Data);

	}

	/**
	 * @param delim
	 * @param charset
	 * @throws IOException
	 */
	private void tstCsvFile(String delim, String charset, byte[] dtar020Data) throws IOException {
		LayoutDetail dtar020 = XSchemas.getDtar020();
		AbstractLineReader lineReader = getReader(dtar020, dtar020Data);
		AbstractLine line;
		int recNum = 0;
		RecordDetail record = dtar020.getRecord(recNum);
		int fieldCount = record.getFieldCount();
		String sep = "";
		StringBuilder b  = new StringBuilder();
		for (int i = 0; i < fieldCount; i++) {
			b.append(sep).append(record.getField(i).getName());
			sep = delim;
		}
		
		while ((line = lineReader.read()) != null) {
			b.append('\n');
			
			sep = "";
			for (int i = 0; i < fieldCount; i++) {
				b.append(sep).append(getField(line, recNum, i));
				sep = delim;
			}
		}
		
		lineReader.close();
		
		LayoutDetail csvLayout = StandardLayouts.getInstance().getCsvLayoutNamesFirstLine(delim, charset, "\"", false);
		tst(csvLayout, Conversion.getBytes(b.toString(),charset));
	}
	
	private void tst(LayoutDetail l, byte[] data) throws RecordException, IOException {
		AbstractLineReader lineReader = getReader(l, data);
		LineBufferedReader br = new LineBufferedReader(new ByteArrayInputStream(data), l, l, null, true);
		String s;
		AbstractLine line;
		
		int lineNum = 0;
		int recNum = 0;
		while ((line = lineReader.read()) != null && (s = br.readLine()) != null) {
			lineNum += 1;
			
			StringBuilder b = new StringBuilder();
			
			int fieldCount = lineReader.getLayout().getRecord(recNum).getFieldCount();
			for (int i = 0; i < fieldCount; i++) {
				b.append(getField(line, recNum, i)).append('\t');
			}
			assertEquals("line: " + lineNum, b.toString(), s);
		}
		
		String msg = "Line Number: " + lineNum;
		System.out.println(msg);
		assertTrue(msg, lineNum > 1);
		br.close();
		lineReader.close();
	}

	/**
	 * @param l
	 * @return
	 * @throws IOException 
	 * @throws RecordException 
	 */
	private AbstractLineReader getReader(LayoutDetail l, byte[] data) throws RecordException, IOException {
		AbstractLineReader lineReader = LineIOProvider.getInstance().getLineReader(l);
		lineReader.open(new ByteArrayInputStream(data), l);
		return lineReader;
	}

	private String getField(AbstractLine line, int recordNo, int fldNo) {
		Object obj = line.getField(recordNo, fldNo);
		if (obj == null) {
			obj = "";
		}
		String s = obj.toString();
		if (s.indexOf('\t') >= 0 || s.indexOf('\n') >= 0) {
			StringBuilder b = Conversion.replace(new StringBuilder(s), "\\", "\\\\");
			Conversion.replace(b, "\n", "\\n");
			Conversion.replace(b, "\t", "\\t");
			s = b.toString();
		}
		
		return s;
	}

}
