package net.sf.RecordEditor.test.diff;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLStreamException;

import net.sf.JRecord.Details.AbstractLayoutDetails;
import net.sf.RecordEditor.diff.Zold_DoCompare;
import net.sf.RecordEditor.diff.DoCompare;
import net.sf.RecordEditor.re.layoutSelection.def.ISchemaProvider;
import net.sf.RecordEditor.xml.classes.DiffDefinition;
import net.sf.RecordEditor.xml.classes.ReadXml;
import xCommon.XCommonCode;
import xCommon.XSchemas;

public class WriteHtml {

	
	private DiffDefinition createCompareDef(String fileName, boolean allRows, boolean allFields, boolean singleTable)
	throws XMLStreamException, FactoryConfigurationError {
		StringBuilder b = new StringBuilder();
		
		//DTAR020_CmpZ
		b.append("<?xml version=\"1.0\" ?>\n"
			+ "<diff type=\"SingleLayout\" complete=\"YES\">\n"
			+ "    <stripTrailingSpaces>true</stripTrailingSpaces>\n"
			+ "    <layout name=\"DTAR020\" saveFile=\"\" groupHeader=\"\">\n");
		
		if (! allFields) {
			b.append( "        <record name=\"DTAR020\">\n"
			+ "            <field>KEYCODE-NO</field>\n"
			+ "            <field>STORE-NO</field>\n"
			+ "            <field>QTY-SOLD</field>\n"
			+ "            <field>SALE-PRICE</field>\n"
			+ "        </record>\n");
		}
		b.append( "    </layout>\n"
			+ "    <oldfile>\n"
			+ "        <file name=\"");
		b.append(XCommonCode.getFileName("DTAR020.bin"));
		b.append( "\"></file>\n"
			+ "    </oldfile>\n"
			+ "    <newfile>\n"
			+ "        <file name=\"");
		b.append(XCommonCode.getFileName(fileName +".bin"));
		b.append( "\"></file>\n"
			+ "    </newfile>\n"
			+ "    <HtmlOptions HtmlFile=\"/home/bruce/work/temp/CompareOut/cmp/")
				.append(fileName)
				.append('_')
				.append(toChar(allRows))
				.append(toChar(allFields))
				.append(toChar(singleTable))
				.append(".html");
		
		b.append( "\" AllRows=\"");
		b.append(allRows).append("\" AllFields=\"");
		b.append(allFields).append( "\" SingleTable=\"");
		b.append(singleTable).append( "\"></HtmlOptions>\n</diff>");

		return new ReadXml(new StringReader(b.toString())).readDiffDefinition();
	}
	
	char toChar(boolean bool) {
		return bool ? 'Y' : 'N';
	}
	
	static class GetLayout implements ISchemaProvider {
		final AbstractLayoutDetails dtar020;
		
		public GetLayout() throws IOException {
			super();
			this.dtar020 = XSchemas.getDtar020();
		}
		@Override
		public AbstractLayoutDetails getRecordLayout(String name, String fileName) {
			return dtar020;
		}
	}
	
	 public static void main(String[] args) throws XMLStreamException, FactoryConfigurationError, Exception {
		 GetLayout gl = new GetLayout();
		 WriteHtml wh = new WriteHtml();
//		 DoCompare doCompare = DoCompare.getInstance();
		 DoCompare doCompare = DoCompare.getInstance();
		 
		 for (int i = 0; i< 8; i++) {
			 boolean allRows = (i >> 2) % 2  == 0;
			 boolean allFields = (i >> 1) % 2  == 0;
			 boolean singleTable = i % 2 == 0;
			 doCompare.writeHtml(
					 gl, gl, 
					 wh.createCompareDef("DTAR020_CmpZ", allRows, allFields, singleTable));
			 doCompare.writeHtml(
					 gl, gl, 
					 wh.createCompareDef("DTAR020_CmpZ3", allRows, allFields, singleTable));
		 }
	 }
	
	
}
