package net.sf.RecordEditor.test.file1;

import net.sf.JRecord.Details.AbstractLine;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.RecordEditor.re.file.FileView;
import net.sf.RecordEditor.utils.fileStorage.DataStoreLarge;
import net.sf.RecordEditor.utils.fileStorage.DataStoreLargeView;
import net.sf.RecordEditor.utils.fileStorage.DataStoreStd;
import net.sf.RecordEditor.utils.fileStorage.DataStoreWindow;
import net.sf.RecordEditor.utils.fileStorage.FileDetails;
import net.sf.RecordEditor.utils.fileStorage.IDataStore;

/**
 * Defines and holds a Base-File and a File View.
 * It is used in testing
 * 
 * @author Bruce
 *
 */
public class FileAndView {

	public static final int STD_VIEW = 1;
	public static final int LARGE_VIEW = 2;
	public static final int WINDOW_VIEW = 3;
	
	
	public final IDataStore<AbstractLine> parentStore;
	public final FileView parentFile;
	public final IDataStore<AbstractLine> viewStore;
	public final FileView viewFile;
	
	public FileAndView(LayoutDetail schema, boolean normalFile, boolean normalView) {
		this(schema, normalFile, normalView ? STD_VIEW : LARGE_VIEW);
	}
	
	public FileAndView(LayoutDetail schema, boolean normalFile, int viewType) {

		parentStore = getStore(schema, normalFile);
		parentFile =  new FileView(parentStore, null, null);

		switch (viewType) {
		case STD_VIEW: 		viewStore = DataStoreStd.newStore(schema);			break;
		case LARGE_VIEW:	viewStore = new DataStoreLargeView(parentStore);	break;
		case WINDOW_VIEW:	viewStore = new DataStoreWindow<AbstractLine>(parentStore, 0, -1, false);	break;
		default:
			throw new RuntimeException("Invalid view type: " + viewType);
		}

		viewFile =  new FileView(viewStore, parentFile, null);
	}

	
	public FileAndView(LayoutDetail schema, int viewType, int firstRow, int lastRow) {

		parentStore = getStore(schema, true);
		parentFile =  new FileView(parentStore, null, null);

		switch (viewType) {
		case STD_VIEW: 		viewStore = DataStoreStd.newStore(schema);			break;
		case LARGE_VIEW:	viewStore = new DataStoreLargeView(parentStore);	break;
		case WINDOW_VIEW:	viewStore = new DataStoreWindow<AbstractLine>(parentStore, firstRow, lastRow, false);	break;
		default:
			throw new RuntimeException("Invalid view type: " + viewType);
		}

		viewFile =  new FileView(viewStore, parentFile, null);
	}

	
	private IDataStore<AbstractLine> getStore(LayoutDetail schema, boolean normal) {
		if (normal) {
			return DataStoreStd.newStore(schema);
		} else {
			return new DataStoreLarge(schema, FileDetails.FIXED_LENGTH, schema.getMaximumRecordLength());
		}
	}
}
