package com.zbluesoftware.java.bm;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class ZDateFormat implements IZDateFormat {

	private final SimpleDateFormat df;
    private final String dateFormatStr;
    
	public ZDateFormat(String dateFormatStr) {
		super();
		this.dateFormatStr = dateFormatStr;
		df = new SimpleDateFormat(dateFormatStr);
	}

	
	/* (non-Javadoc)
	 * @see com.zbluesoftware.java.bm.IZDateFormat#parse(java.lang.String)
	 */
	@Override
	public Date parse(String dateStr) throws ParseException {
		return df.parse(padZeros(dateFormatStr, dateStr));
	}
	
	/* (non-Javadoc)
	 * @see com.zbluesoftware.java.bm.IZDateFormat#format(java.util.Date)
	 */
	@Override
	public final String format(Date date) {
		return df.format(date);
	}


	/**
     * pad string with zero's to format length
     *
     * @param dateFormatStr date format string
     * @param s string to be padded
     *
     * @return padded string
     */
    public static String padZeros(final String dateFormatStr, final String s) {

        String ret = s;
        if (dateFormatStr != null && s.length() < dateFormatStr.length()
        && (dateFormatStr.startsWith("y") || dateFormatStr.startsWith("M")
        || dateFormatStr.startsWith("d"))) {
            ret = "00000000".substring(0, Math.min(8, dateFormatStr.length() - s.length()))
                + s;
        }
        return ret;
    }

}
