package net.sf.recordEditor.ui.common;

import java.awt.GridLayout;
import java.awt.event.ActionListener;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;

import net.sf.recordEditor.ui.buttons.ArrowButton;
import net.sf.recordEditor.ui.def.common.IListPopupActions;
import net.sf.recordEditor.ui.fileFields.popup.EditFavouriteAction;
import net.sf.recordEditor.ui.icons.IconManager;
import net.sf.recordEditor.ui.listManagers.IBasicList;
import net.sf.recordEditor.ui.listManagers.IRecentList;
import net.sf.recordEditor.ui.listManagers.IUpdateableList;
import net.sf.recordEditor.ui.styles.StyleManager;

public class DropDownFavouriteBtns<What> {

	public final JButton    recentBtn;
	public final JButton  favoriteBtn;
	
	public final MultiComponentBuilder componentBuilder;
	
	public final String typeTxt;
	
	
	public DropDownFavouriteBtns(
			MultiComponentBuilder componentBuilder,
			JComponent owner,
			IRecentList recentList, IUpdateableList favouriteList,
			IListActionBuilder actionBldr, Action addToFavourite) {
		this(   componentBuilder, owner, 
				recentList == null	? favouriteList == null ? "" 
															: " " + favouriteList.getListName()
									: " " + recentList.getListName(), 
				recentList, favouriteList, actionBldr, addToFavourite);
	}
	
	public DropDownFavouriteBtns(
			MultiComponentBuilder componentBuilder,
			JComponent owner, String  typeTxt,
			IRecentList recentList, IUpdateableList favouriteList,
			IListActionBuilder actionBldr, Action addToFavourite) {

		this.componentBuilder = componentBuilder;
		this.typeTxt = typeTxt;
		
		JButton    tRecentBtn = null;
		JButton  tFavoriteBtn = null;
		
        if (recentList == null && favouriteList == null) {
        	
        } else if (recentList != null && favouriteList != null) {       	
        	tRecentBtn = generateRecentBtn(owner, recentList, actionBldr, true);
        	tFavoriteBtn = generateFavoriteBtn(owner, favouriteList, actionBldr, addToFavourite);
//        	System.out.println("--> " + owner.getPreferredSize().height + " " + recentBtn.getPreferredSize().height
//        			+ " " + favoriteBtn.getPreferredSize().height + " " + favourite.getIconHeight());
        } else if (recentList != null) {
        	tRecentBtn = generateRecentBtn(owner, recentList, actionBldr, false);
         } else {
        	tFavoriteBtn = generateFavoriteBtn(owner, favouriteList, actionBldr, addToFavourite);
        }
        
        this.recentBtn   = tRecentBtn;
        this.favoriteBtn = tFavoriteBtn;
	}

	/**
	 * @param favoriteAction
	 * @return
	 */
	public JButton generateFavoriteBtn(JComponent owner, IUpdateableList favList, 
			IListActionBuilder actionBldr, Action addToFavourite) {
		StdComponents.UiButton favoriteButton = new StdComponents.UiButton(
				StyleManager.styles().comboFavorite(), IconManager.ICONS.favourite.icon());
			
		JPanel pnl = null;
		if (addToFavourite != null && favList != null && favList.allowUpdates()) {
			pnl = new JPanel(new GridLayout(2, 1));
			pnl.add(new JButton(addToFavourite));
			
			pnl.add(new JButton(new EditFavouriteAction(owner, favList)));
		}

//		favoriteButton.getMainComponent().addActionListener(
//				new FileListAction(favList, favList, favoriteButton.getUiContainer(), owner, pnl, favoriteAction));
					
		favoriteButton.getMainComponent().addActionListener(
				actionBldr.createAction(favList, favList, favoriteButton.getUiContainer(), owner, pnl));
		//favoriteButton.setToolTip(msg);

		if (componentBuilder != null) {
			componentBuilder.add(favoriteButton.getMainComponent());
		}
		return favoriteButton.getMainComponent();
	}

	/**
	 * @param recentAction
	 * @return
	 */
	public JButton generateRecentBtn(JComponent owner, IRecentList recentList, 
			IListActionBuilder actionBldr, 
			boolean both) {
		
		ArrowButton recentButton = new ArrowButton(StyleManager.styles().comboArrow(), ShapePainters.DOWN);
		String msg = "List recent " + typeTxt;

		
//		recentButton.addActionListener(
//				new FileListAction(recentList, recentButton.getUiContainer(), recentUpd));
		
		recentButton.addActionListener(actionBldr.createAction(recentList, null, recentButton.getUiContainer(), owner, null));
		
		recentButton.setToolTipText(msg);
	
		if (componentBuilder != null) {
			componentBuilder.add(recentButton.getUiContainer());
		}
		


		return recentButton;
	}
	
	public void setButtonNames(String name) {
		setNameOfBtn(favoriteBtn, name + "_Fav");
		setNameOfBtn(recentBtn, name + "_Recent");
	}
	
    
	private void setNameOfBtn(JButton btn, String name) {
		if (btn != null) {
			btn.setName(name);
		}
	}
	
	public static interface IListActionBuilder {
		ActionListener createAction(IBasicList stdList, IUpdateableList updList, 
			JComponent displayItm, JComponent container, 
			JPanel favouritePnl);
		
	}
//	
//	private static class StringListActionBuilder implements IListActionBuilder {
//		final IListPopupActions<String> listner;
//		StringListActionBuilder(IListPopupActions<String> listner) {
//			this.listner = listner;
//			
//		}
//		@Override
//		public ActionListener createAction(IBasicList stdList, IUpdateableList updList, JComponent displayItm,
//				JComponent container, JPanel favouritePnl) {
//			return 
//		}
//		
//		
//	}
}
