package net.sf.recordEditor.ui.fileFields;

import java.awt.event.ActionListener;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;

import net.sf.recordEditor.ui.common.DropDownFavouriteBtns;
import net.sf.recordEditor.ui.common.DropDownFavouriteBtns.IListActionBuilder;
import net.sf.recordEditor.ui.common.AbstractListAction;
import net.sf.recordEditor.ui.common.MultiComponentBuilder;
import net.sf.recordEditor.ui.common.UiSelectionList;
import net.sf.recordEditor.ui.def.common.IListPopupActions;
import net.sf.recordEditor.ui.listManagers.IBasicList;
import net.sf.recordEditor.ui.listManagers.IListChangeListner;
import net.sf.recordEditor.ui.listManagers.IListItem;
import net.sf.recordEditor.ui.listManagers.IRecentList;
import net.sf.recordEditor.ui.listManagers.IUpdateableList;
import net.sf.recordEditor.ui.styles.StyleManager;

public class FileListAction extends AbstractListAction<Path> implements ActionListener, IListChangeListner {

	public static DropDownFavouriteBtns<Path> createDropDown(MultiComponentBuilder componentBuilder,
			JComponent owner,
			IRecentList recentList, IUpdateableList favouriteList,
			IListPopupActions<Path> listner, Action addToFavourite) {
		String typeTxt = recentList == null	
					? favouriteList == null 
							? "" 
							: " " + favouriteList.getListName()
					: " " + recentList.getListName();
		return new DropDownFavouriteBtns<Path>(
				componentBuilder, owner, typeTxt, recentList, favouriteList, 
				new ListActionBldr(listner), 
				addToFavourite);
	}

//	private final JComponent  container;
	private final IListPopupActions<Path> listner;
	private final JPanel favouritePnl;
	

	
	public FileListAction(IBasicList updList,  
			JComponent displayItm, JComponent container, 
			IListPopupActions<Path> listner) {
		this(updList, null, displayItm, container, null, listner);
	}

	public FileListAction(IBasicList stdList, IUpdateableList updList, 
			JComponent displayItm, JComponent container, 
			JPanel favouritePnl,
			IListPopupActions<Path> listner) {
		super(stdList, updList, displayItm, container);
		this.listner = listner;
		this.favouritePnl = favouritePnl;
		
	}


	 
	@Override
	protected UiSelectionList<Path> createList() {
		return new UiFileSelectionList(StyleManager.EMPTY_STYLE, container, true, favouritePnl, listner);
	}

	protected List<Path> loadPopup(List<IListItem> itemList) {
		List<Path> paths = new ArrayList<Path>(itemList.size());
		
		for (IListItem itm : itemList) {
			paths.add(Paths.get(itm.getItemText()));
		}
		
		return paths;
	}

	private static class ListActionBldr implements IListActionBuilder {
		private final IListPopupActions<Path> listner;
		
		public ListActionBldr(IListPopupActions<Path> listner) {
			this.listner = listner;
		}

		@Override
		public ActionListener createAction(IBasicList stdList, IUpdateableList updList, JComponent displayItm,
				JComponent container, JPanel favouritePnl) {
			return new FileListAction(stdList, updList, displayItm, container, favouritePnl, listner);
		}
		
	}
}
