package net.sf.recordEditor.ui.fileTree;

import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.nio.file.Path;
import java.util.ArrayList;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreePath;

import net.sf.recordEditor.ui.def.component.IUiTree;
import net.sf.recordEditor.ui.def.file.IDirectoryChangedListner;
import net.sf.recordEditor.ui.icons.IconManager;
import net.sf.recordEditor.ui.styles.ITreeStyle;
import net.sf.recordEditor.ui.styles.StyleManager;


/**
 * Displays File system in a Tree Structure
 * 
 * @author: Bruce Martin
 *
 */
public class UiFileTree implements IUiTree {

	private final ITreeStyle style;
	private final FileTreeModel treeMdl = FileTreeModel.newModel();
	private final JTree tree = new JTree(treeMdl);
	@SuppressWarnings("serial")
	private final DefaultTreeCellRenderer render = new DefaultTreeCellRenderer() {

		@Override
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded,
				boolean leaf, int row, boolean hasFocus) {
			
			Component r = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
			
			if (value instanceof FileNode && ((FileNode) value).parent == treeMdl.getRoot()) {
				super.setIcon(IconManager.ICONS.drive.icon());				
			}
			return r;
		}
		
	};

	
	private JLabel fieldsLabel;
	
	private ArrayList<IDirectoryChangedListner> listners = new ArrayList<IDirectoryChangedListner>(4);

	public UiFileTree() {
		this(StyleManager.styles().folderTree());
	}
	
	public UiFileTree(ITreeStyle style) {
		super();
		this.style = style;
		
		Icon folderIcon = IconManager.ICONS.folderIcon();
		render.setLeafIcon(folderIcon);
		render.setClosedIcon(folderIcon);
		render.setOpenIcon(folderIcon);
		
		style.register(this);
		style.applyTree(this);
		
		tree.setCellRenderer(render);
		
//		TreeUI ui = tree.getUI();
//		if (ui instanceof BasicTreeUI) {
//			int height = SwingUtils.values().stdIconHeight();
//			int width = (height + 1) / 2;
//			BasicTreeUI treeui = (BasicTreeUI) ui;
//			treeui.setExpandedIcon(new TriangleIcon(ShapePainters.DOWN, Color.GRAY)); //, width, height));
//			treeui.setCollapsedIcon(new TriangleIcon(ShapePainters.RIGHT, Color.GRAY)); //, width, height));
//	   }
		
		tree.addTreeSelectionListener(new TreeSelectionListener() {		
			@Override public void valueChanged(TreeSelectionEvent e) {
				TreePath treePath = e.getNewLeadSelectionPath();
				Path filePath = ((FileNode) treePath.getLastPathComponent()).getFilePath();
				File file = filePath.toFile();
				
				for (IDirectoryChangedListner l : listners) {
					l.directoryChanged(file);
				}
			}
		});
		tree.setRootVisible(false);
	}


	@Override
	public ITreeStyle getStyleId() {
		return style;
	}


	@Override
	public JComponent getUiContainer() {
		return tree;
	}

	@Override
	public JTree getMainComponent() {
		return tree;
	}

	@Override
	public JLabel getFieldsLabel() {
		return fieldsLabel;
	}

	@Override
	public void setFieldsLabel(JLabel label) {
		this.fieldsLabel = label;
	}

	public void showPath(File f) {
		FileNode treePath = treeMdl.getTreePath(f);
		if (treePath != null) {
			tree.scrollPathToVisible(treePath.treePath());
		}
	}
	public void setSelectedPath(File f) {
		FileNode treeNode = treeMdl.getTreePath(f);
		if (treeNode != null) {
			TreePath treePath = treeNode.treePath();
			tree.scrollPathToVisible(treePath);
			tree.setSelectionPath(treePath);
		}
	} 
	
	public void addDirectoryChangedListner(IDirectoryChangedListner l) {
		listners.add(l);
	}

	public static void main(String[] args) {
		UiFileTree fileTree = new UiFileTree();
		JFrame frame = new JFrame();
		
		//tree.path
		//tree.setCellRenderer(x);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		JScrollPane comp = new JScrollPane(fileTree.getMainComponent());
		frame.getContentPane().add(comp);
		
		comp.setPreferredSize(new Dimension(600, 800));
		
		frame.pack();
		
		//FileNode treePath = mdl.getTreePath(new File("/home/bruce/work/"));
		
		fileTree.showPath(new File("/home/bruce/work/cb2xml/source/cb2xml/src/net/sf/cb2xml"));
		
		frame.setVisible(true);
	}

}
