package net.sf.recordEditor.ui.standard.component;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.text.JTextComponent;

import net.sf.recordEditor.ui.buttons.ArrowButton;
import net.sf.recordEditor.ui.common.MultiComponentBuilder;
import net.sf.recordEditor.ui.common.ShapePainters;
import net.sf.recordEditor.ui.styles.StyleManager;

@SuppressWarnings("serial")
public class ComboLikeObject extends BasicPanel implements ActionListener {

	private static final int FIELD_WIDTH = 20;

	protected final JTextComponent valueTxt;

	protected final ArrowButton listBtn = new ArrowButton(StyleManager.styles().comboArrow(), ShapePainters.DOWN);
	//protected final JButton showListBtn = new ArrowButton(ArrowButton.SOUTH);

	private final JComponent[] buttons;

	private int popupHeight = -1;
	//protected boolean visible = false;
	
	private boolean notifyOfAllUpdates = false;
	private long popupBecameInvisibleAt = 0;
	
	private JPopupMenu currentPopup = null;
	private PopupMenuListener popupListner = new PopupMenuListener() {

		@Override 
		public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
		}

		@Override
		public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
			popupBecameInvisibleAt = System.currentTimeMillis();
		}

		@Override
		public void popupMenuCanceled(PopupMenuEvent e) {
			highlightItem(currentPopup, false);
		}
	};
	

	private List<ChangeListener> chgListners = new ArrayList<ChangeListener>(5);

	private String lastVal = null;

	public ComboLikeObject() {
		this(new JPopupMenu());
	}

	public ComboLikeObject(String txtFldName, JComponent... btns) {
		this(txtFldName, new JPopupMenu(), btns);
	}

	public ComboLikeObject(JPopupMenu popup) {
		this(null, popup, (JButton[]) null);
	}


	public ComboLikeObject(String txtFldName, JPopupMenu popup, JComponent... btns) {
		this(new JTextField(), txtFldName, popup, btns);
	}


	public ComboLikeObject(JTextComponent txtComponent, String fieldName, JPopupMenu popup, JComponent... btns) {
		super(StyleManager.styles().panelAndField(), new BorderLayout());

		valueTxt = txtComponent;
		buttons = btns;
		setCurrentPopup(popup);
		
		if (fieldName != null) {
			valueTxt.setName(fieldName);
		}
		style.register(this);
		init(btns);
	}

	protected void init(JComponent[] btns) {

		valueTxt.setOpaque(true);
		valueTxt.setMinimumSize(new Dimension(FIELD_WIDTH, valueTxt.getHeight()));
//		System.out.println(" ++> ++> " +  " "
//				+ valueTxt.getPreferredSize().height + " " + this.getPreferredSize().height);

//		this.setLayout(new BorderLayout());
		this.add(BorderLayout.CENTER, valueTxt);

		MultiComponentBuilder componentBldr = new MultiComponentBuilder(this.getUiContainer(), valueTxt);
		componentBldr.add(listBtn.getUiContainer());
		if (btns != null ) {
			for (JComponent btn : btns) {
				componentBldr.add(btn);
			}
		}
		componentBldr.build();

//		if (! Common.NIMBUS_LAF) {
//			this.setBorder(valueTxt.getBorder());
//		}
		valueTxt.setBorder(BorderFactory.createEmptyBorder());
		listBtn.addActionListener(this);

		valueTxt.addFocusListener(new FocusAdapter() {
			@Override public void focusLost(FocusEvent e) {
				textFieldChanged();
			}
		});
	}
	
	@Override
	public JTextComponent getMainComponent() {
		return valueTxt;
	}

	protected final void setTextFieldWidth(int width) {
		Dimension d = valueTxt.getPreferredSize();
		valueTxt.setPreferredSize(new Dimension(Math.max(d.width, width), d.height));
	}

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {

    	JPopupMenu nm = getPopup();

        long timeDiff = System.currentTimeMillis() - popupBecameInvisibleAt;
        setCurrentPopup(nm);

 		if (currentPopup.isVisible() || (timeDiff > 0 && timeDiff < 650) ) {
        	currentPopup.setVisible(false);
        	highlightItem(currentPopup, false);
        } else {
         	if (currentPopup != null) {
        		int xCoord = 0;
        		int yCoord = valueTxt.getHeight();

        		//System.out.println("Popup Height 1 " + currentPopup.getPreferredSize().height + " " + popupHeight);
        		int popupWidth = getPopupWidth(currentPopup);
				if (currentPopup.getWidth() < popupWidth) {
        			currentPopup.setPopupSize(new Dimension(
        					popupWidth,
        					Math.max(currentPopup.getPreferredSize().height, popupHeight)));
        		}
        		//System.out.println("Popup Height 2 " + currentPopup.getPreferredSize().height + " " + popupHeight);
        		currentPopup.show(valueTxt, xCoord, yCoord);

        		highlightItem(currentPopup, true);
         	}
	    }
	    //visible = ! visible;
	}

	protected int getPopupWidth(JPopupMenu currentPopup) {
		return this.getWidth();
	}

	protected void hidePopup() {
		currentPopup.setVisible(false);
	}

	protected void highlightItem(JPopupMenu currentPopup, boolean visible) {

	}

	protected static JButton[] addToArray(JButton b, JButton[] btns) {
		JButton[] ret = btns;
		if (btns == null || btns.length == 0) {
			ret = new JButton[1];
		} else if (btns.length > 1 || btns[0] != null){
			ret = new JButton[btns.length + 1];
			System.arraycopy(btns, 0, ret, 1, btns.length);
		}

		ret[0] = b;

		return ret;
	}

//	/**
//	 * @return the currentPopup
//	 */
//	public JPopupMenu getCurrentPopup() {
//		return currentPopup;
//	}

	/**
	 * @param newPopup the currentPopup to set
	 */
	protected void setCurrentPopup(JPopupMenu newPopup) {

    	if ( this.currentPopup != newPopup) {
    		if (this.currentPopup != null) {
    			this.currentPopup.removePopupMenuListener(popupListner);
    		}
    		newPopup.addPopupMenuListener(popupListner);
    		this.currentPopup = newPopup;
     	}
	}
	
	public void setEditable(boolean editable) {
		valueTxt.setEditable(editable);
		
		if (buttons != null) {
			for (JComponent b : buttons) {
				b.setEnabled(editable);
			}
		}
	}

	protected JPopupMenu getPopup() {
		return currentPopup;
	}



	/**
	 * @param t
	 * @see javax.swing.text.JTextComponent#setText(java.lang.String)
	 */
	public void setText(String t) {
		setTextSilently(t);
		fireValueChangeListner(null);
	}

	/**
	 * @param t
	 * @see javax.swing.text.JTextComponent#setText(java.lang.String)
	 */
	protected final void setTextInternal(String t) {
		valueTxt.setText(t);
		fireValueChangeListner(null);
	}

	public void setTextSilently(String t) {
		valueTxt.setText(t);
	}


	protected final void setTextSilentlyInternal(String t) {
		valueTxt.setText(t);
	}

	/**
	 * @return
	 * @see javax.swing.text.JTextComponent#getText()
	 */
	public String getText() {
		return valueTxt.getText();
	}

	public JTextComponent getTextCompenent() {
		return valueTxt;
	}



	/**
	 * @param popupHeight the popupHeight to set
	 */
	protected void setPopupHeight(int popupHeight) {
		if (popupHeight > 0) {
			this.popupHeight = popupHeight;

			currentPopup.setPopupSize(new Dimension(
						getPopupWidth(currentPopup),
						popupHeight));
		}
	}

	public final void addTextChangeListner(ChangeListener cl) {
		chgListners.add(cl);
	}

	public final void removeTextChangeListner(ChangeListener cl) {
		chgListners.remove(cl);
	}

	/**
	 * @param name
	 * @see java.awt.Component#setName(java.lang.String)
	 */
	public void setName(String name) {
		valueTxt.setName(name + "_Txt");

		if (buttons != null) {
			int i = 0;
			String btnName = name + "_Btn";
			for (JComponent b : buttons) {
				b.setName(btnName + i++);
			}
		}

		super.setName(name);
	}

	protected void textFieldChanged() {
		fireValueChangeListner(null);
	}

	protected void fireValueChangeListner(ChangeEvent e) {

		String textVal = valueTxt.getText();
		if ((lastVal == null || notifyOfAllUpdates || ! lastVal.equals(textVal))) {
			try {
				for (int i = 0; i < chgListners.size(); i++) {
					chgListners.get(i).stateChanged(e);
				}
				lastVal = textVal;
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
	}
//
//	/**
//	 * @param notifyOfAllUpdates the notifyOfAllUpdates to set
//	 */
//	public final void setNotifyOfAllUpdates(boolean notifyOfAllUpdates) {
//		this.notifyOfAllUpdates = notifyOfAllUpdates;
//	}
}
