package net.sf.recordEditor.ui.text;

import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;

import net.sf.recordEditor.ui.common.AbstractListAction;
import net.sf.recordEditor.ui.common.DropDownFavouriteBtns;
import net.sf.recordEditor.ui.common.DropDownFavouriteBtns.IListActionBuilder;
import net.sf.recordEditor.ui.common.MultiComponentBuilder;
import net.sf.recordEditor.ui.common.UiSelectionList;
import net.sf.recordEditor.ui.def.common.IListPopupActions;
import net.sf.recordEditor.ui.listManagers.AddToFavouriteAction;
import net.sf.recordEditor.ui.listManagers.IBasicList;
import net.sf.recordEditor.ui.listManagers.IListItem;
import net.sf.recordEditor.ui.listManagers.IRecentList;
import net.sf.recordEditor.ui.listManagers.IUpdateableList;
import net.sf.recordEditor.ui.styles.StyleManager;
import net.sf.recordEditor.ui.text.param.IRecentText;
import net.sf.recordEditor.ui.text.param.IRecentTextBuilder;
import net.sf.recordEditor.ui.text.param.RecentTextBuilder;

public class UiTextRecentFavourites extends UiTextPanel implements IListPopupActions<String> {
	public static IRecentTextBuilder paramBldr() { return new RecentTextBuilder(); }
	
	private final DropDownFavouriteBtns<String> recentFavBtns;
	private String old;
	private IRecentList recentList;
	
	public UiTextRecentFavourites(IRecentText param) {
		super(param.getStyle());
		
		recentList = param.getRecent();
		JTextField textField = this.getMainComponent();
		IListActionBuilder bldr = new IListActionBuilder() {
			
			@Override
			public ActionListener createAction(IBasicList stdList, IUpdateableList updList, JComponent displayItm,
					JComponent container, JPanel favouritePnl) {
				return new AbstractListAction<String>(stdList, updList, displayItm, container) {
					
					@Override
					protected List<String> loadPopup(List<IListItem> itemList) {
						int size = itemList.size();
						ArrayList<String> ret = new ArrayList<String>(size);
						for (int i = 0; i < size; i++) {
							ret.add(itemList.get(i).getItemText());	
						}
						return ret;
					}
					
					@Override
					protected UiSelectionList<String> createList() {
						return new UiSelectionList<String>(
								StyleManager.EMPTY_STYLE, super.container, true, 
								null,
								UiTextRecentFavourites.this);
					}
				} ;
			}
		};
		MultiComponentBuilder componentBldr = new MultiComponentBuilder(this.getUiContainer(), textField);
		this.recentFavBtns = new DropDownFavouriteBtns<String>(
				componentBldr, this.getUiContainer(), 
				recentList, param.getFavourites(), 
				bldr, 
				new AddToFavouriteAction(param.getFavourites(), this));
				
//				new DropDownFavouriteBtns(
//				componentBldr, this.getUiContainer(), param.getRecent(), param.getFavourites(), 
//				this, 
//				new AddToFavouriteAction(param.getFavourites(), this));
		
//		System.out.println(" >>> " + fileField.getPreferredSize().height
//				 + " "+ fileField.getMinimumSize().height);
//		if (recentFavBtns.recentBtn  != null) {
//			Dimension preferredSize = recentFavBtns.recentBtn.getPreferredSize();
//			int h = Math.max(preferredSize.height, textField.getPreferredSize().height);
//			recentFavBtns.recentBtn.setPreferredSize(new Dimension((h + 1) * 3 / 4, h));
//		}
		componentBldr.build();
		
		if (param.getInitialValue() != null) {
			textField.setText(param.getInitialValue());
		}
		textField.addFocusListener(new FocusAdapter() {
			@Override public void focusLost(FocusEvent e) {
				notifyListners();
			}
		});
		
		old = getText();
	}
	
	
	@Override
	public void fireFileChanged(String item) {
		this.setText(item);
		notifyListners();
	}

	@Override
	public void popuHidden() {
		// TODO Auto-generated method stub
		
	}

	private ArrayList<LTextChangedListner> listners = new ArrayList<LTextChangedListner>(3);
	
	public void addTextChangeListner(LTextChangedListner listner) {
		listners.add(listner);
	}
	
	private void notifyListners() {
		
		String value = getText();
		if (old == null || ! old.equals(value)) {
			for (LTextChangedListner l : listners) {
				l.textChanged(this, old, value);
			}
			
			old = value;
		}
	}


	@Override
	public String getText() {
		String s = super.getText();
	
		recentList.addFileItem(s);
		return s;
	}
}
