/*
 * Generated by XmplLineBuilder from Velocity template LineProcess.vm
 *
 */
package net.sf.RecordEditor.examples.genCode;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.GZIPInputStream;

import net.sf.JRecord.Common.RecordException;
import net.sf.JRecord.Details.LayoutDetail;
import net.sf.JRecord.IO.AbstractLineReader;
import net.sf.JRecord.IO.LineIOProvider;
import net.sf.RecordEditor.utils.CopyBookDbReader;

/**
 * Generated by XmplLineBuilder from Velocity template LineProcess.vm
 *
 */
public class EdiPoProcess  {

    private int lineNumber;

    /**
     * process an input file
     *
     * @param fileName filename to process
     *
     * @throws IOException any IO error that occurs
     */
    public void process(String fileName)  throws IOException, RecordException {
        process(fileName, getGroup());
    }


    /**
     * process an input file
     *
     * @param fileName filename to process
     * @param group record layout
     *
     * @throws IOException any IO error that occurs
     */
    public void process(String fileName, LayoutDetail group) 
    throws IOException, RecordException {
        if (fileName.endsWith(".gz")) {
            FileInputStream rf = new FileInputStream(fileName);

            process(new GZIPInputStream(rf), group);

            rf.close();
        } else {
            process(new FileInputStream(fileName), group);
        }
    }

    /**
     * process an input stream
     *
     * @param file input stream process
     *
     * @throws IOException any IO error that occurs
     */
    public void process(InputStream file) throws IOException, RecordException {
        process(file, getGroup());
    }


    /**
     * get Record Layout
     *
     * @return Record Layout
     */
    private LayoutDetail getGroup() {
        return new CopyBookDbReader().getLayout("EDI PO");
    }


    /**
     * process an input stream
     *
     * @param file input stream process
     * @param group record layout
     *
     * @throws IOException any IO error that occurs
     */
    public void process(InputStream file, LayoutDetail group) 
    throws IOException, RecordException {
        int        preferedLayout;
        int         fileStructure = group.getFileStructure();
        LineIOProvider ioProvider = new LineIOProvider(new EdiPoProvider());
        AbstractLineReader      reader = ioProvider.getLineReader(fileStructure);
        EdiPo           line;

        lineNumber = 0;
        reader.open(file, group);
        fileOpen();

        while ((line = (EdiPo) reader.read()) != null) {
            preferedLayout = line.getPreferredLayoutIdx();
            lineNumber += 1;

            if (preferedLayout == EdiPo.getIvr0075dIndex()) {
                processIvr0075d(line);
            } else if (preferedLayout == EdiPo.getIvr0075hIndex()) {
                processIvr0075h(line);
            } else if (preferedLayout == EdiPo.getIvr0075sIndex()) {
                processIvr0075s(line);
            } else {
                unkownRecord(line);
            }
        }

        fileClosed();
        reader.close();
    }


    /**
     * Called after file is opened
     */
    public void fileOpen() { }


    /**
     * Called after file is opened
     */
    public void fileClosed() { }


    /**
     * Called after file is opened
     *
     * @param line Line to be processed
     */
    public void unkownRecord(EdiPo line) { }



    /**
     * process "IVR0075D"
     * $rec.Details.Description
     *
     * @param line Line to be processed
     *
     * @throws IOException any IO error that occurs
     */
    public void processIvr0075d(EdiPo line)
    throws IOException { }


    /**
     * process "IVR0075H"
     * $rec.Details.Description
     *
     * @param line Line to be processed
     *
     * @throws IOException any IO error that occurs
     */
    public void processIvr0075h(EdiPo line)
    throws IOException { }


    /**
     * process "IVR0075S"
     * $rec.Details.Description
     *
     * @param line Line to be processed
     *
     * @throws IOException any IO error that occurs
     */
    public void processIvr0075s(EdiPo line)
    throws IOException { }


    /**
     * @return Returns the lineNumber.
     */
    public int getLineNumber() {
        return lineNumber;
    }
}
